#!/usr/bin/env python
"""Distutils setup file"""

from glob import glob
from setuptools import setup, Extension, find_packages

NAME = "windmill"
VERSION = "0.7.7"

setup(
    name = NAME,
    version = VERSION,
    description  = 'Windmill4 Framework',
    author = 'Radek Kanovsky',
    author_email = 'rk@dat.cz',
    zip_safe = False,
    package_dir = {'':'src'},
    packages = find_packages('src'),
    include_package_data = True,
    ext_modules = [
        Extension("peak.binding._once",
            ["src/peak/binding/_once.pyx", "src/peak/binding/getdict.c"]),
        Extension("wm4.binding._once",
            ["src/wm4/binding/_once.pyx", "src/wm4/binding/getdict.c"]),
        Extension("wm4.config._speedups",
            ["src/wm4/config/_speedups.c"]),
        Extension("peak.util.buffer_gap",
            ["src/peak/util/buffer_gap.pyx"]),
        Extension("peak.util._Code",
            ["src/peak/util/_Code.pyx"]),
        Extension("protocols._speedups",
            ["src/protocols/_speedups.pyx"]),
        Extension("dispatch._d_speedups",
            ["src/dispatch/_d_speedups.pyx"]),
        Extension("peak.persistence._persistence",
            ["src/peak/persistence/persistence.c"]),
        #Extension("peak.util._uuidgen",
        #    ["src/peak/util/_uuidgen.c"]),
        Extension("wm4.running._passfd",
            ["src/wm4/running/_passfd.c"]),
        Extension('adt4.row_acl',
            ['src/adt4/row_acl/acl.c', 'src/adt4/row_acl/py_row_acl.c']),
        Extension('wm3.date',
            glob('src/wm3/src/date*.c')),
        Extension('wm3.pgint',
            ['src/wm3/src/pgint.c'],
            include_dirs=['/usr/include/postgresql'],
            libraries=['pq', 'crypt'])

    ],
    install_requires = [
        'python-dateutil == 1.5',
        'Pillow',
        'docutils',
        'PyGreSQL==4.0',
        'ClientForm',
        'elementtree',
    ],
    entry_points = '''\
        [paste.app_factory]
        main = wm4.web.config:app_factory
    ''',
    dependency_links = ['http://ftp.uh.cz/eggs/'],
)
