BEGIN;

-- Agregacni funkce, ktera vytvari pole agregovanych polozek. Pokud
-- neni pole zadouci, lze na vysledek nasledne pouzit funkci
-- array_to_string(array_accum(...), delim).

CREATE AGGREGATE array_accum (
    sfunc = pg_catalog.array_append,
    basetype = pg_catalog.anyelement,
    stype = pg_catalog.anyarray,
    initcond = '{}'
);

-- Agregacni funkce, ktera vytvari pole poli

CREATE OR REPLACE FUNCTION array_array_accum(IN s anyarray, IN a anyarray)
RETURNS anyarray AS
$BODY$
    SELECT pg_catalog.array_cat($1, ARRAY[$2])
$BODY$ LANGUAGE 'sql' IMMUTABLE STRICT;

CREATE AGGREGATE array_array_accum (
    sfunc = array_array_accum,
    basetype = pg_catalog.anyarray,
    stype = pg_catalog.anyarray,
    initcond = '{}'
);

-- Opak predchozi funkce, ktera funguje i v ramci select vyrazu (neni
-- ji potreba joinovat nebo davat jako subquery):
--
--    SELECT pkey, array_iter(arr) FROM foo;
--
CREATE OR REPLACE FUNCTION array_iter(IN x anyarray,OUT n anyelement,OUT p int4)
RETURNS SETOF RECORD AS
$BODY$
    SELECT $1[s], s - pg_catalog.array_lower($1,1) + 1
      FROM pg_catalog.generate_series(
                pg_catalog.array_lower($1,1),
                pg_catalog.array_upper($1,1), 1
           ) AS g(s)
$BODY$ LANGUAGE 'sql' IMMUTABLE STRICT;

-- Tohle kupodivu nefunguje a rve to, ze funkce vracejici set,
-- nemuze byt pouzita v kontextu select vyrazu.
--
-- CREATE OR REPLACE FUNCTION array_iter(IN x anyarray, OUT n anyelement, OUT p int4)
-- RETURNS SETOF RECORD AS
-- $BODY$
-- DECLARE
--     a ALIAS FOR $1;
--     i INTEGER;
-- BEGIN
--     FOR i IN COALESCE(array_lower(a,1),0) .. COALESCE(array_upper(a,1),-1) LOOP
--         p := i;
--         n := a[i];
--         RETURN NEXT;
--     END LOOP;
--     RETURN;
-- END
-- $BODY$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;

END;
