-- --------------------------------------------------------------------------
-- Triggers for temporal tables
-- --------------------------------------------------------------------------

-- CREATE TRIGGER "10_cms_node_temporal_update"
--   BEFORE UPDATE ON cms_node FOR EACH ROW
--   EXECUTE PROCEDURE temporal_update('validity');


CREATE OR REPLACE FUNCTION temporal_update_before ()
RETURNS TRIGGER LANGUAGE plpythonu AS
'
    from adt.api import validity_datetime, DATETIME_MAX, now
    from datetime import timedelta
    import pgspi

    val_col, = TD["args"]
    old, new = TD["old"], TD["new"]

    if pgspi.recursiveTriggerInvocation() :
        for a in old.iterkeys() :
            if old[a] != new[a] and a != val_col :
                plpy.error("Cannot recursively update column %r" % a)
        return "OK"

    else :
        new_val = validity_datetime(new[val_col])
        old_val = validity_datetime(old[val_col])

        if old_val._till != DATETIME_MAX :
            plpy.error(
                "Only newest record can be updated", TD["relid"], val_col
            )

        changed = {}
        for a in old.iterkeys() :
            if old[a] != new[a] :
                changed[a] = 1
        if len(changed) == 1 and changed.keys() == [val_col] \\
        and old_val._till == DATETIME_MAX :
            return "OK"

        split = now()
        if old_val._from >= split :
            split = old_val._from + timedelta(seconds=1)
        new[val_col] = validity_datetime(split, DATETIME_MAX)
        return "MODIFY"
';



CREATE OR REPLACE FUNCTION temporal_update_after ()
RETURNS TRIGGER LANGUAGE plpythonu AS
'
    from adt.api import validity_datetime, DATETIME_MAX
    from datetime import timedelta
    import pgspi
    
    val_col, = TD["args"]
    old, new = TD["old"], TD["new"]
    new_val = validity_datetime(new[val_col])
    old_val = validity_datetime(old[val_col])

    if new_val._till != DATETIME_MAX :
        return None
    split = new_val._from - timedelta(seconds=1)
    save = old.copy()
    save[val_col] = str(validity_datetime(old_val._from, split))


    if SD.has_key("plan") :
        plan = SD["plan"]
    else :
        SD["plan"] = plan = pgspi.InsertPlan(TD["relid"])

    plan.execute(save)
';
    


CREATE OR REPLACE FUNCTION temporal_delete_before ()
RETURNS TRIGGER LANGUAGE plpythonu AS
'
    from adt.api import validity_datetime, DATETIME_MAX, now
    import pgspi

    val_col, = TD["args"]
    old      = TD["old"]
    old_val  = validity_datetime(old[val_col])

    if old_val._till != DATETIME_MAX :
        plpy.error("Only newest record can be deleted")

    n = now()
    if n < old_val._from :
        n = old_val._from
    new_val = validity_datetime(old_val._from, n)

    if SD.has_key("plan") :
        plan = SD["plan"]
    else :
        SD["plan"] = plan = pgspi.UpdatePlan(TD["relid"], [val_col])
    plan.execute(old, **{val_col: str(new_val)})
    return "SKIP"
';



CREATE OR REPLACE FUNCTION set_temporal_triggers (TEXT,TEXT,INT)
RETURNS TEXT LANGUAGE plpythonu AS
'
    tablename, valcol, priority = args

    q = """CREATE TRIGGER "%d_%s_temporal_update_before"
           BEFORE UPDATE ON "%s" FOR EACH ROW
           EXECUTE PROCEDURE temporal_update_before(''%s'')""" % (
           priority, tablename, tablename, valcol)

    plpy.execute(q)

    q = """CREATE TRIGGER "%d_%s_temporal_update_after"
           AFTER UPDATE ON "%s" FOR EACH ROW
           EXECUTE PROCEDURE temporal_update_after(''%s'')""" % (
           priority, tablename, tablename, valcol)

    plpy.execute(q)

    q = """CREATE TRIGGER "%d_%s_temporal_delete_before"
           BEFORE DELETE ON "%s" FOR EACH ROW
           EXECUTE PROCEDURE temporal_delete_before(''%s'')""" % (
           priority, tablename, tablename, valcol)

    plpy.execute(q)
    return "Temporal triggers on table %r set" % tablename
';
