# The pain that is tsearch2 8.1 -> 8.2 upgrading
# http://people.planetpostgresql.org/xzilla/index.php?/archives/291-guid.html

import sys, pgdb, os
from sql4.utils import sql_escape
from optparse import OptionParser

def getOpts():
    usage = 'usage: %prog [options] dbname1 [dbname2 [dbname3 ..]]'
    parser = OptionParser(usage=usage)
    parser.add_option(
        '-S', '--share', metavar='DIR', help='postgresql share directory',
    )
    opts, databases = parser.parse_args()
    if not databases:
        parser.error('no dbname')
    opts.databases = databases
    return opts


def get_version():
    cur = opts.con.cursor()
    cur.execute('SELECT VERSION()')
    ver, = cur.fetchone()
    cur.close()
    ver = '.'.join(ver.split()[1].split('.')[:2])
    return ver



def get_description(con, tab):
    cur = con.cursor()
    cur.execute('''
        SELECT attname, atttypid::regtype
          FROM pg_attribute
         WHERE attrelid = '%s'::regclass AND attnum >= 0 AND NOT attisdropped
         ORDER BY attnum
    ''' % tab)
    res = cur.fetchall()
    cur.close()
    return res


def alter_column_to_regprocedure(con, tab, col):
    print 'ugrading column definition', tab, col
    cur = con.cursor()
    cur.execute('ALTER TABLE %s ADD COLUMN %s_tmp REGPROCEDURE' % (tab,col))
    cur.execute('UPDATE %s SET %s_tmp = %s' % (tab,col,col))
    cur.execute('ALTER TABLE %s ALTER COLUMN %s_tmp SET NOT NULL' % (tab,col))
    cur.execute('ALTER TABLE %s DROP COLUMN %s' % (tab,col))
    cur.execute('ALTER TABLE %s RENAME %s_tmp TO %s' % (tab,col,col))
    cur.close()


def check_procedure_column_type(con):
    tdef = dict(get_description(con, 'pg_ts_dict'))
    for a in (
        'dict_init', 'dict_lexize'
    ):
        if tdef[a] != 'regprocedure':
            alter_column_to_regprocedure(con, 'pg_ts_dict', a)

    tdef = dict(get_description(con, 'pg_ts_parser'))
    for a in (
        'prs_start','prs_nexttoken','prs_end','prs_headline','prs_lextype'
    ):
        if tdef[a] != 'regprocedure':
            alter_column_to_regprocedure(con, 'pg_ts_parser', a)


def internalize_functions(con):
    cur = con.cursor()
    for fun in (
        'dex_init', 'snb_en_init', 'snb_ru_init', 'spell_init', 'syn_init'
    ):
        cur.execute("""
            SELECT pg_catalog.format_type(prorettype, NULL),
                   pg_catalog.oidvectortypes(proargtypes)
              FROM pg_proc where proname = '%s'""" % fun)
        res = cur.fetchall()
        assert len(res) == 1
        ret, arg = res[0]
        if ret != 'internal' or arg != 'internal':
            print 'upgrading definition of function %s(%s)' % (fun,arg)
            cur.execute('DROP FUNCTION %s(%s)' % (fun, arg))
            cur.execute("""
                CREATE FUNCTION %s(internal) RETURNS internal
                    AS '$libdir/tsearch2' LANGUAGE 'C'""" % fun)
    cur.close()


def check_registered_procedures(con):
    cur = con.cursor()
    ts_dict = {
        'simple': (
            'dex_init(internal)','dex_lexize(internal,internal,integer)'),
        'en_stem': (
            'snb_en_init(internal)', 'snb_lexize(internal,internal,integer)'),
        'ru_stem': (
            'snb_ru_init(internal)', 'snb_lexize(internal,internal,integer)'),
        'ispell_template': (
            'spell_init(internal)', 'spell_lexize(internal,internal,integer)'),
        'synonym': (
            'syn_init(internal)', 'syn_lexize(internal,internal,integer)'),
        'cz_ispell': (
            'spell_init(internal)', 'spell_lexize(internal,internal,integer)'),
    }
    cur.execute('SELECT dict_name,dict_init,dict_lexize FROM pg_ts_dict')
    for n,i,l in cur.fetchall():
        ri, rl = ts_dict[n]
        if i != ri or l != rl:
            print 'upgrading registered procedures in pg_ts_dict', n
            cur.execute("""
                UPDATE pg_ts_dict
                   SET dict_init='%s', dict_lexize='%s'
                 WHERE dict_name='%s'""" % (ri, rl, n))

    ts_parser = {
        'default': dict(
            prs_start = 'prsd_start(internal,integer)',
            prs_nexttoken = 'prsd_getlexeme(internal,internal,internal)',
            prs_end = 'prsd_end(internal)',
            prs_headline = 'prsd_headline(internal,internal,internal)',
            prs_lextype = 'prsd_lextype(internal)'
        ),
    }
    cur.execute('''
        SELECT prs_name, prs_start, prs_nexttoken, prs_end, prs_headline,
               prs_lextype FROM pg_ts_parser''')
    for n,s,t,e,h,l in cur.fetchall():
        r = ts_parser[n]
        if s != r['prs_start'] \
        or t != r['prs_nexttoken'] \
        or e != r['prs_end'] \
        or h != r['prs_headline'] \
        or l != r['prs_lextype']:
            print 'upgrading registered procedures in pg_ts_parser', n
            cur.execute("""
                UPDATE pg_ts_parser
                   SET prs_start = '%s',
                       prs_nexttoken = '%s',
                       prs_end = '%s',
                       prs_headline = '%s',
                       prs_lextype = '%s'
                WHERE prs_name = '%s'""" % (
                    r['prs_start'],
                    r['prs_nexttoken'],
                    r['prs_end'],
                    r['prs_headline'],
                    r['prs_lextype'],
                    n,
            ))



def check_dict_initoption(con):
    
    def check_file(f):
        if os.path.exists(f):
            return
        print 'invalid path in pg_ts_dict.dict_initoption[dict_name=%s]' % n
        if not opts.share:
            print 'ERROR: share directory not specified, use --share/-S option'
            return
        print ' ', f, '->'
        nf = os.path.join(
                os.path.join(opts.share,'contrib'),
                os.path.split(f)[1]
        )
        print ' ', nf
        return nf


    cur = con.cursor()
    cur.execute("select dict_name, dict_init, dict_initoption FROM pg_ts_dict")
    for n,i,o in cur.fetchall():
        no = o
        if i.startswith('snb_'):
            r = check_file(o)
            if r:
                no = no.replace(o, r)
        elif i == 'spell_init(internal)' and o:
            for f in o.split(','):
                f = f.strip()[:-1]
                f = f[f.find('"')+1:]
                r = check_file(f)
                if r:
                    no = no.replace(f, r)
        if no != o:
            print 'updating %s' % n
            cur.execute(
                """UPDATE pg_ts_dict SET dict_initoption='%s'
                    WHERE dict_name='%s'""" % (sql_escape(no), sql_escape(n))
            )
            print
    cur.close()


opts = getOpts()

for dbname in opts.databases:
    opts.dbname = dbname
    opts.con = pgdb.connect(database=dbname)
    opts.version = get_version()
    print opts
    continue
    opts.con.commit()
    
    if len(opts.dbnames) > 1:
        print
        print '----- database: %s -----' % dbname
    check_procedure_column_type(con)
    internalize_functions(con)
    check_registered_procedures(con)
    check_dict_initoption(con)
    con.commit()
