__all__ = ['DomainName']

from lib4.data.top_level_domains import top_level_domains
import re


_domain_name_part_match = re.compile('[0-9a-z][-0-9a-z]*').match


class DomainName (str) :

    def __new__ (klass, dn) :

        if isinstance(dn, DomainName) :
            return dn

        if not dn :
            raise ValueError('Empty domain name')

        dn = str(dn) # Domain must be string

        if dn == '.' :

            # Domain root
            parts = ()
            normalized = '.'

        else :
            normalized = dn.lower()
            if not normalized.endswith('.') :
                normalized = normalized + '.'

            parts = normalized.split('.')[:-1]

            for i, p in enumerate(parts) :

                if not p :
                    raise ValueError('Empty domain name part', dn)

                if p == '*' :
                    if i != 0 :
                        raise ValueError(
                            'Wildcard must be first part of domain name', dn
                        )
                    continue

                if not _domain_name_part_match(p) :
                    raise ValueError('Bad domain name syntax', dn, p)

            parts.reverse()
            if parts[0] not in top_level_domains :
                raise ValueError('Toplevel domain do not exists', dn, parts[0])

        dn = str.__new__(DomainName, dn)
        dn.__parts = tuple(parts)
        dn.__normalized = normalized
        return dn

    def get_parts (self, typ=None) :
        return self.__parts
    parts = property(get_parts)

    def get_normalized (self, typ=None) :
        return self.__normalized
    normalized = property(get_normalized)

    def get_level (self, typ=None) :
        return len(self.__parts)
    level = property(get_level)

    def __len__ (self) :
        return len(self.__parts)

    def isRoot (self) :
        return len(self.__parts) == 0

    def isToplevel (self) :
        return len(self.__parts) == 1

    def isWildcard (self) :
        return len(self.__parts) > 1 and self.__parts[-1] == '*'


    def __getitem__ (self, index) :
        return self.__parts[index]

    def __getslice__ (self, start, end) :
        return self.__parts[start, end]

    def __cmp__ (self, other) :
        other = DomainName(other)
        return cmp(self.__parts, other.__parts)

    def __lt__ (self, other) : return self.__cmp__(other) < 0
    def __le__ (self, other) : return self.__cmp__(other) < 1
    def __eq__ (self, other) : return self.__cmp__(other) == 0
    def __ge__ (self, other) : return self.__cmp__(other) > -1
    def __gt__ (self, other) : return self.__cmp__(other) > 0
    def __ne__ (self, other) : return self.__cmp__(other) != 0
