from itertools import count

__all__ = ['enum', 'Enum']


NO_VALUE = object()


class enum(object):

    next_serial = count().next

    def __init__(self, value=NO_VALUE, position=NO_VALUE):
        self.value = value
        if position is NO_VALUE:
            position = self.next_serial()
        self.position = position


class EnumMeta(type):

    def __init__(klass, name, bases, cdict):
        super(EnumMeta, klass).__init__(name, bases, cdict)
        # Its me
        if name == 'Enum' and cdict.get('__module__') == __name__:
            return

        itemDict = {}

        # enums defined here
        for e, v in cdict.items():
            if isinstance(v, enum):
                ei = klass(e, v.value is NO_VALUE and e or v.value, v.position)
                itemDict[e] = ei
                setattr(klass, e, ei)

        # inherited enums
        for base in klass.__mro__[1:]:
            if not isinstance(base, EnumMeta) or base is Enum:
                continue
            for e, v in base.__itemDict.iteritems():
                if e in itemDict:
                    itemDict[e].position = v.position
                else:
                    ei = klass(v.name, v.value, v.position)
                    itemDict.setdefault(e, ei)
                    setattr(klass, e, ei)

        klass.__itemDict = itemDict
        klass.__itemList = sorted(itemDict.values(), key=lambda v:v.position)

        # set idx for every enum
        for i,e in enumerate(klass.__itemList):
            e.idx = i

    def __getitem__(klass, e):
        return klass.__itemDict[e]

    def __contains__(klass, e):
        return e in klass.__itemDict

    def find(klass, val):
        for a,e in klass.__itemDict.iteritems():
            if e.value == val:
                return e

    def values(klass):
        return klass.__itemList

    def itervalues(klass):
        return klass.__itemList

    def items(klass):
        return [(e.name,e) for e in klass.__itemList]
    iteritems = items

    def keys(klass):
        return [e.name for e in klass.__itemList]
    iterkeys = keys






class Enum(object):

    __metaclass__ = EnumMeta

    __itemDict = {}
    __itemList = []

    def __init__(self, name, value, position):
        self.name = name
        self.value = value
        self.position = position
        self.idx = None

    def __repr__(self):
        return '%s.%s' %(self.__class__.__name__, self.name)

    def __str__(self):
        return self.name
