import re

__all__ = ["LTree"]


_ltree_part_match = re.compile("^[a-zA-Z0-9_]+$").match
_ltree_syntax_match = re.compile("^[a-zA-Z0-9_]+(\\.[a-zA-Z0-9_]+)*$").match



class LTree (str) :

    partMatch = _ltree_part_match
    match = _ltree_syntax_match

    def __new__ (klass, path) :
        if isinstance(path, klass) :
            return path
        p = path.replace("/", ".")
        if not _ltree_syntax_match(p) :
            raise ValueError("Invalid LTree syntax %r" % path)
        parts = tuple(p.split("."))
        class LTree (klass) :
            parts = property(lambda o: parts)
        return str.__new__(LTree, p)

    def __iter__ (self) :
        return iter(self.parts)

    def iterParents (self) :
        p = self.parts
        yield self
        for i in xrange(1, len(p)) :
            yield LTree(".".join(p[:-i]))

    def getParents (self, reverse=False) :
        parents = [p for p in self.iterParents()]
        if reverse:
            parents.reverse()
        return parents

    def __repr__ (self) :
        return "LTree(%s)" % str(self)

    def __mod__ (self, *a) :
        raise NotImplementedError("Operation not implemented")
    __mul__ = __rmul__ = __rmod__ = __mod__

    def __add__ (self, other) :
        other = LTree(other)
        return LTree("%s.%s" % (str(self), str(other)))

    def __len__ (self) :
        return len(self.parts)

    def __getitem__ (self, i) :
        return self.parts[i]

    def __getslice__ (self, i, j) :
        return LTree(".".join(self.parts[i:j]))
