#include <postgres.h>
#include <utils/timestamp.h>
#include <utils/datetime.h>
#include <utils/array.h>
#include <catalog/pg_type.h>
#include <fmgr.h>
#include <miscadmin.h>
#include <ctype.h>

#include "acl.h"

PG_MODULE_MAGIC;


#define PG_GETARG_PNACL(x)      ((pnAcl *) DatumGetPointer( \
                                     PG_DETOAST_DATUM(PG_GETARG_DATUM(x))) \
                                   )

void * _palloc (size_t s)
{
    return palloc(s);
}

void _debug (char *msg)
{
    elog(INFO, msg);
}

pnAclCtx  ctx = {_palloc, repalloc, pfree, _debug};



pnAcl * rownacl_in(char * str)
{
    pnAcl  *acl;

    acl = pnAclFromString(&ctx, str);
    if(acl == NULL) {
        elog(ERROR, "invalid pgacl representation \"%s\"", str);
        return NULL;
    }
    return acl;
}


char * rownacl_out(pnAcl *acl)
{
    return pnAclDumps(&ctx, acl);
}


PG_FUNCTION_INFO_V1(rownacl_allows);
Datum rownacl_allows(PG_FUNCTION_ARGS)
{
    pnAcl       *acl;
    int         role, user, gnum, *groups, i;
    ArrayType   *array;


    if(PG_ARGISNULL(0)) {
        PG_RETURN_BOOL(false);
    } else {
        acl = PG_GETARG_PNACL(0);
    }
    role  = PG_GETARG_INT32(1);
    user  = PG_GETARG_INT32(2);
    array = PG_GETARG_ARRAYTYPE_P(3);

    if (ARR_NDIM(array) != 1) {
        elog(ERROR,"one dimensional array required");
    }
    if (ARR_ELEMTYPE(array) != INT4OID) {
        elog(ERROR,"array of ints required");
    }
    gnum =  ARR_DIMS(array)[0];
    groups = (int *) ARR_DATA_PTR(array);

    i = pnAclAllows(&ctx, acl, role, user, gnum, groups);
    if(i) {
        PG_RETURN_BOOL(true);
    } else {
        PG_RETURN_BOOL(false);
    }
}




PG_FUNCTION_INFO_V1(rownacl_join);
Datum rownacl_join(PG_FUNCTION_ARGS)
{
    pnAcl      *parent;
    pnAcl      *child;
    pnAcl      *join;


    if(PG_ARGISNULL(0)) {
        if(PG_ARGISNULL(1)) {
            PG_RETURN_NULL();
        } else {
            return (Datum) PG_GETARG_PNACL(1);
        }
    }
    if(PG_ARGISNULL(1)) {
        return (Datum) PG_GETARG_PNACL(0);
    }
    
    parent = PG_GETARG_PNACL(0);
    child  = PG_GETARG_PNACL(1);

    join = pnAclJoin(&ctx, parent, child);
    return (Datum) join;
}

PG_FUNCTION_INFO_V1(rownacl_eq);
Datum rownacl_eq(PG_FUNCTION_ARGS)
{
    pnAcl *l = PG_GETARG_PNACL(0);
    pnAcl *r = PG_GETARG_PNACL(1);

    if(pnAclEq(&ctx, l, r)) {
        PG_RETURN_BOOL(true);
    } else {
        PG_RETURN_BOOL(false);
    }
}

PG_FUNCTION_INFO_V1(rownacl_ne);
Datum rownacl_ne(PG_FUNCTION_ARGS)
{
    pnAcl *l = PG_GETARG_PNACL(0);
    pnAcl *r = PG_GETARG_PNACL(1);

    if(pnAclEq(&ctx, l, r)) {
        PG_RETURN_BOOL(false);
    } else {
        PG_RETURN_BOOL(true);
    }
}
