from unittest import *
from datetime import datetime, date, timedelta, tzinfo
from dateutil import tz
from adt4.date import *

D = Date.parse
T = Datetime.parse


class TestDate(TestCase):

    def test10_FromDate(self):
        self.assertEqual(D(date(2000,1,1)), Date(2000,1,1))

    def test11_FromDatetime(self):
        self.assertEqual(D(datetime(2000,1,1,2,3,4)), date(2000,1,1))

    def test12_FromTuple(self):
        self.assertEqual(Date(2000,1,1), date(2000,1,1))

    def test13_FromString(self):
        self.assertEqual(D('2000-2-1'), date(2000,2,1))
        self.assertEqual(D('1.2.2000'), date(2000,2,1))

    def test14_Constants(self):
        self.assertEqual(Date.today(), date.today())



class TestDateTime (TestCase) :

    def test10_FromDate (self) :
        self.assertEqual(
            T(date(2000,1,1)),
            datetime(2000,1,1).replace(tzinfo=TZLOCAL)
        )

    def test11_FromDatetime (self) :
        self.assertEqual(
            T(datetime(2000,1,1,2,3,4)),
            datetime(2000,1,1,2,3,4).replace(tzinfo=TZLOCAL)
        )

    def test12_FromTuple (self) :
        self.assertEqual(
            Datetime(2000,1,1,2,3,4),
            datetime(2000,1,1,2,3,4).replace(tzinfo=TZLOCAL)
        )

    def test13_FromString (self) :
        self.assertEqual(
            T('2000-2-1 12:34'),
            datetime(2000,2,1,12,34).replace(tzinfo=TZLOCAL)
        )
        self.assertEqual(
            T('1.2.2000 12:34'),
            datetime(2000,2,1,12,34).replace(tzinfo=TZLOCAL)
        )
        self.assertEqual(
            T('Mon, 26 May 2008 09:55:54 +0200'),
            datetime(2008,5,26,9,55,54).replace(tzinfo=tz.tzoffset(None, 7200))
        )
        self.assertEqual(
            T('Mon May 26 10:01:12 CEST 2008'),
            datetime(2008,5,26,10,1,12).replace(tzinfo=tz.tzoffset(None, 7200))
        )



M = Month

class TestMonth(TestCase):

    def test01_FromDate(self):
        self.assertEqual(M(D('1.10.2000')), M(2000,10))
        self.assertEqual(M(T('1.10.2000')), M(2000,10))

    def test10_Comparison(self):
        self.failUnless(M(2000,10) == M(2000,10))
        self.failUnless(M(2000,10) >= M(2000,10))
        self.failUnless(M(2000,10) <= M(2000,10))
        self.failIf(M(2000,10) > M(2000,10))
        self.failIf(M(2000,10) < M(2000,10))
        self.failIf(M(2000,10) != M(2000,10))
        self.failUnless(M(2000,6) != M(2000,10))
        self.failUnless(M(2000,6) < M(2000,10))
        self.failUnless(M(2000,6) <= M(2000,10))
        self.failIf(M(2000,6) == M(2000,10))
        self.failIf(M(2000,6) > M(2000,10))
        self.failIf(M(2000,6) >= M(2000,10))

    def test20_Getitem(self):
        self.assertEqual(M(2000,1)[1], Date(2000,1,1))
        self.assertEqual(M(2000,1)[3], Date(2000,1,3))
        self.assertEqual(M(2000,1)[31], Date(2000,1,31))
        self.assertRaises(KeyError, M(2000,1).__getitem__, 32)
        self.assertRaises(KeyError, M(2000,1).__getitem__, 34)
        self.assertRaises(KeyError, M(2000,1).__getitem__, 0)
        self.assertRaises(KeyError, M(2000,1).__getitem__, -1)
        self.assertRaises(KeyError, M(2000,1).__getitem__, -32)

    def test22_Getslice(self):
        self.assertEqual(M(2000,1)[:1], [
        ])
        self.assertEqual(M(2000,1)[:2], [
            Date(2000,1,1),
        ])
        self.assertEqual(M(2000,1)[:3], [
            Date(2000,1,1), Date(2000,1,2)
        ])
        self.assertEqual(M(2000,1)[1:3], [
            Date(2000,1,1), Date(2000,1,2)
        ])
        self.assertEqual(M(2000,1)[3:7], [
            Date(2000,1,3), Date(2000,1,4), Date(2000,1,5), Date(2000,1,6)
        ])
        self.assertEqual(M(2000,1)[-3:], [
            Date(2000,1,29), Date(2000,1,30), Date(2000,1,31)
        ])
        self.assertEqual(M(2000,1)[3:20:4], [
            Date(2000,1,3),Date(2000,1,7),Date(2000,1,11),Date(2000,1,15),Date(2000,1,19)
        ])
        self.assertEqual(M(2000,1)[3:7:2], [
            Date(2000,1,3), Date(2000,1,5)
        ])
        self.assertEqual(M(2008,2)[:], [
            Date(2008, 2, d) for d in range(1,30)
        ])
        self.assertEqual(M(2008,2)[::3], [
            Date(2008, 2, d) for d in range(1,30,3)
        ])

    def test30_Add(self):
        self.assertEqual(M(2000,1)+0, M(2000,1))
        for i in range(1,12):
            self.assertEqual(M(2000,1)+i, M(2000,i+1))
            self.assertEqual(M(2000,1)- -i, M(2000,i+1))
        for i in range(12,24):
            self.assertEqual(M(2000,1)+i, M(2001,i%12+1))
            self.assertEqual(M(2000,1)- -i, M(2001,i%12+1))
        for i in range(24,36):
            self.assertEqual(M(2000,1)+i, M(2002,i%12+1))
            self.assertEqual(M(2000,1)- -i, M(2002,i%12+1))

    def test31_Sub(self):
        self.assertEqual(M(2000,1)-0, M(2000,1))
        for i in range(1,12):
            self.assertEqual(M(2000,12)-i, M(2000,12-i))
            self.assertEqual(M(2000,12)+ -i, M(2000,12-i))
        for i in range(12,24):
            self.assertEqual(M(2000,12)-i, M(1999, 12 - i%12))
            self.assertEqual(M(2000,12)+ -i, M(1999, 12 - i%12))
        for i in range(24,36):
            self.assertEqual(M(2000,12)-i, M(1998, 12 - i%12))
            self.assertEqual(M(2000,12)+ -i, M(1998, 12 - i%12))



def suite() :
    return TestSuite((
        makeSuite(TestDate),
        makeSuite(TestDateTime),
        makeSuite(TestMonth),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
