import unittest
from adt4.enums import *


class A(Enum):
    a = enum('Aa')
    b = enum('Ab')

class B(Enum):
    b = enum('Bb')
    c = enum('Bc')

class C(A, B):
    a = enum('Ca')

    def x(self):
        pass

class S(Enum):
    z = enum()
    a = enum()
    x = enum()
    m = enum()
    n = enum()


class TestEnum(unittest.TestCase):

    def test10_Instances(self):
        self.failUnless(isinstance(A.a, A))
        self.failUnless(isinstance(A.b, A))

        self.failUnless(isinstance(B.b, B))
        self.failUnless(isinstance(B.c, B))

        self.failUnless(isinstance(C.a, C))
        self.failUnless(isinstance(C.b, C))
        self.failUnless(isinstance(C.c, C))

    def test12_Subclasses(self):
        self.failUnless(issubclass(A, Enum))
        self.failUnless(issubclass(B, Enum))
        self.failUnless(issubclass(C, Enum))
        self.failUnless(issubclass(C, A))
        self.failUnless(issubclass(C, B))

    def test14_Names(self):
        self.assertEqual(A.a.name, 'a')
        self.assertEqual(A.b.name, 'b')
        self.assertEqual(B.b.name, 'b')
        self.assertEqual(B.c.name, 'c')
        self.assertEqual(C.a.name, 'a')
        self.assertEqual(C.b.name, 'b')
        self.assertEqual(C.c.name, 'c')

    def test16_Values(self):
        self.assertEqual(A.a.value, 'Aa')
        self.assertEqual(A.b.value, 'Ab')
        self.assertEqual(B.b.value, 'Bb')
        self.assertEqual(B.c.value, 'Bc')
        self.assertEqual(C.a.value, 'Ca')
        self.assertEqual(C.b.value, 'Ab')
        self.assertEqual(C.c.value, 'Bc')

    def test18_Representation(self):
        self.assertEqual(repr(A.a), 'A.a')
        self.assertEqual(repr(A.b), 'A.b')
        self.assertEqual(repr(B.b), 'B.b')
        self.assertEqual(repr(B.c), 'B.c')
        self.assertEqual(repr(C.a), 'C.a')
        self.assertEqual(repr(C.b), 'C.b')
        self.assertEqual(repr(C.c), 'C.c')

    def test20_AsString(self):
        self.assertEqual(str(A.a), 'a')
        self.assertEqual(str(A.b), 'b')
        self.assertEqual(str(B.b), 'b')
        self.assertEqual(str(B.c), 'c')
        self.assertEqual(str(C.a), 'a')
        self.assertEqual(str(C.b), 'b')
        self.assertEqual(str(C.c), 'c')

    def test22_Inheritance(self):
        self.failIf(C.a is A.a)
        self.failIf(C.b is A.b)
        self.failIf(C.b is B.b)
        self.failIf(C.c is B.c)

    def test24_Keys(self):
        self.assertEqual(A.keys(), ['a','b'])
        self.assertEqual(B.keys(), ['b','c'])
        self.assertEqual(C.keys(), ['a','b','c'])
        self.assertEqual(S.keys(), ['z', 'a', 'x', 'm', 'n'])

    def test25_Indexes(self):
        idx = lambda e: [(v.name, v.idx) for v in e.values()]
        self.assertEqual(idx(A), [('a',0), ('b',1)])
        self.assertEqual(idx(B), [('b', 0), ('c', 1)])
        self.assertEqual(idx(C), [('a', 0), ('b', 1), ('c', 2)])
        self.assertEqual(idx(S), [('z', 0), ('a', 1), ('x', 2),
                                  ('m', 3), ('n', 4)])

    def test30_Contains(self):
        self.failUnless('a' in A)
        self.failUnless('b' in A)
        self.failUnless('b' in B)
        self.failUnless('c' in B)
        self.failUnless('a' in C)
        self.failUnless('b' in C)
        self.failUnless('c' in C)

    def test32_DefaultValue(self):
        class D(Enum): a = enum()
        self.assertEqual(D.a.value, 'a')




def suite():
    theSuite = unittest.TestSuite()
    theSuite.addTest(unittest.makeSuite(TestEnum))
    return theSuite


if __name__ == '__main__':
    unittest.main( defaultTest='suite' )
