import unittest
from adt4.ltree import LTree as L


class TestLTree (unittest.TestCase) :

    def test01_Instantiation (self) :
        self.assertRaises(ValueError, L, "#")
        self.assertRaises(ValueError, L, "")

    def test02_Comparison (self) :

        self.assertEqual(L("a.b"), L("a/b"))
        self.assertNotEqual(L("a.b"), L("a.c"))

        self.failUnless(L("a") < L("a.a"))
        self.failUnless(L("a.a") > L("a"))

        self.failUnless(L("a.a") < L("a.b"))
        self.failUnless(L("a.b") > L("a.a"))

        self.failUnless(L("a.a") < L("ab.b"))
        self.failUnless(L("ab.a") > L("aa.b"))

        self.failUnless(L("a.a") < "ab.b")
        self.failUnless("a.a" < L("ab.b"))

        self.failUnless(L("ab.a") > "aa.b")
        self.failUnless("ab.a" > L("aa.b"))

    def test03_Sequence (self) :
        self.assertEqual(L("a.b")[:-1], "a")
        self.assertEqual(L("a.b")[-1:], "b")
        self.assertEqual(L("a.b")[1:], "b")
        self.assertEqual(L("a.b")[:], "a.b")
        self.assertEqual(L("a.b")[0], "a")
        self.assertEqual(L("a.b")[-2], "a")
        self.assertEqual(L("a.b")[1], "b")
        self.assertEqual(L("a.b")[-1], "b")

    def test04_Iteration (self) :
        self.assertEqual(L("a.b.c").parts, ("a", "b", "c"))
        self.assertEqual([p for p in L("a.b.c")], ["a", "b", "c"])

        self.assertEqual(
            L("a.b.c").getParents(), ["a.b.c", "a.b", "a"]
        )
        self.assertEqual(
            L("a.b.c").getParents(reverse=True), ["a", "a.b", "a.b.c"]
        )



    def test05_ReadOnlyness (self) :
        self.assertRaises(AttributeError, setattr, L("a.b.c"), "parts", "x")
        self.assertRaises(AttributeError, delattr, L("a.b.c"), "parts")

    def test06_IsHashable (self) :
        d = { L('a.b.c') : 'a.b.c',
              L('b.a.c') : 'b.a.c',
            }

        for k,v in d.items() :
            self.assertEqual(str(k), v)


def suite():
    theSuite = unittest.TestSuite()
    theSuite.addTest(unittest.makeSuite(TestLTree))
    return theSuite



if __name__ == '__main__':
    unittest.main( defaultTest='suite' )
