import unittest
from adt4.validity import ValidityDatetime as VT, ValidityDate as VD
from lib4.timeutils import Date as D, Datetime as T
from lib4.timeutils import *



class ValidityDateTime (unittest.TestCase) :

    def test01_Instantiation (self) :
        VT()
        VT("from 2004-03-02 12:31:23 till 2005-03-02 12:31")
        VT("from 2004-03-02 12:31:23 till infinity")
        VT("2004-03-02 12:31:23", "infinity")
        VT("from -infinity till infinity")
        VT((2003, 1, 25), (2003, 4, 21, 12, 34))


    def test02_Comparison (self) :

        v1 = VT("from 2004-03-02 12:31:23 till infinity")
        v2 = VT("2004-03-02 12:31:23")

        self.assertEqual(v1, v2)

        self.failIf(v1 < v2)
        self.failIf(v1 > v2)
        self.failIf(v2 > v1)
        self.failIf(v2 < v1)
        self.failIf(v2 != v1)
        self.failIf(v1 != v2)

        v1 = VT("from 2004-03-12 12:00:58+01:00 till 2004-03-12 12:00:58+01:00")
        v2 = VT("from 2004-03-12 12:00:59+01:00 till infinity")

        self.assertNotEqual(v1, v2)

        self.failUnless(v1 != v2)
        self.failUnless(v1 < v2)
        self.failIf(v1 > v2)
        self.failUnless(v2 > v1)
        self.failIf(v2 < v1)


class ValidityDate (unittest.TestCase) :

    def test01_Instantiation (self) :
        VD()
        VD("from 2004-03-02 till 2005-03-02")
        VD("from 2004-03-02 till infinity")
        VD("2004-03-02", "infinity")
        VD("from -infinity till infinity")
        VD((2003, 1, 25), (2003, 4, 21))


    def test02_Comparison (self) :

        v1 = VD("from 2004-03-02 till infinity")
        v2 = VD("2004-03-02")

        self.assertEqual(v1, v2)

        self.failIf(v1 < v2)
        self.failIf(v1 > v2)
        self.failIf(v2 > v1)
        self.failIf(v2 < v1)
        self.failIf(v2 != v1)
        self.failIf(v1 != v2)

        v1 = VD("from 2004-03-12 till 2004-03-12")
        v2 = VD("from 2004-04-12 till infinity")

        self.assertNotEqual(v1, v2)

        self.failUnless(v1 != v2)
        self.failUnless(v1 < v2)
        self.failIf(v1 > v2)
        self.failUnless(v2 > v1)
        self.failIf(v2 < v1)

    def test03_Reduce (self) :

        # Prekryvajici se
        res = VD.reduce([
            VD('from -infinity till 2004-1-1'),
            VD('from 2004-1-1 till infinity'),
        ])
        self.assertEqual(len(res), 1)
        res, = res
        self.assertEqual(res._from, DATE_MIN)
        self.assertEqual(res._till, DATE_MAX)

        res = VD.reduce([
            VD('from 2000-1-1 till 2001-1-1'),
            VD('from 2000-2-1 till 2000-3-1'),
        ])
        self.assertEqual(len(res), 1)
        res, = res
        self.assertEqual(res._from, Date('2000-1-1'))
        self.assertEqual(res._till, Date('2001-1-1'))

        res = VD.reduce([
            VD('from 2000-2-1 till 2000-3-1'),
            VD('from 2000-1-1 till 2001-1-1'),
        ])
        self.assertEqual(len(res), 1)
        res, = res
        self.assertEqual(res._from, Date('2000-1-1'))
        self.assertEqual(res._till, Date('2001-1-1'))

        # Diskjunktni
        res = VD.reduce([
            VD('from -infinity till 2004-1-1'),
            VD('from 2005-1-1 till infinity'),
        ])
        self.assertEqual(len(res), 2)

        # Sousedni
        res = VD.reduce([
            VD('from 2000-1-1 till 2000-1-31'),
            VD('from 2000-2-1 till 2000-3-1'),
        ])
        self.assertEqual(len(res), 1)
        res, = res
        self.assertEqual(res._from, Date('2000-1-1'))
        self.assertEqual(res._till, Date('2000-3-1'))

        # Vse v jednom
        res = VD.reduce([
            VD('from 2000-1-1 till 2001-1-1'),
            VD('from 2000-1-1 till 2000-1-31'),
            VD('from 2000-2-1 till 2000-3-1'),

            VD('from 3000-1-1 till 3001-1-1'),

            VD('from 2010-1-1 till 2010-1-31'),
            VD('from 2010-2-1 till 2010-4-1'),
            VD('from 2010-3-1 till 2020-1-1'),
        ])
        self.assertEqual(len(res), 3)
        v1, v2, v3 = res
        self.assertEqual(v1._from, Date('2000-1-1'))
        self.assertEqual(v1._till, Date('2001-1-1'))

        self.assertEqual(v2._from, Date('2010-1-1'))
        self.assertEqual(v2._till, Date('2020-1-1'))

        self.assertEqual(v3._from, Date('3000-1-1'))
        self.assertEqual(v3._till, Date('3001-1-1'))

        # Hranicni pripady
        self.assertEqual(VD.reduce([]), [])
        self.assertEqual(
                VD.reduce([VD('from 2005-1-1 till infinity')]),
                [VD('from 2005-1-1 till infinity')]
        )

    def test40_NormalizeWithPriority (self) :

        data1 = [
            # version,  published
            (1, VD('from 2000-1-1 till 2000-1-1')),
            (1, VD('from 2001-1-1 till 2001-1-1')),
            (1, VD('from 2004-1-1 till infinity')),

            (2, VD('from 2004-1-1 till 2004-1-31')),
            (2, VD('from 2004-2-1 till infinity')),

            (3, VD('from 2004-1-1 till 2004-2-1')),
            (3, VD('from 2004-4-1 till infinity')),

            (4, VD('from 2004-5-1 till 2004-6-1')),
            (4, VD('from 2004-6-1 till infinity')),
            (4, VD('from 2004-6-1 till infinity')),
            (4, VD('from infinity till infinity')),

            (5, VD('from 2004-6-1 till 2004-7-1')),
        ]
        res1 = [
            (1, VD(' 2000-01-01 till 2000-01-01')),
            (1, VD(' 2001-01-01 till 2001-01-01')),
            (3, VD(' 2004-01-01 till 2004-02-01')),
            (2, VD(' 2004-02-02 till 2004-03-31')),
            (3, VD(' 2004-04-01 till 2004-04-30')),
            (4, VD(' 2004-05-01 till 2004-05-31')),
            (5, VD(' 2004-06-01 till 2004-07-01')),
            (4, VD(' 2004-07-02 till infinity')),
        ]


        data2 = [
            (1, VD('from 2004-6-1 till 2004-7-1'))
        ]
        res2 = [
            (1, VD('from 2004-06-01 till 2004-07-01'))
        ]


        data3 = [
            (1, VD('from 2004-6-1 till infinity'))
        ]
        res3 = [
            (1, VD('from 2004-06-01 till infinity'))
        ]


        data4 = [
            (1, VD('from 2004-5-1 till infinity')),
            (2, VD('from 2004-6-1 till infinity')),
        ]
        res4 = [
            (1, VD('from 2004-05-01 till 2004-05-31')),
            (2, VD('from 2004-06-01 till infinity')),
        ]

        data5 = [
            (T('2004-08-27 12:44:30+02'),VD('from 2003-01-01 till infinity')),
            (T('2004-08-27 12:45:09+02'),VD('from 2004-08-27 till infinity')),
            (T('2004-08-27 12:59:36+02'),VD('from 2004-08-27 till infinity')),
        ]
        res5 = [
            (T('2004-08-27 12:44:30+02'),VD('from 2003-01-01 till 2004-08-26')),
            (T('2004-08-27 12:59:36+02'),VD('from 2004-08-27 till infinity')),
        ]

        def makeTest (data, res) :
            r = VD.normalizeWithPriority(data)
            self.assertEqual(len(res), len(r))
            for (aver, aval), (rver, rval) in map(
                    None, res, r
            ) :
                self.assertEqual(aver, rver)
                self.assertEqual(aval._from, rval._from)
                self.assertEqual(aval._till, rval._till)

        makeTest(data1, res1)
        makeTest(data2, res2)
        makeTest(data3, res3)
        makeTest(data4, res4)
        makeTest(data5, res5)

def suite():
    theSuite = unittest.TestSuite()
    theSuite.addTest(unittest.makeSuite(ValidityDateTime))
    theSuite.addTest(unittest.makeSuite(ValidityDate))
    return theSuite



if __name__ == '__main__':
    unittest.main( defaultTest='suite' )
