import unittest, sys
from adt4.varchar import *


class TestVarchar(unittest.TestCase):

    def test10_MaxVarchar(self):
        va = varchar[4]
        vb = varchar[:4]
        self.failUnless(va is vb)
        self.assertEqual(va.min_length, 0)
        self.assertEqual(va.max_length, 4)
        self.assertEqual(va.__name__, 'varchar_0_4')
        self.assertEqual(va.__mro__, (va,varchar,str,basestring,object))
        ua = uvarchar[4]
        ub = uvarchar[:4]
        self.failUnless(ua is ub)
        self.assertEqual(ua.min_length, 0)
        self.assertEqual(ua.max_length, 4)
        self.assertEqual(ua.__name__, 'uvarchar_0_4')
        self.assertEqual(ua.__mro__, (ua,uvarchar,unicode,basestring,object))

    def test12_MinVarchar(self):
        v = varchar[4:]
        self.assertEqual(v.min_length, 4)
        self.assertEqual(v.max_length, sys.maxint)
        self.assertEqual(v.__name__, 'varchar_4_%s' % sys.maxint)
        u = uvarchar[4:]
        self.assertEqual(u.min_length, 4)
        self.assertEqual(u.max_length, sys.maxint)
        self.assertEqual(u.__name__, 'uvarchar_4_%s' % sys.maxint)

    def test14_MinMaxVarchar(self):
        v = varchar[4:6]
        self.assertEqual(v.min_length, 4)
        self.assertEqual(v.max_length, 6)
        self.assertEqual(v.__name__, 'varchar_4_6')
        u = uvarchar[4:6]
        self.assertEqual(u.min_length, 4)
        self.assertEqual(u.max_length, 6)
        self.assertEqual(u.__name__, 'uvarchar_4_6')

    def test16_ExactVarchar(self):
        v = varchar[4:4]
        self.assertEqual(v.min_length, 4)
        self.assertEqual(v.max_length, 4)
        self.assertEqual(v.__name__, 'varchar_4_4')
        u = uvarchar[4:4]
        self.assertEqual(u.min_length, 4)
        self.assertEqual(u.max_length, 4)
        self.assertEqual(u.__name__, 'uvarchar_4_4')

    def test20_Instances(self):
        v4 = varchar[4]
        self.assertEqual(v4(''), '')
        self.assertEqual(v4('ahoj'), 'ahoj')
        self.assertRaises(ValueError, v4, 'hello')

    def test22_InstancesMin(self):
        v4 = varchar[4:]
        self.assertEqual(v4('ahoj'), 'ahoj')
        self.assertEqual(v4('foobar'), 'foobar')
        self.assertRaises(ValueError, v4, '')
        self.assertRaises(ValueError, v4, 'foo')

    def test24_InstancesMinMax(self):
        v4 = varchar[4:6]
        self.assertEqual(v4('ahoj'), 'ahoj')
        self.assertEqual(v4('foobar'), 'foobar')
        self.assertRaises(ValueError, v4, '')
        self.assertRaises(ValueError, v4, 'foo')
        self.assertRaises(ValueError, v4, 'foobar7')

    def test26_InstancesExact(self):
        v4 = varchar[4:4]
        self.assertEqual(v4('ahoj'), 'ahoj')
        self.assertRaises(ValueError, v4, '')
        self.assertRaises(ValueError, v4, 'foo')
        self.assertRaises(ValueError, v4, 'fooba')

    def test30_OpGetitem(self):
        v = varchar[4]('ahoj')
        self.assertEqual(v[0], 'a')
        self.assertEqual(v[0].__class__, str)
        self.assertEqual(v[2:], 'oj')

    def test31_OpGetitem(self):
        v = uvarchar[4]('ahoj')
        self.assertEqual(v[0], u'a')
        self.assertEqual(v[0].__class__, unicode)
        self.assertEqual(v[2:], u'oj')

    def test32_OpGetitem(self):
        v = varchar[4]('ahoj')
        self.assertEqual(v.split('o'), ['ah', 'j'])


def suite():
    theSuite = unittest.TestSuite()
    theSuite.addTest(unittest.makeSuite(TestVarchar))
    return theSuite


if __name__ == '__main__':
    unittest.main( defaultTest='suite' )
