import re
from lib4.timeutils import *


__all__ = ["ValidityDatetime", "ValidityDate"]


_splitter = re.compile("from|till", re.I)


class ValidityDatetime (object) :

    __slots__ = "_from", "_till"

    resolution = SECOND
    min = DATETIME_MIN
    max = DATETIME_MAX

    # FIXME secure readonliness of _from/_till attributes,
    # ie. immutability of this type.

    def __new__ (klass, a=None, b=None) :

        if isinstance(a, ValidityDatetime) :
            if b is not None :
                raise ValueError("second parameter should be None")
            return a

        self = object.__new__(klass)

        if isinstance(a, basestring) and "till" in a :
            if b is not None :
                raise ValueError("second parameter should be None")
            a, b = filter(None, map(lambda x:x.strip(), _splitter.split(a)))

        if a is None :
            if b is not None :
                raise ValueError("second parameter should be None")
            a = b = Now()
        else :
            a = Datetime(a)
            if b :
                b = Datetime(b)
            else :
                b = a
        if a > b :
            raise ValueError("till before from %r x %r" % (a, b))

        self._from, self._till = a, b
        return self

    def __repr__ (self) :
        def f (d) :
            if d == self.min :
                return "-infinity"
            elif d == self.max :
                return "infinity"
            return str(d)
        return "%s(%r,%r)" % (
                    self.__class__.__name__, f(self._from), f(self._till)
        )

    def __str__ (self) :
        def f (d) :
            if d == self.min :
                return "-infinity"
            elif d == self.max :
                return "infinity"
            return str(d)
        return "from %s till %s" % (f(self._from), f(self._till))

    def __cmp__ (left, right) :
        lf, lt = left._from, left._till
        rf, rt = right._from, right._till
        if lt < rf : return -1
        if lf > rt : return 1
        return 0

    def __lt__ (self, other) : return self.__cmp__(other) < 0
    def __le__ (self, other) : return self.__cmp__(other) < 1
    def __eq__ (self, other) : return self.__cmp__(other) == 0
    def __ne__ (self, other) : return self.__cmp__(other) != 0
    def __gt__ (self, other) : return self.__cmp__(other) > 0
    def __ge__ (self, other) : return self.__cmp__(other) > -1

    def __hash__ (self) :
        return hash(self._from)






class ValidityDate (ValidityDatetime) :

    __slots__ = ()

    resolution = DAY
    min = DATE_MIN
    max = DATE_MAX

    def __new__ (klass, a=None, b=None) :

        if isinstance(a, ValidityDate) :
            if b is not None :
                raise ValueError("second parameter should be None")
            return a

        self = object.__new__(klass)

        if isinstance(a, basestring) and "till" in a :
            if b is not None :
                raise ValueError("second parameter should be None")
            a, b = filter(None, map(lambda x:x.strip(), _splitter.split(a)))

        if a is None :
            if b is not None :
                raise ValueError("second parameter should be None")
            a = b = Today()
        else :
            a = Date(a)
            if b :
                b = Date(b)
            else :
                b = a
        if a > b :
            raise ValueError("till before from %r x %r" % (a, b))

        self._from, self._till = a, b
        return self

    def subtract (self, other) :
        if self != other :
            return self
        klass = self.__class__
        if self._from < other._from :
            pass
        if self._from > other._from :
            pass
        else :
            return klass(
                        min(self._from, other._from),
                        max(self._till, other._till)
            )

    def reduce (klass, items) :
        if len(items) < 2 :
            return items

        items.sort(lambda l,r: cmp((l._from, l._till), (r._from, r._till)))

        rl = len(items)
        i = 1
        prev = items[0]
        while i < rl :
            val = items[i]
            if prev != val :
                if val._from - prev._till == klass.resolution :
                    rl -= 1
                    prev = items[i-1] = klass(prev._from, val._till)
                    del items[i]
                else :
                    prev = val
                    i += 1
                continue
            else :
                rl -= 1
                prev = items[i-1] = klass(
                    min(prev._from, val._from),
                    max(prev._till, val._till)
                )
                del items[i]
        return items
    reduce = classmethod(reduce)


    def union (klass, items) :
        pass
    union = classmethod(union)


    def intersection (klass, items) :
        pass
    intersection = classmethod(intersection)




    def normalizeWithPriority (klass, data) :
        NAN = object()
        if len(data) < 2 :
            return data

        data = list(data)
        data.sort()

        datesIdx = {}
        for ver, val in data :
            datesIdx[val._from] = 0
            datesIdx[val._till] = 0
            if val._from > DATE_MIN :
                datesIdx[val._from-DAY] = 0
            if val._till < DATE_MAX :
                datesIdx[val._till+DAY] = 0

        datesArray = datesIdx.keys()
        datesArray.sort()
        for i,d in enumerate(datesArray) :
            datesIdx[d] = i

        timeline = [NAN] * len(datesIdx)
        for ver, val in data :
            for i in xrange(datesIdx[val._from], datesIdx[val._till]+1) :
                timeline[i] = ver

        result = []
        start = datesArray[0]
        for (fi,fv), (si,sv)  in map(
                    None, enumerate(timeline[0:-1]), enumerate(timeline[1:])
        ) :
            fi += 0
            si += 1
            if fv != sv :
                if fv is not NAN :
                    result.append((fv, ValidityDate(start, datesArray[fi])))
                start = datesArray[si]
        if sv is not NAN :
            result.append((sv, ValidityDate(start, datesArray[si])))
        return result
    normalizeWithPriority = classmethod(normalizeWithPriority)
