/*
 * TODO  
 *   - remove strndup
 *   - replace all allocations with palloc/pfree
 */

#include <postgres.h>
#include <utils/timestamp.h>
#include <utils/datetime.h>
#include <fmgr.h>
#include <miscadmin.h>

#include "validity.h"

extern char *strndup(const char *s, size_t n);

PG_MODULE_MAGIC;

ValidityDate * validity_date_in(char * str)
{
    ValidityDate   *result;
    char           *from=NULL, *till=NULL, *q;

    if( strncasecmp(str, "from ", 5) != 0 ) {
        elog(ERROR, "missing \'from\' in validity representation \"%s\"", str);
        return NULL;
    }
    for( q = str + 5; *q; ++q ) {
        if(strncasecmp(q, " till ", 6) == 0) {
            from = (char *) strndup(str+5, q-str-5);
            till = strdup(q+6);
            break;
        }
    }
    if(!till) {
        elog(ERROR, "missing \'till\' in validity representation \"%s\"", str);
        return NULL;
    }
    result = (ValidityDate *) palloc(sizeof(ValidityDate));
    if(strcmp(from, "-infinity") == 0) {
        result->from = DATE_MIN;
        /* 
           DatumGetDateADT(
                DirectFunctionCall1(date_in, CStringGetDatum("0001-1-1")));
           elog(NOTICE, "-infinity: %d", result->from);
        */
    } else {
        result->from = DatumGetDateADT(
                DirectFunctionCall1(date_in, CStringGetDatum(from)));
    }
    free(from);
    if(strcmp(till, "infinity") == 0) {
        result->till = DATE_MAX;
        /*
           result->till = DatumGetDateADT(
                DirectFunctionCall1(date_in, CStringGetDatum("9999-12-31")));
           elog(NOTICE, "infinity: %d", result->till);
        */
    } else {
        result->till = DatumGetDateADT(
                DirectFunctionCall1(date_in, CStringGetDatum(till)));
    }
    free(till);
    if(result->from < DATE_MIN) {
        elog(ERROR,
                "\'from\' is lower than -infinity (0001-1-1) (%d<%d)",
                result->from, DATE_MIN
        );
    }
    if(result->till > DATE_MAX) {
        elog(ERROR,
                "\'till\' is greater than infinity (9999-12-31) (%d>%d)",
                result->from, DATE_MIN
        );
    }
    if(result->till < result->from) {
        elog(ERROR, "\'till\' before \'from\' in \'%s\'", str);
        return NULL;
    }
    return result;
}

char * validity_date_out(ValidityDate *val)
{
    char	*s, buf[MAXDATELEN*2+12];

    if (val == NULL) return NULL;

    strcpy(buf, "from ");
    if(val->from <= DATE_MIN) {
        strcat(buf, "-infinity");
    } else {
        s =  DatumGetCString(DirectFunctionCall1(date_out, val->from));
        strcat(buf, s);
        pfree(s);
    }
    strcat(buf, " till ");
    if(val->till >= DATE_MAX) {
        strcat(buf, "infinity");
    } else {
        s =  DatumGetCString(DirectFunctionCall1(date_out, val->till));
        strcat(buf, s);
        pfree(s);
    }
    return pstrdup(buf);
}

ValidityDate *validity_date_from_date(DateADT *d)
{
    ValidityDate *result;

    result = (ValidityDate *) palloc(sizeof(ValidityDate));
    result->from = (DateADT) d;
    result->till = (DateADT) d;
    return result;
}

ValidityDate *validity_date_from_date2(DateADT *from, DateADT *till)
{
    ValidityDate *result;

    result = (ValidityDate *) palloc(sizeof(ValidityDate));
    result->from = (DateADT) from;
    result->till = (DateADT) till;
    return result;
}

Datum validity_from_date(ValidityDate *val)
{
    return DateADTGetDatum(val->from);
}

Datum validity_till_date(ValidityDate *val)
{
    return DateADTGetDatum(val->till);
}

int4 validity_date_cmp(ValidityDate *a1, ValidityDate *a2)
{
    if(a1->till < a2->from) return -1;
    if(a1->from > a2->till) return 1;
    return 0;
}

bool validity_date_lt(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) < 0);
}

bool validity_date_le(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) <= 0);
}

bool validity_date_eq(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) == 0);
}

bool validity_date_ge(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) >= 0);
}

bool validity_date_gt(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) > 0);
}

bool validity_date_ne(ValidityDate *a1, ValidityDate *a2)
{
    return (validity_date_cmp(a1, a2) != 0);
}














ValidityTimestampTz * validity_timestamptz_in(char * str)
{
    ValidityTimestampTz   *result;
    char                  *from=NULL, *till=NULL, *q;
    Datum                 d;

    if( strncasecmp(str, "from ", 5) != 0 ) {
        elog(ERROR, "missing \'from\' in validity representation \"%s\"", str);
        return NULL;
    }
    for( q = str + 5; *q; ++q ) {
        if(strncasecmp(q, " till ", 6) == 0) {
            from = (char *) strndup(str+5, q-str-5);
            till = strdup(q+6);
            break;
        }
    }
    if(!till) {
        elog(ERROR, "missing \'till\' in validity representation \"%s\"", str);
        return NULL;
    }
    result = (ValidityTimestampTz *) palloc(sizeof(ValidityTimestampTz));

    d =  DirectFunctionCall3( timestamptz_in,
                               CStringGetDatum(from),
                               ObjectIdGetDatum(InvalidOid),
                               Int32GetDatum(0)
                              );
    free(from);
    result->from = DatumGetTimestampTz(d);

    d =  DirectFunctionCall3( timestamptz_in,
                               CStringGetDatum(till),
                               ObjectIdGetDatum(InvalidOid),
                               Int32GetDatum(0)
                              );
    free(till);
    result->till = DatumGetTimestampTz(d);
    if(result->till < result->from) {
        elog(ERROR, "\'till\' before \'from\' in \'%s\'", str);
        return NULL;
    }
    return result;
}

char * validity_timestamptz_out(ValidityTimestampTz *val)
{
    char	*s, buf[128];

    if (val == NULL) return NULL;

    strcpy(buf, "from ");
    s =  DatumGetCString(
                DirectFunctionCall1(
                        timestamptz_out,
                        TimestampTzGetDatum(val->from)
                ));
    strcat(buf, s);
    pfree(s);
    strcat(buf, " till ");
    s =  DatumGetCString(
                DirectFunctionCall1(
                        timestamptz_out,
                        TimestampTzGetDatum(val->till)
                ));
    strcat(buf, s);
    pfree(s);
    return pstrdup(buf);
}

ValidityTimestampTz *validity_timestamptz_from_timestamptz(TimestampTz *t)
{
    ValidityTimestampTz *result;

    /* FIXME round up to seconds */
    result = (ValidityTimestampTz *) palloc(sizeof(ValidityTimestampTz));
    result->from = *t;
    result->till = *t;
    return result;
}

ValidityTimestampTz *validity_timestamptz_from_timestamptz2(
            TimestampTz *from, TimestampTz *till)
{
    ValidityTimestampTz *result;

    /* FIXME round up to seconds */
    result = (ValidityTimestampTz *) palloc(sizeof(ValidityTimestampTz));
    result->from = *from;
    result->till = *till;
    return result;
}


Datum validity_from_timestamptz(ValidityTimestampTz *val)
{
    return TimestampTzGetDatum(val->from);
}

Datum validity_till_timestamptz(ValidityTimestampTz *val)
{
    return TimestampTzGetDatum(val->till);
}

int4 validity_timestamptz_cmp(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    if(a1->till < a2->from) return -1;
    if(a1->from > a2->till) return 1;
    return 0;
}

bool validity_timestamptz_lt(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) < 0);
}

bool validity_timestamptz_le(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) <= 0);
}

bool validity_timestamptz_eq(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) == 0);
}

bool validity_timestamptz_ge(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) >= 0);
}

bool validity_timestamptz_gt(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) > 0);
}

bool validity_timestamptz_ne(ValidityTimestampTz *a1, ValidityTimestampTz *a2)
{
    return (validity_timestamptz_cmp(a1, a2) != 0);
}

