CREATE OR REPLACE FUNCTION validity_date_in(cstring)
RETURNS validity_date
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_out(validity_date)
RETURNS cstring
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE TYPE validity_date (
	INTERNALLENGTH = 8,
	INPUT = validity_date_in,
	OUTPUT = validity_date_out
);


CREATE OR REPLACE FUNCTION validity_date(date)
RETURNS validity_date
AS '$libdir/validity.so',
   'validity_date_from_date'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date(date, date)
RETURNS validity_date
AS '$libdir/validity.so',
   'validity_date_from_date2'
LANGUAGE 'C' IMMUTABLE STRICT;


CREATE CAST (date AS validity_date)
WITH FUNCTION validity_date(date) AS IMPLICIT;


CREATE OR REPLACE FUNCTION validity_date_lt(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_le(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_eq(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_ge(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_gt(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_date_ne(validity_date, validity_date)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OPERATOR < (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	NEGATOR = >=,
	PROCEDURE = validity_date_lt
);

CREATE OPERATOR <= (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	NEGATOR = >,
	PROCEDURE = validity_date_le
);

CREATE OPERATOR = (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	COMMUTATOR = =,
	NEGATOR = <>,
	PROCEDURE = validity_date_eq
);

CREATE OPERATOR >= (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	NEGATOR = <,
	PROCEDURE = validity_date_ge
);

CREATE OPERATOR > (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	NEGATOR = <=,
	PROCEDURE = validity_date_gt
);

CREATE OPERATOR <> (
	LEFTARG = validity_date,
	RIGHTARG = validity_date,
	NEGATOR = =,
	PROCEDURE = validity_date_ne
);

CREATE OR REPLACE FUNCTION validity_date_cmp(validity_date, validity_date)
RETURNS integer
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_from(validity_date)
RETURNS date
AS '$libdir/validity.so', 'validity_from_date'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_till(validity_date)
RETURNS date
AS '$libdir/validity.so', 'validity_till_date'
LANGUAGE 'C' IMMUTABLE STRICT;


CREATE OPERATOR CLASS validity_date_ops
DEFAULT FOR TYPE validity_date USING btree 
AS
        OPERATOR        1       < ,
        OPERATOR        2       <= ,
        OPERATOR        3       = ,
        OPERATOR        4       >= ,
        OPERATOR        5       > ,
        FUNCTION        1       validity_date_cmp(validity_date, validity_date);












CREATE OR REPLACE FUNCTION validity_timestamptz_in(cstring)
RETURNS validity_timestamptz
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_out(validity_timestamptz)
RETURNS cstring
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE TYPE validity_timestamptz (
	INTERNALLENGTH = 16,
	INPUT = validity_timestamptz_in,
	OUTPUT = validity_timestamptz_out
);


CREATE OR REPLACE FUNCTION validity_timestamptz(timestamptz)
RETURNS validity_timestamptz
AS '$libdir/validity.so',
   'validity_timestamptz_from_timestamptz'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz(timestamptz, timestamptz)
RETURNS validity_timestamptz
AS '$libdir/validity.so',
   'validity_timestamptz_from_timestamptz2'
LANGUAGE 'C' IMMUTABLE STRICT;


CREATE CAST (timestamptz AS validity_timestamptz)
WITH FUNCTION validity_timestamptz(timestamptz) AS IMPLICIT;


CREATE OR REPLACE FUNCTION validity_timestamptz_lt(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_le(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_eq(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_ge(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_gt(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_timestamptz_ne(validity_timestamptz, validity_timestamptz)
RETURNS bool
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OPERATOR < (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	NEGATOR = >=,
	PROCEDURE = validity_timestamptz_lt
);

CREATE OPERATOR <= (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	NEGATOR = >,
	PROCEDURE = validity_timestamptz_le
);

CREATE OPERATOR = (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	COMMUTATOR = =,
	NEGATOR = <>,
	PROCEDURE = validity_timestamptz_eq
);

CREATE OPERATOR >= (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	NEGATOR = <,
	PROCEDURE = validity_timestamptz_ge
);

CREATE OPERATOR > (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	NEGATOR = <=,
	PROCEDURE = validity_timestamptz_gt
);

CREATE OPERATOR <> (
	LEFTARG = validity_timestamptz,
	RIGHTARG = validity_timestamptz,
	NEGATOR = =,
	PROCEDURE = validity_timestamptz_ne
);

CREATE OR REPLACE FUNCTION validity_timestamptz_cmp(validity_timestamptz, validity_timestamptz)
RETURNS integer
AS '$libdir/validity.so'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_from(validity_timestamptz)
RETURNS timestamptz
AS '$libdir/validity.so', 'validity_from_timestamptz'
LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION validity_till(validity_timestamptz)
RETURNS timestamptz
AS '$libdir/validity.so', 'validity_till_timestamptz'
LANGUAGE 'C' IMMUTABLE STRICT;


CREATE OPERATOR CLASS validity_timestamptz_ops
DEFAULT FOR TYPE validity_timestamptz USING btree 
AS
        OPERATOR        1       < ,
        OPERATOR        2       <= ,
        OPERATOR        3       = ,
        OPERATOR        4       >= ,
        OPERATOR        5       > ,
        FUNCTION        1       validity_timestamptz_cmp(validity_timestamptz, validity_timestamptz);
