import sys

__all__ = ['varchar', 'uvarchar']


class varchar_meta(type):

    def __getitem__(klass, max):
        min = 0
        name = '%s_%d_%d' % (klass.__name__, min, max)
        glob = globals()
        cls = glob.get(name)
        if cls is None:
            cls = glob[name] = type(klass)(name, (klass,), {'max_length':max})
        return cls

    def __getslice__(klass, min, max):
        name = '%s_%d_%d' % (klass.__name__, min, max)
        glob = globals()
        cls = glob.get(name)
        if cls is None:
            cls = glob[name] = type(klass)(
                name, (klass,), {'min_length':min, 'max_length':max}
            )
        return cls

    def __call__(klass, s, *a, **kw):
        if len(s) > klass.max_length:
            raise ValueError("max_length exceeded", klass.max_length, s)
        if len(s) < klass.min_length:
            raise ValueError("min_length required", klass.min_length, s)
        return type.__call__(klass, s, *a, **kw)


class varchar(str):
    __metaclass__ = varchar_meta
    __slots__     = ()
    max_length    = sys.maxint
    min_length    = 0

class uvarchar(unicode):
    __metaclass__ = varchar_meta
    __slots__     = ()
    max_length    = sys.maxint
    min_length    = 0
