from wm4.api import *
from wm4.web.authorization import ListPerm
from lib4.ast_builder import *
from adt4.serial import Serial

__all__ = ['ACL', 'ListPerm', 'AccessList', 'AccessListComponent']


class ACL(web.Permission):
    
    def __init__(self, domain, key=None, role=None, default=True):
        self.domain  = domain
        self.key     = key
        self.role    = role
        self.default = default

@web.PermissionChecker.allows.when(
    'self in web.PermissionChecker and perm in ACL')
def allow_acl(self, perm, environ):
    ws = settings.workspace()
    acl = ws['/acl'].get(perm.domain, perm.key, perm.role)
    if acl is None:
        if perm.default:
            return True
        else:
            return web.Denial()
    return self.allows(acl, environ)


class AccessList(schema.Entity):
    pkey      = schema.Attribute(Serial)
    domain    = schema.Attribute(str, schema.Required)
    key       = schema.Attribute(unicode)
    role      = schema.Attribute(str)
    acl_expr  = schema.Attribute(unicode, schema.Required)

    binding.metadata(
        schema.PrimaryKey('pkey'), schema.Unique('domain', 'key', 'role')
    )



class AccessListComponent(schema.SchemaComponent):
    
    permissions = binding.Make(
                    lambda: config.Property('comp4.acl.permissions')())
    permissions_rev = binding.Make(
                    lambda s: dict((v,k) for k,v in s.permissions.items()))
    names = binding.Make(
                    lambda s: dict((v,k) for k,v in s.permissions.items()))
    builder = binding.Make(
                    lambda s: ACLBuilder(s.permissions), suggestParent=False)
    
    globals = {
        'list':ListPerm, 'user':web.UserName, 'group':web.MemberOf,
        'auth':web.Authenticated,
    }

    checker = binding.Make(lambda s: web.permission_checker())

    cache = storage.TxnAttr(dict)

    def get(self, domain, key=None, role=None):
        key = key and str(key) or None
        idx = (domain, key, role)
        acl = self.cache.get(idx, NOT_FOUND)
        if acl is NOT_FOUND:
            expr = ' and '.join(e for e in self.workspace('''[
                a.acl_expr for a in AccessList
                   if a.domain == domain
                  and a.key == key
                  and a.role == role
            ]'''))
            acl = self.cache[idx] = self.checker.parse(expr)
        return acl
    
    def gethier(self, domain, keys, role, root=None):
        if root is None:
            aclAll = ListPerm(default=web.Nobody())
        else:
            assert isinstance(root, ListPerm), repr(root)
            aclAll = root
        aclHere = ListPerm()
        top = str(keys[-1])
        keys = map(str, keys)
        hier = dict((key, expr) for key, expr in self.workspace('''[
            (a.key, a.acl_expr) for a in AccessList
               if a.domain == domain
              and a.key in keys
              and a.role == role
            ]'''
        ))
        for key in keys:
            expr = hier.get(key)
            if expr is None:
                continue
            acl = self.checker.parse(expr)
            if key == top:
                aclHere = acl
            aclAll = aclAll.copy(acl)
        return aclAll, aclHere
