# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.schema.tests.utils import MockWorkspaceCase
from wm4.web.tests.utils import *
from wm4.web.authorization import *
from comp4.acl.api import *
from comp4.users.api import *



g10 = Group(gid=10, name='g10')
g20 = Group(gid=20, name='g20')
u1 = User(uid=1, name='User1', login='u1', email='u1@x')
u2 = User(uid=2, name='User2', login='u2', email='u2@x')
u1.groups = [g10]
u2.groups = [g10]



class TestACL(WebCase, MockWorkspaceCase):

    cfgURLs = (
        'pkgfile:wm4/core.ini',
        'pkgfile:wm4.web/web.ini',
        'pkgfile:comp4.acl/comp.ini',
    )

    def setUp(self):
        WebCase.setUp(self)
        MockWorkspaceCase.setUp(self)

    def tearDown(self):
        MockWorkspaceCase.tearDown(self)
        WebCase.tearDown(self)

    def setUser(self, env, usr):
        class Auth:
            def getUser(self, env):
                return usr
        env.setdefault('windmill.authenticators', {})['remote_user'] = Auth()

    def test01_OutsideTransaction(self):
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop')
        self.assertRaises(exceptions.OutsideTransaction, pc.allows, perm, env)

    def test10_EmptyACL(self):
        self.expects(transaction='begin')
        self.expects(
            sql= ("SELECT acl_expr FROM access_list "
                  "WHERE (domain = 'shop') AND (key ISNULL) AND (role ISNULL)"),
            res = [],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        perm = ACL('shop')

        storage.begin()
        self.failUnless(pc.allows(perm, env))
        storage.commit()

    def test12_ACLWithUserName(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                  "WHERE (domain = 'shop') AND (key ISNULL) AND (role ISNULL)"),
          res = [["user('u1')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop')

        storage.begin()
        self.failUnless(pc.allows(perm, env))
        storage.commit()

    def test13_ACLWithDifferentUserName(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                  "WHERE (domain = 'shop') AND (key ISNULL) AND (role ISNULL)"),
          res = [["user('u2')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop')

        storage.begin()
        self.failIf(pc.allows(perm, env))
        storage.commit()

    def test14_ACLWithKeyAndRoleAgainstUserName(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                "WHERE (domain = 'shop') AND (key = '12') AND (role = 'edit')"),
          res = [["user('u1')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop', key=12, role='edit')

        storage.begin()
        self.failUnless(pc.allows(perm, env))
        storage.commit()

    def test16_MoreThanOneRecordInDatabaseCreatesAndExpression(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                "WHERE (domain = 'shop') AND (key = '12') AND (role = 'edit')"),
          res = [["group('g10')"], ["user('u1')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop', key=12, role='edit')

        storage.begin()
        self.failUnless(pc.allows(perm, env))
        storage.commit()
        
    def test17_MoreThanOneRecordInDatabaseCreatesAndExpression(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                "WHERE (domain = 'shop') AND (key = '12') AND (role = 'edit')"),
          res = [["group('g10')"], ["user('u2')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop', key=12, role='edit')

        storage.begin()
        self.failIf(pc.allows(perm, env))
        storage.commit()
        
    def test20_ACLWithComplexExpression(self):
        self.expects(transaction='begin')
        self.expects(
          sql= ("SELECT acl_expr FROM access_list "
                "WHERE (domain = 'shop') AND (key ISNULL) AND (role ISNULL)"),
          res = [["user('u2') | group('g10')"]],
        )
        self.expects(transaction='commit')
        pc = web.PermissionChecker()
        env = self.env()
        self.setUser(env,u1)
        perm = ACL('shop')

        storage.begin()
        self.failUnless(pc.allows(perm, env))
        storage.commit()


def suite():
    return TestSuite((
        makeSuite(TestACL),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
