# -*- coding: utf-8 -*-

import random
import re

from wm4.api import *
from wm4.web.forms import api as forms
from wm4.web.sessions import SecureValue
from xml4 import xhtml as xh

__all__ = ['JSExprAntispamCtrl', 'JSExprAntispamWidget']


CISLA = range(1,10)
CISLA_TXT = {
        'cz': [u'nula', u'jedna', u'dvě', u'tři', u'čtyři', u'pět',
            u'šest', u'sedm', u'osm', u'devět', u'deset',]
        }
OPERATORY = '+'
OPERATORY_TXT = {
        'cz': [u'plus']
        }
RE_INT = re.compile('([0-9]+)')

class AntispamExpr(SecureValue):
    attributes = [('value','U'), ('timestamp','d')]


def generate_expression():
    p,d = random.choice(CISLA), random.choice(CISLA)
    c1, c2 = max(p,d), min(p,d)
    return '%d %s %d' % (c1, random.choice(OPERATORY), c2)

def expression_to_text(expr, lng='cz'):
    if lng in CISLA_TXT and lng in OPERATORY_TXT:
        t = ''
        for p in RE_INT.split(expr):
            try:
                n = CISLA_TXT[lng][int(p)]
            except:
                n = p
            t += n
        return t
    else:
        return expr

class JSExprAntispamCtrl(forms.InputCtrl):

    ctrlType = int
    ctrlWidget = web.FORM_WIDGET('input.antispam.js_expr')

    expr = forms.ClientState(secure=True)

    def ctrlOnInit(self):
        forms.InputCtrl.ctrlOnInit(self)

        # Replace expr with random generated expression on every
        # form load.
        self.saved = self.expr
        self.expr = generate_expression()

    def commitSelf(self):
        # sent expr was replaced by newly generated
        expr = self.saved
        if not expr:
            self.ctrlError = schema.FeatureError('validator', u'Chybná odpověď')
            forms.OnErrorEvent(self, error=self.ctrlError)
            return False
        try:
            self.newValue = self.parseValue(self.inputText)
        except schema.SchemaError, e:
            self.ctrlError = e
            self.newValue = NOT_GIVEN
            forms.OnErrorEvent(self, error=e)
            return False
        if eval(expr) != self.newValue:
            self.ctrlError = schema.FeatureError('validator', u'Chybná odpověď')
            forms.OnErrorEvent(self, error=self.ctrlError)
            return False
        # Don't announce changes here, only errors!
        return True


class JSExprAntispamWidget(forms.InputWidget):
    type  = 'text'

    def view(self, state, felem, req):
        return None

    def hidden(self, state, felem, req):
        attrs = {
            'type' : 'hidden',
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        attrs.update(self.acquireAttrs(state, ('class','id')))
        return xh.input(attrs)

    def edit(self, state, felem, req):
        # we need xGetElementById
        web.getResource('wm4.web.static/js/x_core.js').render(req)
        attrs = {
            'type' : self.type,
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','tabindex','accesskey') #+ALL_JS,
        ))

        divid = 'JS-EXPR-VALIDATOR-DIV-%s' % id(felem)
        attrs['id'] = inpid = 'JS-EXPR-VALIDATOR-INP-%s' % id(felem)
        return xh.div(
            felem.expr, xh.input(attrs),
            xh.script((
                "xGetElementById('%s').value = %s;"
                "xDisplay(xGetElementById('%s'), 'none');"
                ) % (inpid, felem.expr, divid),
                type="text/javascript",),
            id=divid,
        )


class JSExprAntispamWidgetNoAuto(JSExprAntispamWidget):
    type  = 'text'

    def edit(self, state, felem, req):
        # we need xGetElementById
        #web.getResource('wm4.web.static/js/x_core.js').render(req)
        attrs = {
            'type' : self.type,
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','tabindex','accesskey') #+ALL_JS,
        ))

        divid = 'JS-EXPR-VALIDATOR-DIV-%s' % id(felem)
        attrs['id'] = inpid = 'JS-EXPR-VALIDATOR-INP-%s' % id(felem)
        return xh.div(
            felem.expr, ' = ', xh.input(attrs),
            id=divid,
        )




web.FORM_WIDGET('input.antispam.js_expr', JSExprAntispamWidget)
