#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms import api as forms
from wm4.web.forms.listings import *
from wm4.utils.clspatch import patch

from comp4.dbgettext.web import DBGettextCtrl, DBGettextHtmlCtrl

from lib4.containers import ReadonlySortedDict
from lib4.images import loadImage, getMIME, getStorageSize, ImageConverter

from entities import *


__all__ = ['BannerListing', 'BannerForm',
           'BannerLocationListing', 'BannerLocationForm', ]


_ = i18n.MessageIDFactory('comp4.banner')






class BannerForm(forms.makeFormCtrl(Banner)):

    class server(forms.InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            return config.Property('comp4.banner.server_name')()

    class location(forms.SelectCtrl):
        ctrlLabel = _(u'Location')
        ctrlRequired = True

        selectOptions = binding.Make(lambda s: [
            (l.pkey, u'%s (%s x %s)' % (l.name, l.max_width, l.max_height), l) 
            for l in settings.workspace()('''[l for l in 'banner/Location']''')
            ])

    class file(forms.FileUploadCtrl):
        ctrlType = BannerFile
        ctrlLabel = _(u'File')

        def makeFile(self):
            return settings.workspace().new(BannerFile,
                filename=self.filename, blob=open(self.path), size=self.size,
                content_type=self.content_type,)

        def getInputText(self):
            if not self.formAction:
                return ''
            val = self.ctrlReq.getUpload(self.ctrlId)
            if val is not None:
                self.filename = val.filename
                self.path = val.path
                self.deleted = 0
                self.content_type = val.content_type
                self.size = val.size
                if self.content_type.startswith('image/'):
                    img = loadImage(val.path) # XXX may raise error
                    loc = settings.workspace().get(
                            'banner/Location', self['../location'].inputText)
                    converter = ImageConverter(
                            maxSize=(loc.max_width, loc.max_height), 
                            ratio='fixed')
                    _img = img
                    img = converter(_img)
                    if _img is not img:
                        img.save(val.path, img.format)
                    self.content_type = getMIME(img)
                    self.width, self.height = img.size
                    self.size = getStorageSize(img)
            return ''
        inputText = binding.Make(getInputText)

    class _alt(DBGettextCtrl):
        ctrlLabel = _(u'Alternative text')
        set_on_change = '../alt'

    class _description(DBGettextHtmlCtrl):
        ctrlLabel = _(u'Description')
        set_on_change = '../description'


    patch({
        'url.ctrlLabel': _(u'URL'),
        'alt.ctrlLabel': _(u'Alternative text'),
        'description.ctrlLabel': _(u'Description'),

        'ordernum.ctrlLabel': _(u'Order in location'),

        'max_impressions.ctrlLabel': _(u'Limit of impressions count'),
        'max_clicks.ctrlLabel': _(u'Limit of clicks count'),
        'show_from.ctrlLabel': _(u'Show from'),
        'show_till.ctrlLabel': _(u'Show till'),

        'cnt_impressions.ctrlReadOnly': True,
        'cnt_impressions.ctrlLabel': _(u'Impressions count'),
        'cnt_clicks.ctrlReadOnly': True,
        'cnt_clicks.ctrlLabel': _(u'Clicks count'),
        })




class BannerListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'banner/Banner']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class cnt_impressions(Column):
        queryExpr = 'obj.cnt_impressions'
        ctrlLabel = _(u'Impressions')

    class cnt_clicks(Column):
        queryExpr = 'obj.cnt_clicks'
        ctrlLabel = _(u'Clicks')

    class location(SelectFilter):
        queryExpr = 'obj.location'
        ctrlLabel = _(u'Location')
        noneLabel = _(u'-- location --')

        def selectOptions(self):
            ws = settings.workspace()
            return [(l._oid, l.name, l)
                    for l in ws('''[l for l in 'banner/Location']''')]
        selectOptions = binding.Make(selectOptions)






class BannerLocationForm(forms.makeFormCtrl(BannerLocation)):
    patch({
        'name.ctrlLabel': _(u'Name'),
        'max_width.ctrlLabel': _(u'Location width'),
        'max_height.ctrlLabel': _(u'Location height'),
        })


class BannerLocationListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'banner/Location']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class max_width(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'Location width')

    class max_height(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'Location height')

    class domain(Column):
        queryExpr = 'obj.domain'
        ctrlLabel = _(u'Doména')



