# -*- coding: ISO-8859-2 -*-

from datetime import date

from wm4.api import *
from wm4.schema.api import *

from adt4.serial import Serial

from comp4.files import api as files
import comp4.dbgettext.api as dbgettext

__all__ = ['BannerLocation', 'Banner', 'BannerFile',
           'BannerComponent',
        ]

class BannerLocation(Entity):
    pkey = Attribute(Serial)

    name = Attribute(unicode, Required)
    _name = dbgettext.Translations('name')

    max_width = Attribute(int, Required)
    max_height = Attribute(int, Required)

    domain = Attribute(unicode)

    binding.metadata(PrimaryKey('pkey'))


class Banner(Entity):
    pkey = Attribute(Serial)
    server = Attribute(unicode, Required)
    location = Reference(
                'comp4.banner.entities.BannerLocation',
                Required)

    url = Attribute(unicode, Required)
    alt = Attribute(unicode, Required)
    _alt = dbgettext.Translations('alt')

    description = Attribute(unicode)
    _description = dbgettext.Translations('description', useDefault=False)

    ordernum = Attribute(int)
    max_impressions = Attribute(int)
    max_clicks = Attribute(int)
    show_from = Attribute(date)
    show_till = Attribute(date)

    cnt_impressions = Attribute(int, Required, Default(0))
    cnt_clicks = Attribute(int, Required, Default(0))

    file = Reference(
                'comp4.banner.entities.BannerFile',
                Required)

    binding.metadata(PrimaryKey('pkey'))

class BannerFile(files.File):
    pass


@schema.Workspace.do_remove.before('obj in Banner')
def remove_files(ws, obj):
    ws.remove(obj.file)



class BannerComponent(SchemaComponent):
    txnAttrs = ()

    def choice(self, location, **kw):
        ws = settings.workspace()
        today = date.today()

        if isinstance(location, BannerLocation):
            loid = location._oid
        else:
            loid = int(location)

        q = ws.parse('''[
            (banner, bf)
                for banner in 'banner/Banner'
                for bf in 'banner/File'
                 if banner.location.pkey==loid
                and banner.file==bf
                and (banner.max_impressions is None
                     or banner.max_impressions>banner.cnt_impressions)
                and (banner.max_clicks is None
                     or banner.max_clicks>banner.cnt_clicks)
                and (banner.show_from is None
                     or banner.show_from<=today)
                and (banner.show_till is None
                     or banner.show_till>=today)
                and query.OrderBy(query.Random())
            ]''')
        q.limit = query.Const(1)
        banner = list(ws(q))
        if banner:
            return banner[0][0]
        else:
            return None

    def list(self, location, limit, orderby="random"):
        ws = settings.workspace()
        if isinstance(location, BannerLocation):
            idloc = location._oid
        else:
            idloc = location
        today = date.today()
        if orderby=='ordernum':
            orderby = 'query.OrderBy(banner.ordernum)'
        else:
            orderby = 'query.OrderBy(query.Random())'

        q = ws.parse('''[
            (banner, bf)
                for banner in 'banner/Banner'
                for bf in 'banner/File'
                 if banner.location._oid==idloc
                and banner.file==bf
                and (banner.max_impressions is None
                     or banner.max_impressions>banner.cnt_impressions)
                and (banner.max_clicks is None
                     or banner.max_clicks>banner.cnt_clicks)
                and (banner.show_from is None
                     or banner.show_from<=today)
                and (banner.show_till is None
                     or banner.show_till>=today)
                and %s
            ]''' % orderby)
        q.limit = query.Const(limit)
        return [b[0] for b in ws(q)]


