#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import date, timedelta
import calendar

from lib4.imports import importObject

from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import *

from comp4.cms import api as cms


_ = i18n.MessageIDFactory('comp4.cms')
_w = i18n.MessageIDFactory('wm4')


class CalendarListing(ListingCtrl):
    today = binding.Make(lambda: date.today())

    itemLimit = None

    baseQuery = query.query('''[
        (rn, cpv.changeset) 
            for rn in cms.RepositoryNode
            for cpv in cms.CmsPublicVersion
             if cpv.inode == rn.inode
            and rn.changeset_from <= cpv.changeset <= rn.changeset_till
            and cpv.pstart <= date.today() <= cpv.pend
            ]''')

    items = binding.Make(lambda s: [
                s.workspace.get('cms/PublicNode', n.inode, default=None)
                for n, chset in s.workspace(s.itemQuery)])

    class tag(HiearchicalMultiCheckboxCtrl, FilterMixin):

        def valuesRoot(self):
            try:
                ident = cms.RootProperty('a', 'tag_property.root')()
                return settings.workspace()['tags'].idents.get(ident, None)
            except (exceptions.PropertyNotFound, schema.ElementNotFound):
                return None

        valuesRoot = binding.Make(valuesRoot)
        valuesGetKey = lambda s, v: unicode(v.pkey)
        valuesGetLabel = lambda s, v: v.name

        def makeCond(self):
            val = self.newValue
            if val in (NOT_GIVEN, None, ''):
                return
            else:
                val = [v._oid for v in val]
                yield self.parseSubexpr(
                '''rn.inode in [e.inode 
                     for e in cms.RepositoryElement
                      if e.inode == rn.inode
                     and e.changeset_from <= cpv.changeset <= e.changeset_till
                     and e.kind=='prop' and e.name=='tag'
                     and e.value in [t.end for t in 'tags/TagTreeCache'
                                            if t.begin in val]
                     ]''')

    class month(SelectCtrl): 
        ctrlRequired = True
        formAction = None # aby se pouzil defaultInput
        selectOptions = [
                (1, _w('January'), 1),
                (2, _w('February'), 2),
                (3, _w('March'), 3),
                (4, _w('April'), 4),
                (5, _w('May'), 5),
                (6, _w('June'), 6),
                (7, _w('July'), 7),
                (8, _w('August'), 8),
                (9, _w('September'), 9),
                (10, _w('October'), 10),
                (11, _w('November'), 11),
                (12, _w('December'), 12),
                ]
        def defaultInput(self):
            return self['../'].today.month

    class year(SelectCtrl): 
        ctrlRequired = True
        formAction = None # aby se pouzil defaultInput
        today = binding.Make(lambda s: s['../'].today)
        selectOptions = binding.Make(
                lambda s: [(unicode(y), unicode(y), y) 
                           for y in range(s.today.year-5, s.today.year+6)])
        def defaultInput(self):
            return self['../'].today.year

    class calendar(InputCtrl, FilterMixin):

        def getLimits(self):
            def mrange(year, month):
                od = date(year, month, 1)
                do = date(year, month, calendar.monthrange(year, month)[1])
                return od, do

            today = self['../'].today
            month = self['../month'].newValue
            year = self['../year'].newValue

            if month not in (None, NOT_GIVEN) and year not in (None, NOT_GIVEN):
                # zadan mesic a rok
                od, do = mrange(year, month)
            else:
                # default aktualni mesic
                od, do = mrange(today.year, today.month)

            return od, do

        def makeCond(self):
            od, do = self.getLimits()
            od = od.strftime('%Y%m%d')
            do = do.strftime('%Y%m%d')
            yield self.parseSubexpr(
            '''rn.inode in [e.inode 
                    for e in cms.RepositoryElement
                     if e.inode == rn.inode
                    and e.kind == 'prop' and e.name == 'calendar'
                    and od <= e.value <= do
                    and e.changeset_from <= cpv.changeset <= e.changeset_till
                 ]''')





class CalendarHandler(web.Handler):

    def do_list(self):
        ws = settings.workspace()
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing = lst)

