import glob, sys, os
from StringIO import StringIO
from os.path import join, splitext, split, isdir, isfile
from elementtree import ElementTree

from wm4.api import *
from wm4.running import commands, options
from sql4.utils import *
from lib4.content_types import magicService, guessExtension
from lib4 import images

from entities import *
from config import MAX_CHSET

__all__ = ['PublishLast', 'ImageResize', 'FulltextIndexPublished', 'ImportXML']





class PublishLast(commands.ConfiguredCommandLineApp):
    '''
        $ wm4 comp4.cms.commands.PublishLast --ini='pkgfile:myapp/app.ini'
    '''
    
    dry_run = binding.Constant(False,   
        options.Set('--dry-run', '-n', value=True, help='Rollback transaction'))

    #partitions = binding.Make(list,
    #    options.Append('--partition', '-p', type=str, help='Partition'))

    def main(self):
        from datetime import date
        super(PublishLast, self).main()

        q = '''[
            (rn.inode, lm.changeset, cp)
                for rn in RepositoryNode
                for lm in RepositoryNodeLastmod
                for cp in CmsPublicVersion
                 if rn.inode == lm.inode
                and rn.changeset_till == MAX_CHSET
                and query.LeftJoin(
                    rn.inode == cp.inode and
                    cp.pend == PUBLISH_END and
                    lm.changeset != cp.changeset
                )
        ]'''
        #if self.partitions:
        #    query = query % 'and rn.partition in self.partitions'
        ws = settings.workspace()
        storage.begin()
        today = date.today()
        for inode, changeset, cp in list(ws(q)):
            if cp is None:
                print 'new', inode, changeset
                ws.new(CmsPublicVersion,
                    inode     = inode,
                    changeset = changeset,
                    pstart    = today,
                    pend      = PUBLISH_END,
                )
            else:
                print 'update', inode, cp.changeset, '->', changeset
                cp.changeset = changeset
            if not self.dry_run:
                storage.commit()
                storage.begin()
        if self.dry_run:
            storage.abort()
        else:
            storage.commit()




class UnpublishChildrenOfUnpublished(commands.ConfiguredCommandLineApp):
    '''
        $ wm4 comp4.cms.commands.UnpublishChildrenOfUnpublished --ini='pkgfile:myapp/app.ini'
    '''

    dry_run = binding.Constant(False,   
        options.Set('--dry-run', '-n', value=True, help='Rollback transaction'))

    def main(self):
        super(UnpublishChildrenOfUnpublished, self).main()

        import datetime
        ws = settings.workspace()
        td = datetime.date.today()
        fc = settings.workspace()['fulltext']

        def iter_nodes():
            td = datetime.date.today()
            for i in ws('''[cpv.inode for cpv in 'cms/CmsPublicVersion'
                                       if cpv.pend>=td]'''):
                node = ws.get('cms/PublicNode', i, None)
                if node:
                    yield node

        def check_public(node):
            def walk_parents(node):
                try:
                    p = node.parent
                except KeyError:
                    return False
                if p is None:
                    return True
                else:
                    return walk_parents(node.parent)
            return walk_parents(node)

        storage.begin()
        i = 1
        for node in iter_nodes():
            if not check_public(node):
                print '%d.' % i, node.partition, node.inode
                node.partMgr.unpublishNode(node, td)
                i += 1
        if self.dry_run:
            storage.abort()
        else:
            storage.commit()










class ImageResize(commands.ConfiguredCommandLineApp):

    ratio = binding.Constant('keep',
        options.Set('--ratio', '-R', type=str, help='keep/resize/crop [keep]'))

    align = binding.Constant('middle center',
        options.Set('--align', '-A', type=str, help='middle center'))

    maxw = binding.Constant(800,
        options.Set('--maxw', '-W', type=int, help='max width [800]'))

    maxh = binding.Constant(600,
        options.Set('--maxh', '-H', type=int, help='max height [600]'))
    
    def converter(self):
        return images.ImageConverter(
            ratio   = self.ratio,
            align   = self.align,
            maxSize = (self.maxw,self.maxh),
        )
    converter = binding.Make(converter)

    def main(self):
        from lib4.timeutils import Now
        super(ImageResize, self).main()
        conv = self.converter
        ws = settings.workspace()
        bm = settings.blob_manager()

        storage.begin()
        for ri in ws('[ri for ri in RepositoryImage]'):
            ri.blob.open()
            try:
                old = images.loadImage(ri.blob)
                new = conv(old)
                if old is new:
                    continue
                print 'pkey=%s blob=%s %s->%s' % (
                    ri._oid, ri.blob.oid, old.size, new.size
                )
                data = images.getImageData(new)
            finally:
                ri.blob.close()

            bm.delete(ri.blob.oid)
            nb = bm.new()
            nb.open('w')
            nb.write(data)
            nb.close()
            ri.blob = nb
            ri.size = len(data)
            ri.width = new.size[0]
            ri.height = new.size[1]
            ri.content_type = images.getMIME(new)
            ri.modified = Now()
            ws.flush(ri)
        storage.commit()













class FulltextIndexPublished(commands.ConfiguredCommandLineApp):

    def main(self):
        from comp4.cms.fulltext import updateFulltext
        super(FulltextIndexPublished, self).main()
        ws = settings.workspace()
        storage.begin()
        for inode in ws('''[cpv.inode for cpv in CmsPublicVersion]'''):
            print inode, '... ',
            node = ws.get('/cms/PublicNode', inode, None)
            if node is None: continue
            updateFulltext(node)
            print 'OK'
        storage.commit()

    








class ImportXML(commands.CommandLineApp):
    
    usage = 'Imports documents stored in XML files into CMS repository\n'

    dbURL = binding.Require('Database URL',
        options.Set('--db', type=str, help="Database URL"))

    root = binding.Constant(None,
        options.Set('--root', type=int, help="Root node ID"))

    workspace = binding.Make(lambda: settings.workspace())

    def root_node(self):
        if self.root is None:
            return None
        res = ~self.workspace('''[
            obj for obj in RepositoryNode
                 if obj.inode == %d and obj.changeset_till == %d
        ]''' % (self.root, MAX_CHSET))
        return Node.makeInstance(
                    self.root, res, MAX_CHSET
        )
    root_node = binding.Make(root_node)

    def partition(self):
        if not self.root:
            raise commands.InvocationError('missing root or partition')
        return self.root_node.partition
    partition = binding.Make(partition,
        options.Set('--partition', type=str, help="Repository partition"))

    def main(self):
        self.parsed_args
        settings.storage_url(self.dbURL)

        storage.begin()

        parent = self.root_node or None
        for xml_file in self.parsed_args:
            self.traverse(xml_file, parent)

        #storage.abort()
        storage.commit()
    
    def traverse(self, xml_file, parent):
        xml_dir, _ = splitext(xml_file)
        print xml_file
        doc = self.create_document(xml_file, parent)
        if isdir(xml_dir):
            for child in glob.glob(xml_dir + '/*.xml'):
                self.traverse(child, doc)

    def create_document(self, xml_file, parent):
        doc = self.workspace.new(Node, 
                parent_node = parent and parent.inode or None,
                partition = parent and parent.partition or self.partition,
                klass = 'cms',
        )
        et = ElementTree.parse(xml_file)
        for xml_elem in et.findall('./element'):
            self.create_element(doc, xml_elem)
        return doc

    def create_element(self, doc, xml_elem, parent=None):
        value = xml_elem.find('./value')
        if value is None:
            pass
        elif value.attrib.get('type') is None:
            value = value.text
        elif value.attrib.get('type') == 'image':
            data = value.text.decode('base64')
            blob = StringIO(data)
            content_type = magicService().buffer(data)
            _img = images.loadImage(blob)
            blob.seek(0,0)
            filename = value.attrib.get('filename')
            if filename is None:
                filename = 'image' + guessExtension(content_type)

            obj = self.workspace.new(
                RepositoryImage,
                width = _img.size[0],
                height = _img.size[1],
                size = len(data),
                filename = filename,
                content_type = content_type,
                blob = blob,
            )
            value = str(obj.pkey)

        elif value.attrib.get('type') == 'file':
            data = value.text.decode('base64')
            blob = StringIO(data)
            content_type = magicService().buffer(data)
            filename = value.attrib.get('filename')
            if filename is None:
                filename = 'image' + guessExtension(content_type)
            blob.seek(0,0)
            obj = self.workspace.new(
                RepositoryFile,
                size = len(data),
                filename = filename,
                content_type = content_type,
                blob = blob,
            )
            value = str(obj.pkey)

        else:
            raise NotImplementedError(value.attrib.get('type'))

        kind = xml_elem.attrib['kind']
        name = xml_elem.attrib['name']
        
        elem = self.workspace.new(Element, 
                inode = doc.inode,
                kind = kind,
                name = name,
                value = value,
                parent_id = parent and parent.eid or None,
        )

        for sub_elem in xml_elem.findall('./element'):
            self.create_element(doc, sub_elem, elem)



if __name__ == '__main__':
    #cmd = ImportXML()
    cmd = ImageResize()
    cmd.run()
