import datetime
from wm4.api import NOT_GIVEN, settings
from wm4.config.context import AbstractProperty,Property,ExpressionRule,Resource

__all__ = [
    'MAX_CHSET', 'PUBLISH_END', 'DEFAULT_POSITION',
    'changeset_manager', 'manager', 'RootProperty',
    'getRootNode', 'getRootManager',
]


# don't use sys.maxint, amd64 has 9223372036854775807
MAX_CHSET = 2147483647

PUBLISH_END = datetime.date(3000,1,1)

DEFAULT_POSITION = -100000


changeset_manager = Resource(
    'changeset_manager', 'comp4.cms.data_managers.ChangesetManager')


def manager():
    return settings.workspace()['cms']


class RootProperty(AbstractProperty):
    
    def __new__(klass, root, prop, default=NOT_GIVEN):
        self = AbstractProperty.__new__(klass)
        self.root = Property(root)
        self.prop = Property(prop)
        if default is not NOT_GIVEN:
            self.setdefault(default)
        return self

    def regKey(self):
        return (self.__class__, self.root.regKey(), self.prop.regKey())
        
    def lookupKeys(self):
        klass = self.__class__

        rkeys = self.root.lookupKeys()
        pkeys = self.prop.lookupKeys()
        return [
            (klass, rk, pk)
            for rk in rkeys
            for pk in pkeys
        ]

    def makeValue(self, rule, reg, key):
        return rule(reg[1:], key[1:])

    def __repr__(self):
        return 'RootProperty(%r,%r)' % (self.root.value, self.prop.value)

def getRootNode(part, public=True):
    return manager()[part].getRoot(public=public)

def getRootManager(part):
    return manager()[part]

def cms_root(parser, section, name, value, lineInfo):
    for root in section.split()[2:]:
        prop = RootProperty(root, name)
        prop(ExpressionRule(value, parser.globalDict))

