# -*- coding: utf-8 -*-

from datetime import date, timedelta
from ftplib import FTP
import os.path

from wm4.api import *
import wm4.web.forms.controllers
from wm4.web.forms.api import *
from wm4.web.forms.listings import *
from comp4.dbgettext.web import *
from lib4.imports import importObject, importString

from facades import LastNode
from config import *
from entities import *
from pagelets import *

__all__ = [
    'ChildrenCtrl', 'BaseCtrl', 'PageletCtrl',
    'MandatorySingleCtrl', 'OptionalSingleCtrl',
    'ToplevelCtrl', 'HeadingCtrl', 'AbstractCtrl', 'RestCtrl', 
    'HtmlCtrl', 'HtmlSrcCtrl',
    'ContentsCtrl', 
    'SiblingsCtrl',
    'ContentsByTagCtrl', 'ContentsByTagListingCtrl',
    'BaseImageCtrl', 'IllustrationCtrl', 'ImageCtrl', 
    'PhotogalleryImageCtrl', 'PhotogalleryCtrl',
    'BaseFileCtrl', 'FileCtrl', 
    'LinkBoxCtrl', 'LinkBoxLinkCtrl',
    'DiscussionCtrl', 'PollCtrl', 'MailFormCtrl',
    'VideoFtpCtrl',
    'NodeForm', 'makeForm',
]

_ = i18n.MessageIDFactory('comp4.cms')




def pageletCtrl(part, kind, name):
    return RootProperty(part, 'element_ctrl.%s.%s' % (kind, name))




class ChildrenCtrl(SequenceCtrl):

    def ctrlPagelet(self):
        return RepeatWidget

    partMgr = binding.Delegate('ctrlParent')
    partition = binding.Delegate('partMgr')

    def makeItem(self, idx, oldValue=NOT_GIVEN):
        if oldValue is NOT_GIVEN:
            # XXX ???
            cid = '%s$%d' % (self.ctrlId, idx)
            kind = self.ctrlReq.getValue(cid+'$ekind')
            name = self.ctrlReq.getValue(cid+'$ename')
        else:
            kind = oldValue.kind
            name = oldValue.name
        fact = pageletCtrl(self.partition, kind, name)()
        item = fact(
            self.ctrlReq, str(idx), self, oldValue=oldValue,
            ekind=kind, ename=name
        )
        return item
    
    def addNewItem(self):
        raise NotImplementedError

    def makeDefaultItems(self):
        items = []
        for i, d in enumerate(self.ctrlParent.defaultPagelets):
            d = d.copy()
            d['ekind'] = d.pop('kind', 'cms')
            d['ename'] = d.pop('name')
            fact = pageletCtrl(self.partition, d['ekind'], d['ename'])()
            item = fact(self.ctrlReq, str(i), self, **d)
            items.append(item)
        return items




class BaseCtrl(InstanceCtrl):
    
    partMgr = binding.Make(lambda s: s.ctrlParent.partMgr)
    partition = binding.Delegate('partMgr')

    ctrlType = binding.Make(lambda s: RootProperty(
        s.partition,'element_factory.%s.%s' % (s.ekind, s.ename))())

    ctrlWidget = binding.Make(lambda s: RootProperty(
        s.partition,'element_widget.%s.%s' % (s.ekind, s.ename)))

    ctrlLabel = binding.Make(lambda s: s.ctrlType.plName)
    
    def nodeCtrl(self):
        while self:
            if isinstance(self, NodeForm):
                return self
            self = self.ctrlParent
    nodeCtrl = binding.Make(nodeCtrl)

    class eid(InputCtrl):
        ctrlType = int
        ctrlVisibility = binding.Constant('hidden')

    ekind = ClientState(default=lambda s,o,a:'cms')

    class kind(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultInput(self):
            return self.ctrlParent.ekind
        def defaultOutput(self):
            return self.ctrlParent.ekind

    ename = ClientState()

    class name(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultInput(self):
            return self.ctrlParent.ename
        def defaultOutput(self):
            return self.ctrlParent.ename

    css = binding.Make(lambda s:(
        'cms4-element-'+'-'.join(p.lower() for p in s.ename.split('.'))))

    def editEntity(self):
        # old value can be None
        obj = self.oldValue
        if obj is None:
            obj = super(BaseCtrl, self).addEntity()
        else:
            obj._setattrs(**self.newFeatures)
        return obj

    def commitAfter(self):
        res = True
        for a in self._commitAfter:
            elem = self[a]
            if elem.commitChanges():
                if isinstance(elem, (BaseCtrl, ChildrenCtrl)):
                    new = elem.newValue
                    if new is NOT_GIVEN:
                        try:
                            delattr(self.newValue, a)
                        except:
                            pass
                    else:
                        setattr(self.newValue, a, new)
            else:
                res = False
        return res



class PageletCtrl(BaseCtrl, SequenceItemCtrl):

    class removeItem(SequenceItemCtrl.removeItem):
        def jsOnClick(self):
            lbl = web.render(self.ctrlReq, self['..'].ctrlLabel)
            msg = _(u'Really delete $pagelet?') % {'pagelet':lbl}
            txt = web.render(self.ctrlReq, msg)
            return "return confirm(%s)" % web.render_js_value(self.ctrlReq, txt)
        jsOnClick = binding.Make(jsOnClick)




class MandatorySingleCtrl(BaseCtrl):

    class value(InputCtrl):
        pass


class OptionalSingleCtrl(MandatorySingleCtrl):

    def addEntity(self):
        if self['value'].newValue:
            return super(OptionalSingleCtrl,self).addEntity()
        else:
            return NOT_GIVEN

    def editEntity(self):
        if self['value'].newValue:
            return super(OptionalSingleCtrl,self).editEntity()
        else:
            return NOT_GIVEN





class ToplevelCtrl(BaseCtrl):

    ctrlType = Toplevel

    class title(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'head.title')
        ctrlLabel = _(u'Title')
        class value(InputCtrl):
            ctrlRequired = True
        class _value(DBGettextCtrl):
            #XXX set_on_change = '../value'
            pass

    featureCommiting = {
        '*':'before', 'title':'after', 'pagelets':'after', 'add_pagelet':'none'}

    def defaultPagelets(self):
        return RootProperty(
            self.partition,'default_pagelets.%s.%s' % (self.ekind,self.ename))()
    defaultPagelets = binding.Make(defaultPagelets)

    class pagelets(ChildrenCtrl):
        
        ctrlWidget = binding.Make(lambda s: RootProperty(s.ctrlParent.partition,
            'pagelets_widget.%s.%s' % (s.ctrlParent.ekind, s.ctrlParent.ename))
        )
        
        def addNewItem(self):
            kind, name = str(self['../add_pagelet'].inputText).split(':')
            fact = pageletCtrl(self.partition, kind, name)()
            return fact(
                self.ctrlReq, str(self.itemCount), self, ekind=kind, ename=name)

    def offeredPagelets(self):
        return RootProperty(
            self.partition,'offered_pagelets.%s.%s' % (self.ekind,self.ename))()
    offeredPagelets = binding.Make(offeredPagelets)

    class add_pagelet(SelectCtrl):
        ctrlRequired = True
        def selectOptions(self):
            opts = []
            part = self.ctrlParent.partition
            for o in self.ctrlParent.offeredPagelets:
                kind = o.get('kind','cms')
                name = o['name']
                pcls = RootProperty(part,'element_factory.%s.%s'% (kind,name))()
                key = '%s:%s' % (kind,name)
                opts.append((key, pcls.plName, key))
            return opts
        selectOptions = binding.Make(selectOptions)
        









class HeadingCtrl(PageletCtrl):

    ctrlType = Heading

    class value(InputCtrl):
        ctrlRequired = binding.Delegate('ctrlParent')

    class _value(DBGettextCtrl):
        set_on_change = '../value'



class AbstractCtrl(PageletCtrl):

    ctrlType = Abstract

    class value(TextAreaCtrl):
        ctrlRequired = binding.Delegate('ctrlParent')
        rows = 5
        cols = None

    class _value(DBGettextAreaCtrl):
        set_on_change = '../value'



class RestCtrl(PageletCtrl):

    ctrlType = Rest

    class value(RestCtrl):
        ctrlRequired = binding.Delegate('ctrlParent')
        rows = 15
        cols = None

    class _value(DBGettextAreaCtrl):
        set_on_change = '../value'
        rows = 15
        cols = None

class HtmlCtrl(PageletCtrl):

    ctrlType = Html

    class value(HtmlCtrl):
        ctrlRequired = binding.Delegate('ctrlParent')
        toolbarSet = binding.Make(lambda s: RootProperty(
            s.ctrlParent.partition, 'pagelet_html.toolbar_set')())

    class _value(DBGettextHtmlCtrl):
        set_on_change = '../value'
        toolbarSet = binding.Make(lambda s: RootProperty(
            s.ctrlParent.partition, 'pagelet_html.toolbar_set')())

class HtmlSrcCtrl(PageletCtrl):

    ctrlType = HtmlSrc

    class value(TextAreaCtrl):
        ctrlRequired = binding.Delegate('ctrlParent')
        rows = 15
        cols = None

    class _value(DBGettextAreaCtrl):
        set_on_change = '../value'
        rows = 15
        cols = None



class ContentsCtrl(PageletCtrl):

    ctrlType = Contents

    class value(SelectCtrl):
        ctrlRequired = True
        
        def selectOptions(self):
            return RootProperty(
                self.ctrlParent.partition, 'pagelet_contents.options')()
        selectOptions = binding.Make(selectOptions)


class SiblingsCtrl(ContentsCtrl):
    ctrlType = Siblings
    featureCommiting = {'*': 'before', 'title': 'after'}

    class value(SelectCtrl):
        ctrlRequired = True
        
        def selectOptions(self):
            return RootProperty(
                self.ctrlParent.partition, 'pagelet_siblings.options')()
        selectOptions = binding.Make(selectOptions)

    class title(OptionalSingleCtrl):
        ename = ClientState(
                default=lambda s,o,a:'pagelet.siblings.title')
        ctrlLabel = _(u'Title')
        class _value(DBGettextCtrl):
            set_on_change = '../value'


class ContentsByTagCtrl(PageletCtrl):
    ctrlType = ContentsByTag

    featureCommiting = {'*':'before', 'date_range':'after'}

    class value(SelectCtrl):
        ctrlRequired = True

        def selectOptions(self):
            obj = self.ctrlParent
            ws = settings.workspace()
            try:
                ident = RootProperty(obj.partition, 'tag_property.root')()
                root = ws['tags'].idents[ident]
                tags = root.iterDepthFirst(include_self=False)
            except exceptions.PropertyNotFound:
                tags = []
            return [(str(t.pkey), t.name, str(t.pkey)) for t in tags]
        selectOptions = binding.Make(selectOptions)

    class date_range(MandatorySingleCtrl):
        ename = ClientState(
                    default=lambda s,o,a:'pagelet.contents_by_tag.date_range')
        ctrlLabel = _(u'Date range')

        class value(SelectCtrl):
            ctrlRequired = True
            selectOptions = [
                    ('all', _(u'all documents'), 'all'),
                    ('future', _(u'documents in future'), 'future'),
                    ('future30', _(u'documents in future 30 days'), 'future30'),
                    ('past', _(u'documents in past'), 'past'),
                    ('past30', _(u'documents in past 30 days'), 'past30'),
                    ]



class ContentsByTagListingCtrl(ListingCtrl):
    felem = binding.Require()
    today = binding.Make(lambda: date.today())
    itemLimit = None

    baseQuery = query.query('''[
        (rn, cpv.changeset)
            for rn in RepositoryNode
            for cpv in CmsPublicVersion
             if cpv.inode == rn.inode
            and rn.changeset_from <= cpv.changeset <= rn.changeset_till
            and cpv.pstart <= date.today() <= cpv.pend
            ]''')

    items = binding.Make(lambda s: [
                s.workspace.get('cms/PublicNode', n.inode, default=None)
                for n, chset in s.workspace(s.itemQuery)])


    class tag(InputCtrl, FilterMixin):
        def makeCond(self):
            val = self['../'].felem['value'].newValue
            if val in (NOT_GIVEN, None, ''):
                return
            else:
                yield self.parseSubexpr(
                    '''rn.inode in [e.inode
                 for e in RepositoryElement
                  if e.inode == rn.inode
                 and e.changeset_from <= cpv.changeset <= e.changeset_till
                 and e.kind=='prop' and e.name=='tag'
                 and e.value in [str(t.end) for t in 'tags/TagTreeCache'
                                             if t.begin==val]
                 ]''')

    class date_range(InputCtrl, FilterMixin):
        def makeCond(self):
            val = self['../'].felem['date_range/value'].newValue
            if val in (NOT_GIVEN, None, '', 'all'):
                return
            else:
                today = date.today()
                monthago = (today - timedelta(days=30)).strftime('%Y%m%d')
                monthfut = (today + timedelta(days=30)).strftime('%Y%m%d')
                today = today.strftime('%Y%m%d')
                q = '''e.inode
                 for e in RepositoryElement
                  if e.inode == rn.inode
                 and e.changeset_from <= cpv.changeset <= e.changeset_till
                 and e.kind == 'prop' and e.name == 'calendar' '''
                if val == 'future':
                    q += 'and e.value >= today'
                elif val == 'past':
                    q += 'and e.value < today'
                elif val == 'past30':
                    q += 'and e.value < today and e.value>=monthago'
                elif val == 'future30':
                    q += 'and e.value >= today and e.value<monthfut'
                yield self.parseSubexpr('''rn.inode in [%s]''' % q)



class BaseImageCtrl(PageletCtrl):

    converter = binding.Make(lambda s: RootProperty(
                        s.partition, 'base_image.converter')())
    
    class value(InputCtrl):
        ctrlVisibility = binding.Constant('hidden')

    class img(ImageUploadCtrl):
        converter = binding.Obtain('../converter')

        ctrlType = RepositoryImage
        ctrlRequired = True
        ctrlLabel = _(u'Image')
        def makeFile(self):
            return settings.workspace().new(self.ctrlType,
                blob=open(self.path), filename=self.filename,
                size=self.size, content_type=self.content_type,
                width=self.width, height=self.height
            )

    @event_handler(OnChangeEvent, 'img')
    def change_value(self, evt):
        val = self['value']
        pkey = str(evt.new.pkey)
        val.inputText = val.newValue = pkey
        OnChangeEvent(val, old=val.oldValue, new=val.newValue)

class ImageCtrl(BaseImageCtrl):

    ctrlType = Image

    featureCommiting = {
            '*':'before', 
            'align':'after', 
            'description':'after',
            'url':'after',
            'alt':'after',
            }

    class align(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.image.align')
        ctrlLabel = _(u'Image align')
        class value(SelectCtrl):
            selectOptions = binding.Constant([
            ('cms4-align-box-left',_(u'Box left'),'cms4-align-box-left'),
            ('cms4-align-box-center',_(u'Box center'),'cms4-align-box-center'),
            ('cms4-align-box-right',_(u'Box right'),'cms4-align-box-right'),
            ('cms4-align-flow-left',_(u'Flow left'),'cms4-align-flow-left'),
            ('cms4-align-flow-right',_(u'Flow right'),'cms4-align-flow-right'),
            ])

    class url(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,y:'pagelet.image.url')
        ctrlLabel = _(u'URL')

    class description(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.image.description')
        ctrlLabel = _(u'Description')
        class _value(DBGettextCtrl):
            set_on_change = '../value'

    class alt(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.image.alt')
        ctrlLabel = _(u'Alt')
        class _value(DBGettextCtrl):
            set_on_change = '../value'

class IllustrationCtrl(BaseImageCtrl):
    ctrlType = Illustration

    def converter(self):
        from lib4.images import ImageConverter
        maxSize = RootProperty(self.partition, 'illustration.size')()
        ratio = RootProperty(self.partition, 'illustration.ratio')()
        return ImageConverter(maxSize=maxSize, ratio=ratio)
    converter = binding.Make(converter)






class PhotogalleryImageCtrl(BaseImageCtrl):

    ctrlType = PhotogalleryImage

    featureCommiting = {'*':'before', 'description':'after','thumbnail':'after'}

    class description(OptionalSingleCtrl):
        ename = ClientState(
                default=lambda s,o,a:'pagelet.photogallery.image.description')
        ctrlLabel = _(u'Description')
        class _value(DBGettextCtrl):
            set_on_change = '../value'

    class thumbnail(OptionalSingleCtrl):
        ename = ClientState(
                default=lambda s,o,a:'pagelet.photogallery.image.thumbnail')


    thumbnail_converter = binding.Make(lambda s: RootProperty(
                s.partition, 'pagelet_photogallery.thumbnail_converter')())
            
    img_api = binding.Make(lambda: importString('lib4.images'))

    @event_handler(OnChangeEvent, 'img')
    def make_thumbnail(self, evt):
        api = self.img_api
        thumb = self.thumbnail_converter(evt.sender.path)
        size = api.getStorageSize(thumb)
        fd = api.getFileDesc(thumb)
        fd.seek(0,0)
        repo_file = settings.workspace().new(RepositoryImage,
            blob         = fd,
            size         = size,
            content_type = api.getMIME(thumb),
            filename     = 'thumb-' + evt.sender.filename,
            width        = thumb.size[0],
            height       = thumb.size[1],
        )
        val = self['thumbnail/value']
        pkey = str(repo_file.pkey)
        val.inputText = val.newValue = pkey
        OnChangeEvent(val, old=val.oldValue, new=val.newValue)

class PhotogalleryCtrl(PageletCtrl):

    ctrlType = Photogallery

    class value(InputCtrl):
        ctrlLabel = _(u'Photogallery title')
    class _value(DBGettextCtrl):
        set_on_change = '../value'

    class description(OptionalSingleCtrl):
        ename = ClientState(
                default=lambda s,o,a:'pagelet.photogallery.description')
        ctrlLabel = _(u'Description')
        class value(TextAreaCtrl):
            pass
        class _value(DBGettextAreaCtrl):
            set_on_change = '../value'

    defaultPagelets = ()

    featureCommiting = {
        '*':'before', 
        'title':'after', 'description':'after', 'images':'after',
        'archiv':'after'
    }

    class images(ChildrenCtrl):

        class addItemCount(SelectCtrl):
            ctrlType = int
            ctrlRequired = True
            selectOptions = [(str(i),str(i),i) for i in [1,2,3,4,5,7,10,15,20]]
            def defaultInput(self):
                return '1'
            def defaultOutput(self):
                return 1

        def addNewItem(self):
            kind, name = 'cms', 'pagelet.photogallery.image'
            fact = pageletCtrl(self.partition, kind, name)()
            return fact(
                self.ctrlReq, str(self.itemCount), self, ekind=kind, ename=name
            )


    class archiv(FileUploadCtrl):
        ctrlLabel = _(u'ZIP archive')

        def makeFile(self):
            import zipfile
            from lib4.images import loadImage
            from lib4.content_types import guessContentType

            upld = importObject(web.upload_manager())()
            zip = zipfile.ZipFile(self.path, 'r')
            images = self['../images']

            for item in zip.infolist():
                temp = upld.mktemp()
                fd = open(temp, 'w+b')
                data = zip.read(item.filename)
                content_type = guessContentType(item.filename, temp)
                if not content_type or not content_type.startswith('image/'):
                    continue
                fd.write(data)
                fd.close()
                imgObj = loadImage(temp)
                images.onAddItem(None)
                pagelet = images.ctrlItems[-1]
                for child in pagelet.ctrlIter():
                    child.ctrlOnInit()
                    child.ctrlSetupHandlers()
                img = pagelet['img']
                img.filename = item.filename
                img.path = temp
                img.size = len(data)
                img.width = imgObj.size[0]
                img.height = imgObj.size[1]
                img.newValue = img.makeFile()
                OnChangeEvent(img, old=img.oldValue, new=img.newValue)
                pagelet.commitChanges()
            return None







class BaseFileCtrl(PageletCtrl):

    class value(InputCtrl):
        ctrlVisibility = binding.Constant('hidden')

    class file(FileUploadCtrl):
        ctrlType = RepositoryFile
        ctrlRequired = True
        ctrlLabel = _(u'File')
        def makeFile(self):
            return settings.workspace().new(self.ctrlType,
                blob=open(self.path), filename=self.filename,
                size=self.size, content_type=self.content_type,
            )

    @event_handler(OnChangeEvent, 'file')
    def change_value(self, evt):
        val = self['value']
        pkey = str(evt.new.pkey)
        val.inputText = val.newValue = pkey
        OnChangeEvent(val, old=val.oldValue, new=val.newValue)


class FileCtrl(BaseFileCtrl):

    ctrlType = File

    featureCommiting = {'*':'before', 'description':'after'}

    class description(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.file.description')
        ctrlLabel = _(u'Description')
        class _value(DBGettextCtrl):
            set_on_change = '../value'


class DiscussionCtrl(PageletCtrl):

    ctrlType = Discussion

    featureCommiting = {
        '*':'before', '_updateforum':'after',
    }

    class value(InputCtrl):
        ctrlVisibility = binding.Constant('none')
        ctrlRequired = True

        class administrator(SelectCtrl):
            ctrlType = int
            ctrlVisibility = binding.Constant('visible')
            def selectOptions(self):
                from comp4.users.api import User
                ws = settings.workspace()
                return [
                    (u.uid, '%s (%s)' % (u.name, u.login), u)
                    for u in ws('[u for u in User if query.OrderBy(u.name)]')
                ]
            selectOptions = binding.Make(selectOptions)
            def defaultInput(self):
                return self.ctrlReq.user

        def commitChanges(self):
            if self.oldValue is NOT_GIVEN:
                import comp4.forum.api as forum
                import comp4.users.api as users

                if not self.commitBefore():
                    return False

                ws = settings.workspace()

                admin = self['administrator'].newValue
                if admin in (NOT_GIVEN, None):
                    admin = None
                obj = ws.new(
                    forum.Forum, 
                    name = self['../../../title/_value'].inputText,
                    klass = 'cms/PublicElement',
                    administrator = admin,
                )
                self.newValue = str(obj.pkey)
                OnChangeEvent(self, old=self.oldValue, new=self.newValue)
            return True

    class _updateforum(InputCtrl):
        ctrlVisibility = binding.Constant('none')
        ctrlRequired = True
        def commitChanges(self):
            value = self['../value']
            if value.oldValue is NOT_GIVEN:
                import comp4.forum.api as forum
                ws = settings.workspace()
                obj = ws.get(forum.Forum, value.newValue)
                obj.key = self['../'].newValue.eid
            return True




_poll = i18n.MessageIDFactory('comp4.poll')

class PollOptionCtrl(SequenceItemCtrl):
    class option(InputCtrl):
        ctrlLabel = _poll(u'option')
        ctrlRequired = True

class PollCtrl(PageletCtrl):

    ctrlType = Poll
    defaultPagelets = ()

    featureCommiting = {'*':'before', '_updatepoll':'after'}

    class question(InputCtrl):
        ctrlLabel = _poll(u'Question')
        ctrlRequired = True

    class options(SequenceCtrl):
        ctrlLabel = _poll(u'Options')
        itemClass = binding.Constant(PollOptionCtrl)
        class addItem(ButtonCtrl):
            ctrlLabel = binding.Constant(_poll(u'Add option'))
        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])


    class value(InputCtrl):
        ctrlVisibility = binding.Constant('none')
        ctrlRequired = True
        def commitChanges(self):
            if self.oldValue is NOT_GIVEN:
                import comp4.poll.api as poll
                ws = settings.workspace()
                obj = ws.new(poll.Poll, 
                             question=self['../question'].inputText,
                             klass='cms/PublicElement',
                             )
                for (ordnum, option) in enumerate(self['../options'].ctrlItems):
                    ws.new(poll.PollOption,
                           poll=obj,
                           ordernum=ordnum,
                           option=option['option'].inputText)
                self.newValue = str(obj.pkey)
                OnChangeEvent(self, old=self.oldValue, new=self.newValue)
            return True

    class _updatepoll(InputCtrl):
        ctrlVisibility = binding.Constant('none')
        ctrlRequired = True
        def commitChanges(self):
            value = self['../value']
            if value.oldValue is NOT_GIVEN:
                import comp4.poll.api as poll
                ws = settings.workspace()
                obj = ws.get(poll.Poll, value.newValue)
                obj.key = self['../'].newValue.eid
            return True









class LinkBoxLinkCtrl(PageletCtrl):
    ctrlType = LinkBoxLink
    featureCommiting = {'*': 'before', 'description': 'after'}

    class value(InputCtrl):
        ctrlLabel = _(u'URL')
        ctrlRequired = True

        def parseValue(self, txt):
            from lib4.validators import normalizeURL
            txt = super(LinkBoxLinkCtrl.value, self).parseValue(txt)
            return normalizeURL(txt)
    
    class description(OptionalSingleCtrl):
        ename = ClientState(
                default=lambda s,o,a:'pagelet.linkbox.link.description')
        ctrlLabel = _(u'Link text')
        class _value(DBGettextCtrl):
            set_on_change = '../value'




class LinkBoxCtrl(PageletCtrl):
    ctrlType = LinkBox
    defaultPagelets = ()

    class value(InputCtrl):
        ctrlLabel = _(u'Linkbox title')
    class _value(DBGettextCtrl):
        set_on_change = '../value'

    featureCommiting = {'*': 'before', 'links': 'after'}

    class links(ChildrenCtrl):
        class addItem(ButtonCtrl):
            ctrlLabel = binding.Constant(_(u'Add link'))
        def addNewItem(self):
            kind, name = 'cms', 'pagelet.linkbox.link'
            fact = pageletCtrl(self.partition, kind, name)()
            return fact(
                self.ctrlReq, str(self.itemCount), self, ekind=kind, ename=name
            )





class MailFormCtrl(PageletCtrl):

    ctrlType = MailForm

    class value(SelectCtrl):
        ctrlRequired = True
        
        def selectOptions(self):
            return [(n,n,n) for n in self.ctrlParent.partMgr.mailforms.keys()]
        selectOptions = binding.Make(selectOptions)

    class email(MandatorySingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.mailform.email')
        ctrlLabel = _(u'Email')
        class value(MandatorySingleCtrl.value):
            ctrlRequired = True









class InputCtrl_Default(InputCtrl):
    def defaultInput(self):
        return RootProperty(
                self['../../'].partition, '%s.default' % self['../'].ename)()

    def getInputText(self):
        fid = self.ctrlId
        req = self.ctrlReq
        if fid in req:
            txt = req.getValue(fid, u'')
            txt = self.stripInput(txt)
            return txt
        else:
            if self.oldValue is NOT_GIVEN:
                return self.renderValue(self.defaultInput())
            else:
                return self.renderValue(self.oldValue)
    inputText = binding.Make(getInputText)

class VideoFtpCtrl(PageletCtrl):
    ctrlType = VideoFtp
    featureCommiting = {
            '*': 'before',
            'width': 'after',
            'height': 'after',
            'description': 'after',
    }

    class value(SelectCtrl):
        ctrlRequired = True
        def selectOptions(self):
            partition = self['../../'].partition
            def rp(n): 
                return RootProperty(partition, 'pagelet.videoftp.%s' % n)()
            ftp = FTP(rp('ftp_server'), rp('ftp_login'), rp('ftp_password'))
            files = [#unicode(os.path.splitext(f)[0], 'utf-8')
                     unicode(f, 'utf-8')
                     for f in ftp.nlst('*')
                     if unicode(f, 'utf-8')[-4:].lower() in ('.flv', '.m4v', '.mp4', '.m4a', '.mov', 'mp4v', '.3gp', '.3g2')
                    ]
            return [(f, f, f) for f in files]
        selectOptions = binding.Make(selectOptions)

    class width(MandatorySingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.videoftp.width')
        ctrlLabel = _(u'width')
        class value(InputCtrl_Default):
            ctrlType = int

    class height(MandatorySingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.videoftp.height')
        ctrlLabel = _(u'height')
        class value(InputCtrl_Default):
            ctrlType = int

    class description(MandatorySingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.videoftp.description')
        ctrlLabel = _(u'description')
        class value(wm4.web.forms.controllers.HtmlCtrl): 
            pass











class NodeForm(EntityCtrl, FormCtrl):

    partMgr = binding.Require()
    parentNode = None
    ctrlType = LastNode

    class partition(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            return self.ctrlParent.partMgr.partition

    class parent_node(InputCtrl):
        ctrlType = int
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            parentNode = self.ctrlParent.parentNode
            if self.oldValue is NOT_GIVEN and parentNode is not None:
                return parentNode.inode
            else:
                return self.oldValue

    def editEntity(self):
        obj = self.oldValue
        obj._setattrs(**self.newFeatures)
        return obj

    def commitAfter(self):
        res = True
        for a in self._commitAfter:
            if not self[a].commitChanges():
                res = False
        if res:
            self.newValue.document = self['document'].newValue
        return res

    class inode(InputCtrl):
        ctrlVisibility = binding.Constant('hidden')
    
    featureCommiting = {'*':'before', 'document':'after'}

    class document(AbstractCtrl):
        ekind = 'cms'
        def __new__(
            klass,ctrlReq,ctrlName=None,ctrlParent=None,oldValue=NOT_GIVEN,**kw
        ):
            ekind = klass.ekind
            if oldValue is NOT_GIVEN:
                ename = ctrlReq.params.get('toplevel', 'toplevel.document')
            else:
                ename = oldValue.name
            fact = pageletCtrl(ctrlParent.partMgr.partition, ekind, ename)()
            kw = kw.copy()
            kw['ekind'] = ekind
            kw['ename'] = ename
            return fact(ctrlReq, ctrlName, ctrlParent, oldValue, **kw)

def makeForm(req, doc, **kw):
    kw['ctrlReq'] = req
    kw.setdefault('oldValue', doc)
    kw.setdefault('partMgr', doc.partMgr)
    kw.setdefault('parentNode', doc.parent)
    kw.setdefault('ctrlReadOnly', True)
    form = NodeForm(**kw)
    return form
