# -*- coding: utf-8 -*-

from datetime import date, timedelta
from wm4.api import *

from comp4.dbgettext.api import Message

from config import PUBLISH_END, MAX_CHSET, RootProperty
from config import manager, changeset_manager, getRootNode
from entities import RepositoryChangeset, RepositoryNode, RepositoryElement
from entities import OnPublish, OnUnpublish, CmsPublicVersion
from facades import NodeVersion, PublicNode, LastNode

__all__ = ['ChangesetManager', 'PartitionManager', 'CmsComponent']



class ChangesetManager(storage.TransactionComponent):
    '''Manages creation of new changeset during transaction on request
       and hold cache for ChangesetedAttr
    '''

    # XXX tenhle objekt by mohl poskytovat nejakou udalost NewChangeset
    #     ktera by byla vyvolana na zacatku transakce, pokud by doslo
    #     ke zmene changesetu.

    workspace = binding.Make(lambda: settings.workspace())

    def newChangeset(self):
        '''Changeset for current transation'''
        # we must register last changeset before creating new one
        # because of data dictionary that is reseted on changeset
        # change
        self.lastChangeset
        # create new changeset
        return self.workspace.add(RepositoryChangeset())
    newChangeset = storage.TxnAttr(newChangeset)

    def lastChangeset(self):
        q = '''[c for c in RepositoryChangeset
                   if query.OrderBy(query.Desc(c.pkey))]'''
        q = self.workspace.parse(q)
        q.limit = query.Const(1)
        return -self.workspace(q)
    lastChangeset = storage.TxnAttr(lastChangeset)

    _data = binding.Make(dict)

    def data(self):
        # XXX tady by se spis melo kontrolovat, jestli existuji
        #     changesety s vetsim pkey, nez pkey, ktery je ulozen
        #     v _data.
        self.joinedTxn
        data = self._data
        ch = data.get('__lastChangeset__')
        if ch != self.lastChangeset.pkey:
            data.clear()
            data['__lastChangeset__'] = self.lastChangeset.pkey
        return data
    data = property(data)




class PartitionManager(storage.TransactionComponent):

    #txnAttrs = 'public_versions',

    partition = binding.Require()

    workspace = binding.Make(lambda: settings.workspace())

    publishing = binding.Make(
        lambda s: RootProperty(s.partition, 'publishing')(default='last'))

    permissions = binding.Make(
        lambda s: RootProperty(s.partition, 'permissions')(default='static'))

    def mailforms(self):
        from lxml import etree
        from copy import deepcopy

        antispam = etree.fromstring('''
            <div id="" class="js_antispam" xmlns="http://www.w3.org/1999/xhtml">
                <span class="as_expr">EXPR</span> =
                <input name="as_result" size="4" />?
                <input type="hidden" name="as_token" />
                <script>
                </script>
            </div>''')

        spec = RootProperty(self.partition, 'mailforms')(default={})
        parsed = {}
        forms = {}
        for name, url in spec.items():
            url, id = url.split("#")
            html = parsed.get(url)
            if html is None:
                src = naming.lookup('pkgfile:%s' % url).open('r').read()
                html = parsed[url] = etree.fromstring(src)
            for frm in html.xpath(
                "//h:form[@id='%s']" % id,
                namespaces={'h':'http://www.w3.org/1999/xhtml'}
            ):
                #frm.insert(0, deepcopy(antispam))
                forms[name] = frm
                break
        return forms
    mailforms = binding.Make(mailforms)

    def offeredToplevels(self):
        offer = []
        for name in RootProperty(self.partition,'offered_toplevels')():
            klass = RootProperty(
                self.partition, 'element_factory.cms.%s' % name
            )()
            offer.append((name, klass))
        return offer
    offeredToplevels = binding.Make(offeredToplevels)

    def offeredPagelets(self):
        offer = []
        for name in RootProperty(self.partition,'offered_pagelets')():
            klass = RootProperty(
                self.partition, 'element_factory.cms.%s' % name
            )()
            offer.append((name, klass))
        return offer
    offeredPagelets = binding.Make(offeredPagelets)

    translations = binding.Make(
        lambda s: RootProperty(s.partition,'translations')()
    )

    def getRoot(self, default=None, public=True):
        mgr = manager()
        inode = mgr.roots.get(self.partition)
        if inode is None:
            return None
        else:
            if public:
                return  mgr.getNode(PublicNode, inode)
            else:
                return  mgr.getNode(LastNode, inode)

    def recentChanges(self, limit=10):
        q = '''[
            (nd.inode, et.value, ab.value, ch.pkey, ch.creation_time)
                for pv in CmsPublicVersion
                for ch in RepositoryChangeset
                for nd in RepositoryNode
                for et in RepositoryElement
                for ab in RepositoryElement
                 if pv.changeset == ch.pkey
                and pv.inode == nd.inode == et.inode == ab.inode
                and nd.changeset_from <= ch.pkey <= nd.changeset_till
                and nd.partition == self.partition
                and et.changeset_from <= ch.pkey <= et.changeset_till
                and et.kind == 'cms' and et.name == 'head.title'
                and query.LeftJoin(
                        ab.changeset_from <= ch.pkey <= ab.changeset_till
                        and ab.kind == 'cms' and ab.name == 'head.title'
                    )
                and query.OrderBy(query.Desc(ch.creation_time))
        ]'''
        mgr = manager()
        q = self.workspace.parse(q)
        q.limit = query.Const(limit)
        return [
            (mgr.getNode(LastNode,n), t, a, c)
            for n,t,a,k,c in self.workspace(q)
        ]

    def updatePublicVersions(self, inode, pstart, pend, changeset):
        from lib4.containers import RangeValueMap
        old = {}
        map = RangeValueMap(resolution=timedelta(1))
        ws = self.workspace
        for pv in ws('''[
            pv for pv in CmsPublicVersion
                if pv.inode == inode and query.OrderBy(pv.pstart)
        ]'''):
            old[(pv.pstart, pv.pend, pv.changeset)] = pv
            map.setrange(pv.pstart, pv.pend, pv.changeset)
        map.setrange(pstart, pend, changeset)
        for (s,e), c in map.items():
            k = (s,e,c)
            if k in old:
                del old[k]
            elif c is not None:
                ws.new(CmsPublicVersion,
                    inode=inode, changeset=c, pstart=s, pend=e)
        for p in old.values():
            ws.remove(p)

    def publishNode(self, node, changeset, pstart, pend):
        self.updatePublicVersions(node.inode, pstart, pend, changeset)
        self.workspace['cms'].flush()
        if pstart <= date.today() <= pend:
            pub = self.workspace.get('cms/PublicNode', node.inode)
            OnPublish(pub.__class__, node=pub, publishing=self.publishing)

    def unpublishNode(self, node, pstart, pend=PUBLISH_END):
        self.updatePublicVersions(node.inode, pstart, pend, None)
        if pstart <= date.today() <= pend:
            OnUnpublish(node.__class__, node=node)
        ectx = self.workspace.contextFor(PublicNode)
        ectx.cache.pop(node.inode, None)

    def onNewNode(self, node):
        if self.publishing == 'last':
            chset = changeset_manager().newChangeset.pkey
            self.publishNode(node, chset, date.today(), PUBLISH_END)

    def onUpdateNode(self, node):
        if self.publishing == 'last':
            chset = changeset_manager().newChangeset.pkey
            self.publishNode(node, chset, date.today(), PUBLISH_END)

    def onRemoveNode(self, node):
        if self.publishing == 'last':
            self.unpublishNode(node, date.today(), PUBLISH_END)
        if 0:
            self.workspace('''[
                query.Delete(cpw) for cpw in CmsPublishingWorkflow
                                   if cpw.inode == inode
            ]''')
            self.workspace.new(CmsPublishingWorkflow,
                inode     = obj.inode,
                changeset = -1,
                req       = 'delete',
                req_chset = manager().changeset.pkey,
                pstart    = date.today(),
                pend      = PUBLISH_END,
            )

    def __repr__(self):
        return 'PartitionManager(%r)' % self.partition













class CmsComponent(schema.SchemaComponent):

    # XXX nasledujici atributy by se mohly klidne kesovat, pokud
    #     se nezmeni changeset, to same asi plati pro nakesovane nody
    #     a elementy. Takze by asi bylo dobre zjistit na zacatku
    #     transakce posledni changeset a pokud bude vyssi nez
    #     poznaceny, vsechno smazat.
    txnAttrs = 'inodes', 'branches', 'roots', 'documents'

    # ------- cms API ---------------------

    # cache for PatitionManagers
    partitions = binding.Make(dict)

    def getPartition(self, partId):
        partition = self.partitions.get(partId)
        if partition is None:
            partition = self.partitions[partId] = PartitionManager(
                self, partId, partition=partId
            )
        return partition
    __getitem__ = getPartition

    def partitionForInode(self, inode):
        inode = int(inode)
        part = self.inodes[inode][2]
        return self[part]

    # ------- node SPI ---------------------

    def branches_inodes_roots(self):
        q = '''   SELECT DISTINCT ON (rn.inode)
                         rn.inode, rn.parent_node, rn.partition,
                         rn.changeset_from, rn.changeset_till,
                         cpv.changeset,
                         dp.value,
                         ti.value, ti.eid
                    FROM repository_node rn
               LEFT JOIN cms_public_version cpv
                      ON cpv.inode = rn.inode
                     AND CURRENT_DATE BETWEEN cpv.pstart AND cpv.pend
               LEFT JOIN repository_element dp
                      ON dp.inode = rn.inode
                     AND cpv.changeset BETWEEN dp.changeset_from
                                           AND dp.changeset_till
                     AND dp.name = 'dir.positions'
               LEFT JOIN repository_element ti
                      ON ti.inode = rn.inode
                     AND cpv.changeset BETWEEN ti.changeset_from
                                           AND ti.changeset_till
                     AND ti.name = 'head.title'
                ORDER BY rn.inode, rn.changeset_from DESC
        '''
        self.joinedTxn
        branches = {}
        inodes = {}
        for row in self.workspace.connection(q):
            inode = row[0]
            parent = row[1]
            if inode is not None:
                inode = int(inode)
            if parent is not None:
                parent = int(parent)
            inodes[inode] = row
            branches.setdefault(parent, []).append(inode)
        roots = dict((inodes[i][2], i) for i in branches.get(None,()))
        return locals()
    branches = inodes = roots = binding.MultiMake(branches_inodes_roots)

    def flush(self):
        for a in 'branches', 'inodes', 'roots':
            self.__dict__.pop(a, None)

    def getNodeData(self, inode):
        row = self.inodes.get(inode)
        if row is None:
            return None
        if row[1] is None:
            parent = None
        else:
            parent = int(row[1])
        public_version = row[5]
        if public_version is not None:
            public_version = int(public_version)
        return dict(
            inode = inode,
            parent_node = parent,
            partition = row[2],
            changeset_from = int(row[3]),
            changeset_till = int(row[4]),
            public_version = public_version,
            _preloaded_dir_positions = row[6],
            _preloaded_title = (row[7], row[8])
        )

    def getNode(self, klass, inode, changeset=None, default=NOT_GIVEN):
        if klass == NodeVersion:
            try:
                inode = int(inode)
                changeset = int(changeset)
            except:
                raise KeyError('Invalid key', inode, changeset)
            oid = (inode, changeset)
        else:
            try:
                inode = int(inode)
            except:
                raise KeyError('Invalid key', inode)
            oid = inode
        self.joinedTxn
        ws = self.workspace
        ectx = ws.contextFor(klass)
        obj = ectx.cache.get(oid)
        if obj is not None:
            return obj
        node = object.__new__(klass)
        node.__dict__['_oid'] = oid
        node.__dict__['_workspace'] = ws
        try:
            self.preloadNode(node)
        except KeyError:
            if default is NOT_GIVEN:
                raise
            return default
        ectx.cache[oid] = node
        return node

    def preloadNode(self, node, data=None):
        klass = node.__class__
        if klass == NodeVersion:
            inode, changeset = node._oid
        else:
            inode = node._oid
        if data is None:
            data = self.getNodeData(inode)
            if data is None:
                raise KeyError('Document not found', inode)
        if klass == NodeVersion:
            pass
        elif klass == LastNode:
            changeset = data['changeset_till']
        elif klass == PublicNode:
            changeset = data['public_version']
            if changeset is None:
                raise KeyError('Document has no public version', inode)
            if data['changeset_till'] != MAX_CHSET:
                raise KeyError('Document was deleted', inode)
        else:
            raise AssertionError('unsupported node class', klass)
        for k,v in data.iteritems():
            node.__dict__.setdefault(k, v)
        node.__dict__.setdefault('changeset', changeset)
        node.__dict__.setdefault('klass', 'cms')
        return node

    def getRootNode(self, part, public=True):
        return self[part].getRoot(public=public)

    def lastParentFor(self, node):
        if node.parent_node is None:
            return None
        return self.getNode(LastNode, node.parent_node)

    def lastChildrenFor(self, node):
        return tuple(
            self.getNode(LastNode, inode)
                for inode in self.branches.get(node.inode, ())
        )

    def publicParentFor(self, node):
        if node.parent_node is None:
            return None
        return self.getNode(PublicNode, node.parent_node)

    def publicChildrenFor(self, node):
        partMgr = self[node.partition]
        children = []
        for inode in self.branches.get(node.inode, ()):
            node = self.getNode(PublicNode, inode, default=None)
            if node is not None:
                children.append(node)
        return children

    def updatePositions(self, node, children):
        # Delete old positions and create new which is valid forever
        ws = self.workspace
        ws('''[
            query.Delete(e)
                for e in RepositoryElement
                 if e.inode == node.inode
                and e.kind == 'navig'
                and e.name == 'dir.positions'
        ]''')

        changeset_from = -ws('''[
            query.Min(n.changeset_from)
                for n in RepositoryNode
                 if n.inode == node.inode
        ]''')
        positions = ','.join(
            '%d=%d' % (node.inode, idx) for idx,node in enumerate(children))
        ws.new(
            RepositoryElement,
            inode = node.inode,
            kind = 'navig',
            name = 'dir.positions',
            value = positions,
            changeset_from = changeset_from,
        )

    def lockNode(self, inode):
        for rn in self.workspace('''[
            query.ExclusiveLock(obj)
                for obj in RepositoryNode
                 if obj.inode == inode
                and query.OrderBy(obj.changeset_till)
        ]'''):
            if rn.changeset_till == MAX_CHSET:
                return rn
        raise ValueError('Cannot lock deleted node', inode)

    def addLastNode(self, obj):
        rn = RepositoryNode(
                inode = obj.inode,
                partition = obj.partition,
                changeset_from = changeset_manager().newChangeset.pkey,
                changeset_till = MAX_CHSET,
                klass = obj.klass,
                parent_node = obj.parent_node,
        )
        self.workspace.add(rn)
        self.saveDocument(obj.elements)
        if obj.parent is not None:
            self.updatePositions(obj.parent, [obj] + obj.parent.validChildren)
        # XXX
        for a in 'title', '_preloaded_title', 'title__e':
            obj.__dict__.pop(a, None)
        self[obj.partition].onNewNode(obj)
        return obj

    def updateLastNode(self, obj):
        last = obj.__lock__
        if obj.inode != last.inode:
            raise AttributeError(
                'Attribute inode is immutable', last.inode, obj.inode)
        upd = {}
        for attr in 'klass', 'parent_node', 'partition':
            ov = getattr(last, attr)
            nv = getattr(obj, attr)
            if ov != nv:
                upd[attr] = nv
        if upd:
            chset = changeset_manager().newChangeset.pkey
            if chset > last.changeset_from:
                # first change in transaction
                last.changeset_till = chset - 1
                self.workspace.flush(last)
                rn = RepositoryNode(
                        inode = last.inode,
                        partition = upd.get('partition', last.partition),
                        changeset_from = chset,
                        changeset_till = MAX_CHSET,
                        klass = upd.get('klass', last.klass),
                        parent_node = upd.get('parent_node', last.parent_node),
                )
                self.workspace.add(rn)
            else:
                # second, third, ...  change in transaction
                last._setattrs(**upd)
                self.workspace.flush(last)
        self.saveDocument(obj.elements)
        self[obj.partition].onUpdateNode(obj)

    def removeLastNode(self, obj):
        obj.elements.children[:] = []
        chset = changeset_manager().newChangeset.pkey
        last = obj.__lock__
        inode = last.inode
        if chset > last.changeset_from:
            # old node
            last.changeset_till = chset - 1
            self.workspace.flush(last)
        else:
            # node inserted in current transaction
            self.workspace.remove(last)
        self.saveDocument(obj.elements) # XXX is it necessary?
        self[obj.partition].onRemoveNode(obj)

    def undeleteNode(self, node_version):
        new_chset = changeset_manager().newChangeset.pkey
        und_chset = node_version.changeset
        inode = node_version.inode
        q = """INSERT INTO repository_node
               SELECT partition, inode, %d, %d, klass, parent_node
                 FROM (SELECT partition, inode, changeset_from, changeset_till,
                              klass, parent_node
                         FROM repository_node
                        WHERE inode = %d
                     ORDER BY changeset_till DESC LIMIT 1
               ) AS rn WHERE changeset_till <> %d""" % (
                new_chset, MAX_CHSET, inode, MAX_CHSET)
        self.workspace.connection(q)
        q = """INSERT INTO repository_element (
                    inode, eid, changeset_from, changeset_till,
                    kind, name, value, parent_id, position
               ) SELECT inode, eid, %d, %d,
                        kind, name, value, parent_id, position
                   FROM repository_element
                  WHERE inode = %d
                    AND %s between changeset_from and changeset_till""" % (
                        new_chset, MAX_CHSET, inode, und_chset)
        self.workspace.connection(q)
        self[node_version.partition].onNewNode(node_version)
        return new_chset


    # ------- element SPI ---------------------

    def getPublicElement(self, klass, key, default=NOT_GIVEN):
        eid = int(key)
        q = self.workspace.parse(
                '[e.inode for e in RepositoryElement if e.eid == eid]')
        q.limit = query.Const(1)
        inode = -self.workspace(q)
        if inode is None:
            if default is NOT_GIVEN:
                raise KeyError('PublicElement not found', key)
            return default
        pn = self.getNode(PublicNode, inode)
        return pn.findElementByID(eid)

    documents = binding.Make(dict)

    def getDocumentData(self, inode, changeset, reload=False):
        key = (inode, changeset)
        document = self.documents.get(key)
        if reload or document is None:
            branches = {}
            elements = {}
            for elem in self.workspace('''[
                obj for obj in RepositoryElement
                     if obj.inode == inode
                    and obj.changeset_from <= changeset <= obj.changeset_till
                    and query.OrderBy(obj.position)
            ]'''):
                branches.setdefault(elem.parent_id, []).append(elem)
                elements[elem.eid] = elem
            document = self.documents[key] = dict(
                    elements=elements,
                    branches=branches
            )
        return document

    def getDocument(self, klass, node_version, default=NOT_GIVEN):
        try:
            facade = node_version.__class__
            inode = int(node_version.inode)
            changeset = int(node_version.changeset)
        except:
            raise KeyError('Invalid key', key)
        oid = (facade, inode, changeset)
        self.joinedTxn
        ws = self.workspace
        ectx = ws.contextFor(klass)
        obj = ectx.cache.get(oid)
        if obj is not None:
            return obj
        dct = self.getDocumentData(inode, changeset)
        if dct is None:
            return None
        obj = schema.Entity.__new__(klass)
        obj.__dict__.update(dct)
        obj.__dict__['_workspace'] = ws
        obj.__dict__['_oid'] = oid
        obj.__dict__['inode'] = inode
        obj.__dict__['changeset'] = changeset
        obj.__dict__['node'] = node_version
        obj.__dict__['facade'] = facade
        ectx.cache[oid] = obj
        return obj

    def preloadDocument(self, obj):
        facade, inode, changeset = obj._oid
        dct = self.getDocumentData(inode, changeset)
        if dct is None:
            return None
        for k,v in dct.iteritems():
            if k not in obj.__dict__:
                obj.__dict__[k] = v
        obj.__dict__.setdefault('facade', facade)
        obj.__dict__.setdefault('inode', inode)
        obj.__dict__.setdefault('changeset', changeset)
        if 'node' not in obj.__dict__:
            obj.__dict__['node'] = self.getNode(
                klass=facade, inode=inode, changeset=changeset
            )

    def saveDocument(self, obj):
        chmgr = changeset_manager()
        elements = obj.elements.copy()
        ws = settings.workspace()
        chset = changeset_manager().newChangeset.pkey
        for elem in obj.iterDepthFirst(include_self=False):
            old = elements.pop(elem._oid, None)
            if old is None:
                new = RepositoryElement(
                        eid = elem.eid,
                        inode = obj.inode,
                        kind = elem.kind,
                        name = elem.name,
                        value = elem.value,
                        position = elem.position,
                        parent_id = elem.parent_id,
                        changeset_from = chset,
                )
                ws.add(new)
                elem.__dict__['eid'] = new.eid
                elem.__dict__['_oid'] = new.eid
            else:
                upd = {}
                for a in 'kind', 'name', 'value', 'position', 'parent_id':
                    ov = getattr(old, a)
                    nv = getattr(elem, a)
                    if ov != nv:
                        upd[a] = nv
                if upd:
                    if chset > old.changeset_from:
                        old.changeset_till = chset - 1
                        ws.flush(old)
                        new = RepositoryElement(
                                inode = obj.inode,
                                eid = old.eid,
                                kind = upd.get('kind', old.kind),
                                name = upd.get('name', old.name),
                                value = upd.get('value', old.value),
                                position = upd.get('position', old.position),
                                parent_id = upd.get('parent_id', old.parent_id),
                                changeset_from = chset,
                        )
                        ws.add(new)
                    else:
                        old._setattrs(**upd)
                        ws.flush(old)
            if hasattr(elem, '_value'):
                key = elem.__class__._value.key(elem)
                if hasattr(elem, 'value_translations'):
                    ws['dbgettext'].update(key, elem.value_translations)
                elif isinstance(elem._value,Message) and elem._value.translations:
                    ws['dbgettext'].update(key, elem._value.translations)
                else:
                    ws['dbgettext'].update(key, {})
        for eid, old in elements.iteritems():
            if chset > old.changeset_from:
                old.changeset_till = chset - 1
                ws.flush(old)
            else:
                ws.remove(old)
        self.getDocumentData(obj.inode, MAX_CHSET, reload=True)
