BEGIN;

CREATE TABLE repository_changeset (
    pkey                SERIAL NOT NULL,
    created_by          VARCHAR,
    creation_time       TIMESTAMP WITH TIME ZONE NOT NULL,

    PRIMARY KEY (pkey)
);

CREATE TABLE repository_node (
    partition       VARCHAR NOT NULL,
    inode           SERIAL NOT NULL,
    changeset_from  INTEGER NOT NULL,
    changeset_till  INTEGER NOT NULL,

    klass           VARCHAR NOT NULL,
    parent_node     INTEGER,

    PRIMARY KEY (inode, changeset_from)
);

CREATE TABLE repository_element (
    inode               INTEGER NOT NULL,

    eid                 SERIAL NOT NULL,
    changeset_from      INTEGER NOT NULL,
    changeset_till      INTEGER NOT NULL,

    kind                VARCHAR NOT NULL,
    name                VARCHAR,
    value               VARCHAR,

    parent_id           INTEGER,
    position            INTEGER NOT NULL,

    PRIMARY KEY (eid, changeset_from),
    CHECK (changeset_from <= changeset_till)
);
CREATE INDEX idx_repository_element_inode ON repository_element (inode);
CREATE INDEX idx_repository_element_name ON repository_element (kind, name);

CREATE TABLE repository_file (
    pkey            SERIAL NOT NULL,
    blob            OID NOT NULL,
    size            INTEGER NOT NULL,
    content_type    VARCHAR NOT NULL DEFAULT 'application/octet-stream',
    filename        VARCHAR,
    modified        TIMESTAMP WITH TIME ZONE NOT NULL,
    changeset_from  INTEGER NOT NULL,
    changeset_till  INTEGER NOT NULL,

    PRIMARY KEY(pkey)
);

CREATE TABLE repository_image (
    pkey            SERIAL NOT NULL,
    blob            OID NOT NULL,
    size            INTEGER NOT NULL,
    content_type    VARCHAR NOT NULL,
    filename        VARCHAR,
    modified        TIMESTAMP WITH TIME ZONE NOT NULL,
    width           INTEGER,
    height          INTEGER,
    changeset_from  INTEGER NOT NULL,
    changeset_till  INTEGER NOT NULL,

    PRIMARY KEY(pkey)
);

CREATE VIEW repository_node_versions AS
    SELECT inode, changeset_from AS changeset
      FROM repository_node
  UNION
    SELECT inode, changeset_from AS changeset
      FROM repository_element
  UNION
    SELECT inode, changeset_till+1 AS changeset
      FROM repository_element
     WHERE changeset_till != 2147483647
;

CREATE VIEW repository_node_lastmod AS
    SELECT inode, max(changeset) AS changeset
      FROM repository_node_versions
  GROUP BY inode
;

CREATE TABLE cms_public_version (
    pkey        SERIAL NOT NULL,
    inode       INTEGER NOT NULL,
    changeset   INTEGER NOT NULL,
    pstart      DATE NOT NULL,
    pend        DATE NOT NULL DEFAULT '3000-1-1',

    PRIMARY KEY (pkey)
);


CREATE TABLE cms_publishing_workflow (
    inode       INTEGER NOT NULL,
    changeset   INTEGER NOT NULL,
    req         VARCHAR NOT NULL DEFAULT 'publish',
    req_chset   INTEGER NOT NULL,
    pstart      DATE NOT NULL,
    pend        DATE NOT NULL DEFAULT '3000-1-1',
    res         VARCHAR,
    res_chset   INTEGER,
    message     VARCHAR,

    PRIMARY KEY (inode)
);

END;
