from datetime import date, datetime

from lib4.imports import importObject
from lib4.trees import AbstractTreeNode
from adt4.serial import Serial

from wm4.api import *
from wm4.schema.api import Entity, Attribute, Reference, Required, Default
from wm4.schema.api import defaultDatetime, ComputeDefault, PrimaryKey
from wm4.schema.api import Workspace

from comp4.files.entities import FileBase, ImageBase

from config import MAX_CHSET, manager, changeset_manager

__all__ = [
    'OnPublish', 'OnUnpublish',
    'default_changeset', 'ChangesetedAttr',
    'RepositoryChangeset', 
    'RepositoryNode', 'RepositoryNodeVersions', 'RepositoryNodeLastmod',
    'RepositoryElement', 'RepositoryFile', 'RepositoryImage',
    'CmsPublicVersion', 'CmsPublishingWorkflow', 
]




#################### Events ######################


class OnPublish(events.Event):
    '''Triggered when node is published (automaticaly or manually).

       The sender is Node class.
    '''
    __slots__ = 'node', 'publishing'


class OnUnpublish(events.Event):
    '''Triggered when node is removed.

       The sender is Node class.
    '''
    __slots__ = 'node',




#################### Utilities ######################


def _default_changeset(obj, attr, values):
    return changeset_manager().newChangeset.pkey
default_changeset = config.Value(_default_changeset)



class ChangesetedAttr(object):
    '''Attribute that is cached across transaction boundaries
       and is invalidated when new changeset appears in database
    '''
    
    def __init__(self, recipe):
        self.recipe = recipe

    def __get__(self, obj, typ=None):
        key = (id(obj), id(self))
        val = changeset_manager.data.get(key, NOT_FOUND)
        if val is NOT_FOUDN:
            changeset_manager.data[key] = val = self.recipe(obj)
        return val

    def __set__(self, obj, val):
        key = (id(obj), id(self))
        changeset_manager.data[key] = val

    def __delete__(self, obj):
        key = (id(obj), id(self))
        del changeset_manager.data[key]






#################### Pure database entities ######################


class RepositoryChangeset(Entity):
    def default_user(o,a,v):
        uf = settings.current_user_factory()
        if uf is None:
            return None
        ui = uf()
        if ui is None:
            return None
        else:
            return ui.login
    pkey = Attribute(Serial)
    created_by = Attribute(unicode, ComputeDefault(config.Value(default_user)))
    creation_time = Attribute(datetime, Required, defaultDatetime)

    binding.metadata(PrimaryKey('pkey'))


class RepositoryNode(Entity):
    partition       = Attribute(unicode, Required)
    inode           = Attribute(Serial)
    changeset_from  = Attribute(int)
    changeset_till  = Attribute(int, Required, Default(MAX_CHSET))

    klass           = Attribute(unicode, Required)
    parent_node     = Attribute(int)

    binding.metadata(PrimaryKey('inode', 'changeset_from'))


class RepositoryNodeVersions(Entity):
    '''View repository_node_versions'''
    inode           = Attribute(int)
    changeset       = Attribute(int)
    binding.metadata(PrimaryKey('inode', 'changeset'))


class RepositoryNodeLastmod(Entity):
    '''View repository_node_versions'''
    inode           = Attribute(int)
    changeset       = Attribute(int)
    binding.metadata(PrimaryKey('inode'))


class RepositoryElement(Entity):
    inode           = Attribute(int, Required)

    eid             = Attribute(Serial)
    changeset_from  = Attribute(int)
    changeset_till  = Attribute(int, Required, Default(MAX_CHSET))

    kind            = Attribute(str, Required)
    name            = Attribute(unicode)
    value           = Attribute(unicode)

    parent_id       = Attribute(int)
    position        = Attribute(int, Required, Default(0))

    binding.metadata(PrimaryKey('eid', 'changeset_from'))


class RepositoryFile(FileBase):
    pkey            = Attribute(Serial)
    changeset_from  = Attribute(int,Required,ComputeDefault(default_changeset))
    changeset_till  = Attribute(int,Required,Default(MAX_CHSET))

    binding.metadata(PrimaryKey('pkey'))


class RepositoryImage(ImageBase):
    pkey            = Attribute(Serial)
    changeset_from  = Attribute(int,Required,ComputeDefault(default_changeset))
    changeset_till  = Attribute(int,Required,Default(MAX_CHSET))

    binding.metadata(PrimaryKey('pkey'))


class CmsPublicVersion(Entity):
    pkey      = Attribute(Serial)
    inode     = Attribute(int, Required)
    changeset = Attribute(int, Required)
    pstart    = Attribute(date, Required)
    pend      = Attribute(date, Required)

    binding.metadata(PrimaryKey('pkey'))


class CmsPublishingWorkflow(Entity):
    inode     = Attribute(int)
    changeset = Attribute(int, Required)
    req       = Attribute(str, Required)
    req_chset = Attribute(int, Required)
    pstart    = Attribute(date, Required)
    pend      = Attribute(date, Required)
    res       = Attribute(str)
    res_chset = Attribute(int)
    message   = Attribute(unicode)

    req_changeset = binding.Make(lambda s: s._workspace.get(
                        RepositoryChangeset, s.req_chset, default=None
    ))

    res_changeset = binding.Make(lambda s: s._workspace.get(
                        RepositoryChangeset, s.res_chset, default=None
    ))
    
    node = binding.Make(lambda s: s._workspace.get('cms/LastNode', s.inode))

    binding.metadata(PrimaryKey('inode'))





