# -*- coding: utf-8 -*-

from wm4.api import *
from dispatch.strategy import default
from lib4.text import unicode2ascii

from pagelets import *
from config import RootProperty, manager
from facades import PublicNode

__all__ = ['fulltextVector', 'getVector']


def plainVector(element, weight, plain):
    yield element.value, plain, weight
    if RootProperty(element.node.partition, 'fulltext.index_ascii')(
                                                            default=False):
        try:
            asc = unicode2ascii(element.value)
            yield asc, asc, weight
        except:
            pass
    if hasattr(element, '_value') and RootProperty(element.node.partition,
                                'fulltext.index_translations')(default=False):
        for t in element.node._workspace['dbgettext'].get(
                                            element._value.key).itervalues():
            yield t, t, weight


@dispatch.generic()
def fulltextVector(element):
    pass


@fulltextVector.when(default)
def default_ft_vector(element):
    return []

@fulltextVector.when('element in Title')
def default_ft_vector(element):
    return plainVector(element, 'A', element.value)

@fulltextVector.when('element in Abstract')
def default_ft_vector(element):
    return plainVector(element, 'C', element.value)

@fulltextVector.when('element in Rest')
def default_ft_vector(element):
    return plainVector(element, 'D', element.value)

@fulltextVector.when('element in Html')
def default_ft_vector(element):
    from elementtree import HTMLTreeBuilder
    from StringIO import StringIO
    def getText(e):
        if e.text:
            yield e.text
        for c in e._children:
            for t in getText(c):
                yield t
        if e.tail:
            yield e.tail
    html = (u'<html>%s</html>' % element.value)
    try:
        plain=''.join(getText(HTMLTreeBuilder.parse(StringIO(html)).getroot()))
    except:
        return []

    return plainVector(element, 'D', plain)



def getVector(node):
    from comp4.fulltext import api
    vector = []
    plain  = []
    for e in node.elements.iterDepthFirst():
        for t,p,w in fulltextVector(e):
            if t:
                vector.append((t.lower(),w))
            if p:
                plain.append(p)
    vector = api.TSVector('cs', *vector)
    plain = ' '.join(plain)
    return vector, plain


def updateFulltext(node):
    try:
        disabled = config.Property('fulltext.disabled_partitions')()
    except exceptions.PropertyNotFound:
        disabled = []
    if node.partition in disabled: 
        removeFulltext(node)
        return None
    vector, plain = getVector(node)
    return node._workspace['fulltext'].updateVector(
        'cms/PublicNode', node.inode, vector, plain, node.partition
    )

def updateFulltextOnPublish(evt):
    # XXX Tohle je prispevek do souteze o nejblbejsi funkci na svete.
    #     Jelikoz v CMS nejde udelat nic, aniz by to nerozmrdalo
    #     vsechny aplikace, uchylime se zde ke dvema forkum a cele
    #     fulltextovani provedeme v demonovi v separe procesu.
    #     Nekdy se to ovsem nemusi povest, protoze je zde natvrdo
    #     cekani 3 sekundy, nez se bourliva situace po zavedeni
    #     dokumentu do databaze uklidni a budou k dispozici vsechna
    #     potrebna data pro fulltext. Jeste jednou sorry.
    #     V pristi verzi uz budeme mit message queue, ktera tyhle
    #     veci bude resit.
    from wm4 import app4
    import time
    import sys
    import os
    import signal
    inode = evt.node.inode
    app_name = settings.application_name()
    if os.fork() == 0:
        os.setsid()
        signal.signal(signal.SIGHUP, signal.SIG_IGN)
        if os.fork() > 0:
            os._exit(0)
        try:
            maxfd = os.sysconf("SC_OPEN_MAX")
        except (AttributeError, ValueError):
            # default maximum
            maxfd = 256
        for fd in range(maxfd):
            try:
                os.close(fd)
            except OSError:
                pass
        os.open("/dev/null", os.O_RDONLY) # standard input (0)
        os.open("/dev/null", os.O_RDWR)   # standard output (1)
        os.open("/dev/null", os.O_RDWR)   # standard error (2)
        try:
            # Tady to vlastne zacina
            app4.config_app(app_name)
            time.sleep(2)
            storage.begin()
            mgr = manager()
            node = mgr.getNode(PublicNode, inode)
            updateFulltext(node)
        except Exception:
            storage.abort()
        else:
            storage.commit()
        os._exit(0)

def removeFulltext(node):
    node._workspace['fulltext'].removeVector('cms/PublicNode', node.inode)

def updateFulltextOnUnpublish(evt):
    removeFulltext(evt.node)


