# -*- coding: utf-8 -*-
from wm4.api import *
from xml4.elements import Literal
from comp4.files.web import makeResponse
from comp4.dbgettext.api import Translations
import os

from config import RootProperty
from facades import Element, ChildProxy, ChildrenProxy
from entities import RepositoryFile, RepositoryImage

__all__ = [
    'PageElement', 'Toplevel', 'TranslatablePagelet',
    'Pagelet', 'Title', 'Heading', 'Abstract', 'Rest', 'Html', 'HtmlSrc',
    'Contents', 'ContentsByTag', 'Siblings',
    'ImageBase', 'Image', 'Illustration', 
    'PhotogalleryImage', 'PhotogalleryImageThumbnail', 'Photogallery',
    'FileBase', 'File', 'LinkBox', 'LinkBoxLink',
    'Discussion', 'Poll', 'MailForm', 'VideoFtp',
]

_ = i18n.MessageIDFactory('comp4.cms')


class PageElement(Element):
    plName = _(u'PageElement')


class Toplevel(PageElement):
    plName = _(u'Document')

    title = ChildProxy('cms', 'head.title')
    pagelets = ChildrenProxy('cms', 'pagelet.')


class Pagelet(PageElement):
    _dbgettext_key = binding.Make(lambda s: s.eid)
    _value = Translations('value')



class TranslatablePagelet(Pagelet):
    _value = Translations('value')
                

class Title(TranslatablePagelet):
    plName = _(u'Title')

class Heading(TranslatablePagelet):
    plName = _(u'Heading')

class Abstract(TranslatablePagelet):
    plName = _(u'Abstract')


class Rest(TranslatablePagelet):
    plName = _(u'Structured text')

    def html(self):
        from lib4.rest import reST2html
        if self.value:
            return Literal(reST2html(self.value, initial_header_level=2))
        else:
            return ''
    html = binding.Make(html)

class Html(TranslatablePagelet):
    plName = _(u'HTML text')

class HtmlSrc(TranslatablePagelet):
    plName = _(u'HTML source')

class Contents(Pagelet):
    plName = _(u'Contents')
    

class ContentsByTag(Pagelet):
    plName = _(u'Tagged documents')
    listing = None
    date_range = ChildProxy('cms','pagelet.contents_by_tag.date_range')


class Siblings(Pagelet):
    plName = _(u'Sibling documents')
    title = ChildProxy('cms','pagelet.siblings.title')

class ImageBase(Pagelet):

    def img(self):
        if not self.value:
            return None
        else:
            return settings.workspace().get(RepositoryImage, self.value)
    img = binding.Make(img)

    def do_download(self, req):
        if not self.img:
            raise web.NotFound(req.requestURL)
        return makeResponse(req, self.img, disposition='inline')

class Image(ImageBase):
    plName = _(u'Image')
    align = ChildProxy('cms','pagelet.image.align')
    description = ChildProxy('cms','pagelet.image.description')
    alt = ChildProxy('cms','pagelet.image.alt')
    url = ChildProxy('cms','pagelet.image.url')


class Illustration(ImageBase):
    plName = _(u'Illustration')

    
class PhotogalleryImage(ImageBase):
    plName = _(u'Photogallery image')
    description = ChildProxy('cms','pagelet.photogallery.image.description')
    thumbnail = ChildProxy('cms','pagelet.photogallery.image.thumbnail')



class PhotogalleryImageThumbnail(ImageBase):
    plName = _(u'Photogallery thumbnail')
    


class Photogallery(Pagelet):
    plName = _(u'Photogallery')
    description = ChildProxy('cms','pagelet.photogallery.description')
    images = ChildrenProxy('cms','pagelet.photogallery.image')

    def do_add(self, req):
        from lib4.images import loadImage, getMIME
        img_conv = RootProperty(
            self.node.partition, 'base_image.converter')()
        thumb_conv = RootProperty(
            self.node.partition, 'pagelet_photogallery.thumbnail_converter')()

        img = req.getUpload('image')
        filename = img.filename
        ipath = img.path

        img = loadImage(ipath)
        img = img_conv(img)
        img.save(ipath, img.format)
        imgR = self._workspace.new(RepositoryImage,
            blob         = open(ipath),
            size         = os.stat(ipath).st_size,
            content_type = getMIME(img),
            filename     = filename,
            width        = img.size[0],
            height       = img.size[1],
        )
        pos = 0
        if self.children:
            pos = max(c.position for c in self.children) + 1
        imgE = self._workspace.new(Element,
            inode     = self.inode,
            kind      = 'cms',
            name      = 'pagelet.photogallery.image',
            value     = imgR._oid,
            parent_id = self.eid,
            position  = pos,
        )

        tpath = ipath + '_thumb'
        thumb = thumb_conv(img)
        thumb.save(tpath, thumb.format)
        thumbR = self._workspace.new(RepositoryImage,
            blob         = open(tpath),
            size         = os.stat(tpath).st_size,
            content_type = getMIME(thumb),
            filename     = filename,
            width        = thumb.size[0],
            height       = thumb.size[1],
        )
        thumbE = self._workspace.new(Element,
            inode     = self.inode,
            kind      = 'cms',
            name      = 'pagelet.photogallery.image.thumbnail',
            value     = thumbR._oid,
        )
        desc = req.getValue('description', default='').strip()
        if desc:
            descE = self._workspace.new(Element,
                inode     = self.inode,
                kind      = 'cms',
                name      = 'pagelet.photogallery.image.description',
                value     = desc,
                parent_id = imgE.eid,
                position  = 1,
            )
        resp = req.response
        resp.body = ['OK']
        return resp




class FileBase(Pagelet):

    plName = _(u'File')

    def file(self):
        if not self.value:
            return None
        else:
            return settings.workspace().get(RepositoryFile, self.value)
    file = binding.Make(file)

    def do_download(self, req):
        if not self.file:
            raise web.NotFound(req.requestURL)
        if self.file.content_type.split(';')[0] in (
            'application/xhtml+xml', 'text/html', 'text/plain',
            'image/jpeg', 'image/png', 'image/jpeg',
        ):
            return makeResponse(req, self.file, disposition='inline')
        else:
            return makeResponse(req, self.file, disposition='attachment')


class File(FileBase):
    description = ChildProxy('cms','pagelet.file.description')














class Discussion(Pagelet):
    plName = _(u'Discussion')

    def forum(self):
        from comp4.forum.api import Forum
        return settings.workspace().get(Forum, self.value)
    forum = binding.Make(forum)

    def do_discussion(self, req):
        tmpl = RootProperty(self.node.partition,'pagelet_discussion.template')()
        tmpl = web.getResource(tmpl)
        return tmpl.handleHTTP(req, forum=self.forum, node=self.node)


class Poll(Pagelet):
    plName = _(u'Poll')

    question = None
    options = ()

    def do_poll(self, req):
        from comp4.poll.api import Poll
        poll_id = 'doc.%d.%d' % (self.node.inode, self.eid)
        tmpl = RootProperty(self.node.partition,'pagelet_poll.template')()
        tmpl = web.getResource(tmpl)
        poll = Poll(poll_id=poll_id)
        return tmpl.handleHTTP(req, poll=poll, node=self.node)


class LinkBoxLink(Pagelet):
    plName = _(u'Link')
    description = ChildProxy('cms','pagelet.linkbox.link.description')

class LinkBox(Pagelet):
    plName = _(u'Linkbox')
    links = ChildrenProxy('cms', 'pagelet.linkbox.link')




class MailForm(Pagelet):
    plName = _(u'Mailform')
    email = ChildProxy('cms', 'pagelet.mailform.email')


class VideoFtp(Pagelet):
    plName = _(u'Video')

    width = ChildProxy('cms', 'pagelet.videoftp.width')
    height = ChildProxy('cms', 'pagelet.videoftp.height')
    description = ChildProxy('cms', 'pagelet.videoftp.description')

