# -*- coding: utf-8 -*-
from wm4.api import *
from xml4 import xhtml as xh

from facades import Element
from entities import RepositoryNode, RepositoryElement, CmsPublicVersion

__all__ = [
    'StylesPropertyManager', 'styles_property_manager',
    'MetadataElement', 'MetadataPropertyManager', 'metadata_property_manager',
    'CalendarListing', 'CalendarElement', 'CalendarPropertyManager',
    'calendar_property_manager',
    'TagElement', 'TagPropertyManager', 'tag_property_manager',
]


class SelfRemovablePropertyElement(Element):

    def do_delete(self, req):
        obj = req.params['obj']
        url = req.absURL(obj, 'properties')
        last = obj.findElementByID(self.eid, default=None)
        if last is None:
            raise web.SeeOther(url)
        last.parent.children.remove(last)
        storage.commit()
        storage.begin()
        raise web.SeeOther(url)



class StylesPropertyManager(object):

    def __call__(self, obj, req):
        from config import RootProperty
        styles = RootProperty(
            obj.partition, 'document_styles.%s' % obj.toplevel)()
        current = obj.style

        action = self.__class__.__name__
        form_id = action.upper()

        if req.getValue(action, None):
            ws = settings.workspace()
            new = req.getValue('style', default=None)
            obj.document.value = new
            url = req.absURL(obj, 'properties')
            storage.commit()
            storage.begin()
            raise web.SeeOther(url)

        sel = xh.select(xh.option(''), name='style')
        for s in styles:
            if s == current:
                sel.append(xh.option(s, selected='selected'))
            else:
                sel.append(xh.option(s))

        return xh.div(
            xh.h3(u'Styl dokumentu'),
            xh.form(
                dict(
                    action = req.absURL(obj,'properties'),
                    name = form_id,
                    id = form_id,
                    method = 'post',
                ),
                xh.div(
                    sel,
                    xh.input(type='submit', name=action, value=u'Nastavit')
                ),
            )
        )


styles_property_manager = config.Resource(
    'styles_property_manager', StylesPropertyManager
)





class MetadataElement(Element):
    pass


class MetadataPropertyManager(object):

    fields = 'title', 'author', 'description', 'keywords'
    
    def __call__(self, obj, req):
        action = self.__class__.__name__
        form_id = action.upper()

        old = dict(
            (f,obj.findElement('meta',f,default=None)) for f in self.fields
        )
            
        if req.getValue(action, None):
            ws = settings.workspace()
            for field in self.fields:
                value = req.getValue(field, default=None)
                if value:
                    if old[field] is None:
                        elem = obj.elements.children.append(Element(
                            kind = 'meta',
                            name = field,
                            value = value,
                        ))
                    else:
                        if old[field].value != value:
                            old[field].value = value
                else:
                    if old[field] is not None:
                        old[field].parent.children.remove(old[field])
                    
            url = req.absURL(obj, 'properties')
            storage.commit()
            storage.begin()
            raise web.SeeOther(url)

        v = dict((f,e and e.value or '') for f,e in old.items())

        return xh.div(
            xh.h3(u'Metadata'),
            xh.form(
                dict(
                    action = req.absURL(obj,'properties'),
                    name = form_id,
                    id = form_id,
                    method = 'post',
                ),
                xh.h4(u'Titulek'),
                xh.div(
                    xh.input(name='title', size="60", value=v['title'])
                ),
                xh.h4(u'Autor'),
                xh.div(
                    xh.input(name='author', size="40", value=v['author']),
                ),
                xh.h4(u'Popis'),
                xh.div(
                    xh.textarea(
                        v['description'],
                        name='description', rows="5", cols="60"),
                ),
                xh.h4(u'Klíčová slova'),
                xh.div(
                    xh.input(name='keywords', size="60", value=v['keywords']),
                ),
                xh.div(
                    xh.input(type='submit', name=action, value=u'Uložit'),
                ),
            )
        )

metadata_property_manager = config.Resource(
    'metadata_property_manager', MetadataPropertyManager
)









def CalendarListing(req, date, template):
    import datetime
    ws = settings.workspace()
    ad = date.replace(day=1)
    mgr = calendar_property_manager()
    prev = (ad - datetime.timedelta(25)).replace(day=1)
    next = (ad + datetime.timedelta(33)).replace(day=1)
    events = mgr.eventsForDate(ad)
    weeks = mgr.weeks(ad, events)
    events_for_date = events.get(date.day, [])
    tmpl = req.getResource(template)
    return tmpl.handleHTTP(
        req,
        weeks=weeks, date=date, events = events,
        events_for_date = events_for_date,
        prev = prev, next = next,
    )

class CalendarElement(SelfRemovablePropertyElement):

    def date(self):
        from lib4.timeutils import Date
        return Date(self.value)
    date = binding.Make(date)


class CalendarPropertyManager(object):
    
    def eventsForDate(self, ad):
        # XXX schvalovani
        from entities import CmsPublicVersion
        import datetime
        ws = settings.workspace()
        today = datetime.date.today()
        events = {}
        q = '''[
            (n.inode, n.partition, e.value)
                for n in RepositoryNode
                for e in RepositoryElement
                for p in CmsPublicVersion
                 if n.inode == p.inode == e.inode
                and n.changeset_from <= p.changeset <= n.changeset_till
                and e.changeset_from <= p.changeset <= e.changeset_till
                and p.pstart <= today <= p.pend
                and e.kind == 'prop' and e.name == 'calendar'
                and e.value[:6] == '%04d%02d'
            ]''' % (ad.year, ad.month)
        for (inode, part, day) in ws(q):
            doc = ws.get('cms/PublicNode', inode, default=None)
            if doc:
                events.setdefault(int(day[6:]),[]).append(doc)
        return events

    def nextEvents(self, refDate=None):
        from entities import CmsPublicVersion
        from lib4.timeutils import Date, Today
        today = Today()
        if refDate is None:
            refDate = today
        ws = settings.workspace()
        events = []
        inodes = {}
        q = '''[
            (n.inode, n.partition, e.value)
                for n in RepositoryNode
                for e in RepositoryElement
                for p in CmsPublicVersion
                 if n.inode == p.inode == e.inode
                and n.changeset_from <= p.changeset <= n.changeset_till
                and e.changeset_from <= p.changeset <= e.changeset_till
                and p.pstart <= today <= p.pend
                and e.kind == 'prop'
                and e.name == 'calendar'
                and e.value >= '%04d%02d%02d'
                and query.OrderBy(e.value)
            ]''' % (refDate.year, refDate.month, refDate.day)
        for (inode, part, day) in ws(q):
            doc = inodes.get(inode) # XXX bad caching
            if doc is None:
                doc = inodes[inode] = ws.get(
                    'cms/PublicNode', inode, default=None
                )
            if doc:
                events.append((Date(day), doc))
        return events

    def nextEventsGrouped(self, refDate=None):
        from datetime import date
        events = self.nextEvents(refDate)
        grouped = {}
        for d,e in events:
            g = date(d.year, d.month, 1)
            grouped.setdefault(g, {}).setdefault(e, []).append(d)
        for g, md in grouped.items():
            grouped[g] = sorted([(tuple(d),e) for e, d in md.items() ])
        return sorted(grouped.items())
        

    def weeks(self, ad, events=None):
        import calendar
        from datetime import date
        if events is None:
            events = self.eventsForDate(ad)
        y = ad.year
        m = ad.month

        class Day:
            css = None
            day = None
            date = None
            events = ()

        weeks = []
        for w in calendar.monthcalendar(y,m):
            week = []
            for i,d in enumerate(w):
                css = []
                day = Day()
                day.day = d
                if not d:
                    css.append('other-month')
                else:
                    day.date = date(y,m,d)
                    day.events = events.get(d,())
                    if day.events:
                        css.append('events')
                if i > 4:
                    css.append('weekend')
                day.css = ' '.join(css) or None
                week.append(day)
            weeks.append(week)
        return weeks
    
    def __call__(self, obj, req):
        action = self.__class__.__name__
        if req.getValue(action, None):
            from lib4.timeutils import Date
            d = Date(req.getValue('date'))
            obj.elements.children.append(Element(
                kind = 'prop',
                name = 'calendar',
                value = '%04d%02d%02d' % (d.year,d.month,d.day),
            ))
            url = req.absURL(obj, 'properties')
            storage.commit()
            storage.begin()
            raise web.SeeOther(url)
        clst = list(obj.findElements('prop', 'calendar'))
        existing = xh.ul()
        for ce in clst:
            url = req.get_url(obj, ce, 'delete')
            existing.append(xh.li(
                ce.date, ' ', xh.a('[ smazat ]', href=url)
            ))
        frm = xh.form(
            xh.input(type='text', size="12", name='date', id='CMS4-D-PROP'),
            xh.script(
              "$(document).ready(function(){$('#CMS4-D-PROP').datepicker();})",
              type="text/javascript"
            ),
            ' ',
            xh.input(type='submit', name=action, value=u'Přidat',klass='admin'),
            action = req.absURL(obj, 'properties'),
            name = 'CPF',
        )
        for res in (
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/jquery/ui.datepicker.js',
            'wm4.web.static/jquery/ui.datepicker-cs.js',
            'wm4.web.static/jquery/ui.datepicker.css',
        ):
            web.getResource(res).render(req)
        return  xh.div(
            xh.h3(u'Události v kalendáři'),
            existing,
            xh.p(
                xh.strong(u'Přidat dokument do kalendáře s datem:'),
            ),
            frm,
        )

calendar_property_manager = config.Resource(
    'calendar_property_manager', CalendarPropertyManager
)






class TagElement(SelfRemovablePropertyElement):
    tag = binding.Make(lambda s: settings.workspace().get('tags/Tag', s.value))



class TagPropertyManager(object):

    def __call__(self, obj, req):
        from config import RootProperty
        action = self.__class__.__name__
        ws = settings.workspace()

        if req.getValue(action, None):
            t = req.getValue('tag', default=None)
            e = obj.elements.children.append(Element(
                    kind = 'prop',
                    name = 'tag',
                    value = t,
            ))
            url = req.absURL(obj, 'properties')
            storage.commit()
            storage.begin()
            raise web.SeeOther(url)

        try:
            ident = RootProperty(obj.partition, 'tag_property.root')()
            root = ws['tags'].idents[ident]
            tags = root.iterDepthFirst(include_self=False)
        except exceptions.PropertyNotFound:
            tags = []

        form_id = action.upper()
        tlst = list(obj.findElements('prop', 'tag'))
        existing = xh.ul()
        for te in tlst:
            url = req.get_url(obj, te, 'delete')
            existing.append(xh.li(
                te.tag.name, ' ', xh.a('[ smazat ]', href=url)
            ))

        sel = xh.select(name='tag')
        if tags:
            for t in tags:
                sel.append(xh.option('..' * (t.treeLevel-1) + t.name, 
                                     value=t.pkey))
            frm = xh.form(
                xh.p(
                    xh.strong(u'Přidat štítek'),
                ),
                sel,
                ' ',
                xh.input(type='submit', name=action, value=u'Přidat',
                    klass='admin'),
                action = req.absURL(obj, 'properties'),
                name = form_id,
                id = form_id,
            )
        else:
            frm = ''
        return  xh.div(
            xh.h3(u'Štítky'),
            existing,
            frm,
        )



tag_property_manager = config.Resource(
    'tag_property_manager', TagPropertyManager
)


