# -*- coding: ISO-8859-2 -*-

from wm4.api import *
import comp4.acl.api as acl
from wm4.web.routes import *
from wm4.web.sitemaps import *
from wm4.web.navigation import Link
from wm4.utils.clspatch import patch

from facades import NodeVersion, LastNode, PublicNode, Element
from data_managers import PartitionManager
from config import RootProperty, manager, MAX_CHSET

__all__ = ['CmsSitemap', 'CmsSitemapWithAcl', 
           'AttachedCmsSitemap', 'AttachedCmsSitemapWithAcl']


_ = i18n.MessageIDFactory('comp4.cms')


def pathLinksWithAction(self):
    req = web.request()
    path = web.pathLinks(self)
    for o in self.pathObjs:
        r = req.route_for(o, self.action)
        path.append(Link(
            label=o.title, pathInfo=r.pathInfo, req=req
        ))
    return path



class CmsSitemap(Route):
    '''At least partition must be defined
    '''
    # API
    partition  = None

    # Parameters
    clsObj     = Const(LastNode)
    partMgr    = Param(lambda s: manager()[s.partition], typ=PartitionManager)
    controller = Const('comp4.cms.controllers.NodeForm')
    handler    = Const('comp4.cms.web.CMSHandler')

    # Defaults
    toplevel   = Const('toplevel.document')

    # Permissions
    viewPerm    = Const(None)
    editPerm    = Const(web.GroupName('admin'))
    publishPerm = Const(web.GroupName('admin'))
    permPerm    = Const(web.GroupName('admin'))

    permission = Param(lambda s:s.editPerm)
    workspace = Param(lambda s:settings.workspace(), typ=schema.Workspace)

    pathLinks = Const(())
    pathSlice = Const((0,-1))

    root_deletion = Const(False)

    def get_template(self, obj, req, root=False):
        # [CMS Root *]
        # document_styles.* = ['styleA', 'styleB']
        # view_template.* = 'myapp/default.wt'
        # view_template.toplevel.document.styleA = 'myapp/styleA.wt'
        # view_template.toplevel.document.styleB = 'myapp/styleB.wt'
        if root:
            prop = 'root_template.%s' % obj.document.name
        else:
            prop = 'view_template.%s' % obj.document.name
        if obj.document.value:
            prop += '.' + obj.document.value
        res = RootProperty(self.partition, prop)()
        return req.getResource(res) 
    get_template = Const(get_template)

    class part(Route):
        virtual = True
        pathLinks = Lazy(web.pathLinks)

        class root(Route):
            match = ''
            action = Const('root')
            signature = Signature('partMgr')
            obj = Param(
                lambda s: s.partMgr.getRoot(public=False),typ=LastNode)
            public = Param(
                lambda s: s.partMgr.getRoot(public=True),typ=PublicNode)
            empty = Const('comp4.cms.templates/empty.wt')
            permission = Param(lambda s:s.viewPerm)
            metadata = Lazy(lambda s: s.obj and s.obj.metadata)

            def actionLinks(self):
                req = web.request()
                if not self.obj:
                    return [Link(
                        label = _(u'Create main section document'),
                        pathInfo = (self / './add_root').pathInfo,
                        enabled = req.allows(self.editPerm),
                    )]
                else:
                    route = self.__class__.__root__.route_for(self.obj)
                    actions = web.actionLinks(
                        route, 'edit', 'properties', 'versions',
                        'permissions', 'directory', 'delete', 'add',
                    )
                    actions.insert(0, Link(
                        label=_(u'View'), pathInfo=self.pathInfo,
                        enabled = req.allows(self.viewPerm),
                    ))
                    return actions
            actionLinks = Lazy(actionLinks, inherit=False)

            class add_root(Route):
                match = 'add_root'
                action = Const('add_root')
                signature = Signature('partMgr', 'action')
                # XXX lookup in config or parent
                toplevel = Const('toplevel.document')
                template = Const('comp4.cms.templates/add.wt')
                actionLinks = Const(())
                permission = Param(lambda s:s.editPerm)

                class toplevel(Route):
                    match = '/toplevel/:toplevel'
                    action = Const('add_root')
                    toplevel = Param()
                    signature = Signature('partMgr', 'action', 'toplevel')
                class redir(Route):
                    match = '/redir'
                    action = Const('add_redir')
                    signature = Signature('partMgr', 'action')

            class mailform(Route):
                match = 'mailform'
                action = Const('mailform')
                permission = Param(lambda s:s.viewPerm)
                template = Const('comp4.cms.templates/mailform.wt')
                signature = Signature('partMgr', 'action')

        class multiop(Route):
            match = 'multiop'
            permission = Param(lambda s:s.editPerm)
            action = Const('multiop')
            class delete_many(Route):
                match = '/delete'
                action = Const('delete_many')
            class move_many(Route):
                match = '/move'
                action = Const('move_many')
                template = Const('comp4.cms.templates/move.wt')

        class popup(Route):
            match = 'popup'
            permission = Param(lambda s:s.editPerm)
            class tree(Route):
                match = '/tree'
                template = Const('comp4.cms.templates/popup_tree.wt')
    
    class publishing_requests(Route):
        match = 'publishing_requests'
        template = Const('comp4.cms.templates/publishing_requests.wt')
        action = Const('publishing_requests')
        permission = Param(lambda s:s.editPerm | s.publishPerm)
        
    class doc(Route):
        match = ':inodepath/'
        virtual = True

        def inodepath(self):
            obj = self.__dict__.get('obj')
            if isinstance(obj, LastNode):
                o = self.obj
            else:
                o = self.public
            return o.inodepath
        inodepath = Param(inodepath, typ=str)

        def inode(self):
            return int(self.inodepath.split('-')[0])
        inode = Param(inode, typ=int)

        obj = Param(
                lambda s: s.workspace.get(LastNode, s.inode, default=None),
                typ=LastNode)

        public = Param(
                lambda s: s.workspace.get(PublicNode, s.inode, default=None),
                typ=PublicNode)

        partition = Param(lambda s: s.obj and s.obj.partition or None, typ=unicode,
                          equalTo='../partition')

        metadata = Lazy(lambda s: s.obj.metadata)
        asAction = Lazy(Link.fromRoute, virtual=True)
        actionLinks = Lazy(lambda s: web.actionLinks(
            s,
            '', 
            'edit', 
            'properties', 
            'versions',
            'permissions', 
            'directory', 
            'delete',
            'add'
        ))

        def pathObjs(self):
            l,h = self.pathSlice
            objs = [p for p in self.obj.iterParentsRev()]
            return objs[l:h]
        pathObjs = Lazy(pathObjs)

        def pathLinks(self):
            req = web.request()
            path = web.pathLinks(self)
            for o in self.pathObjs:
                r = req.route_for(o)
                path.append(Link(
                    label=o._title, pathInfo=r.pathInfo, req=req
                ))
            return path
        pathLinks = Lazy(pathLinks)

        class view(Route):
            match = ''
            action = Const('view')
            signature = Signature('obj') | Signature('obj', 'action') |\
                        Signature('public') | Signature('public', 'action')
            permission = Param(lambda s:s.viewPerm)

            def asAction(self):
                label = _(u'View')
                if self.obj.parent:
                    return Link.fromRoute(self, label=label)
                else:
                    return Link(
                        label=label, pathInfo=(self/'../').pathInfo
                    )
            asAction = Lazy(asAction)

        class element(Route):
            match = 'element/:eid/'
            elemAction = Const('view')
            signature = Signature('obj','elem') | Signature('public','elem')
            action = Const('element')
            permission = Param(lambda s:s.viewPerm)
            eid = Param(lambda s: s.elem.eid, typ=int)
            def elem(s):
                if s.public is None:
                    return None
                else:
                    return s.public.findElementByID(s.eid, default=None)
            elem = Param(elem, typ=Element)

            class elem_action(Route):
                match = ':elemAction'
                elemAction = Param(typ=str)
                signature = Signature('obj','elem','elemAction') | \
                            Signature('public','elem','elemAction')

        class version(Route):
            match = 'version/:changeset'
            action = Const('version')
            permission = Param(lambda s:s.editPerm)
            changeset = Param(lambda s:s.version.changeset, typ=int)
            version = Param(
                lambda s: s.workspace.get(
                        NodeVersion,(s.inode,s.changeset), default=None),
                typ=NodeVersion)
            signature = Signature('obj', 'action', 'changeset') | \
                        Signature('version')
            _inode = Param(lambda s: s.version.inode, equalTo='../inode')

            class undelete(Route):
                match = '/undelete'
                action = Const('undelete')
                signature = Signature('obj', 'action', 'changeset')

            class element(Route):
                match = '/element/:eid/'
                elemAction = Const('view')
                signature = Signature('version','elem')
                action = Const('element')
                eid = Param(lambda s: s.elem.eid, typ=int)
                def elem(s):
                    e = s.version.findElementByID(s.eid, default=None)
                    if e is None:
                        raise web.NotFound(
                            'element not found', s.obj.inode, s.eid)
                    return e
                elem = Param(elem, typ=Element)
                class elem_action(Route):
                    match = ':elemAction'
                    elemAction = Param(typ=str)
                    signature = Signature('version','elem','elemAction')

        class add(InstanceRoute):
            # XXX lookup in config or parent
            toplevel = Const('toplevel.document')
            template = Const('comp4.cms.templates/add.wt')
            asAction = Lazy(lambda s:Link.fromRoute(s,label=_(u'New document')))
            def pathObjs(self):
                l,_ = self.pathSlice
                objs = [p for p in self.obj.iterParentsRev()]
                # When adding we want also the last node in path
                return objs[l:]
            pathObjs = Lazy(pathObjs)
            def pathLinks(self):
                req = web.request()
                path = web.pathLinks(self)
                for o in self.pathObjs:
                    r = req.route_for(o)
                    path.append(Link(
                        label=o._title, pathInfo=r.pathInfo, req=req
                    ))
                return path
            pathLinks = Lazy(pathLinks)
            class toplevel(Route):
                match = '/:toplevel'
                action = Const('add')
                signature = Signature('obj', 'action', 'toplevel') |\
                            Signature('public', 'action', 'toplevel')
                toplevel = Param(typ=unicode)

            class redir(Route):
                match = '/redir'
                action = Const('add_redir')
                signature = Signature('obj', 'action')


        class edit(InstanceRoute):
            template = Const('comp4.cms.templates/edit.wt')
            asAction = Lazy(lambda s:Link.fromRoute(s, label=_(u'Edit')))

        class versions(InstanceRoute):
            template = Const('comp4.cms.templates/versions.wt')
            asAction = Lazy(lambda s:Link.fromRoute(s, label=_(u'Versions')))

        class properties(InstanceRoute):
            template = Const('comp4.cms.templates/properties.wt')
            def hasProperties(s):
                return bool(RootProperty(
                    s.partMgr.partition,'property_factories.%s' % s.obj.toplevel
                )())
            hasProperties = Lazy(hasProperties)
            def asAction(self):
                lnk = Link.fromRoute(self, label=_(u'Properties'))
                if lnk.enabled and not self.hasProperties:
                    lnk.visible = False
                return lnk
            asAction = Lazy(asAction)
            pathLinks = Lazy(pathLinksWithAction)

        class directory(InstanceRoute):
            template = Const('comp4.cms.templates/directory.wt')
            pathLinks = Lazy(pathLinksWithAction)
            asAction = Lazy(lambda s:Link.fromRoute(s, label=_(u'Directory')))

        class permissions(InstanceRoute):
            template = Const('comp4.cms.templates/permissions.wt')
            permission = Param(lambda s:s.permPerm)
            def asAction(self):
                return Link.fromRoute(self, 
                    visible = self.partMgr.permissions != 'static' \
                            and bool(web.request().allows(
                                    getattr(self, 'permission', None))),
                    label   = _(u'Permissions'),
                )
            asAction = Lazy(asAction)
            pathLinks = Lazy(pathLinksWithAction)

        class delete(InstanceRoute):
            def asAction(self):
                return Link.fromRoute(self,
                  label   = _(u'Delete'),
                  confirm = _(u'Are you sure you want to delete this document?')
                )
            asAction = Lazy(asAction)

        class order(InstanceRoute):
            pass



class CmsSitemapWithAcl(CmsSitemap):

    viewPerm = Const(None)
    editPerm = Const(acl.ListPerm(web.GroupName('admin')))
    publishPerm = Const(acl.ListPerm(web.GroupName('admin')))
    permPerm = Const(acl.ListPerm(web.GroupName('admin')))

    def permission_for_obj(self, obj, role):
        if obj:
            a, h = obj._workspace['acl'].gethier(
                domain = 'cms',
                keys   = [p.inode for p in obj.iterParentsRev()],
                role   = role,
                root   = getattr(self, '%sPerm' % role)
            )
            return a
        else:
            return self.viewPerm
    permission_for_obj = Const(permission_for_obj)

    #'permission': Lazy(lambda s:s.editPerm),
    #'part.root.permission': Lazy(lambda s:s.viewPerm),
    #'part.root.add_root.permission': Lazy(lambda s:s.editPerm),
    #'part.multiop.permission': Lazy(lambda s:s.editPerm),
    #'part.popup.permission': Lazy(lambda s:s.editPerm),
    #'part.publishing_request.permission': Lazy(
    #    lambda s:s.editPerm | s.publishPerm),
    patch({
        'part.root.permission': Lazy(
            lambda s: s.permission_for_obj(s, s.obj, 'view')),
        'doc.permission': Lazy(
            lambda s: s.permission_for_obj(s, s.obj, 'edit')),
        'doc.view.permission': Lazy(
            lambda s: s.permission_for_obj(s, s.obj, 'view')),
        'doc.version.permission': Lazy(
            lambda s: s.permission_for_obj(s, s.obj, 'edit')),
        'doc.element.permission': Lazy(
            lambda s: s.permission_for_obj(s, s.obj, 'view')),
    })


class AttachedCmsSitemap(CmsSitemap):
    '''
        class Obj(Route): 
            match = ':oid/'
            oid   = Param(lambda s: s.obj._oid, typ=int)

            class cms(DynamicSitemap):
                match = 'cms/'
                prefix = Const('obj')
    '''

    prefix    = None

    partition = Param(lambda s:'%s.%s' % (s.parentRoute.prefix,s.oid), typ=str)
    _prefix   = Param(lambda s:s.partition.split('.')[0], equalTo='prefix')
    _oid      = Param(lambda s:s.partition.split('.')[-1], equalTo='../oid')

    root_deletion = Const(False)

class AttachedCmsSitemapWithAcl(CmsSitemapWithAcl):
    prefix    = None

    partition = Param(lambda s:'%s.%s' % (s.parentRoute.prefix,s.oid), typ=str)
    _prefix   = Param(lambda s:s.partition.split('.')[0], equalTo='prefix')
    _oid      = Param(lambda s:s.partition.split('.')[-1], equalTo='../oid')

    root_deletion = Const(False)

if __name__ == '__main__':
    for x in CmsSitemap.__traversal_table__:
        print x
