CREATE OR REPLACE FUNCTION cms_last_node_path(n INTEGER)
RETURNS SETOF repository_node LANGUAGE plpgsql AS $$
DECLARE
    r repository_node%ROWTYPE;
    p repository_node%ROWTYPE;
BEGIN
    SELECT INTO r rn.* FROM repository_node rn WHERE rn.inode = n;
    IF NOT FOUND THEN
        RETURN;
    END IF;
    IF r.parent_node NOTNULL THEN
        FOR p IN SELECT * FROM cms_last_node_path(r.parent_node) LOOP
            RETURN NEXT p;
        END LOOP;
    END IF;
    RETURN NEXT r;
END;
$$;






CREATE OR REPLACE FUNCTION cms_public_node_path(n INTEGER)
RETURNS SETOF repository_node LANGUAGE plpgsql AS $$
DECLARE
    r repository_node%ROWTYPE;
    p repository_node%ROWTYPE;
BEGIN
    SELECT INTO r rn.*
      FROM repository_node rn
           INNER JOIN cms_public_version pv ON (
                rn.inode = pv.inode AND
                'today'::DATE BETWEEN pv.pstart AND pv.pend
           )
     WHERE rn.inode = n;
    IF NOT FOUND THEN
        RETURN;
    END IF;
    IF r.parent_node NOTNULL THEN
        FOR p IN SELECT * FROM cms_public_node_path(r.parent_node) LOOP
            RETURN NEXT p;
        END LOOP;
    END IF;
    RETURN NEXT r;
END;
$$;



CREATE TYPE cms_path AS (
    inode       INTEGER,
    level       INTEGER,
    path        VARCHAR
);


CREATE OR REPLACE FUNCTION _cms_last_node_tree(
    root_id INTEGER, depth INTEGER, lvl INTEGER, pth VARCHAR
) RETURNS SETOF cms_path LANGUAGE plpgsql AS $$
DECLARE
    res cms_path%ROWTYPE;
    nod repository_node%ROWTYPE;
    cpth varchar;
    clvl integer;
BEGIN
    clvl := lvl + 1;
    FOR nod IN 
        SELECT *
          FROM repository_node
         WHERE parent_node = root_id AND changeset_till = 2147483647
    LOOP
        cpth      := pth || '/' || nod.inode;
        res.inode := nod.inode;
        res.level := clvl;
        res.path  := cpth;
        RETURN NEXT res;

        FOR res IN
            SELECT * FROM _cms_last_node_tree(nod.inode, depth-1, clvl, cpth)
        LOOP
            RETURN NEXT res;
        END LOOP;
    END LOOP;
END
$$;

    

CREATE OR REPLACE FUNCTION cms_last_node_tree(root_id INTEGER, depth INTEGER)
RETURNS SETOF cms_path LANGUAGE plpgsql AS $$
DECLARE
    res cms_path%ROWTYPE;
    nod repository_node%ROWTYPE;
    pth varchar;
    lvl integer;
    stack integer[];
BEGIN
    lvl := 0;

    FOR nod IN
        SELECT *
          FROM cms_last_node_path(root_id)
    LOOP
        IF lvl = 0 THEN 
            pth := nod.inode::text;
        ELSE
            pth := pth || '/' || nod.inode::text;
        END IF;
        lvl := lvl + 1;
    END LOOP;

    FOR res IN
        SELECT * FROM _cms_last_node_tree(root_id, depth, lvl, pth)
    LOOP
        RETURN NEXT res;
    END LOOP;
    RETURN;


    res.inode := root_id;
    res.level := lvl;
    res.path  := pth;

    RETURN NEXT res;

    lvl := lvl + 1;

    FOR nod IN 
        SELECT *
          FROM repository_node
         WHERE parent_node = root_id AND changeset_till = 2147483647
    LOOP
        res.inode := nod.inode;
        res.level := lvl;
        res.path  := pth || '/' || nod.inode;
        RETURN NEXT res;
        
    END LOOP;

    RETURN;
END;
$$;


-- SELECT * FROM cms_public_node_path(1876);
-- SELECT * FROM cms_last_node_tree(775);
-- SELECT * FROM cms_last_node_path(1876);
-- SELECT * FROM cms_last_node_tree(772, 3);
