CREATE OR REPLACE FUNCTION cms_directory_set_abs() RETURNS TRIGGER AS
'
    rec = TD["new"]
    
    if not rec["path"] :
        plpy.error("path is NULL")
    if rec["acl"] is None :
        plpy.error("acl is NULL")
    if rec["cmsid"] == rec["parent"] :
        plpy.error("parent equals to cmsid")
        
    chg = None

    if TD["event"] == "INSERT" :
        chg = 1

    elif TD["event"] == "UPDATE" :
        old = TD["old"]
        if old["path"] != rec["path"] :
            chg = 1
        if old["parent"] != rec["parent"] :
            chg = 1
        if old["acl"] != rec["acl"] :
            chg = 1

    if not chg :
        return "OK"

    if rec["parent"] :
        q = """SELECT abs_path, join(abs_acl,''%s'') AS abs_acl
                 FROM cms_directory
                WHERE cmsid=%d""" % (rec["acl"], rec["parent"])
        pres = plpy.execute(q, 1)
        if len(pres) != 1 :
            plpy.error("parent with cmsid %s not found" % rec["parent"])
        parent = pres[0]
        abs_path = "%s.%s" % (parent["abs_path"], rec["path"])
        abs_acl = parent["abs_acl"]

        if TD["event"]=="UPDATE" :
            if parent["abs_path"].startswith(old["abs_path"]) :
                plpy.error("cannot move node inside its subtree")
    else :
        parent = None
        abs_path = rec["path"]
        abs_acl = rec["acl"]

    rec["abs_path"] = abs_path
    rec["abs_acl"] = abs_acl

    return "MODIFY"
' LANGUAGE plpythonu;
    





CREATE OR REPLACE FUNCTION cms_directory_update_subtree() RETURNS TRIGGER AS
'
    old, new = TD["old"], TD["new"]
    ret = "OK"

    if old["path"] != new["path"] or old["parent"] != new["parent"] :
        l = old["abs_path"].count(".") + 1
        q = ("UPDATE cms_directory "
             "SET abs_path = ''%s''::ltree || subpath(abs_path,%d) "
             "WHERE abs_path <@ ''%s''" ) % (
                new["abs_path"], l, old["abs_path"]
            )
        plpy.execute(q)
        ret = "MODIFY"

    if old["acl"] != new["acl"] or old["parent"] != new["parent"] :
        q = """SELECT MAX(NLEVEL(abs_path)) AS lev_max
                 FROM cms_directory
                WHERE abs_path <@ ''%s''""" % new["abs_path"]
        lev_max = plpy.execute(q)[0]["lev_max"]

        lev = new["abs_path"].count(".") + 2
        
        q = """
          UPDATE cms_directory
             SET abs_acl = v_abs_acl
            FROM (
                SELECT cp.cmsid, cc.cmsid, join(cp.abs_acl, cc.acl)
                  FROM cms_directory cc
                  LEFT JOIN cms_directory cp ON (cc.parent = cp.cmsid)
            ) AS cms_acl_view (v_parent,v_cmsid, v_abs_acl)
           WHERE cmsid = v_cmsid AND abs_path <@ ''%s''
             AND nlevel(abs_path) = %%d;""" % new["abs_path"]
        for l in xrange(lev, lev_max + 1) :
            res = plpy.execute(q % l)
        ret = "MODIFY"
            
        
    return ret 
' LANGUAGE plpythonu;




CREATE OR REPLACE FUNCTION cms_notify_version(integer, timestamptz, text)
RETURNS timestamptz LANGUAGE plpythonu AS '
    from adt.validity import datetime

    cmsid, version, status = args

    q = """SELECT * FROM cms_version WHERE cmsid=%d
           ORDER BY version DESC LIMIT 1""" % cmsid
    last = plpy.execute(q, 1)

    if len(last) < 1 :
        q = """INSERT INTO cms_version (cmsid, version, status)
               VALUES (%d, ''%s'', ''%s'')""" % (cmsid, version, status)
        plpy.execute(q)
        return version
    
    last = last[0]
    last_version = datetime(last["version"])
    version = datetime(version)

    if last["status"] == status :
        if datetime(version) >= datetime(last["version"]) :
            q = "UPDATE cms_version SET version=''%s'' WHERE vid=%d" % (
                    version, last["vid"]
            )
            plpy.execute(q)
            return version
        else :
            return last_version

    if last_version == version :
        if last["status"] != status :
            q = "UPDATE cms_version SET status=''%s'' WHERE vid=%d" % (
                    status, last["vid"]
            )
            plpy.execute(q)
        return version

    q = """INSERT INTO cms_version (cmsid, version, status)
           VALUES (%d, ''%s'', ''%s'')""" % (cmsid, version, status)
    plpy.execute(q)
    return version
';



CREATE OR REPLACE FUNCTION cms_notify_by_cmsid()
RETURNS TRIGGER LANGUAGE plpythonu AS '
    from adt.validity import validity_datetime, DATETIME_MAX
    if TD["event"] in ("INSERT", "UPDATE") :
        rec = TD["new"]
    elif TD["event"] == "DELETE" :
        rec = TD["old"]
    cmsid = rec["cmsid"]
    if "validity" in rec :
        val = validity_datetime(rec["validity"])
        if TD["event"] == "DELETE" :
            version = val._till
        else :
            if val._till != DATETIME_MAX :
                return "OK"
            version = val._from
    else :
        version = None

    if version is None :
        q = "SELECT cms_notify_version(%d,NOW(),''modified'')" % cmsid
    else :
        q = "SELECT cms_notify_version(%d,TIMESTAMPTZ(''%s''),''modified'')" % (
                cmsid, str(version)
        )
    plpy.execute(q)
    return "OK"
';




CREATE OR REPLACE FUNCTION cms_notify_pagelet_change ()
RETURNS TRIGGER LANGUAGE plpythonu AS
'
    from adt.validity import validity_datetime, DATETIME_MAX

    relid = TD["relid"]
    
    table = plpy.execute(
        "SELECT relname FROM pg_class WHERE oid=%s" % relid,1
    )[0]["relname"]

    if TD["event"] == "INSERT" :
        plid = TD["new"]["plid"]
        r = plpy.execute("SELECT * FROM cms_pagelet WHERE plid=%s" % plid, 1)
        if len(r) < 1 :
            plpy.error("cms_pagelet[%d] does not exists" % plid)
        return "OK"

    old = TD["old"]
    plid = old["plid"]

    if TD["event"] == "UPDATE" :
        new = TD["new"]
        if old["item"] != new["item"] :
            plpy.error("%s.item %s is immutable" % (table, old["item"]))
        if plid != new["plid"] :
            plpy.error("%s.plid %s is immutable" % (table, plid))
        if new.has_key("validity") :
            val = validity_datetime(new["validity"])
            if val._till != DATETIME_MAX :
                plpy.execute(
                    """DELETE FROM cms_pagelet WHERE
                       plid=%d AND validity=NOW()""" % plid
                )
            else :
                plpy.execute(
                    """UPDATE cms_pagelet SET log=''update %s.%s''
                       WHERE plid=%d AND validity=NOW()""" % (
                       table, old["item"], plid)
                )
        else :
            plpy.execute(
                """UPDATE cms_pagelet SET log=''update %s.%s''
                    WHERE plid=%d AND validity=NOW()""" % (
                    table, old["item"], plid)
            )
        return "OK"

    elif TD["event"] == "DELETE" :

        plpy.execute(
            "DELETE FROM cms_pagelet WHERE plid=%d AND validity=NOW()" % plid
        )
        return "SKIP"

';







CREATE OR REPLACE FUNCTION cms_publishing_trigger() RETURNS TRIGGER AS
'
    from adt.api import validity_date, validity_datetime, datetime, DATETIME_MAX

    new = TD["new"]

    status = new["ack"]
    if status is None :
        status = "request"
    elif status :
        status = "published"
    else :
        status = "rejected"

    vid = None

    if TD["event"] == "INSERT" :
        val = validity_datetime(new["validity"])
        if new["ack"] :
            vid = new["vid"]
        if val._till < DATETIME_MAX :
            #plpy.notice("INSERTING OLD")
            return "OK"
        
    elif TD["event"] == "UPDATE" :
        old = TD["old"]
        if old["vid"] != new["vid"] :
            plpy.error("cms_publishing_request.reqid is immutable")

        if old["ack"] != new["ack"] :
            vid = new["vid"]
        if old["publish"] != new["publish"] :
            vid = new["vid"]

    #plpy.notice("ACK %r STAT %r" % (new["ack"], status))
    plpy.execute(
      "UPDATE cms_version SET status=''%s'' WHERE vid=%d" % (status, new["vid"])
    )

    if vid is None :
        return "OK"
    
    verec = plpy.execute("SELECT * FROM cms_version WHERE vid=%d" % vid, 1)[0]
    cmsid = verec["cmsid"]
    q = """SELECT v.version, r.publish
             FROM cms_version v, cms_publishing_request r
            WHERE v.vid = r.vid AND v.cmsid=%d AND r.ack=''t''
              AND r.validity=''infinity''::timestamptz""" % cmsid
    publish = [
        (datetime(r["version"]), validity_date(r["publish"]))
        for r in plpy.execute(q)
    ]
    plpy.execute("DELETE FROM cms_published WHERE cmsid=%d" % cmsid)
    for v,p in validity_date.normalizeWithPriority(publish) :
        #plpy.notice("PUBL: %d %r" % (cmsid, repr((v,p))))
        plpy.execute("""INSERT INTO cms_published (cmsid, version, published)
                         VALUES (%d, ''%s'', ''%s'')""" % (
                         cmsid, str(v), str(p))
        )
    return "OK" 
' LANGUAGE plpythonu;



