# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from datetime import datetime
from lib4.timeutils import *
from wm4.schema.tests.utils import MockWorkspaceCase
from comp4.cms.api import *
from comp4.cms.pagelets import *
from comp4.cms import repository

MAX_CHSET = repository.MAX_CHSET

class MyTimeService(TimeService):
    
    def __init__(self):
        self.stamp = datetime(2006,1,1,10,15,0, tzinfo=TZUTC)

    def now(self):
        s = self.stamp
        self.stamp = s + DAY
        return s


class RepositoryCase(MockWorkspaceCase):
    
    cfgURLs = ('pkgfile:wm4/core.ini', 'pkgfile:comp4.cms/comp.ini')

    def setUp(self):
        MockWorkspaceCase.setUp(self)
        settings.time_service(MyTimeService)


class TestNodes(RepositoryCase):
    
    def test10_PublicNode(self):
        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT changeset FROM cms_public_version "
            "WHERE (inode = 13) AND (pstart <= '2007-05-22') "
            "AND ('2007-05-22' <= pend)"),
            res = [[444]],
        )
        self.expects(sql=(
            "SELECT partition, inode, changeset_from, changeset_till, "
            "klass, parent_node FROM repository_node "
            "WHERE (inode = 13) "
            "AND (changeset_from <= 444) AND (444 <= changeset_till)"),
            res=[['CMS', 13, 8, MAX_CHSET, 'cms', None]],
        )
        self.expects(transaction='commit')

        storage.begin()

        doc = self.ws.get('cms/PublicNode', 13)
        self.failUnless(isinstance(doc, Node))
        self.assertEqual(doc._oid, (13, 444))
    
        doc2 = self.ws.get('cms/PublicNode', 13)
        assert doc is doc2

        storage.commit()


def suite():
    return TestSuite((
        makeSuite(TestNodes),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
