# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from datetime import datetime
from lib4.timeutils import *
from wm4.schema.tests.utils import MockWorkspaceCase
from comp4.cms.api import *

class MyTimeService(TimeService):

    def __init__(self):
        self.stamp = datetime(2006,1,1,10,15,0, tzinfo=TZUTC)

    def now(self):
        s = self.stamp
        self.stamp = s + DAY
        return s


class RepositoryCase(MockWorkspaceCase):

    cfgURLs = ('pkgfile:wm4/core.ini', 'pkgfile:comp4.cms/comp.ini')

    def setUp(self):
        MockWorkspaceCase.setUp(self)
        settings.time_service(MyTimeService)
        self.mgr = manager()

    def expectsTreeQuery(self, res=[]):
        self.expects(
            sql='''SELECT DISTINCT ON (rn.inode)
                         rn.inode, rn.parent_node, rn.partition,
                         rn.changeset_from, rn.changeset_till,
                         cpv.changeset,
                         dp.value,
                         ti.value, ti.eid
                    FROM repository_node rn
               LEFT JOIN cms_public_version cpv
                      ON cpv.inode = rn.inode
                     AND CURRENT_DATE BETWEEN cpv.pstart AND cpv.pend
               LEFT JOIN repository_element dp
                      ON dp.inode = rn.inode
                     AND cpv.changeset BETWEEN dp.changeset_from
                                           AND dp.changeset_till
                     AND dp.name = 'dir.positions'
               LEFT JOIN repository_element ti
                      ON ti.inode = rn.inode
                     AND cpv.changeset BETWEEN ti.changeset_from
                                           AND ti.changeset_till
                     AND ti.name = 'head.title'
                ORDER BY rn.inode, rn.changeset_from DESC''',
            res=res
        )


class TestPagelets(RepositoryCase):

    def test10_NodeElementsTree(self):
        self.expects(transaction='begin')
        self.expectsTreeQuery(res=[
            (13, None, 'doc', 600, MAX_CHSET, 637, None, 'D13', 4001),
        ])
        self.expects(sql=(
            "SELECT inode, eid, changeset_from, changeset_till, kind, "
            "name, value, parent_id, position FROM repository_element "
            "WHERE (inode = 13) AND (changeset_from <= 637) "
            "AND (637 <= changeset_till) ORDER BY position"),
            res=[
                [13,100,8,MAX_CHSET,'navig','menu.position','100',None,1],
                [13,101,8,MAX_CHSET,'cms','toplevel.document',None,None,2],
                [13,102,8,MAX_CHSET,'cms','head.title','Foo',101,1],
                [13,103,8,MAX_CHSET,'cms','pagelet.rest','para',101,2],
            ],
        )
        self.expects(transaction='commit')

        storage.begin()

        doc = self.mgr.getNode(PublicNode, 13)
        self.failUnless(isinstance(doc, PublicNode))
        self.assertEqual(doc._oid, 13)

        self.assertEqual(doc.document, doc.elements.children[1])
        self.failUnless(isinstance(doc.document, Toplevel))
        self.failUnless(isinstance(doc.document.children[0], Title))
        self.failUnless(isinstance(doc.document.children[1], Rest))
        self.failUnless(u'<p>para</p>' in doc.document.children[1].html)
        self.assertEqual(doc.document, doc.elements.children[1])
        self.failUnless(isinstance(doc.elements.children[0], Element))
        storage.commit()


def suite():
    return TestSuite((
        makeSuite(TestPagelets),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
