# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from datetime import datetime, date
from lib4.timeutils import *
from wm4.web.routes import *
from wm4.schema.tests.utils import MockWorkspaceCase
from comp4.cms.api import *


class Obj(schema.Entity):
    pkey = schema.Attribute(int)
    binding.metadata(schema.PrimaryKey('pkey'))

class R(Route):

    class static(CmsSitemap):
        match = 'static/'
        #prefix = Const('static')
        partition = Const('static')

    class obj(Route):
        match  = 'obj/:oid/'
        oid    = Param(lambda s: s.obj._oid, typ=int)
        obj    = Param(lambda s: settings.workspace().get(Obj,s.oid), typ=Obj)
        signature = Signature('obj')

        class cms(AttachedCmsSitemap):
            match = 'docs/'
            prefix = Const('obj_cms')



class RepositoryCase(MockWorkspaceCase):

    cfgURLs = ('pkgfile:wm4/core.ini', 'pkgfile:comp4.cms/comp.ini')

    def setUp(self):
        MockWorkspaceCase.setUp(self)

    def expectsTreeQuery(self, res=[]):
        self.expects(
            sql='''SELECT DISTINCT ON (rn.inode)
                         rn.inode, rn.parent_node, rn.partition,
                         rn.changeset_from, rn.changeset_till,
                         cpv.changeset,
                         dp.value,
                         ti.value, ti.eid
                    FROM repository_node rn
               LEFT JOIN cms_public_version cpv
                      ON cpv.inode = rn.inode
                     AND CURRENT_DATE BETWEEN cpv.pstart AND cpv.pend
               LEFT JOIN repository_element dp
                      ON dp.inode = rn.inode
                     AND cpv.changeset BETWEEN dp.changeset_from
                                           AND dp.changeset_till
                     AND dp.name = 'dir.positions'
               LEFT JOIN repository_element ti
                      ON ti.inode = rn.inode
                     AND cpv.changeset BETWEEN ti.changeset_from
                                           AND ti.changeset_till
                     AND ti.name = 'head.title'
                ORDER BY rn.inode, rn.changeset_from DESC''',
            res=res
        )


class TestSitemap(RepositoryCase):

    def test10_TraverseToStaticNodeView(self):
        self.expects(transaction='begin')
        self.expectsTreeQuery(res=[
            (42, None, 'static', 600, MAX_CHSET, 637, None, 'D42', 4001),
        ])
        self.expects(transaction='commit')
        storage.begin()
        r = R.traverse('static/42/')
        self.assertEqual(r.pathInfo, '/static/42/')
        self.assertEqual(r.action, 'view')
        self.assertEqual(r.partition, 'static')
        self.assertEqual(r.partMgr, manager()['static'])
        storage.commit()

    def test12_RouteForStaticNode(self):
        self.expects(transaction='begin')
        self.expectsTreeQuery(res=[
            (42, None, 'static', 600, MAX_CHSET, 637, None, 'D42', 4001),
        ])
        self.expects(transaction='commit')
        storage.begin()
        n = self.ws.get('cms/PublicNode', 42)
        r = R.route_for(n)
        self.assertEqual(r.pathInfo, '/static/42/')
        self.assertEqual(r.action, 'view')
        self.assertEqual(r.partition, 'static')
        self.assertEqual(r.partMgr, manager()['static'])
        storage.commit()

    def test50_TraverseToAttachedNodeView(self):
        self.expects(transaction='begin')
        self.expectsTreeQuery(res=[
            (42, None, 'obj_cms.12', 600, MAX_CHSET, 637, None, 'D42', 4001),
        ])
        self.expects(sql="SELECT pkey FROM obj WHERE pkey = 12", res=[[12]])
        self.expects(transaction='commit')
        storage.begin()
        r = R.traverse('obj/12/docs/42/')
        self.assertEqual(r.pathInfo, '/obj/12/docs/42/')
        self.assertEqual(r.action, 'view')
        self.assertEqual(r.partition, 'obj_cms.12')
        self.assertEqual(r.partMgr, manager()['obj_cms.12'])
        storage.commit()

    def test52_RouteForAttachedNode(self):
        self.expects(transaction='begin')
        self.expectsTreeQuery(res=[
            (42, None, 'obj_cms.12', 600, MAX_CHSET, 637, None, 'D42', 4001),
        ])
        self.expects(sql="SELECT pkey FROM obj WHERE pkey = 12", res=[[12]])
        self.expects(transaction='commit')
        storage.begin()
        n = self.ws.get('cms/PublicNode', 42)
        r = R.route_for(n)
        self.assertEqual(r.pathInfo, '/obj/12/docs/42/')
        self.assertEqual(r.action, 'view')
        self.assertEqual(r.partition, 'obj_cms.12')
        self.assertEqual(r.partMgr, manager()['obj_cms.12'])
        storage.commit()


def suite():
    return TestSuite((
        makeSuite(TestSitemap),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
