# -*- coding: utf-8 -*-

from wm4.api import *
from comp4.files.web import *
from datetime import date
from wm4.web.sessions import SecureValue
from comp4.antispam.web import AntispamExpr

from entities import *
from facades import LastNode, NodeVersion
from controllers import NodeForm
from config import RootProperty, MAX_CHSET, PUBLISH_END, changeset_manager
from config import getRootManager

__all__ = ['CMSHandler']

_ = i18n.MessageIDFactory('comp4.cms')


class CMSHandler(web.EntityHandler):

    oidParam = 'inode'

    entity = binding.Constant(LastNode)

    formCtrl = binding.Constant(NodeForm)

    def do_root(self, req, obj, public, partMgr):
        if obj is None or public is None:
            tmplPath = RootProperty(partMgr.partition, 'empty_template')()
            tmpl = req.getResource(tmplPath)
            return tmpl.handleHTTP(req)
        form = self.formCtrl(
          ctrlReq=req, oldValue=public, partMgr=partMgr, parentNode=None,
          ctrlReadOnly=True,
        )
        form()
        tmpl = req.route.get_template(req.route, public, req, True)
        return tmpl.handleHTTP(req, obj=public, form=form, mode='view')
    
    def do_view(self, req, obj, public, partMgr, inodepath):
        if obj is None:
            raise web.NotFound()
        if public is None:
            raise web.NotFound()
        if inodepath <> obj.inodepath:
            url = req.get_url(self.obj)
            raise web.SeeOther(url)
        p = public
        while p is not None:
            try:
                p = p.parent
            except KeyError:
                raise web.NotFound()
        # form controller
        form = self.formCtrl(
            ctrlReq=req,
            oldValue=public,
            ctrlReadOnly=True,
            partMgr=partMgr,
            parentNode=public.parent
        )
        form()
        # template for given toplevel document
        if obj.parent:
            tmpl = req.route.get_template(req.route, obj, req, False)
        else:
            tmpl = req.route.get_template(req.route, obj, req, True)
        return tmpl.handleHTTP(req, obj=public, form=form, mode='view')
    
    def do_version(self, req, obj, version, partMgr, changeset):
        last = obj
        form = self.formCtrl(
            ctrlReq=req,
            oldValue=version,
            ctrlReadOnly=True,
            partMgr=partMgr,
            parentNode=last.parent
        )
        form()
        tmplPath = RootProperty(
            partMgr.partition, 'view_template.%s' % version.document.name)()
        tmpl = req.getResource(tmplPath)
        return tmpl.handleHTTP(
            req, **self.viewNS(obj=version, form=form, mode='view')
        )

    def do_element(self, req, inode, eid, elem, elemAction):
        if elem is None:
            raise web.NotFound('element not found', inode, eid)
        handler = getattr(elem, 'do_'+ elemAction, None)
        if handler is None:
            raise web.NotFound('method not found', elem, elemAction)
        return handler(req)

    def do_directory(self, req, obj, template):
        tmpl = req.getResource(template)
        return tmpl.handleHTTP(req,obj=obj,today=date.today(), pend=PUBLISH_END)

    def do_add_redir(self, req, toplevel):
        assert toplevel.count('.') < 5
        url = req.get_url(self.obj, 'add', toplevel)
        raise web.SeeOther(url)

    def addForm(self):
        route = self.req.route
        return self.formCtrl(
            ctrlReq=self.req, parentNode=route.obj, partMgr=route.partMgr
        )
    
    def do_add_root(self):
        return self.do_add()

    def editForm(self):
        obj = self.req.route.obj
        return self.formCtrl(
           ctrlReq=self.req, oldValue=obj, partMgr=obj.partMgr,
           parentNode=obj.parent
        )

    def editResult(self, obj):
        # XXX tohle neni pekne
        chset = int((-obj._workspace.connection('''
            SELECT MAX(changeset)
              FROM repository_node_versions
             WHERE inode = %d''' % obj._oid))[0])
        raise web.SeeOther(self.req.get_url(obj, 'version', chset))
    addResult = editResult
    
    def do_delete(self, obj, root_deletion):
        if obj.parent_node is None and not root_deletion:
            raise web.NotAllowed()
        try:
            self.workspace.remove(obj)
            storage.commit()
            storage.begin()
        except:
            raise
        else:
            return self.deleteResult(obj)

    def deleteResult(self, obj):
        if obj.parent is not None:
            raise web.SeeOther(self.req.get_url(obj,'versions'))
        else:
            raise NotImplementedError
    
    def do_properties(self, req, obj, template):
        tmpl = req.getResource(template)
        return tmpl.handleHTTP(req, obj=obj)

    def do_versions(self, req, obj, partMgr, template, publishPerm, editPerm):
        tmpl = req.getResource(template)

        min_date = date(1900,1,1)
        if req.getValue('order') == 'date':
            orderBy = 'query.Desc(query.Coalesce(pv.pstart, min_date)),'\
                      'query.Desc(ch.pkey)'
        else:
            orderBy = 'query.Desc(ch.pkey)'
        if hasattr(req.route, 'permission_for_obj'):
            canEdit = req.allows(
                req.route.permission_for_obj(req.route, obj, 'edit'))
            canPublish = req.allows(
                req.route.permission_for_obj(req.route, obj, 'publish'))
        else:
            canPublish = req.allows(publishPerm)
            canEdit = req.allows(editPerm)

        publishingRequest = -self.workspace('''[
            pw for pw in CmsPublishingWorkflow if pw.inode == obj.inode
        ]''')

        res = self.workspace('''[
            (nv.inode, nv.changeset, ch.created_by, ch.creation_time,
             pv.pkey, pv.pstart, pv.pend)
                for nv in RepositoryNodeVersions
                for ch in RepositoryChangeset
                for pv in CmsPublicVersion
                 if query.LeftJoin(nv.changeset == ch.pkey)
                and query.LeftJoin(
                    nv.inode == pv.inode and nv.changeset == pv.changeset)
                and nv.inode == obj.inode
                and query.OrderBy(%s)]''' % orderBy)
        fields = [d[0] for d in res.cursor.description]
        class Version:
            def __init__(self, row):
                self.__dict__.update(dict((k,v) for k,v in zip(fields, row)))
        versions = []
        tday = date.today()
        for row in res:
            v = Version(row)
            v.currentlyPublic = (
                v.pstart is not None and v.pstart <= tday <= v.pend)
            versions.append(v)

        if req.getValue('publishing_request'):
            pstart = req.getValue('pstart', '').strip()
            if pstart:
                pstart = schema.normalize(pstart, date)
            else:
                pstart = tday
            pend = req.getValue('pend', '').strip()
            if pend:
                pend = schema.normalize(pend, date)
            else:
                pend = PUBLISH_END
            assert pstart <= pend
            self.workspace.new(CmsPublishingWorkflow,
                inode     = obj.inode,
                changeset = versions[0].changeset,
                req       = 'publish',
                req_chset = changeset_manager().newChangeset.pkey,
                pstart    = pstart,
                pend      = pend,
            )
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        elif req.getValue('result_ack'):
            if publishingRequest.req == 'delete':
                obj.partMgr.unpublishNode(
                    obj, publishingRequest.pstart, publishingRequest.pend
                )
            else:
                obj.partMgr.publishNode(
                    obj, publishingRequest.changeset, 
                    publishingRequest.pstart, publishingRequest.pend
                )
            self.workspace.remove(publishingRequest)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)
            
        elif req.getValue('result_rej'):
            self.workspace.remove(publishingRequest)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        return tmpl.handleHTTP(req, 
            obj=obj, versions=versions, today=tday,
            canPublish=canPublish, canEdit=canEdit,
            publishingRequest=publishingRequest,
            publishing=partMgr.publishing,
        )

    def do_publishing_requests(self):
        req = self.req
        ws = self.workspace
        
        if req.getValue('submit'):
            inodes = [int(i) for i in req.getList('inode')]
            for inode in inodes:
                pr = ws.get(CmsPublishingWorkflow, inode)
                obj = ws.get(LastNode, inode)
                if pr.req == 'delete':
                    obj.partMgr.unpublishNode(
                        obj, pr.pstart, pr.pend
                    )
                else:
                    obj.partMgr.publishNode(
                        obj, pr.changeset, 
                        pr.pstart, pr.pend
                    )
                self.workspace.remove(pr)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        requests = list(ws('''[
            w for w in CmsPublishingWorkflow
               if query.OrderBy(query.Desc(w.req))
        ]'''))
        return self.render(requests=requests)

    def do_multiop(self, req):
        action = None
        for n in ('__delete__', '__move__'):
            if n in req:
                action = n[2:-2]
                break
        nodes = []
        for i in range(min(int(req.getValue('node_max',0)),1000)):
            n = req.getValue('node%d' % i)
            if n is not None:
                nodes.append(int(n))
        params = dict(nodes=','.join(str(n) for n in nodes))
        if action == 'delete':
                params['result'] = req.environ.get('HTTP_REFERER','')
        url = (req.requestURL / ('./multiop/%s' % action)) % params
        raise web.SeeOther(url)

    def do_delete_many(self, req, nodes, result):
        for inode in nodes.split(','):
            inode = inode.strip()
            if not inode:
                continue
            inode = int(inode)
            node = self.workspace.get(LastNode, inode, default=None)
            if node is not None:
                self.workspace.remove(node)
        storage.commit()
        storage.begin()
        raise web.SeeOther(result)

    def do_move_many(self, req, nodes, partMgr, target, template):
        nodeIds = set([int(n) for n in nodes.split(',') if n])
        get = lambda inode: self.workspace.get(LastNode, inode)
        nodes = [get(n) for n in nodeIds]
        if target:
            target = int(target)
            targetObj = get(target)
            parentIds = set([p.inode for p in targetObj.iterParents()])
            for n in nodes:
                assert n.inode not in parentIds
                n.parent_node = targetObj.inode
            url = req.get_url(targetObj, 'directory')
            storage.commit()
            storage.begin()
            raise web.SeeOther(url)
        else:
            tmpl = req.getResource(template)
            root = partMgr.getRoot(public=False)
            return tmpl.handleHTTP(req, nodes=nodes, root=root, nodeIds=nodeIds)

    def do_order(self, req, obj, place):
        parent = obj.parent
        directory = list(parent.validChildren)
        oldPos = [c.inode for c in directory].index(obj.inode)
        del directory[oldPos]
        if place == 'start':
            newPos = 0
        elif place == 'end':
            newPos = len(directory)
        elif place == 'up':
            newPos = max(0, oldPos-1)
        elif place == 'down':
            newPos = min(len(directory), oldPos+1)
        else:
            newPos = int(place)
            if newPos >= oldPos:
                newPos = newPos - 1
        directory.insert(newPos, obj)
        self.workspace['cms'].updatePositions(parent, directory)
        url = req.absURL(obj.getParent(), 'directory')
        storage.commit()
        storage.begin()
        raise web.SeeOther(url)
        
    def do_undelete(self, req, obj, version):
        chset = self.workspace['cms'].undeleteNode(version)
        url = req.absURL(obj, 'version', chset)
        storage.commit()
        storage.begin()
        raise web.SeeOther(url)

    def do_popup_tree(self, req, template):
        tmpl = req.getResource(template)
        return tmpl.handleHTTP(req)#, nodes=nodes, root=root, nodeIds=nodeIds)


    def do_permissions(self, req, obj, partMgr, template):
        from comp4.acl import api
        from comp4.acl.web import renderPerm
        ws = self.workspace
        tmpl = req.getResource(template)

        aclKeys = [p.inode for p in obj.iterParentsRev()]
        if partMgr.publishing == 'manual':
            aclRoles = ['view', 'edit', 'publish']
        else:
            aclRoles = ['view', 'edit']
        aclAll = {}
        aclHere = {}
        for role in aclRoles:
            p,h = ws['acl'].gethier(
                domain = 'cms',
                keys   = aclKeys,
                role   = role,
                root   = req.params['%sPerm' % role]
            )
            aclAll[role] = p
            aclHere[role] = h

        userCls = ws['users/User']
        userLst = list(self.workspace(
            "[u for u in userCls if query.OrderBy(u.login)]"))
        groupCls = ws['users/Group']
        groupLst = list(self.workspace(
            "[g for g in groupCls if query.OrderBy(g.name)]"))

        if req.getValue('set_user'):
            role = str(req.getValue('role'))
            login = str(req.getValue('user_login'))
            if req.getValue('user_not') == "0":
                perm = web._Not(web.UserName(login))
            else:
                perm = web.UserName(login)
            lst = aclHere[role]
            lst.add_perm(perm)
            old = -ws('''[
                a for a in api.AccessList
                   if a.domain == 'cms'
                  and a.key == str(self.obj.inode)
                  and a.role == role]''')
            if old is None:
                ws.new(api.AccessList,
                    domain = 'cms', key=str(obj.inode), 
                    role = role, acl_expr = repr(lst)
                )
            else:
                old.acl_expr = repr(lst)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        elif req.getValue('set_group'):
            role = str(req.getValue('role'))
            group = str(req.getValue('group_name'))
            if req.getValue('group_not') == "0":
                perm = web._Not(web.MemberOf(group))
            else:
                perm = web.MemberOf(group)
            lst = aclHere[role]
            lst.add_perm(perm)
            old = -ws('''[
                a for a in api.AccessList
                   if a.domain == 'cms'
                  and a.key == str(self.obj.inode)
                  and a.role == role]''')
            if old is None:
                ws.new(api.AccessList,
                    domain = 'cms', key=str(obj.inode), 
                    role = role, acl_expr = repr(lst)
                )
            else:
                old.acl_expr = repr(lst)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        elif req.getValue('set_default'):
            role = str(req.getValue('role'))
            perm = web.PermissionChecker.globals[req.getValue('default')]()
            lst = aclHere[role]
            lst.add_perm(perm)
            old = -ws('''[
                a for a in api.AccessList
                   if a.domain == 'cms'
                  and a.key == str(self.obj.inode)
                  and a.role == role]''')
            if old is None:
                ws.new(api.AccessList,
                    domain = 'cms', key=str(obj.inode), 
                    role = role, acl_expr = repr(lst)
                )
            else:
                old.acl_expr = repr(lst)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        elif req.getValue('switch'):
            role = req.getValue('switch')
            acl = aclHere[role].copy()
            all = aclAll[role]
            kind = req.getValue('kind')
            name = req.getValue('name')
            if kind == 'user':
                acl.users[name] = not all.users[name]
            elif kind == 'group':
                acl.groups[name] = not all.groups[name]
            else:
                raise NotImplementedError
            old = -ws('''[
                a for a in api.AccessList
                   if a.domain == 'cms'
                  and a.key == str(self.obj.inode)
                  and a.role == role]''')
            if old is None:
                ws.new(api.AccessList,
                    domain = 'cms', key=str(obj.inode), 
                    role = role, acl_expr = repr(acl)
                )
            else:
                old.acl_expr = repr(acl)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        elif req.getValue('del'):
            role = req.getValue('del')
            acl = aclHere[role].copy()
            kind = req.getValue('kind')
            if kind == 'user':
                acl.users.pop(req.getValue('name'))
            elif kind == 'group':
                acl.groups.pop(req.getValue('name'))
            else:
                raise NotImplementedError
            old = -ws('''[
                a for a in api.AccessList
                   if a.domain == 'cms'
                  and a.key == str(self.obj.inode)
                  and a.role == role]''')
            if old is None:
                ws.new(api.AccessList,
                    domain = 'cms', key=str(obj.inode), 
                    role = role, acl_expr = repr(acl)
                )
            else:
                old.acl_expr = repr(acl)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        return tmpl.handleHTTP(req, 
            obj        = obj,
            partMgr    = partMgr,
            aclHere    = aclHere,
            aclAll     = aclAll,
            renderPerm = renderPerm,
            userLst    = userLst,
            groupLst   = groupLst,
        )

    def do_mailform(self, req, template):
        from comp4.sendmail.api import sendmail, Plain
        XH = 'http://www.w3.org/1999/xhtml'
        NS = {'h': XH}
        data = {}
        for s in ('mailform_id', 'mailform_to', 'mailform_part'):
            data[s] = SecureValue.unpack(
                req.getValue(s), secret = settings.secret()
            ).value
        if 'mailform_expr' in req.fields:
            data['mailform_expr'] = AntispamExpr.unpack(
                    req.getValue('mailform_expr'), secret = settings.secret())

        as_value = req.getList('as_val')
        if as_value:
            as_value = as_value[0]
            me = data.get('mailform_expr', None)
            if (not me) or (as_value.strip() != str(eval(me.value))):
                return self.render(message=u'Nebyl správně vyplněn kontrolní kód.',
                        ok=False)

        part = getRootManager(data['mailform_part'])
        form = part.mailforms[data['mailform_id']]
        msg = []
        radio_seen = set()
        check_seen = set()
        for i in form.xpath(
            ".//h:input | .//h:textarea | .//h:select", 
            namespaces=NS
        ):
            name = i.get('name')
            tag = i.tag
            typ = i.get('type','text')
            if tag == '{%s}input' % XH:
                if typ in ('text','checkbox'):
                    if name not in check_seen:
                        msg.append((name, '\n'.join(req.getList(name))))
                        check_seen.add(name)
                elif typ == 'radio':
                    if name not in radio_seen:
                        msg.append((name, req.getValue(name, default='')))
                        radio_seen.add(name)
            elif tag == '{%s}select' % XH:
                msg.append((name, req.getValue(name, default='')))
            elif tag == '{%s}textarea' % XH:
                msg.append((name, req.getValue(name, default='')))
        body = []
        body.append(u'URL formuláře: %s' % req.environ.get('HTTP_REFERER', 'N/A'))
        body.append('')
        for a, v in msg:
            v = v.strip().splitlines()
            if not v:
                body.append('%s:' % a)
            elif len(v) == 1:
                body.append('%s: %s' % (a, v[0]))
            elif len(v) > 1:
                body.append('%s:' % a)
                for l in v:
                    body.append('  ' + l.rstrip())
        addrs = [a.strip() for a in data['mailform_to'].split(';')]
        sendmail(
            'cms_mailform',
            sender = addrs[0],
            recipients = addrs,
            subject = data['mailform_id'],
            message = Plain(data='\n'.join(body).encode('utf-8') + '\n')
        )
        return self.render(ok=True, message=u'We have received your message and will be in touch if required.')

