# -*- coding: utf-8 -*-

import os.path
from ftplib import FTP
import time

from wm4.api import *
from xml4 import xhtml as xh
from wm4.web.forms.api import *
from wm4.web.sessions import SecureValue
from adt4.byte_count import ByteCount
from lib4.imports import importObject
from comp4.antispam.web import generate_expression, expression_to_text, AntispamExpr

from config import *

__all__ = [
    'ToplevelWidget', 'PortalWidget', 'PageletsRepeatWidget',
    'PageletWidget', 'getTranslation', 'TranslatablePageletWidget',
    'TitleWidget', 'HeadingWidget', 'AbstractWidget',
    'RestWidget', 'HtmlWidget', 'HtmlSrcWidget',
    'ContentsWidget', 'ContentsByTagWidget',
    'SiblingsWidget',
    'ImageWidget', 'IllustrationWidget', 'PhotogalleryImageWidget',
    'PhotogalleryWidget', 'LightboxPhotogalleryWidget', 'FileWidget',
    'DiscussionWidget', 'PollWidget', 'LinkBoxLinkWidget', 'LinkBoxWidget',
    'MailFormWidget', 'VideoFtpWidget',
]

_ = i18n.MessageIDFactory('comp4.cms')


class ToplevelWidget(FormWidget):

    def view(self, state, felem, req):
        return web.parseSrc('''
            <div class="$form.css">
                <div form:input="title" />
                <div form:repeat="pagelets" py:strip="">
                    <span form:input=""/>
                </div>
            </div>
        ''')

    def edit(self, state, felem, req):
        return web.parseSrc('''
        <?py _ = i18n.MessageIDFactory('comp4.cms')?>
        <span py:strip="">
            <fieldset form:fieldset="" class="${form['title'].css}">
                <title>${form['title'].ctrlLabel}</title>
                <div form:input="title" />
            </fieldset>

            <div form:repeat="pagelets" py:strip="">
                <fieldset form:fieldset="" class="${form.css}">
                    <title>$form.ctrlLabel</title>
                    <actions>
                        <input form:button="moveUp" class="button-move-up"/>
                        <input form:button="moveDown" class="button-move-down"/>
                        <input form:button="removeItem" class="button-remove-item"/>
                    </actions>
                    <input form:input=""/>
                </fieldset>
            </div>

            <fieldset form:fieldset="">
                <title>${_(u'Add new page element')}</title>
                <input form:input="add_pagelet"/>
                <input form:button="pagelets/addItem"/>
            </fieldset>
        </span>''')




class PortalWidget(ToplevelWidget):

    def view(self, state, felem, req):
        from lib4.iterutils import sequenceToTableVertically
        from xml4.serialization import XHTMLSerializer
        pagelets = []
        for pelem in felem['pagelets'].ctrlItems:
            scp = state.copy()
            scp['form'] = pelem
            widg = importObject(pelem.ctrlWidget())(
                'form:input', '', self.gdict
            )
            ser = XHTMLSerializer()
            html = ser(widg(scp))
            if not html:
                continue
            html = html.strip()
            if not html:
                continue
            pagelets.append(html)
        h = (len(pagelets)+1) / 2
        return xh.div(
            xh.h1(felem.oldValue.title.value), '\n',
            xh.table(
                xh.colgroup(xh.col(width="49%"), xh.col(width="49%")), '\n',
                xh.tr(
                    '\n',
                    xh.td(xh.Literal('\n'.join(pagelets[:h]))), '\n',
                    xh.td(xh.Literal('\n'.join(pagelets[h:]))), '\n',
                ),
                klass='comp4-portal',
            ),
        )


class PageletsRepeatWidget(FormWidget):

    def view(self, state, felem, req):
        return web.parseSrc('''
            <div form:repeat="" py:strip="">
                <span form:input=""/>
            </div>
        ''')

    def edit(self, state, felem, req):
        return web.parseSrc('''<span>
            <?py _ = i18n.MessageIDFactory('comp4.cms')?>
            <div form:repeat="" py:strip="">
                <fieldset form:fieldset="" class="${form.css}">
                    <title>$form.ctrlLabel</title>
                    <actions>
                        <input form:button="moveUp" class="button-move-up"/>
                        <input form:button="moveDown" class="button-move-down"/>
                        <input form:button="removeItem" class="button-remove-item"/>
                    </actions>
                    <input form:input=""/>
                </fieldset>
            </div>

            <fieldset form:fieldset="">
                <title>${_(u'Add new page element')}</title>
                <input form:input="../add_pagelet"/>
                <input form:button="addItem"/>
            </fieldset>

        </span>
        ''')


class PageletWidget(FormWidget):

    def view(self, state, felem, req):
        return xh.div(felem['value'].inputText, klass=felem.css)

    def edit(self, state, felem, req):
        return web.parseSrc(
            '<span>'
                '<input form:input="value"/><input form:errors="value"/>'
            '</span>'
        )

def getTranslation(felem, req, useDefault=True):
    lang = req.language
    for child in felem['_value'].ctrlChildren():
        if child.ctrlName == lang and child.inputText:
            return child.inputText
    if useDefault or lang not in web.translations():
        return felem['value'].inputText
    else:
        return ''


class TranslatablePageletWidget(PageletWidget):

    useDefault = True

    def view(self, state, felem, req):
        txt = getTranslation(felem, req, useDefault=self.useDefault)
        return xh.div(txt, klass=felem.css)

    def edit(self, state, felem, req):
        return web.parseSrc("""<span>
                <div><span wm:lang.js.switch="" /></div>
                <input form:input="_value" class="%s"/><input form:errors="_value"/>
            </span>
        """ % self.__class__.__name__)

class TitleWidget(TranslatablePageletWidget):

    def view(self, state, felem, req):
        txt = getTranslation(felem, req, useDefault=self.useDefault)
        return xh.h1(txt, klass=felem.css)


class HeadingWidget(TranslatablePageletWidget):
    useDefault = False

    def view(self, state, felem, req):
        txt = getTranslation(felem, req, useDefault=self.useDefault)
        if not txt:
            try:
                title = felem['../../title']
                txt = getTranslation(title, req, useDefault=self.useDefault)
            except KeyError:
                pass
        return xh.h1(txt, klass=felem.css)


class AbstractWidget(TranslatablePageletWidget):
    useDefault = False

class RestWidget(TranslatablePageletWidget):
    useDefault = False

    def view(self, state, felem, req):
        from xml4.rest import reST2html
        rest = getTranslation(felem, req, useDefault=self.useDefault)
        if rest and rest.strip():
            x = xh.div(xh.Literal(
                reST2html(rest, initial_header_level=2)), klass=felem.css)
            x.append(xh.div(klass='wm4-cleaner'))
            return x
        else:
            return None


class HtmlWidget(TranslatablePageletWidget):
    useDefault = False

    def view(self, state, felem, req):
        html = getTranslation(felem, req, useDefault=self.useDefault)
        if html:
            x = xh.div(xh.Literal(html), klass=felem.css)
            x.append(xh.div(klass='wm4-cleaner'))
            return x
        else:
            return None

class HtmlSrcWidget(HtmlWidget):
    pass

class ContentsWidget(PageletWidget):

    def children(self, req, node):
        prop = RootProperty(node.partition, 'pagelet_contents.check_perm')
        if prop(default=False):
            children = []
            for child in node.children:
                perm = req.route_for(child).permission
                if req.allows(perm):
                    children.append(child)
            return children
        else:
            return node.children

    def pageletdiv(self, felem, req, cont):
        return xh.div(cont, klass=felem.css)

    def view(self, state, felem, req):
        style = felem['value'].oldValue
        if not style:
            style = RootProperty(felem.partition, 'pagelet_contents.default')()
        node = felem.nodeCtrl.oldValue
        children = self.children(req, node)
        if not children:
            return None
        meth = getattr(self, 'view_' + str(style))
        return meth(state, felem, req, node, children)

    def view_subdir(self, state, felem, req, node, children):
        ul = xh.ul('\n')
        for child in children:
            ul.append(xh.li(
                xh.a(web.render(req,child._title), href=req.get_url(child))
            ))
            ul.append('\n')
        return self.pageletdiv(felem, req, ul)

    def view_subtree(self, state, felem, req, node, children):
        raise NotImplementedError
    view_subtree = view_subdir

    def view_alltree(self, state, felem, req, node, children):
        raise NotImplementedError
    view_alltree = view_subdir

    def view_subdir_abstract(self, state, felem, req, node, children):
        ul = xh.ul('\n')
        for child in children:
            li = xh.li()
            li.append(xh.a(web.render(req,child._title),
                           href=req.get_url(child)))
            if child._abstract:
                li.append(xh.div(web.render(req, child._abstract),
                                 klass='wm4-contents-item-abstract'))
            ul.append(li)
            ul.append('\n')
        return self.pageletdiv(felem, req, ul)

    def illustrated_item(self, item, state, size, req):
        illustration = item.findElement('cms','pagelet.illustration',None)
        u = req.get_url(item)
        i = xh.div(klass='comp4-illustrated-content-item-image')
        if size:
            if illustration and illustration.value:
                i.append(xh.a(xh.img(
                    src=req.get_url(item,illustration,'download'),
                    width=size[0] or illustration.img.width,
                    height=size[1] or illustration.img.height,
                    alt=item.title
                ), href=u))
            else:
                i.append(xh.img(
                    src='/++res++/comp4.style.static/img/transparent2x2.gif',
                    width=size[0], height=size[1], alt=''
                ))
        i = xh.div(i, klass='comp4-illustrated-content-item')
        t = xh.div(klass='comp4-illustrated_contents-text')
        t.append(xh.h2(xh.a(web.render(req, item._title), href=u)))
        if item.abstract:
            t.append(xh.div(web.render(req, item._abstract),
                            klass='comp4-illustrated-content-abstract'))
        i.append(t)
        return i

    def view_illustrated(self, state, felem, req, node, children):
        from lib4.iterutils import sequenceToTableHorizontally
        size = RootProperty(felem.partition, 'illustration.thumb_size')()
        if not size:
            size = RootProperty(felem.partition, 'illustration.size')()
        ct = xh.table(klass='cms4-illustrated-content', summary='')
        ncols = RootProperty(felem.partition, 'illustrated_contents.cols')()
        for row in sequenceToTableHorizontally(children, ncols):
            tr = xh.tr()
            for ci, cell in enumerate(row):
                if cell is None:
                    tr.append(xh.td(
                        xh.nbsp,
                        klass='cms4-illustrated-content-col%d' % (ci+1)
                    ))
                else:
                    tr.append(xh.td(
                        self.illustrated_item(cell, state, size, req),
                        klass='cms4-illustrated-content-col%d' % (ci+1)
                    ))
                ct.append('\n')
            ct.append(tr)
            ct.append('\n')
        return self.pageletdiv(felem, req, ct)



class SiblingsWidget(ContentsWidget):

    def children(self, req, node):
        if node.parent:
            prop = RootProperty(node.partition, 'pagelet_contents.check_perm')
            pc = [n for n in node.parent.children if n.inode!=node.inode]
            if prop(default=False):
                children = []
                for child in pc:
                    perm = req.route_for(child).permission
                    if req.allows(perm):
                        children.append(child)
                return children
            else:
                return pc
        else:
            return None

    def pageletdiv(self, felem, req, cont):
        base_class = felem.css
        tit = getTranslation(felem['title'], req)
        if tit:
            tit = xh.h3(tit)
        return xh.div(xh.div(xh.div(tit, cont,
                                    klass="%s-in" % base_class),
                             klass="%s-gfx" % base_class),
                      klass=base_class)

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
                <input form:input="value" />
                <div><span wm:lang.js.switch="" /></div>
                <div><label form:label="title" />: <input form:input="title/_value" />
                </div>
            </div>
        ''')





class ContentsByTagWidget(PageletWidget):

    def sorted_lst(self, felem, req):
        from controllers import ContentsByTagListingCtrl
        lst = ContentsByTagListingCtrl(req, felem=felem)
        lst()
        items_by_date = {}
        for item in lst.items:
            cals = [e.value for e in item.findElements('prop', 'calendar')]
            if cals:
                for c in cals:
                    items_by_date.setdefault(c,[]).append(item)
            else:
                items_by_date.setdefault(None,[]).append(item)
        dates = sorted(items_by_date.keys())
        items = []
        for d in dates:
            if not items_by_date[d] in items:
                items.extend(items_by_date[d])
        lst.items = items
        return lst


    def view(self, state, felem, req):
        from xml4.serialization import XHTMLSerializer
        # felem - controller
        # felem.oldValue - pagelet

        lst = self.sorted_lst(felem, req)

        s = xh.div(klass="cms4-contentstag")

        # pager
        scp = state.copy()
        scp['form'] = lst
        widg = ListingPagerWidget('form:listing.pager', '', self.gdict, tag='p')
        s.append(xh.Literal((XHTMLSerializer()(widg(scp)) or '').strip()))

        # Node list
        for item in lst.items:
            s.append(xh.h3(xh.a(item.title, href=req.get_url(item)),
                            klass="title"))
            cals = [e.value for e in item.findElements('prop', 'calendar')]
            if cals:
                s.append(xh.ul(klass="calendar",
                    *[xh.li('%s.%s.%s' % (c[-2:], c[4:6], c[:4]))
                      for c in cals]))
            s.append(xh.div(item.abstract or '', klass="abstract"))

        return s


    def edit(self, state, felem, req):
        return web.parseSrc("""<div>
            <?py _ = i18n.MessageIDFactory('comp4.cms')?>
            <div>
                ${_(u'Tag')}: <input form:input="value"/><input form:errors="value"/>
            </div>
            <div>
                ${_(u'Date range')}: <input form:input="date_range/value"/><input form:errors="date_range/value"/>
            </div>
        </div>
        """)


class ContentsByTagIllustrationsWidget(ContentsByTagWidget):

    def view(self, state, felem, req):
        from xml4.serialization import XHTMLSerializer

        lst = self.sorted_lst(felem, req)

        s = xh.div(klass="cms4-contentstag")

        # pager
        scp = state.copy()
        scp['form'] = lst
        widg = ListingPagerWidget('form:listing.pager', '', self.gdict, tag='p')
        s.append(xh.Literal((XHTMLSerializer()(widg(scp)) or '').strip()))

        # Node list
        for item in lst.items:
            s_i = xh.div()
            illustration = item.findElement('cms','pagelet.illustration',None)
            if illustration and illustration.value:
                s_i.append(xh.div(
                    xh.img(src=req.get_url(item, illustration, 'download'),
                            alt=item.title),
                    klass='illustration'))


            s_i.append(xh.h3(xh.a(item.title, href=req.get_url(item)),
                            klass="title"))
            cals = [e.value for e in item.findElements('prop', 'calendar')]
            if cals:
                s_i.append(xh.ul(klass="calendar",
                    *[xh.li('%s.%s.%s' % (c[-2:], c[4:6], c[:4]))
                      for c in cals]))
            s_i.append(xh.div(item.abstract or '', klass="abstract"))
            s.append(s_i)

        return s


class ImageWidget(PageletWidget):

    def view(self, state, felem, req):
        plet = felem.oldValue
        img = plet.img

        align = felem['align/value'].oldValue
        attrs = {}
        if align and align is not NOT_GIVEN:
            attrs['class'] = felem.css + ' ' + align
            if 'flow' in align:
                attrs['style'] = 'width: %dpx;' % img.width
        else:
            attrs['class'] = felem.css
        desc = getTranslation(felem['description'], req)

        img_alt = getTranslation(felem['alt'], req)
        if img_alt:
            alt = img_alt
        else:
            alt = '%s, %dx%d, %s' % (
                desc or img.undir, img.width, img.height, ByteCount(img.size)
            )

        src=req.get_url(plet.node, plet, 'download')

        im = xh.img(src=src, width=img.width, height=img.height, alt=alt)
        url = felem['url/value'].oldValue
        if url not in (None, NOT_GIVEN, ''):
            im = xh.a(im, href=url, title=alt)

        return xh.div(
            attrs,
            im,
            desc and xh.span(desc) or '',
        )

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
                <div><span wm:lang.js.switch="" /></div>
                <div><label form:label="description" /></div>
                <input form:input="description/_value" size="30" />

                <div><label form:label="align" /></div>
                <input form:input="align/value"/>
                <input form:errors="align/value"/>

                <div><label form:label="url" /></div>
                <input form:input="url/value"/>
                <input form:errors="url/value"/>

                <div><label form:label="img" /></div>
                <input form:input="img"/>
                <input form:errors="img"/>
            </div>
        ''')

class ImageAltWidget(ImageWidget):

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
                <div><span wm:lang.js.switch="" /></div>
                <div><label form:label="description" /></div>
                <input form:input="description/_value" size="30" />

                <div><label form:label="alt" /></div>
                <input form:input="alt/_value" size="30" />

                <div><label form:label="align" /></div>
                <input form:input="align/value"/>
                <input form:errors="align/value"/>

                <div><label form:label="url" /></div>
                <input form:input="url/value"/>
                <input form:errors="url/value"/>

                <div><label form:label="img" /></div>
                <input form:input="img"/>
                <input form:errors="img"/>
            </div>
        ''')

class IllustrationWidget(PageletWidget):

    def view(self, state, felem, req):
        if not RootProperty(felem.partition, 'illustration.show')():
            return None
        plet = felem.oldValue
        attrs = {}
        attrs['class'] = felem.css
        (w, h) = (plet.img.width, plet.img.height)
        return xh.div(
                attrs,
                xh.img(src=req.get_url(plet.node, plet, 'download'),
                       width=w, height=h,
                       alt='')
                )

    def edit(self, state, felem, req):
        try:
            (w, h) = RootProperty(felem.partition, 'illustration.size')()
        except:
            return web.parseSrc('''
                <?py _ = i18n.MessageIDFactory('comp4.cms')?>
                <div>
                    ${_(u'Illustration size is not set. To enable illustrations, please contact webmaster.')}
                </div>
                ''')
        return web.parseSrc('''
            <?py _ = i18n.MessageIDFactory('comp4.cms')?>
            <div>
                <div>
                    <div>${_(u'Optimal size of image is')} %sx%s ${_(u'pixels')}</div>
                    <label form:label="img" />
                </div>
                <input form:input="img"/>
                <input form:errors="img"/>
            </div>
        ''' % (w, h))




class PhotogalleryImageWidget(PageletWidget):

    def view(self, state, felem, req):
        pass

    def edit(self, state, felem, req):
        return web.parseSrc((
            '<div>\n'
                '%s: <input form:input="description/_value" size="30" />\n\n'
                '<input form:input="img"/><input form:errors="img"/>\n'
            '</div>\n'
        ) % ( _(u'Description')))



class PhotogalleryWidget(PageletWidget):

    def view(self, state, felem, req):
        gal = felem.oldValue
        if not gal.images:
            return None
        h = xh.div(klass=felem.css)
        if gal.value not in (None, NOT_GIVEN, ''):
            h.append(xh.h2(gal.value))
        if (gal.description not in (None, NOT_GIVEN)) and \
                (gal.description.value not in (None, NOT_GIVEN)):
            h.append(xh.p(gal.description.value))
        for plet in gal.images:
            h.append('\n')
            img = plet.img
            node = plet.node
            if plet.description is NOT_GIVEN:
                alt = img.undir
            else:
                alt = plet.description.value or img.undir
            alt = alt + ' (%dx%d, %s)' % (
                img.width, img.height, ByteCount(img.size)
            )
            if plet.thumbnail is not NOT_GIVEN:
                thumb = plet.thumbnail
                h.append(xh.div(
                    xh.a(
                        xh.img(
                            src=req.get_url(node, thumb, 'download'),
                            width=thumb.img.width, height=thumb.img.height,
                            alt=alt,
                        ),
                        title=alt,
                        href=req.get_url(node, plet, 'download'),
                    ),
                    klass=felem.css+'-item',
                ))
            else:
                h.append(xh.div(
                    xh.img(
                        src=req.get_url(node, plet, 'download'),
                        width=img.width, height=img.height, alt=alt,
                    ),
                    klass=felem.css+'-item',
                ))
            h.append('\n')
        h.append(xh.div(klass='wm4-clear-both'))
        return h

    def edit(self, state, felem, req):
        return web.parseSrc('''
        <div>
            <div><span wm:lang.js.switch="" /></div>
            <div>
                <div><label form:label="value"/></div>
                <input form:input="_value" size="40"/>
                <input form:errors="value"/>
            </div>
            <div>
                <div><label form:label="description"/></div>
                <input form:input="description/_value"/>
                <input form:errors="description/_value"/>
            </div>
            <!-- div>
                <div><label form:label="archiv"/></div>
                <input form:input="archiv"/>
                <input form:errors="archiv"/>
            </div -->
            <ol class="${form.css+'-items'}">
                <li form:repeat="images" class="$form.css">
                    <div class="wm4-form-fieldset-actions">
                    <input form:button="moveUp" class="button-move-up"/>
                    <input form:button="moveDown" class="button-move-down"/>
                    <input form:button="removeItem" class="button-remove-item"/>
                    </div>
                    <div>
                        <span form:label="description"/>
                        <input form:input="description/_value" size="30" />
                    </div>

                    <div><span form:label="img"/></div>
                    <input form:input="img"/>
                    <input form:errors="img"/>

                </li>
            </ol>
            <div>
                <input form:input="images/addItemCount"/>
                <input form:button="images/addItem"/>
            </div>
        </div>''')

class LightboxPhotogalleryWidget(PhotogalleryWidget):

    def view(self, state, felem, req):
        gal = felem.oldValue
        if not gal.images:
            return None

        for res in [
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/jquery/jquery.lightbox.js',
            'wm4.web.static/jquery/jquery.lightbox.css',
        ]:
            web.getResource(res).render(req)
        h = xh.div(klass=felem.css)
        if gal.value not in (None, NOT_GIVEN, ''):
            h.append(xh.h2(gal.value))
        if (gal.description not in (None, NOT_GIVEN)) and \
                (gal.description.value not in (None, NOT_GIVEN)):
            h.append(xh.p(gal.description.value))
        imgs = xh.div(id='phg-%s' % gal.eid)
        for plet in gal.images:
            imgs.append('\n')
            img = plet.img
            node = plet.node
            if (plet.description in (None, NOT_GIVEN)) or \
                    (plet.description.value in (None, NOT_GIVEN)):
                alt = img.undir
            else:
                alt = web.render(req, plet.description._value)
            alt = alt + ' (%dx%d, %s)' % (
                img.width, img.height, ByteCount(img.size)
            )
            if plet.thumbnail is not NOT_GIVEN:
                thumb = plet.thumbnail
                imgs.append(xh.div(
                    xh.a(
                        xh.img(
                            src=req.get_url(node, thumb, 'download'),
                            width=thumb.img.width, height=thumb.img.height,
                            alt=alt,
                        ),
                        title=alt,
                        href=req.get_url(node, plet, 'download'),
                        klass="phg-image",
                    ),
                    klass=felem.css+'-item',
                ))
            else:
                imgs.append(xh.div(
                    xh.img(
                        src=req.get_url(node, plet, 'download'),
                        width=img.width, height=img.height, alt=alt,
                    ),
                    klass=felem.css+'-item',
                ))
            imgs.append('\n')
        h.append(imgs)
        h.append(xh.div(klass='wm4-clear-both'))
        h.append(xh.script(
            "$(function() {$('#phg-%s a.phg-image').lightBox();});" % gal.eid,
            type="text/javascript"))
        return h



class FileWidget(PageletWidget):

    def view(self, state, felem, req):
        plet = felem.oldValue
        file = plet.file
        desc = getTranslation(felem['description'], req) or file.undir
        return xh.div(
            xh.a(desc, href=req.get_url(plet.node, plet, 'download')),
            u' (%s, %s)' % (file.ext, ByteCount(file.size)),
            klass = felem.css + ' ' + web.mime2css(file.content_type),
        )

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
                <div><span wm:lang.js.switch="" /></div>
                <div><label form:label="description" /></div>
                <input form:input="description/_value" size="30" />

                <div><label form:label="file" /></div>
                <input form:input="file"/>
                <input form:errors="file"/>
            </div>
        ''')


class DiscussionWidget(PageletWidget):

    def view(self, state, felem, req):
        from comp4.forum.api import Forum
        _ = i18n.MessageIDFactory('comp4.cms')
        pl = felem.oldValue
        forum = settings.workspace().get(Forum, pl.value)
        return web.parseSrc('''<span py:strip="">
            <?py _ = i18n.MessageIDFactory('comp4.cms')?>
            <div class="%s">
                <a href="%s">${_(u'Discussion')}</a>
            </div>
            </span>''' % (felem.css, req.get_url(forum)))

class PollWidget(PageletWidget):

    def view(self, state, felem, req):
        from comp4.poll.api import Poll
        from comp4.poll.web import PollWidget
        pl = felem.oldValue
        poll = settings.workspace().get(Poll, pl.value)
        tmpl = web.parseSrc('''
            <div>
                <?py from comp4.poll.entities import Poll ?>
                <div wm:comp4.poll.view="settings.workspace().get(
                                            Poll, form.oldValue.value)" />
            </div>''')
        return xh.div(
            tmpl,
            klass = felem.css,
        )

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div py:if="form.oldValue is NOT_GIVEN">
                <div>
                    <div><label form:label="question" /></div>
                    <input form:input="question" />
                </div>
                <ol>
                    <li form:repeat="options">
                        <div class="wm4-form-fieldset-actions">
                            <input form:button="moveUp"
                                   class="button-move-up"/>
                            <input form:button="moveDown"
                                   class="button-move-down"/>
                            <input form:button="removeItem"
                                   class="button-remove-item"/>
                        </div>
                        <div>
                            <span form:label="option"/>
                        </div>
                        <input form:input="option" size="30" />
                    </li>
                </ol>
                <div>
                    <input form:button="options/addItem"/>
                </div>
            </div>
        ''')


class LinkBoxLinkWidget(PageletWidget):

    def view(self, state, felem, req):
        pass

    def edit(self, state, felem, req):
        return web.parseSrc((
            '<div>\n'
                '%s: <input form:input="description/_value" size="30" />\n\n'
                '<input form:input="link"/><input form:errors="link"/>\n'
            '</div>\n'
        ) % ( _(u'Link text')))

class LinkBoxWidget(PageletWidget):

    def view(self, state, felem, req):
        lb = felem.oldValue
        if not lb.links:
            return None
        h = xh.div(klass=felem.css)
        desc = getTranslation(felem, req)
        if desc:
            h.append(xh.h2(desc))
        ul = xh.ul()
        for link in felem['links'].ctrlItems:
            href = link['value']
            desc = getTranslation(link['description'], req)
            if not desc:
                desc = href.inputText
            ul.append(xh.li(xh.a(desc, href=href.inputText, title=desc)))
        h.append(ul)
        return h

    def edit(self, state, felem, req):
        return web.parseSrc('''
        <div>
            <div><span wm:lang.js.switch="" /></div>
            <div>
                <div><label form:label="value"/></div>
                <input form:input="_value" size="40"/>
                <input form:errors="value"/>
            </div>
            <ol class="${form.css+'-items'}">
                <li form:repeat="links" class="$form.css">
                    <div class="wm4-form-fieldset-actions">
                    <input form:button="moveUp" class="button-move-up"/>
                    <input form:button="moveDown" class="button-move-down"/>
                    <input form:button="removeItem" class="button-remove-item"/>
                    </div>
                    <div>
                        <span form:label="description"/>
                        <input form:input="description/_value" size="30" />
                    </div>
                    <div><span form:label="value"/></div>
                    <input form:input="value"/>
                    <input form:errors="value"/>
                </li>
            </ol>
            <div>
                <input form:button="links/addItem"/>
            </div>
        </div>''')


NS = {'h':'http://www.w3.org/1999/xhtml'}

class MailFormWidget(PageletWidget):

    def view(self, state, felem, req):
        from lxml import etree
        from copy import deepcopy
        formid = felem.oldValue.value
        html = deepcopy(felem.partMgr.mailforms[formid])
        secret = settings.secret()
        to = SecureValue(
                value=felem.oldValue.children[0].value, secret=secret).pack()
        fid = SecureValue(
                value=formid, secret=secret).pack()
        part = SecureValue(
                value=felem.partition, secret=secret).pack()
        partMgr = felem.partMgr
        # XXX skaredy hak, protoze nasledujici kod nefunguje
        #     kvuli chybe ve wm4.web.routes
        # action = req.get_url(partMgr, 'add_root', 'toplevel.document')
        action = '/doc/mailform'
        for i in range(5):
            try:
                r = req.route / (('../' * i) + 'mailform')
            except Exception:
                r = None
            if r is not None:
                action = req.appURL.rstrip('/') + r.pathInfo
                break
        #
        antispam_expr = generate_expression()
        for resource in (
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/jquery/jquery.validate/jquery.validate.js',
            'wm4.web.static/jquery/jquery.validate/localization/messages_cs.js',
            'wm4.web.static/jquery/jquery.validate/jquery.validate.css',
        ):
            web.getResource(resource).render(req)
        for e in html.xpath("//h:form", namespaces=NS):
            e.set('action', action)
            e.set('method', 'post')
            e.insert(0, etree.Element(
                'input', {'type':'hidden', 'name':'mailform_id', 'value':fid}
            ))
            e.insert(1, etree.Element(
                'input', {'type':'hidden', 'name':'mailform_to', 'value':to}
            ))
            e.insert(2, etree.Element(
                'input', {'type':'hidden', 'name':'mailform_part', 'value':part}
            ))
            e.insert(3, etree.Element(
                'input', {'type':'hidden', 'name':'mailform_expr',
                        'value':AntispamExpr(
                            value=antispam_expr, timestamp=time.time(),
                            secret=secret).pack()}
            ))

        # javascriptovy antispam. ostrozsko/mailform.html
        for e in html.xpath("//h:span[@class='as_expr']", namespaces=NS):
            e.text = antispam_expr
        for e in html.xpath("//h:span[@class='as_expr_text']", namespaces=NS):
            e.text = expression_to_text(antispam_expr)

        # jazykove verze (priklad pouziti v kongresy/mailform.html)
        message_id = None
        for e in html.xpath("//h:meta[@name='message_id_factory']",
                namespaces=NS):
            message_id = e.attrib['content']
            html.remove(e)
        if message_id:
            _ = i18n.MessageIDFactory(message_id)
            for e in html.xpath("//h:*[@class='mf-translate']", namespaces=NS):
                e.text = web.render(req, _(e.text))
            # submit takovymto hnusnym hackem
            for e in html.xpath("//h:input[@type='submit']", namespaces=NS):
                e.set('value', web.render(req, _(e.attrib['value'])))

        return xh.div(xh.Literal(etree.tostring(html)), klass=felem.css)

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
                <div>Formulář</div>
                <div>
                    <input form:input="value" />
                    <input form:errors="value" />
                </div>
                <div>Cílový email</div>
                <div><small>
                    Je možné zadat víc emailů oddělených středníkem.
                </small></div>
                <div>
                    <input form:input="email/value" />
                    <input form:errors="email/value" />
                </div>
            </div>
        ''')




class VideoFtpWidget(PageletWidget):

    def view(self, state, felem, req):
        def rp(n):
            return RootProperty(felem.partition, 'pagelet.videoftp.%s' % n)()
        video = felem['value'].oldValue
        if not '.' in video:
            video_file = video + '.flv'
            image_file = video + '.jpg'
        else:
            video_file = video
            image_file = os.path.splitext(video)[0] + '.jpg'
        try:
            width = int(felem['width/value'].oldValue)
        except:
            width = int(rp('width.default'))
        try:
            height = int(felem['height/value'].oldValue)+26
        except:
            height = int(rp('height.default'))+26
        ftp_server = rp('ftp_server')
        ftp_root_url = rp('ftp_root_url')
        ftp_url = os.path.join('http://', ftp_server, ftp_root_url)
        video_file_url = os.path.join(ftp_url, video_file)
        image_file_url = os.path.join(ftp_url, image_file)
        id = felem.oldValue.eid
        h = xh.div(klass=felem.css)

        h.append(xh.Literal('''
            <object id="player" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" name="player" width="%s" height="%s">
                <param name="movie" value="/++res++/wm4.web.static/jwflv/player.swf" />
                <param name="allowfullscreen" value="true" />
                <param name="allowscriptaccess" value="always" />
                <param name="flashvars" value="file=%s&image=%s" />
                <embed
                        type="application/x-shockwave-flash"
                        id="player2"
                        name="player2"
                        src="/++res++/wm4.web.static/jwflv/player.swf"
                        width="%s"
                        height="%s"
                        allowscriptaccess="always"
                        allowfullscreen="true"
                        flashvars="file=%s&image=%s"
                />
            </object>
        ''' % (width, height,
               video_file_url, image_file_url,
               width, height,
               video_file_url, image_file_url,
               )))

        if felem['description/value'].oldValue not in (NOT_GIVEN, None):
            h.append(xh.div(xh.Literal(felem['description/value'].oldValue),
                            klass="video-description"))
        return h

    def edit(self, state, felem, req):
        return web.parseSrc('''<div>
                <div>
                    video file: <input form:input="value" />
                </div>
                <div>
                    <label form:label="width" />: <input form:input="width/value" />
                    <label form:label="height" />: <input form:input="height/value" />
                    <label form:label="description" />: <input form:input="description/value" />
                </div>
            </div>
            ''')

