from wm4.api import *
from wm4.web.forms.controllers import InputCtrl

__all__ = ['Message', 'Translations', 'DBGettextTranslation', 'DBGettextComp']



class Message(object):

    __slots__ = 'message', 'key', 'workspace', 'translations', 'useDefault'

    def __init__(self, message, key=None, workspace=None, translations=None,
        useDefault=True
    ):
        self.message = message
        self.key = key
        self.workspace = workspace
        self.translations = translations
        self.useDefault = useDefault

    def __repr__(self):
        return '<Message key=%r message=%r translations=%r>' % (
            self.key, self.message, self.translations
        )
    
    def __str__(self):
        return self.message or ''

class Translations(object):
    
    def __init__(self, ofAttr, useDefault=True):
        self.ofAttr = ofAttr
        self.useDefault = useDefault

    def key(self, obj):
        typ = obj.__class__
        oid = getattr(obj, '_dbgettext_key', None)
        if oid is None:
            oid = obj._oid
        return '%s.%s:%s/%s' % (
            typ.__module__, typ.__name__, self.ofAttr, str(oid)
        )

    def __get__(self, obj, typ):
        if obj is None:
            return self
        dfl = getattr(obj, self.ofAttr)
        key = self.key(obj)
        msg = Message(
            dfl, key, workspace=obj._workspace, useDefault=self.useDefault
        )
        return msg
    
    def __set__(self, obj, val):
        if val is None:
            setattr(obj, self.ofAttr, val)
        elif isinstance(val, Message):
            setattr(obj, self.ofAttr, val.message)
            obj.__dict__[self.ofAttr + '_translations'] = val.translations
            if obj._workspace and val.translations is not None:
                key = self.key(obj)
                obj._workspace['dbgettext'].update(key, val.translations)
        elif isinstance(val, basestring):
            setattr(obj, self.ofAttr, val)
        else:
            raise ValueError('unsupported type', type(val), val)


class DBGettextTranslation(schema.Entity):
    key         = schema.Attribute(str)
    language    = schema.Attribute(str)
    translation = schema.Attribute(unicode, schema.Required)

    binding.metadata(schema.PrimaryKey('key','language'))



class DBGettextComp(schema.SchemaComponent):
    
    cache = storage.TxnAttr(dict)

    workspace = binding.Make(lambda: settings.workspace())

    def _get(self, key):
        trans = {}
        for t in self.workspace(
            '[t for t in DBGettextTranslation if t.key == key]'
        ):
            trans[t.language] = t
        return trans

    def get(self, key):
        return dict((k,v.translation) for k,v in self._get(key).items())
        
    def update(self, key, trans):
        ws = self.workspace
        old = self._get(key)
        for k,n in trans.items():
            if k is None or n in ('', None, NOT_GIVEN):
                continue
            o = old.pop(k, None)
            if o is None:
                    ws.new(
                        DBGettextTranslation,
                        key=key, language=k, translation=n
                    )
            elif o.translation != n:
                o.translation = n
        for t in old.values():
            ws.remove(t)
