from wm4.api import *
from wm4.web.forms.controllers import InputCtrl, OnChangeEvent, event_handler
from wm4.web.forms.widgets import (
    InputWidget, AreaWidget, HtmlFckWidget, HtmlFckUploadWidget,
    HtmlTinyWidget,
    ALL_JS)
from wm4.web.utils import render_js_value
from xml4.elements import Comment
import xml4.xhtml as xh

from entities import *

__all__ = ['DBGettextCtrl', 'DBGettextAreaCtrl', 'DBGettextHtmlCtrl',
           'DBGettextHtmlWidget', 'DBGettextHtmlUploadWidget',
           'DBGettextHtmlTinyWidget',
           ]


@web.render.when('ob in Message')
def render_message(req, ob):
    wt = web.translations()
    if wt:
        trans = settings.workspace()['dbgettext'].get(ob.key)
        if not req.language in wt:
            default = ob.message
        elif ob.useDefault:
            default = ob.message
        else:
            default = ''
        return trans.get(req.language, default)
    else:
        return web.render(req, ob.message)

class DBGettextCtrl(InputCtrl):
    ctrlWidget = web.FORM_WIDGET('input.dbgettext')
    tcomp      = binding.Make(lambda: settings.workspace()['dbgettext'])
    languages  = binding.Make(lambda: web.translations())
    base       = binding.Make(lambda: settings.language())
    set_on_change = None

    @event_handler(OnChangeEvent)
    def on_change_callback(self, evt):
        if not self.set_on_change:
            return
        value = evt.sender.newValue
        nctrl = evt.sender[self.set_on_change]
        nctrl.inputText = value
        nctrl.newValue  = value
        OnChangeEvent(nctrl, old=nctrl.oldValue, new=nctrl.newValue)

    def getInputText(self):
        fid = self.ctrlId
        req = self.ctrlReq
        if self.formAction or fid in req:
            txt = req.getValue(fid, u'')
            txt = self.stripInput(txt)
            return txt
        else:
            if self.oldValue is NOT_GIVEN:
                txt = self.defaultInput()
            else:
                txt = self.oldValue
        if txt in (None, NOT_GIVEN):
            return ''
        else:
            return unicode(txt)
    inputText = binding.Make(getInputText)

    def _ctrlSetupChildren(self):
        features = {}
        data = self.oldValue
        if not self.languages:
            return features
        if data not in (NOT_GIVEN, None):
            trans = self.tcomp.get(data.key)
        else:
            trans = {}
        for l in self.languages:
            old = trans.get(l, None)
            if old not in (None, NOT_GIVEN):
                features[l] = InputCtrl(self.ctrlReq, l, self, oldValue=old)
            else:
                features[l] = InputCtrl(self.ctrlReq, l, self)
        return features

    def newValues(self):
        return dict(
            (l,c.newValue) for l, c in self._ctrlChildren.items()
                            if c.newValue not in (None, NOT_GIVEN)
        )

    def commitSelf(self):
        res = super(DBGettextCtrl, self).commitSelf()
        if not res or not self.languages:
            return res
        if self.newValue is NOT_GIVEN:
            self.newValue = NOT_GIVEN
        elif isinstance(self.newValue, Message):
            pass
        else:
            self.newValue = Message(self.newValue, 
                                    translations=self.newValues())
        OnChangeEvent(self, old=self.oldValue, new=self.newValue)
        return True


class DBGettextAreaCtrl(DBGettextCtrl):
    ctrlWidget = web.FORM_WIDGET('input.dbgettext.area')
    rows = None
    cols = None
    stripInput = staticmethod(unicode)

class DBGettextHtmlCtrl(DBGettextCtrl):
    ctrlWidget = web.FORM_WIDGET('input.dbgettext.html')
    rows = None
    cols = None
    stripInput = staticmethod(unicode)
    toolbarSet = 'Default'




class DBGettextWidget(InputWidget):
    
    def edit(self, state, felem, req):
        if not felem.languages:
            return super(DBGettextWidget, self).edit(state, felem, req)
        attrs = {
            'type' : self.type,
            'name' : felem.ctrlId,
            'value': felem.inputText,
            'lang' : felem.base,
        }
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey')+ALL_JS
        ))
        w = xh.span(
            xh.input(attrs),
            klass='wm4-language-switch-widget',
        )
        for l in felem.languages:
            c = felem._ctrlChildren[l]
            attrs = {
                'type' : self.type,
                'name' : c.ctrlId,
                'value': c.inputText,
                'lang' : l,
                'style': 'display:none;',
            }
            attrs.update(self.acquireAttrs(state,
                ('size','class','style')+ALL_JS
            ))
            w.append(xh.input(attrs))
        return w


class DBGettextAreaWidget(AreaWidget):
    
    def edit(self, state, felem, req):
        if not felem.languages:
            return super(DBGettextAreaWidget, self).edit(state, felem, req)
        _attrs = self.acquireAttrs(state,
            ('rows','cols','class','style',),
            rows=felem.rows, cols=felem.cols
        )
        attrs = _attrs.copy()
        attrs['name'] = felem.ctrlId
        attrs['lang'] = felem.base
        w = xh.span(
            xh.textarea(felem.inputText, **attrs),
            klass='wm4-language-switch-widget',
        )
        for l in felem.languages:
            c = felem._ctrlChildren[l]
            attrs = _attrs.copy()
            attrs['name'] = c.ctrlId
            attrs['lang'] = l
            attrs['style'] = 'display:none;'
            w.append(xh.textarea(c.inputText, **attrs))
        return w



class _DBGettextHtmlMixin(object):

    def edit(self, state, felem, req):
        if not felem.languages:
            return self.parentedit(state, felem, req)
        web.getResource('wm4.web.static/fck/fckeditor.js').render(req)
        web.getResource('wm4.web.static/jquery/jquery.js').render(req)


        attrs = self.acquireAttrs(state,
            ('rows','cols','class','style','id','tabindex','accesskey')+ALL_JS,
            rows=felem.rows, cols=felem.cols, id=felem.ctrlId
        )
        attrs['name'] = felem.ctrlId

        fckd = self.get_fck_script_attrs(state, felem, req, attrs)

        w = xh.span(
            xh.div(
                xh.script(Comment(
                    self._FCK_SCRIPT % fckd)),
                xh.textarea(felem.inputText, **attrs),
                lang=felem.base
            ),
            klass='wm4-language-switch-widget',
        )
        for l in felem.languages:
            c = felem._ctrlChildren[l]
            a = attrs.copy()
            a['name'] = c.ctrlId
            a['id'] = c.ctrlId
            fckd['id'] = render_js_value(req, c.ctrlId)
            w.append(
                xh.div(
                    xh.script(Comment(
                        self._FCK_SCRIPT % fckd)),
                    xh.textarea(c.inputText, **a),
                    lang=l, style='display:none;',
                ),
            )
        return w


class DBGettextHtmlWidget(_DBGettextHtmlMixin, HtmlFckWidget):

    def parentedit(self, state, felem, req):
        return HtmlFckWidget.edit(self, state, felem, req)
    


class DBGettextHtmlUploadWidget(_DBGettextHtmlMixin, HtmlFckUploadWidget):
    
    def parentedit(self, state, felem, req):
        return HtmlFckUploadWidget.edit(self, state, felem, req)



class DBGettextHtmlTinyWidget(HtmlTinyWidget):

    def edit(self, state, felem, req):
        if not felem.languages:
            return super(DBGettextHtmlTinyWidget, self).edit(state, felem, req)
        # zatim neresim pro vicejazycne weby, uvidime jestli to bude potreba
        raise Exception()

