#!/usr/bin/env python
# -*- coding: utf-8 -*-


import datetime
import lxml.etree as ET

from wm4.api import *
from wm4.schema.api import *

from adt4.serial import Serial



__all__ = [
        'EpocasiCache',
        'EpocasiComponent',
]



class EpocasiCache(Entity):
    pkey = Attribute(Serial)
    acquired = Attribute(datetime.datetime, defaultDatetime)
    data = Attribute(unicode)

    def parse_xml(self):
        datumy = {}
        obce = {}

        pocasi_xml = ET.fromstring(self.data)
        for den in  pocasi_xml.xpath('/pocasi/den'):
            datum = den.attrib['datum']
            try:
                datum = datetime.date(*[int(i) for i in datum.split('-')])
            except:
                pass
            den_obce = {}
            for obec in den.xpath('mesto'):
                obec_nazev = obec.attrib['mesto']
                data = (obec.attrib['den'], 
                        obec.attrib['noc'], 
                        obec.attrib['icon'])
                den_obce[obec_nazev] = data
                if not obec_nazev in obce:
                    obce[obec_nazev] = {}
                obce[obec_nazev][datum] = data
            datumy[datum] = den_obce

        return dict(datumy=datumy, obce=obce)
    obce = datumy = binding.MultiMake(parse_xml)

    binding.metadata(PrimaryKey('pkey'))




class EpocasiComponent(schema.SchemaComponent):


    def acquireData(self):
        ws = settings.workspace()
        url = config.Property('epocasi.xml_data_url')()
        pocasi_xml = ET.parse(url)
        obj = EpocasiCache(data=ET.tounicode(pocasi_xml))
        ws.add(obj)
        return obj 

    def lastData(self):
        ws = settings.workspace()
        q = ws.parse('''[sc for sc in 'epocasi/EpocasiCache'
                                 if query.OrderBy(query.Desc(sc.pkey))]''')
        q.limit = query.Const(1)
        obj = ws(q)
        if not obj:
            obj = self.acquireData()
        else:
            obj = list(obj)[0]
        return obj




