BEGIN;


CREATE TABLE event_calendar(
    pkey            SERIAL NOT NULL,

    name            VARCHAR NOT NULL,
    description     VARCHAR,

    privacy         VARCHAR NOT NULL,
    owner           INTEGER,

    PRIMARY KEY (pkey)
);


CREATE TABLE event (
    pkey            SERIAL NOT NULL,
    parent          INTEGER,
    calendar        INTEGER,

    name            VARCHAR NOT NULL,
    description     VARCHAR,
    place           VARCHAR,
    privacy         VARCHAR NOT NULL,

    PRIMARY KEY (pkey)
);


CREATE TABLE event_time (
    pkey            SERIAL NOT NULL,
    event           INTEGER NOT NULL,

    -- recurrence
    -- none - does not recurre
    -- daily each interval-th day
    -- weekly each interval*7 day
    -- monthly_day
    -- monthly_week (each interval-th monday...) 
    -- annually
    recurrence      VARCHAR NOT NULL DEFAULT 'none', 
    interval        INTEGER NOT NULL DEFAULT 1,
    repeat_until    TIMESTAMP WITH TIME ZONE,

    "begin"         TIMESTAMP WITH TIME ZONE NOT NULL,
    "end"           TIMESTAMP WITH TIME ZONE NOT NULL,

    description     VARCHAR,

    PRIMARY KEY (pkey)
);
CREATE INDEX idx_event_time_event ON event_time(event);


-- for recurrent events
CREATE TABLE event_time_cache(
    pkey            SERIAL NOT NULL,
    event_time      INTEGER NOT NULL,

    "begin"         TIMESTAMP WITH TIME ZONE NOT NULL,
    "end"           TIMESTAMP WITH TIME ZONE NOT NULL,

    PRIMARY KEY (pkey)
);
CREATE INDEX idx_event_time_cache_time ON event_time_cache(event_time);


CREATE TABLE event_time_exception (
    pkey            SERIAL NOT NULL,
    event_time      INTEGER NOT NULL,

    "time"          TIMESTAMP WITH TIME ZONE NOT NULL,
    description     VARCHAR,

    PRIMARY KEY (pkey)
);

CREATE TABLE event_reminder (
    "pkey"          SERIAL NOT NULL,
    "event_time"    INTEGER NOT NULL,

    "method"        VARCHAR NOT NULL, -- email, sms...
    "address"       VARCHAR NOT NULL, -- email, phone number...
    "time"          TIMESTAMP WITH TIME ZONE NOT NULL,
    "status"        VARCHAR NOT NULL, -- sent, waiting...

    "text"          TEXT,

    PRIMARY KEY (pkey)
);



CREATE TABLE event_resource (
    pkey            SERIAL NOT NULL,
    event           INTEGER NOT NULL,
    -- resource (user, group, room,...)
    klass           VARCHAR,
    ident           VARCHAR,
    -- owner, guest, ...
    role            VARCHAR,

    description     VARCHAR,

    PRIMARY KEY (pkey)
);


END;

