#!/usr/bin/env python
# -*- coding: utf-8 -*-

import cgi

from wm4.api import *

from fckutil import *


class FCKConnectorHandler(web.Handler):

    queryString = binding.Make(
            lambda s: dict(cgi.parse_qsl(s.req.environ.get('QUERY_STRING'))))

    # URL parameters

    def getValue(self, key, default=None):
        '''workaround: module cgi ignores query string if method is post'''
        v = self.req.getValue(key)
        if v is None:
            return self.queryString.get(key, default)
        else:
            return v

    def commandName(self):
        cmd = self.getValue('Command')
        if cmd in ["GetFolders", "GetFoldersAndFiles", 
                   "CreateFolder", "FileUpload", "QuickUpload"]:
            return cmd
        else:
            raise web.NotFound()
    commandName = binding.Make(commandName)

    def resourceType(self):
        type = self.getValue('Type', 'File')
        if type in ["File", "Image", "Flash", "Media"]:
            return type
        else:
            raise web.NotFound()
    resourceType = binding.Make(resourceType)

    def folderPath(self):
        pth = getCurrentFolder(self.getValue('CurrentFolder', '/'))
        if pth is None:
            raise web.NotFound()
        return pth
    folderPath = binding.Make(folderPath)

    newFolderName = binding.Make(lambda s: sanitizeFolderName(
                                    s.req.getValue('NewFolderName')))





    fckUploadId = binding.Make(lambda s: sanitizeFolderName(
                                    s.req.params['fckUploadId']))

    def userFilesURL(self):
        root = config.Property('fckconnector.FilesURL')()
        return os.path.join(root, self.fckUploadId, self.resourceType)
    userFilesURL = binding.Make(userFilesURL)

    folderURL = binding.Make(lambda s: mapServerFolder(
        s.userFilesURL, s.folderPath))

    def userFilesFolder(self):
        root = config.Property('fckconnector.FilesAbsolutePath')()
        return os.path.join(root, self.fckUploadId, self.resourceType)
    userFilesFolder = binding.Make(userFilesFolder)



    def render(self, template, **kw):
        req = self.req
        path = req.params[template]
        tmpl = req.getResource(path)
        if tmpl is None:
            raise ValueError('cannot find template', path)
        kw.update(dict(commandName=self.commandName, 
                       resourceType=self.resourceType,
                       folderPath=self.folderPath,
                       folderURL=self.folderURL))
        return tmpl.handleHTTP(req, **kw)


    def dirlist(self):
        # Map the virtual path to our local server
        serverPath = mapServerFolder(self.userFilesFolder, 
                                     self.folderPath)
        if not os.path.exists(serverPath):
            os.makedirs(serverPath)
        folders = []
        files = []
        for someObject in os.listdir(serverPath):
            someObjectPath = mapServerFolder(serverPath, someObject)
            if os.path.isdir(someObjectPath):
                folders.append(someObject)
            elif os.path.isfile(someObjectPath):
                files.append((someObject, os.path.getsize(someObjectPath)))
        return folders, files




    # commands

    def fc_GetFoldersAndFiles(self):
        """
        Purpose: command to recieve a list of folders and files
        """
        folders, files = self.dirlist()
        return self.render('filesTemplate', folders=folders, files=files)

    def fc_GetFolders(self):
        """
        Purpose: command to recieve a list of folders
        """
        folders, files = self.dirlist()
        return self.render('filesTemplate', folders=None, files=files)

    def fc_CreateFolder(self):
        """
        Purpose: command to create a new folder
        """
        errorNo = 0 
        errorMsg =''
        if self.newFolderName:
            try:
                newFolderPath = mapServerFolder(self.userFilesFolder, 
                            combinePaths(self.folderPath, self.newFolderName))
                os.makedirs(newFolderPath)
            except Exception, e:
                errorMsg = str(e).decode('iso-8859-1').encode('utf-8')
                if hasattr(e,'errno'):
                    if e.errno==17: #file already exists
                        errorNo=0
                    elif e.errno==13: # permission denied
                        errorNo = 103
                    elif e.errno==36 or e.errno==2 or e.errno==22: # filename too long / no such file / invalid name
                        errorNo = 102
                else:
                    errorNo = 110
        else:
            errorNo = 102
        return self.render('createFolderTemplate', 
                            errorNo=errorNo, errorMsg=errorMsg)

    def fc_FileUpload(self):
        """
        Purpose: command to upload files to server (same as FileUpload)
        """
        errorNo = 0
        errorMsg = ''
        newFileUrl = None
        newFileName = None

        if self.req.has_key('NewFile'):
            newFile = self.req.getUpload("NewFile")
            newFileName = sanitizeFileName(newFile.filename)
            newFileNameOnly = removeExtension(newFileName)
            newFileExtension = getExtension(newFileName).lower()
            allowedExtensions = config.Property(
                    'fckconnector.AllowedExtensions')().get(
                            self.resourceType, None)
            deniedExtensions = config.Property(
                    'fckconnector.DeniedExtensions')().get(
                            self.resourceType, None)
            if allowedExtensions:
                isAllowed = newFileExtension in allowedExtensions
            elif deniedExtensions:
                isAllowed = not newFileExtension in deniedExtensions
            else:
                isAllowed = True

            if isAllowed:
                currentFolderPath = mapServerFolder(self.userFilesFolder, 
                                     self.folderPath)
                i = 0
                while True:
                    newFilePath = os.path.join(currentFolderPath, newFileName)
                    if os.path.exists(newFilePath):
                        i += 1
                        newFileName = "%s_%04d.%s" % (newFileNameOnly,
                                                       i, newFileExtension)
                        errorNo = 201
                    else:
                        break
                fin = file(newFile.path, 'r')
                fout = file(newFilePath, 'wb')
                while True:
                    chunk = fin.read(100000)
                    if not chunk: break
                    fout.write(chunk)
                fout.close()
                newFileUrl = os.path.join(self.folderURL, newFileName)
            else:
                errorNo = 203
                errorMsg = "Extension not allowed"
        else:
            errorNo = 202
            errorMsg = "No File"

        return self.render('uploadTemplate', 
                            errorNo=errorNo, errorMsg=errorMsg, 
                            newFileUrl=newFileUrl, newFileName=newFileName)


    def fc_QuickUpload(self):
        currentFolderPath = mapServerFolder(self.userFilesFolder, 
                                     self.folderPath)
        if not os.path.exists(currentFolderPath):
            try:
                os.makedirs(currentFolderPath)
            except:
                raise web.NotFound()
        return self.fc_FileUpload()





    


    def do_fckcommand(self):
        return getattr(self, 'fc_%s' % self.commandName)()


