# -*- coding: utf-8 -*-

import textwrap, smtplib, sys

from wm4.api import *
from wm4.running import commands, options
from lib4.timeutils import *

if sys.version_info >= (2,5):
    from email.header import Header
    from email.mime.text import MIMEText
    from email.encoders import encode_7or8bit
else:
    from email.Header import Header
    from email.MIMEText import MIMEText
    from email.Encoders import encode_7or8bit

import comp4.registration.api as registration

from entities import *

class Mailinfo(commands.ConfiguredCommandLineApp):

    verbose = binding.Constant(False,
        options.Set('--verbose','-v',value=True)
    )

    dry_run = binding.Constant(False,
        options.Set('--dry-run','-n',value=True)
    )

    app_url = binding.Constant('http://www.valasskemezirici.cz/',
        options.Set('--app-url', '-A', type=str)
    )

    setup_url = binding.Constant('http://www.valasskemezirici.cz/mujportal/',
        options.Set('--setup-url', '-S', type=str)
    )

    test_email = binding.Constant(None,
        options.Set('--test-email', type=str)
    )
        
    sender = binding.Constant('noreply@valasskemezirici.cz',
        options.Set('--sender', type=str)
    )

    def main(self):
        super(Mailinfo, self).main()

        today = Today() + DAY
        yesterday = today - DAY - DAY

        sitemap = web.MiddlewareRegistry('sitemap')()['sitemap']
        url = web.URL(self.app_url)

        ws = settings.workspace()
        storage.begin()

        q = '''[
            (ru.email, rp.value)
                for ru in registration.RegisteredUser
                for rp in registration.RegisteredUserPreferences
                 if ru == rp.user
                and rp.pref == 'subscription'
                and ru.active == True
                and ru.approved is not None
        ]'''

        users = {}
        for e, v in ws(q):
            cl = [int(s[5:]) for s in v.split(',') if s.startswith('feed_')]
            users[e] = cl

        channels = list(ws('[c for c in FeedChannel]'))
        q = '''[
            (p,c.channel._oid)
                for p in FeedPost
                for c in FeedPostChannel
                 if p == c.post
                and query.OrderBy(query.Desc(p.updated))
                and yesterday <= p.updated < today
        ]'''
        
        chp = {}
        for p,c in ws(q):
            obj = ws.get(p.klass, p.key)
            chp.setdefault(c, []).append((p,obj))

        for rcpt, cl in users.items():
            seen = set()
            posts = []
            for c in cl:
                for (p,o) in chp.get(c,()):
                    k = (p.klass, p.key)
                    if k not in seen:
                        posts.append((p,o))
                        seen.add(k)
            if not posts:
                continue

            body = [
                u'Tato zpráva obsahuje seznam novinek na serveru %s' % url,
                u'a byla Vám zaslána automaticky na Vaši žádost.',
                u'Nastavení zasílání novinek si můžete nastavit na adres',
                u'%s .' % self.setup_url,
                u'',
            ]

            for post,obj in posts:
                tLines = textwrap.wrap(' '.join(post.title.split()), 70)
                tSize = max([len(l) for l in tLines])
                tFmt = u':: %%-%ds ::' % tSize
                body.extend([tFmt % l for l in tLines])
                body.append(url / sitemap.route_for(obj).pathInfo)
                if post.description:
                    body.extend(textwrap.wrap(
                        ' '.join(post.description.split())
                    ))
                body.append(u'')

            msg = MIMEText(
                u'\n'.join(body).encode('utf-8', 'ignore'),
                'plain', 'utf-8',
            )
            if self.test_email :
                finalRcpt = self.test_email
                msg['X-ThisIsTestMail-RealRecipientIs'] = rcpt
            else :
                finalRcpt = rcpt

            msg['Subject'] = Header(
                (u'Novinky z %s' % url).encode('utf-8'), 'utf-8')
            msg['From'] = self.sender
            msg['To'] = finalRcpt
            
            #encode_7or8bit(msg)
            if self.dry_run:
                if self.verbose :
                    print 'snt:', rcpt, 'TEST'
            else :
                try :
                    smtp = smtplib.SMTP('localhost')
                    smtp.sendmail(self.sender, finalRcpt, str(msg))
                    smtp.quit()
                except :
                    print 'snt:', rcpt, 'ERROR'
                else :
                    if self.verbose :
                        print 'snt:', rcpt, 'OK'

        storage.commit()



