from wm4.api import *
from wm4.schema.api import *
from adt4.serial import Serial
from datetime import datetime
from lib4.timeutils import Now

__all__ = ['FeedChannel', 'FeedPost', 'FeedPostChannel', 'FeedComponent']

class FeedChannel(Entity):
    pkey = Attribute(Serial)

    is_default = Attribute(bool, Required, Default(False))
    title = Attribute(unicode, Required)
    link = Attribute(str, Required)
    description = Attribute(unicode, Required)
    max_posts = Attribute(int, Required, Default(20))
    language = Attribute(str)
    editor = Attribute(str, Required)
    webmaster = Attribute(str)

    def posts(self):
        q = self._workspace.parse('''[
            p for p in FeedPost
              for pc in FeedPostChannel
              if pc.channel==self 
                 and pc.post==p
                 and query.OrderBy(query.Desc(p.updated))
            ]''')
        q.limit = query.Const(self.max_posts)
        return [r for r in self._workspace(q) if r.related_object]
    posts = binding.Make(posts)

    binding.metadata(PrimaryKey('pkey'))

def onlyOneDefault(ws, obj):
    if obj.is_default:
        ws('''[
            query.Update(c, is_default=False)
            for c in FeedChannel
                if c.pkey<>obj.pkey
            ]''')
@Workspace.do_add.before('obj in FeedChannel')
def onlyOneDefaultAdd(ws, obj):
    onlyOneDefault(ws, obj)
@Workspace.do_update.before('obj in FeedChannel')
def onlyOneDefaultAdd(ws, obj):
    onlyOneDefault(ws, obj)


class FeedPost(Entity):
    klass = Attribute(str, Required)
    key = Attribute(str, Required)

    title = Attribute(unicode, Required)
    description = Attribute(unicode, Required)
    author = Attribute(unicode)
    published = Attribute(datetime, defaultDatetime)
    updated = Attribute(datetime, defaultDatetime)

    def related_object(self):
        try:
            return settings.workspace().get(self.klass, self.key)
        except:
            return None
    related_object = binding.Make(related_object)

    binding.metadata(PrimaryKey('klass', 'key'))


class FeedPostChannel(Entity):
    post = Reference(FeedPost, Required)
    channel = Reference(FeedChannel, Required)

    binding.metadata(PrimaryKey('post', 'channel'))




class FeedComponent(schema.SchemaComponent):

    def defaultFeed(self):
        for c in self.allFeeds:
            if c.is_default:
                return c
    defaultFeed = storage.TxnAttr(defaultFeed)

    def allFeeds(self):
        return tuple(settings.workspace()('[ c for c in FeedChannel]'))
    allFeeds = storage.TxnAttr(allFeeds)

    def updatePost(self, klass, key, title,
        description=None, author=None, published=None, updated=None,
        channels=()
    ):
        old = -self.workspace('''[
            query.ExclusiveLock(p) for p in FeedPost 
                if p.klass == klass and p.key == key]''')
        if old is None:
            new = self.workspace.new(FeedPost,
                klass       = klass,
                key         = key,
                title       = title,
                description = description,
                author      = author,
            )

            channels = set([int(c) for c in channels])
            channels.add(self.defaultFeed.pkey)

            for pk in channels:
                chobj = self.workspace.get(FeedChannel, pk, default=None)
                if chobj is None:
                    continue
                self.workspace.new(FeedPostChannel, channel=chobj, post=new)

        else:
            # XXX SELECT ... FOR UPDATE is called twice
            old.title       = title
            old.description = description
            old.author      = author
            old.updated     = Now()
            self.workspace.flush(old)

            channels = set([int(c) for c in channels])
            channels.add(self.defaultFeed.pkey)

            for pc in self.workspace(
               '[pc for pc in FeedPostChannel if pc.post == old]'
            ):
                pk = pc.channel._oid
                if pk in channels:
                    channels.remove(pk)
                else:
                    self.workspace.remove(pc)
            for pk in channels:
                chobj = self.workspace.get(FeedChannel, pk, default=None)
                if chobj is None:
                    continue
                self.workspace.new(FeedPostChannel, channel=chobj, post=old)

    def removeVector(self, klass, key):
        raise NotImplementedError
