from wm4.api import *
from wm4.schema.api import *
from adt4.serial import Serial
from lib4.trees import AbstractTreeNode
from datetime import datetime

__all__ = ['Forum', 'ForumComment', 'ForumStopwords', 'ForumBlockedip']

_ = i18n.MessageIDFactory('comp4.forum')

class Forum(Entity):
    pkey = Attribute(Serial)
    klass = Attribute(str)
    key = Attribute(str)
    name = Attribute(unicode, Required)
    administrator = Reference('comp4.users.entities.User')
    moderated = Attribute(bool, Required, Default(False))
    closed = Attribute(bool, Required, Default(False))
    created = Attribute(datetime, defaultDatetime)

    #comments = ReferenceEnd('comp4.forum.entities.ForumComment', 'forum_id',
    #                        schema.ReferenceEndOrder('created'))
    def branches_and_comments(self):
        branches = {}
        comments = {}
        for com in settings.workspace()('''[
            com for com in ForumComment
                 if com.forum.pkey == self.pkey
                and query.OrderBy(com.created)
        ]'''):
            comments[com.pkey] = com
            branches.setdefault(com.parent, []).append(com)
        for par, children in branches.iteritems():
            if par:
                par.__dict__['children'] = children
        for com in comments.itervalues():
            com.__dict__.setdefault('children', ())
        return locals()
    branches = comments = binding.MultiMake(branches_and_comments)

    def children(self):
        return self.branches.get(None,())
    children = binding.Make(children)

    def related_object(self):
        obj = settings.workspace().get(self.klass, self.key)
        return obj
    related_object = binding.Make(related_object)

    binding.metadata(PrimaryKey('pkey'))


class ForumComment(Entity, AbstractTreeNode):
    pkey = Attribute(Serial)
    forum = Reference('comp4.forum.entities.Forum', Required)
    sender = Attribute(unicode)
    email = Attribute(unicode)
    www = Attribute(unicode)
    ip = Attribute(str)
    created = Attribute(datetime, defaultDatetime)
    published = Attribute(bool, Required, Default(True))
    deleted = Attribute(bool, Required, Default(False))
    subject = Attribute(unicode)
    message = Attribute(unicode, Required)
    parent = Reference(THIS_CLASS)

    visible = binding.Make(lambda s: not s.deleted and s.published)

    markup = binding.Obtain('message')

    children = ReferenceEnd(THIS_CLASS, 'parent')

    binding.metadata(PrimaryKey('pkey'))


class ForumStopwords(Entity):
    pkey = Attribute(Serial)
    word = Attribute(unicode, Required)

    binding.metadata(PrimaryKey('pkey'))


class ForumBlockedip(Entity):
    pkey = Attribute(Serial)
    ip = Attribute(unicode, Required)

    binding.metadata(PrimaryKey('pkey'))



