# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms import api as forms
from wm4.web.forms.listings import *

from xml4 import xhtml as xh
from xml4.templates import Domlet
from lib4.imports import importObject

from comp4.antispam.web import JSExprAntispamCtrl
from comp4.antispam.stopwords import StopWordsValidator
from comp4.antispam.ipaddress import IPAddressValidator
from comp4.users.entities import User

from entities import *

__all__ = ['CommentForm']


_ = i18n.MessageIDFactory('comp4.forum')



class CommentForm(forms.makeFormCtrl(ForumComment)):
    forum = binding.Require()
    parent_id = forms.ClientState(int)
    shown = forms.ClientState()

    class parent(forms.InputCtrl):
        def defaultOutput(self):
            par = self.ctrlParent
            return par.forum.comments.get(par.parent_id)

    class forum(forms.InputCtrl):
        def defaultOutput(self):
            return self.ctrlParent.forum

    class sender(forms.InputCtrl):
        ctrlLabel = _(u'Name')
        def defaultInput(self):
            u = self.ctrlReq.user
            if u:
                return u.name
            return ''

    class email(forms.InputCtrl):
        ctrlLabel = _(u'Email')
        def defaultInput(self):
            u = self.ctrlReq.user
            if u:
                return u.email
            return ''

    class preview(forms.ButtonCtrl):
        ctrlLabel = _(u'Preview')

    @forms.event_handler(forms.OnClickEvent, 'preview')
    def do_preview(self, evt):
        self.shown = 'yes'

    @forms.event_handler(forms.OnClickEvent, 'doSubmit')
    def require_preview(self, evt):
        del self.shown

    class message(forms.TextAreaCtrl):
        ctrlRequired = True
        rows = 20
        cols = 60

    def markup(self):
        return self['message'].inputText
    markup = binding.Make(markup)

    class published(forms.BoolCtrl):
        def defaultOutput(self):
            par = self.ctrlParent
            if par.forum.moderated:
                return False
            else:
                return True

    class validator(JSExprAntispamCtrl):
        pass

    class ip(forms.InputCtrl):
        def commitChanges(self):
            env = self.ctrlReq.environ
            ip = env['REMOTE_ADDR']
            if 'HTTP_X_FORWARDED_FOR' in env:
                ip = '%s/%s' % (ip, env['HTTP_X_FORWARDED_FOR'])
            self.newValue = ip
            forms.OnChangeEvent(self, old=self.oldValue, new=self.newValue)
            return True

    class stopwords_validator(StopWordsValidator):
        def get_text(self):
            text = u' '.join([self['../message'].inputText,
                              self['../subject'].inputText,
                              self['../sender'].inputText,
                              self['../email'].inputText,
                              self['../www'].inputText])
            return text
        def get_stopwords(self):
            return settings.workspace()(
                    '''[o.word for o in ForumStopwords]''')

    class ipaddress_validator(IPAddressValidator):
        def get_ip(self):
            env = self.ctrlReq.environ
            ip = env['REMOTE_ADDR']
            if 'HTTP_X_FORWARDED_FOR' in env:
                    ip = '%s/%s' % (ip, env['HTTP_X_FORWARDED_FOR'])
            return ip

        def get_blockedips(self):
            return settings.workspace()(
                    '''[o.ip for o in ForumBlockedip]''')

class ForumForm(forms.makeFormCtrl(Forum)):

    class administrator(forms.SelectCtrl):
        def selectOptions(self):
            return [
                (o._oid, o.name, o)
                for o in settings.workspace()(
                    '[o for o in User if query.OrderBy(o.name)]')
            ]
        selectOptions = binding.Make(selectOptions)


class ForumListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'forum/Forum']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = _(u'Name')

    class moderated(Column):
        queryExpr = 'obj.moderated'
        ctrlLabel = _(u'Moderated')

    class closed(Column):
        queryExpr = 'obj.closed'
        ctrlLabel = _(u'Closed')




class ListingHandler(web.EntityHandler):
    def do_list(self):
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing=lst)


class ForumHandler(ListingHandler):
    entity = binding.Constant(Forum)
    formCtrl = binding.Constant(ForumForm)

    def informAdministrator(self, forum, comment):
        __ = lambda m: web.render(self.req, _(m))
        if forum.administrator and forum.administrator.email:
            # poslat mail
            recipient = [forum.administrator.email]
            sender = 'noreply@muvalmez.cz'
            subject = __(u'New comment in forum').encode('utf-8')
            url = self.req.get_url(forum, 'edit')
            body = u'\n'.join([
                __(u'New comment in forum') + ' "%s": ',
                '',
                __(u'Edit URL') + ': %s',
                '',
                __(u'Sender') + ': %s',
                __(u'Email') + ': %s',
                __(u'WWW') + ': %s',
                __(u'IP') + ': %s',
                '',
                __(u'Subject') + ': %s',
                '%s']) % (
                    forum.name,
                    url,
                    comment['sender'] or '',
                    comment['email'] or '',
                    comment['www'] or '',
                    comment['ip'] or '',
                    comment['subject'] or '',
                    comment['message'])

            from lib4 import mail
            mail.sendMail(sender, recipient, subject, body.encode('utf-8'),
                          charset='utf-8')


    def do_comment(self):
        obj = self.obj
        form = CommentForm(ctrlReq=self.req, forum=obj)
        pid = self.req.getValue('parent_id', None)
        if pid is not None:
            form.parent_id = pid
        if form():
            self.informAdministrator(obj, form.newFeatures)
            storage.commit()
            storage.begin()
            return self.editResult(self.obj)
        else:
            if form.formErrors:
                storage.abort()
                storage.begin()
            return self.render(**self.editNS(form=form, obj=obj, mode='edit'))


class ForumCommentHandler(web.EntityHandler):
    entity = binding.Constant(ForumComment)

    def editResult(self, obj):
        raise web.SeeOther(self.req.get_url(self.obj.forum, 'edit'))



class StopwordsListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'forum/ForumStopwords']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class word(LikeFilter):
        queryExpr = 'obj.word'
        ctrlLabel = _(u'Word')

class ForumStopwordsHandler(ListingHandler):
    entity = binding.Constant(ForumStopwords)

    def addResult(self, obj):
        raise web.SeeOther(self.req.get_url(obj.__class__))

    deleteResult = addResult

class BlockedipListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'forum/ForumBlockedip']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class ip(LikeFilter):
        queryExpr = 'obj.ip'
        ctrlLabel = _(u'IP address')

class ForumBlockedipHandler(ListingHandler):
    entity = binding.Constant(ForumBlockedip)

    def addResult(self, obj):
        raise web.SeeOther(self.req.get_url(obj.__class__))

    deleteResult = addResult

