# -*- coding: utf-8 -*-

from sql4.utils import sql_escape
from wm4.api import *
from wm4.storage.type_descriptors import SqlTypeDescriptor

__all__ = [
    'TSVector', 'TSQuery', 'Fulltext', 'FulltextComp',
    'TSMatch', 'TSRank', 'TSHeadline'
]

class TSVector(object):
    def __init__(self, lang, *parts):
        self.lang = lang
        self.parts = parts

class TSQuery(object):
    def __init__(self, lang, expr):
        self.lang = lang
        self.expr = expr


class Fulltext(schema.Entity):
    klass  = schema.Attribute(str, schema.Required)
    key    = schema.Attribute(unicode, schema.Required)
    vector = schema.Attribute(TSVector, schema.Required)
    plain  = schema.Attribute(unicode, schema.Required)
    lang   = schema.Attribute(str, schema.Required)
    partition = schema.Attribute(unicode)

    def related_object(self):
        return settings.workspace().get(self.klass, self.key, None)
    related_object = binding.Make(related_object)

    binding.metadata(schema.PrimaryKey('klass', 'key'))



class FulltextComp(schema.SchemaComponent):
    
    def updateVector(self, klass, key, vector, plain, partition=None):
        ftextCls = self.workspace['fulltext/Fulltext']
	key = unicode(key)
        old = -self.workspace('''[
            query.ExclusiveLock(f) for f in ftextCls
                                    if f.klass == klass and f.key == key]''')
        if old is None:
            new = self.workspace.new(ftextCls,
                klass  = klass,
                key    = key,
                vector = vector,
                plain  = plain,
                lang   = vector.lang,
                partition = partition
            )
            return new
        else:
            # XXX SELECT ... FOR UPDATE is called twice
            old.vector = vector
            old.plain  = plain
            old.lang   = vector.lang
            old.partition = partition
            self.workspace.flush(old)
            return old
    
    def removeVector(self, klass, key):
        ftextCls = self.workspace['fulltext/Fulltext']
        key = unicode(key)
        self.workspace('''[
            query.Delete(f) for f in ftextCls
            if f.klass == klass and f.key == key
        ]''')
        return None






class TSMatch(query.Procedure):
    
    def __init__(self, vect, qry):
        self.args = vect, qry

@storage.PgSqlConnection.renderExpr.when('expr in TSMatch')
def render_ts_match(self, expr, relations):
        return '%s @@ %s' % (
            self.renderExpr(expr.args[0], relations),
            self.renderExpr(expr.args[1], relations),
        )

class TSRank(query.Procedure):

    def __init__(self, vect, qry):
        self.args = vect, qry

@storage.PgSqlConnection.renderExpr.when('expr in TSRank')
def render_ts_rank(self, expr, relations):
    if self.version >= (8,3):
        return 'ts_rank(%s, %s)' % (
            self.renderExpr(expr.args[0], relations),
            self.renderExpr(expr.args[1], relations),
        )
    else:
        return 'rank(%s, %s)' % (
            self.renderExpr(expr.args[0], relations),
            self.renderExpr(expr.args[1], relations),
        )

class TSHeadline(query.Procedure):

    def __init__(self, txt, qry):
        self.args = txt, qry

@storage.PgSqlConnection.renderExpr.when('expr in TSHeadline')
def render_ts_headline(self, expr, relations):
        return 'headline(%s, %s)' % (
            self.renderExpr(expr.args[0], relations),
            self.renderExpr(expr.args[1], relations),
        )




class TSVectorTypeDescriptor(object):
    def __init__(self):
        self.enc = settings.storage_connection().encoding
    def toSql(self, val):
        if not val.parts:
            return 'NULL'
        return ' || '.join( 
            "setweight(to_tsvector('%s',E'%s'), '%s')" % (
                val.lang, sql_escape(str.encode(self.enc)), weight
            )
            for str, weight in val.parts
        )
    def fromSql(self, val):
        if val is None:
            return None
        by_order = {}
        for r in val.decode(self.enc)[1:].split(" '"):
            r = r.strip()
            if not r:
                continue
            (s, owl) = r.rsplit(':', 1)
            s = s[:-1]
            for ow in owl.split(','):
                if ow[-1] in ('A', 'B', 'C', 'D'):
                    w = ow[-1]
                    o = int(ow[:-1])
                else:
                    w = 'D'
                    o = int(ow)
                by_order[o] = (s, w)
        parts = []
        for o in sorted(by_order.keys()):
            parts.append(by_order[o])
        return TSVector(None, *parts)

SqlTypeDescriptor(TSVector, lambda r,f: TSVectorTypeDescriptor())


class TSQueryTypeDescriptor(object):
    def __init__(self):
        self.enc = settings.storage_connection().encoding
    def toSql(self, val):
        return "to_tsquery('%s',E'%s')" % (
            val.lang, sql_escape(val.expr.encode(self.enc))
        )
    def fromSql(self, val):
        return val.replace("'", '')

SqlTypeDescriptor(TSQuery, lambda r,f: TSQueryTypeDescriptor())
