#!/usr/bin/env python
# -*- coding: ISO-8859-2 -*-

from PIL import Image
import aggdraw

from wm4.api import *
from lib4 import images
from lib4.cgi import http_time
from lib4.timeutils import Datetime, Now, timedelta
import lib4.iterutils

from entities import MapManager



def drawTrasa(img, points, line_color=None, show_points=0):
    '''na vstupu je zakladni obrazek s mapou (Image z PILu)
       a seznam jednotlivych bodu trasy (v pixelovych souradnicich obrazku)'''
    if not line_color:
        line_color = "#FF0000"
    line_width = 6
    point_color = "#00FF00"
    line_opacity = 150

    dr = aggdraw.Draw(img)
    pen = aggdraw.Pen(line_color, line_width, line_opacity)
    if len(points) > 1:
        dr.line(list(lib4.iterutils.flatten(points)), pen)

    if show_points=='last' and len(points)>0:
        pen = aggdraw.Pen(point_color, line_width, 255)
        dr.ellipse((points[-1][0]-2, points[-1][1]-2, 
                    points[-1][0]+2, points[-1][1]+2), pen)
    elif show_points:
        pen = aggdraw.Pen(point_color, line_width, 255)
        for (x, y) in points:
            dr.ellipse((x-2, y-2, x+2, y+2), pen)
    dr.flush()
    return img



def makeResponse(req, mapInfo, center, proj='sjtsk', trasa=None):
    resp = req.response
    env = req.environ

    if req.method not in ('GET','HEAD'):
        raise web.MethodNotAllowed(method)

    send_body = True
    mapmod = mapInfo.modified.replace(microsecond=0)
    try:
        lastm = Datetime(env.get('HTTP_IF_MODIFIED_SINCE', req).split(';')[0])
        if lastm == mapmod:
            send_body = False
    except:
        pass
    
    etag = env.get('HTTP_IF_NONE_MATCH', NOT_FOUND)
    if etag is not NOT_FOUND:
        if etag != mapInfo.etag(center, proj):
            send_body = True

    if trasa:
        send_body = True
    
    if not send_body:
        raise web.NotModified

    mapa = mapInfo.mapa
    # zakladni obrazek
    image = mapa.image(center, proj=proj)
    # logo
    if mapInfo.logo:
        logo = images.loadImage(mapInfo.logo)
        (x, y) = mapInfo.logo_position
        if x < 0:
            x = image.size[0] - logo.size[0] + x
        if y < 0:
            y = image.size[1] - logo.size[1] + y
        if logo.mode == 'RGBA':
            image.paste(logo, (x, y), logo)
        else:
            image.paste(logo, (x, y))
    # dokreslit trasu?
    if trasa:
        (x,y) = mapa.convertCoords(center, proj)
        (out_l,out_t,out_r,out_b) = mapa.box(x, y)
        drawTrasa(image,
                  [(x-out_l, y-out_t) for x, y in
                    [mapa.convertCoords((r.sourad_x, r.sourad_y), 'sjtsk')
                     for r in trasa.vedeniSouradnice]],
                  show_points=0)
    # indexovane barvy?
    if 1:
        p = Image.ADAPTIVE
        image = image.convert(mode="P" , palette=p, dither=Image.NONE)
    image.format = mapInfo.format
    image_data = images.getImageData(image)

    resp.addheader('Content-Length', str(len((image_data))))
    resp.addheader('Content-Type', images.getMIME(image))
    if not trasa:
        resp.addheader('ETag', mapInfo.etag(center, proj))
        resp.addheader('Expires', http_time(Now()+timedelta(365)))
        resp.addheader('Last-Modified', http_time(mapInfo.modified))

    resp.body = [image_data]
    return resp



class MapaHandler(web.Handler):

    def getCenter(self):
        '''vrati souradnice stredu mapy'''
        req = self.req
        try:
            x = float(req.params.get('x', 0))
        except:
            x = 0
        try:
            y = float(req.params.get('y', 0))
        except:
            y = 0
        return (x,y)

    def getProj(self):
        '''vrati v jake projekci jsou souradnice'''
        return self.req.params.get('proj', 'sjtsk')


    def do_image(self):
        req = self.req
        druh = req.params.get('druh')
        mapInfo = MapManager().getDruh(druh)
        return makeResponse(req, mapInfo, self.getCenter(), self.getProj(),
                            self.req.params.get('trasa'))



