#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms.controllers import *
from wm4.web.forms.listings import *
from wm4.utils.clspatch import patch

import lib4.images
from lib4.imports import importObject

from comp4.dbgettext.web import DBGettextCtrl, DBGettextAreaCtrl, DBGettextHtmlCtrl
from comp4.relations.entities import Relation


from entities import *


__all__ = [
        'MediaForm', 
        'MediaForm_video', 'MediaForm_audio', 'MediaForm_youtube', 
        'MediaListing',
        ]


_ = i18n.MessageIDFactory('comp4.media')


class MediaFileMixin(object):
    def commitSelf(self):
        if not self.ctrlId in self.ctrlReq:
            # keep old value
            return True
        if self.oldValue is NOT_GIVEN:
            if self.deleted or self.path is NOT_GIVEN:
                nv = None
            elif self.path and not self.size and self.emptyIsNone:
                nv = None
            else:
                nv = 'new'
        else:
            if self.deleted:
                nv = None
            elif self.path is NOT_GIVEN:
                nv = 'old'
            elif self.path and not self.size and self.emptyIsNone:
                nv = None
            else:
                nv = 'new'
        if nv == 'old':
            return True
        elif nv == 'new':
            try:
                self.newValue = self.makeFile()
            except schema.SchemaError, e:
                self.newValue = NOT_GIVEN
                self.ctrlError = e
                OnErrorEvent(self, error=e)
                return False
        else:
            assert nv == None
            if self.ctrlRequired:
                self.ctrlError = schema.RequiredError('')
                self.newValue = NOT_GIVEN
                return False
            else:
                self.newValue = None
        if self.oldValue != self.newValue:
            OnChangeEvent(self, old=self.oldValue, new=self.newValue)
        return True




class MediaFileCtrl(FileUploadCtrl):
    ctrlType = MediaFile

    def makeFile(self):
        nv = settings.workspace().add(self.ctrlType(
            filename=self.filename, blob=open(self.path),
            size=self.size, content_type=self.content_type,
            type=self._type,
            media=self['../'].newValue))
        return nv


class MediaImageCtrl(makeFormCtrl(MediaImage)):
    featureCommiting = {
            '*':'before', 
            'image':'after'
        }

    class media(InputCtrl):
        ctrlType = Media
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            return self['../../'].newValue

    class type(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            return self['../']._type

    class image(ImageUploadCtrl):

        def makeFile(self):
            import lib4.images as img4
            image = self['..'].newValue
            orig = img4.loadImage(self.path)
            ws = settings.workspace()
            ws('''[
                query.Delete(r) for r in MediaImageRes
                                 if r.image == image
            ]''')
            resolutions = {}
            for res, conv in self['../'].converters:
                img = conv(orig)
                size = img4.getStorageSize(img)
                fd = img4.getFileDesc(img)
                fd.seek(0,0)
                r = ws.new(MediaImageRes,
                    image        = image,
                    resolution   = res,
                    blob         = fd,
                    size         = size,
                    content_type = img4.getMIME(img),
                    filename     = res + '_' + self.filename,
                    width        = img.size[0],
                    height       = img.size[1],
                )
                resolutions[res] = r
            return resolutions




class MediaForm(makeFormCtrl(Media)):
    _type = None
    _src = None

    class _name(DBGettextCtrl):
        set_on_change = '../name'

    class description(HtmlCtrl): pass
    class _description(DBGettextHtmlCtrl):
        set_on_change = '../description'

    class type(InputCtrl):
        ctrlVisibility = 'hidden'
        def defaultOutput(self):
            return self['../']._type

    def addEntity(self):
        obj = self._workspace.new(self.ctrlType, **self.newFeatures)
        self._workspace.flush(obj)
        if self._src:
            r = self._workspace.new(Relation, src=self._src, dst=obj)
        return obj



class MediaForm_video(MediaForm):
    _type = MediaType.video
    featureCommiting = {
        '*': 'before',
        'file_video': 'after',
        'image_preview': 'after',
        }

    class file_video(MediaFileCtrl):
        _type='video'
        ctrlRequired = True
    class image_preview(MediaImageCtrl):
        _type='preview'
        converters = binding.Make(lambda s: [
            ('preview', 
             lib4.images.ImageConverter(
                maxSize=config.Property('comp4.media.video.size')(), 
                ratio='fixed'))])


class MediaForm_audio(MediaForm):
    _type = MediaType.audio
    featureCommiting = {
        '*': 'before',
        'file_audio': 'after',
        }

    class file_audio(MediaFileCtrl):
        _type='audio'
        ctrlRequired = True



class MediaForm_youtube(MediaForm):
    pass
    #_type = MediaType.youtube
    






def mediaFormFactory(**kw):
    typ = kw['oldValue'].type
    return importObject(
            'comp4.media.controllers.MediaForm_%s' % kw['oldValue'].type)(**kw)



class MediaListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'media/Media']'''

    class name(Column):
        queryExpr = 'obj.name'

    class type(SelectFilter):
        queryExpr = 'obj.type'
        






