#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.schema.api import *

from adt4.enums import *
from adt4.serial import Serial

from comp4.files import api as files
import comp4.dbgettext.api as dbgettext

__all__ = [
        'MediaType',
        'Media', 'MediaFile', 'MediaImage', 'MediaImageRes',
        'MediaComponent',
        ]


_ = i18n.MessageIDFactory('comp4.media')


class MediaType(Enum):
    video = enum(_(u'video'))
    audio = enum(_(u'audio'))
    #youtube = enum(_(u'youtube'))

class Media(Entity):
    pkey = Attribute(Serial)

    name = Attribute(unicode, Required)
    _name = dbgettext.Translations('name')
    description = Attribute(unicode)
    _description = dbgettext.Translations('description', useDefault=False)
    type = Attribute(MediaType, Required)
    parameters = Attribute(str)

    files = ReferenceEnd('comp4.media.entities.MediaFile', 'media')
    images = ReferenceEnd('comp4.media.entities.MediaImage', 'media')


    def __getattr__(self, attr):
        if attr.startswith('file_'):
            t = attr[5:]
            for f in self.files:
                if f.type == t:
                    return f
        elif attr.startswith('image_'):
            t = attr[6:]
            for f in self.images:
                if f.type == t:
                    return f
        raise AttributeError, attr

    binding.metadata(PrimaryKey('pkey'))


class MediaFile(files.File):
    media = Reference('comp4.media.entities.Media', Required)
    type = Attribute(str, Required)

class MediaImage(Entity):
    pkey = Attribute(Serial)
    media = Reference('comp4.media.entities.Media', Required)
    type = Attribute(str, Required)

    resolutions = ReferenceEnd(
            'comp4.media.entities.MediaImageRes', 'image')
    def get(self, resolution):
        for r in self.resolutions:
            if r.resolution == resolution:
                return r
        return None

    binding.metadata(PrimaryKey('pkey'))

class MediaImageRes(files.Image):
    image = Reference('comp4.media.entities.MediaImage', Required)
    resolution = Attribute(str, Required)




class MediaComponent(SchemaComponent):
    pass

