# -*- coding: utf-8 -*-

from wm4.api import *
from adt4.serial import Serial
from datetime import date, datetime
import os

__all__ = ['Icon', 'News', 'NewsContext', 'section_names']

_ = i18n.MessageIDFactory('comp4.news')


class Icon(binding.Component):
    name = None
    url = None
    filename = None

    def width_height(self):
        from lib4.images import loadImage
        width = height = None
        if self.filename:
            width, height = loadImage(self.filename).size
        return locals()
    width = height = binding.MultiMake(width_height)

    file_obj = None



class News(schema.Entity):
    pkey = schema.Attribute(Serial)

    section = schema.Attribute(unicode, schema.Required)
    title = schema.Attribute(unicode, schema.Required)
    description = schema.Attribute(unicode)
    url = schema.Attribute(unicode)
    icon = schema.Attribute(str)
    created = schema.Attribute(datetime, schema.Required,schema.defaultDatetime)
    priority = schema.Attribute(int, schema.Default(0))
    publish_from = schema.Attribute(date, schema.Required, schema.defaultDate)
    publish_till = schema.Attribute(date, schema.Required,
                        schema.Default(date(3000,1,1)))

    active = binding.Make(
                lambda s: s.publish_from <= date.today() <= s.publish_till)

    def cdate(self):
        c = self.created
        return date(c.year, c.month, c.day)
    cdate = binding.Make(cdate)

    def _icon(self):
        return self._workspace['news'].icon(self.icon)
    _icon = binding.Make(_icon)

    def to_json(self):
        return {
            'section':      self.section,
            'title':        self.title,
            'description':  self.description,
            'url':          self.url,
            'icon':         self.icon,
        }

    binding.metadata(schema.PrimaryKey('pkey'))


section_names = config.Property(
    'comp4.news.section_names', [('news', _(u'News'))])



class NewsContext(schema.SchemaComponent):

    icon_source = binding.Make(
        lambda: config.Property('comp4.news.icon_source')())

    icon_dir = binding.Make(
        lambda: config.Property('comp4.news.icon_dir')())

    icon_prefix = binding.Make(
        lambda: config.Property('comp4.news.icon_prefix')())

    icon_resolution = binding.Make(
        lambda: config.Property('comp4.news.icon_resolution')())

    def icon(self, name):
        if self.icon_source == 'ftp':
            try:
                return Icon(name=name,
                    url = os.path.join(self.icon_prefix, name),
                    filename=name, width='', height='')
            except:
                return None
        else:
            return self.icons.get(name)

    def icons(self):
        import glob, lib4.odict

        icons = []

        if self.icon_source == 'filesystem':
            if self.icon_dir and os.path.isdir(self.icon_dir):
                for filename in sorted(
                        glob.glob(os.path.join(self.icon_dir,'*'))):
                    if not os.path.isfile(filename):
                        continue
                    name = os.path.split(filename)[1]

                    if not(
                        name.endswith('.jpg') or name.endswith('.gif')
                        or name.endswith('.png')
                    ):
                        continue
                    url = os.path.join(self.icon_prefix, name)
                    icons.append((name, Icon(name=name, url=url,
                                             filename=filename)))
        elif self.icon_source == 'photogallery':
            phgal = settings.workspace()['photogallery'].idents.get(
                                        self.icon_dir, None)
            if phgal:
                req = web.request()
                for ph in phgal.photos:
                    if self.icon_resolution in ph.resdict:
                        icons.append((str(ph.pkey), Icon(
                            name=ph.name,
                            url=req.get_url(ph, 'download', self.icon_resolution),
                            filename=ph.pkey,
                            width=ph.resdict[self.icon_resolution].width,
                            height=ph.resdict[self.icon_resolution].height,
                            file_obj=ph)))
        elif self.icon_source == 'ftp':
            from ftplib import FTP
            u, s = self.icon_dir.split('@')
            user, password = u.split(':')
            server, path = s.split('/',1)
            ftp = FTP(server, user, password)
            for l in ftp.nlst(path):
                name = os.path.split(l)[1]
                if not(name.endswith('.jpg') or name.endswith('.gif')
                        or name.endswith('.png')):
                    continue
                icons.append((name, Icon(name=name,
                    url = os.path.join(self.icon_prefix, name),
                    filename=name, width='', height='')))
        else:
            pass
        return lib4.odict.OrderedDict(icons)
    icons = storage.TxnAttr(icons)

    def activeForSection(self, section):
        today = date.today()
        if section:
            return list(settings.workspace()("""[
                n for n in News
                   if n.publish_from <= today <= n.publish_till
                  and n.section == section
                  and query.OrderBy(query.Desc(n.priority),
                      query.Desc(n.created))
                ]"""))
        else:
            return list(settings.workspace()("""[
                n for n in News
                   if n.publish_from <= today <= n.publish_till
                  and query.OrderBy(query.Desc(n.priority),
                      query.Desc(n.created))
                ]"""))

