# -*- coding: utf-8 -*-

import datetime

from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import *

from lib4.imports import importObject

from comp4.feed.web import FeedChannelHandler

from entities import *

__all__ = ['NewsForm', 'NewsHandler', 'NewsListing']

_ = i18n.MessageIDFactory('comp4.news')


class NewsForm(makeFormCtrl(News)):

    class section(SelectCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Section')
        def selectOptions(self):
            return [
                (unicode(n), unicode(l), unicode(n)) for n,l in section_names()
            ]
        selectOptions = binding.Make(selectOptions)

    class title(InputCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Title')

    class description(TextAreaCtrl):
        ctrlLabel = _(u'Short text')

    class priority(SelectCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Priority')
        selectOptions = [
            ('+3', '+3', 3),
            ('+2', '+2', 2),
            ('+1', '+1', 1),
            ('0', '0', 0),
            ('-1', '-1', -1),
            ('-2', '-2', -2),
            ('-3', '-3', -3),
        ]
        def defaultInput(self):
            return 0

    class created(DatetimeCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Created')
        def defaultInput(self):
            return datetime.datetime.now()

    class publish_from(DateCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Publish from')
        def defaultInput(self):
            return datetime.date.today()

    class publish_till(DateCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Publish till')
        def defaultInput(self):
            return datetime.date.today() + datetime.timedelta(14)

    class icon(InputCtrl):
        ctrlLabel = _(u'Icon')

        class popup(PopupCtrl):
            popupMap = dict(icon = '..')
            def popupUrl(self):
                return self.ctrlReq.get_url(News,'icons')
            popupUrl = binding.Make(popupUrl)



class NewsListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'news/News']'''
    defaultOrder = '-created'

    class publish_from(Column):
        queryExpr = 'obj.publish_from'
        ctrlLabel = _(u'Publish from')

    class publish_till(Column):
        queryExpr = 'obj.publish_till'
        ctrlLabel = _(u'Publish till')

    class created(Column):
        queryExpr = 'obj.created'
        ctrlLabel = _('Created')

    class id(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class title(Column):
        queryExpr = 'obj.title'
        ctrlLabel = _(u'Title')

    class section(NewsForm.section, FilterMixin):
        ctrlRequired = False
        queryExpr = 'obj.section'
        noneLabel = binding.Obtain('ctrlLabel')


class NewsHandler(web.EntityHandler):
    entity = binding.Constant(News)
    formCtrl = binding.Constant(NewsForm)
    obj = binding.Make(lambda s: s.getObject())

    def do_icons(self):
        req = self.req
        tmpl = req.getResource(req.params['template'])
        icons = self.workspace['news'].icons
        return tmpl.handleHTTP(req, icons=icons.items())

    def do_list(self):
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing=lst, sectionDict=dict(section_names()))

    def do_section(self, req, section):
        import urlparse
        if section == 'ALL':
            section = None
        news = self.workspace['news'].activeForSection(section)
        acc = req.accept_resolve(['application/json'], 'text/html')
        if acc == 'application/json':
            data = []
            for n in news:
                j = n.to_json()
                u = j['url']
                if u:
                    j['url'] = urlparse.urljoin(req.requestURL, u)
                data.append(j)
            return self.render_json(data, indent=2)
        else:
            return self.render(news=news)



class NewsRssHandler(FeedChannelHandler):

    def do_rss(self, controller, req):
        lst = importObject(controller)(ctrlReq=req)
        lst()
        return self.feed_render('application/rss+xml', listing=lst)

