#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import *

from xml4 import xhtml as xh
from lib4.imports import importObject

from entities import *
from comp4.files.entities import Photo
from comp4.dbgettext.web import DBGettextCtrl, DBGettextAreaCtrl


_ = i18n.MessageIDFactory('comp4.photogallery')



class PhotoWidget(FormWidget):
    def edit(self, state, felem, req):
        t = xh.table()
        tr = xh.tr()
        if felem.oldValue not in (NOT_GIVEN, None):
            tr.append(xh.td(xh.img(src=req.get_url(felem.oldValue.photo,
                                                    'download', 'thumb'))))
        else:
            if felem['photo/orig'].filename not in (NOT_GIVEN, None):
                tr.append(xh.td(felem['photo/orig'].filename))
            else:
                tr.append(xh.td(''))
        tr.append(xh.td(xh.table(
            xh.tr(xh.th(web.render(req, _(u'Quality (1 is the best)'))),
                  xh.td(web.parseSrc('<input form:input="photo/quality" />'))),
            xh.tr(xh.th(web.render(req, _(u'Author'))),
                  xh.td(web.parseSrc('<input form:input="photo/author" />'))),
            xh.tr(xh.th(web.render(req, _(u'Description'))),
                  xh.td(web.parseSrc(
                      '<input form:input="photo/_description" />'))),
            xh.tr(xh.th(web.render(req, _(u'File'))),
                  xh.td(web.parseSrc('<input form:input="photo/orig" />'),
                        web.parseSrc('<input form:errors="photo/name" />')
                      )),
            )))
        t.append(tr)
        return t

    def view(self, state, felem, req):
        h = xh.div(klass="comp4-photogallery-item")
        alt = ''
        if felem.oldValue.photo.description:
            alt += felem.oldValue.photo.description
        else:
            alt += felem.oldValue.photo.name
        if felem.oldValue.photo.author:
            alt += ' (' + web.render(req, _(u'Author')) + \
                    ': %s)' % felem.oldValue.photo.author
        h.append(xh.a(
            xh.img(
                src=req.get_url(felem.oldValue.photo, 'download', 'thumb'),
                alt=alt, title=alt,
            ),
            href=req.get_url(felem.oldValue.photo, 'download'),
            title=alt,
            klass="comp4-photogallery-image",
        ))
        return h

class PhotosWidget(FormWidget):
    def edit(self, state, felem, req):
        return web.parseSrc('''
            <fieldset form:fieldset="">
                <title>${_(u'Photos')}</title>
                <table width="100%">
                    <tr form:repeat="">
                        <td>
                            <input form:input="" />
                            <input form:errors="." />
                        </td>
                        <td>
                            <div class="wm4-form-fieldset-actions">
                                <input form:input="moveUp"
                                    class="button-move-up"/>
                                <input form:input="moveDown"
                                    class="button-move-down"/>
                                <input form:input="removeItem"
                                    class="button-remove-item" />
                            </div>
                        </td>
                    </tr>
                </table>
                <input form:input="addItem" />
            </fieldset>''')

    def view(self, state, felem, req):
        return web.parseSrc('''
            <div form:repeat="">
                <input form:input="" />
            </div>
            ''')

class PhotogalleryWidget(FormWidget):
    def view(self, state, felem, req):
        for res in [
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/jquery/jquery.lightbox.js',
            'wm4.web.static/jquery/jquery.lightbox.css',
            'comp4.photogallery.static/style.css',
        ]:
            web.getResource(res).render(req)

        galid = "comp4-photogallery-%s" % felem.oldValue.pkey
        d = xh.div(id=galid)
        d.append(web.parseSrc('<input form:input="photogallery_photos" />'))
        d.append(xh.script(
          "$(function() {$('#%s a.comp4-photogallery-image').lightBox();});" % (
                                        galid),
            type="text/javascript"))
        return d



class PhotogalleryPhotoCtrl(makeFormCtrl(PhotogalleryPhoto), SequenceItemCtrl):
    ctrlWidget = web.FORM_WIDGET('input.photogallery.photogalleryphoto')

    class ordernum(InputCtrl):
        def defaultOutput(self):
            return self.ctrlParent.itemPosition

    class photogallery(InputCtrl):
        def defaultOutput(self):
            return self['../../../'].newValue

    class photo(makeFormCtrl(Photo)):
        _commitBefore = ['orig', 
                         'pkey', 'author', 'description', '_description',
                         'quality', 'name', 'kind']
        _commitAfter = []

        class description(TextAreaCtrl): pass
        class _description(DBGettextAreaCtrl):
            set_on_change = '../description'
        class quality(SelectCtrl):
            ctrlRequired = True
            selectOptions = binding.Make(
                lambda s: [(i, unicode(i), i) for i in range(5,0, -1)])
        class name(InputCtrl):
            def defaultOutput(self):
                return self['../orig'].filename
        class kind(InputCtrl):
            def defaultOutput(self):
                return 'photogallery.' + self['../../../../'].newValue.section
        class orig(ImageUploadCtrl):
            def makeFile(self):
                return self


class PhotogalleryPhotosCtrl(SequenceCtrl):
    ctrlWidget = web.FORM_WIDGET('input.photogallery.photogalleryphotos')
    itemClass = binding.Constant(PhotogalleryPhotoCtrl)

    class addItem(SequenceCtrl.addItem):
        ctrlLabel = _(u'Add photo')

    def updateContainer(self, removed):
        ws = settings.workspace()
        for i in removed:
            ws.remove(self.oldValue[i].photo)
            ws.remove(self.oldValue[i])




class PhotogalleryForm(makeFormCtrl(Photogallery)):
    ctrlWidget = web.FORM_WIDGET('input.photogallery.photogallery')
    featureCommiting = {
            '*': 'before',
            'photogallery_photos': 'after',
            }

    class section(SelectCtrl):
        ctrlRequired = True
        ctrlLabel = _(u'Section')
        def selectOptions(self):
            return [
                (unicode(n), unicode(l), unicode(n)) 
                for n, l in section_names()
                ]
        selectOptions = binding.Make(selectOptions)

    class description(TextAreaCtrl):
        cols = 60
        rows = 4

    class photogallery_photos(PhotogalleryPhotosCtrl): pass

    class _name(DBGettextCtrl):
        set_on_change = '../name'

    class _description(DBGettextAreaCtrl):
        set_on_change = '../description'


    





class PhotogalleryLst(ListingCtrl):
    baseQuery = '''[obj for obj in 'photogallery/Photogallery']'''

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = _(u'ID')

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = _(u'Name')

    class section(PhotogalleryForm.section, FilterMixin):
        ctrlRequired = False
        queryExpr = 'obj.section'

