#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.schema.api import *

from adt4.serial import Serial

import comp4.dbgettext.api as dbgettext


section_names = config.Property('comp4.photogallery.section_names')


class Photogallery(Entity):
    pkey = Attribute(Serial)

    section = Attribute(unicode)
    ident = Attribute(unicode)
    name = Attribute(unicode, Required)
    _name = dbgettext.Translations('name')
    description = Attribute(unicode)
    _description = dbgettext.Translations('description', useDefault=False)

    def photos(self):
        q = '''[
            (photo, phgalphoto)
                for photo in 'files/Photo'
                for phgalphoto in 'photogallery/PhotogalleryPhoto'
                 if phgalphoto.photo==photo
                and phgalphoto.photogallery==self
                and query.OrderBy(phgalphoto.ordernum)
            ]'''
        photos = []
        photogallery_photos = []
        for (p, pgp) in settings.workspace()(q):
            photos.append(p)
            photogallery_photos.append(pgp)
        return dict(photos=photos, photogallery_photos=photogallery_photos)
    photos = photogallery_photos = binding.MultiMake(photos)

    binding.metadata(PrimaryKey('pkey'))



class PhotogalleryPhoto(Entity):
    pkey = Attribute(Serial)

    photogallery = Reference('comp4.photogallery.entities.Photogallery', 
                             Required)
    photo = Reference('comp4.files.entities.Photo', Required)
    ordernum = Attribute(int, Required, Default(1))

    binding.metadata(PrimaryKey('pkey'))





class PhotogalleryComponent(schema.SchemaComponent):

    txnAttrs = ('idents',)

    def listForSection(self, section):
        return list(settings.workspace()("""[
            p for p in Photogallery
               if p.section==section
              and query.OrderBy(query.Desc(p._oid))
            ]"""))

    def idents(self):
        self.joinedTxn
        ws = settings.workspace()
        return dict(ws(
            '[(o.ident, o) for o in Photogallery if o.ident is not None]'))
    idents = binding.Make(idents)


