from wm4.api import *
from wm4.schema.api import *
from adt4.serial import Serial
from datetime import datetime, timedelta

__all__ = ['Poll', 'PollOption', 'PollVote',
           'PollComponent']

class Poll(Entity):
    pkey = Attribute(Serial)
    klass = Attribute(str)
    key = Attribute(str)
    question = Attribute(unicode, Required)
    description = Attribute(unicode)
    published = Attribute(bool, Required, Default(False))
    closed = Attribute(bool, Required, Default(False))
    vote_timeout = Attribute(int, Required, Default(7200))
    created = Attribute(datetime, defaultDatetime)

    options = ReferenceEnd('comp4.poll.entities.PollOption', 'poll',
                            schema.ReferenceEndOrder('ordernum'))
    options_keys = binding.Make(lambda s: [o.pkey for o in s.options])
    votes = QueryFeature('''[
            v for v in 'poll/PollVote'
               if v.poll_option.pkey in self.options_keys]''')

    def related_object(self):
        if self.klass:
            return settings.workspace().get(self.klass, self.key)
        else:
            return None
    related_object = binding.Make(related_object)

    binding.metadata(PrimaryKey('pkey'))


class PollOption(Entity):
    pkey = Attribute(Serial)
    poll = Reference('comp4.poll.entities.Poll', Required)
    option = Attribute(unicode, Required)
    ordernum = Attribute(int, Required, Default(100))
    votes = Attribute(int, Required, Default(0))

    binding.metadata(PrimaryKey('pkey'))



class PollVote(Entity):
    pkey = Attribute(Serial)
    poll_option = Reference('comp4.poll.entities.PollOption', Required)
    ip = Attribute(str, Required)
    vote_time = Attribute(datetime, defaultDatetime)

    binding.metadata(PrimaryKey('pkey'))

@schema.Workspace.do_add.before('obj in PollVote')
def checkVote(ws, obj):
    poll = obj.poll_option.poll
    if poll.closed:
        allowed = False
    else:
        allowed = True
        last_vote = ws['poll'].lastVote(poll, obj.ip)
        if last_vote:
            if datetime.now(last_vote.tzinfo)-last_vote < \
                    timedelta(seconds=poll.vote_timeout):
                allowed = False
    if not allowed:
        ee = schema.EntityError()
        ee[None] = schema.FeatureError(None, msgId='Vote not allowed')
        raise ee
    else:
        obj.poll_option.votes += 1





class PollComponent(schema.SchemaComponent):

    def lastVote(self, poll, ip):
        q = """[
            query.Max(v.vote_time)
                for v in PollVote
                for o in PollOption
                for p in Poll
                if v.poll_option==o
                   and o.poll==p
                   and p.pkey==%s
                   and v.ip=='%s'
            ]""" % (poll.pkey, ip)
        return list(self.workspace(q))[0]
        
    def activeForSection(self, section):
        return list(settings.workspace()("""[
            p for p in Poll
              if p.key==section
             and p.klass is None
             and p.published==True 
             and p.closed==False
             and query.OrderBy(query.Desc(p.created))
             ]"""))



