# -*- coding: utf-8 -*-

from wm4.api import *

from xml4 import xhtml as xh
from xml4.templates import Domlet

from lib4.imports import importObject

from entities import *
from controllers import *

__all__ = ['PollHandler', 'PollWidget']


_ = i18n.MessageIDFactory('comp4.poll')


class PollHandler(web.EntityHandler):
    entity = binding.Constant(Poll)
    formCtrl = binding.Constant(PollForm)

    def do_list(self):
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing=lst)

    def get_ip(self):
        env = self.req.environ
        ip = env['REMOTE_ADDR']
        if 'HTTP_X_FORWARDED_FOR' in env:
                ip = '%s/%s' % (ip, env['HTTP_X_FORWARDED_FOR'])
        return ip

    def do_vote(self):
        ws = settings.workspace()
        try:
            option = ws.get(PollOption,
                            int(self.req.getValue('option', None)))
            if option.poll is not self.obj:
                raise 'e'
        except:
            option = None
        ip = self.get_ip()
        if option and ip:
            try:
                vote = ws.new(PollVote, poll_option=option, ip=ip)
                storage.commit()
                storage.begin()
            except schema.EntityError:
                pass
        url = None
        if self.obj.klass is None:
            url = self.req.get_url(self.obj)
        else:
            url = self.req.get_url(self.obj.related_object.node)
        if url:
            raise web.SeeOther(url)
        else:
            raise 'e'


    def do_export_csv(self, req):
        import csv, StringIO

        res = req.response
        obj = self.obj
        f = StringIO.StringIO()
        w = csv.writer(f)
        w.writerow(['pkey', 'option_pkey', 'option', 'ip', 'vote_time'])
        for v in obj.votes:
            w.writerow([
                v.pkey, v.poll_option.pkey, v.poll_option.option.encode('utf-8'),
                v.ip, v.vote_time.strftime('%d.%m.%Y %H:%M:%S')
            ])
        data = f.getvalue()
        res.addheader('Content-type', 'text/comma-separated-values')
        res.addheader('Content-Disposition',
            'attachment; filename=votes.csv')
        res.addheader('Content-length', str(len(data)))
        res.body = [data]
        return res




class PollWidget(Domlet):

    def expand(self, state, poll, **options):
        req = state['req']
        try:
            votes_show_style = config.Property('comp4.poll.votes_show_style')()
        except:
            votes_show_style = 'percent'
        for res in ('comp4.poll.static/style.css',):
            web.getResource(res).render(req)
        s = xh.div()
        s.append(xh.h2(poll.question))
        if poll.description:
            s.append(xh.div(poll.description))
        if poll.closed:
            s.append(xh.div(web.render(req, _(u'This poll is closed'))))
        #s.append(xh.h3(web.render(req, _(u'Options'))))
        
        ol = xh.ol()
        votes_sum = sum([opt.votes for opt in poll.options])
        for opt in poll.options:
            optproc = (votes_sum and opt.votes*100./votes_sum or 0)
            li = xh.li()
            a = xh.a(opt.option,
                     href=req.get_url(poll, 'vote') % {'option': opt.pkey})
            li.append(a)
            if votes_show_style == 'percent':
                li.append(' (%.01f%%)' % optproc)
            else:
                li.append(' (%d)' % opt.votes)
            bar = xh.div(klass='bar')
            bar.append(xh.span(xh.nbsp, style='width: %0ipx;' % int(optproc)))
            li.append(bar)
            ol.append(li)
        s.append(ol)

        attrs = self.acquireAttrs(state, ('class', 'id', 'style'))
        if 'class' in attrs:
            attrs['class'] += ' comp4-poll'
        else:
            attrs['class'] = 'comp4-poll'
        return iter(xh.div(s, **attrs).expand(state))




