# -*- coding: utf-8 -*-

from wm4.api import *

import entities

__all__ = ['MyPlugin', 'HomePlugin', 'NewsletterPlugin']


class MyPlugin(binding.Component):
    
    name = req = forUser = binding.Require()

    template = binding.Require()

    macro = binding.Make(lambda s: s.name)

    workspace = binding.Make(lambda: settings.workspace())

    def __call__(self):
        tmpl = self.req.getResource(self.template)
        if self.macro:
            plug = tmpl.template.tmpl_namespace[self.macro]
            return plug(self)
        else:
            raise NotImplementedError



class HomePlugin(MyPlugin):
    
    def __call__(self):
        req = self.req
        if req.getValue('home.submit'):
            self.forUser.home = req.getValue('home.name')
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        return super(HomePlugin, self).__call__()



class NewsletterPlugin(MyPlugin):
    
    channels = binding.Make(lambda: config.Property(
        'comp4.registration.newsletter.channels')(default=None))

    def _subs(self):
        for p in self.forUser._preferences:
            if p.pref == 'newsletter.channels':
                return p
    _subs = binding.Make(_subs)

    def subs(self):
        if self._subs:
            return map(None, self._subs.value.split(','))
        return []
    subs = binding.Make(subs)

    def __call__(self):
        req = self.req
        if req.getValue('%s.submit' % self.name):
            chans = [
                i for i,n in self.channels if req.getValue(i, default=None)
            ]
            if self._subs is None:
                self.workspace.new(
                    entities.RegisteredUserPreferences,
                    user = self.forUser,
                    pref = 'newsletter.channels',
                    value = ','.join(chans),
                )
            else:
                self._subs.value = ','.join(chans)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)
        return super(NewsletterPlugin, self).__call__()
        
