from wm4.api import *
from wm4.web.routes import *
from wm4.utils.clspatch import patch
from wm4.web.navigation import Link, actionLinks, DELIMITER

from entities import *

__all__ = ['RegistrationSitemap']

_ = i18n.MessageIDFactory('comp4.registration')


reg_auth = web.Authenticated(auth_domain='registered_user', throw=True)


class RegistrationSitemap(Route):
    virtual = True

    class myadmin(Route):
        match = 'myadmin/'
        action = Const('myadmin')
        locId = Const('myadmin')
        handler = Const('comp4.registration.web.MyHandler')
        template = Const('comp4.registration/myadmin.wt')
        signature = Signature('locId')

    class myhome(Route):
        match = 'my/:name'
        name = Param()
        locId = Const('myhome')
        action = Const('myhome')
        handler = Const('comp4.registration.web.MyHandler')
        template = Const('comp4.registration/myhome.wt')
        signature = Signature('locId', 'name')


    class registration(Route): 
        match = 'registration/'
        virtual = True
        handler = Const('comp4.registration.web.RegistrationPublicHandler')
        clsObj = Const(RegisteredUser)

        class new(Route):
            match = 'new'
            action = Const('new')
            signature = Signature('clsObj', 'action')
            template = Const('comp4.registration/public_new_user.wt')
            info_template = Const('comp4.registration/public_new_info.wt')

        class verify(Route):
            match = 'verify'
            action = Const('verify')
            signature = Signature('clsObj', 'action')
            template = Const('comp4.registration/public_verify.wt')

        class login(Route):
            match = 'login'
            action = Const('login')
            signature = Signature('clsObj', 'action')
            template = Const('comp4.registration/public_login.wt')

        class logout(Route):
            match = 'logout'
            action = Const('logout')
            signature = Signature('clsObj', 'action')

        class edit(Route):
            match = 'edit'
            action = Const('edit')
            signature = Signature('clsObj', 'action')
            template = Const('comp4.registration/public_edit.wt')

        class send_lost(Route):
            match = 'send_lost'
            action = Const('send_lost')
            signature = Signature('clsObj', 'action')
            template = Const('comp4.registration/public_lost_password.wt')

        class admin(web.EntitySitemap):
            clsObj = Const(RegisteredUser)
            match = 'admin/'
            handler = Const('comp4.registration.web.RegistrationAdminHandler')
            listingCtrl = Const('comp4.registration.web.UserListing')
            viewPerm = Const(web.Authenticated())

            class cls(web.EntitySitemap.cls):
                actionLinks = Lazy(lambda s: actionLinks(
                    s, '', './add', './sendmail', './sendmail/archive'))
                class sendmail(web.ClassRoute):
                    template = Const('comp4.registration/admin_mail_form.wt')
                    asAction = Lazy(
                            lambda s: Link.fromRoute(s, label=_(u'Send mail')))
                    class archive(web.Route):
                        match = '/archive'
                        template =Const('comp4.registration/admin_mail_list.wt')
                        action = Const('sendmail_archive')
                        controller = Const('comp4.registration.web.RegisteredSendmailLst')
                        asAction = Lazy(
                            lambda s: Link.fromRoute(s, label=_(u'Archiv')))
                        

            patch({
                'cls.list.template': Const('comp4.registration/admin_list.wt'),
                'cls.add.template' : Const('comp4.registration/admin_form.wt'),
                'obj.view.template': Const('comp4.registration/admin_form.wt'),
                'obj.edit.template': Const('comp4.registration/admin_form.wt'),
            })

