# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import ListingCtrl, Column
from lib4.timeutils import Now
from lib4.imports import importObject

from comp4.simpleparams.controllers import SingleParametersCtrl
from comp4.antispam.web import JSExprAntispamCtrl

from entities import *

__all__ = ['AUTH_DOMAIN']

_ = i18n.MessageIDFactory('comp4.registration')


AUTH_DOMAIN = 'registered_user'


class LoginForm(FormCtrl):
    
    ctrlAllowedMethods = set([FormMethod.post])
    
    class ref(InputCtrl):
        ctrlVisibility = "hidden"
        def defaultInput(self):
            return self.ctrlReq.environ.get('HTTP_REFERER')

    class username(InputCtrl):
        pass

    class password(PasswordCtrl):
        pass



class RegistrationPublicForm(makeFormCtrl(RegisteredUser)):

    class uid(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')

    class password(ValidatingPasswordCtrl):
        ctrlRequired = True
        minLength = binding.Make(
                    config.Property('comp4.registration.password_min_length'))

    class login(InputCtrl):
        ctrlRequired = True

        def defaultOutput(self):
            return self['../email'].defaultOutput()

        def inputText(self):
            return self['../email'].inputText
        inputText = binding.Make(inputText)
        
        @event_handler(OnChangeEvent, '../email')
        def onEmailChange(self, evt):
            OnChangeEvent(self, old=evt.old, new=evt.new)

    class email(InputCtrl):
        ctrlRequired = True
        ctrlReadOnly = binding.Make(lambda s:s.ctrlReq.params['action']=='edit')

    class key(InputCtrl):
        
        def defaultOutput(self):
            reg = settings.workspace()['registration']
            return reg.genKey(RegisteredUser, 'key')

        def commitSelf(self):
            if self.oldValue is NOT_GIVEN:
                self.newValue = self.defaultOutput()
            if self.oldValue != self.newValue:
                OnChangeEvent(self, old=self.oldValue, new=self.newValue)
            return True
    
    class params(SingleParametersCtrl):
        plugin = 'comp4.registration.params'

    class validator(JSExprAntispamCtrl): pass



class RegistrationPublicHandler(web.Handler):

    timeout = 3600

    formCtrl = binding.Constant(RegistrationPublicForm)

    def do_new(self):
        req = self.req
        form = self.formCtrl(ctrlReq=req)
        if form():
            user = form.newValue

            authenticator = web.getAuthenticator(req.environ, AUTH_DOMAIN)
            token = authenticator.createToken(req.environ, user, self.timeout)
            authenticator.setToken(req.environ, token)

            reg = settings.workspace()['registration']
            reg.sendVerification(
                user, req.environ['SERVER_NAME'],
                req.get_url(RegisteredUser,'verify'),
            )

            storage.commit()
            storage.begin()

            tmpl = web.getResource(req.params['info_template'])
            return tmpl.handleHTTP(req, user=user)

        else:
            if form.formErrors:
                storage.abort()
                storage.begin()
        return self.render(form=form)

    def do_verify(self):
        req = self.req
        key = req.getValue('key')
        reg_user = -self.workspace('''[
            ru for ru in RegisteredUser
                if ru.key == key
        ]''')
        message = None
        login_url = None
        if not reg_user:
            message = u'Chyba: registrační klíč %s nenalezen.' % key
        else:
            login_url = req.get_url(RegisteredUser, 'login') % dict(
                ref=req.appURL
            )
            if reg_user.verified:
                message = u'Registrace již byla potvrzena dříve.'
            else:
                reg_user.verified = reg_user.approved = Now()
                message = u'Registrace byla úspěšně dokončena'
        return self.render(message=message, login_url=login_url)

    def default_login_ref(self):
        return self.req.appURL

    def do_login(self):
        req = self.req
        authenticator = web.getAuthenticator(req.environ, AUTH_DOMAIN)

        form = LoginForm(ctrlReq=req)
        res = form()
        username = form['username'].newValue
        password = form['password'].newValue

        user = authenticator.doLogin(req, username, password)

        message = None
        if not res or user is None:
            if res:
                message = _(u'Špatné jméno nebo heslo')
            template = req.getResource(req.params['template'])
            return template.handleHTTP(
                req, authenticator=authenticator, form=form, message=message
            )
        loc = req.getValue('ref')
        if not loc:
            loc = self.default_login_ref()
        raise web.SeeOther(location=str(loc))

    def do_logout(self):
        req = self.req
        authenticator = web.getAuthenticator(req.environ, AUTH_DOMAIN)
        authenticator.doLogout(req)
        #loc = req.getValue('ref')
        #if not loc:
        #    loc = req.environ.get('HTTP_REFERER')
        #if not loc:
        #    loc = req.appURL
        loc = req.appURL
        raise web.SeeOther(location=str(loc))

    def do_edit(self):
        req = self.req
        user = web.getUser(req.environ, 'registered_user')
        form = self.formCtrl(ctrlReq=req, oldValue=user)
        if form():
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)
        else:
            if form.formErrors:
                storage.abort()
                storage.begin()
        return self.render(form=form)

    def do_send_lost(self):
        req = self.req
        send = req.getValue('send')
        message = email = None
        if send:
            email = req.getValue('email')
            if not email:
                message = web.render(req, _(u"E-mail wasn't specified"))
                send = None
            else:
                reg_user = -self.workspace('''[
                    ru for ru in RegisteredUser
                        if ru.email == email
                       and ru.verified is not None
                       and ru.approved is not None
                       and ru.active == True
                ]''')
                if not reg_user:
                    message = u'Registrace s emailem %s nenalezena.' % email
                    send = None
                else:
                    reg = settings.workspace()['registration']
                    reg.sendLostPassword(reg_user, req.environ['SERVER_NAME'])
                    message = web.render(req, _(u'The password has been sent to')) + ' %s.' % email
        return self.render(send=send, message=message, email=email)



class RegistrationAdminForm(makeFormCtrl(RegisteredUser)):

    class params(SingleParametersCtrl):
        plugin = 'comp4.registration.params'



class RegisteredSendmailLst(ListingCtrl):
    baseQuery = binding.Constant(
        "[rs for rs in '/registration/RegisteredSendmail']")
    class sent(Column):
        queryExpr='rs.sent'
    class subject(Column):
        queryExpr='rs.subject'
    defaultOrder = '-sent'

class RegisteredSendmailForm(FormCtrl):

    class subject(InputCtrl):
        ctrlLabel = u'Předmět'

    class body(TextAreaCtrl):
        ctrlLabel = u'Zpráva'

    class sender(InputCtrl):
        ctrlLabel = u'Odesílatel'

    class attachment(FileUploadCtrl):
        def makeFile(self):
            return dict(
                    blob = open(self.path),
                    size = self.size,
                    content_type = self.content_type,
                    filename = self.filename
                    )


class UserListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'registration/RegisteredUser']'''

class RegistrationAdminHandler(web.EntityHandler):
    entity = binding.Constant(RegisteredUser)
    formCtrl = binding.Constant(RegistrationAdminForm)

    channels = binding.Make(lambda: config.Property(
        'comp4.registration.newsletter.channels')(default=None))

    def do_list(self):
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing=lst)

    def do_sendmail_archive(self, req, controller):
        lst = importObject(controller)(ctrlReq=req)
        lst()
        return self.render(listing=lst, action='list')

    def do_sendmail(self, req):
        from comp4.sendmail.api import sendmail_comp

        form = RegisteredSendmailForm(req)
        form()

        smcomp = sendmail_comp()
        regcomp = self.workspace['registration']
        default_sender = sender = smcomp.params(regcomp.sendmail_name)['sender']
        if not isinstance(sender, basestring):
            sender = sender[1]
        send = req.getValue('doSubmit')
        subject = message = body = None
        if send:
            subject = req.getValue('subject')
            if not subject:
                message = u'Nebyl zadán předmět'
                send = None
            body = req.getValue('body')
            if not body:
                message = u'Nebyl zadán text mailu'
                send = None
            sender = req.getValue('sender')
            if not sender:
                sender = default_sender
            if not self.channels or req.getValue('target',default='all')=='all':
                chans = None
            else:
                chans = [i for i,n in self.channels
                            if req.getValue(i, default=None)]
            attachment = form['attachment'].newValue
            if attachment in (None, NOT_GIVEN, '', dict()):
                attachment = None
        if send:
            recipients = regcomp.mailThemAll(
                subject, body, sender=sender, channels=chans,
                attachment=attachment)
            message = u'Mail byl odeslán'
        else:
            recipients = None
        return self.render(
            send=send,
            message=message, 
            subject=subject,
            body=body,
            channels=self.channels,
            sender=sender,
            recipients=recipients,
            form=form
        )


class MyHandler(web.Handler):
    
    def do_myhome(self):
        req = self.req
        registration = self.workspace['registration']
        forUser = registration.userByHome(req.params['name'])
        regUser = web.getUser(req.environ,'registered_user')
        if not forUser:
            raise web.NotFound()
        from lib4.odict import OrderedDict
        plugins = OrderedDict(
            (n, fact(name=n, req=req, forUser=forUser, **params)())
            for n, (fact,params) in registration.myHomePlugins.items()
        )
        return self.render(forUser=forUser, regUser=regUser, plugins=plugins)

    def do_myadmin(self):
        req = self.req
        ws = self.workspace
        registration = self.workspace['registration']
        forUser = web.getUser(req.environ,'registered_user')
        if not forUser:
            raise web.NotAllowed()
        from lib4.odict import OrderedDict
        plugins = OrderedDict(
            (n, fact(name=n, req=req, forUser=forUser, **params)())
            for n, (fact,params) in registration.myAdminPlugins.items()
        )
        return self.render(forUser=forUser, plugins=plugins)


class RegistrationMiddleware(web.CookieAuthenticationMiddleware):
    
    login_url = '/jeseniky/registration/login'
    login_handler = None
    logout_url = '/jeseniky/registration/logout'
    logout_handler = None
    auth_domain = AUTH_DOMAIN
    on_ports = []

    workspace = binding.Make(lambda: settings.workspace())

    @staticmethod
    def find_user(self, environ, login):
        return -self.workspace('''[
                    ru for ru in RegisteredUser
                        if ru.login == login
                       and ru.verified is not None
                       and ru.approved is not None
                       and ru.active == True
            ]''')
