from wm4.api import *
from adt4.serial import Serial

__all__ = ['Relation', 'RelationMaxweightbydst', 'RelationMaxweightbysrc',
           'EntityWithRelations', 'RelationsComp']


class EntityWithRelations(object):
    _rel_comp = binding.Make(lambda s: s._workspace['relations'])
    _relations = binding.Make(lambda s: s._rel_comp.forSrc(s))

binding.activateClass(EntityWithRelations)


@schema.Workspace.do_remove.after('obj in EntityWithRelations')
def do_remove_tag_objekt(ws, obj):
    cls = ws.getPath(obj.__class__)
    key = str(obj._oid)
    ws('''[query.Delete(o) for o in Relation
                            if (o.src_cls == cls and o.src_key == key)
                            or (o.dst_cls == cls and o.dst_key == key)
    ]''')




class Relation(schema.Entity):
    pkey = schema.Attribute(Serial)
    src_cls = schema.Attribute(str, schema.Required)
    src_key = schema.Attribute(unicode)
    kind = schema.Attribute(unicode)
    dst_cls = schema.Attribute(str, schema.Required)
    dst_key = schema.Attribute(unicode)
    weight = schema.Attribute(float, schema.Required, schema.Default(1.0))
    position = schema.Attribute(int, schema.Required, schema.Default(0))
    note = schema.Attribute(unicode)

    binding.metadata(schema.PrimaryKey('pkey'))

    src = binding.Make(lambda s: s._workspace.get(s.src_cls, s.src_key))
    dst = binding.Make(lambda s: s._workspace.get(s.dst_cls, s.dst_key))


    def make_attrs(self, obj, ws=None):
        if ws is None:
            ws = self._workspace
        cls = obj.__class__
        return {'cls':ws.getPath(cls), 'key':str(obj._oid)}

    def defaultValues(self, values):
        if 'src_cls' not in values and 'src_key' not in values \
        and 'src' in values:
            o = values['src']
            a = self.make_attrs(o, o._workspace)
            values['src_cls'] = a['cls']
            values['src_key'] = a['key']
        if 'dst_cls' not in values and 'dst_key' not in values \
        and 'dst' in values:
            o = values['dst']
            a = self.make_attrs(o, o._workspace)
            values['dst_cls'] = a['cls']
            values['dst_key'] = a['key']
        return super(Relation, self).defaultValues(values)


class RelationMaxweightbydst(Relation): pass
class RelationMaxweightbysrc(Relation): pass


class RelationsComp(schema.SchemaComponent):

    def forSrc(self, obj, klass=None, kind=None):
        cls = self.workspace.getPath(obj.__class__)
        key = str(obj._oid)
        return self._forSrc(cls, key, klass, kind)

    def _forSrc(self, cls, key, klass=None, kind=None):
        q = '''r for r in Relation
                  if r.src_cls == cls and r.src_key == key
                 and query.OrderBy(query.Desc(r.weight), r.position) '''
        if klass:
            q += 'and r.dst_cls == "%s" ' % self.workspace.getPath(klass)
        if kind:
            q += 'and r.kind=="%s" ' % kind
        return tuple(settings.workspace()('[%s]' % q))

    def forDst(self, obj, klass=None):
        cls = self.workspace.getPath(obj.__class__)
        key = str(obj._oid)
        return self._forDst(cls, key, klass)

    def _forDst(self, cls, key, klass=None):
        q = '''r for r in Relation
                  if r.dst_cls == cls and r.dst_key == key
                 and query.OrderBy(query.Desc(r.weight)) '''
        if klass:
            q = q + 'and r.src_cls == "%s"' % self.workspace.getPath(klass)
        return tuple(settings.workspace()('[%s]' % q))
