from wm4.api import *
from wm4.web.forms.api import *

from entities import *

__all__ = ['RelationItemCtrl', 'RelationsCtrl', 'RelationsWidget']

_ = i18n.MessageIDFactory('comp4.relations')



class RelationItemCtrl(makeFormCtrl(Relation), SequenceItemCtrl):
    
    ctrlType = Relation

    class src_cls(InputCtrl):
        ctrlRequired = True
        ctrlType = str
        ws = binding.Make(lambda: settings.workspace())
        def defaultOutput(self):
            return self['../..'].src_cls

    class src_key(InputCtrl):
        ctrlRequired = True
        ctrlType = unicode
        def defaultOutput(self):
            return self['../..'].src_key
    
    class description(InputCtrl):
        ctrlLabel = _(u'Object')
        ctrlType = unicode
        #ctrlReadOnly = True
        repr = binding.Make(
                lambda: config.Property('comp4.relations.object_description')())

        def obj(self):
            cls = self['../dst_cls'].oldValue
            key = self['../dst_key'].oldValue
            if cls in (None, NOT_GIVEN) or key in (None, NOT_GIVEN):
                return None
            return settings.workspace().get(cls, key)
        obj = binding.Make(obj)

        def defaultInput(self):
            return self.repr(self.obj)

        def commitSelf(self):
            return True

    class dst_cls(InputCtrl):
        ctrlVisibility = 'hidden'

    class dst_key(InputCtrl):
        ctrlVisibility = 'hidden'

    class popup(PopupCtrl):
        popupMap = dict(
            cls = '../dst_cls',
            key = '../dst_key',
            description = '../description',
        )

        def popupUrl(self):
            return self['../..'].popupUrl
        popupUrl = binding.Make(popupUrl)

        
    class weight(SelectCtrl):
        ctrlLabel = _(u'Weight')
        ctrlRequired = True
        selectOptions = [
            ('1', u'1', 1),
            ('2', u'2', 2),
            ('3', u'3', 3),
            ('4', u'4', 4),
            ('5', u'5', 5),
        ]

    class position(InputCtrl):
        ctrlType = int
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')
        def defaultOutput(self):
            return self.ctrlParent.itemPosition

    class note(TextAreaCtrl):
        ctrlLabel = _(u'Note')
        rows = 2
        cols = 30



class RelationsCtrl(SequenceCtrl):
    itemClass = binding.Constant(RelationItemCtrl)
    ctrlWidget = web.FORM_WIDGET('input.relations')

    popupUrl = binding.Require()

    ws = binding.Make(lambda: settings.workspace())
    src_cls = binding.Make(lambda s: s.ws.getPath(s['..'].newValue.__class__))
    src_key = binding.Make(lambda s: s['..'].newValue._oid)

    def updateContainer(self, removed):
        ws = settings.workspace()
        for i in removed:
            ws.remove(self.oldValue[i])





class RelationsWidget(FormWidget):

    def view(self, state, felem, req):
        return web.parseSrc('''<span>
          <?py repr = config.Property('comp4.relations.object_description')() ?>
          <ul>
            <li form:repeat="">
                <a href="${req.get_url(form.oldValue.dst)}"
                    >${repr(form.oldValue.dst)}</a> ($form.oldValue.weight)
            </li>
          </ul>
        </span>''')

    def edit(self, state, felem, req):
        return web.parseSrc('''<div class="relations4-widget">
          <ol>
            <li form:repeat="">
                <div class="wm4-form-fieldset-actions">
                    <input form:button="moveUp" class="button-move-up"/>
                    <input form:button="moveDown" class="button-move-down"/>
                    <input form:button="removeItem" class="button-remove-item"/>
                </div>
                <div>
                    <input form:input="dst_cls" />
                    <input form:input="dst_key" />

                    <label form:label="description" />
                    <input form:input="description" />
                    <input form:input="popup" />
                    
                    <label form:label="weight" />
                    <input form:input="weight" />

                    <div>
                        <label form:label="note" />
                        <input form:input="note" />
                    </div>
                </div>
            </li>
          </ol>
          <input form:input="addItem"/>
        </div>''')

