# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms.api import *

from comp4.cms.pagelets import Pagelet
from comp4.cms.controllers import (
    PageletCtrl, OptionalSingleCtrl, ChildrenCtrl, pageletCtrl)
from comp4.cms.widgets import PageletWidget
from comp4.cms.facades import ChildrenProxy

from xml4 import xhtml as xh

from controllers import CategoryCtrl, GoodsCtrl


__all__ = [
    'ShopGoods', 'ShopGoodsCtrl', 'ShopGoodsWidget',
    'ShopCategories', 'ShopCategoriesCtrl', 'ShopCategoriesWidget',
    'ShopListing', 'ShopListingCtrl', 'ShopListingWidget',
]



class ShopGoodsGoods(Pagelet):
    plName = u'Zboží'
    
    def goods(self):
        return settings.workspace().get(
            'shop/Goods', self.value, default=None)
    goods = binding.Make(goods)

class ShopGoodsGoodsCtrl(PageletCtrl):
    featureCommiting = {'*': 'before'}
    ctrlType = ShopGoodsGoods

    class value(GoodsCtrl):
        ctrlRequired = True
        def defaultOutput(self):
            return self['pkey'].newValue

        class pkey(GoodsCtrl.pkey):
            def defaultInput(self):
                old = self['../'].oldValue
                if not old or old in (NOT_GIVEN, None):
                    return ''
                return old

        class name(GoodsCtrl.name):
            def defaultInput(self):
                old = self['../'].oldValue
                if not old or old in (NOT_GIVEN, None):
                    return ''
                return settings.workspace().get('shop/Goods', int(old)).name

class ShopGoodsGoodsWidget(PageletWidget):
    pass

class ShopGoods(Pagelet):
    plName = u'Odkazy na zboží z eshopu'

    goods = ChildrenProxy('cms', 'pagelet.shop_goods.goods')


class ShopGoodsCtrl(PageletCtrl):
    featureCommiting = {'*':'before', 'goods':'after'}

    ctrlType = ShopGoods
    defaultPagelets = ()

    class goods(ChildrenCtrl):
        class addItemCount(SelectCtrl):
            ctrlType = int
            ctrlRequired = True
            selectOptions = [(str(i),str(i),i) for i in [1,2,3,4,5,7,10,15,20]]
            def defaultInput(self):
                return '1'
            def defaultOutput(self):
                return 1

        def addNewItem(self):
            kind, name = 'cms', 'pagelet.shop_goods.goods'
            fact = pageletCtrl(self.partition, kind, name)()
            return fact(
                self.ctrlReq, str(self.itemCount), self, ekind=kind, ename=name
            )

class ShopGoodsWidget(PageletWidget):

    def view(self, state, felem, req):
        cl = xh.ul()
        for c in felem.oldValue.goods:
            if c.goods is None:
                continue
            li = xh.li(xh.a(c.goods.name, href=req.get_url(c.goods)))
            cl.append(li)
        return cl

    def edit(self, state, felem, req):
        return web.parseSrc('''
        <div>
            <ol class="${form.css+'-items'}">
                <li form:repeat="goods" class="$form.css">
                    <div class="wm4-form-fieldset-actions">
                    <input form:button="moveUp" class="button-move-up"/>
                    <input form:button="moveDown" class="button-move-down"/>
                    <input form:button="removeItem" class="button-remove-item"/>
                    </div>
                    <input form:input="value"/>
                    <input form:errors="value"/>
                </li>
            </ol>
            <div>
                <input form:input="goods/addItemCount" />
                <input form:button="goods/addItem"/>
            </div>
        </div>''')















class ShopCategoriesCategory(Pagelet):
    
    def category(self):
        return settings.workspace().get(
            'shop/Category', self.value, default=None)
    category = binding.Make(category)

class ShopCategoriesCategoryCtrl(PageletCtrl):
    ctrlType = ShopCategoriesCategory
    class value(InputCtrl):
        ctrlRequired = True

class ShopCategoriesCategoryWidget(PageletWidget):
    pass

class ShopCategories(Pagelet):
    plName = u'Výpis kategorií z eshopu'

    def categories(self):
        return [
            e for e in self.children
            if e.kind=='cms' and e.name=='pagelet.shop_categories.category'
        ]
    categories = binding.Make(categories)

class ShopCategoriesCtrl(PageletCtrl):
    ctrlType = ShopCategories
    featureCommiting = {'*':'before', 'categories':'after'}
    defaultPagelets = ()

    class categories(ChildrenCtrl):

        def addNewItem(self):
            kind, name = 'cms', 'pagelet.shop_categories.category'
            fact = pageletCtrl(self.partition, kind, name)()
            return fact(
                self.ctrlReq, str(self.itemCount), self, ekind=kind, ename=name
            )

class ShopCategoriesWidget(PageletWidget):

    def view(self, state, felem, req):
        cl = xh.ul()
        for c in felem.oldValue.categories:
            if c.category is None:
                continue
            li = xh.li(xh.a(c.category.name, href=req.get_url(c.category)))
            if c.category.children:
                cl2 = xh.ul()
                for c2 in c.category.children:
                    cl2.append(xh.li(xh.a(c2.name, href=req.get_url(c2))))
                li.append(cl2)
            cl.append(li)
        return cl

    def edit(self, state, felem, req):
        return web.parseSrc('''
        <div>
            <ol class="${form.css+'-items'}">
                <li form:repeat="categories" class="$form.css">
                    <div class="wm4-form-fieldset-actions">
                    <input form:button="moveUp" class="button-move-up"/>
                    <input form:button="moveDown" class="button-move-down"/>
                    <input form:button="removeItem" class="button-remove-item"/>
                    </div>
                    <input form:input="value"/>
                    <input form:errors="value"/>
                </li>
            </ol>
            <div>
                <input form:button="categories/addItem"/>
            </div>
        </div>''')















class ShopListing(Pagelet):
    plName = u'Výpis zboží z eshopu'

    title = binding.Make(
        lambda s: s.findElement('cms','pagelet.shop_listing.title'))
    listing_kind = binding.Make(
        lambda s: s.findElement('cms','pagelet.shop_listing.kind'))



class ShopListingCtrl(PageletCtrl):

    class value(CategoryCtrl):
        ctrlRequired = True
        def defaultOutput(self):
            return self['pkey'].newValue

        class name(CategoryCtrl.name):
            def defaultInput(self):
                old = self['../pkey'].oldValue
                if not old or old in (NOT_GIVEN, None):
                    return ''
                return settings.workspace().get(ShopCategory, old).name
    
    class title(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.shop_listing.title')

    class listing_kind(OptionalSingleCtrl):
        ename = ClientState(default=lambda s,o,a:'pagelet.shop_listing.kind')
        class value(SelectCtrl):
            selectOptions = [
                ('tips', 'tips (default)', 'tips'),
                ('all',  'all', 'all'),
            ]


class ShopListingWidget(PageletWidget):

    def view(self, state, felem, req):
        ws = settings.workspace()
        shop = ws['shop']
        return xh.div('ShopListing')

    def edit(self, state, felem, req):
        return web.parseSrc('''
            <div>
              <table>
                <tr>
                  <th>Kategorie zboží</th>
                  <td><input form:input="value" /></td>
                </tr>
                <tr>
                  <th>Titulek</th>
                  <td>
                    <input form:input="title/value" size="30" />
                    <div>Pokud nevyplníte, použije se název kategorie</div>
                  </td>
                </tr>
                <tr>
                  <th>Způsob výpisu</th>
                  <td><input form:input="listing_kind/value" /></td>
                </tr>
              </table>
            </div>
        ''')
