#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.running import commands, options

from entities import ShopGoods
from fulltext import updateFulltext
from utils import *

__all__ = [
    'Params2INI', 'TidyDescription', 'ExtractShortDescription', 'Fulltext',
]

class Params2INI(commands.ConfiguredCommandLineApp):
    
    def main(self):
        super(Params2INI, self).main()
        print "[comp4.shop]"
        print "goods_params = [%s]" % ', '.join(
                repr(p) for p in self.parsed_args)
        print
        for p in self.parsed_args:
            t = p.capitalize().replace('_', ' ')
            print "[Plugin comp4.shop.goods_param:%s]" % p
            print "title = u'%s'"  % t
            print

class TidyDescription(commands.ConfiguredCommandLineApp):
    '''Check and repair XHTML goods descriptions'''

    def main(self):
        super(TidyDescription, self).main()
        ws = settings.workspace()
        storage.begin()
        for goods in ws('[o for o in ShopGoods if o.description is not None]'):
            log = []
            try:
                html = tidy_description(goods.description, log)
            except:
                print goods.pkey, log
                print goods.description
                raise
            if html != goods.description:
                log.append('edit')
                print goods.pkey, ' '.join(log)
                goods.description = html
                ws.flush(goods)
            elif log:
                print goods.pkey, ' '.join(log)
        storage.commit()
    
class ExtractShortDescription(commands.ConfiguredCommandLineApp):

    def main(self):
        super(ExtractShortDescription, self).main()
        ws = settings.workspace()
        storage.begin()
        for goods in ws('[o for o in ShopGoods if o.description is not None]'):
            short = extract_short_description(goods.description)
            if goods.short != short:
                print goods.pkey, short
                goods.short = short
                ws.flush(goods)
        storage.commit()
    


class Fulltext(commands.ConfiguredCommandLineApp):
    '''Reindex all goods in fulltext database.'''

    def main(self):
        super(Fulltext, self).main()
        ws = settings.workspace()
        storage.begin()
        ftent = [ShopGoods]
        for cls in ftent:
            print cls
            for obj in ws.find(cls):
                print obj._oid
                updateFulltext(obj)
                #ws.flush(obj)
        storage.commit()

