import sys, pgdb, csv, optparse, encodings.utf_8
from wm4.api import *
from comp4.shop.api import *


def get_opts():
    usage = 'usage: %prog [options] APPLICATION'
    parser = optparse.OptionParser(usage=usage)

    parser.add_option('--encoding', default='utf-8')
    parser.add_option('--params', default=None)
    parser.add_option('--catid', type='int', default=None)
    parser.add_option('--roles', default=None)

    opts, args = parser.parse_args()
    if len(args) != 1:
        parser.error('application name needed')

    opts.app_name = args[0]
    config.setupApp(opts.app_name)

    opts.con = settings.storage_connection()
    opts.ws = settings.workspace()
    opts.shop = opts.ws['shop']

    if opts.params is None:
        opts.params =  opts.shop.goodsParams.keys()
    else:
        opts.params = [p.strip() for p in opts.params.split(',') if p.strip()]

    if opts.roles is None:
        opts.roles = [r for r,_ in opts.shop.person_roles]
    else:
        opts.roles = [p.strip() for p in opts.roles.split(',') if p.strip()]

    return opts


opts = get_opts()
#print opts
#sys.exit(0)
storage.begin()


if opts.catid:
    q = '''[obj for obj in ShopGoods
                 if obj.pkey in [
                    gc.goods.pkey
                        for gc in ShopGoodsCategory
                        for tc in ShopCategory.treeCache
                         if gc.category.pkey == tc.end
                        and tc.begin == opts.catid
                    ]
                and obj.deleted == False
                and query.OrderBy(obj.name)
            ]'''
else:
    q = '''[obj for obj in ShopGoods
                 if obj.deleted == False
                and query.OrderBy(obj.name)
           ]'''

w = csv.writer(sys.stdout)


header = [
    'pkey', 'typ', 'hidden', 'name', 'price', 'description', 'manufacturer'
]

header.extend(['person:%s' % r for r in opts.roles])
header.extend(['param:%s' % p for p in opts.params])
header.extend(['categories', 'relations', 'tip'])

w.writerow(header)

for _i, g in enumerate(opts.ws(q)):
    #if _i > 100:
    #    break
    row = [
        g.pkey, g.typ, g.hidden and 'A' or 'N', g.name.encode('utf-8'),
        g.price,
        g.description and g.description.encode('utf-8') or None,
        g.manufacturer and g.manufacturer.name.encode('utf-8') or None,
    ]
    # persons
    for r in opts.roles:
        row.append('; '.join([
            '%s (%s)' % (p.name, p.birth_date or '')
            for p in getattr(g.persons, r)
        ]).encode('utf-8'))
    # parametry
    for p in opts.params:
        v = getattr(g.params, p)
        if isinstance(v, unicode):
            v = v.encode('utf-8')
        row.append(v)
    # categories
    row.append(';'.join([
            ('%d %s' % (c.pkey, c.name)).encode('utf-8') for c in g.categories
    ]))
    # relations
    row.append(','.join(str(r.pkey) for r in g.related))
    row.append(','.join(str(t.category.pkey) for t in g._tips))
    w.writerow(row)
