#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, pgdb, re, optparse, encodings.utf_8
from wm4.api import *
from lib4.csvutils import *
from lib4.symbols import *
from lib4.timeutils import *
from sql4.utils import sql_escape, sql_name

#sys.stdout = encodings.utf_8.StreamWriter(sys.stdout)


def getOpts():
    usage = 'usage: %prog [options] DATABASE CSV_FILE'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('--encoding', default='utf-8')
    opts, args = parser.parse_args()
    if len(args) != 2:
        parser.error('DATABASE and CSV file needed')
    opts.database, opts.csv_file = args
    opts.con = pgdb.connect(database=opts.database)
    return opts


class Manufacturer:
    pkey = None
    name = None
    def __init__(self, **kw):
        self.__dict__.update(kw)
    def __repr__(self):
        return (
            u'Manufacturer(%s,%s)' % (self.pkey, self.name)
        ).encode('utf-8')

class Person:
    pkey = None
    name = None
    birth_year = None
    birth_month = None
    birth_day = None
    death_year = None
    death_month = None
    death_day = None
    def __init__(self, **kw):
        self.__dict__.update(kw)
    def __repr__(self):
        return (
            u'Person(%s,%s,%s)' % (self.pkey, self.name, self.birth_year)
        ).encode('utf-8')

class Goods(binding.Component):
    __metaclass__ = binding.Activator
    _lineno = None
    _fields = dict(
        pkey = int,
        name = unicode,
        price = float,
        typ = unicode,
        #hidden = bool,
    )
    manufacturer = NOT_GIVEN
    persons = binding.Make(dict)
    parameters = binding.Make(dict)
    categories = binding.Make(set)

    def __init__(self, **kw):
        for a,t in self._fields.items():
            v = kw.get(a, NOT_GIVEN)
            if v is not NOT_GIVEN and v is not None:
                if not isinstance(v, t):
                    v = t(v)
            setattr(self, a, v)


def manufacturerDict(opts):
    cur = opts.con.cursor()
    cur.execute('SELECT pkey, name FROM shop_manufacturer ORDER BY pkey')
    byKey = {}
    byName = {}
    for p, n in cur.fetchall():
        o = Manufacturer(pkey=int(p), name=unicode(n, 'utf-8'))
        byKey[o.pkey] = o
        byName[o.name] = o
    cur.close()
    return byKey, byName


def personDict(opts):
    cur = opts.con.cursor()
    cur.execute('SELECT * FROM shop_person ORDER BY pkey')
    attrs = [d[0] for d in cur.description]
    byKey = {}
    bySig = {}
    for row in cur.fetchall():
        row = dict(zip(attrs, row))
        o = Person(
            pkey = int(row['pkey']),
            name = unicode(row['name'],'utf-8'),
            birth_year = row['birth_year'],
            birth_month = row['birth_month'],
            birth_day = row['birth_day'],
            death_year = row['death_year'],
            death_month = row['death_month'],
            death_day = row['death_day'],
        )
        o.sig = (o.name, o.birth_year)
        byKey[o.pkey] = o
        bySig[o.sig] = o
    cur.close()
    return byKey, bySig


def categoryDict(opts):
    cur = opts.con.cursor()
    cur.execute('SELECT pkey,name FROM shop_category')
    return dict((p,n) for p,n in cur.fetchall())


goods_types = [
    'kniha', 'kniha_cd', 'kniha_mc', 'kniha_disketa', 'kniha_dvd',
    'kniha_cd_rom', 'kniha_priloha', 'casopis_periodikum',
    'casopis_cd_dvd_cd_rom', 'kniha_cizojazycna', 'kniha_vicejazycna',
    'kniha_s_hrou_hrackou', 'hudebnina', 'zvukova_kniha', 'cd', 'dvd',
    'vhs', 'mc', 'hra_hracka', 'mapa', 'nastenna_mapa', 'globus', 'karty',
    'vyukova_tabulka', 'pohlednice', 'kalendar', 'diar', 'ostatni',
]


def parseCSV(opts):

    def normalizeRecord(rec):
        for k,v in rec.items():
            if v in ('null', 'Memo', 'memo'):
                rec[k] = v = None
            elif v:
                rec[k] = v = v.strip() or None
            else:
                rec[k] = v = None
        return rec
    
    reader = LazyRecordReader(
        opts.csv_file, encoding=opts.encoding, mangle_fields=False
    )
    opts.params = [
        f[6:] for f in reader.fields if f.startswith('param:')
    ]
    records = []
    for idx, rec in enumerate(reader):
        rec = normalizeRecord(rec)

        goods = Goods(**dict(
            (ga,rec.get(ga,NOT_GIVEN)) for ga in (
                'pkey','name','hidden','price','typ')
        ))
        goods._lineno = idx + 2
        if goods.typ and goods.typ is not NOT_GIVEN:
            goods.typ = sql_name(goods.typ)
            goods.typ = {
                'hdebnina': 'hudebnina',
                'kniha_brozura': 'kniha',
                'cd_rom': 'kniha_cd_rom', 
            }.get(goods.typ, goods.typ)
            assert goods.typ in goods_types, goods.typ
        else:
            goods.typ = 'kniha'
        records.append(goods)

        # parameters
        for a,v in rec.items():
            if a.startswith('param:'):
                goods.parameters[a[6:]] = v

        # manufacturer
        m = rec.get('manufacturer', NOT_GIVEN)
        if m is not NOT_GIVEN:
            if m is None:
                i = None
            else:
                i = opts.manufacturerByName.get(m)
                if i is None:
                    i = opts.manufacturerByName[m] = Manufacturer(name=m)
                goods.manufacturer = i

        # persons
        for a,v in rec.items():
            if a.startswith('person:'):
                if not v:
                    continue
                role = a[7:]
                persons = []
                for p in v.split(';'):
                    try:
                        p = p.strip()
                        m = re.match('^(.*)\\((.+?)\\)$', p)
                        if m:
                            n = m.group(1).strip()
                            y = int(
                              (m.group(2).split('-')[0].split('.')[-1]).strip())
                        else:
                            n = p
                            y = None
                        sig = (n,y)
                        i = opts.personBySig.get(sig)
                        if i is None:
                            i = Person(name=n, birth_year=y)
                            opts.personBySig[sig] = i
                        persons.append(i)
                    except:
                        pass
                goods.persons[role] = persons

        # XXX categories
        for a in ('cat1', 'cat2', 'cat3', 'cat4'):
            v = rec[a]
            if not v: continue
            v = v.strip()
            if not v: continue
            m = re.search('^(\\d+)', v)
            if not m: continue
            c = int(v[m.start():m.end()])
            assert c in opts.categories
            goods.categories.add(c)
    return records


def esc(val):
    if val is None:
        return 'NULL'
    if isinstance(val, basestring):
        val = val.encode('utf-8')
    else:
        val = str(val)
    return "'%s'" % sql_escape(val)

def insertManufacturers(opts):
    print "=" * 50
    print 'Synchronizace vyrobcu'
    print "=" * 50
    cur = opts.con.cursor()
    for m in opts.manufacturerByName.values():
        if m.pkey is not None:
            continue
        print m.name
        cur.execute("SELECT NEXTVAL('shop_manufacturer_pkey_seq')")
        m.pkey = cur.fetchone()[0]
        cur.execute(
            "INSERT INTO shop_manufacturer (pkey, name) VALUES (%d,%s)" % (
            m.pkey, esc(m.name)
        ))
    cur.close()
    opts.con.commit()


def insertPersons(opts):
    print "=" * 50
    print 'Synchronizace osob'
    print "=" * 50
    cur = opts.con.cursor()
    for p in opts.personBySig.values():
        if p.pkey is not None:
            continue
        print p.name, p.birth_year
        cur.execute("SELECT NEXTVAL('shop_person_pkey_seq')")
        p.pkey = cur.fetchone()[0]
        cur.execute('''INSERT INTO shop_person (
                pkey, name,
                birth_day, birth_month, birth_year,
                death_day, death_month, death_year
            ) VALUES (%d, %s, %s, %s, %s, %s, %s, %s)''' % (
                p.pkey, esc(p.name),
                esc(p.birth_day),esc(p.birth_month),esc(p.birth_year),
                esc(p.death_day),esc(p.death_month),esc(p.death_year),
            ))
    cur.close()
    opts.con.commit()


def newGoods(opts, g):
    cur = opts.con.cursor()
    print '%06d %s' % (g.pkey, g.name)
    print '  insert goods[%s]' % g.pkey
    q = ("INSERT INTO shop_goods (pkey,name,description,price,hidden) "
         "VALUES (%d, %s, NULL, NULL, False)") % (g.pkey, esc(g.name))
    cur.execute(q)
    cur.close()
    opts.con.commit()
    #updateGoods(opts, g)


def deleteGoods(opts, rec):
    cur = opts.con.cursor()
    id = rec['id']
    print '%06d %s' % (id, rec['nazev'])
    cur.execute('SELECT name FROM shop_goods WHERE pkey=%d' % id)
    if not cur.fetchall():
        cur.close()
        return
    print '  delete goods[%s]' % id
    cur.execute('DELETE FROM shop_goods WHERE pkey=%d' % id)
    cur.execute('DELETE FROM shop_goods_param WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_goods_category WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_goods_image WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_goods_related WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_goods_tip WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_invoice_item WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_basket_item WHERE goods=%d' % id)
    cur.execute('DELETE FROM shop_news WHERE goods=%d' % id)
    cur.close()
    #opts.con.commit()




def updateGoods(opts, g):
    pk = g.pkey

    head = []
    def log(msg):
        if not head:
            print '%06d %s' % (pk, g.name.encode('utf-8'))
            head.append(1)
        print '  ' + msg.encode('utf-8')
    
    cur = opts.con.cursor()

    # name, manufacturer
    cur.execute('''
        SELECT g.typ, g.name, g.manufacturer, g.price, g.hidden, i.stamp
          FROM shop_goods g 
               LEFT JOIN shop_import i ON (g.pkey = i.pkey)
         WHERE g.pkey=%d
    ''' % pk)
    old = cur.fetchone()
    if old is None:
        return newGoods(opts, g)
    typ, name, manufacturer, price, hidden, stamp = old
    name = unicode(name, 'utf-8')
    upd = {}
    if g.typ != typ:
        log(u'update goods.typ %s -> %s' % (typ, g.typ))
        upd['typ'] = esc(g.typ)
    if g.name is not NOT_GIVEN and name != g.name:
        log(u'update goods.name %s -> %s' % (name, g.name))
        upd['name'] = esc(g.name)
    if g.price is not NOT_GIVEN and price != g.price:
        log(u'update goods.price %s -> %s' % (price, g.price))
        upd['price'] = esc('%.2f' % g.price)
    if g.manufacturer is NOT_GIVEN:
        pass
    elif g.manufacturer is None and manufacturer is None:
        pass
    elif g.manufacturer is None:
        log(u'update goods.manufacturer %s -> NULL' % manufacturer)
        upd['manufacturer'] = 'NULL'
    elif manufacturer is None:
        log(u'update goods.manufacturer NULL -> %s' % g.manufacturer.pkey)
        upd['manufacturer'] = str(g.manufacturer.pkey)
    elif g.manufacturer.pkey != manufacturer:
        log(u'update goods.manufacturer %s -> %s' % (
            manufacturer, g.manufacturer.pkey))
        upd['manufacturer'] = str(g.manufacturer.pkey)
    else:
        assert manufacturer == g.manufacturer.pkey
    if upd:
        cur.execute('UPDATE shop_goods SET %s WHERE pkey=%d' % (
            ', '.join('%s=%s' % (c,v) for c,v in upd.items()),
            g.pkey
        ))

    # parameters
    cur.execute(
        '''SELECT pkey, param, value
             FROM shop_goods_param
            WHERE goods = %d AND param IN (%s)
            ORDER BY position''' % (
                pk, ','.join(esc(p) for p in opts.params)
    ))
    parameters = {}
    for k,p,v in cur.fetchall():
        v = v and unicode(v, 'utf-8') or None
        parameters[p] = k, v
    for p in opts.params:
        k, o = parameters.get(p, (None, None))
        n = g.parameters.get(p)
        if k is not None and o in (None, ''):
            log(u'delete null param[%s] %s' % (p, o))
            cur.execute('DELETE FROM shop_goods_param WHERE pkey=%d' % k)
        if o == n:
            continue
        if n in (None, u''):
            log(u'delete param[%s] %s' % (p, o))
            cur.execute('DELETE FROM shop_goods_param WHERE pkey=%d' % k)
        elif k is None:
            log(u'insert param[%s] %s' % (p, n))
            cur.execute((
                'INSERT INTO shop_goods_param (goods, param, value) '
                'VALUES (%d, %s, %s)') % (pk, esc(p), esc(n))
            )
        else:
            log(u'update param[%s] %s -> %s' % (p, o, n))
            cur.execute('UPDATE shop_goods_param SET value=%s WHERE pkey=%d' % (
                esc(n), k
            ))

    # persons
    cur.execute(
        '''SELECT role, person
             FROM shop_goods_person
            WHERE goods = %d
            ORDER BY position''' % pk)
    persons = {}
    for r, p in cur.fetchall():
        persons.setdefault(r, set()).add(p)
    for r, pl in g.persons.items():
        new = set(p.pkey for p in pl)
        old = persons.get(r, set())
        for p in old - new:
            log(u'remove person %d' % p)
            cur.execute('''DELETE FROM shop_goods_person
                WHERE goods=%d AND role=%s AND person=%d''' % (
                pk, esc(r), p
            ))
        for p in new - old:
            log(u'insert person %d' % p)
            cur.execute((
                'INSERT INTO shop_goods_person (goods, person, role) '
                'VALUES (%d, %d, %s)') % (pk, p, esc(r))
            )

    # catetories

    cur.execute('SELECT category FROM shop_goods_category WHERE goods = %d' %pk)
    oldcat = set(c for c, in cur.fetchall())
    # insert
    for c in (g.categories - (g.categories & oldcat)):
        log(u'insert category %d' % c)
        cur.execute('''INSERT INTO shop_goods_category (goods,category)
                       VALUES (%d, %d)''' % (pk, c))
    # delete
    for c in (oldcat - (g.categories & oldcat)):
        log(u'delete category %d' % c)
        cur.execute('''DELETE FROM shop_goods_category
                        WHERE goods=%d AND category=%d''' % (pk, c))
    # stamp
    if not stamp:
        cur.execute('INSERT INTO shop_import (pkey) VALUES (%d)' % pk)

    cur.close()
    opts.con.commit()
    return


        
        

opts = getOpts()
opts.manufacturerByKey, opts.manufacturerByName = manufacturerDict(opts)
opts.personByKey, opts.personBySig = personDict(opts)
opts.categories = categoryDict(opts)
opts.goods = parseCSV(opts)
insertManufacturers(opts)
insertPersons(opts)

print "=" * 50
print 'Synchronizace zbozi'
print "=" * 50
for g in opts.goods:
    updateGoods(opts, g)
