#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import pgdb
import re
import optparse
import encodings.utf_8
import app4
from itertools import count
from wm4.api import *
from lib4.csvutils import *
from lib4.symbols import *
from lib4.timeutils import *
from sql4.utils import sql_escape
from unicodedata import normalize

sys.stdout = encodings.utf_8.StreamWriter(sys.stdout)


def getOpts():
    usage = 'usage: %prog [options] CSV_FILE'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('--app', default='portaluh')
    parser.add_option('--encoding', default='windows-1250')
    opts, args = parser.parse_args()
    if len(args) != 1:
        parser.error('CSV file needed')
    opts.csv_file = args[0]

    app4.config_app(opts.app)
    opts.url = naming.toName(settings.storage_url())
    opts.con = pgdb.connect(
        database = opts.url.database,
        user = opts.url.user,
        password = opts.url.password,
        host = opts.url.host,
    )
    return opts

opts = getOpts()

def esc(val):
    if val is None:
        return 'NULL'
    if isinstance(val, basestring):
        val = val.encode('utf-8')
    else:
        val = str(val)
    return "'%s'" % val.replace("'", "''")


class Manufacturer:
    pkey = None
    name = None

    def __init__(self, **kw):
        self.__dict__.update(kw)

    @classmethod
    def get(klass, name):
        if not name:
            return None
        obj = klass.byName.get(name.lower(), None)
        #if obj is None:
        #    obj = klass.byName[name.lower()] = klass(name=name)
        return obj

    @classmethod
    def existing(klass):
        cur = opts.con.cursor()
        cur.execute('SELECT pkey, name FROM shop_manufacturer')
        byKey = {}
        byName = {}
        for p, n in cur.fetchall():
            o = Manufacturer(
                    pkey=int(p), name=normalize('NFKC',unicode(n, 'utf-8'))
            )
            byKey[o.pkey] = byName[o.name.lower()] = o
        cur.close()
        return byKey, byName
   
    @classmethod
    def missing(klass):
        cur = opts.con.cursor()
        for manuf in sorted(Manufacturer.byName.values(), key=lambda s:s.name):
            if manuf.pkey is not None:
                continue
            print 'NEW MANUFACTURER', manuf.name
            cur.execute("SELECT NEXTVAL('shop_manufacturer_pkey_seq')")
            manuf.pkey = int(cur.fetchone()[0])
            cur.execute(
                "INSERT INTO shop_manufacturer (pkey, name) VALUES (%s,%s)" % (
                manuf.pkey, esc(manuf.name)
            ))
        cur.close()
        opts.con.commit()

Manufacturer.byKey, Manufacturer.byName = Manufacturer.existing()


class Person:
    pkey = None
    name = None

    def __init__(self, **kw):
        self.__dict__.update(kw)

    @classmethod
    def get(klass, name):
        if not name:
            return None
        obj = klass.byName.get(name.lower(), None)
        #if obj is None:
        #    obj = klass.byName[name.lower()] = klass(name=name)
        return obj

    @classmethod
    def existing(klass):
        cur = opts.con.cursor()
        cur.execute('SELECT * FROM shop_person')
        attrs = [d[0] for d in cur.description]
        byKey = {}
        byName = {}
        for row in cur.fetchall():
            row = dict(zip(attrs, row))
            o = Person(
                pkey = int(row['pkey']),
                name = normalize('NFKC', unicode(row['name'],'utf-8')),
            )
            byKey[o.pkey] = byName[o.name.lower()] = o
        cur.close()
        return byKey, byName

    @classmethod
    def missing(klass):
        cur = opts.con.cursor()
        for pers in sorted(Person.byName.values(), key=lambda s:s.name):
            if pers.pkey is not None:
                continue
            print 'NEW PERSON', pers.name
            cur.execute("SELECT NEXTVAL('shop_person_pkey_seq')")
            pers.pkey = int(cur.fetchone()[0])
            cur.execute(
                "INSERT INTO shop_person (pkey, name) VALUES (%s,%s)" % (
                pers.pkey, esc(pers.name)
            ))
        cur.close()
        opts.con.commit()


Person.byKey, Person.byName = Person.existing()


# Mapa kategorii
def Categories():
    cur = opts.con.cursor()
    cur.execute('''
            SELECT s.nazev_soumar, c.pkey
             FROM shop_soumar_categories s
       INNER JOIN shop_category c ON s.id_web = c.pkey
    ''')
    return dict(
        (normalize('NFKC',unicode(n,'utf-8')).lower().strip(),p)
        for n,p in cur.fetchall()
    )

Categories = Categories()


class Goods(binding.Component):

    __metaclass__ = binding.Activator

    pkey         = None
    name         = None
    manufacturer = None
    soumar       = None
    abra         = None
    price_common = None
    price        = None
    autor        = None
    parameters   = binding.Make(dict)
    category     = None
    note         = None

FIELDS = (
    'soumar', 'name', 'person:autor', 'mnozstvi', 'price_common', 'param:isbn',
    'manufacturer', 'param:ean', 'category', 'abra'
)
    


# Rozparsuj CSV
goods_list = []
for idx, rec in enumerate(
    LazyRecordReader(
        opts.csv_file,
        fields     = FIELDS,
        encoding   = opts.encoding,
        delimiter  = '\t',
)):
    price_common = rec['price_common']
    if price_common is not None:
        price_common = float(price_common.replace(',','.'))
    if price_common > 10:
        price = int(price_common * 0.9)
    else:
        price = price_common

    cat = rec['category']
    if cat is not None:
        cat = cat.lower().strip()
        cat = Categories.get(cat, None)
        
    goods = Goods(
        name = rec['name'], 
        price_common = price_common,
        price = price,
        soumar = rec['soumar'],
        abra = rec['abra'],
        manufacturer = Manufacturer.get(rec['manufacturer']),
        autor = Person.get(rec['person:autor']),
        category = cat,
        note = 'autor: %s\nnakladatel: %s' % (
            rec['person:autor'] or '', rec['manufacturer'] or '')
    )
    for k,v in rec.items():
        if not k.startswith('param:'):
            continue
        goods.parameters[k[6:]] = v
    goods_list.append(goods)




# Zaved chybejici osoby a vyrobce
#Person.missing()
#Manufacturer.missing()

# Zaved nove zbozi
for goods in goods_list:
    cur = opts.con.cursor()

    if goods.soumar or goods.abra:
        cond = []
        if goods.soumar:
            cond.append("soumar='%s'" % sql_escape(goods.soumar))
        if goods.abra:
            cond.append("abra='%s'" % sql_escape(goods.abra))
        q = 'SELECT goods FROM shop_sync WHERE %s' % (' OR '.join(cond))
        cur.execute(q)
        if cur.rowcount > 0:
            old = cur.fetchone()[0]
            print 'OLD %s (soumar=%r, abra=%r)' % (old,goods.soumar,goods.abra)
            q = "UPDATE shop_sync SET sync=CURRENT_DATE WHERE goods=%d" % old
            cur.execute(q)
            opts.con.commit()
            continue

    cur.execute("SELECT NEXTVAL('shop_goods_pkey_seq')")
    goods.pkey = pkey = int(cur.fetchone()[0])
    print 'NEW %s (soumar=%r, abra=%r)' % (goods.pkey, goods.soumar, goods.abra)
    m = goods.manufacturer
    if m is not None:
        m = m.pkey
    print '  name = %s' % goods.name
    print '  price = %s' % goods.price
    print '  price_common = %s' % goods.price_common
    print '  manufacturer = %s' % m
    cur.execute((
      "INSERT INTO shop_goods (pkey, name, price, price_common, "
      "manufacturer, hidden, typ, note) "
      "VALUES (%d, %s, %s, %s, %s, false, 'kniha', %s)") % (
      pkey, esc(goods.name), esc(goods.price), esc(goods.price_common), esc(m),
      esc(goods.note),
    ))
    cur.execute(
        "INSERT INTO shop_sync (goods, soumar, abra, sync) VALUES (%s, %s, %s, CURRENT_DATE)" % (
        pkey, esc(goods.soumar), esc(goods.abra), 
    ))
    if goods.autor is not None:
        print '  autor = %s' % goods.autor.name
        cur.execute((
            "INSERT INTO shop_goods_person (goods,person,role) "
            "VALUES (%d, %d, 'autor')") % (pkey, goods.autor.pkey)
        )
    else:
        print '  autor = None'

    for k,v in goods.parameters.items():
        if v is not None:
            print '  %s = %s' % (k,v)
            cur.execute((
                "INSERT INTO shop_goods_param (goods, param, value) "
                "VALUES (%d, %s, %s)") % (pkey, esc(k), esc(v))
            )
    if goods.category is not None:
        print '  category = %d' % goods.category
        cur.execute(
         "INSERT INTO shop_goods_category (goods, category) VALUES (%d, %d)" % (
         pkey, goods.category
        ))
    else:
        print '  category = None'

    cur.execute((
        'INSERT INTO shop_news VALUES '
        '(DEFAULT, %d, CURRENT_DATE, CURRENT_DATE + 14)') % pkey)
    cur.close()
    opts.con.commit()
    sys.stdout.flush()


# U zbozi, ktere nebylo synchronizovano nastav cenu na 0
q = """
    UPDATE shop_goods
       SET not_avail = 'vyprodáno'
     WHERE pkey IN (
            SELECT goods FROM shop_sync
             WHERE sync ISNULL or sync  < CURRENT_DATE
    )
"""
