# -*- coding: utf-8 -*-
import sys
from wm4.api import *
from wm4.web.forms.controllers import *
from wm4.web.forms.listings import *

from comp4.antispam.web import JSExprAntispamCtrl
from comp4.fulltext.entities import Fulltext, TSMatch, TSQuery, TSRank
from comp4.dbgettext.web import DBGettextCtrl, DBGettextAreaCtrl, DBGettextHtmlCtrl

from lib4.imports import importObject
from entities import *

__all__ = [
    "StateCtrl", "FirstLetterFilter",
    "BaseMultiparamCtrl", "ImageParamCtrl", "FileParamCtrl",
    "SelectParamCtrl",
    "CategoryCtrl", "GoodsCtrl", 
    "CategoryForm", "ParamsCtrl",
    "GoodsForm", "GoodsParamSelectFilter", "GoodsCategorySelectFilter",
    "GoodsLst", 'GoodsAdmLst', 'GoodsCommentForm',
    "CustomerPublicForm", "PaymentPublicForm",
    "CustomerForm", "CustomerLst",
    "InvoiceForm", "InvoiceLst",
    "DiscountTicketBatchForm",
    "ReclaimForm",
]

_ = i18n.MessageIDFactory('comp4.shop')


class StateCtrl(SelectCtrl):
    def defaultInput(self):
        return u'CZ'
    def selectOptions(self):
        from lib4.data.states import state_list
        return [(s,'%s (%s)' % (n,s),s) for s,n in state_list]
    selectOptions = binding.Make(selectOptions)


class FirstLetterFilter(InputFilter):
    queryExpr = letters = binding.Require()
    ctrlWidget = web.FORM_WIDGET('input.shop.first_letter')






class BaseMultiparamCtrl(SequenceCtrl):

    ctrlWidget = web.FORM_WIDGET('input.shop.multiparam')

    def commitSelf(self):
        self.newValue = [i.newValue for i in self.ctrlItems]
        if self.newValue != self.oldValue:
            OnChangeEvent(self, old=self.oldValue, new=self.newValue)
        return True

class ImageParamCtrl(ImageUploadCtrl):
    ctrlType = ShopImage
    
    def makeFile(self):
        return settings.workspace().add(self.ctrlType(
            filename=self.filename, blob=open(self.path), size=self.size,
            content_type=self.content_type, width=self.width, height=self.height
        ))

class FileParamCtrl(FileUploadCtrl):
    ctrlType = ShopFile
    
    def makeFile(self):
        return settings.workspace().add(self.ctrlType(
            filename=self.filename, blob=open(self.path), size=self.size,
            content_type=self.content_type
        ))

class SelectParamCtrl(SelectCtrl):
    paramDef = binding.Require()
    def selectOptions(self):
        opts = []
        for o in self.paramDef.selectOptions:
            if isinstance(o, (tuple,list)):
                t, l, o = o
            else:
                t = l = o
            opts.append((t,l,o))
        return opts
    selectOptions = binding.Make(selectOptions)
    noneLabel = binding.Make(lambda s: getattr(s.paramDef, 'noneLabel', ''))




class ParamsCtrl(ContainerCtrl):
    paramDefs = binding.Require()

    def _ctrlSetupChildren(self):
        features = {}
        oldValue = self.oldValue
        for name, pdef in self.paramDefs:
            attrs = {}
            ov = getattr(oldValue, name, NOT_GIVEN)
            if ov is not NOT_GIVEN:
                attrs['oldValue'] = ov
            if pdef.title:
                attrs['ctrlLabel'] = pdef.title
            if pdef.lowerBound > 0:
                attrs['ctrlRequired'] = True
            if pdef.type:
                attrs['ctrlType'] = pdef.type
            if pdef.upperBound == 1:
                ctrlCls = importObject(pdef.controller)
                if hasattr(ctrlCls, 'paramDef'):
                    attrs['paramDef'] = pdef
                ctrl = ctrlCls(self.ctrlReq, name, self, **attrs)
            else:
                itemCtrl = importObject(pdef.controller)
                class ctrlCls(BaseMultiparamCtrl):
                    class itemClass(itemCtrl, SequenceItemCtrl):
                        ctrlRequired = True
                        paramDef = binding.Constant(pdef)
                ctrl = ctrlCls(self.ctrlReq, name, self, **attrs)
            features[name] = ctrl
        features.update(super(ParamsCtrl,self)._ctrlSetupChildren())
        return features

    def commitSelf(self):
        values = {}
        for name, pdef in self.paramDefs:
            elem = self[name]
            if pdef.upperBound == 1:
                values[name] = [elem.newValue]
            else:
                values[name] = elem.newValue
        self.newValue = values
        OnChangeEvent(self, old=self.oldValue, new=self.newValue)
        return True





class CategoryCtrl(InputCtrl):
    
    ctrlWidget = web.FORM_WIDGET('input.shop.category')

    class popup(PopupCtrl):
        popupMap = dict(
            pkey = '../pkey',
            name = '../name',
            path = '../path',
        )
        def popupUrl(self):
            return self.ctrlReq.get_url(ShopCategory, 'popup')
        popupUrl = binding.Make(popupUrl)

    class pkey(InputCtrl):
        ctrlVisibility = 'hidden'

    class name(InputCtrl):
        def defaultInput(self):
            old = self['../pkey'].oldValue
            if old in (NOT_GIVEN, None):
                return ''
            return settings.workspace().get(ShopCategory, old).name

    class path(InputCtrl):
        ctrlVisibility = 'hidden'
        def defaultInput(self):
            old = self['../pkey'].oldValue
            if old in (NOT_GIVEN, None):
                return ''
            return settings.workspace().get(ShopCategory, old).path

        class span(DependentSpanCtrl):
            dependsOn = '..'
            defaultText = u'... klikněte pro výběr kategorie ...'
            ctrlVisibility = 'visible'
    
    def defaultOutput(self):
        cid = self['pkey'].newValue
        ws = settings.workspace()
        try:
            return ws.get(ShopCategory, cid)
        except:
            return None





class GoodsCtrl(InputCtrl):
    ctrlWidget = web.FORM_WIDGET('input.shop.goods')

    class popup(PopupCtrl):
        popupMap = dict(pkey='../pkey', name='../name')
        def popupUrl(self):
            return self.ctrlReq.get_url(ShopGoods, 'popup')
        popupUrl = binding.Make(popupUrl)
    class pkey(InputCtrl):
        ctrlVisibility = 'hidden'
        pass
    class name(InputCtrl):
        ctrlVisibility = 'hidden'
        def defaultInput(self):
            old = self['../pkey'].oldValue
            if old in (NOT_GIVEN, None):
                return ''
            return settings.workspace().get(ShopGoods, old).name
        class span(DependentSpanCtrl):
            dependsOn = '..'
            defaultText = u'... klikněte pro výběr zboží ...'
            ctrlVisibility = 'visible'
    def defaultOutput(self):
        pkey = self['pkey'].newValue
        ws = settings.workspace()
        try:
            return ws.get(ShopGoods, pkey)
        except:
            return None


class PersonCtrl(InputCtrl):
    
    ctrlWidget = web.FORM_WIDGET('input.shop.person')

    class popup(PopupCtrl):
        popupMap = dict(
            pkey = '../pkey',
            name = '../name',
        )
        def popupUrl(self):
            return self.ctrlReq.get_url(ShopPerson, 'popup')
        popupUrl = binding.Make(popupUrl)

    class pkey(InputCtrl):
        ctrlVisibility = 'hidden'

    class name(InputCtrl):
        ctrlVisibility = 'hidden'
        def defaultInput(self):
            old = self['../pkey'].oldValue
            if old in (NOT_GIVEN, None):
                return ''
            return settings.workspace().get(ShopCategory, old).name
        class span(DependentSpanCtrl):
            dependsOn = '..'
            defaultText = u'... klikněte pro výběr osoby ...'
            ctrlVisibility = 'visible'
    
    def defaultOutput(self):
        cid = self['pkey'].newValue
        ws = settings.workspace()
        try:
            return ws.get(ShopPerson, cid)
        except:
            return None



class ManufacturerCtrl(InputCtrl):
    
    ctrlWidget = web.FORM_WIDGET('input.shop.manufacturer')

    class popup(PopupCtrl):
        popupMap = dict(
            pkey = '../pkey',
            name = '../name',
        )
        def popupUrl(self):
            return self.ctrlReq.get_url(ShopManufacturer, 'popup')
        popupUrl = binding.Make(popupUrl)

    class pkey(InputCtrl):
        ctrlVisibility = 'hidden'

    class name(InputCtrl):
        ctrlVisibility = 'hidden'
        def defaultInput(self):
            old = self['../pkey'].oldValue
            if old in (NOT_GIVEN, None):
                return ''
            return settings.workspace().get(ShopCategory, old).name
        class span(DependentSpanCtrl):
            dependsOn = '..'
            defaultText = u'... klikněte pro výběr výrobce ...'
            ctrlVisibility = 'visible'
    
    def defaultOutput(self):
        cid = self['pkey'].newValue
        ws = settings.workspace()
        try:
            return ws.get(ShopManufacturer, cid)
        except:
            return None

    





class CategoryForm(makeFormCtrl(ShopCategory)):
    
    featureCommiting = {'*': 'before', 'children': 'after'}

    class parent(CategoryCtrl):
        pass

    class template(SelectCtrl):
        def selectOptions(self):
            return [(o,o,o) for o in self.ctrlReq.params['category_templates']]
        selectOptions = binding.Make(selectOptions)
        
    class description(HtmlCtrl):
        pass

    class children(SequenceCtrl):

        class itemClass(makeFormCtrl(ShopCategory), SequenceItemCtrl):

            class position(InputCtrl):
                ctrlType = int
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self.ctrlParent.itemPosition

        def updateContainer(self, removed):
            assert not removed

    class params(ParamsCtrl):
        protocols.advise(instancesProvide=[web.IFormInput])

        paramDefs = binding.Make(
            lambda: settings.workspace()['shop'].categoryParams)













class GoodsForm(makeFormCtrl(ShopGoods)):
    
    featureCommiting = {
        'pkey':'before', 'typ':'before',
        'name':'before', '_name': 'before',
        'short':'before', '_short': 'before',
        'description':'before', '_description': 'before',
        'note':'before',
        'price':'before', 'price_common':'before', 'price_action':'before',
        'not_avail':'before', 'hidden':'before', 'params':'before',
        'manufacturer':'before',
        '*':'after'
    }

    class _name(DBGettextCtrl):
        set_on_change = '../name'

    class params(ParamsCtrl):
        protocols.advise(instancesProvide=[web.IFormInput])

        paramDefs = binding.Make(
            lambda: settings.workspace()['shop'].goodsParams)

    class typ(SelectCtrl):
        ctrlRequired = True
        def selectOptions(self):
            return [
                (n, t.title or n, n)
                for n,t in settings.workspace()['shop'].goodsTypes.items()
            ]
        selectOptions = binding.Make(selectOptions)

    class hidden(BoolCtrl):
        ctrlRequired = True
        def defaultInput(self):
            return 'no'

    class short(TextAreaCtrl):
        pass
    class _short(DBGettextAreaCtrl):
        set_on_change = '../short'

    class description(HtmlCtrl):
        pass
    class _description(DBGettextHtmlCtrl):
        set_on_change = '../description'

    class manufacturer(ManufacturerCtrl):
        pass

    class _persons(SequenceCtrl):
        ctrlWidget = web.FORM_WIDGET('input.shop.persons')
        class itemClass(makeFormCtrl(ShopGoodsPerson), SequenceItemCtrl):
            class person(PersonCtrl):
                pass
            class role(SelectCtrl):
                ctrlRequired = True
                def selectOptions(self):
                    return [(k,n,k)
                        for k,n in settings.workspace()['shop'].person_roles
                    ]
                selectOptions = binding.Make(selectOptions)
            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

        class addItem(SequenceCtrl.addItem):
            ctrlLabel = u'Přidat osobu'

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])

    class note(TextAreaCtrl):
        pass

    class _cats(SequenceCtrl):
        ctrlWidget = web.FORM_WIDGET('input.shop.categories')

        class itemClass(makeFormCtrl(ShopGoodsCategory), SequenceItemCtrl):
            
            class category(CategoryCtrl):
                pass

            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue


        class addItem(SequenceCtrl.addItem):
            ctrlLabel = u'Přidat kategorii'

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])
        
    class _tips(SequenceCtrl):
        ctrlWidget = web.FORM_WIDGET('input.shop.categories')

        class itemClass(makeFormCtrl(ShopGoodsTip), SequenceItemCtrl):
            
            class category(CategoryCtrl):
                pass

            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

        class addItem(SequenceCtrl.addItem):
            ctrlLabel = u'Přidat jako tip do kategorie'

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])
        
    class _related(SequenceCtrl):

        class itemClass(makeFormCtrl(ShopGoodsRelated), SequenceItemCtrl):

            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

            class related(GoodsCtrl):
                ctrlRequired = True

            class description(InputCtrl):
                pass


        class addItem(SequenceCtrl.addItem):
            ctrlLabel = u'Přidat související zboží'

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])

    class news(SequenceCtrl):

        class itemClass(makeFormCtrl(ShopNews), SequenceItemCtrl):

            class announce(TextAreaCtrl):
                pass

            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

        class addItem(SequenceCtrl.addItem):
            ctrlLabel = u'Přidat termín'

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])

    class photogallery(SequenceCtrl):

        class itemClass(makeFormCtrl(ShopPhotogalleryImage), SequenceItemCtrl):
            featureCommiting = {'*':'before', 'image':'after'}

            class goods(InputCtrl):
                ctrlType = ShopGoods
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

            class position(InputCtrl):
                ctrlType = int
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self.ctrlParent.itemPosition

            class description(InputCtrl):
                ctrlLabel = u'Popis'

            class image(ImageUploadCtrl):
                ctrlLabel = u'Obrázek'
                ctrlRequired = True
                def makeFile(self):
                    import lib4.images as img4
                    gallery = self['..'].newValue
                    try:
                        orig = img4.loadImage(self.path)
                    except:
                        raise schema.FeatureSyntaxError(self.ctrlName, u'Nepodporovaný typ souboru.')
                    ws = settings.workspace()
                    ws('''[
                        query.Delete(r) for r in ShopPhotogalleryRes
                                         if r.photogallery == gallery
                    ]''')
                    resolutions = {}
                    for res, conv in ws['shop'].goods_image_convertors:
                        img = conv(orig)
                        size = img4.getStorageSize(img)
                        fd = img4.getFileDesc(img)
                        fd.seek(0,0)
                        img = ws.new(ShopImage,
                            blob         = fd,
                            size         = size,
                            content_type = img4.getMIME(img),
                            filename     = res + '_' + self.filename,
                            width        = img.size[0],
                            height       = img.size[1],
                        )
                        r = ws.new(ShopPhotogalleryRes,
                            photogallery = gallery,
                            resolution   = res,
                            image        = img,
                        )
                        resolutions[res] = r
                    return resolutions['orig']

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])



class GoodsParamSelectFilter(SelectFilter):
    
    paramName = binding.Make(lambda s: s.ctrlName)
    
    def selectOptions(self):
        opts = []
        for r in self.ctrlParent.distinctParamValues.get(self.paramName,()):
            if r:
                opts.append((r, r, r))
        return opts
    selectOptions = binding.Make(selectOptions)

    def makeCond(self): 
        val = self.newValue
        if val in (NOT_GIVEN, None, ''):
            return
        yield self.parseSubexpr('''obj in [
            p.goods for p in 'shop/GoodsParam'
                     if p.param == self.paramName
                    and p.value == val
        ]''')



class GoodsCategorySelectFilter(SelectFilter):

    parentCategoryMark = None
    searchSubcategories = False

    def selectOptions(self):
        ws = self.ctrlParent.workspace
        cm = self.parentCategoryMark
        if cm is None:
            children = ws['shop'].categories.children
        else:
            children = ws['shop'].categoryByMark(cm).children
        opts = []
        for c in children:
            opts.append((c.pkey, c.name, c))
        return opts
    selectOptions = binding.Make(selectOptions)

    def makeCond(self): 
        val = self.newValue
        if val in (NOT_GIVEN, None, ''):
            return
        ws = self.ctrlParent.workspace
        GC = ws['shop/GoodsCategory']
        TC = ws['shop/Category'].treeCache
        if self.searchSubcategories:
            yield self.parseSubexpr('''obj in [
                gc.goods for gc in GC
                         for tc in TC
                          if tc.begin == val.pkey
                         and gc.category.pkey == tc.end
            ]''')
        else:
            yield self.parseSubexpr('''obj in [
                gc.goods for gc in GC
                          if gc.category == val
            ]''')



class GoodsLst(ListingCtrl):

    baseQuery = '''[obj for obj in 'shop/Goods'
                         if obj.hidden == False
                        and obj.deleted == False]'''
    
    defaultOrder = '+name'

    distinctValuesForParams = ()

    def distinctParamValues(self):
        values = {}
        for p,v in self.workspace('''[
            query.Distinct(p.param, p.value)
                    for p in 'shop/GoodsParam'
                    for g in 'shop/Goods'
                     if p.param in self.distinctValuesForParams
                    and p.goods == g
                    and g.deleted == False
        ]'''):
            values.setdefault(p, []).append(v)
        return values
    distinctParamValues = binding.Make(distinctParamValues)
        

    class pkey(InputFilter):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Kód'

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = u'Název'
        
    class typ(SelectFilter):
        queryExpr = 'obj.typ'
        ctrlLabel = u'Typ'
        noneLabel = u'- typ -'
        def selectOptions(self):
            return [
              (n,t.title,n)
                for n,t in settings.workspace()['shop'].goodsTypes.items()
            ]
        selectOptions = binding.Make(selectOptions)
        
    class price(Column):
        queryExpr = 'obj.price'
        ctrlLabel = u'Cena'
        
    class fulltext(MulticolLikeFilter):
        filteredExpressions = (
            'obj.name', 'obj.description')
    
    def setFilters(self, *goods):
        pass
        
class GoodsAdmLst(GoodsLst):

    baseQuery = '''[obj for obj in 'shop/Goods' if obj.deleted == False]'''
    defaultOrder = '-pkey'

    class hidden(SelectFilter):
        queryExpr = 'obj.hidden'
        ctrlLabel = u'Skryté'
        noneLabel = u'- skryté -'
        selectOptions = [
            ('t', u'skryté', True),
            ('f', u'veřejné', False),
        ]



class GoodsFulltextLst(ListingCtrl):
    
    baseQuery = '''[
        obj for obj in 'shop/Goods'
            for ft in 'fulltext/Fulltext'
             if obj.hidden == False
            and obj.deleted == False
            and ft.klass == 'shop/Goods'
            and obj.pkey == ft.key
    ]'''

    defaultOrder = '-tsrank'
    
    class term(InputFilter):
        
        def makeCond(self): 
            val = self.newValue
            if val in (NOT_GIVEN, None, ''):
                return
            tsq = TSQuery('cs', ' & '.join([t.lower() for t in val.split()]))
            yield self.parseSubexpr('''TSMatch(ft.vector, tsq)''')
    
    class tsrank(Column):
        
        def parseSubexpr(self, expr):
            frame = sys._getframe(1)
            return self.listingCtrl.parsedQuery.parseSubexpr(expr, frame)

        def queryExpr(self): 
            val = self['../term'].newValue
            if val in (NOT_GIVEN, None, ''):
                return None
            tsq = TSQuery('cs', ' & '.join([t.lower() for t in val.split()]))
            return self.parseSubexpr('''TSRank(ft.vector, tsq)''')
        queryExpr = binding.Make(queryExpr)

        


class GoodsCommentForm(makeFormCtrl(ShopGoodsComment)):
    goods = binding.Require()
    parent_id = ClientState(int)
    shown = ClientState()

    class parent(InputCtrl):
        def defaultOutput(self):
            par = self.ctrlParent
            return None
            return par.forum.comments.get(par.parent_id)

    class goods(InputCtrl):
        def defaultOutput(self):
            return self.ctrlParent.goods

    class sender(InputCtrl):
        ctrlLabel = _(u'Name')
        def defaultInput(self):
            u = self.ctrlReq.user
            if u:
                return u.name
            return ''

    class email(InputCtrl):
        ctrlLabel = _(u'Email')
        def defaultInput(self):
            u = self.ctrlReq.user
            if u:
                return u.email
            return ''

    class rating(SelectCtrl):
        selectOptions = [
            (1, u'1 - nejhorší', 1),
            (2, u'2', 2),
            (3, u'3 - střední', 3),
            (4, u'4', 4),
            (5, u'5 - nejlepší', 5),
        ]
        
    class preview(ButtonCtrl):
        ctrlLabel = _(u'Náhled')

    @event_handler(OnClickEvent, 'preview')
    def do_preview(self, evt):
        self.shown = 'yes'

    @event_handler(OnClickEvent, 'doSubmit')
    def require_preview(self, evt):
        del self.shown

    class message(TextAreaCtrl):
        ctrlRequired = True
        rows = 20
        cols = 60

    def markup(self):
        return self['message'].inputText
    markup = binding.Make(markup)

    class published(BoolCtrl):
        def defaultOutput(self):
            return True
            par = self.ctrlParent
            if par.forum.moderated:
                return False 
            else:
                return True

    class validator(JSExprAntispamCtrl):
        pass

    class ip(InputCtrl):
        def commitChanges(self):
            env = self.ctrlReq.environ
            ip = env['REMOTE_ADDR']
            if 'HTTP_X_FORWARDED_FOR' in env:
                ip = '%s/%s' % (ip, env['HTTP_X_FORWARDED_FOR'])
            self.newValue = ip
            OnChangeEvent(self, old=self.oldValue, new=self.newValue)
            return True
    
    class session(InputCtrl):
        pass


class GoodsCommentLst(ListingCtrl):

    baseQuery = '''[
            (obj, goods)
                for obj in 'shop/GoodsComment'
                for goods in 'shop/Goods'
                 if obj.goods == goods
        ]'''

    defaultOrder = '-created'

    class sender(InputFilter):
        queryExpr = 'obj.sender'
        ctrlLabel = u'Odesílatel'

    class goods(InputFilter):
        queryExpr = 'goods.name'
        ctrlLabel = u'Zboží'

    class subject(InputFilter):
        queryExpr = 'obj.subject'
        ctrlLabel = u'Předmět'

    class created(Column):
        queryExpr = 'obj.created'
        ctrlLabel = u'Čas'

    class rating(Column):
        queryExpr = 'obj.rating'
        ctrlLabel = u'Hodnocení'

    class visible(Column):
        queryExpr = '(obj.published and obj.deleted==False)'
        ctrlLabel = u'Veřejný'

    class text(MulticolLikeFilter):
        filteredExpressions = (
            'obj.sender', 'obj.subject', 'obj.message', 'goods.name')





class CustomerPublicForm(makeFormCtrl(ShopCustomer)):

    regUser = binding.Make(
                lambda s: web.getUser(s.ctrlReq.environ,'registered_user'))

    class pkey(InputCtrl):
        def defaultOutput(self):
            return -1

    class name(InputCtrl):
        ctrlRequired = True
        def defaultInput(self):
            ru = self['..'].regUser
            if ru is not None:
                return ru.name
            return None

    class email(InputCtrl):
        ctrlRequired = True
        def defaultInput(self):
            ru = self['..'].regUser
            if ru is not None:
                return ru.email
            return None
        def parseValue(self, txt):
            val = InputCtrl.parseValue(self, txt)
            from lib4.validators import checkEmail
            if val is None:
                return val
            if not checkEmail(val):
                raise schema.FeatureSyntaxError(
                    self.ctrlName, u'Neplatná emailová adresa')
            return val

    class phone(InputCtrl):
        ctrlRequired = True
        def defaultInput(self):
            ru = self['..'].regUser
            if ru is not None:
                return ru.phone
            return None

    class country(SelectCtrl):
        ctrlRequired = True
        selectOptions = [('cz', u'Česká republika', 'cz')]

    class dcountry(SelectCtrl):
        selectOptions = [('cz', u'Česká republika', 'cz')]

    class reg_user(InputCtrl):
        def defaultOutput(self):
            return None

    def addEntity(self):
        return self.ctrlType(**self.newFeatures)

    def editEntity(self):
        obj = self.oldValue
        obj._setattrs(**self.newFeatures)
        return obj
    
    class doSubmit(SubmitCtrl):
        ctrlLabel = u'Pokračovat'


class PaymentPublicForm(InstanceCtrl, FormCtrl):
    
    class payment(InputCtrl):
        pass







class ManufacturerForm(makeFormCtrl(ShopManufacturer)):

    class description(HtmlCtrl):
        pass

    class state(StateCtrl):
        pass

    class image(ImageParamCtrl):
        converter = binding.Make(lambda:(
            config.Property('comp4.shop.manufacturer.image_converter')(
                default=None)
        ))

    is_editor = binding.Make(
                    lambda s: s.ctrlReq.allows(s.ctrlReq.params['editPerm']))

    goodsLstCtrl = binding.Make(lambda s:(
        importObject(s.ctrlReq.params['goodsLstCtrl'])))

    goodsAdmLstCtrl = binding.Make(lambda s:(
        importObject(s.ctrlReq.params['goodsAdmLstCtrl'])))

    def goodsLst(self):
        if self.oldValue in (None, NOT_GIVEN): return None
        if self.is_editor:
            lst = self.goodsAdmLstCtrl(ctrlReq=self.ctrlReq)
        else:
            lst = self.goodsLstCtrl(ctrlReq=self.ctrlReq)
        lst.precond = lst.parsedQuery.parseSubexpr(
            'obj.manufacturer._oid == self.oldValue._oid')
        lst()
        return lst
    goodsLst = binding.Make(goodsLst)

class ManufacturerLst(ListingCtrl):

    baseQuery = '''[
        (obj,
         query.Upper(obj.name[0:1]),
         int(query.Coalesce(
            query.Sum(
                query.Case(
                    (goods.hidden == False and goods.deleted == False, 1),
                    default = None
                )
            ), 0)),
         query.Count(goods)
        )
            for obj in 'shop/Manufacturer'
            for goods in 'shop/Goods'
             if query.LeftJoin(goods.manufacturer == obj)
    ]'''

    defaultOrder = '+name'

    is_editor = binding.Make(
                    lambda s: s.ctrlReq.allows(s.ctrlReq.params['editPerm']))

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Kód'
    class first(FirstLetterFilter):
        queryExpr = 'query.Upper(obj.name[0:1])'
        letters = binding.Make(
            lambda: settings.workspace()['shop'].manufacturerLetters)
    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = u'Výrobce'
    class address(LikeFilter):
        queryExpr = 'obj.address'
        ctrlLabel = u'Adresa'
    class goods_count_all(Column):
        queryExpr = 'query.Count(goods)'
        ctrlLabel = u'Položek'
    class goods_count(Column):
        queryExpr = '''int(query.Coalesce(
            query.Sum(query.Case(
                    (goods.hidden == False and goods.deleted == False, 1),
                    default = None
            )), 0))'''
        ctrlLabel = u'Položek'
    class state(StateCtrl, FilterMixin):
        queryExpr = 'obj.state'
        ctrlLabel = u'Země'
        noneLabel = u'- země -'

        








class PersonForm(makeFormCtrl(ShopPerson)):
    class description(HtmlCtrl):
        pass
    class image(ImageParamCtrl):
        converter = binding.Make(lambda:(
            config.Property('comp4.shop.person.image_converter')(default=None)
        ))
    class state(StateCtrl):
        pass

    is_editor = binding.Make(
                    lambda s: s.ctrlReq.allows(s.ctrlReq.params['editPerm']))
    goodsLstCtrl = binding.Make(lambda s:(
        importObject(s.ctrlReq.params['goodsLstCtrl'])))
    goodsAdmLstCtrl = binding.Make(lambda s:(
        importObject(s.ctrlReq.params['goodsAdmLstCtrl'])))

    def goodsLst(self):
        if self.oldValue in (None, NOT_GIVEN): return None
        roles = [k for k,n in settings.workspace()['shop'].person_roles]
        if not roles: return None
        if self.is_editor:
            lst = self.goodsAdmLstCtrl(ctrlReq=self.ctrlReq)
        else:
            lst = self.goodsLstCtrl(ctrlReq=self.ctrlReq)
        lst.precond = lst.parsedQuery.parseSubexpr('''obj.pkey in [
            p.goods.pkey for p in 'shop/GoodsPerson'
                          if p.person.pkey == self.oldValue.pkey
        ]''')
        lst()
        return lst
    goodsLst = binding.Make(goodsLst)

class PersonLst(ListingCtrl):

    person_roles = binding.Make(
                    lambda: settings.workspace()['shop'].person_roles)

    def baseQuery(self):
        return '''[
            (obj,
             query.Upper(obj.name[0:1]),
             int(query.Coalesce(
                query.Sum(
                    query.Case(
                        (goods.hidden == False and goods.deleted == False, 1),
                        default = None
                    )
                ), 0)),
                 query.Count(gp)
            )
                for obj in 'shop/Person'
                for gp in 'shop/GoodsPerson'
                for goods in 'shop/Goods'
                 if query.LeftJoin(obj == gp.person)
                and query.LeftJoin(gp.goods == goods)
        ]'''
    baseQuery = binding.Make(baseQuery)

    defaultOrder = '+name'

    is_editor = binding.Make(
                    lambda s: s.ctrlReq.allows(s.ctrlReq.params['editPerm']))

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Kód'
    class first(FirstLetterFilter):
        queryExpr = 'query.Upper(obj.name[0:1])'
        letters = binding.Make(
            lambda: settings.workspace()['shop'].personLetters)
    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = u'Jméno'
    class birth_year(Column):
        queryExpr = 'obj.birth_year'
        ctrlLabel = u'Narození'
    class death_year(Column):
        queryExpr = 'obj.death_year'
        ctrlLabel = u'Úmrtí'
    class goods_count_all(Column):
        queryExpr = 'query.Count(gp)'
        ctrlLabel = u'Položek'
    class goods_count(Column):
        queryExpr = '''int(query.Coalesce(
                query.Sum(
                    query.Case(
                        (goods.hidden == False and goods.deleted == False, 1),
                        default = None
                    )
                ), 0))'''
        ctrlLabel = u'Položek'
    class role(SelectFilter):
        noneLabel = u'- role -'
        def selectOptions(self):
            return [(k,n,k)
                for k,n in settings.workspace()['shop'].person_roles
            ]
        selectOptions = binding.Make(selectOptions)
        def makeCond(self):
            val = self.newValue
            if val in (NOT_GIVEN, None, ''):
                return
            q = """obj.pkey in [
                p.person.pkey for p in 'shop/GoodsPerson' if p.role == val
            ]"""
            yield self.parseSubexpr(q)
    class state(StateCtrl, FilterMixin):
        queryExpr = 'obj.state'
        ctrlLabel = u'Země'
        noneLabel = u'- země -'






class CustomerForm(makeFormCtrl(ShopCustomer)):
    pass

class CustomerLst(ListingCtrl):

    baseQuery = '''[
            (obj, query.Count(inv))
                for obj in 'shop/Customer'
                for inv in 'shop/Invoice'
                 if query.LeftJoin(obj == inv.customer)
                and query.GroupBy(obj)
        ]'''

    defaultOrder = '+name'

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Kód'

    class name(Column):
        queryExpr = 'obj.name'
        ctrlLabel = u'Zákazník'

    class street(Column):
        queryExpr = 'obj.street'
        ctrlLabel = u'Ulice'

    class city(Column):
        queryExpr = 'obj.city'
        ctrlLabel = u'Město'

    class fulltext(MulticolLikeFilter):
        filteredExpressions = ('obj.name', 'obj.street', 'obj.city')
        
    class count(Column):
        queryExpr = 'query.Count(inv)'
        ctrlLabel = u'Objednávek'












class InvoiceForm(makeFormCtrl(ShopInvoice)):
    
    featureCommiting = {'*':'before', 'items':'after'}

    class customer(InputCtrl):
        class popup(PopupCtrl):
            popupMap = dict(pkey='../pkey', name='../name')
            def popupUrl(self):
                return self.ctrlReq.get_url(ShopCustomer, 'popup')
            popupUrl = binding.Make(popupUrl)
        class pkey(InputCtrl):
            pass
        class name(InputCtrl):
            def defaultInput(self):
                old = self['../pkey'].oldValue
                if old in (NOT_GIVEN, None):
                    return ''
                return settings.workspace().get(ShopCustomer, old).name
        def defaultOutput(self):
            pkey = self['pkey'].newValue
            ws = settings.workspace()
            try:
                return ws.get(ShopCustomer, pkey)
            except:
                return None

    class delivery(SelectCtrl):
        ctrlRequired = True
        def selectOptions(self):
            return [
                (m.key, m.name, m.key)
                for m in settings.workspace()['shop'].delivery_modes
            ]
        selectOptions = binding.Make(selectOptions)

    class payment(SelectCtrl):
        ctrlRequired = True
        def selectOptions(self):
            return [
                (m.key, m.name, m.key)
                for m in settings.workspace()['shop'].payment_modes
            ]
        selectOptions = binding.Make(selectOptions)

    class items(SequenceCtrl):

        class itemClass(makeFormCtrl(ShopInvoiceItem), SequenceItemCtrl):

            class invoice(InputCtrl):
                ctrlType = ShopInvoice
                ctrlRequired = True
                ctrlVisibility = binding.Constant('hidden')
                def defaultOutput(self):
                    return self['../../..'].newValue

            class goods(GoodsCtrl):
                pass

        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])

    class note(TextAreaCtrl):
        pass


class InvoiceLst(ListingCtrl):

    baseQuery = '''[
            (obj, cust, reg,
                query.Sum(item.quantity),
                query.Sum(item.unit_price * item.quantity)
                + obj.postage + obj.cod + obj.packaging - obj.discount
            )
                for obj in 'shop/Invoice'
                for cust in 'shop/Customer'
                for reg in 'registration/RegisteredUser'
                for item in 'shop/InvoiceItem'
                 if obj.customer == cust
                and query.LeftJoin(reg == cust.reg_user)
                and query.LeftJoin(obj == item.invoice)
                and query.GroupBy(obj, cust, reg)
        ]'''

    defaultOrder = '-created'

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Číslo'

    class created(Column):
        queryExpr = 'obj.created'
        ctrlLabel = u'Datum'

    class customer(Column):
        queryExpr = 'cust.name'
        ctrlLabel = u'Zákazník'

    class login(Column):
        queryExpr = 'reg.login'
        ctrlLabel = u'Login'

    class fulltext(MulticolLikeFilter):
        filteredExpressions = (
            'cust.name', 'reg.login', 'query.Cast(obj.pkey,str)',
            'cust.city', 'cust.street',
        )
        
    class status(EnumCtrl, FilterMixin):
        ctrlType = InvoiceStatus
        ctrlLabel = u'Stav'
        queryExpr = 'obj.status'
        defaultOptions = binding.Make(lambda s: [('',_(u'-- status --'),None)])

    class quantity_sum(Column):
        queryExpr = 'query.Sum(item.quantity)'
        ctrlLabel = u'Kusů'

    class total(Column):
        queryExpr = '''(query.Sum(item.unit_price * item.quantity) +
                        obj.postage + obj.cod + obj.packaging - obj.discount)'''
        ctrlLabel = u'K úhradě'








class DiscountTicketBatchForm(makeFormCtrl(ShopDiscountTicketBatch)):

    class note(TextAreaCtrl):
        pass

class DiscountTicketBatchLst(ListingCtrl):

    baseQuery = '''[
            (obj, query.Count(tck), query.Count(tck.invoice.pkey))
                for obj in 'shop/DiscountTicketBatch'
                for tck in 'shop/DiscountTicket'
                 if query.LeftJoin(obj == tck.batch)
                and query.GroupBy(obj)
        ]'''

    defaultOrder = '-valid_from'

    class pkey(Column):
        queryExpr = 'obj.pkey'
        ctrlLabel = u'Kód'

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = u'Dávka'

    class percent(Column):
        queryExpr = 'obj.percent'
        ctrlLabel = u'Procent'

    class absolute(Column):
        queryExpr = 'obj.absolute'
        ctrlLabel = u'Částka'

    class valid_from(Column):
        queryExpr = 'obj.valid_from'
        ctrlLabel = u'Od'

    class valid_till(Column):
        queryExpr = 'obj.valid_till'
        ctrlLabel = u'Do'

    class count(Column):
        queryExpr = 'query.Count(tck)'
        ctrlLabel = u'Celkem'

    class used(Column):
        queryExpr = 'query.Count(tck.invoice.pkey)'
        ctrlLabel = u'Použito'




class ReclaimForm(FormCtrl):
    
    class order_num(InputCtrl):
        ctrlRequired = True

    class order_date(DateCtrl):
        ctrlRequired = True

    class reclaim_reason(SelectCtrl):
        selectOptions = []

    class reclaim_description(SelectCtrl):
        selectOptions = []
