--   windmill4/src/comp4/trees/ddl.sql
--   windmill4/sql/arrays.sql

BEGIN;

CREATE TABLE shop_category (
    pkey        SERIAL NOT NULL,
    name        TEXT NOT NULL,
    parent      INTEGER,
    mark        VARCHAR,
    template    VARCHAR,
    description TEXT,
    position    INTEGER NOT NULL DEFAULT 0,

    UNIQUE(mark),
    PRIMARY KEY(pkey)
);

CREATE TABLE shop_category_param (
    pkey        SERIAL NOT NULL,
    category    INTEGER NOT NULL,
    param       TEXT NOT NULL,
    value       TEXT,
    position    INTEGER NOT NULL DEFAULT 1,

    PRIMARY KEY(pkey)
);
CREATE INDEX shop_category_param_category_idx ON shop_category_param (category);

CREATE TABLE shop_category_tree_cache (
    "begin"   INTEGER NOT NULL,
    "end"     INTEGER NOT NULL,
    "edges"   INTEGER NOT NULL,
    "path"    VARCHAR NOT NULL,

    PRIMARY KEY("begin", "end")
);

CREATE VIEW shop_category_tree_paths AS
    SELECT k AS pkey, p AS path,
           array_to_string(array_accum(n), ' / ') AS titles
      FROM (
                SELECT tt.path, tt."end", t.name
                  FROM (
                         SELECT *,
                         array_iter(string_to_array(path, ',')) AS pelem
                         FROM shop_category_tree_cache
                       ) AS tt, shop_category t
                 WHERE (tt.pelem).n = t.pkey
                   AND tt."begin" IN (
                        SELECT pkey FROM shop_category WHERE parent ISNULL)
                 ORDER BY (tt.pelem).p
                
           ) AS x (p,k,n)
     GROUP BY p, k
;


CREATE TABLE shop_goods (
    pkey            SERIAL NOT NULL,
    typ             VARCHAR NOT NULL,
    name            TEXT NOT NULL,
    short           TEXT,
    description     TEXT,
    manufacturer    INTEGER,

    price           FLOAT,
    price_common    FLOAT,
    price_action    FLOAT,

    hidden          BOOLEAN NOT NULL DEFAULT false,
    deleted         BOOLEAN NOT NULL DEFAULT false,

    not_avail       VARCHAR,
    note            TEXT,

    PRIMARY KEY(pkey)
);



CREATE TABLE shop_goods_category (
    goods       INTEGER NOT NULL,
    category    INTEGER NOT NULL,

    PRIMARY KEY(goods, category)
);

CREATE TABLE shop_goods_tip (
    goods       INTEGER NOT NULL,
    category    INTEGER NOT NULL,
    description VARCHAR,

    PRIMARY KEY(goods, category)
);

CREATE TABLE shop_goods_related (
    goods       INTEGER NOT NULL,
    related     INTEGER NOT NULL,
    description VARCHAR,

    PRIMARY KEY(goods, related)
);

CREATE TABLE shop_news (
    pkey        SERIAL NOT NULL,
    goods       INTEGER NOT NULL,
    new_from    DATE NOT NULL,
    new_till    DATE NOT NULL,
    announce    TEXT,

    PRIMARY KEY(pkey)
);
CREATE INDEX shop_news_goods_idx ON shop_news (goods);


CREATE TABLE shop_goods_param (
    pkey        SERIAL NOT NULL,
    goods       INTEGER NOT NULL,
    param       TEXT NOT NULL,
    value       TEXT,
    position    INTEGER NOT NULL DEFAULT 1,

    PRIMARY KEY(pkey)
);
CREATE INDEX shop_goods_param_goods_idx ON shop_goods_param (goods);

CREATE TABLE shop_photogallery_image (
    "pkey"          SERIAL NOT NULL,
    "goods"         INTEGER NOT NULL,
    "description"   VARCHAR,
    "position"      INTEGER NOT NULL DEFAULT 10,

    PRIMARY KEY("pkey")
);
CREATE INDEX shop_photogallery_image_goods_idx ON shop_photogallery_image (goods);

CREATE TABLE shop_photogallery_res (
    "pkey"          SERIAL NOT NULL,
    "photogallery"  INTEGER NOT NULL,
    "image"         INTEGER NOT NULL,
    "resolution"    VARCHAR NOT NULL DEFAULT 'orig',

    PRIMARY KEY("pkey")
);
CREATE INDEX shop_photogallery_res_photogallery_idx ON shop_photogallery_res (photogallery);


CREATE TABLE shop_image (
    "pkey"          SERIAL NOT NULL,
    "blob"          OID NOT NULL,
    "size"          INTEGER NOT NULL,
    "content_type"  VARCHAR NOT NULL DEFAULT 'application/octet-stream',
    "filename"      VARCHAR,
    "modified"      TIMESTAMP WITH TIME ZONE NOT NULL,
    "width"         INTEGER NOT NULL,
    "height"        INTEGER NOT NULL,

    PRIMARY KEY("pkey")
);

CREATE TABLE shop_file (
    "pkey"          SERIAL NOT NULL,
    "blob"          OID NOT NULL,
    "size"          INTEGER NOT NULL,
    "content_type"  VARCHAR NOT NULL DEFAULT 'application/octet-stream',
    "filename"      VARCHAR,
    "modified"      TIMESTAMP WITH TIME ZONE NOT NULL,

    PRIMARY KEY(pkey)
);





CREATE TABLE shop_manufacturer (
    pkey            SERIAL NOT NULL,
    name            TEXT NOT NULL,
    description     TEXT,
    url             VARCHAR,
    email           VARCHAR,
    phone           VARCHAR,
    address         VARCHAR,
    state           VARCHAR,
    image           INTEGER,

    PRIMARY KEY(pkey)
);


CREATE TABLE shop_person (
    pkey            SERIAL NOT NULL,
    name            TEXT NOT NULL,
    image           INTEGER,
    state           VARCHAR,
    url             VARCHAR,
    description     TEXT,
    birth_day       INTEGER,
    birth_month     INTEGER,
    birth_year      INTEGER,
    death_day       INTEGER,
    death_month     INTEGER,
    death_year      INTEGER,

    PRIMARY KEY(pkey)
);

CREATE TABLE shop_goods_person (
    pkey            SERIAL NOT NULL,
    goods           INTEGER NOT NULL,
    person          INTEGER NOT NULL,
    role            VARCHAR NOT NULL,
    position        INTEGER NOT NULL DEFAULT 0,
    UNIQUE (goods, person, role),
    PRIMARY KEY (pkey)
);




CREATE TABLE shop_goods_comment (
    pkey        SERIAL,

    goods       INTEGER NOT NULL,
    parent      INTEGER,

    sender      VARCHAR,
    email       VARCHAR,
    www         VARCHAR,
    subject     VARCHAR,
    message     VARCHAR NOT NULL,
    rating      INTEGER,

    ip          VARCHAR,
    session     VARCHAR,
    created     TIMESTAMP WITH TIME ZONE DEFAULT NOW(),
    published   BOOLEAN NOT NULL DEFAULT True,
    deleted     BOOLEAN NOT NULL DEFAULT False,

    PRIMARY KEY(pkey)
);
CREATE INDEX shop_goods_comment_goods_idx ON shop_goods_comment (goods);



CREATE TABLE shop_basket_item (
    pkey        SERIAL NOT NULL,
    token       VARCHAR NOT NULL,
    goods       INTEGER NOT NULL,
    unit        VARCHAR NOT NULL DEFAULT 'piece',
    quantity    FLOAT NOT NULL DEFAULT 1,
    unit_price  FLOAT NOT NULL,
    modified    TIMESTAMP WITH TIME ZONE NOT NULL,
    invoice     INTEGER,

    PRIMARY KEY(pkey)
);


CREATE TABLE shop_goods_bookmark (
    pkey        SERIAL NOT NULL,
    goods       INTEGER NOT NULL,
    reg_user    INTEGER NOT NULL,
    wish        BOOLEAN,
    note        VARCHAR,

    PRIMARY KEY(pkey)
);





CREATE TABLE shop_customer (
    pkey        SERIAL NOT NULL,
    reg_user    INTEGER,

    name        VARCHAR NOT NULL,

    company     VARCHAR,
    ico         VARCHAR,
    dic         VARCHAR,

    email       VARCHAR NOT NULL,
    phone       VARCHAR NOT NULL,
    web         VARCHAR,

    street      VARCHAR NOT NULL,
    city        VARCHAR NOT NULL,
    zip         VARCHAR NOT NULL,
    country     VARCHAR NOT NULL,

    dstreet     VARCHAR,
    dcity       VARCHAR,
    dzip        VARCHAR,
    dcountry    VARCHAR,

    PRIMARY KEY(pkey)
);
    


CREATE TABLE shop_invoice (
    pkey        SERIAL NOT NULL,
    customer    INTEGER NOT NULL,
    recipient   INTEGER,

    delivery    VARCHAR NOT NULL,
    payment     VARCHAR NOT NULL,

    postage     FLOAT NOT NULL DEFAULT 0,
    cod         FLOAT NOT NULL DEFAULT 0,
    packaging   FLOAT NOT NULL DEFAULT 0,
    
    discount    FLOAT NOT NULL DEFAULT 0,
    discount_ticket INTEGER,
    status      VARCHAR NOT NULL,
    created     TIMESTAMP WITH TIME ZONE NOT NULL,

    ext_invoice VARCHAR,

    note        VARCHAR,
    
    PRIMARY KEY(pkey)
);

CREATE TABLE shop_invoice_item (
    pkey        SERIAL NOT NULL,
    invoice     INTEGER NOT NULL,
    
    goods       INTEGER NOT NULL,
    unit        VARCHAR NOT NULL DEFAULT 'piece',
    quantity    FLOAT NOT NULL DEFAULT 1,
    unit_price  FLOAT NOT NULL,

    note        VARCHAR,
    available   BOOLEAN NOT NULL DEFAULT True,

    PRIMARY KEY(pkey)
);


CREATE TABLE shop_discount_ticket_batch (
    pkey        SERIAL NOT NULL,
    name        VARCHAR NOT NULL,
    valid_from  DATE NOT NULL,
    valid_till  DATE NOT NULL,

    percent     FLOAT,
    absolute    FLOAT,

    note        TEXT,
    created     TIMESTAMP WITH TIME ZONE NOT NULL,

    PRIMARY KEY(pkey)
);

CREATE TABLE shop_discount_ticket (
    pkey        SERIAL NOT NULL,
    code        VARCHAR NOT NULL,
    batch       INTEGER NOT NULL,
    invoice     INTEGER,
    blocked     BOOLEAN NOT NULL DEFAULT false,

    PRIMARY KEY(pkey)
);
CREATE UNIQUE INDEX shop_discount_ticket_code_idx ON shop_discount_ticket(code);


CREATE FUNCTION shop_gen_discount_tickets(
    IN batch INTEGER, IN tnum INTEGER
) RETURNS VARCHAR AS $$
DECLARE
    c VARCHAR;
    t VARCHAR;
    i INTEGER;
    e INTEGER;
BEGIN
    FOR i IN 1..tnum LOOP
        LOOP
            t := '';
            FOR x IN 1..8 LOOP
              SELECT INTO c substring(
                'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
                FROM (random()*61+1)::int FOR 1);
              t := t || c;
            END LOOP;
            SELECT INTO e COUNT(*) FROM shop_discount_ticket WHERE code = t;
            IF e = 0 THEN
              INSERT INTO shop_discount_ticket VALUES (DEFAULT, t, batch);
              EXIT;
            END IF;
        END LOOP;
    END LOOP;
    RETURN 'OK';
END
$$ LANGUAGE plpgsql VOLATILE STRICT;


CREATE TABLE shop_import (
    pkey    INTEGER PRIMARY KEY,
    stamp   TIMESTAMP (0) WITH TIME ZONE NOT NULL DEFAULT NOW()
);

END;
