# -*- coding: utf-8 -*-
import datetime
from wm4.api import *
from wm4.web.routes import *
from wm4.web.sitemaps import *
from wm4.utils.clspatch import patch
import comp4.cms.api as cms
from wm4.web.sessions import TrustedClientData


from entities import *

__all__ = ['ShopInvoiceToken', 'ShopSitemap', 'getBasket', 'renderMoney']

def Tmpl(filename):
    return Const('comp4.shop.templates/%s' % filename)

def Ctrl(name):
    return Const('comp4.shop.controllers.%s' % name)


class ShopInvoiceToken(TrustedClientData):
    attributes = [('pkey', 'd')]
    digest_attributes = ('secret',)



def renderMoney(req, val, cur=u'Kč'):
    if val is None:
        return ''
    if isinstance(val, basestring):
        val = val.replace(',','.').strip()
        val = float(val)
    ival = int(val)
    if ival == val:
        return '%s %s' % (ival, cur)
    return '%.02f %s' % (val, cur)

def getBasket(req):
    token = req.getCookie('wm4sess')
    if token:
        return ShopBasket(token=token)
    else:
        return None



def mainMenu(route):
    from xml4 import xhtml as xh
    s = xh.select(
        onchange="if (this.value != 'menu') window.location.href = this.value;",
    )
    for r in route.mainMenuItems:
        l = r.asMainMenu
        v = r.pathInfo
        if l == route.asMainMenu:
            s.append(xh.option(l, value=v, selected='selected'))
        else:
            s.append(xh.option(l, value=v))
        s.append('\n')
    return xh.form(s, style="display: inline;")
    

class ShopSitemap(Route):
    match = 'shop/'
    virtual = True

    isActiveShopCategory = Const(lambda c: False)
    isActiveShopManufacturer = Const(lambda m: False)

    mainMenuItems = Lazy(lambda s: [s/p for p in (
                'category/list_adm', 'goods/list_adm', 'invoice/', 'customer/',
                'discount_ticket_batch/', 'comment/',
                'manufacturer/', 'person/',
            )])
    

    renderMoney = Const(renderMoney)

    category_templates = Lazy(
        lambda s: config.Property('comp4.shop.category.templates')())
    
    manufacturer_use = Lazy(
        lambda s: config.Property('comp4.shop.manufacturer.use')())

    persons_use = Lazy(
        lambda s: bool(settings.workspace()['shop'].person_roles))

    reg_user = Lazy(
            lambda s: web.getUser(settings.environ(),'registered_user'))
    RegisteredUser = Lazy(
            lambda s: settings.workspace()['registration/RegisteredUser'])
    def bookmarksForUser(self):
        if self.reg_user:
            return settings.workspace()['shop'].bookmarksForUser(self.reg_user)
        else:
            return []
    bookmarksForUser = Lazy(bookmarksForUser)

    pathSlice = Const((0,-1))

        

    class category(web.EntitySitemap):
        match = 'category/'
        clsObj = Const(ShopCategory)
        handler = Const('comp4.shop.web.CategoryHandler')
        asMainMenu = Const(u'Kategorie zboží')

        goodsListing = Ctrl('GoodsLst')
        
        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                            s, '', './add', web.DELIMITER, mainMenu(s)))
            class list_adm(ClassRoute):
                template = Tmpl('category_list_adm.wt')
                asAction = Lazy(lambda s: web.Link.fromRoute(
                    s, label=i18n.MessageID(u'List','wm4')))

            class popup(ClassRoute):
                def actionLinks(self):
                    return [web.Link(
                        label='X', pathInfo="#",
                        onclick="return wmPopupHide(event)"
                    )]
                actionLinks = Lazy(actionLinks)

        class obj(web.EntitySitemap.obj):

            class move_goods(InstanceRoute):
                pass

            def pathObjs(self):
                l,h = self.pathSlice
                objs = [p for p in self.obj.iterParentsRev()]
                return objs[l:h]
            pathObjs = Lazy(pathObjs)

            def pathLinks(self):
                req = web.request()
                path = web.pathLinks(self)
                for o in self.pathObjs:
                    r = req.route_for(o)
                    path.append(web.Link(
                        label=o.name, pathInfo=r.pathInfo, req=req
                    ))
                return path
            pathLinks = Lazy(pathLinks)

            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', 
                web.Link(label='Dokumenty', pathInfo = s.pathInfo + 'cms/'),
                './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s),
            ))

            class cms(cms.AttachedCmsSitemap):
                match = 'cms/'
                prefix = Const('shop_category')
                pathSlice = Const((1,-1))
                category = Acquire('../obj')

            prefix = Const('shop_category')

            def isActiveShopCategory(self):
                def isActiveShopCategory(cat):
                    c = self.obj
                    while c:
                        if cat == c:
                            return True
                        c = c.parent
                    return False
                return isActiveShopCategory
            isActiveShopCategory = Lazy(isActiveShopCategory)


        patch({
           'cls.list.template' :Tmpl('category_list.wt'),
           'cls.popup.template':Tmpl('category_popup.wt'),
           'cls.add.template'  :Tmpl('category_form.wt'),
           'obj.view.template' :Tmpl('category_view.wt'),
           'obj.edit.template' :Tmpl('category_form.wt'),
        })

    class goods(web.EntitySitemap):
        match = 'goods/'
        clsObj = Const(ShopGoods)
        handler = Const('comp4.shop.web.GoodsHandler')
        asMainMenu = Const(u'Zboží')

        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                          s, './list_adm', './add', web.DELIMITER, mainMenu(s)))

            controller = Ctrl('GoodsLst')

            class list_adm(ClassRoute):
                template = Tmpl('goods_list_adm.wt')
                asAction = Lazy(lambda s: web.Link.fromRoute(
                    s, label=i18n.MessageID(u'List','wm4')))
                controller = Ctrl('GoodsAdmLst')

            class search(ClassRoute):
                template = Tmpl('goods_search.wt')
                controller = Ctrl('GoodsFulltextLst')

            class xml_zbozi_seznam(ClassRoute):
                template = Tmpl('xml_zbozi_seznam.wt')

            class popup(ClassRoute):
                def actionLinks(self):
                    return [web.Link(
                        label='X', pathInfo="#",
                        onclick="return wmPopupHide(event)"
                    )]
                actionLinks = Lazy(actionLinks)
                controller = Ctrl('GoodsAdmLst')

        class obj(web.EntitySitemap.obj):

            prefix = Const('shop_goods')

            class view(web.EntitySitemap.obj.view):
                listingCtrl = Ctrl('GoodsLst')

                class detail(InstanceRoute):
                    match = 'detail'

            class bookmark(InstanceRoute):
                match = 'bookmark'

            class basket(InstanceRoute):
                match = 'basket'

            class cms(cms.AttachedCmsSitemap):
                match = 'cms/'
                prefix = Const('shop_goods')

            class comments(InstanceRoute):
                template = Tmpl('goods_comments.wt')

                class add_root_comment(Route):
                    match = '/add'
                    action = Const('add_comment')
                    parent_id = Const(None)
                    template =Tmpl('goods_add_comment.wt')
                    signature = Signature('obj', 'action')

                class add_comment(add_root_comment):
                    match = '/add/:parent_id'
                    parent_id = Param(typ=str)
                    signature = Signature('obj', 'action', 'parent_id')
                
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', 
                web.Link(label='Dokumenty', pathInfo = s.pathInfo + 'cms/'),
                './delete', web.DELIMITER,
                '../list_adm', '../add', web.DELIMITER, mainMenu(s)
            ))

            asPath = Lazy(
                lambda s: web.Link.fromRoute(s, label=s.obj.name), inherit=False
            )

            isActiveShopCategory = Lazy(lambda s: s.obj.categories)
            def isActiveShopCategory(self):
                def isActiveShopCategory(cat):
                    for c in self.obj.categories:
                        while c:
                            if cat == c:
                                return True
                            c = c.parent
                    return False
                return isActiveShopCategory
            isActiveShopCategory = Lazy(isActiveShopCategory)
            def isActiveShopManufacturer(self):
                def isActiveShopManufacturer(m):
                    return m == self.obj.manufacturer
                return isActiveShopManufacturer
            isActiveShopManufacturer = Lazy(isActiveShopManufacturer)

        patch({
            'cls.list.template' :Tmpl('goods_list.wt'),
            'cls.add.template'  :Tmpl('goods_form.wt'),
            'cls.popup.template':Tmpl('goods_popup.wt'),
            'obj.view.template' :Tmpl('goods_view.wt'),
            'obj.edit.template' :Tmpl('goods_form.wt'),
        })

    class goods_image(web.EntitySitemap):
        match = 'goods_image/'
        clsObj = Const(ShopPhotogalleryImage)
        handler = Const('comp4.shop.web.PhotogalleryHandler')

        class cls(web.EntitySitemap.cls):
            class set(ClassRoute):
                pass

    class file(web.EntitySitemap):
        match = 'file/'
        clsObj = Const(ShopFile)
        handler = Const('comp4.shop.web.FileHandler')

    class image(web.EntitySitemap):
        match = 'image/'
        clsObj = Const(ShopImage)
        handler = Const('comp4.shop.web.ImageHandler')
        class obj(web.EntitySitemap.obj):
            class view(web.EntitySitemap.obj.view):
                class view_with_suff(Route):
                    match = ':suffix'
                    suffix = Param()
                    action = Const('view')


    class comment(web.EntitySitemap):
        match = 'comment/'
        clsObj = Const(ShopGoodsComment)
        handler = Const('comp4.shop.web.GoodsCommentHandler')
        asMainMenu = Const(u'Komentáře')
        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                          s, '', web.DELIMITER, mainMenu(s)))
        patch({
            'cls.list.template':   Tmpl('comment_list.wt'),
            'cls.list.controller': Ctrl('GoodsCommentLst'),
        })

    class manufacturer(web.EntitySitemap):
        match = 'manufacturer/'
        clsObj = Const(ShopManufacturer)
        handler = Const('comp4.shop.web.ManufacturerHandler')
        asMainMenu = Const(u'Výrobci')
        class cls(web.EntitySitemap.cls):
            class popup(ClassRoute):
                def actionLinks(self):
                    return [web.Link(
                        label='X', pathInfo="#",
                        onclick="return wmPopupHide(event)"
                    )]
                actionLinks = Lazy(actionLinks)
            actionLinks = Lazy(lambda s: web.actionLinks(
                          s, '', './add', web.DELIMITER, mainMenu(s)))
        class obj(web.EntitySitemap.obj):
            prefix = Const('shop_manufacturer')
            goodsLstCtrl = Const('comp4.shop.controllers.GoodsLst')
            goodsAdmLstCtrl = Const('comp4.shop.controllers.GoodsAdmLst')
            goodsListing = Ctrl('GoodsLst')
            class cms(cms.AttachedCmsSitemap):
                match = 'cms/'
                prefix = Const('shop_manufacturer')
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', 
                web.Link(label='Dokumenty', pathInfo = s.pathInfo + 'cms/'),
                './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s)
            ))
            asPath = Lazy(
                lambda s: web.Link.fromRoute(s, label=s.obj.name), inherit=False
            )
            def isActiveShopManufacturer(self):
                def isActiveShopManufacturer(m):
                    return m == self.obj
                return isActiveShopManufacturer
            isActiveShopManufacturer = Lazy(isActiveShopManufacturer)

        patch({
            'cls.list.controller' : Ctrl('ManufacturerLst'),
            'cls.list.template'   : Tmpl('manufacturer_list.wt'),
            'cls.popup.controller': Ctrl('ManufacturerLst'),
            'cls.popup.template'  : Tmpl('manufacturer_popup.wt'),
            'cls.add.controller'  : Ctrl('ManufacturerForm'),
            'cls.add.template'    : Tmpl('manufacturer_form.wt'),
            'obj.view.controller' : Ctrl('ManufacturerForm'),
            'obj.view.template'   : Tmpl('manufacturer_form.wt'),
            'obj.edit.controller' : Ctrl('ManufacturerForm'),
            'obj.edit.template'   : Tmpl('manufacturer_form.wt'),
        })

    class person(web.EntitySitemap):
        match = 'person/'
        clsObj = Const(ShopPerson)
        handler = Const('comp4.shop.web.PersonHandler')
        asMainMenu = Const(u'Osoby')
        class cls(web.EntitySitemap.cls):
            class popup(ClassRoute):
                def actionLinks(self):
                    return [web.Link(
                        label='X', pathInfo="#",
                        onclick="return wmPopupHide(event)"
                    )]
                actionLinks = Lazy(actionLinks)
            actionLinks = Lazy(lambda s: web.actionLinks(
                          s, '', './add', web.DELIMITER, mainMenu(s)))
        class obj(web.EntitySitemap.obj):
            prefix = Const('shop_person')
            goodsLstCtrl = Const('comp4.shop.controllers.GoodsLst')
            goodsAdmLstCtrl = Const('comp4.shop.controllers.GoodsAdmLst')
            goodsListing = Ctrl('GoodsLst')
            class cms(cms.AttachedCmsSitemap):
                match = 'cms/'
                prefix = Const('shop_person')
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', 
                web.Link(label='Dokumenty', pathInfo = s.pathInfo + 'cms/'),
                './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s)
            ))
            asPath = Lazy(
                lambda s: web.Link.fromRoute(s, label=s.obj.name), inherit=False
            )
        patch({
            'cls.list.controller' : Ctrl('PersonLst'),
            'cls.list.template'   : Tmpl('person_list.wt'),
            'cls.popup.controller': Ctrl('PersonLst'),
            'cls.popup.template'  : Tmpl('person_popup.wt'),
            'cls.add.controller'  : Ctrl('PersonForm'),
            'cls.add.template'    : Tmpl('person_form.wt'),
            'obj.view.controller' : Ctrl('PersonForm'),
            'obj.view.template'   : Tmpl('person_form.wt'),
            'obj.edit.controller' : Ctrl('PersonForm'),
            'obj.edit.template'   : Tmpl('person_form.wt'),
        })

    class news(Route):
        match = 'news/'
        handler = Const('comp4.shop.web.news_handler')
        template = Tmpl('news.wt')
        location_id = Const('shop.news')
        signature = Signature('location_id')
        for_date = Param(lambda s: datetime.date.today())
        limit = Const(10)
        class explicit_date(Route):
            match = ':for_date'
            for_date = Param(typ=datetime.date)
            signature = Signature('location_id', 'for_date')

    class reclaim(Route):
        match = 'reclaim/'
        handler = Const('comp4.shop.web.reclaim_handler')
        template = Tmpl('reclaim.wt')
        controller = Ctrl('ReclaimForm')
        location_id = Const('shop.reclaim')
        signature = Signature('location_id')
        
    class basket(Route):
        match = 'basket/'
        virtual = True
        clsObj = Const(ShopBasket)
        handler = Const('comp4.shop.web.BasketHandler')

        class view(ListRoute):
            template = Tmpl('basket_view.wt')

        class update(ClassRoute):
            pass

        class remove(ClassRoute):
            pass

        class order_customer(ClassRoute):
            template = Tmpl('order_customer.wt')

        class order_delivery(ClassRoute):
            template = Tmpl('order_delivery.wt')
    
        class order_payment(ClassRoute):
            template = Tmpl('order_payment.wt')

        class order_confirm(ClassRoute):
            template = Tmpl('order_confirm.wt')

        class order_result(ClassRoute):
            template = Tmpl('order_result.wt')

    class customer(web.EntitySitemap):
        match = 'customer/'
        clsObj = Const(ShopCustomer)
        handler = Const('comp4.shop.web.CustomerHandler')
        viewPerm = Const(web.GroupName('admin'))
        asMainMenu = Const(u'Kontakty')

        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                            s, '', './add', web.DELIMITER, mainMenu(s)))
            class popup(ClassRoute):
                def actionLinks(self):
                    return [web.Link(
                        label='X', pathInfo="#",
                        onclick="return wmPopupHide(event)"
                    )]
                actionLinks = Lazy(actionLinks)
        class obj(web.EntitySitemap.obj):
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s)
            ))

        patch({
           'cls.list.template'   :Tmpl('customer_list.wt'),
           'cls.list.controller' :Ctrl('CustomerLst'),
           'cls.add.template'    :Tmpl('customer_form.wt'),
           'cls.popup.template'  :Tmpl('customer_popup.wt'),
           'obj.view.template'   :Tmpl('customer_form.wt'),
           'obj.edit.template'   :Tmpl('customer_form.wt'),
        })

    class invoice(web.EntitySitemap):
        match = 'invoice/'
        clsObj = Const(ShopInvoice)
        handler = Const('comp4.shop.web.InvoiceHandler')
        viewPerm = Const(web.GroupName('admin'))
        asMainMenu = Const(u'Objednávky')

        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                            s, '', './add', web.DELIMITER, mainMenu(s)))
        class obj(web.EntitySitemap.obj):
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s)
            ))

        class view_pub(Route):
            match = 'code-:token'
            def token(self):
                return str(ShopInvoiceToken(
                    pkey   = self.obj.pkey,
                    secret = settings.secret(),
                ))
            token = Param(token, typ=str)
            def oid(self):
                k = self.token
                while not k.endswith('=='):
                    k = k + '='
                try:
                    t = ShopInvoiceToken.unpack(k, secret=settings.secret())
                except:
                    raise web.NotFound('cannot verify invoice token', k)
                return t.pkey
            oid = Param(oid)
            obj = Param(lambda s: web.findEntity(s, s.clsObj, s.oid))
            action = Const('view_pub')
            template = Tmpl('order_result.wt')
            signature = Signature('obj', 'action')

        patch({
            'cls.list.template'   :Tmpl('invoice_list.wt'),
            'cls.list.controller' :Ctrl('InvoiceLst'),
            'cls.add.template'    :Tmpl('invoice_form.wt'),
            'obj.view.template'   :Tmpl('invoice_view.wt'),
            'obj.edit.template'   :Tmpl('invoice_form.wt'),
        })

    class discount_ticket_batch(web.EntitySitemap):
        match = 'discount_ticket_batch/'
        clsObj = Const(ShopDiscountTicketBatch)
        handler = Const('comp4.shop.web.DiscountTicketBatchHandler')
        viewPerm = Const(web.GroupName('admin'))
        asMainMenu = Const(u'Slevové kupóny')

        class cls(web.EntitySitemap.cls):
            actionLinks = Lazy(lambda s: web.actionLinks(
                            s, '', './add', web.DELIMITER, mainMenu(s)))
        class obj(web.EntitySitemap.obj):
            class csv(InstanceRoute):
                permission = Param(lambda s: s.editPerm)
            class xls(InstanceRoute):
                permission = Param(lambda s: s.editPerm)
            class gen(InstanceRoute):
                permission = Param(lambda s: s.editPerm)
            actionLinks = Lazy(lambda s: web.actionLinks(
                s, '', './edit', './delete', web.DELIMITER,
                '../', '../add', web.DELIMITER, mainMenu(s)
            ))

        patch({
            'cls.list.template'   :Tmpl('discount_ticket_batch_list.wt'),
            'cls.list.controller' :Ctrl('DiscountTicketBatchLst'),
            'cls.add.template'    :Tmpl('discount_ticket_batch_form.wt'),
            'obj.view.template'   :Tmpl('discount_ticket_batch_view.wt'),
            'obj.edit.template'   :Tmpl('discount_ticket_batch_form.wt'),
        })


    class my(Route):
        match = 'my/'
        virtual = True
        locId = Const('myshop')
        permission = Const(web.Authenticated(auth_domain='registered_user'))

        class orders(Route):    
            match = 'orders'
            handler = Const('comp4.shop.web.MyHandler')
            action = Const('orders')
            signature = Signature('locId', 'action')
            template = Tmpl('my_orders.wt')

        class bookmarks(Route):    
            match = 'bookmarks'
            handler = Const('comp4.shop.web.MyHandler')
            action = Const('bookmarks')
            signature = Signature('locId', 'action')
            template = Tmpl('my_bookmarks.wt')
