# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.testing import *
from comp4.shop.api import *
from lib4.timeutils import *
import os

TEST_INI = '''\
# -*- coding: utf-8

[Plugin comp4.sendmail:DEFAULT]
port = 2525

[comp4.shop]
goods_params = ['foo', 'bar', 'baz']

[Plugin comp4.shop.goods_type:book]
title = u"Book"
params = 'foo', 'bar'

[Plugin comp4.shop.goods_type:multimedia]
title = u"Multimédia"
params = 'foo', 'baz'

[comp4.shop]
goods_types = ('book', 'multimedia')



[Plugin comp4.shop.goods_param:foo]
title = "Foo"

[Plugin comp4.shop.goods_param:bar]
factory = "comp4.shop.parameters.IntegerParam"
title = u"Barč"
lowerBound = 1
upperBound = 3

[Plugin comp4.shop.goods_param:baz]
title = u"Báz"
'''



class TestOrder(MockWorkspaceCase):
    
    cfgURLs = (
        'pkgfile:wm4/core.ini',
        'pkgfile:comp4.cms/comp.ini',
        'pkgfile:comp4.shop/comp.ini',
        naming.data_url(TEST_INI),
    )

    def setUp(self):
        super(TestOrder, self).setUp()
        from wm4.testing.smtp import MockSmtpServer
        self.smtpd = MockSmtpServer(port=2525).spawn()

    def tearDown(self):
        if self.smtpd is not None:
            self.smtpd.exit()
        super(TestOrder, self).tearDown()


    def expects_invoice(self, pkey, **kw):
        select = ('pkey, customer, delivery, payment, postage, cod, '
                  'packaging, discount, discount_ticket, status, '
                  'created, ext_invoice, note')
        fields = [s.strip() for s in select.split(',')]
        for f in ['postage', 'cod', 'packaging', 'discount']:
            kw.setdefault(f, 0.0)
        kw.setdefault('created', Now().isoformat())
        rec = [kw.get(f) for f in fields]
        rec[0] = pkey
        self.expects(
            sql='SELECT %s FROM shop_invoice WHERE pkey = %d' % (select, pkey),
            res = [rec],
        )

    def expects_invoice_items(self, pkey, res):
        select = 'pkey, invoice, goods, unit, quantity, unit_price'
        fields = [s.strip() for s in select.split(',')]
        r = []
        for idx, rec in enumerate(res):
            rec['invoice'] = pkey
            rec.setdefault('pkey', idx)
            rec.setdefault('unit', 'piece')
            rec.setdefault('quantity', 1)
            r.append([rec.get(f) for f in fields])
        self.expects(sql=(
         'SELECT %s FROM shop_invoice_item WHERE invoice = %d' % (select,pkey)),
         res = r
        )

    def expects_customer(self, pkey, **kw):
        select = ('pkey, reg_user, name, email, phone, web, company, '
                  'ico, dic, street, city, zip, country, dstreet, dcity, '
                  'dzip, dcountry')
        fields = [s.strip() for s in select.split(',')]
        kw['pkey'] = pkey
        kw.setdefault('name', 'Jan Novák')
        kw.setdefault('street', 'Dílce 1637')
        kw.setdefault('city', 'Golčův Jeníkov')
        kw.setdefault('zip', '45677')
        rec = [kw.get(f) for f in fields]
        self.expects(
            sql='SELECT %s FROM shop_customer WHERE pkey = %d' % (select, pkey),
            res = [rec],
        )

    def expects_goods(self, pkey, **kw):
        select = ('pkey, typ, name, short, description, price, price_common, '
                  'price_action, not_avail, note, hidden, deleted')
        fields = [s.strip() for s in select.split(',')]
        kw['pkey'] = pkey
        rec = [kw.get(f) for f in fields]
        self.expects(
            sql='SELECT %s FROM shop_goods WHERE pkey = %d' % (select, pkey),
            res = [rec],
        )

    def test10_EmailOrderAck(self):
        self.expects(transaction='begin')
        self.expects_invoice(234, customer=12)
        self.expects_customer(
            12, email='cust@example.com', dzip='12333',
            company='Solmundr, s.r.o.')
        self.expects_invoice_items(234, [
            dict(goods=10, unit_price=100),
            dict(goods=11, unit_price=12.50),
        ])
        self.expects_goods(
            10, name='Zbozi s velmi dlouhym jmenem foo bar baz foo bar baz foo'
        )
        self.expects_goods(11, name='Zbozicko')
        self.expects(transaction='commit')
        storage.begin()
        inv = self.ws.get(ShopInvoice, 234)
        url = 'http://asdf.cz/order/234/'
        self.ws['shop'].sendmail_order_ack(inv, url)
        storage.commit()

        m, = self.smtpd.messages()
        print m.msg
        print
        print m.get('From')
        print m.get('To')
        print m.get('Subject')
        print
        print
        print m.get_payload()
        print
        print


suite = Suite(TestOrder)

if __name__ == '__main__':
    Main()
