# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.testing import *
from comp4.shop.api import *
import os

TEST_INI = '''\
# -*- coding: utf-8
[comp4.shop]
goods_params = ['foo', 'bar', 'baz']

[Plugin comp4.shop.goods_type:book]
title = u"Book"
params = 'foo', 'bar'

[Plugin comp4.shop.goods_type:multimedia]
title = u"Multimédia"
params = 'foo', 'baz'

[comp4.shop]
goods_types = ('book', 'multimedia')



[Plugin comp4.shop.goods_param:foo]
title = "Foo"

[Plugin comp4.shop.goods_param:bar]
factory = "comp4.shop.parameters.IntegerParam"
title = u"Barč"
lowerBound = 1
upperBound = 3

[Plugin comp4.shop.goods_param:baz]
title = u"Báz"
'''


class TestParamConfig(ConfiguredTestCase):
    
    cfgURLs = (
        'pkgfile:wm4/core.ini',
        'pkgfile:comp4.cms/comp.ini',
        'pkgfile:comp4.shop/comp.ini',
        naming.data_url(TEST_INI),
    )

    def test01_ParameterDefinition(self):
        EQ = self.assertEqual
        ws = settings.workspace()

        p = ws['shop'].goodsParams
        EQ(set(p.keys()), set(['foo', 'bar', 'baz']))

        foo = p['foo']
        EQ(foo.name, 'foo')
        EQ(foo.title, 'Foo')
        EQ(foo.lowerBound, 0)
        EQ(foo.upperBound, 1)

        bar = p['bar']
        EQ(bar.name, 'bar')
        EQ(bar.title, u'Barč')
        EQ(bar.lowerBound, 1)
        EQ(bar.upperBound, 3)
    
    def test02_GoodsTypes(self):
        EQ = self.assertEqual
        ws = settings.workspace()

        t = ws['shop'].goodsTypes
        p = ws['shop'].goodsParams
        EQ(t.keys(), ['book', 'multimedia'])

        book = t['book']
        EQ(book.name, 'book')
        EQ(book.title, 'Book')
        EQ(book.params, (p['foo'], p['bar']))

        multi = t['multimedia']
        EQ(multi.name, 'multimedia')
        EQ(multi.title, u'Multimédia')
        EQ(multi.params, (p['foo'], p['baz']))







class TestParams(MockWorkspaceCase):
    
    cfgURLs = (
        'pkgfile:wm4/core.ini',
        'pkgfile:comp4.cms/comp.ini',
        'pkgfile:comp4.shop/comp.ini',
        naming.data_url(TEST_INI),
    )

    def test10_Get(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
          "SELECT "
              "pkey, typ, name, short, description, price, price_common, "
              "price_action, not_avail, note, hidden, deleted, manufacturer"
          " FROM shop_goods WHERE pkey = 2"),
          res=[[
          2,'goods','G2','D2','D2',20.0,None,None,None,None,False,False,None
          ]]
        )
        self.expects(sql=(
          "SELECT pkey, goods, param, value, position "
          "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
          res=[
                [21, 2, 'foo', 'FOO', 1],
                [24, 2, 'bar', '201', 2],
                [25, 2, 'bar', '202', 3]],
        )
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        self.failUnless(isinstance(g.params, PersistentParamDict))
        EQ(sorted(g.params), ['bar','baz','foo'])
        self.failUnless(isinstance(g.params['foo'], PersistentParamList))
        self.failUnless(isinstance(g.params['bar'], PersistentParamList))
        self.failUnless(isinstance(g.params['baz'], PersistentParamList))
        EQ(g.params['foo'], ['FOO'])
        EQ(g.params['foo'][0], 'FOO')
        EQ(g.params['bar'], [201, 202])
        EQ(g.params['bar'][0], 201)
        EQ(g.params['bar'][1], 202)
        EQ(g.params['baz'], [])

        storage.commit()

    def test12_GetattrIface(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
          "SELECT "
              "pkey, typ, name, short, description, price, price_common, "
              "price_action, not_avail, note, hidden, deleted, manufacturer"
          " FROM shop_goods WHERE pkey = 2"),
          res=[[
          2,'goods','G2','D2','D2',20.0,None,None,None,None,False,False,None
          ]]
        )
        self.expects(sql=(
          "SELECT pkey, goods, param, value, position "
          "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
          res=[
                [21, 2, 'foo', 'FOO', 1],
                [24, 2, 'bar', '201', 2],
                [25, 2, 'bar', '202', 3]],
        )
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        EQ(g.params.foo, 'FOO')
        EQ(g.params.bar, [201, 202])
        EQ(g.params.baz, None)
        self.assertRaises(AttributeError, getattr, g.params, 'neex')
        storage.commit()

    def test20_NewWithoutParams(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql="SELECT NEXTVAL('shop_goods_pkey_seq')", res=[[2]])
        self.expects(sql=(
          "INSERT INTO shop_goods ("
            "pkey, typ, name, short, description, price, price_common, "
            "price_action, not_avail, note, hidden, deleted, manufacturer"
          ") VALUES ("
          "2, 'book', 'GN', 'DN', 'DN', NULL, NULL, NULL, NULL, NULL, "
          "'f', 'f', NULL)"))
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.new(ShopGoods, typ='book', name='GN', description='DN')
        EQ(g._oid, 2)
        EQ(g.pkey, 2)
        self.failUnless(isinstance(g.params, PersistentParamDict))
        EQ(sorted(g.params), ['bar','baz','foo'])
        self.failUnless(isinstance(g.params['foo'], PersistentParamList))
        self.failUnless(isinstance(g.params['bar'], PersistentParamList))
        self.failUnless(isinstance(g.params['baz'], PersistentParamList))
        EQ(g.params['foo'], [])
        EQ(g.params['bar'], [])
        EQ(g.params['baz'], [])
        storage.commit()

    def test21_NewWithParams(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql="SELECT NEXTVAL('shop_goods_pkey_seq')", res=[[2]])
        self.expects(sql=(
          "INSERT INTO shop_goods ("
            "pkey, typ, name, short, description, price, price_common, "
            "price_action, not_avail, note, hidden, deleted, manufacturer"
          ") VALUES ("
          "2, 'book', 'GN', 'DN', 'DN', NULL, NULL, NULL, NULL, NULL, "
          "'f', 'f', NULL)"))
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')", res=[[42]])
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (42, 2, 'foo', 'FOO', 0)"))
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')", res=[[43]])
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (43, 2, 'bar', '201', 0)"))
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')", res=[[44]])
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (44, 2, 'bar', '202', 1)"))
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.new(
            ShopGoods, typ='book', name='GN', description='DN',
            params={'foo':['FOO'], 'bar':[201, 202]}
        )
        EQ(g._oid, 2)
        EQ(g.pkey, 2)
        self.failUnless(isinstance(g.params, PersistentParamDict))
        EQ(sorted(g.params), ['bar','baz','foo'])
        self.failUnless(isinstance(g.params['foo'], PersistentParamList))
        self.failUnless(isinstance(g.params['bar'], PersistentParamList))
        self.failUnless(isinstance(g.params['baz'], PersistentParamList))
        EQ(g.params['foo'], ['FOO'])
        EQ(g.params['bar'], [201,202])
        EQ(g.params['baz'], [])
        storage.commit()

    def test30_SetValue(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(
            sql=("SELECT pkey, goods, param, value, position "
                "FROM shop_goods_param WHERE pkey = 21 FOR UPDATE"),
            res=[[21, 2, 'foo', 'FOO', 0]],
        )
        self.expects(
            sql="UPDATE shop_goods_param SET value='BAR' WHERE pkey = 21")
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        g.params['foo'] = ['BAR']
        EQ(g.params['foo'], ['BAR'])
        storage.commit()

    def test31_SetList(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(
            sql=("SELECT pkey, goods, param, value, position "
                "FROM shop_goods_param WHERE pkey = 24 FOR UPDATE"),
            res=[[24, 2, 'bar', '201', 0]],
        )
        self.expects(
            sql="UPDATE shop_goods_param SET value='211' WHERE pkey = 24")
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        EQ(g.params['bar'], [201, 202])
        g.params['bar'][0] = 211
        EQ(g.params['bar'], [211,202])
        storage.commit()

    def test32_ManyInsert(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(
            sql=("SELECT pkey, goods, param, value, position "
                "FROM shop_goods_param WHERE pkey = 25 FOR UPDATE"),
            res=[[25, 2, 'bar', '202', 1]],
        )
        self.expects(
            sql="UPDATE shop_goods_param SET value='203' WHERE pkey = 25")
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')",
            res=[[42]],
        )
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (42, 2, 'bar', '202', 2)"
        ))
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        g.params['bar'].insert(1, 203)
        EQ(g.params['bar'], [201,203,202])
        storage.commit()

    def test33_ManyRemove(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql="DELETE FROM shop_goods_param WHERE pkey = 25")
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        g.params['bar'].pop()
        EQ(g.params['bar'], [201])
        storage.commit()

    def test40_FeatureUpdatedByDict(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(
            sql=("SELECT pkey, goods, param, value, position "
                "FROM shop_goods_param WHERE pkey = 25 FOR UPDATE"),
            res=[[25, 2, 'bar', '202', 1]],
        )
        self.expects(
            sql="UPDATE shop_goods_param SET value='203' WHERE pkey = 25")
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')",
            res=[[42]],
        )
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (42, 2, 'bar', '204', 2)"
        ))
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')",
            res=[[43]],
        )
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (43, 2, 'baz', 'BAZ', 0)"
        ))
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        g.params = {'foo':['FOO'], 'bar':[201,203,204], 'baz':['BAZ']}
        self.failUnless(isinstance(g.params, PersistentParamDict))
        EQ(sorted(g.params), ['bar','baz','foo'])
        self.failUnless(isinstance(g.params['foo'], PersistentParamList))
        self.failUnless(isinstance(g.params['bar'], PersistentParamList))
        self.failUnless(isinstance(g.params['baz'], PersistentParamList))
        EQ(g.params['foo'], ['FOO'])
        EQ(g.params['bar'], [201,203,204])
        EQ(g.params['baz'], ['BAZ'])
        storage.commit()

    def test42_Setattr(self):
        EQ = self.assertEqual

        self.expects(transaction='begin')
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2"),
           res=[[
           2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(sql=(
            "SELECT pkey, goods, param, value, position "
            "FROM shop_goods_param WHERE goods = 2 ORDER BY position"),
            res=[
                [21, 2, 'foo', 'FOO', 0],
                [24, 2, 'bar', '201', 0],
                [25, 2, 'bar', '202', 1]],
        )
        self.expects(sql=(
            "SELECT "
                "pkey, typ, name, short, description, price, price_common, "
                "price_action, not_avail, note, hidden, deleted, manufacturer"
            " FROM shop_goods WHERE pkey = 2 FOR UPDATE"),
          res=[[
          2,'goods','G2',None,'D2',20.0,None,None,None,None,False,False,None]]
        )
        self.expects(
            sql=("SELECT pkey, goods, param, value, position "
                "FROM shop_goods_param WHERE pkey = 25 FOR UPDATE"),
            res=[[25, 2, 'bar', '202', 1]],
        )
        self.expects(
            sql="UPDATE shop_goods_param SET value='203' WHERE pkey = 25")
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')",
            res=[[42]],
        )
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (42, 2, 'bar', '204', 2)"
        ))
        self.expects(
            sql="SELECT NEXTVAL('shop_goods_param_pkey_seq')",
            res=[[43]],
        )
        self.expects(sql=(
          "INSERT INTO shop_goods_param (pkey, goods, param, value, position) "
          "VALUES (43, 2, 'baz', 'BAZ', 0)"
        ))
        self.expects(transaction='commit')

        storage.begin()
        g = self.ws.get('shop/Goods', 2)
        g.params.bar = [201,203,204]
        g.params.baz = 'BAZ'
        self.failUnless(isinstance(g.params['foo'], PersistentParamList))
        self.failUnless(isinstance(g.params['bar'], PersistentParamList))
        self.failUnless(isinstance(g.params['baz'], PersistentParamList))
        EQ(g.params['foo'], ['FOO'])
        EQ(g.params['bar'], [201,203,204])
        EQ(g.params['baz'], ['BAZ'])
        storage.commit()



suite = Suite(TestParamConfig, TestParams)

if __name__ == '__main__':
    Main()
