from xml4.textutils import text_to_html, HTML_FRAGMENT_ENVELOP

__all__ = ['tidy_description', 'extract_short_description']


def tidy_html(descr, log):
    if 'tidy' in log:
        raise RuntimeError('recursive-tidy')
    from lib4.htmlproc import tidyHTML
    html = tidyHTML(descr, add_xml_decl=1, indent=0)
    log.append('tidy')
    return html


def body_fragment(descr, log):
    log.append('fragment-slice')
    s = descr.find('<body>')
    e = descr.find('</body>')
    if s <= 0:
        raise ValueError('cannot find body start')
    if e <= 0:
        raise ValueError('cannot find body end')

def tidy_description(descr, log):
    if not descr:
        return None
    if '<' not in descr:
        return text_to_html(descr)
    else:
        from xml4.parser import SimpleXMLParser as SP
        from xml.parsers import expat
        sp = SP()
        if '<html' in descr or '<head' in descr or '<body' in descr:
            log.append('full-html')
            html = descr
            frag = False
        else:
            html = HTML_FRAGMENT_ENVELOP % descr
            frag = True
        try:
            sp.parse(html.encode('utf-8'), encoding='utf-8')
        except:
            log.append('parse-error')
            html = tidy_html(html, log)
            return tidy_descr(html, log)
        else:
            if not frag:
                return body_fragment(html, log)
            else:
                return descr

def extract_short_description(descr, maxlen=150):
    if not descr:
        return None
    from xml4.textutils import html_to_text
    txt = html_to_text(descr, fragment=True)
    return txt[:maxlen]
