# -*- coding: utf-8 -*-
from wm4.api import *
from lib4.imports import importObject
from comp4.files.web import makeResponse
import re

from entities import *
from controllers import *

__all__ = [
    "CategoryHandler", "GoodsHandler",
    "PhotogalleryHandler", "BasketHandler", "CustomerHandler",
    "InvoiceHandler", "MyHandler", "DiscountTicketBatchHandler",
]

_ = i18n.MessageIDFactory('comp4.shop')


class CategoryHandler(web.EntityHandler):
    entity = binding.Constant(ShopCategory)
    formCtrl = binding.Constant(CategoryForm)
    obj = binding.Make(lambda s: s.getObject())
    
    def do_list(self):
        return self.render(categories=self.workspace['shop'].categories)

    def do_list_adm(self):
        return self.do_list()

    do_popup = do_list

    def do_view(self, obj, goodsListing):
        req = self.req
        ws = self.workspace
        shop = ws['shop']
        lst = importObject(goodsListing)(req, baseQuery=obj.goods_query());lst()
        items = shop.preloadGoods(lst.items)
        shop.preloadSupercategories(obj)
        tmplname = obj.template or 'DEFAULT'
        tmpl = req.getResource(
            config.Property('comp4.shop.category.template.%s' % tmplname)()
        )
        return tmpl.handleHTTP(req, obj=self.obj, listing=lst)
        
    #@web.with_template()
    def category_move_goods(req):
        ws = settings.workspace()
        obj = ws.get(ShopCategory, int(req.params['oid']))
        target = None
        if req.getValue('target'):
            target = ws.get(ShopCategory, int(req.getValue('target')))
            if req.getValue('confirm'):
                ws('''[
                    query.Update(gc, category=target)
                    for gc in ShopGoodsCategory if gc.category == obj
                ]''')
                storage.commit()
                storage.begin()
                raise web.SeeOther(req.get_url(target))
        return dict(
            obj = obj,
            target = target,
            category_tree = ws['shop'].categories,
        )
        



class GoodsHandler(web.EntityHandler):
    entity = ShopGoods
    formCtrl = binding.Make(lambda s: s.req.params.get('formCtrl', GoodsForm))

    token = binding.Make(lambda s: s.req.getCookie('wm4sess'))
    basket = binding.Make(lambda s: ShopBasket(token=s.token))
    regUser = binding.Make(lambda s: s.req.getUser('registered_user'))

    def do_list(self):
        ctrl = importObject(self.req.params['controller'])
        lst = ctrl(self.req); lst()
        return self.render(listing=lst)

    do_list_adm = do_search = do_list

    def do_view(self):
        obj = self.obj
        req = self.req
        if obj.deleted:
            raise web.NotFound()
        listingCtrl = importObject(req.params['listingCtrl'])
        listing = listingCtrl(req)
        listing.setFilters(obj)
        listing.formURL = req.get_url(obj.__class__)
        return self.render(obj=obj, listing=listing)

    def do_detail(self):
        return self.do_view()

    def do_delete(self):
        ws = self.workspace
        try:
            self.obj.deleted = True
            storage.commit()
            storage.begin()
        except:
            raise
        else:
            return self.deleteResult(self.obj)

    def do_bookmark(self, req, obj):
        if not self.regUser:
            raise web.NotAllowed()
        b = self.workspace.new(ShopGoodsBookmark,
            goods = obj,
            reg_user = self.regUser,
            note = req.getValue('note', None),
            wish = req.getValue('wish', None),
        )
        storage.commit()
        storage.begin()

        redir = req.environ.get('HTTP_REFERER')
        if not redir:
            redir = req.get_url(obj)
        raise web.SeeOther(redir)

    def do_basket(self):
        req = self.req
        obj = self.obj
        redir = req.environ.get('HTTP_REFERER')
        if not redir:
            redir = req.get_url(ShopBasket)
        i = self.workspace.new(ShopBasketItem,
            token = self.token,
            goods = obj,
            quantity = 1,
            unit_price = obj.order_price,
        )
        storage.commit()
        storage.begin()
        raise web.SeeOther(redir)

    def do_comments(self):
        obj = self.obj
        if obj.deleted:
            raise web.NotFound()
        req = self.req
        commentPerm = req.allows(req.params['editPerm'])

        if commentPerm and req.getValue('delete'):
            for p in req.getList('comment'):
                c = obj.commentDict[int(p)]
                c.deleted = True
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.requestURL)

        return self.render(obj=obj, commentPerm=commentPerm)

    def do_add_comment(self):
        if self.obj.deleted:
            raise web.NotFound()
        form = GoodsCommentForm(ctrlReq=self.req, goods=self.obj)
        pid = self.req.params['parent_id']
        if pid is not None:
            form.parent_id = pid
        if form():
            storage.commit()
            storage.begin()
            return self.editResult(self.obj)
        else:
            if form.formErrors:
                storage.abort()
                storage.begin()
            return self.render(form=form, obj=self.obj, mode='edit')

    def do_popup(self):
        return self.do_list()

    def do_order(self):
        shop = self.req.environ['comp4.shop']
        token = self.req.environ['comp4.shop.token']
        quant = int(self.req.getValue('quantity', 1))
        item = self.workspace.new(BasketItem,
            token = str(token),
            goods = self.obj,
            quantity = quant,
            unit_price = self.obj.price,
        )
        storage.commit()
        storage.begin()
        raise web.SeeOther(self.req.get_url(BasketItem))

    def do_xml_zbozi_seznam(self):
        tns = {'mode': 'view',
               'goods_list': self.workspace('''[
                    g for g in ShopGoods
                       if g.hidden == False
                      and g.deleted == False
                    ]'''),
        }

        req = self.req
        tmpl = req.getResource(req.params['template'])
        resp = req.response
        enc = req.output_encoding
        output = tmpl.render(req, **tns).encode(enc)
        resp.addheader('Content-type', 'text/xml;charset=%s' % (
                                    req.output_encoding))
        resp.addheader('Content-Length', str(len(output)))
        resp.body = [output]
        return resp






@web.with_template()
def news_handler(req):
    for_date = req.params['for_date']
    news = settings.workspace()['shop'].allNews()
    return dict(news=news, for_date=for_date)



class PhotogalleryHandler(web.EntityHandler):

    entity = ShopPhotogalleryImage

    def do_view(self):
        return makeResponse(self.req, self.req.params['obj'], 'inline')

    def do_set(self):
        import lib4.images as img4

        req = self.req
        param = req.getValue('param')
        value = req.getValue('value')
        upload = req.getUpload('image')
        img = img4.loadImage(upload.path)

        if param == 'oid':
            goods = [self.workspace.get(ShopGoods, value)]
        else:
            goods = list(self.workspace('''[
               g for g in ShopGoods
                 for p in ShopGoodsParam
                  if p.goods == g
                 and p.param == param
                 and p.value == value
            ]'''))

        body = []
        shop = self.workspace['shop']
        for g in goods:
            body.append('GOODS %s\n' % g.pkey)
            shop.imageForGoods(g, img, 'front', upload.filename or str(g.pkey))
        
        res = req.response
        res.addheader('Content-type', 'text/plain')
        res.body = body
        return res



class FileHandler(web.EntityHandler):
    entity = ShopFile
    def do_view(self):
        return makeResponse(self.req, self.req.params['obj'], 'attachment')


class ImageHandler(web.EntityHandler):
    entity = ShopImage
    def do_view(self):
        return makeResponse(self.req, self.req.params['obj'], 'inline')


class GoodsCommentHandler(web.EntityHandler):
    entity = ShopGoodsComment
    
    def do_list(self):
        ctrl = importObject(self.req.params['controller'])
        lst = ctrl(self.req); lst()
        return self.render(listing=lst)






class ManufacturerHandler(web.BaseHandler):

    def do_view(self, obj, goodsListing):
        req = self.req
        ws = self.workspace
        shop = ws['shop']
        lst = importObject(goodsListing)(req, baseQuery=obj.goods_query());lst()
        items = shop.preloadGoods(lst.items)
        #shop.preloadSupercategories(obj)
        return self.render(obj=obj, listing=lst)

class PersonHandler(ManufacturerHandler):
    pass









class BasketHandler(web.Handler):
    
    token = binding.Make(lambda s: s.req.getCookie('wm4sess'))

    basket = binding.Make(lambda s: ShopBasket(token=s.token))

    regUser = binding.Make(
        lambda s: web.getUser(s.req.environ,'registered_user'))

    shop = binding.Make(lambda s: s.workspace['shop'])

    def do_view(self):
        return self.render(
            obj = self.basket,
            regUser = self.regUser,
            ShopBasket = ShopBasket,
            ShopBasketItem = ShopBasketItem,
        )

    def do_update(self):
        req = self.req
        basket = self.basket
        if req.getValue('recalc'):
            for item in basket.items:
                q = int(req.getValue('quantity%s' % item.pkey))
                if q < 0:
                    q = 1
                item.quantity = q
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url(ShopBasket))

        elif req.getValue('order'):
            items = []
            for item in basket.items:
                if req.getValue('invoice%s' % item.pkey):
                    items.append(str(item.pkey))
            req.session['shop.items'] = items
            raise web.SeeOther(req.get_url(ShopBasket, 'order_customer'))

        elif req.getValue('delete'):
            for item in basket.items:
                if req.getValue('invoice%s' % item.pkey):
                    self.workspace.remove(item)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url(ShopBasket))
            
        elif req.getValue('bookmark'):
            for item in basket.items:
                if req.getValue('invoice%s' % item.pkey):
                    b = self.workspace.new(ShopGoodsBookmark,
                        reg_user = self.regUser,
                        goods = item.goods,
                    )
                    self.workspace.remove(item)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url(ShopBasket))

        else:
            raise NotImplementedError

    def do_remove(self):
        req = self.req
        pkey = int(req.getValue('item'))
        for item in self.basket.items:
            if item.pkey == pkey:
                self.workspace.remove(item)
        storage.commit()
        storage.begin()
        raise web.SeeOther(req.get_url(ShopBasket))
    
    def do_order_customer(self):
        req = self.req
        reg_user = req.params['reg_user']
        sess = req.session

        old = req.session.get('shop.customer')

        if reg_user == None:
            prev_cust_records = []
        else:
            prev_cust_records = list(self.workspace(
                '[c for c in ShopCustomer if c.reg_user == reg_user]'
            ))

        trade = req.getValue('trade', default='').strip()
        trade_msg = None
        if not trade:
            trade_msg = u'Pro pokračování musíte souhlasit s obchodními podmínkami'
        sel = req.getValue('sel', default=None)
        if sel is None:
            trade_msg = None
            if old is None:
                form = CustomerPublicForm(ctrlReq=req)
            else:
                if old.pkey == -1:
                    form = CustomerPublicForm(ctrlReq=req, oldValue=old)
                else:
                    form = CustomerPublicForm(ctrlReq=req)
                sel = old.pkey
        elif sel == '-1':
            sel = int(sel)
            if old is None:
                form = CustomerPublicForm(ctrlReq=req)
            else:
                form = CustomerPublicForm(ctrlReq=req, oldValue=old)
            if form():
                cust = form.newValue
                req.session['shop.customer'] = cust
                raise web.SeeOther(req.get_url(ShopBasket, 'order_delivery'))
        else:
            if not trade_msg:
                sel = int(sel)
                for p in prev_cust_records:
                    if sel == p.pkey:
                        req.session['shop.customer'] = p
                        raise web.SeeOther(
                            req.get_url(ShopBasket,'order_delivery')
                        )
            form = CustomerPublicForm(ctrlReq=req)

        return self.render(
            basket=self.basket,
            form=form,
            sel=sel,
            prev_cust_records=prev_cust_records,
            ShopBasket=ShopBasket,
            ShopBasketItem=ShopBasketItem,
            trade_msg = trade_msg,
        )
        
    def do_order_delivery(self):
        req = self.req
        errmsgs = []
        ticket_errors = []

        shop = self.workspace['shop']

        if req.getValue('doSubmit'):

            delivery = req.getValue('delivery')
            payment = req.getValue('payment')
            note = req.getValue('note')
            if not delivery:
                errmsgs.append(u'Nebyl vybrán způsob dodání')
            if not payment:
                errmsgs.append(u'Nebyl vybrán způsob platby')
            if not errmsgs:
                dp = (delivery, payment)
                if dp not in shop.delivery_payment:
                    errmsgs.append(
                        u'Nepovolená kombinace způsobu platby a dodání')

            ticket_code = req.getValue('ticket_code')
            if ticket_code:
                ticket = -self.workspace.find(
                    ShopDiscountTicket, code=ticket_code)
                if ticket is None or not ticket.free:
                    ticket_errors.append(u'Neplatný slevový kód')

            if not errmsgs and not ticket_errors:
                req.session['shop.delivery'] = delivery
                req.session['shop.payment'] = payment
                req.session['shop.ticket_code'] = ticket_code
                req.session['shop.note'] = note
                raise web.SeeOther(req.get_url(ShopBasket, 'order_confirm'))
        else:
            delivery = req.session.get('shop.delivery')
            payment = req.session.get('shop.payment')
            ticket_code = req.session.get('shop.ticket_code')
            note = req.session.get('shop.note')

        basket = self.basket
        bi = []
        basket_json = dict(
            items = bi,
            total = basket.total,
        )
        for i in basket.items:
            bi.append(dict(
                goods = i.goods._oid,
                unit  = i.unit,
                quantity = i.quantity,
                unit_price = i.unit_price,
            ))

        dp_json = {}
        for (d,p), i in shop.delivery_payment.items():
            dp_json['%s+%s' % (d,p)] = i(basket)
        taxes_json = [t for (t,n) in shop.taxes]
        from simplejson import dumps
        return self.render(
            shop                  = shop,
            delivery              = delivery,
            payment               = payment,
            ticket_code           = ticket_code,
            note                  = note,
            basket                = basket,
            basket_json           = dumps(basket_json, ensure_ascii=False),
            delivery_payment_json = dumps(dp_json, ensure_ascii=False),
            taxes_json            = dumps(taxes_json, ensure_ascii=False),
            errmsgs               = errmsgs,
            ticket_errors         = ticket_errors,
            ShopBasket            = ShopBasket,
            ShopBasketItem        = ShopBasketItem,
        )

    def do_order_confirm(self):
        req = self.req
        ws = self.workspace
        shop = ws['shop']
        cust = req.session['shop.customer']
        payment = req.session['shop.payment']
        delivery = req.session['shop.delivery']
        ticket_code = req.session['shop.ticket_code']
        note = req.session['shop.note']
        ticket = None
        if ticket_code:
            ticket = -self.workspace.find(
                ShopDiscountTicket, code=ticket_code)
            if ticket is None or not ticket.free:
                errmsgs.append(u'Neplatný slevový kód')

        selected = [int(i) for i in req.session['shop.items']]
        items = [i for i in self.basket.items if i.pkey in selected]
        total = sum([(i.quantity * i.unit_price) for i in items])

        total_total = total
        _taxes = shop.get_taxes(delivery, payment, self.basket)
        taxes = {}
        for k,v in _taxes.items():
            if isinstance(v, (float, int)):
                total_total += v
                v = req.params['renderMoney'](req, v)
            taxes[k] = v
        if ticket is not None:
            total_total -= ticket.compute_discount(total)

        if req.getValue('doSubmit'):

            if cust.pkey == -1:
                atts = cust.__dict__.copy()
                del atts['pkey']
                del atts['invoices']
                atts['reg_user'] = web.getUser(req.environ,'registered_user')
                cust = ws.new(ShopCustomer, **atts)
            else:
                cust = ws.get(ShopCustomer, cust.pkey)
                
            invoice = dict(
                customer = cust,
                delivery = delivery,
                payment = payment,
                status = InvoiceStatus.new,
                note = note,
            )
            for t in ('postage', 'cod', 'packaging'):
                v = _taxes.get(t)
                if isinstance(v, (int,float)):
                    invoice[t] = v
            if ticket:
                invoice['discount'] = ticket.compute_discount(total)
                invoice['discount_ticket'] = ticket

            invoice = ws.new(ShopInvoice, **invoice)
            for it in items:
                sii = ws.new(ShopInvoiceItem, 
                    invoice = invoice,
                    goods = it.goods,
                    unit = it.unit,
                    quantity = it.quantity,
                    unit_price = it.unit_price,
                )
                ws.remove(it)
            pub_url = req.get_url(invoice, 'view_pub')
            adm_url = str(req.get_url(invoice))
            storage.commit()
            storage.begin()
            # XXX pokud se posila email pred commitem a objednavka
            #     ma slevu, tak neni dobre spoctena celkova cena
            try:
                shop.sendmail_order_ack(invoice, str(pub_url))
            except:
                req.logger.exception('cannot mail order_ack')
            try:
                shop.sendmail_order_adm(invoice, str(adm_url))
            except:
                req.logger.exception('cannot mail order_adm')
            raise web.SeeOther(str(pub_url % {'show_steps':'1'}))

        return self.render(
            shop = shop,
            cust = cust,
            delivery = self.shop.delivery_dict[delivery],
            payment = self.shop.payment_dict[payment],
            taxes = taxes,
            items = items,
            total = total,
            total_total = total_total,
            ticket = ticket,
            ShopBasket=ShopBasket,
            ShopBasketItem=ShopBasketItem,
        )












class CustomerHandler(web.EntityHandler):
    entity = ShopCustomer
    formCtrl = binding.Constant(CustomerForm)

    def do_list(self):
        ctrl = importObject(self.req.params['controller'])
        lst = ctrl(self.req); lst()
        return self.render(listing=lst)

    do_popup = do_list




class InvoiceHandler(web.EntityHandler):
    entity = ShopInvoice
    formCtrl = binding.Constant(InvoiceForm)
    
    def do_list(self):
        ctrl = importObject(self.req.params['controller'])
        lst = ctrl(self.req); lst()
        return self.render(listing=lst)

    def do_view_pub(self):
        return self.render(
            obj=self.req.params['obj'],
            show_steps=self.req.getValue('show_steps',False),
        )






class DiscountTicketBatchHandler(web.EntityHandler):
    entity = ShopDiscountTicketBatch
    formCtrl = binding.Constant(DiscountTicketBatchForm)

    def do_list(self):
        ctrl = importObject(self.req.params['controller'])
        lst = ctrl(self.req); lst()
        return self.render(listing=lst)

    def do_csv(self):
        import csv, StringIO

        res = self.req.response
        obj = self.req.params['obj']
        f = StringIO.StringIO()
        w = csv.writer(f)
        w.writerow(['davka', 'kod', 'objednavka', 'blokovany'])
        for t in obj.tickets:
            w.writerow([
                obj.name.encode('utf-8'),
                t.code, t.invoice and t.invoice.pkey or '',
                t.blocked and 'ano' or 'ne'
            ])
        data = f.getvalue()
        res.addheader('Content-type', 'text/comma-separated-values')
        res.addheader('Content-Disposition',
            'attachment; filename=slevove_kody.csv')
        res.addheader('Content-length', str(len(data)))
        res.body = [data]
        return res

    def do_xls(self):
        import pyExcelerator, tempfile, os

        wb = pyExcelerator.Workbook()
        ws = wb.add_sheet(u'Slevové kupóny')
        ws.write(0, 0, u'dávka')
        ws.write(0, 1, u'kód')
        ws.write(0, 2, u'objednávka')
        ws.write(0, 3, u'blokovaný')

        res = self.req.response
        obj = self.req.params['obj']

        for i,t in enumerate(obj.tickets):
            ws.write(1+i, 0, obj.name)
            ws.write(1+i, 1, t.code)
            ws.write(1+i, 2, t.invoice and t.invoice.pkey or '')
            ws.write(1+i, 3, t.blocked and 'ano' or 'ne')

        fd, path = tempfile.mkstemp('.xls')
        try:
            wb.save(path)
            data = open(path).read()
        finally:
            os.close(fd)
            os.unlink(path)

        res.addheader('Content-type', 'application/vnd.ms-excel')
        res.addheader('Content-Disposition',
            'attachment; filename=slevove_kody.xls')
        res.addheader('Content-length', str(len(data)))
        res.body = [data]
        return res
    
    def do_gen(self):
        req = self.req
        count = int(req.getValue('count', '100'))
        assert count <= 1000
        obj = req.params['obj']
        obj.gen(count)
        url = str(self.req.get_url(obj))
        storage.commit()
        storage.begin()
        raise web.SeeOther(url)






class MyHandler(web.Handler):

    regUser = binding.Make(
        lambda s: web.getUser(s.req.environ,'registered_user'))

    token = binding.Make(lambda s: s.req.getCookie('wm4sess'))
    basket = binding.Make(lambda s: ShopBasket(token=s.token))
    
    def do_orders(self):
        return self.render(
            regUser = self.regUser,
            orders = self.workspace['shop'].ordersForUser(self.regUser),
        )

    def do_bookmarks(self):
        req = self.req
        bookmarks = self.workspace['shop'].bookmarksForUser(self.regUser)

        if req.getValue('update'):
            for b,g in bookmarks:
                note = req.getValue('note%d' % b.pkey)
                wish = req.getValue('wish%d' % b.pkey)
                if b.note != note:
                    b.note = note
                if wish == '1':
                    if b.wish != True:
                        b.wish = True
                else:
                    if b.wish != False:
                        b.wish = False
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url('myshop','bookmarks'))

        elif req.getValue('basket'):
            for b,g in bookmarks:
                if req.getValue('bookmark%s' % b.pkey):
                    bi = self.workspace.new(ShopBasketItem,
                        token = self.token,
                        goods = g,
                        quantity = 1,
                        unit_price = g.order_price,
                    )
                    self.workspace.remove(b)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url('myshop','bookmarks'))

        elif req.getValue('delete'):
            for b,g in bookmarks:
                if req.getValue('bookmark%s' % b.pkey):
                    self.workspace.remove(b)
            storage.commit()
            storage.begin()
            raise web.SeeOther(req.get_url('myshop','bookmarks'))


        return self.render(
            regUser = self.regUser,
            bookmarks = bookmarks,
        )


def reclaim_handler(req, controller, template):
    form = importObject(controller)(ctrlReq=req)
    if form():
        raise NotImplementedError
    return web.render_template(req, template, form=form)
