# -*- coding: utf-8 -*-

from lib4.imports import importObject
from wm4.api import *
from wm4.web.forms.controllers import SelectCtrl, ContainerCtrl, OnChangeEvent

from features import parameters_manager



__all__ = ['SelectParamCtrl', 'SingleParametersCtrl']



class SelectParamCtrl(SelectCtrl):

    paramDef = binding.Require()

    def selectOptions(self):
        opts = []
        for o in self.paramDef.selectOptions:
            if isinstance(o, (tuple,list)):
                t, l, o = o
            else:
                t = l = o
            opts.append((t,l,o))
        return opts
    selectOptions = binding.Make(selectOptions)





class SingleParametersCtrl(ContainerCtrl):

    plugin = binding.Require()

    paramDefs = binding.Make(lambda s: parameters_manager()[s.plugin])

    def _ctrlSetupChildren(self):
        features = {}
        oldValue = self.oldValue
        for name, pdef in self.paramDefs:
            attrs = {}
            ov = getattr(oldValue, name, NOT_GIVEN)
            if ov is not NOT_GIVEN:
                attrs['oldValue'] = ov
            if pdef.title:
                attrs['ctrlLabel'] = pdef.title
            if pdef.lowerBound > 0:
                attrs['ctrlRequired'] = True
            ctrlCls = importObject(pdef.controller)
            if hasattr(ctrlCls, 'paramDef'):
                attrs['paramDef'] = pdef
            features[name] = ctrlCls(self.ctrlReq, name, self, **attrs)
        features.update(super(SingleParametersCtrl,self)._ctrlSetupChildren())
        return features

    def commitSelf(self):
        values = {}
        for name, pdef in self.paramDefs:
            elem = self[name]
            values[name] = elem.newValue
        self.newValue = values
        OnChangeEvent(self, old=self.oldValue, new=self.newValue)
        return True
