from lib4.imports import importObject
from wm4.api import *
from wm4.schema.containers import PersistentList, PersistentDict

from parameters import UnicodeParam

__all__ = [
    'PersistentParamList',
    'SinglePersistentParamDict', 'ManyPersistentParamDict',
    'ParametersManager', 'parameters_manager', 
    'SingleParametersFeature', 'ManyParametersFeature'
]



class PersistentParamList(PersistentList):
    
    _p_dict = None
    _p_param = None

    def _p_register(self, _):
        self._p_dict._p_changed



class SinglePersistentParamDict(PersistentDict):
    
    __metaclass__ = binding.Activator

    def __init__(self, initdict=None, **kw):
        PersistentDict.__init__(self, initdict)
        self.__dict__.update(kw)

    obj = oldValues = paramDefs = binding.Require()

    def _p_item_register(self, pl):
        self._p_changed

    def _p_register(self, _):
        self.obj.onChange({}, {})

    def __getattr__(
        self, attr,
        iattrs = set(('obj', 'oldValues', 'paramDefs')),
        oga = PersistentDict.__getattribute__
    ):
        if attr in iattrs:
            return oga(self, attr)
        pdef = oga(self, 'paramDefs').get(attr)
        if pdef is None:
            raise AttributeError(attr)
        return self[attr]

    def __setattr__(self, attr, value):
        pdef = self.paramDefs.get(attr)
        if pdef is None:
            raise AttributeError(attr)
        self[attr] = value



class ManyPersistentParamDict(SinglePersistentParamDict):

    def _p_wrap(self, k, v):
        pl = PersistentParamList(v)
        pl._p_dict = self
        return k, pl

    def __getattr__(
        self, attr,
        iattrs = set(('obj', 'oldValues', 'paramDefs')),
        oga = PersistentDict.__getattribute__
    ):
        if attr in iattrs:
            return oga(self, attr)
        pdef = oga(self, 'paramDefs').get(attr)
        if pdef is None:
            raise AttributeError(attr)
        vals = self[attr]
        if pdef.upperBound == 1:
            if vals:
                return vals[0]
            else:
                return None
        else:
            return vals

    def __setattr__(self, attr, value):
        pdef = self.paramDefs.get(attr)
        if pdef is None:
            raise AttributeError(attr)
        if pdef.upperBound == 1:
            value = [value]
        self[attr] = value


class ParametersManager(binding.Component):

    cache = binding.Make(dict)

    def __getitem__(self, plugin):
        params = self.cache.get(plugin)
        if params is not None:
            return params
        params = []
        plugins = config.pluginsFor(plugin)
        for name in config.Property(plugin)(default=()):
            pdef = plugins.get(name, {}).copy()
            factory = importObject(pdef.pop('factory', UnicodeParam))
            params.append((name, factory(name=name, **pdef)))
        params = self.cache[plugin] = schema.SortedFeatureDict(params)
        return params

parameters_manager = config.Resource('parameters_manager', ParametersManager)








class SingleParametersFeature(schema.Feature):
    cardinality = 'dict'
    refPrimitives = ()
    plugin  = None
    typeRef = None
    typeObj = None
    refAttr = None
    _pdict = SinglePersistentParamDict

    copyAttrs=schema.Feature.copyAttrs+('plugin','typeRef','typeObj','refAttr')

    def __init__(
        self, plugin, typeRef, refAttr, *metadata, **kw
    ):
        super(SingleParametersFeature, self).__init__(None, *metadata, **kw)
        self.plugin = plugin
        self.typeRef = typeRef
        self.refAttr = refAttr

    paramDefs = property(lambda s: parameters_manager()[s.plugin])

    def activateFeature(self, parent, attr):
        cp = super(SingleParametersFeature, self).activateFeature(parent, attr)
        cp.typeObj = schema.getTypeObj(cp, cp.typeRef)
        cp.plugin = self.plugin
        return cp

    def default(self, obj, values={}):
        # XXX check values
        paramDefs = parameters_manager()[self.plugin]
        pdict = self._pdict(
            dict((p,None) for p in paramDefs.iterkeys()),
            obj = obj, 
            paramDefs = paramDefs,
            oldValues = {},
        )
        return pdict

    def oldValues(self, obj):
        ov = {}
        if obj._workspace is None:
            return ov
        for p in obj._workspace('''[
            p for p in self.typeObj if p.%s == obj
        ]''' % self.refAttr):
            ov[p.param] = p
        return ov
        
    def get(self, obj):
        odct = obj.__dict__
        attr = self.attrName
        pdict = odct.get(attr)
        if pdict is None:
            paramDefs = parameters_manager()[self.plugin]
            values = {}
            oldValues = self.oldValues(obj)
            for p, pd in paramDefs.iteritems():
                v = oldValues.get(p)
                if v is None:
                    values[p] = pd.unpickle(obj, None)
                else:
                    values[p] = pd.unpickle(obj, v.value)
            pdict = odct[attr] = self._pdict(
                values,
                obj       = obj, 
                paramDefs = paramDefs,
                oldValues = oldValues,
            )
        return pdict

    def preload(klass, obj, _oldValues):
        attr = klass.attrName
        if attr in obj.__dict__:
            return
        paramDefs = parameters_manager()[self.plugin]
        values = {}
        oldValues = {}
        for p in _oldValues:
            oldValues.setdefault(p.param, p)
        for p, pd in paramDefs.iteritems():
            v = oldValues.get(p)
            if v is None:
                values[p] = pd.unpickle(obj, None)
            else:
                values[p] = pd.unpickle(obj, v.value)
        obj.__dict__[attr] = self._pdict(
            values,
            obj       = obj, 
            paramDefs = paramDefs,
            oldValues = oldValues,
        )
        
    def set(self, obj, val):
        # XXX check values
        attr = self.attrName
        paramDefs = parameters_manager()[self.plugin]
        pdict = obj.__dict__.get(attr)
        if pdict is not None:
            obj.onChange({}, {})
        if isinstance(val, SinglePersistentParamDict):
            obj.__dict__[attr] = val
        elif isinstance(val, dict):
            if pdict is None:
                oldValues = self.oldValues(obj)
            else:
                oldValues = pdict.oldValues
            for p,d in paramDefs.iteritems():
                val.setdefault(p, None)
            obj.__dict__[attr] = self._pdict(
                val,
                obj       = obj, 
                paramDefs = paramDefs,
                oldValues = oldValues,
            )
        else:
            raise AttributeError('Invalid params container')

    def delete(self, obj):
        # delete all attributes?
        self.set(obj, {})

    def afterUpdate(self, obj):
        pdict = obj.__dict__.get(self.attrName)
        if pdict is None:
            return
        ee = schema.EntityError()
        ws = obj._workspace
        for name, pdef in pdict.paramDefs.iteritems():
            inst = pdict.oldValues.get(name, None)
            val = pdict[name]
            obj = pdict.obj
            try:
                pval = pdef.pickle(pdict, val)
            except schema.SchemaError, se:
                ee.add(se)
                continue
            change = False
            if inst is None:
                if pval is not None:
                    values = dict(
                        param = name,
                        value = pval,
                    )
                    values[self.refAttr] = obj
                    # XXX try?
                    ws.new(self.typeObj, **values)
            elif inst.value != pval:
                if pval is None:
                    ws.remove(inst)
                else:
                    inst.value = pval
                    change = True
            if change:
                ws.flush(inst)
        if ee:
            ne = schema.EntityError()
            ne.add(schema.NestedEntityError(self.attrName, ee))
            raise ne

    afterAdd = afterUpdate




class ManyParametersFeature(schema.Feature):
    pass
    # TODO
