# http://en.wikipedia.org/wiki/Entity-Attribute-Value_model

from wm4.api import *
from lib4.imports import importObject

__all__ = [
    'AbstractParam', 'UnicodeParam', 'HtmlParam', 'IntegerParam',
    'SelectParam', 'BoolParam', 'ReferenceParam',
]


class AbstractParam(object):

    name = None
    title = None
    description = None
    help = None
    autoview = True
    fulltext = None # weight A/B/C/D

    controller = 'wm4.web.forms.controllers.InputCtrl'
    type = unicode
    lowerBound = 0
    upperBound = 1

    def __init__(self, **kw):
        self.__dict__.update(kw)
        if not self.title:
            self.title = self.name
    
    def pickle(self, obj, val):
        if val is None: 
            return None
        #if val == 'ahoj':
        #    raise schema.FeatureError(self.name, val)
        return unicode(val)

    def unpickle(self, obj, val):
        if val is None: 
            return None
        try:
            return self.type(val)
        except:
            raise ValueError('cannot unpickle', val, self.type)
   
    def view(self, obj, val):
        return val

    def add(self, obj, val):
        raise NotImplementedError

    def update(self, obj, val, old):
        raise NotImplementedError

    def remove(self, obj, val):
        raise NotImplementedError

    def __repr__(self):
        return '%s(name=%r)' % (self.__class__.__name__, self.name)

class UnicodeParam(AbstractParam):
    type = unicode

class HtmlParam(AbstractParam):
    type = unicode
    controller = 'wm4.web.forms.controllers.HtmlCtrl'

class IntegerParam(AbstractParam):
    type = int


class SelectParam(AbstractParam):
    selectOptions = []
    controller = 'comp4.simpleparams.controllers.SelectParamCtrl'

class BoolParam(AbstractParam):

    controller = 'wm4.web.forms.controllers.BoolCtrl'

    def pickle(self, obj, val):
        if val is None:
            return None
        return val and 't' or 'f'

    def unpickle(self, obj, val):
        if val is None:
            return None
        if val == 't' : return True
        else: return False

    def view(self, obj, val):
        if val is True:
            return u'ano' # XXX
        elif val is False:
            return u'ne' # XXX
        return None



class ReferenceParam(AbstractParam):

    refEntity = None

    def __init__(self, **kw):
        super(ReferenceParam, self).__init__(**kw)
        self.refEntity = importObject(self.refEntity)
    
    def pickle(self, obj, val):
        if val is None:
            return None
        return val._oid

    def unpickle(self, obj, val):
        if val is None or not val:
            return None
        return settings.workspace().get(self.refEntity, val)
