#!/usr/bin/env python
# -*- coding: utf-8 -*-


import datetime
import lxml.etree as ET

from wm4.api import *
from wm4.schema.api import *

from adt4.serial import Serial



__all__ = [
        'SnowCache',
        'SnowComponent',
]



class SnowRegion(object):
    def __init__(self, **kw):
        for k, d in kw.iteritems():
            setattr(self, k, d)
        self.resorts = []

class SnowResort(object):
    def __init__(self, **kw):
        for k, d in kw.iteritems():
            setattr(self, k, d)

    def __getattr__(self, attr):
        return ''

    def snow_height(self):
        try:
            vs = '%s - %s cm' % (self.snowheight_slopes_min, self.snowheight_slopes_max)
        except:
            vs = ''
        return vs
    snow_height = binding.Make(snow_height)

binding.activateClass(SnowResort)




class SnowCache(Entity):
    pkey = Attribute(Serial)
    acquired = Attribute(datetime.datetime, defaultDatetime)
    data = Attribute(unicode)

    def parse_xml(self):
        regions = {}
        resorts = {}

        snow_xml = ET.fromstring(self.data)
        for region in  snow_xml.xpath('/export/country/region'):
            name = region.attrib['name']
            reg_obj = SnowRegion(name=name, info='')
            regions[name] = reg_obj

            for resort in region.xpath('location'):
                attrs = {'name': resort.attrib['name'],
                         'region': reg_obj}
                acronym = resort.attrib.get('alias', '')
                attrs.update(dict([(at.tag.lower(), (at.text or '').strip())
                       for at in resort.find('loc_info_winter').getchildren()]))
                res_obj = SnowResort(**attrs)
                reg_obj.resorts.append(res_obj)
                resorts[acronym] = res_obj
        return dict(regions=regions, resorts=resorts)
    regions = resorts = binding.MultiMake(parse_xml)

    binding.metadata(PrimaryKey('pkey'))




class SnowComponent(schema.SchemaComponent):


    def acquireData(self):
        ws = settings.workspace()
        url = config.Property('snowreport.xml_data_url')()
        snow_xml = ET.parse(url)
        obj = SnowCache(data=ET.tounicode(snow_xml))
        ws.add(obj)
        return obj 

    def lastData(self):
        ws = settings.workspace()
        q = ws.parse('''[sc for sc in 'snow/SnowCache'
                                 if query.OrderBy(query.Desc(sc.pkey))]''')
        q.limit = query.Const(1)
        obj = ws(q)
        if not obj:
            obj = self.acquireData()
        else:
            obj = list(obj)[0]
        return obj




