BEGIN;

CREATE TABLE tag (
    pkey        SERIAL NOT NULL,
    ident       VARCHAR,
    name        VARCHAR NOT NULL,
    parent      INTEGER,
    position    INTEGER NOT NULL,

    PRIMARY KEY (pkey),
    UNIQUE(ident)
);

CREATE TABLE tag_object (
    pkey        SERIAL NOT NULL,
    tag         INTEGER NOT NULL,
    klass       VARCHAR NOT NULL,
    key         VARCHAR NOT NULL,
    priority    FLOAT NOT NULL,

    PRIMARY KEY (pkey)
);


CREATE TABLE tag_tree_cache (
    "begin"   INTEGER NOT NULL,
    "end"     INTEGER NOT NULL,
    "edges"   INTEGER NOT NULL,
    "path"    VARCHAR NOT NULL,

    PRIMARY KEY("begin", "end")
);

CREATE VIEW tag_paths AS
    SELECT k AS pkey, p AS path,
           array_to_string(array_accum(n), ' / ') AS titles
      FROM (
                SELECT tt.path, tt."end", t.name
                  FROM (
                         SELECT *,
                         array_iter(string_to_array(path, ',')) AS pelem
                         FROM tag_tree_cache
                       ) AS tt, tag t
                 WHERE (tt.pelem).n = t.pkey
                   AND tt."begin" IN (SELECT pkey FROM tag WHERE parent ISNULL)
                 ORDER BY (tt.pelem).p

           ) AS x (p,k,n)
     GROUP BY p, k
;


CREATE OR REPLACE FUNCTION gen_tree_paths (TEXT, TEXT, TEXT, TEXT)
RETURNS TEXT LANGUAGE plpythonu AS $BODY$

    srctable, srckey, srcparent, dsttable = args

    branches = {}
    nodes = []
    q = "SELECT %s, %s FROM %s" % (srckey, srcparent, srctable)
    for row in plpy.execute(q):
        k = row[srckey]
        p = row[srcparent]
        branches.setdefault(p, []).append(k)
        nodes.append(k)

    def iter_depth_first(node, path=()):
        assert node is not None
        path = path + (node,)
        yield path
        for child in branches.get(node, ()):
            for p in iter_depth_first(child, path):
                yield p

    plpy.execute("DELETE FROM %s" % dsttable)
    for start in nodes:
        for path in iter_depth_first(start):
            plpy.execute((
                'INSERT INTO %s ("begin", "end", "edges", "path") '
                "VALUES ('%s','%s','%s','%s')") % (
                dsttable, path[0], path[-1], len(path)-1,
                ','.join([str(p) for p in path])
            ))

    return "OK"
$BODY$;


-- SELECT gen_tree_paths('tag', 'pkey', 'parent', 'tag_tree_cache');

END;
