# -*- coding: utf-8 -*-

from wm4.api import *
from adt4.serial import Serial

from lib4.trees import AbstractTreeNode
import comp4.trees.api as trees

__all__ = ['Tag', 'TagObject', 'TagTreeCache',
           'EntityWithTags',
           'TagComponent']

_ = i18n.MessageIDFactory('comp4.tags')


class Tag(schema.Entity, AbstractTreeNode):
    pkey = schema.Attribute(Serial, schema.Required)
    name = schema.Attribute(unicode, schema.Required)
    ident = schema.Attribute(unicode)
    parent = schema.Reference(schema.THIS_CLASS)
    position = schema.Attribute(int, schema.Required, schema.Default(100))
    
    children = schema.ReferenceEnd(schema.THIS_CLASS, 'parent',
                                   schema.ReferenceEndOrder('position'))

    binding.metadata(schema.PrimaryKey('pkey'))


    @schema.setattrs.before('"parent" in added')
    def check_circle(self, added, removed):
        if not added['parent']: return
        if self in added['parent'].iterParents():
            raise schema.FeatureError("parent", msgId=_(u"Circle reference"))




class TagObject(schema.Entity):
    pkey = schema.Attribute(Serial, schema.Required)
    tag = schema.Reference('comp4.tags.entities.Tag', schema.Required)
    klass = schema.Attribute(str, schema.Required)
    key = schema.Attribute(str, schema.Required)
    priority = schema.Attribute(float, schema.Required, schema.Default(1.0))

    binding.metadata(schema.PrimaryKey('pkey'))


class TagTreeCache(trees.TreeCache):
    pass


class EntityWithTags(object):
    def tags(self):
        tags = []
        tag_objs = []
        for t, to in settings.workspace()('''[
            (t, to) for t in 'tags/Tag'
                    for to in 'tags/TagObject'
                     if to.tag == t
                    and to.klass == self._workspace.getPath(self.__class__)
                    and to.key == self._oid
            ]'''):
            tags.append(t)
            tag_objs.append(to)
        return locals()
    tags = tag_objs = binding.MultiMake(tags)
binding.activateClass(EntityWithTags)


class TagComponent(trees.TreeComp):

    txnAttrs = ('branches', 'tags', 'idents', 'roots')

    def children_and_tags(self):
        self.joinedTxn
        ws = settings.workspace()

        tags = {}
        branches = {}
        idents = {}
        for tag in ws('[t for t in Tag if query.OrderBy(t.position,t.name)]'):
            tags[tag.pkey] = tag
            branches.setdefault(tag.parent, []).append(tag)
            tag.__dict__['children'] = ()
            if tag.ident:
                idents[tag.ident] = tag
        for parent, branch in branches.iteritems():
            if parent is None:
                continue
            parent.__dict__['children'] = tuple(branch)
        roots = branches.get(None, [])
        return locals()
    tags = branches = idents = roots = binding.MultiMake(children_and_tags)


    def cache_tags(self, objs):
        if not objs: return
        ws = settings.workspace()
        odict = {}
        cls = None
        for o in objs:
            if cls is None: cls = o.__class__
            if o.__class__ <> cls: continue
            odict[o._oid] = o
            object.__setattr__(o, 'tags', [])
            object.__setattr__(o, 'tag_objs', [])

        oids = odict.keys()
        for t, to in ws('''[
            (t, to) for t in 'tags/Tag'
                    for to in 'tags/TagObject'
                     if to.tag == t
                    and to.klass == ws.getPath(cls)
                    and to.key in oids
            ]'''):
            odict[int(to.key)].tags.append(t)
            odict[int(to.key)].tag_objs.append(to)



class GenTagTreeCache(query.Procedure):
    def __init__(self, *args):
        self.args = args

@storage.PgSqlConnection.renderExpr.when('expr in GenTagTreeCache')
def render_gen_tag_tree_cache(self, expr, relations):
    return '''gen_tree_paths('tag', 'pkey', 'parent', 'tag_tree_cache')'''


@schema.Workspace.do_add.after('obj in Tag')
def lokalita_add_edge(ws, obj):
    ws("GenTagTreeCache()")

@schema.Workspace.do_update.after('obj in Tag')
def lokalita_update_edge(ws, obj):
    ws("GenTagTreeCache()") 

@schema.Workspace.do_remove.after('obj in Tag')
def lokalita_update_edge(ws, obj):
    ws("GenTagTreeCache()") 




