from wm4.api import *
from lib4.trees import AbstractTreeNode
import sys



__all__ = [
    'TreeNode', 'TreeCache', 'TreeComp', 'GenTreePaths', 'gen_tree_paths']


class TreeNode(schema.Entity, AbstractTreeNode):
    pkey = schema.Attribute(int)
    parent = schema.Reference(schema.THIS_CLASS)

    children = schema.ReferenceEnd(schema.THIS_CLASS,'parent')

    binding.metadata(schema.PrimaryKey('pkey'))

    def treeCache(klass):
        meta = type(schema.Entity)
        name = klass.__name__ + 'TreeCache'
        cls  = meta(
            name,
            (schema.Entity,),
            dict(
                begin = schema.Attribute(int),
                end = schema.Attribute(int),
                edges = schema.Attribute(int, schema.Required),
                path = schema.Attribute(str, schema.Required),
                __module__ = klass.__module__,
            )
        )
        setattr(sys.modules[klass.__module__], name, cls)
        binding.declareClassMetadata(cls, schema.PrimaryKey('begin', 'end'))
        return cls
    treeCache = binding.classAttr(binding.Make(treeCache))



class TreeCache(schema.Entity):
    begin = schema.Attribute(int)
    end   = schema.Attribute(int)
    edges = schema.Attribute(int, schema.Required)
    path  = schema.Attribute(str, schema.Required)

    binding.metadata(schema.PrimaryKey('begin', 'end'))



class TreeComp(schema.SchemaComponent):
    pass



class GenTreePaths(query.Procedure):
    def __init__(self, *args):
        self.args = args


@storage.PgSqlConnection.renderExpr.when('expr in GenTreePaths')
def render_gen_tree_paths(self, expr, relations):
        return 'gen_tree_paths(%s)' % (
            ','.join(self.renderExpr(a,relations) for a in expr.args)
        )




# XXX make triggers deferred to commit time

@schema.Workspace.do_add.after('obj in TreeNode')
def gen_tree_paths(ws, obj):
    t = storage.SqlTableName(obj.__class__)()
    ws("GenTreePaths('%s', 'pkey', 'parent', '%s_tree_cache')" % (t,t))

@schema.Workspace.do_update.after('obj in TreeNode')
def gen_tree_paths(ws, obj):
    t = storage.SqlTableName(obj.__class__)()
    ws("GenTreePaths('%s', 'pkey', 'parent', '%s_tree_cache')" % (t,t))

@schema.Workspace.do_remove.after('obj in TreeNode')
def gen_tree_paths(ws, obj):
    t = storage.SqlTableName(obj.__class__)()
    ws("GenTreePaths('%s', 'pkey', 'parent', '%s_tree_cache')" % (t,t))
