# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.schema.api import *
from adt4.serial import Serial
from adt4.enums import Enum, enum
import comp4.files.api as files

__all__ = [
    'recordKind', 'User', 'UserPhoto', 'Group', 'UserGroup', 'UserContext'
]

_ = i18n.MessageIDFactory('comp4.users')


class recordKind(Enum):
    system = enum()
    user   = enum()


class User(Entity):
    protocols.advise(instancesProvide=[web.IUser])

    uid         = Attribute(Serial)
    login       = Attribute(str, Required, Syntax('^[a-z][._a-z0-9]{1,31}$'))
    nick        = Attribute(str, Syntax('^[a-z][._a-z0-9]{1,31}$'))
    password    = Attribute(unicode)
    name        = Attribute(unicode, Required)
    email       = Attribute(str, Required)
    phone       = Attribute(unicode)
    mobil       = Attribute(unicode)
    skype       = Attribute(unicode)
    workplace   = Attribute(unicode)
    occupation  = Attribute(unicode)
    note        = Attribute(unicode)
    kind        = Attribute(recordKind, Required, Default(recordKind.user))
    active      = Attribute(bool, Required, Default(True))
    hidden      = Attribute(bool, Required, Default(False))
    photo       = Reference('comp4.users.entities.UserPhoto')

    binding.metadata(
        PrimaryKey('uid'), Unique('login'), Unique('nick'),
    )


    def _groups(self):
        _groups = []
        groups = []
        for ug, g in self._workspace('''[
            (ug, g) for ug in 'users/UserGroup'
                    for g  in 'users/Group'
                     if ug.user._oid == self._oid
                    and ug.group == g
                    and query.OrderBy(g.name)
        ]'''):
            _groups.append(ug)
            groups.append(g)
        _groups = tuple(_groups)
        groups = tuple(groups)
        return locals()
    _groups = groups = binding.MultiMake(_groups)

    groupNames = binding.Make(lambda s: [g.name for g in s.groups])


class UserPhoto(files.Image):
    pass




class Group(Entity):
    protocols.advise(instancesProvide=[web.IGroup])

    gid         = Attribute(Serial)
    name        = Attribute(str, Required, Syntax('^[a-z][_a-z0-9]{1,31}$'))
    description = Attribute(unicode)
    kind        = Attribute(recordKind, Required, Default(recordKind.user))
    active      = Attribute(bool, Required, Default(True))
    hidden      = Attribute(bool, Required, Default(False))

    binding.metadata(PrimaryKey('gid'), Unique('name'))

    _users      = ReferenceEnd('comp4.users.entities.UserGroup', 'group')
    # XXX Preload and sort
    users       = binding.Make(lambda s:tuple([u.user for u in s._users]))


class UserGroup(Entity):
    user  = Reference('comp4.users.entities.User')
    group = Reference('comp4.users.entities.Group')

    binding.metadata(PrimaryKey('user', 'group'))


# XXX 
@schema.Workspace.do_add.before('obj in User')
def check_unique_features(ws, obj):
    ee = schema.EntityError()
    klass = obj.__class__
    for ufeat in klass.unique_features:
        cond = dict((f, getattr(obj,f)) for f in ufeat)
        if None in cond.values():
            continue
        if -ws.find(klass, **cond) is not None:
            for f in ufeat:
                ee.add(schema.FeatureError(
                    f, _(u'Tato hodnota již v databázi existuje')
                ))
    if ee:
        raise ee

@schema.Workspace.do_remove.before('obj in User')
def do_remove_user_group_by_user(ws, obj):
    ws('[query.Delete(ug) for ug in UserGroup if ug.user == obj]')

@schema.Workspace.do_remove.before('obj in Group')
def do_remove_user_group_by_group(ws, obj):
    ws('[query.Delete(ug) for ug in UserGroup if ug.group == obj]')





class UserContext(schema.SchemaComponent):
    
    workspace = binding.Make(lambda s: settings.workspace())

    def userByLogin(self, login):
        return dict([(u.login, u) for u in self.allUsers]).get(login, None)

    def groupByName(self, name):
        try:
            return ~self.workspace("[g for g in Group if g.name==name]")
        except exceptions.TooFewResults:
            return None
    
    def allUsers(self):
        return tuple(self.workspace(
            "[u for u in User if query.OrderBy(u.name)]"
        ))
    allUsers = storage.TxnAttr(allUsers)

    def allGroups(self):
        return tuple(self.workspace(
            "[u for u in Group if query.OrderBy(u.name)]"
        ))
    allGroups = storage.TxnAttr(allGroups)
