from wm4.api import *
from unittest import TestCase
from lib4.testing import testRoot
from wm4.utils.config import StringConfig


__all__ = ['setUpData', 'getWS', 'TestBase']

def setUpData (ws) :
    storage.begin(ws)
    for tab in ('user', 'group', 'user_group') :
        ws.sqlCon('DELETE FROM "%s"' % tab)
    for i in range(1,4) :
        ws.sqlCon((
          'INSERT INTO "user" (uid,login,name,email) '
          "VALUES (%d,'login%d','name%d','%d@example.com')"
          ) % (i, i, i, i)
        )
    for i in range(1,3) :
        ws.sqlCon(
          '''INSERT INTO "group" (gid,name) VALUES (%d,'group%d')''' % (i,i)
        )
    ws.sqlCon('INSERT INTO user_group ("user","group") VALUES (3,1)')
    ws.sqlCon('INSERT INTO user_group ("user","group") VALUES (3,2)')
    ws.sqlCon('INSERT INTO user_group ("user","group") VALUES (2,2)')
    ws.sqlCon("SELECT setval('user_uid_seq',3)")
    ws.sqlCon("SELECT setval('group_gid_seq',2)")
    storage.commit(ws)
    


def getWS () :
    global _ws
    try :
        return _ws
    except NameError:
        root = makeRoot(
            config.packageFile('comp4.files','comp.ini'),
            config.packageFile('comp4.users','comp.ini'),
            StringConfig(
                '[Named Services]\n'
                'wm4.storage.connection = naming.LinkRef("pgsql:///test")\n'
            )
        )
        _ws = Workspace(root)
        return _ws


class Workspace(binding.Component):
    
    User = binding.Obtain(schema.ManagerFor('comp4.users.entities.User'))
    Group = binding.Obtain(schema.ManagerFor('comp4.users.entities.Group'))
    UserGroup = binding.Obtain(
                    schema.ManagerFor('comp4.users.entities.UserGroup')
    )

    sqlCon = binding.Obtain(storage.CONNECTION)

    def __call__(self, qry):
        return self.sqlCon(qry)

class TestBase (TestCase) :
    
    _setup = False

    def setUp (self) :
        self.ws = getWS()
        klass = self.__class__
        if not klass._setup :
            setUpData(self.ws)
            klass._setup = True
    
    def tearDown (self) :
        ts = config.lookup(self.ws, storage.ITransactionService)
        if ts.isActive() :
            ts.abort()

