# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from comp4.users.api import *
from comp4.users.tests.common import *


class TestSimpleQueries (TestBase) :
    
    def test10_GetByUID (self) :
        ws = self.ws
        storage.begin(ws)
        u = ws.User.get(1)
        assert isinstance(u, User)
        self.assertEqual(schema.OID.get(u), 1)
        self.assertEqual(u.uid, 1)
        self.assertEqual(u.name, u'name1')
        storage.commit(ws)
        
    def test11_GhostReloading (self) :
        ws = self.ws
        storage.begin(ws)
        u = ws.User.get(1)
        storage.commit(ws)
        assert 1 in ws.User.cache
        assert u is ws.User.cache[1]
        self.assertEqual(schema.OID.get(u), 1)
        assert 'uid' not in u.__dict__
        assert 'name' not in u.__dict__
        storage.begin(ws)
        self.assertEqual(schema.OID.get(u), 1)
        self.assertEqual(u.uid, 1)
        self.assertEqual(u.name, u'name1')
        storage.commit(ws)

    def test22_GetNonexistingUID (self) :
        ws = self.ws
        storage.begin(ws)
        self.assertRaises(KeyError, ws.User.get, 999)

    def test23_GetInvalidUID (self) :
        ws = self.ws
        storage.begin(ws)
        self.assertRaises(KeyError, ws.User.get, 'a')
        self.assertRaises(KeyError, ws.User.get, (1,))

    def test30_QueryAll (self) :
        ws = self.ws
        rv = ws.User.relVar
        storage.begin(ws)
        res = dict([(schema.OID.get(u),u) for u in ws(rv)])
        self.assertEqual(len(res), 3)
        for i in range(1,4) :
            assert i in res
            u = res[i]
            assert schema.OID.get(u) == i
            assert u.uid == i
            assert u.name == u'name%d' % i
        storage.commit(ws)

    def test31_QueryWhere (self) :
        ws = self.ws
        rv = ws.User.relVar
        qr = rv.where(query.GT(rv.uid, query.CONST(1)))
        storage.begin(ws)
        res = dict([(schema.OID.get(u),u) for u in ws(qr)])
        self.assertEqual(len(res), 2)
        for i in range(2,4) :
            assert i in res
            u = res[i]
            assert schema.OID.get(u) == i
            assert u.uid == i
            assert u.name == u'name%d' % i
        storage.commit(ws)

    def test32_QueryWithSelectAndOrder (self) :
        ws = self.ws
        rv = ws.User.relVar
        qr = rv.where(
                query.GT(rv.uid, query.CONST(1))
             ).orderBy(
                query.DESC(rv.uid)
             ).select(
                rv.uid, rv.name
            )
        storage.begin(ws)
        res = list(ws(qr))
        self.assertEqual(res, [[3,u'name3'], [2,u'name2']])
        storage.commit(ws)

    def test33_QueryWithLimit (self) :
        ws = self.ws
        rv = ws.User.relVar
        qr = rv.where(
                query.GT(rv.uid, query.CONST(1))
             ).orderBy(
                query.DESC(rv.uid)
             ).select(
                rv.uid, rv.name
            )
        storage.begin(ws)
        res = list(ws.sqlCon.limit(qr, 1))
        self.assertEqual(res, [[2,u'name2']])
        storage.commit(ws)
        
    def test40_FindAll (self) :
        ws = self.ws
        storage.begin(ws)
        res = dict([(schema.OID.get(u),u) for u in ws.User.find()])
        self.assertEqual(len(res), 3)
        for i in range(1,4) :
            assert i in res
            u = res[i]
            assert schema.OID.get(u) == i
            assert u.uid == i
            assert u.name == u'name%d' % i
        storage.commit(ws)
        
    def test42_FindWhere (self) :
        ws = self.ws
        storage.begin(ws)
        res = dict([(schema.OID.get(u),u) for u in ws.User.find(uid=2)])
        self.assertEqual(len(res), 1)
        assert 2 in res
        u = res[2]
        assert schema.OID.get(u) == 2
        assert u.uid == 2
        assert u.name == u'name2'
        storage.commit(ws)
        




class TestJoinQueries (TestBase) :

    def test10_UserGroup (self) :
        ws = self.ws
        u = User.alias('u')
        g = Group.alias('g')
        ug = UserGroup.alias('ug')
        q = query.Join(
                query.Join(u, ug, query.EQ(u.uid, ug.user)),
                g,
                query.EQ(g.gid, ug.group)
            #).select(
            #    u, g
            ).orderBy(u.uid, g.gid)
        storage.begin(ws)
        res = list(ws(q))
        self.assertEqual(len(res), 3)
        (u1,ug1,g1), (u2,ug2,g2), (u3,ug3,g3) = res
        assert isinstance(u1, User); assert isinstance(g1, Group)
        self.assertEqual(u1.uid,2); self.assertEqual(g1.gid,2)
        self.assertEqual(u2.uid,3); self.assertEqual(g2.gid,1)
        self.assertEqual(u3.uid,3); self.assertEqual(g3.gid,2)
        storage.commit(ws)



class TestReferences (TestBase) :

    def test10_GroupList (self) :
        ws = self.ws
        storage.begin(ws)
        u = ws.User[3]
        ugd = dict([(schema.OID.get(ug), ug) for ug in u._groups])
        self.assertEqual(len(ugd), 2)
        assert (3,1) in ugd
        assert (3,2) in ugd
        storage.commit(ws)

    def test11_UserList (self) :
        ws = self.ws
        storage.begin(ws)
        g = ws.Group[2]
        ugd = dict([(schema.OID.get(ug), ug) for ug in g._users])
        self.assertEqual(len(ugd), 2)
        assert (2,2) in ugd
        assert (3,2) in ugd
        storage.commit(ws)



def suite() :
    return TestSuite((
        makeSuite(TestSimpleQueries),
        makeSuite(TestJoinQueries),
        makeSuite(TestReferences),
    ))



if __name__ == '__main__':
    main(defaultTest='suite')
