# -*- coding: utf-8 -*-

from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import *

from lib4.imports import importObject
from comp4.files.web import makeResponse

from entities import *

__all__ = ['UserDB', 'UserForm', 'UserHandler', 'GroupForm', 'GroupHandler']

_ = i18n.MessageIDFactory('comp4.users')


class UserDB:
    
    def getUser(self, login):
        return -settings.workspace().find(User, login=login, active=True)


def find_user(middleware, environ, username):
    return -settings.workspace().find(User, login=username, active=True)


def list_users():
    return settings.workspace()(
        '[(u.login, u.name) for u in User if query.OrderBy(u.name)]'
    )
        
    

class UserGroupCtrl(makeFormCtrl(UserGroup), SequenceItemCtrl):

    class user(InputCtrl):
        ctrlType = User
        ctrlRequired = True
        def defaultOutput(self):
            return self['../../..'].newValue
    
    class group(SelectCtrl):
        ctrlType = Group
        ctrlRequired = True
        def selectOptions(self):
            return list(settings.workspace()(
                '[(g.gid, g.name, g) for g in Group if query.OrderBy(g.name)]'
            ))
        selectOptions = binding.Make(selectOptions)


class UserForm(makeFormCtrl(User)):
    
    class uid(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')

    class password(ValidatingPasswordCtrl):
        pass

    class kind(InputCtrl):
        ctrlRequired = True
        ctrlEnabled = binding.Constant(False)
        def defaultValue(self):
            return recordKind.user
    
    class email(InputCtrl):
        ctrlWidget = web.FORM_WIDGET('input.hypertextize')
    
    class hidden(InputCtrl):
        ctrlRequired = True
        ctrlEnabled = binding.Constant(False)
        def defaultValue(self):
            return False
    
    class note(TextAreaCtrl):
        pass
    
    class photo(ImageUploadCtrl):
        ctrlType = UserPhoto
        def makeFile(self):
            return settings.workspace().new(self.ctrlType,
                filename=self.filename, content_type=self.content_type, 
                size=self.size, width=self.width, height=self.height,
                blob=open(self.path),
            )

    featureCommiting = {'*':'before', '_groups':'after'}

    class _groups(SequenceCtrl):
        itemClass = binding.Constant(UserGroupCtrl)
        def updateContainer(self, removed):
            ws = settings.workspace()
            for i in removed:
                ws.remove(self.oldValue[i])


class UserListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'users/User']'''
    precond = 'obj.hidden==False'

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = _(u'Name')

    class uid(Column):
        queryExpr = 'obj.uid'
        ctrlLabel = _(u'UID')

    class login(Column):
        queryExpr = 'obj.login'
        ctrlLabel = _(u'Login')

    class email(Column):
        queryExpr = 'obj.email'
        ctrlLabel = _(u'E-mail')

    class phone(Column):
        queryExpr = 'obj.phone'
        ctrlLabel = _(u'Phone')



class ListingHandler(web.EntityHandler):

    def do_list(self):
        ctrl = importObject(self.req.params['listingCtrl'])
        lst = ctrl(self.req)
        lst()
        return self.render(listing=lst)




class UserHandler(ListingHandler):
    entity = binding.Constant(User)
    formCtrl = binding.Constant(UserForm)
    obj = binding.Make(lambda s: s.getObject())

    def listQuery(self):
        return self.workspace.parse(
            '[obj for obj in self.entity if obj.hidden==False]')

    def do_update(self):
        obj = self.req.user
        form = self.formCtrl(ctrlReq=self.req, oldValue=obj)
        if form():
            storage.commit()
            storage.begin()
            raise web.SeeOther(self.req.get_url(obj))
        else:
            if form.formErrors:
                storage.abort()
                storage.begin()
            return self.render(**self.editNS(form=form, obj=obj, mode='edit'))

    def do_sendmail(self):
        req = self.req
        users = list(self.workspace(
            '[u for u in User if query.OrderBy(u.name)]'))
            #'[u for u in User if u.hidden==False and query.OrderBy(u.name)]'))
        groups = list(self.workspace(
            '[u for u in Group if query.OrderBy(u.name)]'))

        msg = None
        ok = None
        emails = set([])
        try:
            if req.getValue('sendmail', default=None):
                
                message = req.getValue('message', default='')
                if not message:
                    msg = 'Empty message'
                    raise StopIteration

                subject = req.getValue('subject', default='').strip()
                #if not subject:
                #    msg = 'Empty subject'
                #    raise StopIteration

                for o in users:
                    k = 'u%d' % o._oid
                    if req.getValue(k, default=None):
                        emails.add(o.email)

                for o in groups:
                    k = 'g%d' % o._oid
                    if req.getValue(k, default=None):
                        for u in o.users:
                            emails.add(u.email)

                if not emails:
                    msg = 'No recipient choosen'
                    raise StopIteration

                # msg

                from email.MIMEText import MIMEText
                from email.Header import make_header

                body = MIMEText(message.encode('utf-8'), 'plain', 'utf-8')
                body['From'] = req.user.email
                body['Subject'] = subject
        
                # XXX fork
                import smtplib
                server = smtplib.SMTP('localhost')
                server.sendmail(req.user.email, list(emails), str(body))
                server.quit()

                ok = 'Email was successfully sent to'

        except StopIteration:
            pass
            
        tmpl = req.getResource(req.params['template'])
        return tmpl.handleHTTP(
            req, users=users, groups=groups, msg=msg, emails=emails, ok=ok
        )

    def do_photo(self):
        obj = self.obj
        if obj.photo is None:
            raise web.NotFound()
        return makeResponse(self.req, obj.photo)




class GroupForm(makeFormCtrl(Group)):
    
    class gid(InputCtrl):
        ctrlRequired = True
        ctrlVisibility = binding.Constant('hidden')

    class kind(InputCtrl):
        ctrlRequired = True
        ctrlEnabled = binding.Constant(False)
        def defaultValue(self):
            return recordKind.user

    class active(InputCtrl):
        ctrlRequired = True
        ctrlEnabled = binding.Constant(False)
        def defaultValue(self):
            return True

    class hidden(InputCtrl):
        ctrlRequired = True
        ctrlEnabled = binding.Constant(False)
        def defaultValue(self):
            return False

    class description(TextAreaCtrl):
        pass


class GroupListing(ListingCtrl):
    baseQuery = '''[obj for obj in 'users/Group']'''
    precond = 'obj.hidden==False'

    class name(LikeFilter):
        queryExpr = 'obj.name'
        ctrlLabel = _(u'Name')

    class gid(Column):
        queryExpr = 'obj.gid'
        ctrlLabel = _(u'ID')

    class description(Column):
        queryExpr = 'obj.description'
        ctrlLabel = _(u'Description')



class GroupHandler(ListingHandler):
    entity = binding.Constant(Group)
    formCtrl = binding.Constant(GroupForm)
    obj = binding.Make(lambda s: s.getObject())

