from wm4.api import *
from adt4.serial import Serial
from datetime import datetime

__all__ = [
    'Changeset', 'Versioning', 'defaultChangeset',
    'EntityWithHistory', 'EntityWithAttrHistory',
]


class Changeset(schema.Entity):

    pkey = schema.Attribute(Serial)

    def default_user(o,a,v):
        uf = settings.current_user_factory()
        if uf is None:
            return None
        ui = uf()
        if ui is None:
            return None
        else:
            return ui.login

    created_by = schema.Attribute(
            unicode, schema.ComputeDefault(config.Value(default_user)))

    creation_time = schema.Attribute(
            datetime, schema.Required, schema.defaultDatetime)

    log = schema.Attribute(unicode) 

    binding.metadata(schema.PrimaryKey('pkey'))


class Versioning(schema.SchemaComponent):

    def changeset(self):
        '''Changeset for current transation'''
        return self.workspace.add(Changeset())
    changeset = storage.TxnAttr(changeset)








def default_changeset_for_entity(obj, attr, values):
    return settings.workspace()['versioning'].changeset

defaultChangeset = schema.ComputeDefault(
    config.Value(default_changeset_for_entity)
)


class EntityWithHistoryMeta(schema.EntityMeta):
    
    def histEntity(klass):
        return type(klass)(klass.__name__ + 'H', (klass,), {
            'deleted': schema.Reference(Changeset, schema.Required),
            '__module__' : klass.__module__,
        })
        # XXX primary key
    histEntity = binding.Make(histEntity)

class EntityWithHistory(schema.Entity):

    __metaclass__ = EntityWithHistoryMeta

    created = schema.Reference(Changeset, schema.Required, defaultChangeset)

    def histAttrs(self):
        return {}
        

    
@schema.Workspace.do_update.before(
    'self in schema.Workspace and obj in EntityWithHistory')
def do_update_entity_with_history(self, obj):
    log = naming.lookup('logger:sql')
    klass = obj.__class__
    table = storage.SqlTableName(klass.histEntity)()
    log.debug('TODO INSERT: %s' % table)
    #self.new(klas.histEntity,)

@schema.Workspace.do_remove.before(
    'self in schema.Workspace and obj in EntityWithHistory')
def do_remove_entity_with_history(self, obj):
    log = naming.lookup('logger:sql')
    klass = obj.__class__
    table = storage.SqlTableName(klass.histEntity)()
    log.debug('TODO DELETE: %s' % table)
    #self.new(klas.histEntity,)







class EntityWithAttrHistory_History(schema.Entity):
    #pkey = schema.Attribute(klass.pkey.typeObj, schema.Required)
    attr = schema.Attribute(str, schema.Required)
    old  = schema.Attribute(unicode)
    changeset = schema.Reference(Changeset, schema.Required, defaultChangeset)
    binding.metadata(schema.PrimaryKey('pkey','attr','changeset'))
    

class EntityWithAttrHistoryMeta(schema.EntityMeta):
    
    def attrHistoryEntity(klass):
        return type(klass)(
            klass.__name__ + 'History',
            (EntityWithAttrHistory_History,),
            {
                'pkey': schema.Attribute(klass.pkey.typeObj, schema.Required),
                '__module__' : klass.__module__,
                
            }
        )
    attrHistoryEntity = binding.Make(attrHistoryEntity)


class EntityWithAttrHistory(schema.Entity):
    
    __metaclass__ = EntityWithAttrHistoryMeta

    changeset = schema.Reference(Changeset, schema.Required, defaultChangeset)

@schema.Workspace.do_update.before(
    'self in schema.Workspace and obj in EntityWithAttrHistory')
def do_update_entity_with_history(self, obj):
    log = naming.lookup('logger:sql')
    klass = obj.__class__
    table = storage.SqlTableName(klass.attrHistoryEntity)()
    log.debug('TODO INSERT: %s %s' % (table, obj.genChanges()))
    #self.new(klas.histEntity,)

