from unittest import *
from wm4.api import *
from datetime import datetime
from lib4.timeutils import *
from wm4.schema.tests.utils import MockWorkspaceCase
from comp4.versioning.api import *

epoch = datetime(2006,1,1,10,15,0, tzinfo=TZUTC)

class MyTimeService(TimeService):
    
    def __init__(self):
        self.stamp = epoch

    def now(self):
        s = self.stamp
        self.stamp = s + DAY
        return s




class TestChangeset(MockWorkspaceCase):
    
    def setUp(self):
        MockWorkspaceCase.setUp(self)
        settings.time_service(MyTimeService)
        self.mgr = changeset_manager()

    def test01_Manager(self):
        self.expects(transaction='begin')
        self.expects(
            sql="SELECT NEXTVAL('changeset_pkey_seq')", res=[[8]],
        )
        self.expects(sql=(
          "INSERT INTO changeset (pkey, created_by, creation_time, log) "
          "VALUES (8, NULL, '2006-01-01 10:15:00+00:00', NULL)"
        ))
        self.expects(transaction='commit')

        storage.begin()
        chset = self.mgr.changeset
        self.assertEqual(chset.pkey, 8)
        self.assertEqual(chset.created_by, None)
        self.assertEqual(chset.creation_time, epoch)
        self.assertEqual(chset.log, None)
        storage.commit()




class ObjBase(ChangesetedBaseEntity):
    key = schema.Attribute(int)
    val = schema.Attribute(unicode, schema.Required)
    binding.metadata(schema.PrimaryKey('key', 'changeset_from'))

class Obj(ChangesetedFacadeEntity):
    baseEntity = ObjBase
    acquiredFeatures = ('key', 'val')
    key = schema.Attribute(int)
    val = schema.Attribute(unicode, schema.Required)
    binding.metadata(schema.PrimaryKey('key'))


class TestChangesetedFacade(MockWorkspaceCase):
    
    def setUp(self):
        MockWorkspaceCase.setUp(self)
        settings.time_service(MyTimeService)
        self.mgr = changeset_manager()

    def test20_Add(self):
        self.expects(transaction='begin')
        self.expects(
            sql="SELECT NEXTVAL('changeset_pkey_seq')",
            res=[[6]],
        )
        self.expects(sql=(
          "INSERT INTO changeset (pkey, created_by, creation_time, log) "
          "VALUES (6, NULL, '2006-01-01 10:15:00+00:00', NULL)"
        ))
        self.expects(sql=(
          "INSERT INTO obj_base (changeset_from, changeset_till, key, val) "
          "VALUES (6, 2147483647, 3, 'foo')"
        ))
        self.expects(transaction='commit')

        storage.begin()
        obj = self.ws.add(Obj(key=3, val='foo'))
        self.failUnless(isinstance(obj, Obj))
        self.assertEqual(obj.key, 3)
        self.assertEqual(obj.val, 'foo')
        oid = (13, MAX_CHSET)
        #self.assertEqual(node._oid, oid)
        #self.assertEqual(node._workspace, self.ws)
        #self.assertEqual(node.klass, 'test1')
        #self.failUnless(oid in self.ws.contexts[Node].cache)
        storage.commit()
        #self.failUnless(oid in self.ws.contexts[Node].cache)





def suite():
    return TestSuite((
        makeSuite(TestChangeset),
        makeSuite(TestChangesetedFacade),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
