# See zope.contentype

__all__ = ['magicService', 'mimeService', 'guessContentType', 'guessExtension']



from lib4.imports import lazyModule
from os.path import isfile, dirname, join
import sys

DATA_DIR = join(dirname(__file__), 'data')

_magicService = None

def magicService():
    global _magicService
    if _magicService is None:
        try:
            import magic
            ms = magic.open(magic.MAGIC_MIME | magic.MAGIC_SYMLINK)
            if sys.platform == 'win32':
                ms.load('C:/Program Files/GnuWin32/share/file/magic')
            else:
                ms.load()
        except:
            class Magic:
                def file(self, file):
                    return 'application/octet-stream'
                def buffer(self, buff):
                    return 'application/octet-stream'
            ms = Magic()
        _magicService = ms
    return _magicService


_mimeService = None

def mimeService():
    global _mimeService
    if _mimeService is None:
        import mimetypes
        mimetypes.init([join(DATA_DIR, 'mime.types')])
        _mimeService = mimetypes
    return _mimeService



NON_CONTENT_TYPES = set([
    'application/octet-stream', 'application/force-download'
])

ENCODING_2_CONTENT_TYPE = {'gzip': 'application/gzip'}

# many applications sends nonstandard mime types
NORMALIZED = {
    'application/word' : 'application/msword',
    'application/excel' : 'application/vnd.ms-excel',
    'application/x-msexcel' : 'application/vnd.ms-excel',
}

def guessContentType(filename, path=None, default='application/octet-stream'):
    '''Returns content type for given filename
    '''
    ct, enc = mimeService().guess_type(filename, False)
    if enc:
        ct = ENCODING_2_CONTENT_TYPE.get(enc)
    if (not ct or ct in NON_CONTENT_TYPES) and path and isfile(path):
        ct = magicService().file(path)
    if not ct:
        ct = default
    return NORMALIZED.get(ct, ct)


def guessExtension(content_type):
    content_type = content_type.split(';')[0].strip()
    ext = mimeService().guess_extension(content_type)
    if not ext or ext == '.bin':
        ext = {
            'image/bmp': '.bmp',
            'image/pjpeg': '.jpg',
            'image/jpeg': '.jpg',
            'image/gif': '.gif',
            'image/png': '.png',
            'application/word': '.doc',
            'application/excel': '.xls',
        }.get(content_type, '.bin')
    ext = {
        '.jpeg':'.jpg',
        '.jpe':'.jpg',
        '.obj':'.bin',
        '.ksh':'.txt',
        '.dot':'.doc',
        '.xlb':'.xls',
        '.pwz':'.ppt',
        '.wiz':'.doc',
    }.get(ext, ext)
    return ext
