import csv

__all__ = ['LazyRecordReader', 'RecordReader']



class LazyRecordReader(object):

    def __init__ (
        self, file, encoding='utf-8', dialect='excel',
        fields=None, mangle_fields=True, unicodenormalizaction='NFKC',
        strip=False,
        **fmtparam
    ):

        if unicodenormalizaction:
            from unicodedata import normalize
            def normalization(u):
                return normalize(unicodenormalizaction, u)
        else:
            def normalization(u):
                return u

        if strip:
            def convert(s):
                if s is None:
                    return None
                u = unicode(s, encoding)
                u = u.strip()
                if not u:
                    return None
                return normalization(u)
        else:
            def convert(s):
                if s is None:
                    return None
                u = unicode(s, encoding)
                return normalization(u)

        self.convert = convert

        if isinstance(file, str):
            fd = open(file)
        else:
            fd = file
        self.reader = csv.reader(fd, 'excel', **fmtparam)
        self.encoding = encoding
        if fields is None :
            self.fields = [convert(h) for h in self.reader.next()]
            if mangle_fields:
                from sql4.utils import sql_name
                self.fields = [sql_name(h) for h in self.fields]
        else :
            self.fields = fields


    def __iter__(self):
        return self

    def next(self):
        try:
            row = self.reader.next()
        except StopIteration:
            raise
        flds = self.fields
        assert len(row) <= len(flds), (len(flds), flds,  len(row), row)
        return dict([
                (f, self.convert(v)) for f,v in map(None, flds, row)
        ])


class RecordReader(LazyRecordReader):

    def __init__ (self, *args, **kwds):
        LazyRecordReader.__init__(self, *args, **kwds)
        self.records = []
        append = self.records.append
        next = self.next
        try:
            while True:
                append(next())
        except StopIteration:
            pass

    def __iter__(self):
        return iter(self.records)

    def __getitem__ (self, key) :
        return self.records[key]

    def __len__ (self) :
        return len(self.records)
