#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import urllib2
import cookielib
import lxml.etree as ET


__all__ = ['ExpritCommand', 'ExpritResult', 'ExpritConnection'
       ]

class ExpritCommand(object):

    def dict2xml(self, d):
        x = ''
        for (k, v) in d.iteritems():
            x += '<%s>' % k
            if type(v) is dict:
                x += self.dict2xml(v)
            else:
                x += v
            x += '</%s>' % k
        return x

    def __call__(self, command, **attrs):
        c = '<SPScmd><%s>' % command
        c += self.dict2xml(attrs)
        c += '</%s></SPScmd>' % command
        return c

_charset = re.compile('charset=([^" \n\f\r\t]+)', re.I).search
class ExpritResult(object):

    def encoding(self, res, default_enc):
        chs = _charset(res.info().get('Content-Type', ''))
        if chs :
            encoding = chs.group(1).lower()
        else :
            encoding = defaultEncoding
        return encoding

    def ok(self):
        ret_at = self.ret.attrib
        if ret_at.has_key('ok') and ret_at['ok'] == 'yes':
            return True
        else:
            return False

    def getValue(self, key):
        e = self.ret.find(key)
        ch = e.getchildren()
        if ch:
            return ch
        else:
            return e.text

    def getElem(self, key):
        return self.ret.find(key)

    def debug(self):
        return ET.tostring(self.ret)

    def __init__(self, res, default_enc='windows-1250'):
        header = '<?xml version="1.0" encoding="%s"?>' % self.encoding(
                                                            res, default_enc)
        self.res = res
        r = res.read()
        r = r.replace('<SPScmd>', '<SPScmd xmlns:xqw="XQW">')
        #file('debug.xml', 'w').write(r)
        self.ret = ET.fromstring(r).find('ret')



class ExpritConnection(object):

    def __init__(self, **kw):
        self.url = kw['url']
        self.ec = ExpritCommand()
        self.cookiejar = cookielib.CookieJar()
        self.urlopener = urllib2.build_opener(
                urllib2.HTTPCookieProcessor(self.cookiejar))

    def buildRequest(self, command, **attrs):
        req = urllib2.Request(self.url,
                              data=self.ec(command, **attrs),
                              headers={'Content-Type': 'text/xml'})
        return req

    def command(self, command, **attrs):
        req = self.buildRequest(command, **attrs)
        res = self.urlopener.open(req)
        self.cookiejar.extract_cookies(res, req)
        return ExpritResult(res)

    def login(self, login, funkcni_misto=None):
        res = self.command('login', prihlasovaci_jmeno=login)
        ok = res.ok()
        if not ok:
            return False
        pracovnik_sid = res.getValue('pracovnik/sid')
        if funkcni_misto is None:
            res = self.command('nacistFunkcniMista',
                               pracovnik={'sid': pracovnik_sid})
            ok = ok and res.ok()
            if not ok:
                return False
            funkcni_misto = res.getValue('funkcni_mista')[0].find(
                                                            'kod').text
        res = self.command('nastavitFunkcniMisto',
                           funkcni_misto={'kod': funkcni_misto})
        ok = ok and res.ok()
        return ok
