#!/usr/bin/python
#
# Copyright (c) 2007 Brian "Beej Jorgensen" Hall
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import sys
import os.path
import xml.dom.minidom


def getTextBelow(node, subnodename, handleNone=False):
    subnodes = node.getElementsByTagName(subnodename)
    if subnodes != []:
        return subnodes[0].firstChild.data
    if handleNone:
        return ""
    return None


def gpx2kml(gpx, line_color='7fff0000', line_width='4', ns='http://earth.google.com/kml/2.1'):
    '''
    color: AARRGGBB
        AA - alpha
        BBGGRR - RGB color
    '''
    dom = xml.dom.minidom.parseString(gpx)
    gpx = dom.firstChild
    #ns = 'http://www.opengis.net/kml/2.2'
    kml = ["""<?xml version="1.0" encoding="utf-8"?>
<kml xmlns="%(ns)s">
<Document>
<Style id="trackStyle">
<LineStyle>
<color>%(line_color)s</color>
<width>%(line_width)s</width>
</LineStyle>
</Style>""" % (locals())]

    waypoints = gpx.getElementsByTagName("wpt")
    if waypoints != []:
        for wp in waypoints:
            lat = wp.getAttribute("lat")
            lon = wp.getAttribute("lon")
            name = getTextBelow(wp, "name", True)
            sym = getTextBelow(wp, "sym")
            desc = getTextBelow(wp, "desc")
            ele = getTextBelow(wp, "ele")

            kml.append('<Placemark>')
            kml.append('<name>%s</name>' % name)
            if desc:
                kml.append('<description>%s</description>' % desc)
            kml.append("<styleUrl>#trackStyle</styleUrl>")
            kml.append('<Point>')
            kml.append('<coordinates>')
            kml.append('%s,%s</coordinates>' % (lon, lat))
            kml.append('</Point>')
            kml.append('</Placemark>')

    tracks = gpx.getElementsByTagName("trk")
    if tracks != []:
        for i, trk in enumerate(tracks):
            name = getTextBelow(trk, "name", True)
            desc = getTextBelow(trk, "desc")

            kml.append('<Placemark id="track-%i">' % i)
            kml.append('<name>%s</name>' % name)
            if desc:
                kml.append('<description>%s</description>' % desc)
            # z nejakeho duvodu mi api map seznamu ten styl nahore nenajde
            # tak ho sem zopakuju
            kml.append('''<Style><LineStyle>
<color>%(line_color)s</color>
<width>%(line_width)s</width>
</LineStyle></Style>''' % locals())
            kml.append("<styleUrl>#trackStyle</styleUrl>")
            kml.append("<LineString>")
            kml.append("<coordinates>")
            for seg in trk.getElementsByTagName("trkseg"):
                for trkpt in seg.getElementsByTagName("trkpt"):
                    lat = trkpt.getAttribute("lat")
                    lon = trkpt.getAttribute("lon")
                    ele = getTextBelow(trkpt, "ele")
                    kml.append("%s,%s" % (lon, lat))
            kml.append("</coordinates>")
            kml.append("</LineString>")
            kml.append('</Placemark>')

    kml.append('</Document>')
    kml.append('</kml>')

    return '\n'.join(kml)

