#!/usr/bin/env python

# http://www.w3.org/TR/REC-xml/#sec-guessing
# http://pixel-apes.com/safehtml/ (strips dangerous tags)
# http://www.aminus.org/rbre/python/cleanhtml.py
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/364225
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/302700
# http://oregonstate.edu/~barnesc/htmldata/
# http://www-128.ibm.com/developerworks/xml/library/x-starlet.html?ca=drs-tp2405


import urllib2, re, urlparse, os
import os.path as path

__all__ = [ 'request', 'html2unicode', 'url2unicode', 'tidyHTML', 'parseHTML',
            'downloadURL', 'downloadFromPage',
]



USER_AGENT='Netscape/3.0'

def request(url, **headers):
    req = urllib2.Request(url=url)
    req.add_header('User-Agent', headers.get('user_agent', USER_AGENT))
    req.add_header('Accept', headers.get('accept', '*/*'))
    if 'referer' in headers:
        req.add_header('Referer', headers['referer'])
    return req



_http_equiv_search = re.compile('charset=([^" \n\f\r\t]+)', re.I).search

def html2unicode(html, defaultEncoding='utf-8'):
    m = _http_equiv_search(html)
    if m :
        encoding = m.group(1).lower()
    else :
        encoding = defaultEncoding
    return unicode(html, encoding, errors='ignore')



_charset_match = re.compile('^text/.+;charset=(.+)$').match

def url2unicode(url, defaultEncoding='iso-8859-1', referer=None):
    encoding = None
    if isinstance(url, (basestring,urllib2.Request)) :
        _u = url
        try :
            url = urllib2.urlopen(_u)
        except ValueError :
            url = open(url)
    if getattr(url, 'headers', None) :
        ct = url.headers.getheader('content-type', '').lower().replace(' ','')
        m = _charset_match(ct)
        if m :
            encoding = m.group(1)
    html = url.read()
    if encoding is None :
        return html2unicode(html, defaultEncoding)
    else :
        return unicode(html, encoding, errors='ignore')


def tidyHTML(html, **options):
    from tidy import parseString
    opts = dict(
                output_xhtml=1,
                add_xml_decl=1,
                indent=1,
                tidy_mark=0,
                drop_proprietary_attributes=1,
                clean=1,
                bare=1,
                drop_font_tags=1,
                word_2000=1,
                char_encoding='utf8',
                input_encoding='utf8',
                output_encoding='utf8',
                force_output=1,
    )
    opts.update(options)
    return unicode(str(parseString(html.encode('utf-8'), **opts)), 'utf-8')




def parseHTML(url, defaultEncoding='utf-8'):

    from StringIO import StringIO
    from elementtree.ElementTree import ElementTree
    from elementtree.HTMLTreeBuilder import HTMLTreeBuilder

    html = tidyHTML(url2unicode(url, defaultEncoding=defaultEncoding))
    doc = ElementTree().parse(StringIO(html), HTMLTreeBuilder())
    return doc






def downloadURL(url, base='.', referer=None, ifExists=None):
    req = urllib2.Request(url=url)
    req.add_header('User-Agent', USER_AGENT)
    req.add_header('Accept', '*/*')
    if referer :
        req.add_header('Referer', referer)
    f = urllib2.urlopen(req)
    data = f.read()

    _,h,p,_,_,_ = urlparse.urlparse(url)
    if h :
        p = "%s%s" % (h,p)
    else :
        p = p[1:]
    abspath = path.abspath(path.normpath(path.join(base, p)))
    dir, file = path.split(abspath)
    if not path.isdir(dir) :
        if path.isfile(dir) :
            raise OSError('Regular file found when directory expected', dir)
        os.makedirs(dir)
    open(abspath, 'w').write(data)




def downloadFromPage(url, matching, base='.', referer=None, ifExists=None):
    if isinstance(matching, basestring) :
        matching = re.compile(matching)
    doc = parseHTML(url)
    for a in doc.findall('.//a') :
        href = a.get('href', '')
        if not matching.match(href) :
            continue
        href = urlparse.urljoin(url, href)
        downloadURL(href, base=base, referer=url, ifExists=ifExists)





if __name__ == '__main__' :

    import sys
    out = parseHTML(sys.argv[1])
    #out = url2unicode(sys.argv[1])
    print out
