#!/usr/bin/python

import os
import ctypes

__all__ = ['get_errno', 'shm_open', 'shm_unlink']


__libc = ctypes.cdll.LoadLibrary('libc.so.6')
__libc.__errno_location.restype = ctypes.POINTER(ctypes.c_int)


def get_errno():
    return __libc.__errno_location().contents.value


def _librt():
    global __librt
    try:
        __librt
    except NameError:
        __librt = ctypes.cdll.LoadLibrary('librt.so')
        print 'LOADED', __librt
    return __librt


def shm_open(path, oflags=os.O_CREAT | os.O_RDWR, mode=0644):
    res = _librt().shm_open(ctypes.c_char_p(path+'\0'), oflags, mode)
    if res == -1:
        raise OSError(get_errno(), 'cannot create shm', path, oflags, mode)
    return res


def shm_unlink(path):
    res = _librt().shm_unlink(ctypes.c_char_p(path+'\0'))
    if res == -1:
        raise OSError(get_errno(), 'cannot unlink shm', path)
    return res
    


if __name__ == '__main__':
    if 0:
        h = shm_open('/ahoj')
        os.lseek(h, 0, os.SEEK_END)
        os.write(h, 'foo\n')
    else:
        shm_unlink('/ahoj')
