from unittest import *
from StringIO import StringIO
from os.path import abspath, split, join
from PIL import Image
import glob
from lib4.images import *


IMGDIR= join(split(__file__)[0], 'imgs')

class TestBase(TestCase):

    def setUp(self):
        self.images = dict([
            (split(f)[1], abspath(f)) for f in glob.glob("%s/*" % IMGDIR)
        ])

class TestLoadImage(TestBase):

    def test01_FromImageInstance(self):
        im1 = Image.open(self.images['p.jpeg'])
        im = loadImage(im1)
        assert im1 is im
        self.assertEqual(getMIME(im), 'image/jpeg')
        self.assertEqual(getExt(im), 'jpg')
        self.assertEqual(getStorageSize(im), 14590)

    def test02_FromPath(self):
        im = loadImage(self.images['p.jpeg'])
        self.assertEqual(im.format, 'JPEG')
        self.assertEqual(im.size, (177,262))
        self.assertEqual(getMIME(im), 'image/jpeg')
        self.assertEqual(getExt(im), 'jpg')
        self.assertEqual(getStorageSize(im), 14590)

    def test03_FromFileDescriptor(self):
        im = loadImage(open(self.images['p.jpeg']))
        self.assertEqual(im.format, 'JPEG')
        self.assertEqual(im.size, (177,262))
        self.assertEqual(getMIME(im), 'image/jpeg')
        self.assertEqual(getExt(im), 'jpg')
        self.assertEqual(getStorageSize(im), 14590)

    def test04_FromCGIUpload(self):
        from lib4.cgi import TestUpload, test_environ
        from cgi import FieldStorage
        e,i = test_environ(post={'img':TestUpload(self.images['p.jpeg'])})
        fs = FieldStorage(fp=i, environ=e)['img']
        im = loadImage(fs)
        self.assertEqual(im.format, 'JPEG')
        self.assertEqual(im.size, (177,262))
        self.assertEqual(getMIME(im), 'image/jpeg')
        self.assertEqual(getExt(im), 'jpg')
        self.assertEqual(getStorageSize(im), 14590)

ccfr = calcCropForRatio

class TestCalcs(TestCase):

    def test01(self):
        self.assertEqual(ccfr((100,100),(20,25)), (80,100))
        self.assertEqual(ccfr((100,100),(25,20)), (100,80))
        self.assertEqual(ccfr((200,100),(25,20)), (125,100))
        self.assertEqual(ccfr((100,100),(250,200)), (100,80))
        self.assertEqual(ccfr((100,100),(50,50)), (100,100))


class TestConverter(TestBase):

    def test01(self):
        conv = ImageConverter(maxSize=(200,400))
        im = conv(self.images['sovabg.tiff'])
        self.assertEqual(im.format, 'JPEG')
        self.assertEqual(im.size, (200,276))
        self.assertEqual(im.mode, 'RGB')
        self.assertEqual(getMIME(im), 'image/jpeg')
        self.assertEqual(getExt(im), 'jpg')

    def test02(self):
        conv = ImageConverter(maxSize=(60,60))
        im = conv(self.images['p.gif'])
        self.assertEqual(im.format, 'GIF')
        self.assertEqual(im.size, (40,60))
        self.assertEqual(im.mode, 'P')
        self.assertEqual(getMIME(im), 'image/gif')
        self.assertEqual(getExt(im), 'gif')

    def test03(self):
        conv = ThumbnailConverter(maxSize=(60,60))
        im = conv(self.images['p.gif'])
        self.assertEqual(im.format, 'PNG')
        self.assertEqual(im.size, (60,60))
        self.assertEqual(im.mode, 'P')
        self.assertEqual(getMIME(im), 'image/png')
        self.assertEqual(getExt(im), 'png')


class TestGetSize(TestBase):

    def test01_FromImageInstance(self):
        im = loadImage(open(self.images['p.jpeg']))
        th = im.resize((10,10), Image.ANTIALIAS)
        th.save('/tmp/x.jpeg')
        th = loadImage('/tmp/x.jpeg')
        print im, getattr(im, 'fp', None)
        print getStorageSize(im)
        print th, getattr(th, 'fp', None)
        print getStorageSize(th)

def suite():
    return TestSuite((
        makeSuite(TestLoadImage),
        makeSuite(TestCalcs),
        makeSuite(TestConverter),
        makeSuite(TestGetSize),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
