from unittest import *
from lib4.trees import *

class N(AbstractTreeNode):
    parent = None
    def __init__(self, id, *children):
        self.id = id
        self.children = children
        for c in children:
            c.parent = self
    def getNodeId(self):
        return str(self.id)
    def __getitem__(self, idx):
        return self.children[idx]
    def __repr__(self):
        return 'N(%s)' % repr(self.id)



t1 = N(1)

tA = N(1,
        N(11,
            N(111),
            N(112)),
        N(12,
            N(121,
                N(1211))),
        N(13))

tL = N(1,
        N(11,
            N(111,
                N(1111,
                    N(11111)))))


C = lambda n,c,r: TreeTableCell(node=n, colspan=c, rowspan=r)


class TestTrees(TestCase):

    def test10_TreeLevel(self):
        self.assertEqual(tA.treeLevel, 0)
        self.assertEqual(tA[0].treeLevel, 1)
        self.assertEqual(tA[0][1].treeLevel, 2)
        self.assertEqual(tA[1][0][0].treeLevel, 3)

    def test20_SubtreeDepth(self):
        self.assertEqual(t1.subtreeDepth, 1)
        self.assertEqual(tA.subtreeDepth, 4)
        self.assertEqual(tA[0].subtreeDepth, 2)
        self.assertEqual(tA[1].subtreeDepth, 3)
        self.assertEqual(tA[2].subtreeDepth, 1)
        self.assertEqual(tA[0][1].subtreeDepth, 1)
        self.assertEqual(tA[1][0].subtreeDepth, 2)
        self.assertEqual(tL.subtreeDepth, 5)

    def test30_NodeNum(self):
        self.assertEqual(t1.nodeNum, 1)
        self.assertEqual(tA.nodeNum, 8)
        self.assertEqual(tA[1].nodeNum, 3)
        self.assertEqual(tL.nodeNum, 5)

    def test40_LeafNum(self):
        self.assertEqual(t1.leafNum, 1)
        self.assertEqual(tA.leafNum, 4)
        self.assertEqual(tA[0].leafNum, 2)
        self.assertEqual(tA[1].leafNum, 1)
        self.assertEqual(tA[2].leafNum, 1)
        self.assertEqual(tL.leafNum, 1)

    def test50_DepthFirstIterator(self):
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst()],
            [1, 11, 111, 112, 12, 121, 1211, 13]
        )
        pf = lambda n: n.treeLevel > - 1
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst(filter=pf)],
            []
        )
        pf = lambda n: n.treeLevel > 0
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst(filter=pf)],
            [1]
        )
        pf = lambda n: n.treeLevel > 1
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst(filter=pf)],
            [1, 11, 12, 13]
        )
        pf = lambda n: n.treeLevel > 2
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst(filter=pf)],
            [1, 11, 111, 112, 12, 121, 13]
        )
        pf = lambda n: n.treeLevel > 3
        self.assertEqual(
            [n.id for n in tA.iterDepthFirst(filter=pf)],
            [1, 11, 111, 112, 12, 121, 1211, 13]
        )

    def test51_DepthFirstIterator(self):
        self.assertEqual(
            [n.id for n in t1.iterDepthFirst()],
            [1]
        )

    def test52_DepthFirstIterator(self):
        self.assertEqual(
            [n.id for n in tL.iterDepthFirst()],
            [1, 11, 111, 1111, 11111]
        )

    def test60_BreadthFirstIterator(self):
        self.assertEqual(
            [n.id for n in tA.iterBreadthFirst()],
            [1, 11, 12, 13, 111, 112, 121, 1211]
        )

    def test61_BreadthFirstIterator(self):
        self.assertEqual(
            [n.id for n in t1.iterBreadthFirst()],
            [1]
        )

    def test62_BreadthFirstIterator(self):
        self.assertEqual(
            [n.id for n in tL.iterBreadthFirst()],
            [1, 11, 111, 1111, 11111]
        )

    def test80_TreeNodeWrapper(self):
        w1 = TreeNodeWrapper(t1)
        self.assertEqual(w1.node, t1)
        self.assertEqual(w1.getParent(), None)
        self.assertEqual(w1.getChildren(), [])
        self.assertEqual(w1.getNodeId(), '1')
        self.assertEqual(w1.treeLevel, 0)
        self.assertEqual(w1.subtreeDepth, 1)
        self.assertEqual(w1.nodeNum, 1)
        self.assertEqual(w1.leafNum, 1)

    def test81_TreeNodeWrapper(self):
        wA = TreeNodeWrapper(node=tA)
        self.assertEqual(wA.node, tA)
        self.assertEqual(wA.getParent(), None)
        self.assertEqual(wA.getNodeId(), '1')
        self.assertEqual(wA.treeLevel, tA.treeLevel)
        self.assertEqual(wA.subtreeDepth, tA.subtreeDepth)
        self.assertEqual(wA.nodeNum, tA.nodeNum)
        self.assertEqual(wA.leafNum, tA.leafNum)
        self.assertEqual(len(wA.getChildren()), 3)

        w11 = wA.children[0]
        self.failUnless(isinstance(w11, TreeNodeWrapper))
        self.assertEqual(w11.getParent(), wA)
        self.assertEqual(len(w11.getChildren()), 2)

        t11 = tA[0]
        self.assertEqual(w11.treeLevel, t11.treeLevel)
        self.assertEqual(w11.subtreeDepth, t11.subtreeDepth)
        self.assertEqual(w11.nodeNum, t11.nodeNum)
        self.assertEqual(w11.leafNum, t11.leafNum)

        w111 = w11.children[0]
        self.failUnless(isinstance(w111, TreeNodeWrapper))
        self.assertEqual(w111.getParent(), w11)

    def test82_TreeNodeWrapperWithFilter(self):
        wA = TreeNodeWrapper(node=tA, filter=lambda n: '2' in str(n.id))
        self.assertEqual(
            [n.getNodeId() for n in wA.iterDepthFirst()],
            ['1', '11', '111', '13'],
        )

    def test83_TreeNodeWrapperFromNonRoot(self):
        w0 = TreeNodeWrapper(node=tA[0], filter=lambda n: '2' in str(n.id))
        self.assertEqual(
            [n.getNodeId() for n in w0.iterDepthFirst()],
            ['11', '111'],
        )
        self.assertEqual(w0.treeLevel, 0)
        self.assertEqual(w0.children[0].treeLevel, 1)

    def test90_TreeFromNodeList(self):
        t = treeFromNodeList([tA[1][0], tA[0][1]])
        self.assertEqual(
            [n.node for n in t.iterDepthFirst()],
            [tA, tA[0], tA[0][1], tA[1], tA[1][0]]
        )
        t = treeFromNodeList([tA[1][0], tA[0][1], tA[0][0]])
        self.assertEqual(
            [n.node for n in t.iterDepthFirst()],
            [tA, tA[0], tA[0][0], tA[0][1], tA[1], tA[1][0]]
        )
        t = treeFromNodeList([])
        self.assertEqual(t, None)
        t = treeFromNodeList([tA])
        self.assertEqual([n.node for n in t.iterDepthFirst()], [tA])

    def test95_TreeToHTable(self):
        self.assertEqual(treeToHTable(t1), [[C(t1, 1, 1)]])

    def test96_TreeToHTable(self):
        self.assertEqual(treeToHTable(tL), [[
            C(tL, 1, 1),
            C(tL[0], 1, 1),
            C(tL[0][0], 1, 1),
            C(tL[0][0][0], 1, 1),
            C(tL[0][0][0][0], 1, 1),
        ]])

    def test97_TreeToHTable(self):
        self.assertEqual(treeToHTable(tA), [
          [C(tA, 1, 4), C(tA[0], 1, 2), C(tA[0][0], 2, 1)                     ],
          [                             C(tA[0][1], 2, 1),                    ],
          [             C(tA[1], 1, 1), C(tA[1][0], 1, 1), C(tA[1][0][0],1,1) ],
          [             C(tA[2], 3, 1)                                        ],
        ])




def suite():
    return TestSuite((
        makeSuite(TestTrees),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
