from unittest import *
from lib4.validators import *

class TestDomain(TestCase):

    def test1_Valid(self):
        self.failUnless(checkDomain('bar.com'))
        self.failUnless(checkDomain('dat.cz'))

    def test2_Invalid(self):
        self.failIf(checkDomain('foo'))
        self.failIf(checkDomain('example.nex'))

class TestEmail(TestCase):

    def test1_Valid(self):
        self.failUnless(checkEmail('foo@bar.com'))
        self.failUnless(checkEmail('John.Doe@example.com'))

    def test2_Invalid(self):
        self.failIf(checkEmail('foo'))
        self.failIf(checkEmail('John.Doe@example.nex'))

class TestIP(TestCase):

    def test1_Valid(self):
        self.failUnless(checkIP('1.1.1.1'))
        self.failUnless(checkIP('0.0.0.0'))
        self.failUnless(checkIP('255.255.255.255'))

    def test2_Invalid(self):
        self.failIf(checkIP('-1.1.1.1'))
        self.failIf(checkIP('256.1.1.1'))


class TestPhone(TestCase) :

    def test1(self) :
        self.failUnless(checkPhone('+420572554789'))
        self.failUnless(checkPhone('+420-572-554-789'))
        self.failUnless(checkPhone('+420 572 554 789'))
        self.failUnless(checkPhone('+420 572 55 47 89'))
        self.failUnless(checkPhone('572 554 789 ext. 23'))
        self.failUnless(checkPhone('572 554 789 kl. 23'))

    def test2(self) :
        self.failIf(checkPhone('55 47 89'))
        self.failIf(checkPhone('554 789'))

class TestNormalizeURL(TestCase):

    def test1(self):
        self.assertEqual(normalizeURL('www.uh.cz'), 'http://www.uh.cz')
        self.assertEqual(normalizeURL('ftp.uh.cz'), 'ftp://ftp.uh.cz')
        self.assertEqual(normalizeURL('ftp://ahoj.uh.cz'), 'ftp://ahoj.uh.cz')
        self.assertEqual(normalizeURL('http://www.uh.cz'), 'http://www.uh.cz')
        self.assertEqual(normalizeURL('ahoj://www.uh.cz'), 'http://www.uh.cz')

def suite():
    return TestSuite((
        makeSuite(TestDomain),
        makeSuite(TestEmail),
        makeSuite(TestIP),
        makeSuite(TestPhone),
        makeSuite(TestNormalizeURL),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
