# http://groups.google.com/group/comp.lang.python/browse_thread/thread/dfb305247edd7c6c/
# http://www.regular-expressions.info/
# http://en.wikipedia.org/wiki/E-mail_address

import re

__all__ = [
    'TOPLEVEL_DOMAIN_REGEX', 'DOMAIN_REGEX', 'EMAIL_REGEX', 'IP_REGEX',
    'checkDomain', 'checkEmail', 'checkIP', 'checkPhone',
    'normalizeURL',
]


def TOPLEVEL_DOMAIN_REGEX():
    from data.top_level_domains import top_level_domains
    return '(?:%s)' % '|'.join(top_level_domains)

def DOMAIN_REGEX():
    return '(?:[a-zA-Z0-9][-a-zA-Z0-9]*\\.)+%s' % TOPLEVEL_DOMAIN_REGEX()

def EMAIL_REGEX():
    return '[-+a-zA-Z_0-9.]+@%s' % DOMAIN_REGEX()

IP_REGEX = '(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)'

def checkDomain(input):
    global checkDomain
    match = re.compile('^%s$' % DOMAIN_REGEX()).match
    def check(inp):
        return bool(match(inp))
    checkDomain = check
    return check(input)

def checkEmail(input):
    global checkEmail
    match = re.compile('^%s$' % EMAIL_REGEX()).match
    def check(inp):
        return bool(match(inp))
    checkEmail = check
    return check(input)

def checkIP(input):
    global checkIP
    match = re.compile('^%s$' % IP_REGEX).match
    def check(inp):
        return bool(match(inp))
    checkIP = check
    return check(input)

def checkPhone(input):
    if len([c for c in input if c in '0123456789']) < 9 :
        return False
    else :
        return True

def normalizeURL(input):
    if input.startswith('/'): 
        return input
    l = input.split('://')
    if len(l) == 1:
        proto = 'http'
        if l[0].startswith('ftp.'):
            proto = 'ftp'
        l.insert(0, proto)
    elif not l[0] in ('http', 'ftp'):
        l[0] = 'http'
    return '://'.join(l[:2])
