
#define TCP_NODELAY      1      /* Don't delay send to coalesce packets  */
#define TCP_MAXSEG       2      /* Set maximum segment size  */
#define TCP_CORK         3      /* Control sending of partial frames  */
#define TCP_KEEPIDLE     4      /* Start keeplives after this period */
#define TCP_KEEPINTVL    5      /* Interval between keepalives */
#define TCP_KEEPCNT      6      /* Number of keepalives before death */
#define TCP_SYNCNT       7      /* Number of SYN retransmits */
#define TCP_LINGER2      8      /* Life time of orphaned FIN-WAIT-2 state */
#define TCP_DEFER_ACCEPT 9      /* Wake up listener only when data arrive */
#define TCP_WINDOW_CLAMP 10     /* Bound advertised window */
#define TCP_INFO         11     /* Information about this connection. */
#define TCP_QUICKACK     12     /* Bock/reenable quick ACKs.  */
#define TCP_CONGESTION   13     /* Congestion control algorithm.  */
#define TCP_MD5SIG       14     /* TCP MD5 Signature (RFC2385) */

# struct tcp_info
# {
#   u_int8_t	tcpi_state;
#   u_int8_t	tcpi_ca_state;
#   u_int8_t	tcpi_retransmits;
#   u_int8_t	tcpi_probes;
#   u_int8_t	tcpi_backoff;
#   u_int8_t	tcpi_options;
#   u_int8_t	tcpi_snd_wscale : 4, tcpi_rcv_wscale : 4;
# 
#   u_int32_t	tcpi_rto;
#   u_int32_t	tcpi_ato;
#   u_int32_t	tcpi_snd_mss;
#   u_int32_t	tcpi_rcv_mss;
# 
#   u_int32_t	tcpi_unacked;
#   u_int32_t	tcpi_sacked;
#   u_int32_t	tcpi_lost;
#   u_int32_t	tcpi_retrans;
#   u_int32_t	tcpi_fackets;
# 
#   /* Times. */
#   u_int32_t	tcpi_last_data_sent;
#   u_int32_t	tcpi_last_ack_sent;	/* Not remembered, sorry.  */
#   u_int32_t	tcpi_last_data_recv;
#   u_int32_t	tcpi_last_ack_recv;
# 
#   /* Metrics. */
#   u_int32_t	tcpi_pmtu;
#   u_int32_t	tcpi_rcv_ssthresh;
#   u_int32_t	tcpi_rtt;
#   u_int32_t	tcpi_rttvar;
#   u_int32_t	tcpi_snd_ssthresh;
#   u_int32_t	tcpi_snd_cwnd;
#   u_int32_t	tcpi_advmss;
#   u_int32_t	tcpi_reordering;
# 
#   u_int32_t	tcpi_rcv_rtt;
#   u_int32_t	tcpi_rcv_space;
# 
#   u_int32_t	tcpi_total_retrans;
# };


import ctypes


class tcp_info(ctypes.Structure):
        
    _fields_ = [
        ('tcpi_state', ctypes.c_uint8),
        ('tcpi_ca_state', ctypes.c_uint8),
        ('tcpi_retransmits', ctypes.c_uint8),
        ('tcpi_probes', ctypes.c_uint8),
        ('tcpi_backoff', ctypes.c_uint8),
        ('tcpi_options', ctypes.c_uint8),
        ('tcpi_wscale', ctypes.c_uint8), # tcpi_snd_wscale:4, tcpi_rcv_wscale:4

        ('tcpi_rto', ctypes.c_uint32),
        ('tcpi_ato', ctypes.c_uint32),
        ('tcpi_snd_mss', ctypes.c_uint32),
        ('tcpi_rcv_mss', ctypes.c_uint32),
     
        ('tcpi_unacked', ctypes.c_uint32),
        ('tcpi_sacked', ctypes.c_uint32),
        ('tcpi_lost', ctypes.c_uint32),
        ('tcpi_retrans', ctypes.c_uint32),
        ('tcpi_fackets', ctypes.c_uint32),

        # Times
        ('tcpi_last_data_sent', ctypes.c_uint32),
        ('tcpi_last_ack_sent', ctypes.c_uint32),
        ('tcpi_last_data_recv', ctypes.c_uint32),
        ('tcpi_last_ack_recv', ctypes.c_uint32),

        # Metrics
        ('tcpi_pmtu', ctypes.c_uint32),
        ('tcpi_rcv_ssthresh', ctypes.c_uint32),
        ('tcpi_rtt', ctypes.c_uint32),
        ('tcpi_rttvar', ctypes.c_uint32),
        ('tcpi_snd_ssthresh', ctypes.c_uint32),
        ('tcpi_snd_cwnd', ctypes.c_uint32),
        ('tcpi_advmss', ctypes.c_uint32),
        ('tcpi_reordering', ctypes.c_uint32),

        ('tcpi_rcv_rtt', ctypes.c_uint32),
        ('tcpi_rcv_space', ctypes.c_uint32),

        ('tcpi_total_retrans', ctypes.c_uint32),
    ]

__libc = ctypes.cdll.LoadLibrary('libc.so.6')

def get_sock_info(sck):
    # int getsockopt(int sockfd, int level, int optname,
    #                void *optval, socklen_t *optlen);
    #o = s.getsockopt(socket.SOL_TCP, socket.TCP_INFO, 300)
    optval = tcp_info()
    optlen = ctypes.c_int(ctypes.sizeof(tcp_info))
    res = __libc.getsockopt(
            sck.fileno(), 
            socket.SOL_TCP,
            socket.TCP_INFO,
            ctypes.byref(optval),
            ctypes.byref(optlen)
    )
    assert res == 0
    return optval

if __name__ == '__main__':
    
    import socket

    def print_info(ti):
        for n,d in ti._fields_:
            print '%-20s %s' % (n, getattr(ti, n))

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(('ms.uh.cz', 25))
    #s.connect(('virt.uh.cz', 80))
    print_info(get_sock_info(s))
    print
    print repr(s.recv(1024))
    print_info(get_sock_info(s))
    s.send('EHLO q.uh.cz\r\n')
    print
    print repr(s.recv(1024))
    print_info(get_sock_info(s))
    s.send('MAIL FROM: <rk@dat.cz>\r\n')
    print
    print repr(s.recv(1024))
    print_info(get_sock_info(s))
    s.send('QUIT\r\n')
    print
    print repr(s.recv(1024))
    print_info(get_sock_info(s))
