import sys, plpy

__all__ = [
    'recursiveTriggerInvocation', 'relName', 'relId', 'relAttrs', 'relPKey',
    'InsertPlan', 'UpdatePlan'
]

def recursiveTriggerInvocation():
    me = sys._getframe(1)
    frame = me.f_back
    while frame:
        if frame.f_code is me.f_code:
            return True
        frame = frame.f_back
    return False



def relName(relid):
    return plpy.execute(
        "SELECT relname FROM pg_catalog.pg_class WHERE oid=%s" % relid,1
    )[0]["relname"]


def relId(relname):
    return plpy.execute(
        "SELECT oid FROM pg_catalog.pg_class WHERE relname='%s'" % relname,1
    )[0]["oid"]

def relAttrs(relid):
    attres = plpy.execute(
        """SELECT a.attname, t.typname
             FROM pg_catalog.pg_attribute a, pg_catalog.pg_type t
            WHERE a.atttypid = t.oid AND a.attrelid = '%s'
              AND a.attnum > 0 AND NOT a.attisdropped""" % relid)
    attrs = {}
    for i in range(len(attres)):
        r = attres[i]
        attrs[r["attname"]] = r["typname"]
    return attrs

def relPKey(relid):
    q = """
        SELECT a.attname
          FROM pg_catalog.pg_index i, pg_catalog.pg_class c, pg_attribute a
         WHERE i.indrelid = %s AND i.indexrelid = c.oid
           AND i.indisprimary = 't' AND i.indexrelid = a.attrelid
           AND a.attisdropped = 'f'
          ORDER BY a.attnum;""" % relid
    res = plpy.execute(q)
    pkey = [ res[i]["attname"] for i in range(len(res)) ]
    return pkey


class InsertPlan:

    def __init__(self, relid, exclude=()):
        self.relid = relid
        try:
            int(relid)
        except:
            table = relid
            relid = relId(table)
        else:
            table = relName(relid)
        self.attrs = [
            (a,t) for a,t in relAttrs(relid).items() if a not in exclude
        ]
        self.query = 'INSERT INTO "%s"(%s) VALUES(%s)' %(
            table,
            ",".join(['"%s"' % c for c,_ in self.attrs]),
            ",".join(["$%d" %(i+1) for i in range(len(self.attrs))])
        )
        self.plan = plpy.prepare(self.query, [t for _,t in self.attrs])
    
    def execute(self, rec):
        return plpy.execute(self.plan, [rec[a] for a,_ in self.attrs])
            

class UpdatePlan:

    def __init__(self, relid, updateCols):
        self.updateCols = updateCols
        try:
            int(relid)
        except:
            table = relid
            relid = relId(table)
        else:
            table = relName(relid)
        attrs = relAttrs(relid)
        self.pkey = relPKey(relid)

        q = 'UPDATE "%s" SET %s WHERE %s' %(
            table,
            ", ".join(['"%s"=$%d' %(c, i+1) for i,c in enumerate(updateCols)]),
            " AND ".join(['"%s"=$%d' %(k, i+1+len(updateCols))
                           for i,k in enumerate(self.pkey)])
        )
        self.plan = plpy.prepare(q, [attrs[c] for c in updateCols + self.pkey])
    
    def execute(self, old, **update):
        args = [update[c] for c in self.updateCols]
        args.extend([old[c] for c in self.pkey])
        return plpy.execute(self.plan, args)
            

if __name__ == '__main__':
    import pgdb
    
    class fakePLPY:
        def __init__(self):
            self.con = pgdb.connect(database='test')
        def execute(self, q, *a):
            cur = self.con.cursor()
            cur.execute(q)
            descr = cur.description
            return [
                dict([(d[0],v) for v,d in zip(r,descr)])
                for r in cur.fetchall()
            ]
        def notice(self, msg):
            print "MSG", msg
    
    plpy = fakePLPY()
    plan = InsertPlan(105986)
    plan.execute({'cmsid':1, 'title':'ahoj', 'abstract':None,
                  'content_class':'dir', 'content_id': 12,
                  'validity': 'from -infinity till infinity',
                  'owner': 0,
    })
