# -*- coding: utf-8 -*-

import sys

from wm4.api import *
from wm4.running import commands, options

from sql4.utils import *

__all__ = ['GenEntity']


class GenEntity(commands.ConfiguredCommandLineApp):
    '''
         $ wm4 sql4.commands.GenEntity database_url [schema.]table
    '''

    #dry_run = binding.Constant(False,
    #    options.Set('--dry-run', '-n', value=True, help='Rollback transaction'))

    #partitions = binding.Make(list,
    #    options.Append('--partition', '-p', type=str, help='Partition'))

    def main(self):
        super(GenEntity, self).main()
        pa = self.parsed_args
        if len(pa) != 2:
            print >>self.stderr, 'database_url and table needed'
            sys.exit(1)
        dburl, table = pa
        dbcon = naming.lookup(dburl)

        storage.begin()

        print 'class %s(schema.Entity):' % table
        features = []
        fmax = 0
        for attr, typ, notnull in dbcon('''
            SELECT a.attname AS attr,
                   a.atttypid::regtype AS typ,
                   a.attnotnull AS notnull
              FROM pg_attribute a
             WHERE a.attrelid = '%s'::regclass
               AND a.attnum > 0
               AND a.attisdropped = false
          ORDER BY a.attnum
        ''' % sql_escape(table)):
            typ = {
                'character': 'str',
                'character varying': 'unicode',
                'boolean': 'bool',
                'integer': 'int',
                'date': 'datetime.date',
                'timestamp with time zone': 'datetime.datetime',
                'timestamp': 'datetime.datetime',
                'double precision': 'float',
            }[typ]
            if notnull == 't':
                f = 'schema.Attribute(%s, schema.Required)' % typ
            else:
                f = 'schema.Attribute(%s)' % typ
            features.append((attr, f))
            fmax = max(len(attr), fmax)
        for a, f in features:
            print '    %-*s = %s' % (fmax, a, f)

        pk = dbcon('''
             SELECT pg_attribute.attname
               FROM pg_attribute, pg_index
              WHERE pg_attribute.attrelid = pg_index.indexrelid
                AND pg_index.indrelid = '%s'::regclass
                AND(pg_index.indkey[0] = pg_attribute.attnum
                    OR pg_index.indkey[1] = pg_attribute.attnum
                    OR pg_index.indkey[2] = pg_attribute.attnum
                    OR pg_index.indkey[3] = pg_attribute.attnum
                    OR pg_index.indkey[4] = pg_attribute.attnum
                    OR pg_index.indkey[5] = pg_attribute.attnum
                    OR pg_index.indkey[6] = pg_attribute.attnum)
                AND pg_index.indisprimary = 't'
           ORDER BY attnum
        ''' % sql_escape(table))
        pk = [a for a, in pk]
        if pk:
            print
            print "    binding.metadata(schema.PrimaryKey(%s))" % (
                ','.join(repr(a) for a in pk)
            )
        print

if __name__ == '__main__':

    cmd = GenEntity()
    cmd.run()
