import pgdb
from sql4 import utils

__all__ = ['Cursor', 'Connection', 'connect']

PGMAJOR = int(pgdb.version.split('.')[0])


class Cursor(pgdb.pgdbCursor):
    '''Cursor without typecast'''

    if PGMAJOR >= 4:
        def fetchmany(self, size=None, keep=False):
            if size == None:
                size = self.arraysize
            if keep == 1:
                self.arraysize = size
            try:
                res = self._src.fetch(size)
            except pgdb.Error, e:
                raise pgdb.DatabaseError, str(e)
            return res
    else:
        def fetchmany(self, size=None, keep=0):
            if size == None:
                size = self.arraysize
            if keep == 1:
                self.arraysize = size
            try:
                res = self._pgdbCursor__source.fetch(size)
            except pgdb.Error, e:
                raise pgdb.DatabaseError, str(e)
            return res


class Connection(pgdb.pgdbCnx):
    '''Connection that does not convert row values'''

    if PGMAJOR >= 4:
        def cursor(self):
            try:
                return Cursor(self)
            except Exception, e:
                raise pgdb.OperationalError, "invalid connection. %s" % repr(e)
    else:
        def cursor(self):
            try:
                src = self._pgdbCnx__cnx.source()
                return Cursor(src, self._pgdbCnx__cache)
            except Exception, e:
                raise pgdb.OperationalError, "invalid connection. %s" % repr(e)

    def getTables(self, reload=False, kinds='r'):
        q = "SELECT relname,oid FROM pg_class WHERE %s" % (
            ' OR '.join("relkind='%c'" % c for c in kinds)
        )
        cur = self.cursor()
        cur.execute(q)
        res = cur.fetchall()
        cur.close()
        return dict(res)

    def getTable(self, tablename, reload=False):
        cur = self.cursor()
        q = '''
            SELECT a.attname, a.atttypid::regtype,
                   a.attnotnull, d.adsrc
              FROM pg_attribute a
                   LEFT JOIN pg_attrdef d ON (
                        a.attrelid = d.adrelid AND a.attnum = d.adnum)
             WHERE a.attrelid = '%s'::regclass::oid
               AND a.attnum >= 0
               AND NOT a.attisdropped
             ORDER BY a.attnum
        ''' % tablename
        cur.execute(q)
        res = cur.fetchall()
        cur.close()
        return dict((a,(t,n,d)) for a,t,n,d in res)

    def addColumn(self, tablename, column, type):
        cur = self.cursor()
        cur.execute('ALTER TABLE %s ADD COLUMN %s %s' % (
                utils.escape_name(tablename),
                utils.escape_name(column),
                type,
        ))
        cur.close()

    def renameColumn(self, tablename, oldname, newname):
        cur = self.cursor()
        cur.execute('ALTER TABLE %s RENAME %s TO %s' % (
                utils.escape_name(tablename),
                utils.escape_name(oldname),
                utils.escape_name(newname),
        ))
        cur.close()

    def dropColumn(self, tablename, column):
        cur = self.cursor()
        cur.execute('ALTER TABLE %s DROP COLUMN %s' % (
                utils.escape_name(tablename),
                utils.escape_name(column),
        ))
        cur.close()


def connect(dsn=None, user=None, password=None, host=None, database=None):
    # first get params from DSN
    dbport = -1
    dbhost = ""
    dbbase = ""
    dbuser = ""
    dbpasswd = ""
    dbopt = ""
    dbtty = ""
    try:
        params = dsn.split(":")
        dbhost = params[0]
        dbbase = params[1]
        dbuser = params[2]
        dbpasswd = params[3]
        dbopt = params[4]
        dbtty = params[5]
    except:
        pass

    # override if necessary
    if user != None:
        dbuser = user
    if password != None:
        dbpasswd = password
    if database != None:
        dbbase = database
    if host != None:
        try:
            params = host.split(":")
            dbhost = params[0]
            dbport = int(params[1])
        except:
            pass

    # empty host is localhost
    if dbhost == "":
        dbhost = None
    if dbuser == "":
        dbuser = None

    # open the connection
    cnx = pgdb._connect_(dbbase, dbhost, dbport, dbopt, dbtty, dbuser, dbpasswd)
    return Connection(cnx)



if __name__ == '__main__':

    con = connect(database='test')
    print 'CONNECTION', con
    cur = con.cursor()
    print 'CURSOR', cur
    cur.execute('SELECT * FROM pg_database')
    for r in cur.fetchall():
        print r
