# From simpleSQL.py
# http://pyparsing.wikispaces.com/space/showimage/simpleSQL.py
#
# DDL -> graphviz
# http://energyblog.blogspot.com/2006/04/blog-post_20.html

import sys
from pyparsing import (
    Literal, CaselessLiteral, Word, delimitedList, Optional, Combine,
    Group, alphas, nums, alphanums, Forward, oneOf,
    Keyword, CaselessKeyword,
    sglQuotedString, dblQuotedString, MatchFirst,
    OneOrMore, ZeroOrMore, CharsNotIn, Suppress, Regex,
    LineEnd, StringEnd
)

class SQL:
    def __getattr__(self, kw):
        return CaselessKeyword(kw)
SQL = SQL()


parenthesis = Forward()
parenthesis <<= "(" + ZeroOrMore(CharsNotIn("()") | parenthesis)  + ")"

sql_name = \
    Regex('[a-z_][a-z0-9_$]*') | \
    Combine(
        Suppress(Literal('"')) + \
        OneOrMore(CharsNotIn('"') | Literal('""')) + \
        Suppress(Literal('"'))
    ).setParseAction(lambda i,l,t: ''.join(t).replace('""','"'))

sql_name = \
    Regex('[a-z_][a-z0-9_$]*') | \
    dblQuotedString.setParseAction(
        lambda i,l,t: ''.join(t).replace('""','"')[1:-1])

const_string = Suppress(Optional('E')) + sglQuotedString.setParseAction(
        lambda i,l,t: ''.join(t).replace("''","'")[1:-1])



eat_rest = ZeroOrMore(const_string | Regex("[^';]+"))

create_table = sql_name + '(' + eat_rest + ')' + Optional(SQL.WITH + SQL.OIDS)

create = SQL.CREATE + MatchFirst([
            SQL.TABLE + create_table,
            SQL.SEQUENCE + eat_rest,
            SQL.INDEX + eat_rest,
            Optional(SQL.OR + SQL.REPLACE) + SQL.FUNCTION + eat_rest,
     ])

alter = SQL.ALTER + (
            SQL.TABLE | SQL.SEQUENCE | SQL.DATABASE
        ) + eat_rest

select = SQL.SELECT + SQL.FROM + eat_rest

update = SQL.UPDATE + SQL.SET + SQL.WHERE + eat_rest


insert = SQL.INSERT + SQL.INTO + eat_rest


comment = Suppress(Literal("--") + ~LineEnd())


any_command = create | alter | select | update | insert | comment

suite = ZeroOrMore(any_command + (Literal(";") | StringEnd()))

statement = any_command + Optional(";") + StringEnd()








field_def = OneOrMore(Word(alphanums+"_\"':-") | parenthesis)

def field_act(s,loc,tok):
    return ("<"+tok[0]+"> " + " ".join(tok)).replace("\"","\\\"")

field_def.setParseAction(field_act)

field_list_def =  field_def + ZeroOrMore(Suppress(",") + field_def)

def field_list_act(s,loc,tok):
    return " | ".join(tok)

field_list_def.setParseAction(field_list_act)

create_table_def = Literal("CREATE")+"TABLE"+Word(alphas).setResultsName("tableName")+"("+field_list_def.setResultsName("fields")+")"+";"

def create_table_act(s,loc,tok):
    return """"%(tableName)s" [\n\t label="<%(tableName)s> %(tableName)s | %(fields)s"\n\t shape="record"\n];""" % tok

create_table_def.setParseAction(create_table_act)

add_fkey_def=Literal("ALTER")+"TABLE"+"ONLY" + Word(alphanums+"_").setResultsName("tableName") + "ADD"     + "CONSTRAINT" + Word(alphanums+"_") + "FOREIGN"+"KEY"+"("+Word(alphanums+"_").setResultsName("keyName")+")"     +"REFERENCES"+Word(alphanums+"_").setResultsName("fkTable")+"("+Word(alphanums+"_").setResultsName("fkCol")+")"+";"

def add_fkey_act(s,loc,tok):
    return """ "%(tableName)s":%(keyName)s -> "%(fkTable)s":%(fkCol)s """ % tok

add_fkey_def.setParseAction(add_fkey_act)

other_statement_def = ( OneOrMore(CharsNotIn(";") )  + ";")

def other_statement_act(s,loc,tok):
    return ""

other_statement_def.setParseAction(other_statement_act)

comment_def = "--" + ZeroOrMore(CharsNotIn("\n"))

comment_def.setParseAction(other_statement_act)

statement_def =  comment_def | create_table_def | add_fkey_def | other_statement_def

defs =  OneOrMore(statement_def)

if __name__ == '__main__':
    parser = OneOrMore(sql_name | const_string) + StringEnd()
    for i in parser.parseString(sys.stdin.read()):
        print repr(i)
