import fcgiapp, sys, os, urlparse, urllib,  __builtin__, cgi
import traceback, stat, os
import utils, traversables, errors, date
from logger import getLogger

__all__ = ['Config', 'Globals', 'Record', 'Request', 'Application', 'debug']

log = getLogger('wm3.web')



debug = utils.debug

error = 'WindmillError'

class Config:
    proces_cycles = 100
    app_dir = None
    app_url_root = '/' # Musi se zadavat s lomitkem nakonci
    debug = None
    debug_app = None
    session = None
    pre_traverse_fun = None
    pre_request_fun = None
    post_request_fun = None
    post_traverse_fun = None
    headers = []
    mime_types_config = ['/etc/mime.types']
    denied_extensions = ['.pyc', '.pyo', '.so', '.o']
    extension_handlers = {}
    show_traceback = False
    output_encoding = 'latin2' # XXX
    input_encoding = 'latin2' # XXX

    def __init__(self, *roots, **kw):
        self.server_roots_unparsed = roots
        self.server_roots = []
        for r in roots:
            self.server_roots.append(urlparse.urlparse(r))
        self.__dict__.update(kw)


class Globals:

    def __init__(self, **kw):
        self.__dict__.update(kw)

Record = Globals



class Request(object):
    args = []
    kw = {}
    conf = None
    data = None
    file = None
    glob = None
    module = None
    object = None
    object_name = None
    object_id = None
    param = None
    url = None
    session_id = None
    session_token = None
    session_2url = 1

    def __init__(self, env):
        self.env = env
        self.status = '200 OK'
        self.body = []
        self.headers = []
        self.cookies = {}

    def write(self, *arg):
        self.body.extend(arg)

    def flush(self):
        pass

    def cookie(self, a, v, p=None, e=None):
        if not p:
            p = self.conf.app_url_root
        self.cookies[a] =(v,p,e)

    output_encoding = property(lambda s: s.conf.output_encoding)
    input_encoding = property(lambda s: s.conf.input_encoding)

    def header(self, a, v):
        a = str(a)
        v = str(v)
        if a == 'Location':
            self.status = '303 See Other'
        elif a == 'Status':
            self.status = v
            return
        self.headers.append((a, v))

    def get_headers(self):
        di = []
        hdr = []
        for i in range(len(self.headers)-1, -1, -1):
            a,v = self.headers[i]
            la = a.lower()
            if la in di:
                continue
            di.append(la)
            hdr.insert(0,(a,v))

        for a,(v,p,e) in self.cookies.items():
            v = utils.urlquote(str(v))
            if e is None:
                # expirace nebyla zadana
                hdr.append(('Set-cookie', '%s=%s; path=%s' % (a,v,p)))
            else:
                if isinstance(e, date.DateType):
                    e = e.fmt('%a, %d-%m-%Y 00:00:00 GMT')
                elif isinstance(e, date.DateTimeType):
                    e = e.fmt('%a, %d-%m-%Y %H:%M:%S GMT','GMT')
                else:
                    e = str(e)
                # expirace ve formatu Wdy, DD-Mon-YYYY HH:MM:SS GMT
                hdr.append((
                    'Set-cookie','%s=%s; path=%s; expires=%s' % (a, v, p, e)
                ))
        return hdr

    def relurl(self, obj, **kw):
        obj = os.path.join(self.app_url_root, obj)
        u = apply(utils.Url, (obj,), kw)
        return u

    def relurlh(self, obj, **kw):
        obj = os.path.join(self.app_url_root, obj)
        kw['host'] = self.app_server_root
        u = apply(utils.Url, (obj,), kw)
        return u



# Definuj globalni "builtin" funkci getreq, ktera vrati
# instance req kdekoliv.
_global_req = None
def getreq():
    return _global_req
__builtin__.__dict__['getreq'] = getreq



def terminate_handler(signum, frame):
    global Application
    Application.terminate = 1


class Application(traversables.TraverseApplication):

    terminate = 0

    def __init__(self, conf, glob, exceptionHandler=None):
        self.conf = conf
        self.glob = glob
        if exceptionHandler is None:
            exceptionHandler = errors.ExceptionHandler()
        self.exceptionHandler = exceptionHandler
        traversables.TraverseApplication.__init__(self, conf.app_dir)

    def make_request(self, env, inp):
        # Vygenerovani objektu req a inicializace zakladnich atributu
        global _global_req
        # Uloz req take do globalni promenne kvuli funkci getreq()
        _global_req = req = Request(env)
        if env.get('REQUEST_METHOD','GET').upper() == 'GET':
            req.param = cgi.FieldStorage(environ=env,keep_blank_values=1)
        else:
            req.param = cgi.FieldStorage(fp=inp,environ=env,keep_blank_values=1)
        req.conf = self.conf
        req.glob = self.glob
        req.data = Record()
        req.headers = list(self.conf.headers)
        req.cookies = {}
        return req

    def setup_virtual(self, req):
        # Nalezeni odpovidajiciho virtualniho serveru z konfigurace aplikace
        i = 0
        req.app_server_index = -1
        for u in self.conf.server_roots:
            sn = u[1]
            try:
                s, po = sn.split(':')
                sn = s
                dpo = 1
            except:
                if u[0] == 'https': po = '443'
                else: po = '80'
                dpo = 0
            if req.env['SERVER_NAME'] == sn \
            and req.env['SERVER_PORT'] == po:
                req.app_url_root = u[2]
                req.app_server_root = u[0] + '://' + u[1]
                req.app_server_index = i
                #if dpo == 1:
                #   req.app_server_root = req.app_server_root + ':' + po
                break
            i = i + 1
        if req.app_server_index == -1:
            raise error,'Cannot find server %s:%s in configuration %s' % (
                req.env['SERVER_NAME'], req.env['SERVER_PORT'],
                self.conf.server_roots
            )

    def setup_environ(self, req):
        # parsed_url (je potreba odeskejpovat)
        req.url_parsed = urlparse.urlparse(req.env['REQUEST_URI'])
        # path and traversal_path
        req.path = urllib.unquote(req.url_parsed[2])[len(req.app_url_root):]
        req.traversal_path = tPath = filter(None, req.path.split('/'))
        # tilde (tilda je vyhozena z traverzovaci cesty)
        if tPath and tPath[0].startswith('~'):
            req.tilde = tPath[0][1:]
            del tPath[0]
        else:
            req.tilde = None
        # keywords
        kw = {}
        if req.url_parsed[3]:
            for a in req.url_parsed[3].split(';'):
                x = a.split('$', 1)
                if len(x) > 1: kw[x[0]] = urllib.unquote(x[1])
                else: kw[x[0]] = None
        req.kw = kw
        # canonical url
        req.url = os.path.join(req.app_url_root, '/'.join(req.traversal_path))

    def setup_cookies(self, req):
        # Zpracovani cookies
        if req.env.has_key('HTTP_COOKIE'):
            for l in req.env['HTTP_COOKIE'].split(';'):
                m = l.strip().split(' ')
                a,v = m[0].strip().split('=', 1)
                if len(m) > 1:
                    n,p = m[1].strip().split('=', 1)
                else:
                    p = None
                req.cookie(a, urllib.unquote(v), p)

    def setup_session(self, req):
        # Nalezeni SessionID
        req.session = self.conf.session
        if self.conf.session:
            req.session_token, req.session_id = self.conf.session.get_id(req)
        else:
            req.session_token = req.session_id = None

    def pre_request(self, req):
        if self.conf.pre_request_fun:
            self.conf.pre_request_fun(req)

    def post_request(self, req):
        if self.conf.post_request_fun:
            self.conf.post_request_fun(req)

    def handle_request(self, req):
        try:
            self.setup_virtual(req)
            self.setup_environ(req)
            self.setup_cookies(req)
            self.setup_session(req)
            self.pre_request(req)
            obj = self.traversePath(req, list(req.traversal_path))
            log.debug('traversed object: %r %r', req.traversal_path, obj)
            obj.handleHTTP(req)
            self.post_request(req)
        except:
            shouldExit = self.exceptionHandler.handleException(
                req, sys.exc_info()
            )
        return req

    def send_data(self, req, out):
        '''Formats response into output stream'''
        out.write('Status: %s\r\n' % req.status)
        for h,v in req.get_headers():
            out.write('%s: %s\r\n' % (h,v))
        out.write('\r\n')
        for b in req.body:
            out.write(b)
        out.flush()

    def start(self):
        '''Run app in raw CGI/FastCGI mode'''
        if fcgiapp.isCGI():
            req = self.make_request(os.environ, sys.stdin)
            self.handle_request(req)
            self.send_data(req, sys.stdout)

        else:
            for cy in xrange(self.conf.proces_cycles):
                if self.terminate: break
                inp, out, err, env = fcgiapp.Accept()
                if self.terminate: break
                req = self.make_request(dict(env), inp)
                if self.terminate: break
                self.handle_request(req)
                if self.terminate: break
                self.send_data(req, out)
                if self.terminate: break
                fcgiapp.Finish()
        sys.exit(0)

    def __call__(self, env, start_response):
        '''WSGI interface'''
        req = self.make_request(env, env['wsgi.input'])
        self.handle_request(req)
        start_response(req.status, req.get_headers())
        return req.body
