import traceback
from logger import getLogger

__all__ = [
    'HTTPException', 'ExceptionHandler',
    # by name
    'Unauthorized', 'NotAllowed', 'NotFound',
    # by status
    'HTTP401', 'HTTP403', 'HTTP404', 'HTTP500',
]


log = getLogger('wm3.web')



class HTTPException(Exception):
    status = '500 Internal Server Error'
    def handleHTTP(self, req):
        req.status = self.status
        req.header('Content-type', 'text/plain')
        req.write('%s\n' % self.status)
HTTP500 = HTTPException


class Unauthorized(HTTPException):
    status = '401 Unauthorized'
    def handleHTTP(self, req):
        req.status = self.status
        req.header('Content-type', 'text/plain')
        req.header('WWW-Authenticate', 'Basic realm="Zadejte jmeno a heslo"')
        req.write('%s\n' % self.status)
HTTP401 = Unauthorized


class NotAllowed(HTTPException):
    status = '403 Forbidden'
HTTP403 = NotAllowed


class NotFound(HTTPException):
    status = '404 Not Found'
HTTP404 = NotFound




class ExceptionHandler:

    def handleException(self, req, exc_info):
        et, ev, tb = exc_info
        try:
            if isinstance(ev, HTTPException):
                ev.handleHTTP(req)
                return 0
            else:
                log.exception('Internal server error')
                req.status = HTTPException.status
                req.header('Content-type', 'text/plain')
                req.write('Internal server error: %s\n' % et)
                return 1
        finally:
            et = ev = tb = exc_info = None

    def formatException(self, req, exc_info, show_traceback=False):
        t,v,s = exc_info
        try:
            if show_traceback:
                return ''.join(traceback.format_exception(t,v,s))
            else:
                return ''.join(traceback.format_exception_only(t,v))
        finally:
            t = v = s = None

    def formatRequest(self, req):
        return ''
