"""Nasledujici tridy slouzi pro uchovani potrebnych informaci
   o jednotlivych databazovych polozkach a pro praci s nimi
   v samotne databazi nebo ve WWW aplikacich.
"""
import date, string, copy, messages, mimetypes, re, exceptions, sys
from utils import *

WM_STRING = 0
WM_INT = 1
WM_LONG = 2
WM_FLOAT = 3
WM_BOOL = 4
WM_DATE = 5
WM_DATETIME = 6
WM_INTERVAL = 7
WM_BYTEA = 8
WM_VALIDITY_TIMESTAMPTZ = 9

typeid2string = {
    WM_STRING : 'WM_STRING',
    WM_INT : 'WM_INT',
    WM_LONG : 'WM_LONG',
    WM_FLOAT : 'WM_FLOAT',
    WM_BOOL : 'WM_BOOL',
    WM_DATE : 'WM_DATE',
    WM_DATETIME : 'WM_DATETIME',
    WM_INTERVAL : 'WM_INTERVAL',
    WM_BYTEA : 'WM_BYTEA',
    WM_VALIDITY_TIMESTAMPTZ : "WM_VALIDITY_TIMESTAMPTZ",
}

if sys.hexversion >= 0x2030000 :
    def validity_conv (val) :
        return val
else :
    validity_conv = lambda val: val


#array_split_regex = re.compile('(,|\\}|\\{|(?:(?!\\\\)")??)')
array_split_regex = re.compile('((?:".*?")|,|\{|\})')

class field :

    # Identifikace pouziteho typu
    typeid = WM_STRING

    # Databazove zalezitosti
    fieldname = None   # Nazev policka
    tablename = None   # Nazev databazove tabulky
    typeoid = None     # Cislo db typu
    typename = None    # Nazev db typu
    deforder = None    # Poradi polozky v definici tabulky
    maxlen = None      # Maximalni velikost polozky
    minlen = None      # Minimalni velikost polozky
    notnull = None     # Atribut je definovan jako notnull
    hasdef = None      # Atribut ma defaultni hodnotu
    dimension = 0      # Dimenze pole (0 = skalar)
    quotechar = "'"    # Znak do nejz se uzaviraji hodnoty
    arrayquote = '"'   # Znak pro uzavirani hodnot v polich

    # Atributy ridici zobrazovani
    title = None            # Titulek databazove polozky
    description = None      # Popis polozky
    help = None             # Napoveda pro vyplnovani

    # Formaty vypisu
    yesno = messages.YESNO_FORMAT
    dateformat = messages.DATE_FORMAT
    intervalformat = messages.INTERVAL_FORMAT

    # Atributy pro SqlList
    aslink = 1         # Zobrazovat tuto polozku jako aktivni

    # Atributy pro HTML formulare
    size = None        # Velikost editovaciho policka
    maxsize = 30       # Maximalni velikost editovaciho policka
    readonly = None    # Needitovatelna polozka. Tuto polozku
                       # je mozno inicializovat asoc. polem
                       # { 'table' : <tb>,
                       #   'view' : [ <f1>, <f2>, ... ],
                       #   'delim' : '/' }
                       # Potom se misto vlastni hodnoty zobrazi
                       # polozky f1,f2,... z tabulky tb. Hodnota
                       # musi byt v polozce, ktera je v tabulce
                       # tb primarni klic.
    hidden = ''        # Skryta polozka formulare E, seznamu L nebo view V
    textarea = None    # Pocet radku a sloupcu u TEXTAREA (r,s)
    textareawrap = None# Jestli zadat parametr WRAP="OFF"
    popup = -1         # Odkaz pro popup okno. -1 = default
                       # Parametr popup se zadava stejne jako je
                       # videt v polozce popup_date. Povinny parametr
                       # je pouze object.
    regex_search = 1

    popup_date = { 'object' : '/__popup__',
                   'kw' : { 'me' : 'date' },
                   'param' : {},
                   'width' : 250, 'height' : 270,
                   'ref' : '<img src="/_wmlib/img/calendar.gif" ' +\
                           'border="0" alt="calendar">',
                   }

    def __init__ ( self, *args, **kw ) :
        """Konstruktor tridy. V konstruktoru lze
           pouzit vsechny vyse deklarovane parametry.
        """
        for k,v in kw.items() :
            setattr( self, k, v )

    def fullname ( self ) :
        if self.tablename and self.fieldname :
            return self.tablename + '.' + self.fieldname
        else :
            return None

    def clone ( self, *args, **kw ) :
        """Metoda slouzi pro vytvoreni noveho objektu
           s mirne pozmenenymi parametry od puvodniho.
           Nove parametry se predavaji stejne jako
           v konstruktoru.
        """
        nd = self.__dict__.copy()
        for k,v in kw.items() :
            nd[k] = v
        return self.__class__(*args, **nd)

    def Title ( self ) :
        if self.title :
            return self.title
        return str(self.fieldname)

    ### Nasledujici metody slouzi pro prevod hodnot v jinych
    ### reprezentacich na pythonovske typy. Pouzivaji se pri
    ### prevodu z CGI parametru a z retezcu, ktere vraci
    ### Postgres server.

    def str2python ( self, s, strict = 1 ) :
        """Metoda pro prevod retezcu na odpovidajici
           pythonovsky typ. Prevede napr. retezec
           '123' na cislo 123, pokud se jedna o objekt
           fieldint.

           Pokud nelze pozadovany prevod provest (napr.
           retezec '12xxFF' na integer), metoda vyvola
           standartni vyjimku ValueError.

           Toto plati jenom tehdy, pokud je nepovinny
           parametr strict nastaven na hodnotu > 0.
           Jinak je chyba ignorovana a metoda vrati None.
        """
        try :
            if self.typeid == WM_INT :
                return string.atoi(s)
            elif self.typeid == WM_FLOAT :
                return string.atof(s)
            elif self.typeid == WM_LONG :
                return string.atol(s)
            elif self.typeid == WM_DATE :
                return date.DateFromString( s )
            elif self.typeid == WM_DATETIME :
                return date.DateTimeFromString( s )
            elif self.typeid == WM_INTERVAL :
                return date.IntervalFromString( s )
            elif self.typeid == WM_BYTEA :
                return bytea_unescape(s)
            elif self.typeid == WM_BOOL :
                if not s : return 0
                elif s in [ 'f', '0', 0, self.yesno[1]] : return 0
                return 1
            else :
                return s
        except :
            if strict :
                raise ValueError, messages.ERR_SYNTAX_ERR_IN_VAL % (
                    s, typeid2string[ self.typeid ] )
            else : return None

    def sql2python ( self, val ) :
        """Metoda pro prevod SQL vysledku do pythonu.
           Meodta vola metodu str2python a osetruje
           vicedimenzionalni polozky (array).
        """
        if self.dimension > 0 :
            class create_array :
                def __call__ (this, self, aspl) :
                    a = []
                    while aspl :
                        x = aspl[0]
                        del aspl[0]
                        if x == '{' :
                            a.append(this(self, aspl))
                        elif x == '}' :
                            return a
                        elif x == ',' :
                            pass
                        else :
                            if x[:1] != '"' :
                                v = '"%s"' % x
                            else :
                                v = x
                            a.append(self.str2python(eval(v)))
                    return a
            #print 'VALUE:', val
            R = string.replace(val, '\\\\', '\\' + 'x5c')
            R = string.replace(R, '\\"', '\\' + 'x22')
            aspl = filter(lambda x: x != '', array_split_regex.split(R))
            #print 'SPLIT:', aspl
            if aspl[0] != '{' :
                raise exceptions.ValueError, 'array syntax error (1)'
            return create_array()(self, aspl[1:])
        else :
            return self.str2python( val )

    def param2python ( self, req, strict = 1 ) :
        """Metoda prevede na pythonovskou hodnotu
           hodnoty predane pres CGI parametry.
           Metoda vraci asociativni pole a muze
           vratit i vice hodnot nebo zadnou.
           Parametr par je definovan jako objekt
           FieldStorage v modulu cgi.
        """
        # FIXME je treba osetrit parametry typu a=11&a=12
        # ktere se jevi jako seznam objektu FieldStorage.
        par = req.param
        if self.tablename :
            ke = self.tablename + '.' + self.fieldname
            if par.has_key( ke ) :
                v = par[ke].value
                if v == '' : return { self.fieldname : None }
                return { self.fieldname : self.str2python( v , strict ) }
        if par.has_key( self.fieldname ) :
            v = par[self.fieldname].value
            if v == '' : return { self.fieldname : None }
            return { self.fieldname : self.str2python( v , strict ) }
        return { self.fieldname : None }

    ### Nasledujici metody slouzi pro prevod hodnot
    ### z pythonovskych typu na jine reprezentace.


    def python2sql ( self, val, quote = "'" ) :
        """Metoda pro prevod pythonovske hodnoty na formu
           vhodnou pro zakomponovani do SQL dotazu. Provadi
           osetreni znaku se specialnim vyznamem a take
           dokaze prevest vicedimenzionalni polozky.
        """
        def python2str ( self, val, quote = "'" ) :
            if self.typeid == WM_DATE :
                if type(val) == type('') :
                    val = date.DateFromString(val)
                return quote + val.postgres + quote
            elif self.typeid == WM_DATETIME :
                if type(val) == type('') :
                    val = date.DateTimeFromString(val)
                return quote + val.postgres + quote
            if self.typeid == WM_INTERVAL :
                if self.typename == 'time' :
                    # if zaporny interval
                    return quote + ("%02d:%02d:%02d" % (\
                            val.hour, val.min, val.sec)) + quote
                return quote + val.fmt3 + quote
            if self.typeid == WM_BOOL :
                if val : return quote + 't' + quote
                else : return quote + 'f' + quote
            if self.typeid == WM_INT or self.typeid == WM_FLOAT :
                return str(val)
            if self.typeid == WM_LONG :
                x = str(val)
                if x[-1:] == 'L' :
                    return x[:-1]
                return x
            if self.typeid == WM_BYTEA :
                return quote + bytea_escape(str(val)) + quote
            return quote + sql_escape(str(val)) + quote

        if val is None :
            return 'NULL'
        if self.dimension > 0 and type(val) == type([]) :
            class list2str :
                def __call__ ( this, self, li, p2s ) :
                    def array_escape ( self, s ) :
                        s = string.replace( s, "\\", "\\\\" )
                        s = string.replace( s, '"', '\\\\"' )
                        s = string.replace( s, "'", "\\'" )
                        s = string.replace( s, "\011", "\\t" )
                        s = string.replace( s, "\012", "\\n" )
                        s = string.replace( s, "\015", "\\r" )
                        return s
                    s = []
                    for l in li :
                        if type(l) == type([]) :
                            s.append( this( self, l, p2s ))
                        else :
                            s.append( p2s( self, l, '"' ))
                    return '{' + string.join( s, ',' ) + '}'
            return quote + list2str()( self, val, python2str ) + quote
        else :
            return python2str( self, val, quote )

    def python2html ( self, val, empt = '' ) :
        """Metoda pro prevod pythonovske hodnoty na formu
           vhodnou pro zakomponovani do HTML kodu. U hodnot
           typu string zaroven provede osetreni znaku, ktere
           maji v HTML specialni vyznam.
        """
        if val is None :
            return empt
        if self.typeid == WM_DATE or self.typeid == WM_DATETIME :
            return getattr(val, self.dateformat)
        if self.typeid == WM_INTERVAL :
            return getattr( val, self.intervalformat )
        if self.typeid == WM_BOOL :
            if val : return self.yesno[0]
            else : return self.yesno[1]
        if self.typeid in [WM_INT, WM_FLOAT, WM_LONG] :
            return str(val)
        val = str(val)
        if not val : return empt
        return html_escape(val, quote=1)

    def python2form (self, req, rec, err, empt = '', **kw) :
        """Metoda pro vykresleni vstupniho policka ve formulari.
        """
        if kw.has_key( 'color_column1' ) : color_column1 = kw['color_column1']
        else : color_column1 = "#FFFFFF"
        if kw.has_key( 'color_column2' ) : color_column2 = kw['color_column2']
        else : color_column2 = "#EEEEEE"
        if kw.has_key( 'color_error' ) : color_error = kw['color_error']
        else : color_error = "#AA0000"

        if 'E' in self.hidden :
            try : v = self.python2html(rec[self.fieldname], empt)
            except : v = empt
            s = '<input type="hidden" name="%s" value="%s">\n' % (
                self.fieldname, v)
        else :
            s = '<tr><td bgcolor="%s" valign="top">' % color_column1
            s = s + '<b>%s</b>' % self.Title()
            if self.notnull : s = s + '<sup>*</sup>'
            if self.description :
                s = s + '<br><i>%s</i>' % self.description
            s = s + '</td><td bgcolor="%s" valign="center">' % color_column2
            try : v = rec[self.fieldname]
            except : v = empt
            s = s + self.input( req, rec, empt, v )
            e = err.GetErr( self.fieldname )
            if e :
                err.DelErr( self.fieldname )
                s = s + '<div><font color="%s">' % color_error
                s = s + string.join( e, '<br>' )
                s = s + '</font></div>'
            s = s + '</td></tr>\n'
        return s

    def input ( self, req, rec, empty, v  ) :
        try : v = self.python2html( v, empty )
        except : v = empty
        if self.readonly :
            ro = self.readonly
            s = '<input type="hidden" name="%s" value="%s">\n' % (
                self.fieldname,v)
            if v != empty and type(ro) == type({}) :
                db = getattr( req.glob, req.data.form_object.database )
                t = db[ro['table']]
                r = t( rec[self.fieldname] )
                if r :
                    sl = []
                    for f in ro['view'] : sl.append( t[f].python2html( r[f]) )
                    try : delim = ro['delim']
                    except : delim = '/'
                    return s + '&nbsp;<b>%s</b>' % string.join( sl, delim )
            s = s + '&nbsp;<b>%s</b>' % v
            return s
        if self.typeid == WM_BOOL :
            s = '<select name="%s">' % self.fieldname
            if not self.notnull :
                s = s + '<option value=""></option>'
            if v == self.yesno[0] :
                s = s+'<option value="t" selected>%s</option>' % (self.yesno[0])
            else :
                s = s + '<option value="t">%s</option>' % (self.yesno[0])
            if v == self.yesno[1] :
                s = s+'<option value="f" selected>%s</option>' % (self.yesno[1])
            else :
                s = s + '<option value="f">%s</option>' % (self.yesno[1])
            s = s + '</select>'
        elif self.typeid in [ WM_INT, WM_LONG, WM_FLOAT ] :
            s = '<input type="text" name="%s" value="%s" size="12">' % (\
                self.fieldname, v )
        elif self.typeid == WM_STRING and self.textarea :
            w = self.textareawrap
            if w : w = 'wrap="%s"' % w
            else : w = ''
            s = '<textarea name="%s" rows="%d" cols="%d" %s>%s</textarea>' % (\
                self.fieldname, self.textarea[0], self.textarea[1], w, v )
        elif self.typeid == WM_DATE :
            s = ('<input type="text" name="%s" value="%s" ' +\
                'size="10" maxlength="10">') % (self.fieldname, v )
            if self.popup == -1 :
                pup = copy.copy( self.popup_date )
                pup['kw']['dateformat'] = self.dateformat
                s = s + self.popup_ref( req, pup )
        else :
            ms = self.maxsize
            if self.maxlen and self.maxlen < ms :
                ms = self.maxlen
            s = '<input type="text" name="%s" value="%s" size="%d"' % (\
                self.fieldname, v, ms )
            if self.maxlen :
                s = s + ' maxlength="%d"' % self.maxlen
            s = s + '>'
        s = s + self.popup_ref( req )
        return s

    def popup_ref ( self, req, pup = None, formname=None, fieldname=None ) :
        if pup is None : pup = self.popup
        if type(pup) != type({}) : return ''

        if pup.has_key( 'kw' ) : kw = pup['kw']
        else : kw = {}
        if pup.has_key( 'param' ) : param = pup['param']
        else : param = {}
        if pup.has_key( 'width' ) : wi = pup['width']
        else : wi = 600
        if pup.has_key( 'height' ) : he = pup['height']
        else : he = 500

        if not formname :
            kw['formname'] = req.data.form_object.form_name
        else :
            kw['formname'] = formname
        kw['fieldname'] = self.fieldname

        u = req.relurlh( pup['object'], kw = kw, param = param )
        s = '&nbsp;<a href="" target="WM_POPUP" onclick='
        if self.regex_search :
            s = s + '''"popup_window('%s','%s','%s',%d,%d);''' % (
                        u, kw['formname'], kw['fieldname'], wi, he )
        else :
            s = s + '''"popup_window_no_subs('%s','%s','%s',%d,%d);''' % (
                        u, kw['formname'], kw['fieldname'], wi, he )
        s = s + 'return false;">'
        if pup.has_key( 'ref' ) :
            s = s + pup['ref'] + '</a>'
        else :
            s = s + '<img src="/_wmlib/img/select.gif" border="0"></a>'
        return s

class field_pre ( field ) :

    def python2html ( self, val, empt = '' ) :
        if val is None :
            val = empt
        return '<pre>' + val + '</pre>\n'

class field_select ( field ) :
    """V konstruktoru teto tridy je treba zadat parametr select.
       Ten muze mit nekolik tvary.

       1. [ 'val1', 'val2', 'val3',.... ]
       2. [ ('val1','desc1'), ('val2','desc2'),.... ]
       3. { 'table' : <tabulka>,
            'key' : <policko_ktere_se_bude_doplnovat>,
            'view' : [<policko_pro_zobrazeni_1>,<policko_pro_zobrazeni_2>,.. ],
            'order' : <policko_pro_trideni>,
            'delim' : <oddelovaci znak pro zobrazene polozky>,
            'where' : <where retezec>,
            }
       Pripady 1 a 2 umoznuji zadat pouze staticky seznam hodnot
       pro vyber. Varianta 3 provede prvni vyber polozek z databaze.
    """
    select = None

    def input ( self, req, rec, empty, v  ) :
        x = self.select
        if type(x) == type({}) :
            sel = []
            db = getattr( req.glob, req.data.form_object.database )
            q = "SELECT " + string.join([x['key']] + x['view'], ', ' )
            q = q + " FROM " + x['table']
            if x.has_key('where') :
                w = x['where'] % rec
                q = q + " WHERE " + w
            if x.has_key('order') : q = q + " ORDER BY " + x['order']
            q = q + ';'
            res = db.Query( q )
            if res.OK() :
                vl = x['view']
                ke = x['key']
                try : de = x['delim']
                except : de = '/'
                for r in res :
                    sel.append(( r[0], string.join(map(str,r[1:]),de) ))
        else :
            sel = self.select

        if self.readonly :
            try : v = self.python2html( v, empty )
            except : v = empty
            s = '<input type="hidden" name="%s" value="%s">\n' % (
                self.fieldname, v)
            for se in sel :
                if (type(se) in [type(''), type(0), type(0.0)] and v == se) :
                    s = s + '&nbsp;<b>%s</b>' % str(se)
                    return s
                elif v == se[0] :
                    s = s + '&nbsp;<b>%s</b>' % str(se[1])
                    return s
            s = s + '&nbsp;<b>%s</b>' % v
            return s

        s = '<select name="%s">' % self.fieldname
        if not self.notnull :
            s = s + '<option value=""></option>'
        for se in sel :
            if type(se) in [type(''), type(0), type(0.0)] :
                if v == se :
                    s = s + '<option value="%s" selected>%s</option>' % (se,se)
                else :
                    s = s + '<option value="%s">%s</option>' % (se,se)
            else :
                if v == se[0] :
                    s = s + '<option value="%s" selected>%s</option>' % (
                        se[0],se[1])
                else :
                    s = s + '<option value="%s">%s</option>' % (se[0],se[1])
        s = s + '</select>'

        return s

    def python2html ( self, val, empt = '' ) :
        sel = self.select
        if type(sel) != type({}) :
            for se in sel :
                if type(se) not in [type(()), type([])] :
                    if se == val : return html_escape(str(val))
                else :
                    if se[0] == val : return html_escape(se[1])
        if not val : return empt
        return apply( field.python2html, (self,val,empt), {} )

class field_hidden ( field ) :

    hidden = 'E'

    def python2form ( self, req, rec, err, empt = '', **kw ) :
        return ''

class field_hidden_user ( field_hidden ) :

    user_var = 'AUTH_PGSQL_LOGIN'
    def param2python ( self, req, strict = 1 ) :
        if hasattr( req.data, 'form_object' ) :
            return { self.fieldname : req.env[self.user_var] }
        else :
            return { self.fieldname : None }

class field_hidden_now ( field_hidden ) :

    def param2python ( self, req, strict = 1 ) :
        if hasattr( req.data, 'form_object' ) :
            return { self.fieldname : date.Now() }
        else :
            return { self.fieldname : None }

class field_file ( field ) :

    def param2python ( self, req, strict = 1 ) :
        def return_str ( self, nm, ke, par ) :
            rs = { nm : par[ke].value }
            try :
                fn = par[ke].filename
                fn = re.split( '[\\\\/]', fn )[-1]
                rs[ nm + '_filename' ] = fn
                m,e = mimetypes.guess_type( fn )
                rs[ nm + '_mimetype' ] = m
            except :
                rs[ nm + '_filename' ] = None
                rs[ nm + '_mimetype' ] = None
            rs[ nm + '_filesize' ] = len(rs[nm])
            return rs

        par = req.param
        if self.tablename :
            ke = self.tablename + '.' + self.fieldname
            if par.has_key( ke ) :
                return return_str ( self, self.fieldname, ke, par )
        if par.has_key( self.fieldname ) :
            return return_str ( self, self.fieldname, self.fieldname, par )
        return {
            self.fieldname : None,
            self.fieldname + '_filename' : None,
            self.fieldname + '_filesize' : None,
            self.fieldname + '_mimetype' : None,
            }

    def input ( self, req, rec, empty, v  ) :
        s = '<input type="file"  name="%s">' % self.fieldname
        return s


class field_virtual ( field ) :

    fieldname = None
    tablename = None

    #def str2python ( self, s, strict = 1 ) :
    #    return None

    #def param2python ( self, req, strict = 1 ) :
    #    return {self.fieldname : None}

    def self2list ( self, req, rec ) :
        return str(rec.keys())

class field_virtual_popup_set ( field_virtual ) :

    label_set = '[Set]'

    def self2list ( self, req, rec ) :
        s = '''<a href="" onclick="fill_form('%s','%s','%s');">''' % (
            req.kw['formname'], req.kw['fieldname'], rec[req.kw['key']])
        s = s + self.label_set + '</a>'
        return s
