import string
from wm3.utils import html_escape

__doc__ = """
Pouziti objektu:

ol = OL( 'prvni', 222, LI('treti'), type="I" )
ol.add( 'ctvrty' )
str(ol)

vysledek:

 <ol type="I">
 <li>prvni</li>
 <li>222</li>
 <li>treti</li>
 <li>ctvrty</li>
 </ol>

tab = TABLE( border=0, cellpadding=3 )
tab.add( TR( 'c11', 'c12' ) )
tab.add( TR( TD( 'c21', bgcolor="green"), TD( 'c22', align="left") ))
str(tab)

vysledek:

  <table border="0" cellpadding="3">
  <tr>
  <td>c11</td>
  <td>c12</td>
  </tr>
  <tr>
  <td bgcolor="green">c21</td>
  <td align="left">c22</td>
  </tr>
  </table>

"""

NOATTR = object()
NOVAL = object()


class HTML :

    _tag = 'html'
    _delim = ''
    _nl = 1
    _require = None
    _allowed_types = [ type(''), type(0) ]
    _content = 1

    def __init__ ( self, *data, **kw ) :
        for k,v in self.__class__.__dict__.items() :
            if type(v) in self._allowed_types :
                setattr( self, k, v )
        for k,v in kw.items() :
            setattr( self, k, v )
        self._data = list(data)

    def set ( self, k, v ) :
        setattr( self, k, v )

    def insert ( self, *data ) :
        self._data = list(data) + self._data

    def add ( self, *data ) :
        self._data = self._data + list(data)

    def __str__ ( self ) :
        s = '<'  + self. _tag
        for k,v in self.__dict__.items() :
            if k[0] == '_' : continue
            k = string.lower(k)
            if v is NOATTR:
                pass
            elif v is None or v is NOVAL:
                s = s + ' %s' % k
            else :
                s = s + ' %s="%s"' % (k,html_escape(str(v)))
        s = s + '>'
        if self._nl > 1 : s = s + '\n'
        if not self._content : return s
        if self._require :
            for r in self._data :
                if type(r) != type(self) or \
                    not isinstance( r, self._require) :
                    r = apply( self._require, (r,), {} )
                s = s + str(r)
        else :
            s = s + string.join( map(str,self._data), self._delim )
        s = s + '</' + self._tag + '>'
        if self._nl : s = s + '\n'
        return s

class HEAD (HTML) :
    _tag = 'head'
    _content = 1
    _nl = 2

class TITLE (HTML) :
    _tag = 'title'
    _content = 1
    _nl = 2

class LINK (HTML) :
    _tag = 'link'
    _content = 0
    _nl = 2

class META (HTML) :
    _tag = 'meta'
    _content = 0
    _nl = 2
    def __str__ (self) :
        s = HTML.__str__(self)
        return string.replace(s, 'http_equiv', 'http-equiv')

class BODY (HTML) :
    _tag = 'body'
    _content = 1
    _nl = 2

class STYLE (HTML) :
    _tag = 'style'
    _content = 1
    _nl = 2


class CONT (HTML) :
    _tag = ''
    _content = 1
    def __str__ ( self ) :
        return string.join(map(str,self._data), self._delim)

class BR (HTML) :
    _tag = 'br'
    _content = 0

class HR (HTML) :
    _tag = 'hr'
    _content = 0

class A (HTML) :
    _tag = 'a'
    _nl = 0


class IMG (HTML) :
    _tag = 'img'
    _nl = 0
    _content = 0

class P (HTML) :
    _tag = 'p'
    _nl = 1

class PRE (HTML) :
    _tag = 'pre'
    _nl = 1

class CENTER (HTML) :
    _tag = 'center'
    _nl = 1

class ABBR (HTML) :
    _tag = 'abbr'
    _nl = 0

class ACRONYM (HTML) :
    _tag = 'acronym'
    _nl = 0

class STRONG (HTML) :
    _tag = 'strong'
    _nl = 0

class B (HTML) :
    _tag = 'b'
    _nl = 0

class I (HTML) :
    _tag = 'i'
    _nl = 0

class SMALL (HTML) :
    _tag = 'small'
    _nl = 0

class CODE (HTML) :
    _tag = 'code'
    _nl = 0

class LI (HTML) :
    _tag = 'li'
    _nl = 1

class FONT (HTML) :
    _tag = 'font'
    _nl = 0

class SUP(HTML):
    _tag = 'sup'
    _nl = 0

class SUB(HTML):
    _tag = 'sub'
    _nl = 0

class UL (HTML) :
    _tag = 'ul'
    _nl = 2
    _require = LI

class OL (HTML) :
    _tag = 'ol'
    _nl = 2
    _require = LI

class DL (HTML) :
    _tag = 'dl'
    _nl = 2
class DD (HTML) :
    _tag = 'dd'
    _nl = 1
class DT (HTML) :
    _tag = 'dt'
    _nl = 1

class TH (HTML) :
    _tag = 'th'
    _nl = 1
class TD (HTML) :
    _tag = 'td'
    _nl = 1
class TDl ( TD ) :
    align="left"
class TDr ( TD ) :
    align="right"
class TDc ( TD ) :
    align="center"
class TR (HTML) :
    _tag = 'tr'
    _nl = 2
class COL (HTML) :
    _tag = 'col'
    _content = 0
class TABLE (HTML) :
    _tag = 'table'
    _nl = 2
    _require = TR
class THEAD (HTML) :
    _tag = 'thead'
    _nl = 2
class TBODY (HTML) :
    _tag = 'tbody'
    _nl = 2

class DIV (HTML) :
    _tag = 'div'
    _nl = 2

class SPAN (HTML) :
    _tag = 'span'
    _nl = 0

class H1 (HTML) :
    _tag = 'h1'
    _nl = 2

class H2 ( H1 ) : _tag = 'h2'
class H3 ( H1 ) : _tag = 'h3'
class H4 ( H1 ) : _tag = 'h4'
class H5 ( H1 ) : _tag = 'h5'
class H6 ( H1 ) : _tag = 'h6'


class BLOCKQUOTE (HTML) :
    _tag = 'blockquote'
    _nl = 2


class FORM (HTML) :
    _tag = 'form'
    _nl = 1

class FIELDSET (HTML) :
    _tag = 'fieldset'
    _nl = 2

class LABEL (HTML) :
    _tag = 'label'
    _nl = 1

class LEGEND (HTML) :
    _tag = 'legend'
    _nl = 2

class INPUT (HTML) :
    _tag = 'input'
    _content = 0

class OPTION (HTML) :
    _tag = 'option'
    _nl = 1

class SELECT (HTML) :
    _tag = 'select'
    _nl = 2
    _require = OPTION

class TEXTAREA (HTML) :
    _tag = 'textarea'
    _nl = 1

class FRAMEDBOX :

    padding_out = 3
    padding_in = 4
    color_in = '#FFFFFF'
    color_out = '#E0C6A8'
    width = None
    _delim = ''

    def __init__ ( self, title, *data, **kw ) :
        for k,v in kw.items() :
            setattr( self, k, v )
        self._data = list(data)
        self.title = title

    def add ( self, *data ) :
        self._data = self._data + map( str, list(data))

    def __str__ ( self ) :
        ti = TABLE( border=0,
                    cellpadding=self.padding_in,
                    cellspacing = 0 )
        if self.width : ti.set( 'width', self.width - (self.padding_out*2) )
        tid = TD( string.join( self._data, self._delim ),
                  bgcolor = self.color_in )
        ti.add( TR( tid ) )
        to = TABLE( border=0,
                    cellpadding=self.padding_out,
                    cellspacing = 0 )
        if self.width : to.set( 'width', self.width )
        tod = TD( '<b>', self.title, '</b>',
                  bgcolor = self.color_out )
        to.add( TR( tod ) )
        tod = TD( str(ti),
                  bgcolor = self.color_out )
        to.add( TR( tod ) )
        return str(to)

class SCRIPT ( HTML ) :
    _tag = 'script'
    _nl = 2


class MAP ( HTML ) :
    _tag = 'map'
    _nl = 2


class AREA ( HTML ) :
    _tag = 'area'
    _nl = 1
    _content = 0


class OBJECT (HTML) :
    _tag = 'object'
    _nl = 2
    _content = 1

class PARAM (HTML) :
    _tag = 'param'
    _nl = 1
    _content = 0

class EMBED (HTML) :
    _tag = 'embed'
    _nl = 2
    _content = 1

class IFRAME (HTML) :
    _tag = 'iframe'
    _content = 1
    _nl = 2
