# -*- coding: ISO-8859-2 -*-

import pgdb, date, fields

__all__ = [
  'istable', 'idtable', 'record_hist', 'table_hist', 'memorytable', 'singleton'
]

class istable (pgdb.table) :
    """Toto je prapredek vsech tabulek v IS3.

       Usnadnuje tvorbu nekterych kontrol.
    """

    ISTABLE_DEFAULTS = {}
    ISTABLE_NOTNULL = ()
    ISTABLE_UNIQUE = ()
    ISTABLE_IMMUTABLE = ()
    ISTABLE_REFERENCES = ()
    ISTABLE_REFENCED_FROM = ()


    __GetIDNameCache = None

    def GetIDName (self) :
        if self.__GetIDNameCache is None :
            self.__GetIDNameCache = self.PrimaryKey()[0]
        return self.__GetIDNameCache


    def _setDefaultValues (self, rec, res, defaults=None) :
        if defaults is None :
            defaults = self.ISTABLE_DEFAULTS
        for f,v in defaults.items() :
            if not rec.has_key(f) :
                rec[f] = v
        return res


    def _checkNotNull (self, rec, res, notnull=None, msg=None) :
        if notnull is None :
            notnull = self.ISTABLE_NOTNULL
        if msg is None :
            msg = "Poloka mus bt vyplnna."
        for f in notnull :
            if rec.get(f) in [None, ''] :
                res.AddErr(f, msg)
        return res

    def _checkUnique (self, old, new, res, unique=None, msg=None) :
        if unique is None :
            unique = self.ISTABLE_UNIQUE
        if msg is None :
            msg = "Tato hodnota se ji v tabulce %s vyskytuje."  % (
                  self.tablename)
        for f in unique :
            val = new.get(f)
            if val is None :
                continue
            if self.NItems(f, val) :
                if old and old[f] == val :
                    continue
                res.AddErr(f, msg)
        return res


    def _checkReferences (self, rec, res, references=None) :
        # Podporovana syntaxe pravidel:
        #
        #  <atr1>[","<atr2>[...]]:<tab>["("<Atr1>[","<Atr2>[...]]")"]
        #
        #  atrN  nazvy atributu v puvodni tabulce
        #  tab   nazev cilove tabulky
        #  AtrN  nazvy atributu v cilove tabulce. Pokud nejsou zadany
        #        predpokladaji se stejne nazvy jako v puvodni tabulce.
        #
        #  Priklad:
        #
        #        idfv:faktura_vydana  = idfv:faktura_vydana(idfv)
        #        id,pc:uhrada_zdroj = id,pc:uhrada_zdroj(id,pc)
        #        polozka:saldo(id)
        #

        def parse (s) :
            f, t = s.split(':')
            ff = f.split(',')
            if t.find('(') > 0 :
                tab, tf = t.split('(')
                if tf[-1:] != ")" :
                    raise SyntaxError("YYY")
                tf = tf[:-1].split(',')
                lt = len(tf)
                lf = len(ff)
                if lt < lf :
                    tf.extend(ff[-(lf-lt):])
                elif lt > lf :
                    raise SyntaxError("XXX")
            else :
                tab = t
                tf = ff
            return ff, tab, tf

        if references is None :
            references = self.ISTABLE_REFERENCES
        db = self.db
        for pattern in references :
            if type(pattern) in [type(()), type([])] :
                msg = pattern[1]
                pattern = pattern[0]
            else :
                msg = None
            ff, tab, tf = parse(pattern)
            ff, tf = tuple(ff), tuple(tf)

            rrec = dict(map(lambda f,t: (t,rec.get(f)), ff, tf))
            if None in rrec.values() :
                continue
            tabhandler = db[tab]
            q = "SELECT * FROM %s WHERE %s FOR UPDATE;" % (
                    tab, tabhandler.Record2Where(rrec))
            rr = db(q, tabhandler.RECORD)
            if len(rr) < 1 :
                if msg is None :
                    msg = "Zznam s hodnotou kle %s v tabulce %s neexistuje."
                    hodnota = ",".join([str(v) for v in rrec.keys()])
                    msg = msg % (hodnota, tab)
                else :
                    msg = msg % rec
                if len(ff) == 1 :
                    res.AddErr(ff[0], msg % rec)
                else :
                    res.AddErr(ff, msg % rec)
            elif len(rr) == 1 :
                if len(ff) == 1 :
                    res._REFERENCES[ff[0]] = rr[0]
                else :
                    res._REFERENCES[ff] = rr[0]
            else :
                res.AddErr(ff, "Bylo nalezeno vic hodnot.")
        return res


    def InsertCheck (self, rec, res) :
        res._REFERENCES = {}

        # Doplni defaultni hodnoty
        res = self._setDefaultValues(rec, res)
        if not res.OK() :
            return res

        # Kontroluje povinne polozky
        res = self._checkNotNull(rec, res)
        if not res.OK() :
            return res

        # Kontroluje jedinecne polozky
        res = self._checkUnique(None, rec, res)
        if not res.OK() :
            return res

        # Kontroluje odkazy do jinych tabulek
        res = self._checkReferences(rec, res)
        return res


    def _checkImmutable (self, old, new, res, immutable=None, msg=None) :
        if immutable is None :
            immutable = self.ISTABLE_IMMUTABLE
        if msg is None :
            msg = "Poloku nelze mnit."
        for f in self.ISTABLE_IMMUTABLE :
            if old[f] != new[f] :
                res.AddErr(f, msg)
        return res


    def UpdateCheck (self, old, new, res) :
        res._REFERENCES = {}

        # Kontroluje povinne polozky
        res = self._checkNotNull(new, res)
        if not res.OK() :
            return res

        # Kontroluje polozky, ktere nelze menit
        res = self._checkImmutable(old, new, res)
        if not res.OK() :
            return res

        # Kontroluje jedinecne polozky
        res = self._checkUnique(old, new, res)
        if not res.OK() :
            return res

        # Kontroluje odkazy do jinych tabulek
        res = self._checkReferences(new, res)
        return res


    def _checkReferencedFrom (self, rec, res, referencedFrom=None, msg=None) :
        if referencedFrom is None :
            referencedFrom = self.ISTABLE_REFENCED_FROM
        if not referencedFrom :
            return res
        if msg is None :
            msg = ("Tento zznam se vyskytuje v souvisejcch tabulkch (%s) "
                   "a proto jej nelze smazat.")
        id = self.GetIDName()
        db = self.db
        rf = []
        for ta in referencedFrom :
            try :
                t,a = ta.split('.')
            except :
                t,a = ta, id
            if db[t].NItems(a, rec[id]) :
                rf.append(t)
        if rf :
            res.AddErr(None, msg % (", ".join(rf)))
        return res


    def DeleteCheck (self, rec, res) :
        res._REFERENCES = {}
        res = self._checkReferencedFrom(rec, res)
        return res


    def _deleteReferences (self, rec, res, references=None, msg=None) :
        if references is None :
            references = self.ISTABLE_REFERENCES
        if msg is None :
            msg = "Nelze smazat souvisejc zznamy z tabulky %s " \
                  "s hodntou kle %%(%s)s "
        db = self.db
        for ta in references :
            if type(ta) in [type(()), type([])] :
                a, t = ta[0].split(':')
                m = ta[1]
            else :
                a, t = ta.split(':')
                m = msg % (a, t)
            if rec.get(a) is None :
                continue
            rr = db[t](rec[a])
            if not rr :
                res.AddErr(a, m % rec)
            res._REFERENCES[a] = rr
        return res








class idtable (istable) :
    """Toto je prapredek vsech tabulek s jedinecnym identifikatorem
       ve tvaru <PREFIX><CISLO>.
    """

    ISTABLE_PREFIX = 'ID'


    def _newID (self, rec, res) :
        # Vygeneruje nove ID
        n = self.db("SELECT nextval('wm3_id_gen');")[0][0]
        p = self.ISTABLE_PREFIX
        rec[self.GetIDName()] = ("%s%%0%dd" % (p, 18-len(p))) % n
        return res

    def _setDefaultValues (self, rec, res, defaults=None) :
        res = istable._setDefaultValues(self, rec, res, defaults)
        if not res.OK() : return res
        return self._newID(rec, res)

    def _checkNotNull (self, rec, res, notnull=None, msg=None) :
        if notnull is None :
            notnull = self.ISTABLE_NOTNULL
        id = self.GetIDName()
        if id not in notnull :
            notnull = list(notnull).append(id)
        return istable._checkNotNull(self, rec, res, notnull, msg)


    def _checkImmutable (self, old, new, res, immutable=None, msg=None) :
        if immutable is None :
            immutable = self.ISTABLE_NOTNULL
        id = self.GetIDName()
        if id not in immutable :
            immutable = list(immutable).append(id)
        return istable._checkImmutable(self, old, new, res, immutable, msg)







class record_hist (pgdb.record) :

    field_platnost_od = fields.field (
        title = 'Platnost zznamu od',
        description = '''Odkdy tento zznam plat. Pokud chcete
                         zmnu naplnovat dopedu, lze nastavit toto datum
                         do budoucnosti. Zmna se potom provede v pslun
                         den automaticky.''',
        )

    field_verze = fields.field (
        title = 'Verze zznamu',
        description = '''slo verze tohoto zznamu. Pi editaci
                         jsou star zznamy ukldny do historie a kadmu
                         zznamu je pidleno poadov slo vzniku.''',
        readonly = 1,
        )





class table_hist (idtable) :

    # Editace techto polozek zpusobi vznik noveho zaznamu v historii
    ISTABLE_HISTORIE = ()


    def __call__ (self, idrec, verze=None) :
        # Aktualni verze
        idattr = self.GetIDName()
        oakt = pgdb.table.__call__(self, idrec)
        if oakt :
            # Aktualni zaznam nalezen
            if verze == None :
                # Nebyla zadana konkretni verze
                return oakt
            elif verze == oakt['verze'] :
                # Verze byla pozadovana a shoduje se
                return oakt
            else :
                # Konkretni verze muze byt v historii, ikdyz
                # aktualni zaznam neexistuje.
                q = "SELECT oid, * FROM %s_h WHERE %s='%s' AND verze=%d" % (
                    self.tablename, idattr, idrec, verze)
                r = self.db(q, rec=self.RECORD)
                if not r.OK() :
                    raise 'error', r.Err2TXT()
                if len(r) < 1 :
                    # Konkretni verze nenalezena
                    return None
                rec = r[0]
                rec.OID = rec['oid']
                del rec['oid']
                return rec
        else :
            # Aktualni zaznam nenalezen
            if verze == None :
                # Byla zadana aktualni verze ale ta neexistuje
                return None
            else :
                # Konkretni verze muze byt v historii, ikdyz
                # aktualni zaznam neexistuje.
                q = "SELECT oid, * FROM %s_h WHERE %s='%s' AND verze=%d" % (
                    self.tablename, idattr, idrec, verze)
                r = self.db(q, rec=self.RECORD)
                if not r.OK() :
                    raise 'error', r.Err2TXT()
                if len(r) < 1 :
                    # Konkretni verze nenalezena
                    return None
                rec = r[0]
                rec.OID = rec['oid']
                del rec['oid']
                return rec
        # Pojistka
        return None


    def _newVersionForID (self, idrec) :
        # Vygeneruje nove cislo verze pro dane ID zaznamu
        idattr = self.GetIDName()
        verze = 1
        qv = """SELECT MAX(verze) FROM %s WHERE %s = '%s'
                UNION
                SELECT MAX(verze) FROM %s_h WHERE %s = '%s';
            """ % ( self.tablename, idattr, idrec,
                    self.tablename, idattr, idrec)
        vm = max(map(lambda x: x[0], self.db(qv)))
        if vm : verze = vm + 1
        return verze


    def _setDefaultValues (self, rec, res, defaults=None) :
        res = idtable._setDefaultValues(self, rec, res, defaults)
        if not res.OK() : return res
        rec["verze"] = 1
        if rec.get("platnost_od") is None :
            rec['platnost_od'] = date.Today()
        return res


    def UpdateTriggerBefore (self, old, new, res) :
        # Pokud se nektera vyznamna polozka lisi, zapis puvodni hodnotu
        # zaznamu do tabulky <tab>_h
        idattr = self.GetIDName()
        for f in self.ISTABLE_HISTORIE :
            if old[f] != new[f] :
                r = self.db[self.tablename + '_h'].Insert(old)
                if not r.OK() :
                    res.AddErr(None, 'Nelze uloit zmnu do historie.')
                    res.Adopt(r)
                    return res
                new['verze'] = self._newVersionForID(new[idattr])
                break
        # Pro jistotu
        new['modifikace'] = date.Now()
        return res

    def Update (self, old, new, res = pgdb.result()) :
        # Tohle se musi delat v transakci
        tr = 1
        if not self.db.INTRANSACTION :
            tr = 0
            r = self.db.Begin()
            assert r.OK()
        try :
            idattr = self.GetIDName()
            # Tady jeste nemam kompletni new zaznam
            if not new.has_key('platnost_od') or \
                        new['platnost_od'] <= date.Today() :
                res = pgdb.table.Update(self, old, new, res)
                return res

            # Ulozeni nove verze do tabulky *_h pokud je platnost_od
            # v budoucnosti

            # Nahrani stareho zaznamu
            try : old = self(old[idattr])
            except : old = None
            if not old :
                res.AddErr(None, 'Nelze nalzt pvodn zznam')
                return res

            # Doplneni starych hodnot
            for k,v in new.items() :
                old[k] = v
            old['verze'] = self._newVersionForID(old[idattr])
            # Provedeni insertu
            res = self.InsertCheck(old, res)
            if not res.OK() :
                return res
            res = self.db[self.tablename+'_h'].Insert(old, res)
            return res

        finally :
            if not tr :
                if res.OK() : r = self.db.End()
                else : r = self.db.Abort()
                assert r.OK()


    def DeleteTriggerBefore (self, rec, res) :
        r = self.db[self.tablename + '_h'].Insert(rec)
        if not r.OK() :
            res.AddErr(None, 'Nelze uloit zmnu do historie.')
            res.Adopt(r)
            return res
        return res










class memorytable (istable) :

    __RecordCache = None

    def _ResetCache (self) :
        self.__RecordCache = None

    def __call__ (self, pkey) :
        if self.__RecordCache is None :
            pkn = self.GetIDName()
            self.__RecordCache = rc = {}
            for r in self.db("""SELECT oid,* FROM "%s";""" % self.dbtablename,
                             self.RECORD) :
                rc[r[pkn]] = r
        return self.__RecordCache.get(pkey)

    def Insert (self, rec, res=None) :
        self._ResetCache()
        return istable.Insert(self, rec, res)

    def Update (self, old, new, res=None) :
        self._ResetCache()
        return istable.Update(self, old, new, res)

    def Delete (self, rec, res=None) :
        self._ResetCache()
        return istable.Delete(self, rec, res)








class singleton (istable) :

    __RecordCache = None

    def __call__ (self) :
        if self.__RecordCache is not None :
            return self.__RecordCache
        q = """SELECT * FROM "%s";""" % self.dbtablename
        r = self.db(q, self.RECORD)
        if len(r) > 0 :
            self.__RecordCache = r[0]
        else :
            self.__RecordCache = self.RECORD()
        return self.__RecordCache
