import sys, string, UserDict, types, re, os
import pgint, fields
from pgdb_misc import *
from pgdb_table import *
from pgdb_result import *
from pgdb_record import *
from pgdb_error import *
from pgdb_lo import *
from messages import *

_special_char = '-<>=+*/~@:#%&^?!'
_token_regex = re.compile("((?:'.*?')|[%s]+|,|;|\\(|\\)|\s+)" % _special_char, re.M)

class db :
    """Vsechny metody a atributy objektu db zacinaji velkymi pismeny.
       Atributy zacinajici malymi pismeny jsou vyhrazeny pro nazvy
       tabulek (objekt table), na ktere se lze odkazovat jako
       na atributy objektu db.
    """
    def __init__ ( self, **kw ) :
        # Zpracovani parametru
        keys = [ 'HOST', 'DB', 'USER', 'PASSWORD', 'LOG', 'LOCAL_LO',
                 'BACKLOG', 'PORT', 'OPTIONS', 'TTY', 'TABLE_MODULES',
                 'CLIENT_ENCODING']
        ukw = {}
        for k,v in kw.items() : ukw[string.upper(k)] = v
        par = {}
        for k in keys :
            try : setattr( self, k, ukw[k] )
            except :
                if not hasattr( self, k ) :
                    setattr( self, k, None )
        if self.HOST :
            self.LOCAL_LO = None

        # Vytvoreni pripojeni na databazi
        self._DB = pgint.new( self.HOST, self.DB, self.USER, self.PASSWORD,
                              self.LOG, self.PORT, self.OPTIONS, self.TTY )
        # Logovani hlasek NOTICE od db serveru
        if self.BACKLOG != None :
            self._DB.setnoticeprocessor( self.BACKLOG )

        # Nastaveni formatu casovych udaju
        r = self._DB.query("SET DATESTYLE TO 'Postgres,US';")
        if r.status() != 'command_ok' :
            raise error, ERR_SET_DATESTYLE % self._DB.errormessage()

        # Nastaveni klientskeho kodovani
        if self.CLIENT_ENCODING is not None:
            r = self._DB.query("SET CLIENT_ENCODING TO '%s';" % (
                                                        self.CLIENT_ENCODING))
            if r.status() != 'command_ok' :
                raise error, ERR_SET_CLIENT_ENCODING % self._DB.errormessage()

        # Vytvoreni mapy typu
        r = self._DB.query( "SELECT oid,typname FROM pg_type;" )
        if r.status() != 'tuples_ok' :
            raise error, ERR_READ_TYPE_DEF % self._DB.errormessage()
        self.OidPythonTypeMap = {}
        for i in range(r.ntuples()) :
            to = string.atoi(r.getvalue( i, 0 ))
            tn = r.getvalue( i, 1 )
            if PGPythonTypeMap.has_key(tn) :
                tid = PGPythonTypeMap[tn]
                tm = fields.field( typeid=tid, typeoid=to, typename=tn )
            elif tn[0] == '_' and PGPythonTypeMap.has_key(tn[1:]) :
                tid = PGPythonTypeMap[tn[1:]]
                tm = fields.field( typeid=tid, typeoid=to,
                                   typename=tn, dimension=1 )
            else :
                tm = fields.field( typeoid = to, typename = tn )
            self.OidPythonTypeMap[to] = tm

        # Nastaveni kesi
        self.MODULE = sys.modules[self.__module__]
        self._TABCACHE = None
        self._VIEWCACHE = {}
        self._MODCACHE = {}
        self._P_INDEXCACHE = None
        self._U_INDEXCACHE = None
        self.INTRANSACTION = None
        self.VERSION = None

    _query_split = re.compile("('[^']*')")

    def queryCheck (self, q) :
        """Kontroluje, zda je zadan pouze jeden SQL dotaz.
        """
        n = 0
        q = string.strip(q)
        q = string.replace(q, "\\'", "\\" + oct(ord("'")))
        for p in  self._query_split.split(q) :
            if p[:1] == "'" and p[-1:] == "'" : continue
            n = n + string.count(p, ";")
        if n > 1 : return 0
        if n == 1 and q[-1:] != ';' : return 0
        return 1


    def Query (self, q, rec=record) :
        """Obecny databazovy dotaz.
        """
        if not self.queryCheck(q) :
            fd = open('/tmp/multi-sql.log','a')
            fd.write(q)
            fd.write('\n\n')
            fd.close()
            raise error,\
                "Only one SQL command allowed in Query argument. [%s]" % q
        r = self._DB.query( q )
        s = r.cmdstatus()
        if s == 'BEGIN' :
            self.INTRANSACTION = 1
        elif s in ['END', 'COMMIT', 'ROLLBACK', 'ABORT' ] :
            self.INTRANSACTION = None
        elif s == 'CREATE' or s == 'DROP' :
            self._TABCACHE = None
            self._VIEWCACHE = None
            self._P_INDEXCACHE = None
            self._U_INDEXCACHE = None
        return result_pgint ( self, r, rec )

    __call__ = Query

    def Begin (self) : return self("BEGIN;")
    def End (self) : return self("COMMIT;")
    def Commit (self) : return self("COMMIT;")
    def Abort (self) : return self("ABORT;")
    def Rollback (self) : return self("ABORT;")


    def Version ( self ) :
        if self.VERSION != None : return self.VERSION
        v = self( "SELECT VERSION();" )[0][0]
        v = string.split( string.split( v, ' ' )[1], '.' )
        self.VERSION = (string.atoi(v[0]), string.atoi(v[1]))
        return self.VERSION

    def __getitem__ ( self, tab ) :
        """Tato funkce zajisti, ze na tabulky se lze odkazovat
           jako db['tabulka']. Objekt db se chova jako asociativni
           seznam.
        """
        try :
            t, f = string.split( tab, '.' )
        except :
            t = tab
            f = None
        if f :
            return self.FindTable(t)[f]
        else :
            return self.FindTable(t)

    def keys (self) :
        self.FindTable(None)
        return self._TABCACHE.keys()

    def values (self) :
        self.FindTable(None)
        return self._TABCACHE.values()

    def FindTable (self, tab) :
        """Metoda nalezne tabulku bud primo v databazi
           nebo jako virtualni tabulku definovanou v modulu.
        """
        # Pokud jeste kes nebyla vytvorena, naplnime
        # ji jmeny tabulek bez definice
        if not self._TABCACHE :

            # Dame do kese seznam tabulek bez definice (None).
            self._TABCACHE = {}
            self._VIEWCACHE = {}
            q = """SELECT relname, relkind FROM pg_class
                   WHERE relkind IN ('r','v') AND relname NOT LIKE 'pg_%';"""
            for t,k in self(q) :
                self._TABCACHE[t] = None
                if k == 'v' :
                    self._VIEWCACHE[t] = 1

            # Projdeme modul s definici databaze a najdeme
            # tridy s nazvem 'table_*'
            if self.TABLE_MODULES :
                for t in self.TABLE_MODULES :
                    self._TABCACHE[t] = None
            for t,v in sys.modules[self.__module__].__dict__.items() :
                if t[:6] == 'table_' and type(v) == types.ClassType :
                    self._TABCACHE[t[6:]] = None
        # Pokud nazev tabulky neni v kesi, tabulka neexistuje
        if not self._TABCACHE.has_key(tab) :
            return None

        if self._TABCACHE[tab] == None :

            # Tabulka existuje ale neni vytvorena instance
            # prislusneho objektu table nebo table_<nazev>

            sm = sys.modules[self.__module__]
            if self.TABLE_MODULES and tab in self.TABLE_MODULES :
                # Tabulka je definovana v separe modulu, ktery je
                # pripraven na lazy import
                if not self._MODCACHE.has_key( tab ) :
                    fn = self.__module__ + '.' + tab
                    __import__ ( fn )
                    m = sys.modules[fn]
                    self._MODCACHE[tab] = m
                else :
                    m = self._MODCACHE[tab]
                tabcls = m.__dict__['table_' + tab]
            elif sm.__dict__.has_key( 'table_' + tab ) :
                # Je definovana specialni trida pro tuto tabulku
                # v modulu databaze, proto ji pouzijeme
                tabcls = sm.__dict__['table_' + tab]
            else :
                # Jinak pouzijeme defaultni tridu pro instanci tabulky
                tabcls = table

            # Volani konstruktoru pro vytvoreni instance tabulky
            self._TABCACHE[tab] = apply(tabcls, (self,tab),
                                        {'isview':self._VIEWCACHE.get(tab)})

        # Vratime instanci tabulky
        return self._TABCACHE[tab]

    def PrimaryKey ( self, tab ) :
        """Metoda vrati seznam polozek, ktere tvori
           primarni klic dane tabulky.
        """
        if self._P_INDEXCACHE == None :
            self._P_INDEXCACHE = {}
            q = """
                SELECT  pg_attribute.attnum, pg_class.relname,
                        pg_attribute.attname
                FROM pg_class, pg_attribute, pg_index
                WHERE pg_class.oid = pg_attribute.attrelid
                    AND pg_class.oid = pg_index.indrelid
                    AND (pg_index.indkey[0] = pg_attribute.attnum
                        OR pg_index.indkey[1] = pg_attribute.attnum
                        OR pg_index.indkey[2] = pg_attribute.attnum
                        OR pg_index.indkey[3] = pg_attribute.attnum
                        OR pg_index.indkey[4] = pg_attribute.attnum
                        OR pg_index.indkey[5] = pg_attribute.attnum
                        OR pg_index.indkey[6] = pg_attribute.attnum)
                    AND pg_index.indisprimary = 't'
                ORDER BY relname, attnum;"""
            r = self(q)
            if not r.OK() :
                raise error, ERR_READ_PKEY_DEF % self._DB.errormessage()
            for x in r :
                if self._P_INDEXCACHE.has_key( x[1] ) :
                    self._P_INDEXCACHE[x[1]].append(x[2])
                else :
                    self._P_INDEXCACHE[x[1]] = [ x[2] ]
        if self._P_INDEXCACHE.has_key( tab ) :
            return self._P_INDEXCACHE[tab]
        else :
            return []

    def UniqueKeys ( self, tab ) :
        """Metoda vrati seznam polozek, ktere tvori
           primarni klic dane tabulky.
        """
        if self._U_INDEXCACHE == None :
            self._U_INDEXCACHE = {}
            q = """
                SELECT  pg_index.indexrelid, pg_attribute.attnum,
                        pg_class.relname, pg_attribute.attname
                FROM pg_class, pg_attribute, pg_index
                WHERE pg_class.oid = pg_attribute.attrelid
                    AND pg_class.oid = pg_index.indrelid
                    AND (pg_index.indkey[0] = pg_attribute.attnum
                        OR pg_index.indkey[1] = pg_attribute.attnum
                        OR pg_index.indkey[2] = pg_attribute.attnum
                        OR pg_index.indkey[3] = pg_attribute.attnum
                        OR pg_index.indkey[4] = pg_attribute.attnum
                        OR pg_index.indkey[5] = pg_attribute.attnum
                        OR pg_index.indkey[6] = pg_attribute.attnum)
                    AND pg_index.indisunique = 't'
                ORDER BY indexrelid, attnum;"""
            r = self(q)
            if not r.OK() :
                raise error, ERR_READ_PKEY_DEF % self._DB.errormessage()
            for x in r :
                if not self._U_INDEXCACHE.has_key( x[2] ) :
                    self._U_INDEXCACHE[x[2]] = {}
                if not self._U_INDEXCACHE[x[2]].has_key( x[0] ) :
                    self._U_INDEXCACHE[x[2]][x[0]] = []
                self._U_INDEXCACHE[x[2]][x[0]].append( x[3] )
        if self._U_INDEXCACHE.has_key( tab ) :
            return self._U_INDEXCACHE[tab]
        else :
            return {}

    def LargeNew ( self, file = None ) :
        lo = apply( large_object, (self,), {} )
        if file :
            lo.load( file )
        return lo

    def LargeGet ( self, oid ) :
        return apply( large_object, (self, oid), {} )
