import pgint

error = "BlobError"

class large_object :
    bl = 64 * 1024
    def __init__ ( self, con, oid = None ) :
        self.con = con
        self.oid = oid
        self.fd = None
        self.closed = 1
        self.trans_add = None

    def open ( self, opt = 'r' ) :
        if self.oid and self.fd != None :
            raise error, 'Already opened.'
        mode = 0
        for o in opt :
            if o == 'r' : mode = mode | pgint.INV_READ
            elif o == 'w' : mode = mode | pgint.INV_WRITE
            else : raise error, "Unkonwn mode. Use 'r', 'w' or 'rw'."
        tr = self.con.INTRANSACTION
        if not self.con.INTRANSACTION :
            self.con.Begin()
            self.trans_add = 1
        if self.oid == None :
            self.oid = pgint.lo_create( self.con._DB, mode )
            if self.oid < 1 :
                self.oid = None
                raise error, 'Cannot create large object.'
        self.fd = pgint.lo_open( self.con._DB, self.oid, mode )
        if self.fd < 0 :
            raise error, 'Cannot open large object oid:%d.' % self.oid
        self.closed = 0
        return tr

    def close ( self, tr = 1 ) :
        if self.fd == None :
            raise error, 'Already closed.'
        r = pgint.lo_close( self.con._DB, self.fd )
        if self.trans_add :
            self.con.End()
        self.fd = None
        self.closed = 1
        self.trans_add = None
        return r

    def flush ( self ) :
        return

    def isatty ( self ) :
        return 0

    def fileno ( self ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        return self.oid

    def write ( self, str ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        r = pgint.lo_write( self.con._DB, self.fd, str )
        if r != len(str) :
            raise error, 'Cannot write to large object.'
        return r

    def read ( self, size = None ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        if size :
            s = pgint.lo_read( self.con._DB, self.fd, size )
        else :
            s = ''
            while 1 :
                r = pgint.lo_read( self.con._DB, self.fd, self.bl )
                s = s + r
                if len(r) < self.bl : break
        return s

    def readlines ( self, sizehint = None ) :
        pass

    def seek ( self, offset, whence = pgint.SEEK_SET ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        r = pgint.lo_lseek( self.con._DB, self.fd, offset, whence )
        if r < 0 :
            raise error, 'Invalid seek in large object.'
        return r

    def tell ( self ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        r = pgint.lo_tell( self.con._DB, self.fd )
        if r < 0 :
            raise error, 'Error in large object.'
        return r

    def __len__ ( self ) :
        if self.fd == None or not self.oid :
            raise error, 'Large object not opened.'
        o = self.tell()
        e = self.seek( 0, pgint.SEEK_END )
        self.seek( o, pgint.SEEK_SET )
        return e

    def unlink ( self ) :
        if not self.oid :
            raise error, 'No oid associated with large object.'
        if self.fd != None :
            raise error, 'Cannot unlink opened large object.'
        r = pgint.lo_unlink( self.con._DB, self.oid )
        if r != 1 :
            raise error, 'Cannot unlink large object oid:%d.' % self.oid
        return 1

    def load ( self, file ) :
        if self.fd != None :
            raise error, 'Cannot load opened large object.'
        tr = self.con.INTRANSACTION
        if not tr : self.con.Begin()
        if not self.oid and self.con.LOCAL_LO and type(file) == type('') :
            self.oid = pgint.lo_import( self.con._DB, file )
        else :
            if not self.oid :
                self.oid = pgint.lo_create( self.con._DB, pgint.INV_WRITE )
            self.open( 'w' )
            if not hasattr(file, 'read' ) :
                file = open( file, 'r' )
            while 1 :
                s = file.read( self.bl )
                self.write( s )
                if len(s) < self.bl : break
        if not tr : self.con.End()
        return self.oid

    def save ( self, file ) :
        if self.fd != None :
            raise error, 'Cannot save opened large object.'
        if not self.oid :
            raise error, 'No oid associated with large object.'
        tr = self.con.INTRANSACTION
        if not tr : self.con.Begin()
        if self.con.LOCAL_LO and type(file) == type('') :
            self.oid = pgint.lo_export( self.con._DB, self.oid, file )
        else :
            self.open( 'r' )
            if not hasattr(file, 'write' ) :
                file = open( file, 'w' )
            while 1 :
                s = self.read( self.bl )
                file.write( s )
                if len(s) < self.bl : break
        if not tr : self.con.End()
        return self.oid
