__all__ = ['record', 'active_record']

class record :
    """Tato trida slouzi pro uchovani jednoho radku databaze.
       Vetsinou se tento objekt nevytvari primo, ale je vytvaren
       metodami pro vyber dat z databaze.

       Kazda tabulka muze mit definovaneho vlastniho potomka
       tridy record se specialnimi metodami.

       K datum lze pristupovat bud metodami asociativniho pole
       nebo metodami seznamu.

         >>> r = record([('ahoj',123),('cau','CAU')])
         >>> r[0]
         123
         >>> r['ahoj']
         123
         >>> for v in r : print v,
         ...
         123 CAU 444
         >>> r.items()
         [('ahoj', 123), ('cau', 'CAU')]

       Pokud jsou v zaznamu polozky se stejnymi jmeny (coz se
       casto stava u dotazu pres vice tabulek), je k duplikatum
       pripojena jednoznacna ciselna pripona ahoj0, ahoj1,...

         >>> r = record([('ahoj',123),('cau','CAU'),('ahoj',444)])
         >>> r.items()
         [('ahoj', 123), ('cau', 'CAU'), ('ahoj0', 444)]

       Parametr konstruktoru lze zadat i jako Dictionary:

         >>> r = record({'ahoj':123, 'cau':'CAU'})

       V tomto pripade nemuze nastat pripad se stejnymi jmeny.
    """
    def __init__(self, rec = []):
        self.FIELDS = []
        self.DICT = {}
        if type(rec) == type({}) : rec = rec.items()
        for k,v in rec :
            kt = k
            for i in xrange(1000) :
                if not self.DICT.has_key(kt) :
                    break
                kt = k+str(i)
            self.FIELDS.append(kt)
            self.DICT[kt] = v

    def __getitem__ (self, i) :
        if type(i) == type(0) :
            return self.DICT[self.FIELDS[i]]
        else :
            return self.DICT[i]

    def get (self, i, d=None) :
        return self.DICT.get(i, d)

    def __setitem__(self, i, v) :
        if type(i) == type(0) :
            self.DICT[self.FIELDS[i]] = v
        else :
            if not self.DICT.has_key(i) :
                self.FIELDS.append(i)
            self.DICT[i] = v

    def __delitem__(self, i) :
        if type(i) == type(0) :
            del self.DICT[self.FIELDS[i]]
            del self.FIELDS[i]
        else :
            del self.DICT[i]
            del self.FIELDS[self.FIELDS.index(i)]

    def __getslice__ (self, l, h) :
        ll = len(self)
        if l < 0 :
            l = ll + l
        if h < 0 :
            h = ll + h
        l = min(ll, l)
        h = min(ll, h)
        sl = []
        for i in xrange(l,h) :
            sl.append(self[i])
        return sl

    def __len__ (self) :
        return len(self.FIELDS)

    def has_key (self, i) :
        return self.DICT.has_key(i)

    def keys (self) :
        return self.FIELDS

    def items (self) :
        i = []
        for f in self.FIELDS :
            i.append((f,self.DICT[f]))
        return i

    def values (self) :
        v = []
        for f in self.FIELDS :
            v.append(self.DICT[f])
        return v

    def update (self, d) :
        for k,v in d.items() :
            self[k] = v

    def copy (self) :
        return self.__class__ (self.items())

    def asdict (self) :
        return self.DICT


class active_record(record, object):

    DB = None

    def __getstate__(self):
        return self.items()

    def __setstate__(self, state):
        self.__init__(state)

    def __getattr__(self, attr):
        try:
            return self[attr]
        except:
            raise AttributeError(attr)
