# $Id: pgdb_result.py,v 1.16 2003/10/16 13:19:37 radekk Exp $

from pgdb_record import *
from pgdb_error import *
from messages import *



class result :
    """Zakladni trida pro praci s vysledky SQL dotazu.
       Tato trida pouze dokaze obslouzit chybova hlaseni.
       Nenese v sobe zadna data.
    """
    def __init__ ( self ) :
        class result_data : pass
        self.ERR_DICT = {}
        self.data = result_data()

    def OK ( self ) :
        if len(self.ERR_DICT) > 0 :
            return None
        return 1

    def AddErr ( self, atr, msg ) :
        if not atr : atr = None
        if not self.ERR_DICT.has_key( atr ) :
            self.ERR_DICT[atr] = []
        self.ERR_DICT[atr].append( msg )

    def Adopt ( self, res2 ) :
        for k,v in res2.ErrDict().items() :
            for e in v : self.AddErr( k, e )

    def GetErr ( self, atr ) :
        try :
            return self.ERR_DICT[atr]
        except :
            return []

    def DelErr ( self, atr ) :
        try : del self.ERR_DICT[atr]
        except : pass

    def ErrDict ( self ) :
        return self.ERR_DICT

    def keys ( self ) :
        return self.ERR_DICT.keys()

    def Err2HTML ( self ) :
        s = '<ul>\n'
        for k in self.ERR_DICT.keys() :
            for e in self.ERR_DICT[k] :
                if not k :
                    s = s + '<li>%s</li>\n' % e
                else :
                    s = s + '<li>[%s] %s</li>\n' % (k,e)
        s = s + '</ul>'
        return s

    def Err2TXT ( self ) :
        s = ''
        for k in self.ERR_DICT.keys() :
            for e in self.ERR_DICT[k] :
                if not k :
                    s = s + '  %s\n' % e
                else :
                    s = s + '  [%s] %s\n' % (k,e)
        return s


class result_pgint(result):
    """Tato odvozena trida zapouzdruje objekt pgint.result.
       Pri dotazu typu SELECT se chova jako seznam, kde jednotlive
       prvky seznamu jsou objekty typu pgdb_record.record
       s jednotlivymi zaznamy databaze. Pro testovani uspesnosti
       dotazu se zase pouziva metoda OK().
    """
    def __init__(self, d, r, rec=record):
        result.__init__ ( self )
        self._DB = d
        self._RES = r
        self._REC = rec
        st = r.status()
        if not st in  [ "empty_query","command_ok","tuples_ok",
                        "copy_out","copy_in"]:
            e = r.errormessage()
            if not e : e = 'Undefined Error'
            self.AddErr( None, e )

    def Status ( self ) :
        """Tato metoda vraci nasledujici retezce pro
           prislusne navratove hodnoty:

             PGRES_EMPTY_QUERY     "empty_query"
             PGRES_COMMAND_OK      "command_ok"
             PGRES_TUPLES_OK       "tuples_ok"
             PGRES_COPY_OUT        "copy_out"
             PGRES_COPY_IN         "copy_in"
             PGRES_BAD_RESPONSE    "bad_response"
             PGRES_NONFATAL_ERROR  "nonfatal_error"
             PGRES_FATAL_ERROR     "fatal_error"

           Prvnich pet navratovych kodu znamena
           uspesne dokonceni dotazu. Zbyle tri
           znamenaji chybu pri zpracovani dotazu.
        """
        return self._RES.status()

    def CmdStatus ( self ) :
        """Vraci navratovy kod SQL dotazu,
           napr: UPDATE 2, INSERT 22343, ...
        """
        return self._RES.cmdstatus()

    def CmdTuples ( self ) :
        """Vraci pocet radku kterych se SQL dotaz INSERT,
           UPDATE nebo DELETE dotkl.
        """
        return self._RES.cmdtuples()

    def OidStatus ( self ) :
        """Vraci OID posledniho vlozeneho zaznamu.
        """
        return self._RES.oidstatus()

    def __len__ ( self ) :
        """Pokud je dotaz typu SELECT nebo FETCH,
           vraci tato funkce pocet nalezenych zaznamu.
           Pouziva se pres funkci len().

             >>> r = db("SELECT * FROM pg_type;")
             >>> print len(r)
             115
        """
        if self._RES.status() != 'tuples_ok' :
            raise error, ERR_NO_TUPLES
        return self._RES.ntuples()

    def Fields ( self ) :
        """Vraci nazvy poli v dotazu typu SELECT.
        """
        if self._RES.status() != 'tuples_ok' :
            raise error, ERR_NO_TUPLES
        f = []
        for i in range( self._RES.nfields() ) :
            f.append( self._RES.fname( i ) )
        return f

    def Record ( self, i, tab = None ) :
        """Tato metoda emuluje chovani seznamu. Pokud
           je dotaz typu select, vraci jednotlive zaznamy.
        """
        if self._RES.status() != 'tuples_ok' :
            raise error, ERR_NO_TUPLES
        try :
            r = xrange(self._RES.ntuples())[i]
        except IndexError :
            raise IndexError, "Index out of range (%d tuples)." % \
                                                self._RES.ntuples()
        rec = []
        fn = self.Fields()
        om = []
        for c in range(len(fn)) :
            om.append( self._DB.OidPythonTypeMap[self._RES.ftype(c)] )
        for c in range(len(fn)) :
            if self._RES.getisnull( r, c ) :
                v = None
            else :
                v = om[c].sql2python( self._RES.getvalue(r,c) )
            rec.append( (fn[c],v) )
        return self.WrapRecord(rec)

    def WrapRecord(self, rec):
        REC = self._REC
        if isinstance(REC, (tuple,list)):
            ret = []
            i = 0
            for l,r in REC:
                ret.append(apply(r, (rec[i:i+l],), {}))
                i += l
        else:
            ret = apply(REC, (rec,), {})
            if issubclass(REC, active_record):
                ret.DB = self._DB
        return ret

    def __getitem__ ( self, i ) :
        """Umoznuje pouzivani objektu result jako seznamu.
        """
        return self.Record(i)

    def __getslice__ ( self, l, h ) :
        """Umoznuje pouzivani objektu result jako seznamu.
        """
        if self._RES.status() != 'tuples_ok' :
            raise error, "%s (%s)" % (ERR_NO_TUPLES, self._RES.status())
        ll = self._RES.ntuples()
        if l < 0 :
            l = ll + l
        if h < 0 :
            h = ll + h
        l = min(ll, l)
        h = min(ll, h)
        sl = []
        for i in xrange(l,h) :
            sl.append(self[i])
        return sl
