# $Id: pgdb_table.py,v 1.44 2004/07/29 08:57:08 cvs Exp $

import string, pgdb_result, messages, sys, os
from pgdb_record import *
from pgdb_misc import *
from pgdb_error import *
from fields import *
try :
    from weakref import proxy
except ImportError :
    proxy = lambda o: o


class table :
    """Trida pro uchovani informaci o jedne databazove
       tabulce. Trida take obsahuje metody pro praci
       se zaznamy (Insert,Update,Delete,...).
    """
    primary_key = None # Urci se automaticky

    def __init__ (self, db, tab, vtabname = None, isview = 0) :
        """Konstruktor tridy se nevola primo,
           ale vola jej automaticky metoda db.FindTable.
        """
        self.db = proxy(db)
        self.tablename = tab
        self.dbtablename = tab
        self.tablename2from = tab
        self.fields = []
        self.fieldmap = {}
        self.isview = isview

        self.RECORD = record
        self.MODULE = sys.modules[self.__module__]
        if self.MODULE.__dict__.has_key( 'record_' + self.tablename ) :
            self.RECORD = self.MODULE.__dict__['record_' + self.tablename]
        elif self.db.MODULE.__dict__.has_key( 'record_' + self.tablename ) :
            self.RECORD = self.db.MODULE.__dict__['record_' + self.tablename]
            self.MODULE = self.db.MODULE
        ver = db.Version()
        q = """SELECT a.attname, a.atttypid, a.attnum, a.attlen,
                      a.atttypmod, a.attnotnull, a.atthasdef,
                      a.attndims AS attnelems
                 FROM pg_attribute a, pg_class c
                WHERE c.oid = a.attrelid AND c.relname = '%s'
                  AND a.attnum > 0 AND a.attisdropped = 'f'
             ORDER BY a.attnum;""" % tab
        r = self.db(q)
        if not r.OK() :
            raise error, messages.ERR_READ_TABLE_DEF % `tab`
        if len(r) < 1 :
            raise error, messages.ERR_READ_TABLE_DEF % `tab`
        if vtabname : tn = vtabname
        else : tn = tab
        for a in r :
            attname = a['attname']
            try:
                f = self.db.OidPythonTypeMap[a['atttypid']]
            except:
                raise TypeError('cannot find type info for', tab, attname, a['atttypid'])
            kw = { 'typeid' : f.typeid,
                   'tablename' : tn,
                   'fieldname' : attname,
                   'typeoid' : a['atttypid'],
                   'typename' : f.typename,
                   'deforder' : a['attnum'],
                   'dimension' : a['attnelems'],
                   }
            if f.typename[:1] == '_' : kw['dimension'] = 1
            if a['attnotnull'] : kw['notnull'] = 1
            if a['atthasdef'] : kw['hasdef'] = 1
            if f.typeid == WM_STRING and a['atttypmod'] > 0 :
                kw['maxlen'] = a['atttypmod'] - 4
            fqn = 'field_%s_%s' % (self.tablename, attname)
            qn = 'field_%s' % attname
            if hasattr(self.RECORD, qn) :
                nf = apply(getattr(self.RECORD, qn).clone, (), (kw))
            elif self.MODULE.__dict__.has_key(fqn) :
                nf = apply(self.MODULE.__dict__[fqn].clone, (), (kw))
            # FIXME tohle je asi zbytecne
            elif db.MODULE.__dict__.has_key(fqn) :
                nf = apply(db.MODULE.__dict__[fqn].clone, (), (kw))
            else :
                nf = apply(f.clone, (), (kw))
            self.fieldmap[a['attname']] = nf
            self.fields.append( a['attname'] )

    ### Emulace datoveho typu Dictionary.
    ### Neemuluji se vsechny metody a operatory.

    def __getitem__ (self, i) :
        """Metoda umoznujici pristup k definici jednotlivych
           polozek tabulky jako k asociativnimu poli.
        """
        if type(i) == type(0) :
            return self.fieldmap[self.fields[i]]
        return self.fieldmap[i]

    def keys (self) :
        """Vrati seznam nazvu polozek tabulky.
        """
        return self.fields

    def values (self) :
        """Vrati seznam definic polozek tabulky.
        """
        l = []
        for f in self.fields :
            l.append(self.fieldmap[f])
        return l

    def items (self) :
        """Vrati seznam dvojic nazev-definice.
        """
        l = []
        for f in self.fields :
            l.append( (f,self.fieldmap[f]) )
        return l

    def PrimaryKey (self) :
        return self.db.PrimaryKey( self.tablename )

    def UniqueKeys (self) :
        return self.db.UniqueKeys( self.tablename )


    ### Prace se zaznamy databaze. Metody
    ### __call__, Insert, Update, Delete a Select
    ### pracuji s objekty typu pgdb_record.record
    ### nebo jejimi potomky.
    ### Metody __call__, Insert, Update a Delete nastavuji
    ### pri uspesnem prubehu do vysledku atribut
    ### OID, kde je ulozeno OID zaznamu se kterym
    ### se pracovalo.

    def __call__ (self, *a, **kw) :
        """Tato metoda umoznuje pouzivat objekt table
           jako funkci. Parametrem pri volani je hodnota
           primarniho klice. Pokud ma tabulka primarni
           klic pres vice polozek, je nutno uvest prislusny
           pocet polozek. Metoda vrati kompletni pozadovany
           zaznam typu record (nebo jeho potomka).

             >>> books = db['book']
             >>> b = books( 'ISBN-2342-234234' )
             >>> b
             <pgdb_record.record at 80b3bb8>

           Pokud metoda nenalezne pozadovany zaznam s danym
           primarnim klicem, vrati None. V pripade chyby
           nastane vyjimka pgdb_error.error.

           Polozky, pres ktere probiha vyhledavani nemusi
           byt nutne primarnim klicem, pokud plati, ze tabulka
           ma pouze jeden UNIQUE index a pocet parametru
           souhlasi s poctem polozek v tomto indexu.

           Primarni index lze stanovit pro danou tabulku
           i staticky v potomku teto tridy atributem primary_key,
           ktery ma tvar seznamu databazovych polozek.
        """
        for k in kw.keys() :
            if k not in ["lock"] :
                raise ValueError("%s is not valid keyword argument" % k)
        if self.primary_key :
            pk = self.primary_key
        else :
            pk = self.PrimaryKey()
            if not pk :
                uk = self.UniqueKeys()
                if len(uk) != 1 :
                    raise error, messages.ERR_NO_PKEY % `self.tablename`
                pk = uk.values()[0]
        if len(pk) > len(a) :
            raise error, messages.ERR_INSUFICIENT_PARAMS % len(pk)
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        if self.isview :
            q = "SELECT * FROM " + only + self.dbtablename + " WHERE "
        else :
            q = "SELECT oid,* FROM " + only + self.dbtablename + " WHERE "
        ql = []
        for i in range(len(pk)) :
            ql.append(pk[i] + " = " + self[pk[i]].python2sql(a[i]))
        if kw.get("lock") :
            q = q + string.join(ql, " AND ") + " FOR UPDATE;"
        else :
            q = q + string.join(ql, " AND ") + " ;"
        r = self.db(q, self.RECORD)
        if not r.OK() :
            raise error, messages.ERR_SELECT_STMT % r.GetErr(None)
        if len(r) != 1 :
            return None
        rec = r[0]
        if self.isview :
            rec.OID = None
        else :
            rec.OID = rec['oid']
            del rec['oid']
        return rec

    def Insert (self, rec, res=None) :
        """Metoda Insert slouzi k vlozeni zaznamu do databaze.
           Tato metoda vola dalsi metody InsertCheck(),
           InsertTriggerBefore(), InsertTriggerAfter(), kterymi
           lze ovlivnit prubeh vkladani pripadne vykonat dalsi
           akce s nim spojene. Parametr rec je typu
           pgdb_record.record nebo jeho potomkem definovanym
           pro konkretni tabulku. Pokud ma rec definovany
           nejake polozky, ktere nepatri do dane tabulky,
           jsou tyto ignorovany.
           V metode InsertTriggerAfter() uz je k dispozici
           kompletni zaznam (i s defaultnimi hodnotami,
           ktere nebyly explicitne definovany).
        """
        if res is None :
            res = pgdb_result.result()
        elif not res.OK() :
            return res
        if len(rec) < 1 :
            raise error, messages.ERR_EMPTY_RECORD
        # Zahajeni transakce pokud jiz nebyla zahajena
        if not self.db.INTRANSACTION :
            tr = 0
            r = self.db.Begin()
            if not r.OK() :
                # Chyba pri zahajeni transakce
                return r
        else :
            tr = 1
        # Provedeni transakce a triggeru
        q = 0
        try :
            # Prevedeni rec na odpovidajici tridu
            if not isinstance(rec, self.RECORD) :
                oldrec = rec
                rec = self.RECORD()
                for k,v in oldrec.items() :
                    rec[k] = v
                #FIX#rec.DB = self.db
            else :
                oldrec = None
            # Kontrola polozek pred vlozenim do databaze
            res = self.InsertCheck(rec, res)
            if not res.OK() : return res
            # Vyvolani triggeru
            res = self.InsertTriggerBefore(rec, res)
            if not res.OK() : return res
            # Vyvolani interniho triggeru
            res = self.InsertInternalTriggerBefore(rec, res)
            if not res.OK() : return res
            # Priprava SQL dotazu INSERT do promenne ins_q
            qf = []
            qv = []
            fm = self.db[self.dbtablename].fieldmap
            for f, v in rec.items() :
                if not fm.has_key( f ) : continue
                qf.append(f)
                qv.append("%s" % self[f].python2sql(v))
            ins_q = "INSERT INTO %s (%s) VALUES (%s);" % (self.dbtablename,
                  string.join(qf,','), string.join(qv,','))
            # Vykonani insertu
            ins_res = self.db(ins_q)
            if not ins_res.OK() : return ins_res
            oid = ins_res.OidStatus()
            # Naplneni kompletniho zaznamu
            ver = self.db.Version()
            if ver[0]*100+ver[1] >= 701 :
                q = "SELECT * FROM ONLY %s WHERE oid = %s;"
            else :
                q = "SELECT * FROM %s WHERE oid = %s;"
            sel_r = self.db(q % (self.dbtablename, oid))
            if not sel_r.OK() or len(sel_r) != 1 :
                return sel_r
            for k,v in sel_r[0].items() :
                rec[k] = v
            # Vyvolani interniho triggeru
            res = self.InsertInternalTriggerAfter(rec, res)
            if not res.OK() : return res
            # Vyvolani triggeru
            res = self.InsertTriggerAfter(rec, res)
            if not res.OK() : return res
            if oldrec :
                for k,v in rec.items() :
                    oldrec[k] = v
            q = 1
        finally :
            if tr == 0 :
                # Dokonceni transakce pripadne jeji zruseni
                if q == 1 : r = self.db.End()
                else : r = self.db.Abort()
                if not r.OK() : return r
        # Nastaveni OID vlozeneho zaznamu
        ins_res.OID = oid
        if ins_res is not res :
            ins_res.__dict__.update(res.__dict__)
        return ins_res

    def InsertCheck (self, rec, res) :
        """Metoda pro kontrolu spravnosti vkladanych
           udaju. Je interne volana z metody Insert().
           Pokud budou nalezeny chyby, insert neprobehne.

           Nastaveni chyb probiha pres parametr res,
           ktery je typu pgdb_result.result.

           Pouziti: Jedna-li se o chybu v polozce 'pol1',
           provedeme nastaveni chybove hlasky pres
           metodu AddErr() objektu res:

              res.AddErr('pol1', 'Polozka musi byt delsi nez 4 znaky')
              res.AddErr('pol1', 'Polozka nesmi obsahovat pismena Y,Z.')

           Pokud se jedna o chybu, ktera se nevztahuje
           k zadne konkretni polozce tabulky, provedeme
           nastaveni chyby volanim:

              res.AddErr(None, 'Nemate pravo zapisovat do teto tabulky')

           Pokud se jedna o nejakou systemovou chybu, provedeme
           nastaveni volanim:

              res.AddErr(None, 'Pristup k souboru "/etc/hokej.conf" odmitnut.')

           Je zajisteno, ze parametr rec je instance tridy specialne
           definova pro tuto tabulku (record_<table>).
        """
        return res

    def InsertTriggerBefore (self, rec, res) :
        """Metoda umoznujici vykonat libovolny kod pred vlastnim
           vlozenim zaznamu do databaze metodou Insert(), ze ktere
           je tato metoda interne volana. Veskere SQL prikazy
           probihaji v transakci otevrene metodou Insert(), ktera
           je v pripade neuspesneho prubehu zrusena.

           Transakce muze byt zrusena nastavenim chyby pres
           objekt res v metodach InsertTriggerBefore(),
           InsertTriggerAfter() nebo chybou v samotne metode Insert().
           Chyby se v teto metode nastavuji stejne jako v metode
           InsertCheck().

           Je zajisteno, ze parametr rec je instance tridy specialne
           definova pro tuto tabulku (record_<table>).
        """
        return res

    def InsertTriggerAfter (self, rec, res) :
        """Metoda umoznujici vykonat libovolny kod po vlastnim
           vlozenim zaznamu do databaze metodou Insert(). Viz
           dokumentaci k InsertTriggerBefore().

           Je zajisteno, ze parametr rec je instance tridy specialne
           definova pro tuto tabulku (record_<table>).
        """
        return res

    def InsertInternalTriggerBefore (self, rec, res) :
        """Tento triger je zamyslen pro pouziti v systemovych potmocich
           teto tridy a nemel by se prepisovat v uzivatelem definovanych
           potomcich.
        """
        return res

    def InsertInternalTriggerAfter (self, rec, res) :
        """Tento triger je zamyslen pro pouziti v systemovych potmocich
           teto tridy a nemel by se prepisovat v uzivatelem definovanych
           potomcich.
        """
        return res

    def Record2Where (self, rec) :
        fm = self.db[self.dbtablename].fieldmap
        w = []
        for f,v in rec.items() :
            if not fm.has_key(f) :
                continue
            if v is None :
                w.append( "%s IS NULL" % f )
            else :
                fi = self[f]
                if fi.typeid == fields.WM_FLOAT :
                    continue
                if not fi.maxlen and fi.typeid == fields.WM_STRING and \
                (fi.typename == 'text' or string.find(fi.typename,'char') > 0) :
                    continue
                if fi.dimension :
                    continue
                qv = fi.python2sql( v )
                w.append("%s = %s" % (f, qv))
        where = string.join(w, ' AND ')
        return where

    def Update (self, old, new, res=None) :
        """Metoda slouzi k oprave zaznamu v databazi. Jeji typicke
           pouziti je nasledujici:

             >>> book = db['book']
             >>> old = book( 'ISBN-23423-23423' )
             >>> new = old.copy()
             >>> new['author'] = 'Josif Stalin'
             >>> new['price'] = 3.9
             >>> r = book.Update( old, new )
             >>> if not r.OK() : print 'Update failed'

           Metoda interne vola metody UpdateCheck(), UpdateTriggerBefore(),
           UpdateTriggerAfter(), kterymi lze ovlivnit prubeh opravy zaznamu,
           pripadne vykonat dalsi pozadovane akce. Metoda UpdateTriggerBefore
           je provedena po SQL dotazu SELECT ... FOR UPDATE a pred vlastnim
           dotazem UPDATE, takze mame jistotu, ze pozadovany zaznam v databazi
           je a nemusime zde existenci kontrolovat. Pokud v databazi neni,
           metoda Update skonci s chybou.

           Parametry old a new jsou typu pgdb_record.record nebo potomky
           definovanymi pro konkretni tabulku. Pokud old nebo new definovany
           nejake polozky, ktere nepatri do dane tabulky, jsou tyto ignorovany.

           Po skonceni metody Update je v parametru new ulozen kompletni novy
           zaznam. Tj. i ty polozky, ktere nebyly definovany.
        """
        if res is None :
            res = pgdb_result.result()
        elif not res.OK() :
            return res
        if len(old) < 1 :
            # Nelze vybrat zadny zaznam
            raise error, messages.ERR_EMPTY_RECORD
        if len(new) < 1 :
            # Neni co updatovat
            return res
        # Priprava SQL dotazu SELECT ... FOR UPDATE.
        # Tento prikaz zpusobi korektni zamceni zaznamu
        # pro UPDATE po svem provedeni.
        fm = self.db[self.dbtablename].fieldmap
        where = self.Record2Where(old)
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        sel_q =  "SELECT oid,* FROM %s%s WHERE %s FOR UPDATE;" % (
                            only, self.dbtablename, where )
        # Zahajeni transakce pokud jiz nebyla zahajena
        if not self.db.INTRANSACTION :
            tr = 0
            r = self.db.Begin()
            if not r.OK() : return r
        else : tr = 1
        # Provedeni transakce a triggeru
        q = 0
        try :
            sel_r = self.db(sel_q, self.RECORD)
            if not sel_r.OK() : return sel_r
            if len(sel_r) < 1 :
                res.AddErr('', messages.ERR_NO_OLD_RECORD)
                return res
            elif len(sel_r) > 1 :
                res.AddErr('', messages.ERR_MORE_OLD_RECORD)
                return res
            oldbak = old
            old = sel_r[0]
            for k,v in old.items() : oldbak[k] = v
            oid = old['oid']
            # Prevedeni new na odpovidajici tridu.
            # U old je to zajisteno predchozim volanim.
            if not isinstance( new, self.RECORD ) :
                newbak = new
                new = self.RECORD()
                for k,v in newbak.items() :
                    new[k] = v
                new.DB = self.db
            else :
                newbak = None
            # Doplneni chybejicich polozek do new
            for k in self.keys() :
                if not new.has_key( k ) : new[k] = old[k]
            res = self.UpdateCheck( old, new, res )
            if not res.OK() : return res
            res = self.UpdateTriggerBefore( old, new, res )
            if not res.OK() : return res
            res = self.UpdateInternalTriggerBefore( old, new, res )
            if not res.OK() : return res
            # Priprava vlastniho SQL prikazu UPDATE.
            # Toto je nutne delat tady, protoze metody
            # UpdateCheck a UpdateTriggerBefore muzou
            # zmenit hodnoty zaznamu new.
            u = []
            for f,v in new.items() :
                if not fm.has_key(f) : continue
                if old[f] == new[f] : continue
                fi = self[f]
                if v is None :
                    u.append("%s = NULL" % f)
                else :
                    u.append( "%s = %s" % (f, fi.python2sql(v)))
            # Je co updatovat?
            if u :
                upd_q = "UPDATE %s%s SET %s WHERE oid=%s;" % (
                    only, self.dbtablename, string.join(u,', '), oid)
                upd_r = self.db(upd_q)
                if upd_r.CmdTuples() != 1 :
                    if upd_r.CmdTuples() < 1 :
                        upd_r.AddErr(None, messages.ERR_NO_OLD_RECORD)
                    else :
                        upd_r.AddErr(None, messages.ERR_MORE_OLD_RECORD)
                    return upd_r
                # Nastaveni OID zaznamu se kterym se pracovalo
                upd_r.OID = oid
            else :
                upd_r = res
            res = self.UpdateInternalTriggerAfter(old, new, res)
            if not res.OK() : return res
            res = self.UpdateTriggerAfter(old, new, res)
            if not res.OK() : return res
            if newbak :
                for k,v in new.items() :
                    newbak[k] = v
            q = 1
        finally :
            if tr == 0 :
                # Dokonceni transakce pripadne jeji zruseni
                if q == 1 : r = self.db.End()
                else : r = self.db.Abort()
                if not r.OK() : return r
        if upd_r is not res :
            upd_r.__dict__.update(res.__dict__)
        return upd_r

    def UpdateCheck (self, old, new, res) :
        """Viz dokumentaci k metode InsertCheck().
           V teto metode mame k dispozici kompletni
           zaznam v parametru old (vcetne polozky oid).

           V parametru new jsou definovany vsechny polozky
           pripadneho noveho zaznamu. Pokud nebyly nektere
           polozky definovany, jsou prevzaty ze stareho zaznamu.

           Je zajisteno, ze parametry old a new jsou instancemi
           tridy specialne definova pro tuto tabulku (record_<table>).
        """
        return res

    def UpdateTriggerBefore (self, old, new, res) :
        """Viz dokumentaci k metode InsertTriggerBefore().
           V teto metode mame k dispozici kompletni
           zaznam v parametru old (vcetne polozky oid).

           V parametru new jsou definovany vsechny polozky
           pripadneho noveho zaznamu. Pokud nebyly nektere
           polozky definovany, jsou prevzaty ze stareho zaznamu.

           Je zajisteno, ze parametry old a new jsou instancemi
           tridy specialne definova pro tuto tabulku (record_<table>).
        """
        return res

    def UpdateTriggerAfter (self, old, new, res) :
        """Viz dokumentaci k metode InsertTriggerAfter().
           V teto metode mame k dispozici kompletni
           zaznam v parametru old (vcetne polozky oid).

           V parametru new jsou definovany vsechny polozky
           pripadneho noveho zaznamu. Pokud nebyly nektere
           polozky definovany, jsou prevzaty ze stareho zaznamu.

           Je zajisteno, ze parametry old a new jsou instancemi
           tridy specialne definova pro tuto tabulku (record_<table>).
        """
        return res

    def UpdateInternalTriggerBefore (self, old, new, res) :
        return res

    def UpdateInternalTriggerAfter (self, old, new, res) :
        return res

    def Delete (self, rec, res=None) :
        """Metoda slouzi k vymazani zaznamu z databaze. Jeji typicke
           pouziti je nasledujici:

             >>> book = db['book']
             >>> rec = book( 'ISBN-23423-23423' )
             ....
             >>> r = book.Delete( rec )
             >>> if not r.OK() : print 'Delete failed'

           Metoda pracuje podobne jako metoda Update, tzn. ze vola
           prvni SQL dotaz SELECT ... FOR UPDATE, ktery overi,
           ze dany zaznam skutecne v databazi je zamce jej pro vymazani.
        """
        if res is None :
            res = pgdb_result.result()
        elif not res.OK() :
            return res
        if len(rec) < 1 :
            # Nelze vybrat zadny zaznam
            raise error, messages.ERR_EMPTY_RECORD
        # Priprava SQL dotazu SELECT ... FOR UPDATE.
        # Tento prikaz zpusobi korektni zamceni zaznamu
        # pro DELETE po svem provedeni.
        fm = self.db[self.dbtablename].fieldmap
        where = self.Record2Where(rec)
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only =  ' ONLY '
        else : only =  ''
        sel_q = "SELECT oid,* FROM %s%s WHERE %s FOR UPDATE;" % (
                    only, self.dbtablename, where)
        # Priprava vlastniho SQL prikazu DELETE.
        del_q = "DELETE FROM %s%s WHERE %s;" % (
                    only, self.dbtablename, where)
        # Zahajeni transakce pokud jiz nebyla zahajena
        if not self.db.INTRANSACTION :
            tr = 0
            r = self.db.Begin()
            if not r.OK() : return r
        else : tr = 1
        # Provedeni transakce a triggeru
        q = 0
        try :
            sel_r = self.db(sel_q, self.RECORD)
            if not sel_r.OK() : return sel_r
            if len(sel_r) < 1 :
                res.AddErr('', messages.ERR_NO_OLD_RECORD)
                return res
            elif len(sel_r) > 1 :
                res.AddErr('', messages.ERR_MORE_OLD_RECORD)
                return res
            recbak = rec
            rec = sel_r[0]
            for k,v in rec.items() : recbak[k] = v
            oid = rec['oid']
            # Kontrola polozek prev vlastnim UPDATEm.
            res = self.DeleteCheck(rec, res)
            if not res.OK() : return res
            res = self.DeleteTriggerBefore(rec, res)
            if not res.OK() : return res
            res = self.DeleteInternalTriggerBefore(rec, res)
            if not res.OK() : return res
            del_r = self.db(del_q)
            if del_r.CmdTuples() != 1 :
                if del_r.CmdTuples() < 1 :
                    del_r.AddErr(None, messages.ERR_NO_OLD_RECORD)
                else :
                    del_r.AddErr(None, messages.ERR_MORE_OLD_RECORD)
                return del_r
            res = self.DeleteInternalTriggerAfter(rec, res)
            if not res.OK() : return res
            res = self.DeleteTriggerAfter(rec, res)
            if not res.OK() : return res
            q = 1
        finally :
            if tr == 0 :
                # Dokonceni transakce pripadne jeji zruseni
                if q == 1 : r = self.db.End()
                else : r = self.db.Abort()
                if not r.OK() : return r
        # Nastaveni OID zaznamu se kterym se pracovalo
        del_r.OID = oid
        if del_r is not res :
            del_r.__dict__.update(res.__dict__)
        return del_r

    def DeleteCheck (self, rec, res) :
        """Viz dokumentaci k metode InsertCheck().
        """
        return res

    def DeleteTriggerBefore (self, rec, res) :
        """Viz dokumentaci k metode InsertTriggerBefore().
        """
        return res

    def DeleteTriggerAfter (self, rec, res) :
        """Viz dokumentaci k metode InsertTriggerAfter().
        """
        return res

    def DeleteInternalTriggerBefore (self, rec, res) :
        return res

    def DeleteInternalTriggerAfter (self, rec, res) :
        return res

    def Record (self, rec) :
        """Este presne nevim, co s tim.
        """
        pass

    def Select (self, *a) :
        """Este presne nevim, co s tim.
        """
        pass

    ### Dalsi pomocne funkce souvisejici s jednou tabulkou

    def NItems (self, col, val) :
        """Metoda vrati pocet polozek col, ktere maji hodnotu val.
           Pokud nastala nejaka chyba, metoda vrati None.
        """
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        if val is None :
            r = self.db("SELECT count(%s) FROM %s%s WHERE %s is NULL;" % (
                        col, only, self.dbtablename, col))
        else :
            r = self.db( "SELECT count(%s) FROM %s%s WHERE %s = %s;" % (
                col, only, self.dbtablename, col, self[col].python2sql(val) ))
        if r.OK() : return r[0][0]
        return None

class cls_table_join ( table ) :
    """Tato trida umoznuje pracovat s vice tabulkami najednou jako s jednou
       virtualni tabulkou.

       Napriklad mejme tabulky:

         CREATE TABLE prim (id char(4) primary key, dd char(4), name char(16));
         CREATE TABLE sec1 (id char(4), poradi int, data char(16));
         CREATE TABLE sec2 (dd char(4), poradi int, data char(16));

       Potom muzeme definovat virtualni tabulku

         class prim_v ( wm3.pgdb_table.table_join ) :
             primary_table = 'prim'
             secondary_tables = {
                 'sec1' : ( 'sec1', 'id', 'id' ),
                 'sec2' : ( 'sec2', 'dd', 'dd','poradi' ),
                 }

        Vyznam jednotlivych poli ( 'sec2', 'dd', 'dd','poradi' ) je naseldujici:

           1. Nazev sekundarni tabulky
           2. Policko v primarni tabulce, podle ktereho se dela join
           3. Policko v sekundarni tabulce, podle ktereho se dela join
           4. Nepovinne policko udavajici, podle ceho tridit sekundarni
              tabulku

        Databazovy zaznam, se kterym se potom pracuje, vypada nasledovne:

           rec = { 'id', 'AAAA',
                   'dd', '0000',
                   'name', 'Jmeno',
                   'sec1' : [ { 'poradi':0, 'data':'aaaaa'},
                              { 'poradi':1, 'data':'bbbbb'},
                              { 'poradi':3, 'data':'ccccc'},
                              ]
                   'sec2' : [ { 'poradi':0, 'data':'aaaaa'},
                              { 'poradi':1, 'data':'bbbbb'},
                              { 'poradi':3, 'data':'ccccc'},
                              ]
                   }

        Takovy zaznam muzeme primo pouzit v metodach __call__, Insert,
        Update, Delete. Tedy napr. db['prim_v'].Insert( rec ) zpusobi,
        ze se zapise celkem 7 zaznamu. Jeden do tabulky prim a po trech
        do tabulky sec1 a sec2.

        Timto zpusobem mohou byt definovany pouze virtualni tabulky.
    """

    primary_table = None
    secondary_tables = {}

    def __init__ (self, db, tab, vtabname = None, isview = 0) :
        if self.primary_table == tab :
            raise error, 'Only virtual table can be defined as table_join.'
        self.db = db
        self.tablename = tab
        self.isview = isview
        self.fields = copy.copy( db[self.primary_table].fields )
        self.fieldmap = copy.copy( db[self.primary_table].fieldmap )
        for v in self.fieldmap.values() :
            v.tablename = self.primary_table
        for k,v in self.secondary_tables.items() :
            f = field(
                fieldname = k,
                tablename = self.primary_table,
                )
            self.fields.append( k )
            self.fieldmap[k] = f
        self.RECORD = record
        self.MODULE = sys.modules[self.__module__]
        if self.MODULE.__dict__.has_key( 'record_' + self.tablename ) :
            self.RECORD = self.MODULE.__dict__['record_' + self.tablename]
        elif self.db.MODULE.__dict__.has_key( 'record_' + self.tablename ) :
            self.RECORD = self.db.MODULE.__dict__['record_' + self.tablename]
        self.dbtablename = self.primary_table
        self.tablename2from = self.primary_table

    def PrimaryKey ( self ) :
        return self.db.PrimaryKey(self.primary_table)

    def __call__ ( self, *a ) :
        rec = apply(table.__call__, (self,) + a, {})
        if not rec : return rec
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        for fn,v in self.secondary_tables.items() :
            stab = v[0]
            pkey = v[1]
            pkeyval = self.db[self.primary_table][pkey].python2sql(rec[pkey])
            skey = v[2]
            q = "SELECT * FROM %s%s WHERE %s = %s" % (only, stab, skey, pkeyval)
            if len(v) > 3 :
                q = q + " ORDER BY " + string.join(v[3:], ', ')
            q = q + ';'
            rec[fn] = self.db(q, self.db[stab].RECORD)[:]
        return rec

    def InsertInternalTriggerAfter (self, rec, res) :
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        for fn,v in self.secondary_tables.items() :
            stab = v[0]
            pkey = v[1]
            pkeyval = self.db[self.primary_table][pkey].python2sql(rec[pkey])
            skey = v[2]
            q = "DELETE FROM %s%s WHERE %s = %s;" % (only, stab, skey, pkeyval)
            r = self.db(q)
            if not r.OK() :
                res.AddErr(fn, messages.ERR_JOIN_DELETE_OLD)
                return res
            if rec.has_key(fn) and rec[fn] != None :
                for v in rec[fn] :
                    v[skey] = rec[pkey]
                    r = self.db[stab].Insert(v)
                    if not r.OK() :
                        for v in r.ErrDict.values() : res.AddErr(fn, v)
                        return res
        return res

    def UpdateInternalTriggerAfter (self, old, new, res) :
        ver = self.db.Version()
        if ver[0]*100+ver[1] >= 701 : only = ' ONLY '
        else : only = ''
        for fn,v in self.secondary_tables.items() :
            stab = v[0]
            pkey = v[1]
            if not new.has_key(pkey) :
                new[pkey] = old[pkey]
            pkeyold = self.db[self.primary_table][pkey].python2sql(old[pkey])
            pkeynew = self.db[self.primary_table][pkey].python2sql(new[pkey])
            skey = v[2]
            # Pokud neni definovana nova hodnota pro svazanou
            # tabulku, tak ji menime pouze pokud se zmeni klic
            # do teto tabulky
            if not new.has_key( fn ) :
                if pkeyold != pkeynew :
                    q = "UPDATE %s%s SET %s = %s WHERE %s = %s;" % (
                        only, stab, skey, pkeynew, skey, pkeyold)
                    r = self.db(q)
                    if not r.OK() :
                        res.AddErr(fn, messages.ERR_JOIN_UPDATE_OLD)
                        return res
                continue
            # Vymazani svazane tabulky podle stareho klice
            q = "DELETE FROM %s%s WHERE %s = %s;" % (only, stab, skey, pkeyold)
            r = self.db( q )
            if not r.OK() :
                res.AddErr(fn, messages.ERR_JOIN_DELETE_OLD)
                return res
            # Vymazani svazane tabulky podle noveho klice, pokud je ruzny
            if pkeyold != pkeynew :
                q = "DELETE FROM %s%s WHERE %s = %s;" % (
                        only, stab, skey, pkeynew)
                r = self.db(q)
                if not r.OK() :
                    res.AddErr(fn, messages.ERR_JOIN_DELETE_OLD)
                    return res
            # Nahrani novych hodnot do sekundarni tabulky
            if new.has_key(fn) and new[fn] != None :
                for v in new[fn] :
                    v[skey] = new[pkey]
                    r = self.db[stab].Insert(v)
                    if not r.OK() :
                        for v in r.ErrDict.values() : res.AddErr(fn, v)
                        return res
        return res

    def DeleteInternalTriggerAfter (self, rec, res) :
        ver = self.db.Version()
        if ver[0] >= 7 and ver[1] >= 1 : only = ' ONLY '
        else : only = ''
        for fn,v in self.secondary_tables.items() :
            stab = v[0]
            pkey = v[1]
            pkeyval = self.db[self.primary_table][pkey].python2sql(rec[pkey])
            skey = v[2]
            q = "DELETE FROM %s%s WHERE %s = %s;" % (only, stab, skey, pkeyval)
            r = self.db(q)
            if not r.OK() :
                res.AddErr(fn, messages.ERR_JOIN_DELETE_OLD)
                return res
        return res

class cls_table_history (table) :
    """Tato trida umoznuje uchovavat historii zmen v tabulce. K tabulce,
       ke ktere chceme tuto historii udrzovat, musime definovat sekundarni
       tabulku (nejlepe potomka), ktera musi mit nasledujici definici
       (predpokladejme, ze mame jiz definovanou tabulku tbl) :

       CREATE TABLE tbl_history (
         mod_time    DATETIME NOT NULL,
         mod_user    VARCHAR(32) NOT NULL DEFAULT getpgusername()
       ) INHERITS ( tbl ) ;

       V techto polozkach, ktere jsou navic oproti puvodni tabulce tbl,
       je potom zapsan cas zmeny a identifikator uzivatele, ktery zmenu provedl.

       Pokud je zadan parametr version, je tento pouzit na generovani
       jednoznacneho cisla verze. Tento parametr musi byt typu SERIAL
       a musi byt definovana sekvence <table>_<verze>_seq.
    """
    postfix = '_history'
    version = None

    def UserID (self) :
        if os.environ.has_key('AUTH_PGSQL_LOGIN') :
            return os.environ['AUTH_PGSQL_LOGIN']
        elif os.environ.has_key('REMOTE_USER') :
            return os.environ['REMOTE_USER']
        elif os.environ.has_key('USER') :
            return os.environ['USER']
        elif os.environ.has_key('LOGNAME') :
            return os.environ['LOGNAME']
        else :
            return None

    def UpdateInternalTriggerBefore (self, old, new, res) :
        nold = copy.copy(old)
        nold['mod_user'] = self.UserID()
        nold['mod_time'] = date.Now()
        if self.version :
            try :
                seq = "%s_%s_seq" % (self.tablename, self.version)
                nvr = self.db("SELECT NEXTVAL('%s');" % seq)
                new[self.version] = nvr[0][0]
            except :
                res.AddErr('HISTORY',
                   "Cannot generate new version number from sequence %s." % seq)
        ht = self.db[self.tablename + self.postfix]
        if not ht :
            res.AddErr('HISTORY', "History table %s doesn't exist" %  (
                            self.tablename+self.postfix))
            return res
        r = ht.Insert(nold)
        if not r.OK() :
            res.AddErr('HISTORY', 'Failed to store old value in history table')
        return res

    def DeleteInternalTriggerAfter (self, rec, res) :
        nold = copy.copy(rec)
        nold['mod_user'] = self.UserID()
        nold['mod_time'] = date.Now()
        r = self.db[self.tablename + self.postfix].Insert(nold)
        if not r.OK() :
            res.AddErr('HISTORY', 'Failed to store old value in history table')
        return res

    def __call__ (self, *a) :
        if self.primary_key :
            pk = self.primary_key
        else :
            pk = self.PrimaryKey()
            if not pk :
                uk = self.UniqueKeys()
                if len(uk) != 1 :
                    raise error, messages.ERR_NO_PKEY % `self.tablename`
                pk = uk.values()[0]
        ver = self.db.Version()
        if ver[0] >= 7 and ver[1] >= 1 : only = 'ONLY '
        else : only = ''
        if len(pk) == len(a) or len(pk) == len(a) -1 :
            if len(pk) == len(a) - 1 :
                q = "SELECT oid,* FROM " + self.dbtablename + "* WHERE "
            else :
                q = "SELECT oid,* FROM " + only + self.dbtablename + " WHERE "
            ql = []
            for i in range(len(pk)) :
                ql.append(pk[i] + " = " + self[pk[i]].python2sql(a[i]))
            if len(pk) == len(a) - 1 :
                ql.append( self.version + " = " + \
                    self[self.version].python2sql(a[-1]) )
            q = q + string.join(ql, " AND ") + " ;"
            r = self.db(q, self.RECORD)
            if not r.OK() :
                raise error, messages.ERR_SELECT_STMT % r.GetErr(None)
            if len(r) != 1 :
                return None
            rec = r[0]
            rec.OID = rec['oid']
            del rec['oid']
            return rec
        else :
            raise error, messages.ERR_INSUFICIENT_PARAMS % len(pk)

class cls_table_alias (table) :
    real_table = None

    def __init__ (self, db, tab, vtabname = None, isview = 0) :
        table.__init__ (self, db, self.real_table, tab)
        self.tablename = tab
        self.tablename2from = self.real_table + ' AS ' + tab
        self.isview = isview

    def PrimaryKey (self) :
        return self.db.PrimaryKey(self.dbtablename)
