import string

def view_def (db, view) :
    q = "SELECT definition FROM pg_views WHERE viewname = '%s';" % view
    r = db(q)
    if len(r) != 1 :
        raise 'error', 'View %s not found in pg_views' % `view`
    return 'v', r[0]['definition']


def table_attributes (db, table) :
    q = "SELECT oid,* FROM pg_class WHERE relname='%s' AND relkind = 'r';" %\
        table
    r = db(q)
    if len(r) != 1 :
        return view_def (db, table)
    tdef = r[0]
    qx = """SELECT a.oid, a.attname, t.typname, a.attlen, a.atttypmod,
                  a.attnotnull, a.atthasdef, a.attnum
           FROM   pg_attribute a, pg_type t
           WHERE  a.attnum > 0 AND a.attrelid = %d AND a.atttypid = t.oid
           ORDER BY a.attnum""" % tdef['oid']
    q = """SELECT a.attname, t.typname, a.attlen, a.atttypmod,
                  a.attnotnull, a.atthasdef, a.attnum
           FROM   pg_attribute a, pg_type t
           WHERE  a.attnum > 0 AND a.attrelid = %d AND a.atttypid = t.oid
           ORDER BY a.attname""" % tdef['oid']
    r = db(q)
    if len(r) != tdef['relnatts'] :
        raise 'error',\
              'Inconsistence in number of attributes of table %s.' % `table`
    adef = {}
    anmax = 0
    for ad in r :
        an = ad['attname']
        if len(an) > anmax : anmax = len(an)
        adef[an] = ad
    for an, ad in adef.items() :
        tn = string.upper(ad['typname'])
        if tn[:1] == '_' :
            tn = tn[1:]
            pole = 1
        else :
            pole = 0
        if tn == 'BPCHAR' : tn = 'CHAR'
        s = ('%%-%ds' % (anmax+2)) % ('"'+an+'"') + ' ' + tn
        if ad['attlen'] == -1 and ad['atttypmod'] != -1 :
            s = s + '(%d)' % (ad['atttypmod']-4)
        if pole : s = s + '[]'
        if ad['attnotnull'] : s = s + ' NOT NULL'
        if ad['atthasdef'] :
            qd = "SELECT * FROM pg_attrdef WHERE adrelid=%d AND adnum=%d;" % (
                    tdef['oid'],ad['attnum'])
            r = db(qd)
            if len(r) != 1 :
                raise 'error',\
                      'Cannot find DEFAULT definition for attribute %s.' % `an`
            s = s + ' DEFAULT ' + r[0]['adsrc']
            ad['attrdef'] = r[0]
        ad['dstring'] = s
    return 'r', adef

def table_indexes ( db, table, ta = None ) :
    q = "SELECT oid,* FROM pg_class WHERE relname='%s' AND relkind = 'r';" % \
        table
    r = db(q)
    if len(r) != 1 :
        raise 'error', 'Table %s not found.' % `table`
    tdef = r[0]
    if not ta : ta = table_attributes( db, table )
    sta = {}
    for a in ta.values() : sta[a['attnum']] = a
    opclass = {}
    for r in db( "SELECT oid,* FROM pg_opclass;" ) :
        opclass[r['oid']] = r
    relam = {}
    for r in db( "SELECT oid,* FROM pg_am;" ) :
        relam[r['oid']] = r
    q = "SELECT * FROM pg_index i WHERE i.indrelid = %d;" % tdef['oid']
    idict = {}
    for ri in db(q) :
        q = "SELECT * FROM pg_class WHERE pg_class.oid = %d;" % \
            ri['indexrelid']
        idef = db(q)[0]
        ri['relam'] = string.upper(relam[idef['relam']]['amname'])
        ri['relname'] = idef['relname']
        ia = map( string.atoi, string.split(ri['indkey']) )
        ic = map( string.atoi,  string.split(ri['indclass']) )
        s = []
        k = []
        for i in range(len(ia)) :
            a = sta[ia[i]]['attname']
            k.append(a)
            s.append( '"%s" "%s"' % (a, opclass[ic[i]]['opcname']))
        ri['attnames'] = k
        if ri['indisunique'] : u = 'UNIQUE '
        else : u = ''
        # FIXME Funkcni index ri['indproc'] != 0
        s = 'CREATE %sINDEX "%s" ON "%s" USING %s (%s)' % (
            u, ri['relname'], table, ri['relam'], string.join( s, ', '))
        ri['dstring'] = s
        idict[ri['relname']] = ri
    return idict

def table_create_string (db, table) :
    kind, ta = table_attributes(db, table)
    if kind != 'r' :
        return ta
    ti = table_indexes(db, table, ta)
    pkey = None
    for i in ti.values() :
        if i['indisprimary'] : pkey = (i['relname'], i['attnames'])
    sd = {}
    for a in ta.values() : sd[a['attnum']] = a
    k = sd.keys()
    k.sort()
    s = 'CREATE TABLE "%s" (\n' % table
    for i in k :
        a = sd[i]
        s = s + '  %s,\n' % a['dstring']
    if pkey :
        s = s + '  PRIMARY KEY (%s)\n);\n' % \
                     string.join(map(lambda x: '"%s"' % x,pkey[1]),', ')
    else :
        s = s[:-2] + '\n);\n'
    k = ti.keys()
    k.sort()
    for x in k :
        i = ti[x]
        if pkey and pkey[0] == i['relname'] : continue
        s = s + i['dstring'] + ';\n'
    # FIXME CONSTRAINTs
    return s

if __name__ == '__main__' :
    import pgdb

    #db = pgdb.db(db = 'radekk')
    db = pgdb.db(db = 'is3')
    #print table_attributes(db, 'faktura_vydana')
    print table_create_string(db, 'faktura_vydana_p')
    print table_create_string(db, 'dns_external_name_ex')
