import sys, md5, date, math, time, struct, base64, pickle, socket, string

_sid_size = struct.calcsize('IH4B')

# Creates new token from IP and timestamp
def tokenCreate (req, secret, ip) :
    counter, tstamp = math.modf(time.time())
    tstamp = int(tstamp)
    counter = int(counter * 65535)
    x = ['IH4B'] + [tstamp, counter] + map(int, ip.split('.'))
    sidbin = apply(struct.pack, tuple(x), {})
    sid = base64.encodestring(sidbin).strip()
    try : agent = string.strip(req.env['HTTP_USER_AGENT'])
    except : agent = 'unknown'
    digstr = sid + ip + secret + str(tstamp) + str(counter) + agent
    m = md5.new()
    m.update(digstr)
    digbin = m.digest()
    tok = string.strip(base64.encodestring(sidbin+digbin))
    tok = tok.replace('=', '')
    tok = tok.replace('/', '@')
    tok = tok.replace('+', '-')
    return (sid, ip, tstamp, counter, tok)



# Validate returned token and returns parsed values
def tokenValidate (req, token, secret) :
    try :
        tok = string.replace(token, '@', '/')
        tok = string.replace(tok, '-', '+')
        x = base64.decodestring(tok+'====\n')
        sidbin = x[:_sid_size]
        par = struct.unpack('IH4B', sidbin)
        tstamp = int(par[0])
        counter = par[1]
        ip = string.join(map(str,par[2:]), '.')
        sid = string.strip(base64.encodestring(sidbin))
        digbin = x[_sid_size:]
        try : agent = string.strip(req.env['HTTP_USER_AGENT'])
        except : agent = 'unknown'
        digstr = sid + ip + secret + str(tstamp) + str(counter) + agent
        m = md5.new()
        m.update(digstr)
        digbin2 = m.digest()
        if digbin != digbin2 : raise ValueError
        return (sid, ip, tstamp, counter)
    except :
        return (None, None, None, None)


class Session :

    def __init__ (self, token_transport='cookie', table = 'wm3session',
                        timeout = 0, secret='x4t5z9', check_ip=1) :
        """Konstruktor:

           token_transport  Zpusob predavani session tokenu mezi
                            klientem a serverem. Povolene hodnoty
                            jsou:

                              cookies  token se predava v cookie 'se'

                              tilde    token se predava jako prefix
                                       kazdeho URL zacinajici tildou
                                       /~<token>/aaaa/bbb.py

                              kw       token se predava jako tzv. kw
                                       /aaaa/bbb.py;se$token.

           table            Nazev tabulky pro ukladani session zaznamu.
                            Tabulka musi obsahovat minimalne ty sloupce,
                            ktere obsahuje defaultni wm3session. Viz metodu
                            create_table.
           timeout          Jak dlouho je session platna od posledniho
                            pristupu. Po teto dobe je session smazana
                            a klient je pripadne vyzvan k opetne autentizaci.
           secret           Sdilene tajemstvi pro generovani tokenu.
           check_ip         Kontroluj striktne IP adresu. Nemusi fungovat
                            pokud je klient za systemem proxy serveru
                            jako ma AOL nebo Cesnet.
        """
        self.table = table
        self.timeout = timeout
        self.secret = secret
        self.token_transport = token_transport
        self.check_ip = check_ip

    def __del__ (self) :
        """Pri ruseni objektu Session je treba korektne zavrit
           databazi, aby ji mohly cist ostatni procesy.
        """
        pass

    def get_id (self, req) :
        """Vygeneruje nove session ID pro daneho klienta.

           FIXME: Neni jasne, nakolik je bezpecne pouzit
           identifikator vygenerovany modulem mod_unique_id.
        """
        tok = None
        sid = None
        s = ''
        tt = self.token_transport
        remote_ip = socket.gethostbyname(req.env['REMOTE_ADDR'])
        if tt == 'kw' and req.kw.has_key('se') :
            tok = req.kw['se']
            s = s + 'kw(%s):' % tok
        elif tt == 'cookie' and req.cookies.has_key('se') :
            tok = req.cookies['se'][0]
            s = s + 'cookie(%s):' % tok
        elif tt == 'tilde' and req.tilde :
            tok = req.tilde
            s = s + 'tilde(%s):' % tok
        else :
            s = s + 'transport(%s):' % tt
        if not tok is None :
            sid, ip, tstamp, counter = tokenValidate(req, tok, self.secret)
            if sid :
                s = s + 'valid:'
            else :
                s = s + 'invalid:'
            if self.check_ip and ip != remote_ip :
                s = s + 'check_ip_failure(%s x %s):' % (ip,remote_ip)
                sid = None
        else :
            s = s + 'notok:'
        if sid is None :
            sid, ip, tstamp, counter, tok = tokenCreate(
                req, self.secret, remote_ip)
            s = s + 'new(%s,%s)' % (sid, tok)
        if tt == 'cookie' :
            s = s + 'cookie_set:'
            req.cookie('se', tok)
        req.session_status = s
        req.session_remote_ip = remote_ip
        return (tok, sid)

    def create_table (self, db) :
        """Vytvori tabulku pro ukladani session
           zaznamu. Pokud nastane chyba pri
           vytvareni tabulky, nastane vyjimka.
        """
        q = """CREATE TABLE %s (
            session_id          VARCHAR(64) NOT NULL PRIMARY KEY,
            object_id           VARCHAR(32) NOT NULL,
            created             TIMESTAMP WITH TIME ZONE NOT NULL,
            remote_ip           CIDR NOT NULL,
            username            VARCHAR(32),
            challenge           VARCHAR(32),
            data                TEXT
            );""" % self.table
        res = db(q)
        if not res.OK() :
            raise 'error', "Cannot create session table %s. %s." % (
                self.table, res.Err2TXT())
        return db[self.table]

    def save (self, req, di) :
        db = req.glob.db
        table = db[self.table]
        if not table :
            table = self.create_table(db)
        try : user = req.data.user.login
        except : user = None
        nr = {
            'session_id' : req.session_id,
            'object_id' :  req.object_id,
            'created' :    date.Now(),
            'remote_ip' :  socket.gethostbyname(req.env['REMOTE_ADDR']),
            'username' :   user,
            'data' :       pickle.dumps(di),
            }
        if table(req.session_id) :
            table.Update({'session_id':req.session_id}, nr)
        else :
            table.Insert(nr)

    def load (self, req) :
        db = req.glob.db
        table = db[self.table]
        if not table :
            self.create_table(db)
            table = db[self.table]
        rec = table(req.session_id)
        if not rec : return {}
        d = pickle.loads(rec['data'])
        for f in table.keys() :
            if f == 'data' : continue
            d['_'+f] = rec[f]
        return d
