import string, wm3.pgdb_result, wm3.pgdb_record, messages, sys, form
from wm3.fields import *
from wm3.utils import *

class SqlForm ( form.Form ) :

    def real_init ( self, req ) :
        if self.initialized : return
        if not self.fields :
            raise error, 'No fields specified'
        db = getattr( req.glob, self.database )
        tbl = {}
        for i in range(len(self.fields)) :
            f = self.fields[i]
            if type(f) == type('') and f[:1] != '@' :
                x = string.split(f,'.')
                if len(x) == 1 :
                    f = db[self.table][f]
                elif len(x) == 2 :
                    f = db[f]
                    tbl[x[0]] = 1
                else :
                    raise 'Error', 'Bad field name %s.' % `f`
                self.fields[i] = f
        if len(tbl) > 1 :
            raise 'Error', 'Multiple tables in form initializer.'
        if not self.table : self.table = tbl.keys()[0]
        if not self.form_name : self.form_name = req.object_id
        if self.multipart : self.method = 'post'
        req.data.form_object = self
        self.initialized = 1



class SqlInsert :

    permission = messages.SQLFORM_INSERT_PERMISSION
    oldnotfound = messages.SQLFORM_OLD_NOT_FOUND
    authorization = None
    location_url = None
    defaults = {}

    def __init__ ( self, form, **kw ) :
        for k,v in kw.items() : setattr( self, k, v )
        self.form = form

    def __call__ ( self, req ) :
        db = getattr( req.glob, self.form.database )
        rec, res = self.form.get_rec( req )
        req.data.sql_rec = rec
        req.data.old_rec = None
        try :
            if self.perm( req ) != 1 :
                raise "PERMISSION"
            if self.form.if_submit( req, 'insert' ) :
                if self.perm( req, rec ) != 1 :
                    raise "PERMISSION"
                res = db[self.form.table].Insert( rec, res )
                if res.OK() :
                    return self.ok( req, rec )
            else :
                for k,v in self.defaults.items() :
                    if not rec.has_key(k) or rec[k] == None : rec[k] = v
            s = self.form.form( req, rec, res, 'insert' )
            return 0, s
        except "PERMISSION" :
            return -1, self.permission

    def ok ( self, req, rec ) :
        if self.location_url :
            q = 1
            di = {}
            while q :
                try :
                    self.location_url % di
                    q = 0
                except KeyError, k :
                    di[k.args[0]] = urlquote(str(rec[k.args[0]]))
            req.header( 'Location', self.location_url % di )
            req.write( 'OK\n' )
            return 1, rec
        else : return 2, rec

    def perm ( self, req, rec = None ) :
        if not self.authorization :
            return 1
        return self.authorization.authorize( req.data.user )


class SqlUpdate ( SqlInsert ) :

    permission = messages.SQLFORM_UPDATE_PERMISSION
    oldnotfound = messages.SQLFORM_OLD_NOT_FOUND
    authorization = None
    location_url = None
    new_values = {}

    def __call__ ( self, req ) :
        db = getattr( req.glob, self.form.database )
        rec, res = self.form.get_rec( req )
        req.data.sql_rec = rec
        req.data.old_rec = None
        try :
            if self.perm( req ) != 1 :
                raise "PERMISSION"
            if self.form.if_submit( req, 'update' ) :
                try : oldrec = req.session.load( req )['oldrec']
                except : raise "OLDNOTFOUND"
                req.data.old_rec = oldrec
                if self.perm( req, oldrec, rec ) != 1 :
                    raise "PERMISSION"
                res = db[self.form.table].Update( oldrec, rec, res )
                if res.OK() :
                    return self.ok( req, rec )
                s = self.form.form( req, rec, res, 'update' )
                return 0, s
            else :
                pk = db[self.form.table].PrimaryKey()
                ar = []
                for p in pk :
                    if not rec.has_key( p ) : raise "OLDNOTFOUND"
                    ar.append( rec[p] )
                oldrec = apply( db[self.form.table], tuple(ar), {} )
                if not oldrec : raise "OLDNOTFOUND"
                req.data.old_rec = oldrec
                #FIX# se = req.session.save( req, { 'oldrec' : oldrec.DICT } )
                se = req.session.save( req, { 'oldrec' : oldrec } )
                for k,v in self.new_values.items() :
                    oldrec[k] = v
                s = self.form.form( req, oldrec, res, 'update' )
                return 0, s
        except "PERMISSION" :
            return -1, self.permission
        except "OLDNOTFOUND" :
            return -1, self.oldnotfound

    def perm ( self, req, old = None, new = None ) :
        if not self.authorization :
            return 1
        return self.authorization.authorize( req.data.user )

class SqlDelete :

    m_permission = messages.SQLFORM_DELETE_PERMISSION
    m_notfound = messages.SQLFORM_DELETE_NOTFOUND
    m_yes = messages.SQLFORM_DELETE_YES
    m_cancel = messages.SQLFORM_DELETE_CANCEL
    m_ok = messages.SQLFORM_DELETE_OK
    database = 'db'
    ok_url = None
    cancel_url = None

    color_column1 = "#EFEFEF"
    color_column2 = "#FFFFFF"
    color_background = "#CCCCCC"
    color_confirm = "#FFEEEE"

    authorization = None

    def __init__ ( self, **kw ) :
        for k,v in kw.items() :
            setattr( self, k, v )
        if not kw.has_key( 'table' ) :
            raise 'SqlDelete', 'Parameter table must be defined.'
        if not kw.has_key( 'view' ) :
            raise 'SqlDelete', 'Parameter view must be defined.'

    def __call__ ( self, req ) :
        db = getattr( req.glob, self.database )
        tab = self.table
        if hasattr( self, 'key' ) :
            pk = self.key
        else :
            pk = db[tab].PrimaryKey()
            if not pk :
                raise 'SqlDelete', 'Parameter key must be defined.'
        try :
            if self.perm( req ) != 1 :
                raise "PERMISSION"
            x = []
            for k in pk :
                if req.param.has_key( k ) :
                    x.append( req.param[k].value )
                elif req.param.has_key( tab + '.' + k ) :
                    x.append( req.param[tab + '.' + k].value )
                else :
                    raise "NOTFOUND", k
            rec = apply( db[tab], tuple(x), {} )
            if not rec :
                raise "NOTFOUND", 'RECORD'
            if self.perm( req, rec ) != 1 :
                raise "PERMISSION"
            if req.param.has_key( 'CONFIRM' ) :
                res = db[tab].Delete( rec )
                if res.OK() :
                    return self.ok( req, rec )
                else :
                    s = res.Err2HTML()
                    if self.cancel_url :
                        q = 1
                        di = {}
                        while q :
                            try :
                                self.cancel_url % di
                                q = 0
                            except KeyError, k :
                                di[k.args[0]] = urlquote(str(rec[k.args[0]]))
                        cu = self.cancel_url % di
                        s = s + '<p><blockquote>'
                        s = s + '<b><a href="%s">[%s]</a></b>' % \
                            (cu, self.m_cancel)
                        s = s + '</blockquote>\n'
                    return -1, s
            else :
                s = '<p><br><center>\n'
                s = s + '<table border="0" cellpadding="2" cellspacing="0">\n'
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
                s = s + '<table border="0" cellpadding="4" cellspacing="2">\n'
                for v in self.view :
                    f = db[tab][v]
                    s = s + '<tr><td bgcolor="%s"><b>\n' % self.color_column1
                    s = s + f.Title()
                    s = s + '</b></td><td bgcolor="%s">\n' % self.color_column2
                    s = s + f.python2html( rec[v], '&nbsp;' )
                    s = s + '</td></tr>\n'
                s = s + '<tr><td colspan="2" bgcolor="%s" align="center">\n' % self.color_confirm
                s = s + '<br><b>'
                param = { 'CONFIRM' : 1 }
                for k in pk :
                    param[k] = str(rec[k])
                yu = Url( req.url, kw = req.kw, param = param )
                s = s + '<a href="%s">[%s]</a>' % ( yu, self.m_yes )
                if self.cancel_url :
                    cu = self.cancel_url % rec
                    s = s + '&nbsp;&nbsp;&nbsp;<a href="%s">[%s]</a>' % \
                        (cu, self.m_cancel)
                s = s + '</b><br>&nbsp;\n'
                s = s + '</td></tr>\n'
                s = s + '</table>\n'
                s = s + '</td></tr>\n'
                s = s + '</table>\n'
                s = s + '</center>\n'
                return 0, s
        except "PERMISSION" :
            return -1, self.m_permission
        except "NOTFOUND" :
            return -1, self.m_notfound

    def ok ( self, req, rec ) :
        if self.ok_url :
            q = 1
            di = {}
            while q :
                try :
                    self.ok_url % di
                    q = 0
                except KeyError, k :
                    di[k.args[0]] = urlquote(str(rec[k.args[0]]))
            req.header( 'Location', self.ok_url % di )
            req.write( 'OK\n' )
            return 1, None
        else : return 2, rec

    def perm ( self, req, rec = None ) :
        if not self.authorization :
            return 1
        return self.authorization.authorize( req.data.user )
