import string, copy, re, pgdb_record, messages, html
from utils import Url
from fields import *

class SqlList :
    database = 'db'
    table = None
    fields = None
    title = 'SqlList'
    list_fields = None
    view_fields = None
    search_fields = None
    fields_exceptions = {}
    add_tables = []
    max = 20
    where = None
    order = None
    distinct = None
    inherited = None
    result_record = pgdb_record.record
    clickable = 1

    # Internal/external objects
    navigation_object = 1
    navigation_inside = 1
    list_object = 1
    view_object = 1
    not_found_object = 1
    not_found_message = messages.SQLLIST_NO_RECORD_FOUND
    view_if_found_one = 1

    numbering = 1
    search_conditions = 10

    # Navigation labels
    label_sum_list = messages.SQLLIST_SUM_LIST
    label_sum_view = messages.SQLLIST_SUM_VIEW
    label_search = messages.SQLLIST_SEARCH
    label_start = messages.SQLLIST_START
    label_prev = messages.SQLLIST_PREV
    label_next = messages.SQLLIST_NEXT
    label_end = messages.SQLLIST_END
    label_list = messages.SQLLIST_LIST
    label_all = messages.SQLLIST_ALL
    label_ps_select = messages.SQLLIST_PS_SELECT
    label_ps_all = messages.SQLLIST_PS_ALL

    # Colors
    color_row1 = '#EEEEEE'
    color_row2 = '#FFFFFF'
    color_background = '#CCCCCC'
    color_heading = '#B0B0B0'
    color_current_page = '#CC0033'
    color_inactive_link = '#666666'
    color_column1 = "#EEEEEE"
    color_column2 = "#FFFFFF"

    # Persistent selection
    ps_menu = []
    ps_default = None
    ps_off = []

    # Other otuput formats (default html)
    #   'cvs' : cvs_generation_method
    #
    # def cvs_generation_method ( self, req ) :
    #     Method has sufficient informations for running Query
    #     req.data.list_rec['where']
    #     req.data.list_rec['count']
    #     self.distinct
    other_formats = []

    def __init__ (self, **kw) :
        self.kw = kw
        self.initialized = None

    def real_init (self, req) :
        if self.initialized : return
        kw = self.kw
        db = getattr( req.glob, self.database )
        for k,v in kw.items() :
            if k[:6] == 'field_' :
                self.fields_exceptions[k] = v
            elif k == 'fields_exceptions' :
                for x,y in v.items() :
                    self.fields_exceptions[x] = y
            else :
                setattr( self, k, v )
        self.tables = {}
        for lfn in ['fields', 'list_fields', 'view_fields', 'search_fields'] :
            lf = getattr( self, lfn )
            if type(lf) == type([]) :
                ln = []
                for f in lf :
                    if isinstance(f, field) :
                        ln.append(f)
                        continue
                    x = string.split(f, '.')
                    if len(x) == 1 :
                        self.tables[self.table] = 1
                        ln.append(db[self.table + '.' + f])
                    elif len(x) == 2 :
                        self.tables[x[0]] = 1
                        if x[1] == '*' :
                            for y in db[x[0]].keys() :
                                ln.append(db[x[0] + '.' + y])
                        else :
                            ln.append(db[f])
                    else :
                        raise 'Error', 'Bad field name %s.' % `f.name`
                setattr(self, lfn, ln)
        self.tables = self.tables.keys() + self.add_tables
        if len(self.tables) == 1 and not kw.has_key('result_record') :
            self.result_record = db[self.tables[0]].RECORD
        if not self.fields : self.fields = self.list_fields
        if not self.list_fields : self.list_fields = self.fields
        if not self.view_fields : self.view_fields = self.fields
        if self.search_fields is None : self.search_fields = self.fields
        self.initialized = 1

    def Where (self, req) :
        kw = {}
        sf = self.search_fields
        p = req.param
        wh = []
        db = getattr(req.glob, self.database)
        # Regex substring
        w = []
        if p.has_key('SUBS') :
            v = p['SUBS'].value.lower()
            if v :
                kw['SUBS'] = p['SUBS'].value
                for fi in self.search_fields :
                    if fi.dimension : continue
                    if not fi.typeid in [WM_STRING, WM_INT] : continue
                    f = fi.fullname()
                    if not f : continue
                    if fi.typename in ('inet', 'cidr') :
                        f = '%s::text' % f
                    va = fi.str2python(v, 0)
                    if not va : continue
                    va = fi.python2sql(va)
                    w.append("lower(%s) ~ %s" % (f, va))
        if w :
            wh.append('(' + string.join( w, ' OR ' ) + ')')
        # Find by field-value
        w = []
        for fi in self.search_fields :
            va = fi.param2python(req, 0)[fi.fieldname]
            if va != None :
                f = fi.fullname()
                kw[f] = va
                va = fi.python2sql(va)
                w.append("%s = %s" % (f, va ))
        if w :
            wh.append('(' + string.join( w, ' AND ' ) + ')')
        # Find by column-operator-value
        w = []
        for i in range(self.search_conditions) :
            if p.has_key('COL%d' % i) :
                co = p['COL%d' % i].value
                fq = None
                for fi in self.search_fields :
                    if isinstance(fi,field_virtual) : continue
                    if co == fi.fullname() or co == fi.fieldname :
                        fq = fi
                        break
                if not fq : continue
                fi = fq
                f = fi.fullname()
                if p.has_key( 'VAL%d' % i ) :
                    pv = p['VAL%d' % i].value
                    va = fi.str2python(pv, 0)
                else :
                    va = None
                if p.has_key('OP%d' % i) :
                    op = p['OP%d' % i].value
                    if not op in ['=', '<', '>', '<=', '>=', '~*', '~',
                        '!~*', '!~', 'LIKE', 'ISNULL', 'NOTNULL', '!=', '<>'] :
                        op = None
                else :
                    op = '='
                if not op : continue
                if op in [ 'ISNULL', 'NOTNULL' ] :
                    kw['COL%d' % i] = co
                    kw['OP%d' % i] = op
                    w.append( "%s %s" % (co,op))
                elif not va :
                    continue
                else :
                    v = fi.python2sql(va)
                    kw['COL%d' % i] = co
                    kw['OP%d' % i] = op
                    kw['VAL%d' % i] = pv
                    w.append( "%s %s %s" % (co,op,v))
        if w :
            if p.has_key('EXSEARCH') and p['EXSEARCH'].value == 'OR' :
                kw['EXSEARCH'] = 'OR'
                sw = '(' + string.join( w, ' OR ' ) + ')'
            else :
                sw = '(' + string.join( w, ' AND ' ) + ')'
            wh.append(sw)
        # Predefined where
        if self.where :
            wh.append(self.where)
        # Persistent selection where
        pskw = { '_ps_00' : '' }
        if self.ps_menu and req.data.list_rec['pswhere'] :
            if not req.data.list_rec['psoff'] :
                wh.append( req.data.list_rec['pswhere'] )
            for i in range( len(self.ps_menu) ) :
                rec = req.data.list_rec['psrec']
                if rec[i] : pskw['_ps_%02d' % i] = rec[i]
        if wh : wh = string.join(wh, ' AND ')
        else : wh = ''
        req.data.list_rec['where'] = wh
        req.data.list_rec['params'] = kw
        req.data.list_rec['psparams'] = pskw
        # End
        if wh : return ' WHERE ' + wh
        else : return ''

    def Order (self, req) :
        ol = []
        req.data.list_rec['col_order'] = None
        req.data.list_rec['order'] = None
        if req.param.has_key('ORDER') :
            o = req.param['ORDER'].value
            if re.match( '^[a-zA-Z0-9_.]+$', o ) :
                ol = [o]
                req.data.list_rec['col_order'] = o
        if self.order :
            if type(self.order) == type('') :
                ol.append( self.order )
            else :
                ol = ol + self.order
        if len(ol) > 0 :
            qol = []
            for o in ol :
                if o[:1] in string.uppercase :
                    qol.append(o + ' DESC')
                else :
                    qol.append(o + ' ASC')
            qo = ' ORDER BY ' + string.join(qol, ', ')
            req.data.list_rec['order'] = qo
            return qo
        return ''

    def Query (self, req, doq = 1) :
        db = getattr(req.glob, self.database)
        ver = db.Version()
        if ver[0] >= 7 and ver[1] >= 1 : v71 = 1
        else : v71 = None
        tablist = []
        for t in self.tables :
            tablist.append(db[t].tablename2from)
        if self.inherited :
            tl = map(lambda x: x + '*', tablist)
        else :
            if v71 : tl = map(lambda x: "ONLY " + x, tablist)
            else : tl = tablist
        tab = string.join( tl, ', ' )
        if self.distinct : dist = "DISTINCT "
        else : dist = ''
        qc = "SELECT COUNT(%s%s) FROM %s" % (
                dist, self.fields[0].fullname(), tab)
        qw = self.Where(req)
        qo = self.Order(req)
        res_count = db(qc + qw + ';')
        if res_count.OK():
            count = db(qc + qw + ';')[0][0]
        else:
            count = 0
        req.data.list_rec['query_count'] = qc + qw + ';'
        if req.data.list_rec['mode'] == 'view' or \
            (count == 1 and req.data.list_rec['params'] and \
            not req.data.list_rec['mode']) :
            # View
            req.data.list_rec['mode'] = 'view'
            vf = []
            for fi in self.view_fields :
                if not fi.fullname() : continue
                vf.append(fi.fullname())
            q = "SELECT " + dist + string.join(vf, ', ') + " FROM " + tab
            ql = ' LIMIT 1 OFFSET %d' % req.data.list_rec['start']
        else :
            # List
            req.data.list_rec['mode'] = 'list'
            lf = []
            for fi in self.fields :
                if not fi.fullname() : continue
                lf.append(fi.fullname())
            q = "SELECT " + dist + string.join(lf, ', ') + " FROM " + tab
            ql = ' LIMIT %d OFFSET %d' % (self.max,req.data.list_rec['start'])
        req.data.list_rec['query'] = query = q + qw + qo + ql + ';'
        if doq :
            resl = db(query, self.result_record)
            li = []
            if resl.OK():
                for r in resl : li.append(r)
            req.data.list_rec['list'] = li
        req.data.list_rec['count'] = count
        return query

    def NavigationUrl (self, req) :
        u = {}
        p = copy.copy(req.data.list_rec['params'])
        psp = {}
        if self.ps_menu :
            psp = copy.copy(req.data.list_rec['psparams'])
            for k,v in psp.items() :
                p[k] = v
        u['all'] = Url(req.url, kw = req.kw, param = psp)
        if req.data.list_rec['col_order'] :
            p['ORDER'] = req.data.list_rec['col_order']
        u['action'] = Url(req.url, kw = req.kw)
        st = req.data.list_rec['start']
        co = req.data.list_rec['count']
        if req.data.list_rec['mode'] == 'list' :
            prev = st - self.max
            if prev < 0 : prev = 0
            end = co - ((co-1) % self.max) - 1
            next = st + self.max
            if next >= co : next = end
            ap = (st/self.max) + 1       # Cislo aktualni stranky od 1
            astart = (ap-1) * self.max       # Zacatek aktualni stranky
            np = ((co-1) / self.max) + 1 # Celkovy pocet stranek
            pl = [(ap, astart, 0)]             # Seznam (stranka, start)
            u['current'] = (ap, astart)
            if np > 1 :
                pi = 1
                if ap >= 5 :
                    for i in range(ap, np) :
                        pi = pi + 1
                        pl.append((i+1, i*self.max, 1))
                        if pi >= 5 : break
                    for i in range(ap-2, ap-14, -1) :
                        pi = pi + 1
                        pl.insert(0, (i+1, i*self.max, 1))
                        if i < 1 : break
                        if pi >= 10 : break
                else :
                    for i in range(ap-2, -1, -1) :
                        pi = pi + 1
                        pl.insert(0, (i+1, i*self.max, 1))
                    for i in range( ap, np ) :
                        pi = pi + 1
                        pl.append((i+1, i*self.max, 1))
                        if pi >= 10 : break
            #p['START'] = '%d'
            u['pages'] = pl
            u['page_url'] = Url(req.url, kw = req.kw, param = p)
        else :
            p['START'] = st
            u['list'] = Url(req.url, kw = req.kw, param = p)
            prev = st - 1
            if prev < 0 : prev = 0
            end = co - 1
            next = st + 1
            if next >= co : next = end
            p['VIEW'] = 1
        p['START'] = 0
        u['start'] = Url(req.url, kw = req.kw, param = p)
        p['START'] = prev
        u['prev'] = Url(req.url, kw = req.kw, param = p)
        p['START'] = next
        u['next'] = Url(req.url, kw = req.kw, param = p)
        p['START'] = end
        u['end'] = Url(req.url, kw = req.kw, param = p)
        req.data.list_rec['navigation'] = u
        return u

    def Navigation (self, req) :
        if self.navigation_object == 1 :
            pass
        elif type(self.navigation_object) == type('') :
            self.NavigationUrl( req )
            return 'NAVIGATION OBJECT'
        elif not self.navigation_object :
            return '\n<!-- no navigation -->\n'
        else :
            return apply( self.navigation_object, ( req,), {} )
        if req.data.list_rec['mode'] == 'view' :
            return self.NavigationView (req)
        else :
            return self.NavigationList (req)

    def NavigationView (self, req) :
        u = self.NavigationUrl(req)
        co = req.data.list_rec['count']
        s = '\n\n<table border="0" cellpadding="0" cellspacing="2">\n'
        s = s + '<tr><td valign="center">\n'
        s = s + '<form method="POST" action="%s">&nbsp;' % u['action']
        s = s + '<font face="helvetica,verdana,geneva" size="2">'
        if not req.data.list_rec['params'].has_key('SUBS') : sub = ''
        else : sub = req.data.list_rec['params']['SUBS']
        if self.ps_menu :
            for k,v in req.data.list_rec['psparams'].items() :
                s = s + '<input type="hidden" name="%s" value="%s">' % (k,v)
        if self.search_fields :
            s = s + '<input type="text" size="12" name="SUBS" value="%s">' % sub
            s = s + '<input type="submit" name="SUBMIT" value="%s">' % \
                self.label_search
        s = s + '</font>&nbsp;'
        s = s + '</form>\n'
        s = s + '</td><td>\n'
        s = s + '&nbsp;&nbsp;</td><td>\n'
        if co < 1 : s = s + '&nbsp;'
        else :
            sd = { 'from' : req.data.list_rec['start']+1,
                   'count' : req.data.list_rec['count'] }
            s = s + self.label_sum_view % sd
        s = s + '</td><td>\n'
        s = s + '&nbsp;&nbsp;&nbsp;</td><td>\n'
        s = s + '<a href="%s">%s</a>' % (u['start'], self.label_start)
        s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (u['prev'], self.label_prev)
        s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (u['next'], self.label_next)
        s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (u['end'], self.label_end)
        if co > 1 :
            s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (
                            u['list'], self.label_list)
        if req.data.list_rec['params'] :
            s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (
                            u['all'], self.label_all)
        s = s + '\n</td></tr></table>\n\n\n'
        return s

    def NavigationList (self, req) :
        u = self.NavigationUrl(req)
        co = req.data.list_rec['count']
        s = '\n\n<table border="0" cellpadding="0" cellspacing="2">\n'
        s = s + '<tr>'
        if self.search_fields :
            s = s + '<td valign="center" rowspan="2">\n'
            s = s + '<form method="POST" action="%s">&nbsp;' % u['action']
            s = s + '<font face="helvetica,verdana,geneva" size="2">'
            if not req.data.list_rec['params'].has_key('SUBS') : sub = ''
            else : sub = req.data.list_rec['params']['SUBS']
            if self.ps_menu :
                for k,v in req.data.list_rec['psparams'].items() :
                    s = s + '<input type="hidden" name="%s" value="%s">' % (k,v)
            s = s + '<input type="text" size="12" name="SUBS" value="%s">' % sub
            s = s + '<input type="submit" name="SUBMIT" value="%s">' % \
                            self.label_search
            s = s + '</font>&nbsp;'
            s = s + '</form>\n'
            s = s + '</td>'
        s = s + '<td rowspan="2">\n'
        s = s + '&nbsp;&nbsp;</td><td valign="bottom" align="center">\n'
        if co < 1 : s = s + '&nbsp;'
        else :
            sd = { 'from' : req.data.list_rec['start']+1,
                   'to' : req.data.list_rec['start'] + \
                            len(req.data.list_rec['list']),
                   'count' : req.data.list_rec['count'] }
            s = s + self.label_sum_list % sd
        s = s + '</td><td rowspan="2">\n'
        s = s + '&nbsp;&nbsp;&nbsp;</td><td colspan="2" valign="bottom">\n'
        if len(u['pages']) > 1 :
            s = s + '<a href="%s">%s</a>' % (u['prev'], self.label_prev)
            ux = u['page_url']
            for p,x,m in u['pages'] :
                ux.param( 'START', x )
                s = s + '&nbsp;&nbsp;'
                if m == 1 :
                    s = s + '<a href="%s"><b>%d</b></a>' % (ux,p)
                else :
                    if self.color_current_page :
                        s = s + '<a href="%s">' % ux + \
                            '<font color="%s"><b>%d</b></font></a>' % \
                                (self.color_current_page, p)
                    else :
                        s = s + '<a href="%s"><b>%d</b></a>' % (ux, p)
            s = s + '&nbsp;&nbsp;<a href="%s">%s</a>' % (
                        u['next'], self.label_next)
        s = s + '\n</td></tr>\n'
        s = s + '\n<tr><td valign="top" align="center">\n'
        if not req.data.list_rec['params'] : s = s + '&nbsp;'
        else : s = s + '<a href="%s">%s</a>' % (u['all'], self.label_all)
        s = s + '</td><td align="left" valign="top">\n'
        if len(u['pages']) > 1 :
            s = s + '<a href="%s">%s</a>' % (u['start'], self.label_start)
        s = s + '</td><td align="right" valign="top">\n'
        if len(u['pages']) > 1 :
            s = s + '<a href="%s">%s</a>' % (u['end'], self.label_end)
        s = s + '\n</td></tr></table>\n\n\n'
        return s

    def ListUrl (self, req) :
        st = req.data.list_rec['start']
        p = copy.copy( req.data.list_rec['params'] )
        if self.ps_menu :
            for k,v in req.data.list_rec['psparams'].items() :
                p[k] = v
        ordr = []
        for fi in self.list_fields :
            f = fi.fullname()
            if not f : continue
            p['ORDER'] = string.lower(f)
            u1 = Url(req.url, kw = req.kw, param = p)
            p['ORDER'] = string.upper(f)
            u2 = Url(req.url, kw = req.kw, param = p)
            ordr.append((u1, u2))
        if req.data.list_rec['col_order'] :
            p['ORDER'] = req.data.list_rec['col_order']
        else :
            del p['ORDER']
        p['VIEW'] = '1'
        li = []
        for i in range(len(req.data.list_rec['list'])) :
            p['START'] = st + i
            li.append(Url( req.url, kw = req.kw, param = p))
        req.data.list_rec['list_url'] = li
        req.data.list_rec['order_url'] = ordr
        return li, ordr

    def listRowStart(self, rec):
        '''umozni podle recordu nejak ostylovat radek ve vypisu'''
        return '<tr>\n'

    def List (self, req) :
        db = getattr( req.glob, self.database )
        if self.navigation_inside :
            s = '<table width="100%" border="0" ' + \
                    'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
            s = s + self.Navigation(req)
        else :
            s = self.Navigation(req)
            s = s + '<table width="100%" border="0" ' + \
                        'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
        s = s + '<table width="100%" border="0" ' + \
                        'cellpadding="2" cellspacing="1">\n'
        i = req.data.list_rec['start'] + 1
        cols = 0
        for fi in self.list_fields :
            if not 'L' in fi.hidden : cols = cols + 1
        s = s + '<tr>\n'
        if self.numbering :
            s = s + '<td>&nbsp;</td>\n'
        od = req.data.list_rec['col_order']
        liu, ordr = self.ListUrl(req)
        db = getattr(req.glob, self.database)
        k = -1
        for fi in self.list_fields :
            k = k + 1
            f = fi.fullname()
            if 'L' in fi.hidden : continue
            if not fi.fullname() :
                if self.color_heading :
                    s = s + '<td align="center" bgcolor="%s"><b>' %\
                        self.color_heading
                else :
                    s = s + '<td align="center"><b>'
                s = s + fi.Title() + '</b></td>\n'
                k = k - 1
            elif od :
                if string.lower(od) == string.lower(f) :
                    if self.color_heading :
                        s = s + '<td align="center" bgcolor="%s"><b>' %\
                                self.color_heading
                    else :
                        s = s + '<td align="center"><b>'
                    if od[0:1] in string.uppercase :
                        s = s + '<a href="%s">' % ordr[k][0]
                        s = s + '<img src="/_wmlib/img/list/sortdesc.gif" ' +\
                                'border="0">'
                        s = s + '&nbsp;'
                    else :
                        s = s + '<a href="%s">' % ordr[k][1]
                        s = s + '<img src="/_wmlib/img/list/sortasc.gif" ' +\
                                'border="0">'
                        s = s + '&nbsp;'
                else :
                    if self.color_heading :
                        s = s + '<td align="center" bgcolor="%s"><b>' %\
                                self.color_heading
                    else :
                        s = s + '<td align="center"><b>'
                    s = s + '<a href="%s">' % ordr[k][0]
                s = s + '%s</a></b></td>\n' % fi.Title()
            else :
                if self.color_heading :
                    s = s + '<td align="center" bgcolor="%s"><b>' %\
                                self.color_heading
                else :
                    s = s + '<td align="center"><b>'
                s = s + '<a href="%s">' % ordr[k][0]
                s = s + '%s</a></b></td>\n' % fi.Title()
        s = s + '</tr>\n'
        for k in range(len(req.data.list_rec['list'])) :
            r = req.data.list_rec['list'][k]
            s += self.listRowStart(r)
            if self.numbering :
                if i % 2 == 0 and self.color_row2 :
                    s = s + '<td bgcolor="%s">%d.</td>\n' % (self.color_row2,i)
                elif i % 2 == 1 and self.color_row1 :
                    s = s + '<td bgcolor="%s">%d.</td>\n' % (self.color_row1,i)
                else :
                    s = s + '<td>%d.</td>\n' % i
            j = 0
            for fi in self.list_fields :
                f = fi.fullname()
                if 'L' in fi.hidden :
                    j = j + 1
                    continue
                if i % 2 == 0 and self.color_row2 :
                    s = s + '<td bgcolor="%s">' % self.color_row2
                elif i % 2 == 1 and self.color_row1 :
                    s = s + '<td bgcolor="%s">' % self.color_row1
                else :
                    s = s + '<td>'
                if isinstance( fi, field_virtual ) :
                    req.data._self2list_liu =liu[k]
                    s = s + fi.self2list( req, r )
                    if fi.fullname() : j = j + 1
                else :
                    if self.clickable and fi.aslink :
                        s = s + '<a href="%s">' % liu[k]
                    s = s + fi.python2html(r[j], '&nbsp;' )
                    if self.clickable and fi.aslink :
                        s = s + '</a>'
                    j = j + 1
                s = s + '</td>\n'
            s = s + '</tr>\n'
            i = i + 1
        s = s + '</table>\n'
        s = s + '</td></tr>\n'
        s = s + '</table>\n'
        return s

    def View (self, req) :
        db = getattr(req.glob, self.database)
        if self.navigation_inside :
            s = '<table width="100%" border="0" ' +\
                    'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
            s = s + self.Navigation( req )
        else :
            s = self.Navigation( req )
            s = s + '<table width="100%" border="0" ' +\
                        'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
        s = s + '<table width="100%" border="0" ' +\
                        'cellpadding="2" cellspacing="1">\n'
        rec = req.data.list_rec['list'][0]
        k = 0
        for fi in self.view_fields :
            if 'V' in fi.hidden :
                k = k + 1
                continue
            s = s + '<tr>\n'
            if self.color_column1 :
                s = s + '<td bgcolor="%s"><b>' % self.color_column1
            else :
                s = s + '<td><b>'
            s = s + fi.Title() + '</b></td>\n'
            if self.color_column2 :
                s = s + '<td bgcolor="%s">' % self.color_column2
            else :
                s = s + '<td>'
            if isinstance(fi, field_virtual ) :
                s = s + fi.self2list( req, rec )
                if fi.fullname() : k = k + 1
            else :
                s = s + fi.python2html(rec[k],'&nbsp;')
                k = k + 1
            s = s + '</td>\n</tr>\n'
        s = s + '</table>\n'
        s = s + '</td></tr>\n'
        s = s + '</table>\n'
        return s

    def NotFound ( self, req ) :
        if self.navigation_inside :
            s = '<table width="100%" border="0" ' +\
                    'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
            s = s + self.Navigation( req )
        else :
            s = self.Navigation( req )
            s = s + '<table width="100%" border="0" ' +\
                    'cellpadding="2" cellspacing="0">\n'
            if self.color_background :
                s = s + '<tr><td bgcolor="%s">\n' % self.color_background
            else :
                s = s + '<tr><td>\n'
        s = s + '<table width="100%" border="0" ' +\
                    'cellpadding="2" cellspacing="1">\n' +\
            '<tr><td bgcolor="#FFFFFF"><b>' +\
                self.not_found_message + '</b></td></tr>\n'
        s = s + '</table>\n'
        s = s + '</td></tr>\n'
        s = s + '</table>\n'
        return s

    def PersistentSelection ( self, req ) :
        if self.ps_menu :
            pslen = len(self.ps_menu)
            rec = [''] * pslen
            req.data.list_rec['psoff'] = 0
            req.data.list_rec['pswhere'] = ''
            if self.ps_off :
                for k in self.ps_off :
                    if req.param.has_key( k ) :
                        req.data.list_rec['psoff'] = 1
            q = 0
            for i in range(pslen) :
                n = "_ps_%02d" % i
                if req.param.has_key( n ) : q = 1
            if q == 0 and self.ps_default :
                rec = self.ps_default
            if not req.param.has_key( '_ps_all' ) :
                wh = []
                for i in range(pslen) :
                    n = "_ps_%02d" % i
                    if req.param.has_key( n ) :
                        rec[i] = req.param[n].value
                    elif rec[i] != None and rec[i] != '' :
                        pass
                    else :
                        continue
                    x = self.ps_menu[i]
                    for y in x[1] :
                        if type(y) == type('') and len(x) > 2 and x[2] :
                            if rec[i] == y :
                                if type(x[2]) == type('') :
                                    wh = wh + [x[2] % y]
                                #else :
                                #    wh = wh + x[2]( req, rec )
                        elif type(y) != type('') and len(y) > 1 :
                            if rec[i] == y[0] :
                                wh = wh + [y[1]]
                for i in range(pslen) :
                    x = self.ps_menu[i]
                    if len(x) > 2 and x[2] and type(x[2]) != type('') :
                        wh = wh + x[2]( req, rec )
                req.data.list_rec['pswhere'] = string.join( wh, ' AND ' )
            req.data.list_rec['psrec'] = rec
            tr = html.TD()
            for i in  range(pslen) :
                x = self.ps_menu[i]
                s = '<font face="helvetica,verdana,geneva" size="1">\n'
                s = s + '<select name="_ps_%02d">\n' % i
                s = s + '<option value="">- %s -</option>' % html_escape(x[0])
                for y in x[1] :
                    if type(y) == type('') : z = y
                    else : z = y[0]
                    if rec and rec[i] == z :
                        s = s + '<option selected>%s</option>' % html_escape(z)
                    else :
                        s = s + '<option>%s</option>' % html_escape(z)
                s = s + '</select>\n'
                s = s + '</font>\n'
                tr.add(s)
            s = '<font face="helvetica,verdana,geneva" size="1">\n'
            s = s + '<input type="submit" name="_ps_select" value="%s">' %\
                    self.label_ps_select
            s = s + '</font>\n'
            tr.add(s)
            s = '<font face="helvetica,verdana,geneva" size="1">\n'
            s = s + '<input type="submit" name="_ps_all" value="%s">' %\
                    self.label_ps_all
            s = s + '</font>\n'
            tr.add(s)
            s = '<form action="%s">\n' % req.relurl( req.url, kw = req.kw )
            s = s + str(html.TABLE(html.TR(tr), border=0, cellpadding=0,
                                    cellspacing=0))
            s = s + '</form>\n'
            return s
        else :
            return ''

    def __call__ ( self, req ) :
        self.real_init( req )
        req.data.list_rec = {}
        req.data.list_rec['object'] = self
        s = self.PersistentSelection(req)
        try :
            req.data.list_rec['start'] = string.atoi(req.param['START'].value)
            if req.data.list_rec['start'] < 0 :
                req.data.list_rec['start'] = 0
        except :
            req.data.list_rec['start'] = 0
        if req.param.has_key( 'VIEW' ) :
            req.data.list_rec['mode'] = 'view'
        else :
            req.data.list_rec['start'] = \
                (req.data.list_rec['start']/self.max)*self.max
            if self.view_if_found_one : req.data.list_rec['mode'] = None
            else : req.data.list_rec['mode'] = 'list'
        self.Query( req )
        # Not found
        if len(req.data.list_rec['list']) < 1 :
            req.data.list_rec['mode'] = 'list'
            if self.not_found_object == 1 :
                return s + self.NotFound( req )
            else :
                return s + apply( self.not_found_object, ( req,), {} )
        # View
        if req.data.list_rec['mode'] == 'view' :
            if self.view_object == 1 :
                s = s + self.View( req )
            else :
                s = s + apply( self.view_object, ( req,), {} )
        # List
        elif req.data.list_rec['mode'] == 'list' :
            fd = {}
            for k,v in self.other_formats : fd[k] = v
            if req.kw.has_key('fmt') and fd.has_key(req.kw['fmt']) :
                fd[req.kw['fmt']]( self, req )
                return None
            else :
                if self.list_object == 1 :
                    s = s + self.List( req )
                else :
                    self.ListUrl( req )
                    s = s + apply( self.list_object, ( req,), {} )
            if self.other_formats :
                ofmt = []
                lr = req.data.list_rec
                p = {}
                if lr['params'] :
                    for k,v in lr['params'].items() : p[k] = v
                if lr['psparams'] :
                    for k,v in lr['psparams'].items() : p[k] = v
                for k,v in self.other_formats :
                    kw = copy.copy( req.kw )
                    kw['fmt'] = k
                    u = Url( req.url, kw = kw, param = p )
                    ofmt.append( '<a href="%s">[%s]</a>' % (u,k))
                s = s + '<p>' + string.join( ofmt, ' ' ) + '</p>\n'
        return s


class SqlListPopUp ( SqlList ) :

    html_header = '''<html><head><title>SqlList</title></head>
<style type="text/css">
A {text-decoration: none; }
body, form, input, td, li, ul, p {
  font-family:"helvetica", "verdana", "arial";
  font-size:9pt; }
</style>
<script type="text/javascript" language="JavaScript1.2" src="/_wmlib/js/wm3.js">
</script>
<body bgcolor="white" link="#0000BB" vlink="#0000BB">
<a href="" onclick="close_me()"><img src="/_wmlib/img/close.gif"
alt="Close window" border="0"></a>'''

    html_footer = '''</body></html>\n'''
    view_if_found_one = 1
    label_set = messages.SQLLIST_POPUP_LABEL_SET
    field_virtual_popup_set = field_virtual_popup_set

    def get_popup_set_field (self, req) :
        return self.field_virtual_popup_set (
                title = '&nbsp;',
                label_set = self.label_set)

    def real_init (self, req) :
        if self.initialized : return
        SqlList.real_init(self, req)
        fvp = self.get_popup_set_field(req)
        self.list_fields.append(fvp)
        if self.list_fields != self.view_fields :
            self.view_fields.append(fvp)
