#include <Python.h>
#include <sys/types.h>

#define SDN_MIN		2
#define SDN_MAX		536838576
#define TIME_MIN	0
#define TIME_MAX	86399
#define USEC_MIN	0
#define USEC_MAX	999999

typedef struct {
	PyObject_HEAD
	/* SND 1 =  November 25, 4714 BC */
	int32_t		sdn;	   /* Serial day number */
	PyObject	*argument; /* Argument for +/- operations */
} DateObject;

typedef struct {
	PyObject_HEAD
	int32_t		sdn;	   /* Serial day number */
	int32_t		time;	   /* Time 0 - 86399 */
	int32_t		usec;	   /* Mikrosekundy */
	PyObject	*argument; /* Argument for +/- operations */
} DateTimeObject;

typedef struct {
	PyObject_HEAD
	char		dir;	   /* '+' or '-' */
	int32_t		days;	   /* Number of days */
	int32_t		time;	   /* Time 0 - 86399 */
	int32_t		usec;	   /* Mikrosekundy */
	PyObject	*argument; /* Argument for +/- operations */
} IntervalObject;

typedef struct {
	char		*tz;	       /* zone name */
	int		tz_offset;    /* zone shift in hours */
	int		dst_offset;   /* daylight saving time delta */
} zone_struct;

#define TZ_NONE		0
#define TZ_ZONE		1
#define TZ_NUMERIC	2

typedef union {
	struct {
		char	*tz;
		int	tz_offset;
		int	dst_offset;
	} zone;
	struct {
		char	tz[8];
		int	tz_offset;
	} numeric;
} zone_t;

typedef struct {
	int32_t		year;
	int8_t		month;
	char		*month_name;
	char		*month_short;
	int8_t		day;
	int8_t		wday;
	char		*wday_name;
	char		*wday_short;
	int16_t		yday;
	int8_t		week;
	int8_t		leap;
	/* Next only in DateTime */
	int8_t		hour;
	int8_t		min;
	int8_t		sec;
	int32_t		usec;
	int		tztype;
	zone_t		tz;
} DateTimeAttr;

typedef struct {
	char		dir;
	int32_t		days;
	int8_t		hour;
	int8_t		min;
	int8_t		sec;
	int32_t		usec;
} IntervalAttr;

/* Macros */
#define PyDate_Check(x)		(((DateObject *)(x))->ob_type == &DateType)
#define PyDateTime_Check(x)	(((DateTimeObject *)(x))->ob_type == &DateTimeType)
#define PyInterval_Check(x)	(((IntervalObject *)(x))->ob_type == &IntervalType)
#define D(c)  (c - '0')


/* dateconst.c */
extern int G_TZ_Offset;
extern int year_days[2][13];
extern int month_days[2][13];
extern char *month_names[13];
extern char *month_shortcuts[13];
extern char *weekday_names[7];
extern char *weekday_shortcuts[7];
extern zone_struct zone_map[];

/* dateattr.c */
extern int dta2date (DateTimeAttr *dta, DateObject *date);
extern int dta2datetime (DateTimeAttr *dta, DateTimeObject *date);
extern int date2dta (DateObject *dt, DateTimeAttr *dta);
extern char *d_euro (DateTimeAttr *dta, char *buf);
extern char *d_postgres (DateTimeAttr *dta, char *buf);
extern char *d_sql (DateTimeAttr *dta, char *buf);
extern char *d_iso (DateTimeAttr *dta, char *buf);
extern char *d_unix (DateTimeAttr *dta, char *buf);
extern char *d_rfc (DateTimeAttr *dta, char *buf);
extern int datetime2dta (DateTimeObject *dt, DateTimeAttr *dta, char *tz);
extern char *dt_euro (DateTimeAttr *dta, char *buf);
extern char *dt_euronotz (DateTimeAttr *dta, char *buf);
extern char *dt_postgres (DateTimeAttr *dta, char *buf);
extern char *dt_sql (DateTimeAttr *dta, char *buf);
extern char *dt_iso (DateTimeAttr *dta, char *buf);
extern char *dt_unix (DateTimeAttr *dta, char *buf);
extern char *dt_rfc (DateTimeAttr *dta, char *buf);
extern int ita2interval (IntervalAttr *ita, IntervalObject *i);
extern int interval2ita (IntervalObject *i, IntervalAttr *ita);
extern char *i_fmt1 (IntervalAttr *i, char *buf);
extern char *i_fmt2 (IntervalAttr *i, char *buf);
extern char *i_fmt3 (IntervalAttr *i, char *buf);

/* dateaux.c */
extern int IS_LEAP_YEAR(int y);
extern int sdn2date(int32_t sdn,  int32_t *y, int8_t *m, int8_t *d);
extern int date2sdn(int32_t y, int8_t m, int8_t d, int32_t *sdn);
extern int time2HMS(int32_t t, int *H, int *M, int *S);
extern int findzone(const char *tzn);
extern int datetime2tzindex (int y, int m, int d, int H, int M, int S);
extern int zone2offset(DateTimeObject *f, const char *tz);
extern int gmt2local_fix (DateTimeObject *f, DateTimeObject *t, const char *tz);
extern int local2gmt_fix (DateTimeObject *f, DateTimeObject *t, const char *tz);
extern int double2interval (double f, IntervalObject *i);

/* dateparse.c */
int parsedate(const char *st, DateTimeAttr *dta);
int parseint(const char *st, IntervalAttr *ita);

/* dateerr.c */
typedef struct {
	int	ec;
	char	*err;
} err_t;
extern char *code2err(err_t *et, int c);

extern err_t err_sdn2date[];
extern err_t err_date2sdn[];
extern err_t err_dta2datetime[];
extern err_t err_ita2interval[];
