#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <sys/types.h>
#include <Python.h>

#include "date.h"


int dta2date (DateTimeAttr *dta, DateObject *date)
{
	return date2sdn(dta->year, dta->month, dta->day, &date->sdn);
}



int dta2datetime (DateTimeAttr *dta, DateTimeObject *date)
{
	int		r;
	char		*tz;
	DateTimeObject	so;

	r = date2sdn(dta->year, dta->month, dta->day, &so.sdn);
	if(r < 0) return r + 400;
	if(dta->hour < 0 || dta->hour > 23) return -505;
	if(dta->min < 0 || dta->min > 59) return -506;
	if(dta->sec < 0 || dta->sec > 61) return -507;
	so.time = dta->hour * 3600 + dta->min * 60 + dta->sec;
	if(dta->usec < 0 || dta->usec > 999999) return -508;
	so.usec = dta->usec;
	if(dta->tztype == TZ_ZONE) tz = dta->tz.zone.tz;
	else if (dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = NULL;
	if(local2gmt_fix(&so, date, tz) < 0) return -509;
	return 0;
}



int date2dta (DateObject *dt, DateTimeAttr *dta)
{
	int	r;

	memset(dta, 0, sizeof(*dta));
	sdn2date(dt->sdn, &dta->year, &dta->month, &dta->day);
	dta->month_name = month_names[dta->month];
	dta->month_short = month_shortcuts[dta->month];
	dta->wday = (dt->sdn % 7) + 1;
	if(dta->wday == 7) dta->wday = 0;
	dta->wday_name = weekday_names[dta->wday];
	dta->wday_short = weekday_shortcuts[dta->wday];
	dta->leap = IS_LEAP_YEAR(dta->year);
	if(dta->leap) dta->yday = year_days[1][dta->month-1] + dta->day;
	else dta->yday = year_days[0][dta->month-1] + dta->day;
	r = dta->wday - (dta->yday  % 7) + 1;
	if(r < 0) r = r + 7;
	else if(r > 6) r = r- 7;
	dta->week = (dta->yday - 1) / 7 + 1;
	if(dta->wday < r) dta->week++;
	return 0;
}



char *d_euro (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%d.%d.%d", dta->day, dta->month, dta->year);
	else
            sprintf(buf, "%d.%d.%d BC", dta->day, dta->month, -dta->year);
	return buf;
}



char *d_postgres (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%02d-%02d-%d", dta->month, dta->day, dta->year);
	else
            sprintf(buf, "%02d-%02d-%d BC", dta->month, dta->day, -dta->year);
	return buf;
}



char *d_sql (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%02d/%02d/%d", dta->month, dta->day, dta->year);
	else
            sprintf(buf, "%02d/%02d/%d BC", dta->month, dta->day, -dta->year);
	return buf;
}



char *d_iso (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%d-%02d-%02d", dta->year, dta->month, dta->day);
	else
            sprintf(buf, "%d-%02d-%02d BC", -dta->year, dta->month, dta->day);
	return buf;
}



char *d_unix (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%s %d %d", dta->month_short, dta->day, dta->year);
	else
            sprintf(buf, "%s %d %d BC", dta->month_short, dta->day, -dta->year);
	return buf;
}



char *d_rfc (DateTimeAttr *dta, char *buf)
{
	if(dta->year > 0)
            sprintf(buf, "%d %s %d", dta->day, dta->month_name, dta->year);
	else
            sprintf(buf, "%d %s %d BC", dta->day, dta->month_name, -dta->year);
	return buf;
}



int datetime2dta (DateTimeObject *dt, DateTimeAttr *dta, char *tz)
{
	int		r, z;
	DateTimeObject	rdt;

	memset(dta, 0, sizeof(*dta));
	if(gmt2local_fix(dt, &rdt, tz) < 0) return -1;
	sdn2date(rdt.sdn, &dta->year, &dta->month, &dta->day);
	dta->month_name = month_names[dta->month];
	dta->month_short = month_shortcuts[dta->month];
	dta->wday = (rdt.sdn % 7) + 1;
	if(dta->wday == 7) dta->wday = 0;
	dta->wday_name = weekday_names[dta->wday];
	dta->wday_short = weekday_shortcuts[dta->wday];
	dta->leap = IS_LEAP_YEAR(dta->year);
	if(dta->leap) dta->yday = year_days[1][dta->month-1] + dta->day;
	else dta->yday = year_days[0][dta->month-1] + dta->day;
	r = dta->wday - (dta->yday  % 7) + 1;
	if(r < 0) r = r + 7;
	else if(r > 6) r = r- 7;
	dta->week = (dta->yday - 1) / 7 + 1;
	if(dta->wday < r) dta->week++;
        time2HMS(rdt.time, &dta->hour, &dta->min, &dta->sec);
	dta->usec = rdt.usec;

	if(! tz) {
		z = datetime2tzindex(dta->year, dta->month, dta->day,
                                     dta->hour, dta->min, dta->sec);
		if(z < 0) return -2;
		dta->tztype = TZ_ZONE;
		dta->tz.zone.tz = zone_map[z].tz;
		dta->tz.zone.tz_offset = zone_map[z].tz_offset;
		dta->tz.zone.dst_offset = zone_map[z].dst_offset;
	} else {
		z = findzone(tz);
		if(z < 0) {
			dta->tz.numeric.tz_offset = zone2offset(&rdt,	tz);
			if(dta->tz.numeric.tz_offset == -1) return -3;
			dta->tztype = TZ_NUMERIC;
			strncpy(dta->tz.numeric.tz, tz, 7);
			dta->tz.numeric.tz[7] = '\0';
		} else {
			dta->tztype = TZ_ZONE;
			dta->tz.zone.tz = zone_map[z].tz;
			dta->tz.zone.tz_offset = zone_map[z].tz_offset;
			dta->tz.zone.dst_offset = zone_map[z].dst_offset;
		}
	}
	return 0;
}



char *dt_euro (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;
	if(dta->year > 0)
		sprintf(buf, "%d.%d.%d", dta->day, dta->month, dta->year);
	else sprintf(buf, "%d.%d.%d BC", dta->day, dta->month, -dta->year);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = dta->tz.zone.tz;
	sprintf(t, " %s", tz);
	strcat(buf, t);
	return buf;
}



char *dt_euronotz (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;
	if(dta->year > 0)
		sprintf(buf, "%d.%d.%d", dta->day, dta->month, dta->year);
	else sprintf(buf, "%d.%d.%d BC", dta->day, dta->month, -dta->year);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	return buf;
}



char *dt_postgres (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;
	if(dta->year > 0)
            sprintf(buf, "%02d-%02d-%d", dta->month, dta->day, dta->year);
	else sprintf(buf, "%02d-%02d-%d BC", dta->month, dta->day, -dta->year);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = dta->tz.zone.tz;
	sprintf(t, " %s", tz);
	strcat(buf, t);
	return buf;
}



char *dt_sql (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;

	if(dta->year > 0)
            sprintf(buf, "%02d/%02d/%d", dta->month, dta->day, dta->year);
	else sprintf(buf, "%02d/%02d/%d BC", dta->month, dta->day, -dta->year);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = dta->tz.zone.tz;
	sprintf(t, " %s", tz);
	strcat(buf, t);
	return buf;
}



char *dt_iso (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;

	if(dta->year > 0)
            sprintf(buf, "%d-%02d-%02d", dta->year, dta->month, dta->day);
	else sprintf(buf, "%d-%02d-%02d BC", -dta->year, dta->month, dta->day);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = dta->tz.zone.tz;
	sprintf(t, " %s", tz);
	strcat(buf, t);
	return buf;
}



char *dt_unix (DateTimeAttr *dta, char *buf)
{
	char	t[16], *tz;
	sprintf(buf, "%s %s %d %d:%02d:%02d", dta->wday_short,
			dta->month_short, dta->day,
			dta->hour, dta->min, dta->sec);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) tz = dta->tz.numeric.tz;
	else tz = dta->tz.zone.tz;
	if(dta->year > 0) sprintf(t, " %s %d", tz, dta->year);
	else sprintf(t, " %s %d BC", tz, dta->year);
	strcat(buf, t);
	return buf;
}



char *dt_rfc (DateTimeAttr *dta, char *buf)
{
	char	t[16];
	int	off;

	/* Wed, 21 Jul 1999 17:11:24 +0200 */
	if(dta->year > 0)
		sprintf(buf, "%s, %d %s %d", dta->wday_short,
				dta->day, dta->month_short, dta->year);
	else
		sprintf(buf, "%s, %d %s %d BC", dta->wday_short,
				dta->day, dta->month_short, dta->year);
	sprintf(t, " %d:%02d:%02d", dta->hour, dta->min, dta->sec);
	strcat(buf, t);
	if(dta->usec > 0) {
		int	i;
		sprintf(t, ".%06d", dta->usec);
		for(i = strlen(t) - 1; i > 0; i--)
			if(t[i] == '0') t[i] = '\0';
			else break;
		strcat(buf, t);
	}
	if(dta->tztype == TZ_NUMERIC) off = dta->tz.numeric.tz_offset;
	else off = dta->tz.zone.tz_offset + dta->tz.zone.dst_offset;
	if(off < 0) {
		off = - off;
		sprintf(t, " -%02d%02d", off / 60, off % 60);
	} else	{
		sprintf(t, " +%02d%02d", off / 60, off % 60);
	}
	strcat(buf, t);
	return buf;
}



int ita2interval (IntervalAttr *ita, IntervalObject *i)
{
	if(ita->hour < 0 || ita->hour > 23) return -500;
	if(ita->min < 0 || ita->min > 59) return -501;
	if(ita->sec < 0 || ita->sec > 59) return -502;
	if(ita->usec < USEC_MIN || ita->usec > USEC_MAX) return -503;
	if(ita->days < 0) {
		i->days = -ita->days;
		i->dir = '-';
	} else {
		i->days = ita->days;
		if(ita->dir == '-') i->dir = '-';
		else i->dir = '+';
	}
	i->time = ita->hour * 3600 + ita->min * 60 + ita->sec;
	i->usec = ita->usec;
	return 0;
}



int interval2ita (IntervalObject *i, IntervalAttr *ita)
{
	int32_t		t;
	ita->dir = i->dir;
	ita->days = i->days;
	ita->hour = i->time / 3600;
	t = i->time % 3600;
	ita->min = t / 60;
	ita->sec = t % 60;
	ita->usec = i->usec;
	return 0;
}



char *i_fmt1 (IntervalAttr *i, char *buf)
{
	char	t[16];
	if(i->dir == '-')
            sprintf(buf, "-%d:%02d:%02d:%02d",i->days, i->hour, i->min, i->sec);
	else
            sprintf(buf, "+%d:%02d:%02d:%02d",i->days, i->hour, i->min, i->sec);
	if(i->usec > 0) {
		sprintf(t, ".%06d", i->usec);
		strcat(buf, t);
	}
	return buf;
}



char *i_fmt2 (IntervalAttr *i, char *buf)
{
	char	t[16];
	if(i->dir == '-')
            sprintf(buf, "%d d %02d:%02d:%02d ago",
                    i->days, i->hour, i->min, i->sec);
	else
            sprintf(buf, "%d d  %02d:%02d:%02d",
                    i->days, i->hour, i->min, i->sec);
	if(i->usec > 0) {
		sprintf(t, ".%06d", i->usec);
		strcat(buf, t);
	}
	return buf;
}



char *i_fmt3 (IntervalAttr *i, char *buf)
{
	char	t[16];
	int	q = 0;

	buf[0] = '\0';
	if(i->days > 1) {
		sprintf(t, "%d days", i->days);
		strcat(buf, t);
		q = 1;
	} else if(i->days == 1) {
		strcat(buf, "1 day");
		q = 1;
	}
	if(i->hour > 1) {
		if(q == 1) strcat(buf, " ");
		sprintf(t, "%d hours", i->hour);
		strcat(buf, t);
		q = 1;
	} else if(i->hour == 1) {
		if(q == 1) strcat(buf, " ");
                strcat(buf, "1 hour");
                q = 1;
        }
	if(i->min > 1) {
		if(q == 1) strcat(buf, " ");
		sprintf(t, "%d mins", i->min);
		strcat(buf, t);
		q = 1;
	} else if(i->min == 1) {
		if(q == 1) strcat(buf, " ");
                strcat(buf, "1 min");
                q = 1;
        }
	if(i->sec > 0 && i->usec > 0) {
		if(q == 1) strcat(buf, " ");
		sprintf(t, "%d.%06d sec", i->sec, i->usec);
		strcat(buf, t);
		q = 1;
	} else if(i->sec > 0) {
		if(q == 1) strcat(buf, " ");
		sprintf(t, "%d sec", i->sec);
		strcat(buf, t);
		q = 1;
	} else if(i->usec > 0) {
		if(q == 1) strcat(buf, " ");
		sprintf(t, "0.%06d sec", i->usec);
		strcat(buf, t);
		q = 1;
	}
	if(q == 0) {
		sprintf(buf, "0 secs");
	}
	if(i->dir == '-') {
		strcat(buf, " ago");
	}
	return buf;
}



#ifdef TEST

int main (int ac, char *av[])
{
	DateTimeObject	d;
	DateTimeAttr	dta;
	int		r;
	char		b[128];

	tzset();
	r = parsedate(av[1], &dta);
	printf("parsedate: %d\n", r);
	r = dta2datetime(&dta, &d);
	printf("dta2datetime: %d\n", r);
	r = datetime2dta(&d, &dta, NULL);
	printf("datetime2dta: %d\n", r);
	printf("year: %d\n", dta.year);
	printf("month: %d %s %s\n", dta.month, dta.month_name, dta.month_short);
	printf("day: %d\n", dta.day);
	printf("yday: %d\n", dta.yday);
	printf("wday: %d %s %s\n", dta.wday, dta.wday_name,  dta.wday_short);
	printf("week: %d\n", dta.week);
	printf("leap: %d\n", dta.leap);
	printf("hour: %d\n", dta.hour);
	printf("min: %d\n", dta.min);
	printf("sec: %d\n", dta.sec);
	printf("usec: %d\n", dta.usec);
	printf("euro: %s\n", dt_euro(&dta, b));
	printf("postgres: %s\n", dt_postgres(&dta, b));
	printf("sql: %s\n", dt_sql(&dta, b));
	printf("tztype: %d\n", dta.tztype);
	printf("tz: %s\n", dta.tz.zone.tz);
	printf("tz_offset: %d\n", dta.tz.zone.tz_offset);
	printf("dst_offset: %d\n", dta.tz.zone.dst_offset);
	return 0;
}

#endif
