#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include "date.h"


#define TOKENLENGTH   24
#define TOKENMAX      24
typedef struct {
	char		typ;
        char		s[TOKENLENGTH+1];
	int		i;
	int		f;
} token_struct;


/* Token types */

#define NUM    'i'
#define STR    's'
#define CHR    'c'

#define DAY    'D'
#define HRS    'H'
#define MIN    'M'
#define SEC    'S'

#define MON    'O'
#define WEK    'W'
#define AGO    'A'
#define TZN    'T'

int tokenize (const char *st, token_struct *ps, char *pat)
{
	int	i, j, k = 0, l, t = -1, pslen;

	l = strlen(st);
	for(i = 0, j = -1; i < l; i++) {
		if(st[i] == ' ') {
			t = ' ';
			continue;
		} else if (isdigit(st[i])) {
			if(t != NUM) { j++; k = 0; t = ps[j].typ = NUM; }
			ps[j].s[k] = st[i];
			ps[j].s[k+1] = '\0';
			if(++k >= TOKENLENGTH) return -1;
		} else if (isalpha(st[i])) {
			if(t != STR) { j++; k = 0; t = ps[j].typ = STR; }
			ps[j].s[k] = st[i];
			ps[j].s[k+1] = '\0';
			if(++k >= TOKENLENGTH) return -1;
		} else if (strchr(".,:-+/@", st[i]) != NULL) {
			j++;
			k = 0;
			t = ps[j].typ = st[i];
			ps[j].s[0] = st[i];
			ps[j].s[1] = '\0';
		} else {
			return -2;
		}
		if(j >= TOKENMAX) return -1;
	}
	pslen = j + 1;
	for(j = 0; j < pslen; j++) {
		ps[j].i = ps[j].f = 0;
		if(ps[j].typ == NUM) {
			ps[j].i = atoi(ps[j].s);
		} else if(ps[j].typ == STR) {
			l = strlen(ps[j].s);
			if(strncasecmp("days", ps[j].s, l) == 0) ps[j].typ = DAY;
			else if(strncasecmp("hours", ps[j].s,l) == 0) ps[j].typ = HRS;
			else if(strncasecmp("mins", ps[j].s, l) == 0) ps[j].typ = MIN;
			else if(strncasecmp("secs", ps[j].s, l) == 0) ps[j].typ = SEC;
			else if(strcasecmp("ago", ps[j].s) == 0) ps[j].typ = AGO;
			else if(strcasecmp("before", ps[j].s) == 0) ps[j].typ = AGO;
			else {
				/* FIXME must be optimized */
				for(i = 1; i < 13; i++) {
					if(strncasecmp(month_names[i], ps[j].s, l) == 0) {
						ps[j].typ = MON;
						ps[j].i = i;
						break;
					}
				}
				for(i = 0; i < 7; i++) {
					if(strncasecmp(weekday_names[i], ps[j].s, l) == 0) {
						ps[j].typ = WEK;
						ps[j].i = i;
						break;
					}
				}
				for(i = 0; zone_map[i].tz; i++) {
					if(strcasecmp(zone_map[i].tz, ps[j].s) == 0) {
						ps[j].typ = TZN;
						ps[j].i = i;
						break;
					}
				}
			}
		}
		pat[j] = ps[j].typ;
	}
	pat[j] = '\0';
	return pslen;
}


int parsedate(const char *st, DateTimeAttr *dta)
{
	/***************************************************
	 * Pattern description:
	 * D = day (1 - 31)
	 * M = month (1 - 12)
	 * A = month as a word (Jan[uary] - Dec[ember])
	 * Y = year (1997, 97, 2300, ...)
	 * h = hour (0 - 23)
	 * m = minute (0 - 59)
	 * s = second (0 - 59)
	 * u = usecond (0-N)
	 * Z = time zone (PST, CEST, -08, +0200, ...)
	 ***************************************************/
	static char *paterns[] = {
		/* German and Czech date style: 17.12.1997 07:34:56.9876 CEST */
		"D.M.Yh:m:s.uZ", "D.M.Yh:m:s.u", "D.M.Yh:m:sZ", "D.M.Yh:m:s",
		"D.M.Yh:mZ", "D.M.Yh:m", "D.M.Y", 
		/* Postgres SQL date format: 12-17-1997 07:34:56.9876 CET */
		"M-D-Yh:m:s.uZ", "M-D-Yh:m:s.u", "M-D-Yh:m:sZ", "M-D-Yh:m:s",
		"M-D-Yh:mZ", "M-D-Yh:m", "M-D-Y", 
		/* SQL traditional date format: 12/17/1997 07:34:56.9876 CET  */
		"M/D/Yh:m:s.uZ", "M/D/Yh:m:s.u", "M/D/Yh:m:sZ", "M/D/Yh:m:s",
		"M/D/Yh:mZ", "M/D/Yh:m", "M/D/Y",
		/* ISO-8601 date format: 1997-12-17 07:34:56.9876 CET  */
		"Y-M-Dh:m:s.uZ", "Y-M-Dh:m:s.u", "Y-M-Dh:m:sZ", "Y-M-Dh:m:s",
		"Y-M-Dh:mZ", "Y-M-Dh:m", "Y-M-D",
		/* UNIX date style: Wed Dec 17 07:34:56.9876 CET 1997 */
		"WADh:m:s.uZY", "WADh:m:s.uYZ", "WADh:m:s.uY",
		"WADh:m:sZY", "WADh:m:sYZ", "WADh:m:sY",
		"WADh:mZY", "WADh:mYZ", "WADh:mY", "WADY", 
		/* Some format: Dec 17 07:34:56.9876 CET 1997 */
		"ADh:m:s.uZY", "ADh:m:s.uYZ", "ADh:m:s.uY",
		"ADh:m:sZY", "ADh:m:sYZ", "ADh:m:sY",
		"ADh:mZY", "ADh:mYZ", "ADh:mY", "ADY",
		/* Some format: Dec 17 1997 07:34:56.9876 CET */
		"ADYh:m:s.uZ", "ADYh:m:s.uZ", "ADYh:m:s.u",
		"ADYh:m:sZ", "ADYh:m:sZ", "ADYh:m:s",
		"ADYh:mZ", "ADYh:mZ", "ADYh:m",
		/* RFC 822 date format: Wed, 17 Dec 1997 07:34:56.9876 +0100 */
		"W,DAYh:m:s.uZ", "W,DAYh:m:s.u", "W,DAYh:m:sZ", "W,DAYh:m:s",
		"W,DAYh:mZ", "W,DAYh:m", "W,DAY",
		/* RFC 822 without weekday name */
		"DAYh:m:s.uZ", "DAYh:m:s.u", "DAYh:m:sZ", "DAYh:m:s",
		"DAYh:mZ", "DAYh:m", "DAY", 
		NULL };
	token_struct	ps[TOKENMAX];
	char		*p, *s, pat[TOKENMAX];
	int		i, i1, i2, l, tl, pl, pi, psi, pati, q;
	
	tl = tokenize (st, ps, pat);
	if(tl < 1)  return -1;
	memset(dta, 0, sizeof(*dta));
	if(pat[0] == NUM && pat[1] == '\0') {
		/* Tar format isn't handled by general algorithm */
		s = ps[0].s;
		l = strlen(s);
		if((l < 8)  || ((l % 2) == 1)) return -2;
		dta->year = D(s[0]) * 1000 + D(s[1]) * 100 + D(s[2]) * 10 + D(s[3]);
		dta->month = D(s[4]) * 10 + D(s[5]);
		dta->day = D(s[6]) * 10 + D(s[7]);
		if(l > 8) dta->hour = D(s[8]) * 10 + D(s[9]);
		if(l > 10) dta->min = D(s[10]) * 10 + D(s[11]);
		if(l > 12) dta->sec = D(s[12]) * 10 + D(s[13]);
		return 0;
	} 
	for(pi = 0; paterns[pi]; pi++) {
		memset(dta, 0, sizeof(*dta));
		dta->tztype = TZ_NONE;
		p = paterns[pi];
		pl = strlen(p);
		for(psi = 0, pati = 0, q = 1; pati < tl && q == 1 && pat[psi]; psi++, pati++) {
			i1 = ps[psi].i;
			i2 = ps[psi].f;
			s = ps[psi].s;
			switch(p[pati]) {
				case 'Y' : 
					if(pat[psi] == NUM) dta->year = ps[psi].i;
					else q = 0;
					break;
				case 'M' : 
					if(pat[psi] == NUM && i1 > 0 && i1 < 13) dta->month = i1;
					else q = 0;
					break;
				case 'A' :
					if(pat[psi] == MON) dta->month = i1;
					else q = 0;
					break;
				case 'D' : 
					if(pat[psi] == NUM && i1 > 0 && i1 < 32) dta->day = i1;
					else q = 0;
					break;
				case 'h' : 
					if(pat[psi] == NUM && i1 < 24) dta->hour = i1;
					else q = 0;
					break;
				case 'm' : 
					if(pat[psi] == NUM && i1 < 60) dta->min = i1;
					else q = 0;
					break;
				case 's' : 
					if(pat[psi] == NUM && i1 < 62) dta->sec = i1;
					else q = 0;
					break;
				case 'u' : 
					if(pat[psi] == NUM && i1 < 1000000) {
						dta->usec = 0;
						for(i=0, l=100000; i<strlen(s); i++, l/=10)
							dta->usec += D(s[i]) * l;
					} else {
						q = 0;
					}
					break;
				case 'W' :
					if(pat[psi] == WEK) dta->wday = i1;
					else q = 0;
					break;
				case 'Z' :
					if(pat[psi] == TZN) {
						dta->tztype = TZ_ZONE;
						dta->tz.zone.tz = zone_map[i1].tz;
						dta->tz.zone.tz_offset = zone_map[i1].tz_offset;
						dta->tz.zone.dst_offset = zone_map[i1].dst_offset;
					} else if((pat[psi]=='-' || pat[psi]=='+') && (pat[psi+1]==NUM)) {
						if(strlen(ps[psi+1].s) == 2 || strlen(ps[psi+1].s) == 4)  {
							dta->tztype = TZ_NUMERIC;
							strcpy(dta->tz.numeric.tz, s);
							strncat(dta->tz.numeric.tz, ps[psi+1].s, 6);
						} else {
							q = 0;
						}
						if( q == 1)  psi++;
					} else {
						q = 0;
					}
					break;
				default :
					if(pat[psi] != p[pati]) q = 0;
					break;
			}
			if(q == 0) break;
		}
		if((q == 1) && (pati == pl) && (psi == tl)) {
			return 0;
		}
	}
	return -4;
}

int parseint(const char *st, IntervalAttr *ita)
{
	token_struct	ps[TOKENMAX];
	char		pat[TOKENMAX], *s;
	int		i, cl, nt, j;
	
	int strnchar (const char *p, int l, char c)
	{
		int     i, r;
		for(i = 0, r = 0; i < l; i++)
			if(p[i] == c) r++;
		return r;
	}

	int toita (IntervalAttr *ita, token_struct *ps, int d, int h, int m, int s, int u)
	{
		if(d != -1) ita->days = ps[d].i;
		if(h != -1) ita->hour = ps[h].i;
		if(m != -1) ita->min = ps[m].i;
		if(s != -1) ita->sec = ps[s].i;
		if(u != -1) {
			int	i, l;
			char	*s = ps[u].s;
			if(ps[u].i > 999999) return -100;
			ita->usec = 0;
			l = 100000;
			for(i = 0; i < strlen(s); i++ , l /= 10)
				ita->usec += D(s[i]) * l;
		}
		return 0;
	}

	nt = tokenize (st, ps, pat);
	if(nt < 1) return -1;
	/*
	printf("PAT: %s\n", pat);
	for(i = 0; i < nt; i++) {
		 printf("TOKEN(%d): %c, %s, %d, %d\n", i, ps[i].typ, ps[i].s, ps[i].i, ps[i].f);
	}
	*/
	memset(ita, 0, sizeof(*ita));
	ita->dir = '+';
	if(strnchar(pat, nt, '-') > 1) return -2;
	if(strnchar(pat, nt, 'D') > 1) return -3;
	if(strnchar(pat, nt, 'H') > 1) return -4;
	if(strnchar(pat, nt, 'M') > 1) return -5;
	if(strnchar(pat, nt, 'S') > 1) return -6;
	if(strnchar(pat, nt, '@') > 1) return -7;
	if(pat[0] == '-') {
		ita->dir = '-';
		for(i = 1; i < nt; i++) {
			ps[i-1].typ = ps[i].typ;
			strcpy(ps[i-1].s, ps[i].s);
			ps[i-1].i = ps[i].i;
			ps[i-1].f = ps[i].f;
			pat[i-1] = pat[i];
		}
		nt--;
		pat[nt] = '\0';
	} else if(pat[nt-1] == AGO) {
		ita->dir = '-';
		nt--;
		pat[nt] = '\0';
	} else if(strchr(pat, '-') != NULL || strchr(pat, AGO) != NULL) {
		return -8;
	}
	if(pat[0] == '@') {
		for(i = 1; i < nt; i++) {
			ps[i-1].typ = ps[i].typ;
			strcpy(ps[i-1].s, ps[i].s);
			ps[i-1].i = ps[i].i;
			ps[i-1].f = ps[i].f;
			pat[i-1] = pat[i];
		}
		nt--;
		pat[nt] = '\0';
	}
	if(nt == 1 && pat[0] == NUM) {
		i = ps[0].i;
		ita->days = i / (24 * 3600);
		i %= 24 * 3600;
		ita->hour = i / 3600;
		i %= 3600;
		ita->min = i / 60;
		ita->sec = i % 60;
		return 0;
	}
	if(nt == 3 && pat[0] == NUM && pat[1] == '.' && pat[2] == NUM) {
		char	*s;
		int	l;
		i = ps[0].i;
		ita->days = i / (24 * 3600);
		i %= 24 * 3600;
		ita->hour = i / 3600;
		i %= 3600;
		ita->min = i / 60;
		ita->sec = i % 60;
		if(ps[2].i > 999999) return -101;
		s = ps[2].s;
		l = 100000;
		for(i = 0; i < strlen(s); i++ , l /= 10)
			ita->usec += D(s[i]) * l;
		return 0;
	}
	cl = strnchar(pat, nt, ':');
	if(cl == 3) {
		if(strcmp(pat, "i:i:i:i") == 0) {
			return toita(ita, ps, 0, 2, 4, 6, -1);
		} else if(strcmp(pat, "i:i:i:i.i") == 0) {
			return toita(ita, ps, 0, 2, 4, 6, 8);
		} else {
			return -10;
		}
	} else if(cl == 2) {
		if(strcmp(pat, "i:i:i") == 0) {
			return toita(ita, ps, -1, 0, 2, 4, -1);
		} else if(strcmp(pat, "ii:i:i") == 0) {
			return toita(ita, ps, 0, 1, 3, 5, -1);
		} else if(strcmp(pat, "ii:i:i.i") == 0) {
			return toita(ita, ps, 0, 1, 3, 5, 7);
		} else if(strcmp(pat, "i:i:i.i") == 0) {
			return toita(ita, ps, -1, 0, 2, 4, 6);
		} else if(strcmp(pat, "iDi:i:i") == 0) {
			return toita(ita, ps, 0, 2, 4, 6, -1);
		} else if(strcmp(pat, "iDi:i:i.i") == 0) {
			return toita(ita, ps, 0, 2, 4, 6, 8);
		} else {
			return -11;
		}
	} else if(cl == 1) {
		if(strcmp(pat, "i:i") == 0) {
			return toita(ita, ps, -1, 0, 2, -1, -1);
		} else if(strcmp(pat, "ii:i") == 0) {
			return toita(ita, ps, 0, 1, 3, -1, -1);
		} else if(strcmp(pat, "iDi:i") == 0) {
			return toita(ita, ps, 0, 2, 4, -1, -1);
		} else {
			return -12;
		}
	} else if(cl == 0) {
		if(strcmp(pat, "ii") == 0) {
			return toita(ita, ps, 0, 1, -1, -1, -1);
		}
	}
	if(nt % 2 != 0) return -13;
	for(i = 0; i < nt; i += 2) {
		switch(pat[i+1]) {
			case DAY : 
				if(pat[i] != NUM) return -14;
				ita->days = ps[i].i;
				break;
			case HRS : 
				if(pat[i] != NUM) return -15;
				cl = ps[i].i;
				if(cl > 23) return -16;
				ita->hour = cl;
				break;
			case MIN : 
				if(pat[i] != NUM) return -17;
				cl = ps[i].i;
				if(cl > 59) return -18;
				ita->min = cl;
				break;
			case SEC : 
				if(pat[i] != NUM) return -19;
				cl = ps[i].i;
				if(cl > 61) return -20;
				ita->sec = cl;
				break;
			case '.' :
				if(pat[i] != NUM || pat[i+2] != NUM ||
                                        pat[i+3] != SEC) return -21;
				cl = ps[i].i;
				if(cl > 61) return -22;
				ita->sec = cl;
				ita->usec = 0;
				cl = 100000;
				s = ps[i+2].s;
				if(ps[i+2].i > 999999) return -102;
				for(j = 0; j < strlen(s); j++ , cl /= 10)
					ita->usec += D(s[j]) * cl;
				i += 2;
				break;
			default :
				return -23;
		}
	}
	return 0;
}



#ifdef TEST

/*
int main(int ac, char *av[])
{

  int		r;
  DateTimeAttr	dta;
  IntervalAttr	ita;
  r = parsedate(av[1], &dta);
  printf("\nParse result: %d\n", r);
  printf("  Year:    %d\n", dta.year);
  printf("  Month:   %d\n", dta.month);
  printf("  Day:     %d\n", dta.day);
  printf("  Hour:    %d\n", dta.hour);
  printf("  Min:     %d\n", dta.min);
  printf("  Sec:     %d\n", dta.sec);
  printf("  uSec:    %d\n", dta.usec);
  printf("  TZ:      %s\n", dta.tz);
  printf("  TZ Off:  %d\n", dta.tz_offset);
  printf("  DST Off: %d\n", dta.dst_offset);
  printf("  Wday:    %d\n", dta.wday);
  r = parseint(av[1], &ita);
  printf("\nParse result: %d\n", r);
  printf("  Dir:   %c\n", ita.dir);
  printf("  Days:  %d\n", ita.days);
  printf("  Hours: %d\n", ita.hour);
  printf("  Mins:  %d\n", ita.min);
  printf("  Secs:  %d\n", ita.sec);
  printf("  uSecs: %d\n", ita.usec);
  return 0;
}
*/

#endif

