#include <string.h>
#include <stdio.h>
#include <stdlib.h>


typedef struct _array_tok {
    char		*token;
    char		typ;
    int			length;
    struct _array_tok	*next;
} array_tok;




void free_array_tok (array_tok *at)
{
    array_tok	*tt=at, *tx;
    while(tt) {
	tx = tt;
	if(tt->typ == 'i') free(tt->token);
	tt = tt->next;
	free(tx);
    }
}




array_tok *array_tokenize (char *astr)
{
    int		bstack=0;	    /* bracket count */
    int		dim=0;		    /* array dimmension */
    int		inside_el = 0;	    /* inside element */
    int		has_quote = 0;	    /* is element quoted */
    char	*c, *rstr, last='\0';
    array_tok	*list_first = malloc(sizeof(array_tok));
    array_tok	*list_last = NULL;
    array_tok	*aatok = NULL;

    list_first->token = rstr = strdup(astr);
    list_first->length = strlen(astr);
    list_first->next = NULL;
    list_first->typ = 'i';
    list_last = list_first;

    for(c=astr; *c; c++) {
	if(inside_el) {
	    if(*c == '\"') {
		if(!has_quote) {
		    /* syntax error: quote inside unquoted element */
		    free_array_tok(list_first);
		    return NULL;
		} else {
		    if(last == '\\') {
			/* escaped quote */
			aatok->length++;
			*rstr = '\"';
			rstr++;
		    } else {
			/* end of element */
			*rstr = '\0';
			rstr++;
			inside_el = 0;
		    }
		}
	    } else if(*c == '\\') {
		if(last == '\\') {
		    /* second backslash is escaped */
		    aatok->length++;
		    *rstr = '\\';
		    rstr++;
		} else {
		    /* ignore first backslash */
		}
	    } else if(*c == ',') {
		/* end of element */
		*rstr = '\0';
		rstr++;
		inside_el = 0;
	    } else if(*c == '}') {
		/* syntax error: unbalanced curly brackets */
		if(bstack < 1) {
		    free_array_tok(list_first);
		    return NULL;
		}
		bstack--;
		aatok = (array_tok*) malloc(sizeof(array_tok));
		aatok->typ = '}';
		aatok->next = NULL;
		list_last->next = aatok;
		list_last = aatok;
		*rstr = '\0';
		rstr++;
		inside_el = 0;
		if(bstack == 0) return list_first;
	    } else {
		aatok->length++;
		*rstr = *c;
		rstr++;
	    }
	} else {
	    if(*c == '{') {
		bstack++;
		/* register array dimmension */
		if(bstack > dim) dim = bstack;
		aatok = (array_tok*) malloc(sizeof(array_tok));
		aatok->typ = '{';
		aatok->next = NULL;
		list_last->next = aatok;
		list_last = aatok;
	    } else if(*c == '}') {
		/* syntax error: unbalanced curly brackets */
		if(bstack < 1) {
		    free_array_tok(list_first);
		    return NULL;
		}
		bstack--;
		aatok = (array_tok*) malloc(sizeof(array_tok));
		aatok->typ = '}';
		aatok->next = NULL;
		list_last->next = aatok;
		list_last = aatok;
		if(bstack == 0) return list_first;
	    } else if(*c == ',') {
		/* ignore */
	    } else {
		aatok = (array_tok*) malloc(sizeof(array_tok));
		aatok->typ = 's';
		aatok->next = NULL;
		if(*c == '\"') {
		    /* start of quoted element */
		    inside_el = 1;
		    has_quote = 1;
		    aatok->length = 0;
		    aatok->token = (c+1);
		    aatok->token = rstr;
		} else {
		    /* first character of unquoted element */
		    inside_el = 1;
		    has_quote = 0;
		    aatok->length = 1;
		    *rstr = *c;
		    aatok->token = rstr;
		    rstr++;
		}
		list_last->next = aatok;
		list_last = aatok;
	    }
	}
	last = *c;
    }
    return list_first;
}


#define TEST 1
#ifdef TEST

#include <unistd.h>
int main (int ac, char *av[])
{
    char	line[512];
    int		i, l;
    array_tok	*res, *ttok;

    while(fgets(line, 512, stdin)) {
	line[511] = '\0';
	l = strlen(line);
	for(--l; l>0; --l) {
	    if(line[l] == '\n' || line[l] == ' ' || line[l] == '\r') {
		line[l] = '\0';
	    } else {
		break;
	    }
	}
	printf("LINE:<<%s>>\n", line);
	res = array_tokenize(line);
	ttok = res;
	while(ttok) {
	    if(ttok->typ == 'i') {
		printf("INITIAL TOKEN(%d)%s\n", ttok->length, ttok->token);
	    } else if(ttok->typ == 's') {
		printf("TOKEN(%d)%s\n", ttok->length, ttok->token);
	    } else {
		printf("TOKEN(%c)\n", ttok->typ);
	    }
	    ttok = ttok->next;
	}
	printf("\n");
	free_array_tok(res);
    }
    for(i=0; i<1000000; i++) {
	res = array_tokenize("{{aaaa},{bbb}}");
	if(((i)%50000) == 0) {
	    printf("i=%d\n", i);
	    sleep(1);
	}
	free_array_tok(res);
    }
    return 0;
}
#endif
