from xml4.templates import *
from xml4.serialization import XHTMLSerializer
from xml4.elements import Node
from lib4.imports import importString
from os.path import join, dirname

__all__ = ['getTemplate', 'render', 'renderForLayout']

_cache = {}

def getTemplate(req, spec):
    tmpl = _cache.get(spec)
    if tmpl is None:
        mod, pth = spec.split('/',1)
        filename = join(dirname(importString(mod).__file__), pth)
        parser = TemplateParser()
        tmpl = _cache[spec] = parser.parseFile(filename)
    return tmpl

def wm3_renderer(state, obj):
    if obj is None:
        return u''
    elif isinstance(obj, Node):
        data = []
        XHTMLSerializer()(obj, data.append, state)
        return u''.join(data)
    else:
        if isinstance(obj, unicode):
            return obj
        elif isinstance(obj, str):
            return unicode(obj, state['req'].input_encoding)
        else:
            return unicode(obj)

def render(req, tmpl, **data):
    if isinstance(tmp, basestring):
        tmpl = getTemplate(req, tmpl)
    state = State(ldict=data, render=wm3_renderer)
    data = u''.join(XHTMLSerializer()(tmpl.expand(scp)))
    return data.encode(req.output_encoding)

def renderForLayout(req, tmpl, *domlets, **data):
    if isinstance(tmpl, basestring):
        tmpl = getTemplate(req, tmpl)
    data['req'] = req
    state = State(ldict=data, renderer=wm3_renderer)
    lparams = {}
    for p in domlets:
        domlet = tmpl.tmpl_namespace[p]()
        scp = state.copy(tmpl.tmpl_namespace)
        data = u''.join(XHTMLSerializer()(domlet.expand(scp)))
        lparams[p] = data.encode(req.output_encoding)
    return lparams


if __name__ == '__main__':
    import os
    from app import Request, Config

    req = Request(os.environ)
    req.conf = Config()

    p = renderForLayout(
        req, 'is3.fakturace/vystaveni_polozky.wt', 'titul', #'hlavni'
    )
    print p
    print _cache
