import string, re, os, messages, date, sys

def sql_escape (s) :
    s = string.replace( s, "\\", "\\\\" )
    s = string.replace( s, "'", "\\'" )
    s = string.replace( s, "\011", "\\t" )
    s = string.replace( s, "\012", "\\n" )
    s = string.replace( s, "\015", "\\r" )
    return s

def html_escape (s, quote = None) :
    s = string.replace(s, "&", "&amp;")
    s = string.replace(s, "<", "&lt;")
    s = string.replace(s, ">", "&gt;",)
    if quote:
        s = string.replace(s, '"', "&quot;")
    return s


_bytea = re.compile("[^a-zA-Z0-9]")

def bytea_escape (s) :
    return _bytea.sub(lambda m: "\\\\%03d" % int(oct(ord(m.group(0)))), s)

_bytea_un = re.compile("\\\\[0-8]{3}")

def bytea_unescape (s) :
    return _bytea_un.sub(
            lambda m: chr(int(m.group(0)[1:],8)),
            s.replace("\\\\", "\\"),
    )


safe_chars = string.join(
    map( lambda x : chr(x), range(ord('A'), ord('Z')+1 ) ) + \
    map( lambda x : chr(x), range(ord('a'), ord('z')+1 ) ) + \
    map( lambda x : chr(x), range(ord('0'), ord('9')+1 ) ) + \
    ['_','-','.'], '' )

def urlquote ( s, sf = '' ) :
    r = list(s)
    safe = safe_chars + sf
    for i in range(len(r)) :
        c = r[i]
        if c not in safe :
            r[i] = '%%%02x' % ord(c)
    return string.join( r, '' )

class Url :
    def __init__ ( self, obj, **kw ) :
        self._obj = str(obj)
        self._args = []
        self._kw = {}
        self._param = {}
        self._session = None
        self._target = None
        self._host = None
        if kw.has_key( 'args' ) :
            for a in kw['args'] :
                self._args.append(urlquote(str(a)))
        if kw.has_key( 'kw' ) :
            for k,v in kw['kw'].items() :
                self._kw[k] = urlquote(str(v))
        if kw.has_key( 'session' ) :
            self._session = urlquote(kw['session'])
        if kw.has_key( 'param' ) :
            for k,v in kw['param'].items() :
                if type(v) == list:
                    self._param[k] = [urlquote(str(i))
                                      for i in v]
                else:
                    self._param[k] = urlquote(str(v))
        if kw.has_key( 'target' ) :
            self._target = str(kw['target'])
        if kw.has_key( 'host' ) :
            self._host = str(kw['host'])

    def args ( self, *ar ) :
        for a in ar :
            self._args.append(urlquote(str(a)))

    def param ( self, k, v ) :
        self._param[k] = urlquote(str(v))

    def kw ( self, k, v ) :
        self._kw[k] = urlquote(str(v))

    def host ( self, h ) :
        self._host = str(h)

    def target ( self, t ) :
        self._target = str(t)

    def __str__ ( self ) :
        if self._args :
            u = os.path.join( self._obj, string.join( self._args, '/' ))
        else :
            u = self._obj
        a = self._kw
        req = getreq()
        tt = req.conf.session.token_transport
        if req.session_2url :
            if tt == 'tilde' and req.session_token :
                if u[:1] == '/' :
                    u = '/~' + req.session_token + u
                else :
                    u = '/~' + req.session_token + '/' + u
            if tt == 'kw' and req.session_token :
                a['se'] = req.session_token
        if len(a) > 0 :
            l = map( lambda x : x[0] + '$' + x[1], a.items() )
            u = u + ';' + string.join( l, ';' )
        if len(self._param) > 0 :
            l = []
            for (k,v) in self._param.items():
                if type(v) == list:
                    l.extend(['%s=%s' % (k,i)
                              for i in v])
                else:
                    l.append('%s=%s' % (k,v))
            u = u + '?' + string.join( l, '&' )
        if self._target :
            u = u + '#' + self._target
        if self._host :
            u = self._host + u
        return u

_email_re = re.compile( '([-a-z_0-9.]+@[-a-z_0-9\.]+)', re.I )
_url_re = re.compile( '((ftp|http|https)://[^," ]+)', re.I )
_tel_re = re.compile('\+[0-9]{12}')

def make_hypertext ( txt, new_window=None ) :
    if not txt : return ''
    so = txt
    m = _email_re.search( so )
    if m :
        ns = ''
        while m :
            #print 'EMAIL:', so
            st = m.start(0)
            en = m.end(0)
            ns = ns + so[:st]
            ad = so[st:en]
            ns = ns + '<a href="mailto:%s">%s</a>' % ( ad, ad )
            so = so[en:]
            m = _email_re.search( so )
        so = ns + so
    m = _url_re.search( so )
    #print 'MEZI:', so
    if m :
        ns = ''
        while m :
            #print 'URL:', so
            st = m.start(0)
            en = m.end(0)
            ns = ns + so[:st]
            ad = so[st:en]
            if new_window:
                ns = ns + '<a href="%s" target="_blank">%s</a>' % ( ad, ad )
            else:
                ns = ns + '<a href="%s">%s</a>' % ( ad, ad )
            so = so[en:]
            m = _url_re.search( so )
        so = ns + so
    return so

def findEmail(s):
    '''najde v retezci vsechny emailove adresy a vrati jejich seznam'''
    if not s:
        return []
    return _email_re.findall(s)

def findTel(s):
    '''vrati seznam vsech potencialnich telefonnich cisel v retezci'''
    if not s:
        return []
    return _tel_re.findall(s)


def month_matrix ( year, month, sunday_first = messages.SUNDAY_FIRST ) :
    date_from = date.Date( year, month )
    date_to = date_from + 32
    date_to = date.Date( date_to.year, date_to.month )
    if not sunday_first :
        def wday ( d ) :
            wd = d.wday - 1
            if wd < 0 : wd = 6
            return wd
    else :
        def wday ( d ) :
            return d.wday
    matrix = []
    week = [None] * wday(date_from)
    for i in range( date_to - date_from ) :
        d = date.Date( date_from.year, date_from.month, i + 1 )
        if wday(d) == 0 :
            if week : matrix.append( week )
            week = []
        week.append( d.day )
    week = week + [None] * (6-wday(date_to -1 ))
    if week : matrix.append( week )
    return matrix



def debug(req, force=0):
    d = req.conf.debug
    if not force:
        if not d:
            return ''
        if type(d) == type([]) and req.env['REMOTE_ADDR'] not in d:
            return ''
    s = '<h1>req object</h1>\n<ul>\n'
    for k in dir(req):
        if k == 'env': continue
        if k.startswith('__'): continue
        s = s + '<li>%s = %s</li>\n' % (k,html_escape(`getattr(req,k)`))
    s = s + '</ul>\n'

    s = s + '<h1>enviroment</h1>\n<ul>\n'
    p = req.env.keys()
    p.sort()
    for k in p:
        s = s + '<li>%s = %s</li>\n' % (k,html_escape(str(req.env[k])))
    s = s + '</ul>\n'

    s = s + '<h1>parameters</h1>\n<ul>\n'
    p = req.param
    for k in p.keys():
        s = s + '<li>%s = %s</li>\n' % (k,html_escape(str(p[k])))
    s = s + '</ul>\n'

    s = s + '<h1>data</h1>\n<ul>\n'
    p = req.data.__dict__
    for k in p.keys():
        v = p[k]
        if hasattr(v,'items'):
            s = s + '<li>%s</li>\n' % k
            s = s + '<ul>\n'
            for x,y in v.items():
                if hasattr(y, 'items'):
                    s = s + '<li>%s</li>\n' % x
                    s = s + '<ul>\n'
                    for x1, y1 in y.items():
                        s = s + '<li>%s = %s</li>\n' % (
                            x1,html_escape(repr(y1))[:100])
                    s = s + '</ul>\n'
                else:
                    s = s + '<li>%s = %s</li>\n' % (
                        x,html_escape(repr(y))[:100])
            s = s + '</ul>\n'
        else:
            s = s + '<li>%s = %s</li>\n' % (k,html_escape(repr(v))[:100])
    s = s + '</ul>\n'
    s = s + '<h1>sys.path</h1>\n<ul>\n'
    for p in sys.path:
        s = s + '<li>%s</li>\n' % p
    s = s + '</ul>\n<p>'
    return s
