import xmlrpclib, sys, os, types


def allow (default={}, **kw) :
    kw[None] = default
    r = {}
    for k,v in kw.iteritems() :
        if isinstance(v, list) or isinstance(v, tuple) :
            r[k] = dict(map(None, v, []))
        elif isinstance(v, dict) :
            r[k] = v
        else :
            r[k] = {v:None}
    return r

NOT_GIVEN = object()


class XMLRPCObject (object) :

    _security = {}

    _req = None

    def _lookup (self, req, name) :
        # Check permissions to attribute `name'
        _s = self._security.get
        groups = _s(name, NOT_GIVEN)
        if groups is NOT_GIVEN :
            groups = _s(None, NOT_GIVEN)
        if groups is NOT_GIVEN :
            raise xmlrpclib.Fault(31, "%s: permission denied" % name)
        q = 0
        for g in req.groups :
            if g in groups :
                q = 1
                break
        if not q :
            raise xmlrpclib.Fault(32, "%s: permission denied" % name)

        # Find attribute
        a = getattr(self, name, None)
        if a is None :
            raise xmlrpclib.Fault(21, "%s not found" % name)
        if isinstance(a, XMLRPCObject) :
            a._req = req
            return a
        if isinstance(a, types.MethodType) :
            return a
        raise xmlrpclib.Fault(22, "%s not found" % name)


class XMLRPCRequest :

    # Mandatory attributes
    user = None
    groups = None
    protocol = 'XML-RPC'
    app = None
    handler = None

    def __init__ (self, **kw) :
        self.__dict__.update(kw)


class XMLRPCHandler :

    input_limit = 1024 * 1024

    def __init__ (self, app, **kw) :
        self.app = app
        self.__dict__.update(kw)

    def sendDump (self, d) :
        o = self.stdout
        o.write("Content-type: text/xml\r\n")
        o.write("Content-length: %d\r\n\r\n" % len(d))
        o.write(d)
        #o.flush()

    def findMethod (self, req, name, params) :
        if not name :
            raise xmlrpclib.Fault(20, "%s not found" % name)
        a = self.app
        for np in name.split('.') :
            a = a._lookup(req, np)
        return a

    def createRequestObject (self) :
        return XMLRPCRequest()

    def handleRequest (self) :
        try :
            # parse request
            env = self.env
            for e in ["REQUEST_METHOD", "CONTENT_TYPE", "CONTENT_LENGTH"] :
                if not env.has_key(e) :
                    self.stderr.write("missing %r in XML-RPC request" % e)
                    sys.exit(-1)
            if env["REQUEST_METHOD"] != "POST" :
                raise xmlrpclib.Fault(10,
                    "Invalid XML-RPC request (post method required)")
            if env["CONTENT_TYPE"] != "text/xml" :
                raise xmlrpclib.Fault(11,
                    "Invalid XML-RPC request (text/xml content-type required)")
            cl = int(env["CONTENT_LENGTH"])
            if self.input_limit and cl > self.input_limit :
                raise xmlrpclib.Fault(12,
                    "Invalid XML-RPC request (input limit exceeded)")

            # parse request data
            data = self.stdin.read(cl)
            params, method_name = xmlrpclib.loads(data)

            # prepare request object
            req = self.createRequestObject()
            req.env = dict(env)

            self.pre_request(req)

            # find desired method and call it
            method = self.findMethod(req, method_name, params)
            res = method(*params)

            # return result
            d = xmlrpclib.dumps((res,), methodresponse=1)
            self.sendDump(d)

        except xmlrpclib.Fault, f :
            d = xmlrpclib.dumps(f)
            self.sendDump(d)

    def pre_request(self, req):
        pass

    def startCGI (self) :
        self.stdin, self.stdout, self.stderr = sys.stdin, sys.stdout, sys.stderr
        self.env = os.environ
        self.handleRequest()
        sys.exit(0)

    def startFCGI (self, max_req_count=3) :
        import fcgiapp
        self.app_term = 0
        req_count = 0
        while not self.app_term and req_count < max_req_count :
            req_count += 1
            try :
                inp, out, err, env = fcgiapp.Accept()
            except:
                sys.exit(1)
            self.stdin, self.stdout, self.stderr = inp, out, err
            self.env = env
            self.handleRequest()
            fcgiapp.Finish()

        sys.exit(0)
