# !!! this module must not import wm4, comp4 or other windmill modules
# !!! before deploy setup

# http://compoundthinking.com/blog/index.php/2008/07/08/setuptools-package-indexes-and-releases/
# http://peak.telecommunity.com/DevCenter/EasyInstall#creating-a-virtual-python
# http://pypi.python.org/pypi/workingenv.py
# http://pypi.python.org/pypi/virtualenv
# http://www.python.org/dev/peps/pep-0370/ Per user site-packages directory
# http://swapoff.org/wiki/blog/2007-09-09--distutils-support-for-a-workingenv-sandbox
# http://blog.ianbicking.org/2007/10/10/workingenv-is-dead-long-live-virtualenv/
# http://blog.ianbicking.org/2007/09/16/2-python-environment-experiments/

import sys, os, imp
from os.path import isdir, isfile, join

__all__ = [
  'get_deploy_params', 'setup_environ', 'setup', 'config_app', 'make_wsgi_app'
]


WINDMILL_HOME = os.environ.get('WINDMILL_HOME', '/home/app')


class DeployParams:
    python = sys.version_info[:2]
    windmill4 = None
    def __repr__(self):
        return 'DeployParams(python=%r, windmill4=%r)' % (
            self.python, self.windmill4
        )

class Version(tuple):
    
    def __new__(klass, ver):
        if isinstance(ver, basestring):
            ver = [int(v.strip()) for v in ver.split('.')]
        return tuple.__new__(klass, ver)


def get_deploy_params(app_name):
    pkgpath = None
    for pkg in app_name.split('.'):
        fd, path, (suffix, mode, module_type) = imp.find_module(pkg, pkgpath)
        pkgpath = [path]
    if module_type != imp.PKG_DIRECTORY:
        raise NotImplementedError(
            'unsupported module type', app_name, suffix, module_type
        )
    deploy = DeployParams()
    filename = join(path, 'deploy.txt')
    if isfile(filename):
        for i,l in enumerate(open(filename)):
            l = l.split('#')[0].strip()
            if not l:
                continue
            param, value = l.split('=',1)
            param = param.strip()
            value = value.strip()
            if param == 'python':
                deploy.python = Version(value)
            elif param == 'windmill4':
                deploy.windmill4 = Version(value)
            else:
                raise NotImplementedError(
                    'unknown parameter %s:%d' % (filename,i+1), param, value
                )
    return deploy

def setup_environ(deploy):
    if deploy.python is not None:
        for d,s in zip(deploy.python, sys.version_info):
            if d != s:
                raise RuntimeError(
                    'unmatched python version', deploy.python, sys.version_info
                )
    found = None
    if deploy.windmill4 is None:
        w = 'windmill4-dev-py%d.%d' % (deploy.python[0], deploy.python[1])
        d = join(WINDMILL_HOME, w, 'src')
        if isdir(d):
            found = d
            sys.path.insert(0, d)
    else:
        w = 'windmill4-%d.%d-py%d.%d' % (
                deploy.windmill4[0], deploy.windmill4[1],
                deploy.python[0], deploy.python[1],
        )
        d = join(WINDMILL_HOME, w, 'src')
        if isdir(d):
            found = d
            sys.path.insert(0, d)
    return found

def setup(app_name):
    deploy = get_deploy_params(app_name)
    return setup_environ(deploy)



def config_app(app_name):
    import wm4
    ver = wm4.VERSION[:2]
    if ver == (0,5):
        from wm4.api import config, naming, settings
        ini_files = ['pkgfile:wm4/core.ini']
        for ini in ('nondist.ini', 'app.ini'):
            iniFile = naming.lookup('pkgfile:%s/%s' % (app_name, ini)).address
            if isfile(iniFile):
                ini_files.append(iniFile)
                break
        config.loadConfig(*ini_files)
    else:
        from wm4.api import config, naming, settings

        anurl = naming.data_url(
            '[Global Settings]\n' +
            'wm4.settings.application_name = "%s"\n' % app_name)
        #settings.application_name(app_name)

        ini_files = ['pkgfile:wm4/core.ini', 'pkgfile:wm4.web/web.ini', anurl]
        for ini in ('nondist.ini', 'app.ini'):
            iniFile = naming.lookup('pkgfile:%s/%s' % (app_name, ini)).address
            if isfile(iniFile):
                ini_files.append(iniFile)
                break
        config.loadConfig(*ini_files)
    
def make_wsgi_app(app_name):
    from lib4.imports import importString
    app_mod = __import__(app_name)
    app_factory = getattr(app_mod, 'make_app', _make_wsgi_app)
    return app_factory(app_name)

def _make_wsgi_app(app_name):
    # XXX this should be in wm4.web.make_wsgi_app
    config_app(app_name)
    import wm4
    ver = wm4.VERSION[:2]
    if ver == (0,5):
        from wm4.api import settings
        return settings.web_app()
    else:
        from wm4.api import web
        return web.makeApp()


if __name__ == '__main__':
    app_name = sys.argv[1]
    p = get_deploy_params(app_name)
    print 'params', p
    found = setup_environ(p)
    print 'found', found
    print make_wsgi_app(app_name)
