from protocols.advice import getMRO
from lib4.symbols import NOT_GIVEN, Symbol
from wm4.exceptions import AlreadyRead, SettingConflict, MetadataNotFound
from wm4.utils.property_rules import *

from attributes import declareAttribute, declareClassMetadata

__all__ = ['ClassMetadata', 'AttributeMetadata']

NOOBJ = Symbol('NOOBJ',__name__)


class MetadataMeta(type):

    def __init__(klass, name, bases, cdict):
        klass._registry = {}

class ClassMetadata(object):

    __metaclass__ = MetadataMeta

    def __init__(self, rule):
        self.rule = IRule(rule)

    @classmethod
    def key(klass, clsObj, attr=None):
        return clsObj

    def register(self, clsObj, attr=None):
        key = self.key(clsObj, attr)
        reg = self.__class__._registry
        if key in reg:
            r,v = reg[key]
            if r is NOOBJ:
                raise AlreadyRead(key)
            else:
                raise SettingConflict(key, r, self.rule)
        reg[key] = (self.rule, NOOBJ)

    @classmethod
    def lookupKeys(klass, clsObj, attr=None):
        if clsObj != '*':
            for base in getMRO(clsObj):
                yield base
        yield '*'

    @classmethod
    def of(klass, clsObj, attr=None, default=NOT_GIVEN):
        registry = klass._registry
        regKey = klass.key(clsObj, attr)
        for key in klass.lookupKeys(clsObj, attr):
            if key in registry:
                rule, val = registry[key]
                if key == regKey and val is not NOOBJ:
                    return val
                if rule is not NOOBJ:
                    val = rule(regKey, key)
                    rrule = registry.get(regKey,(NOOBJ,NOOBJ))[0]
                    registry[regKey] = (rrule , val)
                    return val
            else:
                registry[key] = (NOOBJ, NOOBJ)
        if default is NOT_GIVEN:
            raise MetadataNotFound(klass, regKey)
        return default

declareClassMetadata.addMethod(ClassMetadata, lambda c, m: m.register(c))


class AttributeMetadata(ClassMetadata):

    @classmethod
    def key(klass, clsObj, attr):
        assert attr is not None
        return clsObj, attr

    @classmethod
    def lookupKeys(klass, clsObj, attr):
        mro = getMRO(clsObj)
        if clsObj != '*':
            for base in mro:
                yield base, attr
        yield '*', attr
        if attr != '*':
            if clsObj != '*':
                for base in mro:
                    yield base, '*'
            yield '*', '*'

declareAttribute.addMethod(AttributeMetadata, lambda c, a, m: m.register(c,a))
