from unittest import *
from wm4.api import *
from wm4.binding.api import *
from lib4.tests.test_properties import TestAttrProxy


class TestAttrProxy(TestAttrProxy):

    class Record(binding.Component):
        def __init__(self, **kw):
            self.__dict__.update(kw)
            binding.Component.__init__(self)

    class Rec(Record):
        ap1 = AttrProxy('a')
        ap2 = AttrProxy('r/a')
        pp1 = AttrProxy('ap1', [AttributeMetadata('AP1')])
        fp2 = AttrProxy('f/foo', AttributeMetadata('F/FOO'))

    class Foo(Record):
        def foo(self):
            return self

    def test90_Metadata(self):
        self.assertEqual(AttributeMetadata.of(self.Rec,'ap1',None), None)
        self.assertEqual(AttributeMetadata.of(self.Rec,'pp1'), 'AP1')
        self.assertEqual(AttributeMetadata.of(self.Rec,'fp2'), 'F/FOO')





class MyDescriptor(ClonedDescriptor):
    def computeValue(self, obj, instDict, attrName):
        return self, self.parentClass

class C(binding.Component):
    a = ClonedDescriptor()
    m = MyDescriptor()

class D(C):
    pass

class E(D):
    a = MyDescriptor()


class TestClonedDescriptor(TestCase):

    def test1_Cloning(self):
        assert C.a is not D.a
        assert C.m is not D.m

    def test2_ParentClass(self):
        assert C.a.parentClass is C
        assert C.m.parentClass is C
        assert D.a.parentClass is D
        assert D.m.parentClass is D

    def test3_RepetitiveRetrieving(self):
        a1, a2 = D.a, D.a
        assert a1 is a2
        m1, m2 = D.m, D.m
        assert m1 is m2

    def test4_ComputedValue(self):
        c, d  = C(), D()
        self.assertEqual(c.m,(C.m, C))
        self.assertEqual(d.m,(D.m, D))

    def test9_Metadata(self):
        pass


class TestMultiMake(TestCase):

    def test(self):
        class C(Component):
            a = b = c = MultiMake(lambda: dict(a=1,b=2,c=3))
        c = C()
        self.assertEqual(c.a, 1)
        self.failUnless('b' in c.__dict__)
        self.failUnless('c' in c.__dict__)
        self.assertEqual(c.b, 2)
        self.assertEqual(c.c, 3)


def suite():
    return TestSuite((
        makeSuite(TestAttrProxy),
        makeSuite(TestClonedDescriptor),
        makeSuite(TestMultiMake),
    ))

if __name__ == '__main__':
    main( defaultTest='suite' )
