from wm4.core import *
from lib4.properties import AttrProxy
from new import instancemethod

from once import Descriptor, Attribute, Make
from interfaces import IActiveDescriptor, IRecipe
from attributes import declareAttribute, initAttrs
from _once import get_dictionary

__all__ = [
    'Annotation','AttrProxy','Template','ClonedDescriptor','MultiMake',
]


class Annotation(object):

    __slots__ = 'subj', 'param'

    __metaclass__ = binding.ActiveClass

    def __new__(klass, subj, param=None):
        key = (klass, param)
        try:
            sdict = get_dictionary(subj)
        except AttributeError:
            sdict = subj.__dict__
        self = sdict.get(key)
        if self is None:
            self = super(Annotation,klass).__new__(klass)
            sdict[key] = self
        return self

    def __init__(self, subj, param=None):
        self.subj = subj
        self.param = param




class AttrProxy(AttrProxy):

    protocols.advise(instancesProvide=[IActiveDescriptor])

    upponAssembly = False

    offerAs = ()

    def __init__(self, attrPath, *metadata):
        self.metadata = metadata
        super(AttrProxy,self).__init__(attrPath)

    def activateInClass(self, klass, attrName):
        declareAttribute(klass, attrName, self.metadata)


def Template(template):
    return Make(
        lambda comp: instancemethod(template(comp), comp, comp.__class__)
    )



class ClonedDescriptor(Attribute):

    parentClass = None

    def activateInClass(self, klass, attr):
        ret = super(ClonedDescriptor, self).activateInClass(klass, attr)
        self.parentClass = klass
        return ret

    clonedAttrs = 'computeValue','noCache','onSet','offerAs','uponAssembly',\
                  'doc','adaptTo','suggestParent','metadata','attrName'

    def ofClass(self, attr, klass):
        if self.parentClass is klass:
            return self
        else:
            dCopy = dict([
                    (a, self.__dict__[a]) for a in self.clonedAttrs
                    if a in self.__dict__
            ])
            clone = self.__class__(**dCopy)
            clone.activateInClass(klass, attr)
            klass.__class_descriptors__[attr] = clone
            klass.__all_descriptors__[attr] = clone
            return clone



class MultiMake(Descriptor):

    protocols.advise(instancesProvide=[IActiveDescriptor])

    offerAs = ()
    uponAssembly = False
    metadata = None
    attrNames = ()

    def __init__(self, recipe, metadata=None, **kw):
        self.attrNames = []
        self.attrName = 'a'
        self.recipe = IRecipe(recipe)
        if metadata:
            kw['metadata'] = metadata
        initAttrs(self,kw.iteritems())

    def activateInClass(self, klass, attr):
        self.attrNames.append(attr)
        setattr(klass, attr, self.__class__(
                attrName=attr, attrNames=self.attrNames, recipe=self.recipe
        ))
        declareAttribute(klass,attr,self.metadata)
        return self

    def computeValue(self, obj, iDict, attr):
        resDict = self.recipe(obj, iDict, attr)
        objDict = get_dictionary(obj)
        for a in self.attrNames:
            v = resDict[a]
            if a == attr:
                continue
            objDict[a] = v
            self.onSet(obj, a, v)
        return resDict[attr]
