cdef extern from "Python.h":
    void *PyDict_GetItem(object dict,object key)
    int PyDict_SetItem(object dict,object key,object value)

from lib4 import symbols
from wm4.exceptions import PropertyNotFound

cdef object _NOT_GIVEN
cdef object _NOT_FOUND
cdef object _NOT_OBJECT
cdef object _None

_NOT_GIVEN = symbols.NOT_GIVEN
_NOT_FOUND = symbols.NOT_FOUND
_NOT_OBJECT = symbols.NOT_OBJECT
_None = None


cdef class BaseProperty:
    '''Base class for config properties'''

    cdef public object value
    cdef public object lookup_keys
    cdef public object reg_key

    def lookupRule(self, cfg, rKey, keys):
        data = cfg.data
        for fKey in keys:
            rule, value = data.setdefault(fKey, (_NOT_OBJECT, _NOT_OBJECT))
            if rule is not _NOT_OBJECT:
                return rule, fKey
        if cfg.parent:
            return self.lookupRule(cfg.parent, rKey, keys)
        else:
            return _NOT_OBJECT, _NOT_OBJECT

    def __call__(self, value=_NOT_GIVEN, default=_NOT_GIVEN):
        cdef object cfg
        cdef object val
        cdef object rule
        cdef object fkey
        cdef object data
        
        cfg = self.get_config()
        data = cfg.data

        if self.reg_key is _None:
            self.reg_key = self.regKey()

        if value is _NOT_GIVEN:

            if self.lookup_keys is _None:
                self.lookup_keys = self.lookupKeys()

            rule, val = data.setdefault(
                self.reg_key, (_NOT_OBJECT, _NOT_OBJECT)
            )
            if val is not _NOT_OBJECT:
                if val is _NOT_FOUND:
                    if default is _NOT_GIVEN:
                        raise PropertyNotFound(self.reg_key)
                    else:
                        return default
                return val

            fkey = self.reg_key
            if rule is _NOT_OBJECT:
                rule, fkey = self.lookupRule(cfg,self.reg_key,self.lookup_keys)

            if rule is _NOT_OBJECT:
                val = _NOT_FOUND
            else:
                val = self.makeValue(rule, self.reg_key, fkey)

            data[self.reg_key] = (data[self.reg_key][0], val)

            if val is _NOT_FOUND:
                if default is _NOT_GIVEN:
                    raise PropertyNotFound(self.reg_key)
                else:
                    return default
            else:
                return val
        else:
            cfg[self.reg_key] = (self.rule_interface(value), _NOT_OBJECT)


__all__ = ['BaseProperty']
