from wm4.api import *
from lib4.ini_parser import AbstractConfigParser
from lib4.imports import lazyModule, importString, importObject
from lib4.symbols import NOT_GIVEN
from wm4.utils.property_rules import *
import re, sys, os

from context import *
from utils import getStreamFactory


def SECTION_PARSER(p, default=NOT_GIVEN):
    p = 'wm4.config.section_parser.'+p
    return Property(p, default)

def CONFIG_LOADER(p, default=NOT_GIVEN):
    p = 'wm4.config.loader.'+p
    return Property(p, default)


isIdentifier = re.compile('^[A-Za-z_][A-Za-z0-9_]*$').match

def import_on_demand(parser, section, name, value, lineInfo):
    if not isIdentifier(name):
        raise SyntaxError(("Invalid module shortcut name", name, lineInfo))
    obj = importObject(eval(value, parser.globalDict, {}))
    parser.add_global(name, obj)


def resource_factories(parser, section, name, value, lineInfo):
    try:
        res = importString(name)
    except ImportError:
        raise SyntaxError(("Resource not found", lineInfo))
    fact = eval(value, parser.globalDict, locals())
    cfgs = list(Config.current().iterParents())
    res(fact)

def global_settings(parser, section, name, value, lineInfo):
    s = section.split()
    if len(s) > 2:
        names = [p+'.'+name for p in s[2:]]
    else:
        names = [name]
    for n in names:
        try:
            setting = importString(n)
        except ImportError:
            raise SyntaxError(("Setting not found", lineInfo))
        code = compile(value, lineInfo[0] or '<string>', 'eval')
        value = eval(code, parser.globalDict, locals())
        setting(value)

def load_settings_from(parser, section, name, value, lineInfo):
    loader = importObject(CONFIG_LOADER(name)())
    return loader(parser, value, lineInfo, includedFrom=parser)

def load_settings_from_file(parser, value, lineInfo, includedFrom=None):
    source = eval(value, parser.globalDict, locals())
    stream = getStreamFactory(source)
    subParser = ConfigReader()
    subParser.readStream(stream.open(), stream.address)

def load_settings_from_files(parser, value, lineInfo, includedFrom=None):
    for source in eval(value, parser.globalDict, locals()):
        stream = getStreamFactory(source)
        subParser = ConfigReader()
        subParser.readStream(stream.open(), stream.address)

def load_settings_from_mapping(parser, value, lineInfo, includedFrom=None):
    mapping = eval(value, parser.globalDict, locals())
    for k,v in mapping.items():
        Property(k)(v)


def plugin(parser, section, name, value, lineInfo):
    for spec in section.split()[1:]:
        reg, plug = spec.split(':')
        PluginsFor(reg).set(plug, True)
        Plugin(spec).setFactory(
            str(name),
            lambda: eval(value, parser.globalDict)
        )


class ConfigReader(AbstractConfigParser):

    def __init__(self, gdict=None, add_globals=None):
        if gdict is None:
            self.globalDict = dict(globals())
            self.globalDict.update(os.environ)
            self.globalDict.setdefault('PYTHONMODE', 'production')
        else:
            self.globalDict = gdict
        if add_globals:
            self.globalDict.update(add_globals)

    def parse(self, lineSource):
        cfg = Config.current()
        old = cfg.ignoreConflicts = True
        cfg.ignoreConflicts = True
        super(ConfigReader, self).parse(lineSource)
        cfg.ignoreConflicts = old

    def add_setting(self, section, name, value, lineInfo):
        p = Property(section+name)
        filename, lineno = lineInfo[:2]
        if not filename:
            filename = '<string>'
        if not lineno:
            lineno = 1
        p(ExpressionRule(
            value, self.globalDict,
            name=str(p.value)+'()', filename=filename, lineno=lineno
        ))

    def add_section(self, section, lines, lineInfo):
        try:
            if section is None:
                section='*'
            section = section.strip()
            if ' ' in section:
                sectName = []
                for p in section.lower().split():
                    try:
                        property_name(p)
                        if '*' in p:
                            break
                        sectName.append(p)
                    except:
                        break
                sectName = '.'.join(sectName)
                func = importObject(SECTION_PARSER(sectName)(default=None))
                if func is None:
                    raise SyntaxError(
                        ("Invalid section type", sectName, section, lineInfo))
                handler = lambda *args: func(self, *args)
            else:
                section = Property(section).asPrefix()
                handler = self.add_setting
            self.process_settings(section, lines, handler)
        except Exception,e:
            raise ValueError(lineInfo, (section, lines), str(e))

    def add_global(self, name, value):
        self.globalDict = self.globalDict.copy()
        self.globalDict[name]=value
