# XXX implementation is not thread-safe
# XXX observers should be registered "weakly"

from wm4.core import *


__all__ = ['Cell', 'Value', 'Input', 'InputAttr', 'ValueAttr']


computed_value = config.Global(name='computed_value')


class Cell(object):

    __slots__ = 'observers',

    def __init__(self):
        self.observers = []

    def subscribe(self, cb):
        self.observers.append(cb)

    def on_get(self):
        cell = computed_value()
        if cell is not None and cell not in self.observers:
            self.subscribe(cell.on_change)

    def on_set(self):
        for cb in self.observers:
            cb()
        self.observers[:] = []

    def on_change(self):
        pass


class Input(Cell):

    __slots__ = 'value',

    def __init__(self, value):
        super(Input, self).__init__()
        self.value = value

    def __call__(self, value=NOT_GIVEN):
        if value is NOT_GIVEN:
            self.on_get()
            return self.value
        else:
            old = self.value
            if value != old:
                self.value = value
                self.on_set()


class Value(Cell):

    __slots__ = 'rule', 'value'

    def __init__(self, rule):
        super(Value, self).__init__()
        self.rule = rule
        self.value = NOT_GIVEN

    def on_change(self):
        self.value = NOT_GIVEN
        self.on_set()

    def __call__(self):
        self.on_get()
        if self.value is NOT_GIVEN:
            self.value = val = config.with_(computed_value(self), self.rule)
        return self.value





class InputAttr(object):

    def __init__(self, default):
        self.default = default

    def __get__(self, obj, typ=None):
        if obj is None:
            return self
        dct = obj.__dict__
        cell = dct.get(self)
        if cell is None:
            cell = dct[self] = Input(self.default)
        return cell()

    def __set__(self, obj, value):
        dct = obj.__dict__
        cell = dct.get(self)
        if cell is None:
            cell = dct[self] = Input(self.default)
        cell(value)


class ValueAttrImpl(Value):

    def __call__(self, obj):
        self.on_get()
        if self.value is NOT_GIVEN:
            self.value = val = config.with_(computed_value(self),self.rule,obj)
        return self.value


class ValueAttr(object):

    def __init__(self, rule):
        self.rule = rule

    def __get__(self, obj, typ=None):
        if obj is None:
            return self
        dct = obj.__dict__
        cell = dct.get(self)
        if cell is None:
            cell = dct[self] = ValueAttrImpl(self.rule)
        return cell(obj)
