from wm4.api import *
from types import SliceType

from events import Event

__all__ = ['ListChanged', 'DictChanged', 'ObservableList', 'ObservableDict']

class ListChanged(Event):

    __slots__ = 'added', 'removed'

    def __init__(self, sender, added, removed):
        self.added, self.removed = added, removed
        super(ListChanged,self).__init__(sender)

class DictChanged(Event):

    __slots__ = 'added', 'removed'

    def __init__(self, sender, added, removed):
        self.added, self.removed = added, removed
        super(DictChanged,self).__init__(sender)



class ObservableList(object):

    def __init__(self):
        self.__list_data = []

    def __data__(self):
        return self.__list_data

    def __len__(self):
        return len(self.__list_data)

    def __iter__(self):
        return iter(self.__list_data)

    def __getitem__(self, i):
        return self.__list_data[i]

    def _makeDiff(self, old):
        data = self.__list_data
        newLen = len(data)
        oldLen = len(old)

        added = []
        removed = []
        for i in xrange(min(newLen, oldLen)):
            o = old[i]
            n = data[i]
            if o != n:
                added.append((i,n))
                removed.append((i,o))
        if newLen > oldLen:
            for i in xrange(oldLen, newLen):
                added.append((i,data[i]))
        elif newLen < oldLen:
            for i in xrange(newLen, oldLen):
                removed.append((i,old[i]))
        return added, removed

    def __setitem__(self, i, v):
        data = self.__list_data
        oldLen = len(data)
        if isinstance(i, SliceType):
            old = list(data)
            data[i] = v
            added, removed = self._makeDiff(old)
            if added or removed:
                ListChanged(self, added, removed)
        else:
            if i < 0:
                i = oldLen + i
            o = data[i]
            if o != v:
                removed = [(i,o)]
                data[i] = v
                added = [(i,v)]
                ListChanged(self, added, removed)

    def __delitem__(self, i):
        data = self.__list_data
        old = list(data)
        del data[i]
        added, removed = self._makeDiff(old)
        if added or removed:
            ListChanged(self, added, removed)

    def append(self, v):
        self.__list_data.append(v)
        ListChanged(self, [(len(self.__list_data)-1,v)], [])

    def extend(self, v):
        v = list(v)
        if v:
            l = len(self.__list_data)
            added = [(i+l,x) for i,x in enumerate(v)]
            self.__list_data.extend(v)
            ListChanged(self, added, [])

    def insert(self, i, v):
        if i < 0:
            added = (len(self.__list_data)+i, v)
        else:
            added = (i,v)
        self.__list_data.insert(i,v)
        ListChanged(self, [added], [])




class ObservableDict(object):

    def __init__(self): self.__dict_data = {}

    def __data__(self): return self.__dict_data

    def __contains__(self,key): return self.__dict_data.has_key(key)
    def has_key(self, key): return self.__dict_data.has_key(key)

    def __len__(self): return len(self.__dict_data)
    def __iter__(self): return iter(self.__dict_data)

    def keys(self): return self.__dict_data.keys()
    def values(self): return self.__dict_data.values()
    def items(self): return self.__dict_data.items()
    def iterkeys(self): return self.__dict_data.iterkeys()
    def itervalues(self): return self.__dict_data.itervalues()
    def iteritems(self): return self.__dict_data.iteritems()

    def get(self, key, default=None):
        return self.__dict_data.get(key, default)

    def __getitem__(self, key):
        return self.__dict_data[key]

    def __setitem__(self, key, value):
        dict = self.__dict_data
        ov = dict.get(key,self)
        if ov is self: removed = {}
        else: removed = {key:ov}
        dict[key] = value
        DictChanged(self, {key:value}, removed)

    def __delitem__(self, key):
        dict = self.__dict_data
        if key in dict:
            value = dict[key]
            del dict[key]
            DictChanged(self, {}, {key:value})
        else:
            raise KeyError(key)

    def setdefault(self, key, value):
        dict = self.__dict_data
        if not key in dict:
            dict[key] = value
            DictChanged(self, {key:value}, {})
            return value
        else:
            return dict[key]

    def update(self, other):
        removed = {}
        added = {}
        dict = self.__dict_data
        for k,nv in other.items():
            ov = dict.get(k,self)
            if ov is self:
                added[k] = dict[k] = nv
            elif ov == nv:
                pass
            else:
                removed[k] = ov
                added[k] = dict[k] = nv
        DictChanged(self, added, removed)

    def clear(self):
        removed = self.__dict_data
        self.__dict_data = {}
        DictChanged(self, {}, removed)

    def pop(self, key, *args):
        dict = self.__dict_data
        value = dict.get(key,self)
        if value is self:
            removed = {}
        else:
            removed = {key:value}
        value = dict.pop(key, *args)
        DictChanged(self, {}, removed)
        return value

    def popitem(self):
        k,v = self.__dict_data.popitem()
        DictChanged(self, {}, {k:v})
        return k,v
