# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.events.api import *
from itertools import count

class TestCells(TestCase):

    def test10_Input(self):
        i = Input(1)
        self.assertEqual(i(), 1)
        self.assertEqual(i(), 1)
        i('foo')
        self.assertEqual(i(), 'foo')
        self.assertEqual(i(), 'foo')
        i(None)
        self.assertEqual(i(), None)
        self.assertEqual(i(), None)

    def test12_InputIsObservable(self):
        c = count().next
        i = Input(1)
        l = [None]
        i.subscribe(lambda: l.__setitem__(0,(i(), c())))
        # callback is called only once
        i(2); self.assertEqual(l[0], (2,0))
        i(2); self.assertEqual(l[0], (2,0))
        # subscription is not persistent
        i(3); self.assertEqual(l[0], (2,0))

    def test20_Value(self):
        x, y = Input(1), Input(2)
        v = Value(lambda: x() + y() + 3)
        self.assertEqual(v(), 6)

    def test25_ComputedValueIsCached(self):
        c = count().next
        x = Input(1)
        v = Value(lambda: x() + c())
        self.assertEqual(v(), 1)
        self.assertEqual(v(), 1)

    def test30_ValueIsRecomputedOnInputChange(self):
        c = count().next
        x = Input(1)
        v = Value(lambda: x() + c())
        self.assertEqual(v(), 1)
        self.assertEqual(v(), 1)
        x(2)
        self.assertEqual(v(), 3)
        self.assertEqual(v(), 3)

    def test31_ValueIsRecomputedOnInputChange(self):
        x = Input([Input(1)])
        v = Value(lambda: x()[0]() * 10)
        self.assertEqual(v(), 10)
        x()[0](2)
        self.assertEqual(v(), 20)
        x([Input(3)])
        self.assertEqual(v(), 30)

    def test32_ValueIsNotRecomputedWhenNewValueIsEqualToOld(self):
        c = count().next
        x = Input(1)
        v = Value(lambda: x() + c())
        self.assertEqual(v(), 1)
        x(2)
        self.assertEqual(v(), 3)
        x(2)
        self.assertEqual(v(), 3)

    def test35_ValueIsDependentOnlyOnNeededInputs(self):
        c = count().next
        x, y = Input(1), Input(2)
        v = Value(lambda: not c() and x() or y())
        self.assertEqual(v(), 1)
        y(20)
        self.assertEqual(v(), 1)
        x(10)
        self.assertEqual(v(), 20)
        x(100)
        self.assertEqual(v(), 20)

    def test37_DependencyCascade(self):
        x = Input(1)
        u = Value(lambda: x() * 10)
        v = Value(lambda: u() + 2)
        self.assertEqual(v(), 12)
        x(4)
        self.assertEqual(v(), 42)


class C(object):
    i = InputAttr(1)
    j = InputAttr(2)

class D(object):
    i = InputAttr(1)
    j = InputAttr(2)
    v = ValueAttr(lambda self: self.i * 10)
    w = ValueAttr(lambda self: self.v + 2)

class TestCellAttrs(TestCase):

    def test10_InputAttr(self):
        o = C()
        self.assertEqual(o.i, 1)
        self.assertEqual(o.j, 2)
        o.j = 3
        self.assertEqual(o.i, 1)
        self.assertEqual(o.j, 3)

    def test12_InputAttrIndependence(self):
        o1 = C()
        o2 = C()
        o2.j = 3
        self.assertEqual(o1.i, 1)
        self.assertEqual(o1.j, 2)
        self.assertEqual(o2.i, 1)
        self.assertEqual(o2.j, 3)


    def test20_ValueAttr(self):
        o = D()
        self.assertEqual(o.i, 1)
        self.assertEqual(o.v, 10)
        self.assertEqual(o.w, 12)
        o.i = 4
        self.assertEqual(o.i, 4)
        self.assertEqual(o.v, 40)
        self.assertEqual(o.w, 42)


def suite():
    return TestSuite((
        makeSuite(TestCells),
        makeSuite(TestCellAttrs),
    ))



if __name__ == '__main__':

    main(defaultTest='suite')
