# -*- coding: ISO-8859-2 -*-
from unittest import *
from wm4.api import *
from wm4.i18n.translators import TRANSLATOR

_ = i18n.MessageIDFactory('wm4.i18n.tests')


class TestGettextTranslator(TestCase):

    def setUp(self):
        self.state = config.snapshot()
        config.loadConfig()
        self.ts = settings.translation_service()
        self.gttrans = TRANSLATOR('wm4.i18n.tests')()

    def tearDown(self):
        config.swap(self.state)

    def translate(self, txt, lang=None, default=None):
        return self.gttrans.translate(self.ts, txt, lang=lang, default=default)

    def test01_TranslatorCaching(self):
        t1 = TRANSLATOR('wm4.i18n.tests')()
        t2 = TRANSLATOR('wm4.i18n.tests')()
        assert t1 is t2 is self.gttrans

    def test10_MessageInCatalog(self):
        trans = self.translate
        self.assertEqual(trans('service',lang='cs'), u'sluba')
        self.assertEqual(trans('house',lang='cs'), u'dm')
        self.assertEqual(trans('house',lang='cs',default='H'), u'dm')

    def test11_MessageNotInCatalog(self):
        trans = self.translate
        self.assertEqual(trans('jazz',lang='cs'), u'jazz')
        self.assertEqual(trans('jazz',lang='cs',default=u'J'), u'J')

    def test12_MangledLanguageSpec(self):
        trans = self.translate
        self.assertEqual(trans('house',lang='cs_CZ'), u'dm')

    def test13_NonexistingCatalog(self):
        trans = self.translate
        self.assertEqual(trans('house',lang='en'), u'house')
        self.assertEqual(trans('house',lang='de'), u'house')
        self.assertEqual(trans('house',lang='de',default='H'), u'H')

    def test14_NoLanguage(self):
        trans = self.translate
        self.assertEqual(trans('house',lang=None), u'house')
        self.assertEqual(trans('house',lang=None,default='H'), u'H')


class TestTranslationService(TestCase):

    def setUp(self):
        self.state = config.snapshot()
        config.loadConfig()
        self.ts = settings.translation_service()

    def tearDown(self):
        config.swap(self.state)

    def translate(self, txt, lang=None, default=None):
        return self.ts.translate(
            txt, lang=lang, default=default, domain='wm4.i18n.tests'
        )

    def test01_ResourceCaching(self):
        t1 = settings.translation_service()
        t2 = settings.translation_service()
        assert t1 is t2 is self.ts

    def test11_MessageInCatalog(self):
        trans = self.translate
        self.assertEqual(trans('service',lang='cs'), u'sluba')
        self.assertEqual(trans('house',lang='cs'), u'dm')
        self.assertEqual(trans('house',lang='cs',default='H'), u'dm')

    def test12_MessageNotInCatalog(self):
        trans = self.translate
        self.assertEqual(trans('jazz',lang='cs'), u'jazz')
        self.assertEqual(trans('jazz',lang='cs',default=u'J'), u'J')

    def test13_MangledLanguageSpec(self):
        trans = self.translate
        self.assertEqual(trans('house',lang='cs_CZ'), u'dm')

    def test14_NonexistingCatalog(self):
        trans = self.translate
        self.assertEqual(trans('house',lang='en'), u'house')
        self.assertEqual(trans('house',lang='de'), u'house')
        self.assertEqual(trans('house',lang='de',default='H'), u'H')

    def test15_NoLanguage(self):
        trans = self.translate
        self.assertEqual(trans('house',lang=None), u'house')
        self.assertEqual(trans('house',lang=None,default='H'), u'H')

    def test21_MessageInCatalog(self):
        trans = self.translate
        self.assertEqual(trans(_('service'),lang='cs'), u'sluba')
        self.assertEqual(trans(_('house'),lang='cs'), u'dm')
        self.assertEqual(trans(_('house', default='H'),lang='cs'), u'dm')

    def test22_MessageNotInCatalog(self):
        trans = self.translate
        self.assertEqual(trans(_('jazz'),lang='cs'), u'jazz')
        self.assertEqual(trans(_('jazz',default='J'),lang='cs'), u'J')

    def test23_MangledLanguageSpec(self):
        trans = self.translate
        self.assertEqual(trans(_('house'),lang='cs_CZ'), u'dm')

    def test24_NonexistingCatalog(self):
        trans = self.translate
        self.assertEqual(trans(_('house'),lang='en'), u'house')
        self.assertEqual(trans(_('house'),lang='de'), u'house')
        self.assertEqual(trans(_('house',default='H'),lang='de'), u'H')

    def test25_NoLanguage(self):
        trans = self.translate
        self.assertEqual(trans(_('house'),lang=None), u'house')
        self.assertEqual(trans(_('house',default='H'),lang=None), u'H')

    def test30_TranslationWithMapping(self):
        trans = self.translate
        msg = _(u'number $no') % {'no':42}
        self.assertEqual(trans(msg, lang='cs'), u'slo 42')

    def test31_MappingSubstitution(self):
        trans = self.translate
        msg = _(u'$count records found')
        msg = msg % dict(count=12)
        self.assertEqual(trans(msg), u'12 records found')
        # XXX singular/plural

    def test32_MappingTranslation(self):
        trans = self.translate
        msg = _(u'xxx $val yyy')
        msg = msg % dict(val=_(u'house'))
        self.assertEqual(trans(msg,lang='cs'), u'xxx dm yyy')





def suite():
    return TestSuite((
        makeSuite(TestGettextTranslator),
        makeSuite(TestTranslationService),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
