from protocols import Interface, Attribute

__all__ = [
    'IQuerySrc', 'IExpression', 'IQueryable', 'IVariable', 'IQuery', 'IConnection'
]


class IQuerySrc(Interface):
    '''Query source code'''

    source = Attribute('query source code')
    namespaces = Attribute('bound globals, locals or None')


class IExpression(Interface):
    '''Abstract expression'''

    hashcmp = Attribute('Immutable value for comparing and hashing')

    type = Attribute('Expression result type')

    relvars = Attribute('Dictionary of path:IRelVar items')

    aggregates = Attribute('Set of aggregates')


class IQueryable(Interface):
    '''Object that can be queried'''

    def alias(assign):
        '''Returns IVariable instance for given assignment'''


class IVariable(IExpression):
    '''Queryable interface object (returns set of some instances)'''

    joins = Attribute('Additional joins as sequence of (relvar, cond)')



class IQuery(IExpression):
    '''Query representation (select, update, delete)'''



class IConnection(Interface):

    def transform(exp):
        '''Transform expression'''
