from wm4.api import *
from wm4.query.api import *
from datetime import datetime, date

__all__ = [
    'subvar', 'User', 'Group', 'UserGroup', 'UGStat', 'Folder',
    'u','g','ug','ugs', 'fldr',
    'u_uid','u_login','u_name','g_gid',
    'C', 'V', 'a', 'b', 'c',
]

def subvar(obj, path):
    '''For testing purposes'''
    for a in path.split('.'):
        if a.startswith('$'):
            a = a[1:]
            t = '$'
        elif a.startswith('^'):
            a = a[1:]
            t = '^'
        else:
            t = ''
        try:
            map = obj.pmap.features[a]
        except KeyError:
            raise KeyError('mapping %r has no attribute %s' % (obj.pmap,a))
        #if (obj.parent and isinstance(obj,RelVar)
        #    and obj.var in obj.parent.pmap.pkey
        #):
        #    obj = PKeyVar(obj.var, obj.pmap, obj.parent)
        if isinstance(map, (schema.StructuredTypeConfig,schema.Reference)):
            map = schema.TypeConfig(map.typeObj)()
            if t == '$':
                obj = ClonedVar(a, map, obj)
            elif t == '^':
                obj = PKeyVar(a, map, obj)
            else:
                obj = RelVar(a, map, obj)
        else:
            obj = AttrVar(a, map, obj)
    return obj


class User(schema.Entity):
    uid = schema.Attribute(int)
    name = schema.Attribute(unicode)
    login = schema.Attribute(str, schema.Required)
    mtime = schema.Attribute(datetime)

    groups = schema.QueryFeature(
        "[g for ug in UserGroup for g  in Group "
        "    if ug.user == obj and ug.group == g]"
    )

    ugroups = schema.ReferenceEnd(
            '%s.UserGroup' % __name__, 'user',
            schema.ReferenceEndOrder('group.gid'),
    )

    binding.metadata(schema.PrimaryKey('uid'), schema.Unique('login'))

class Group(schema.Entity):
    gid = schema.Attribute(int)
    name = schema.Attribute(unicode)

    users = schema.QueryFeature(
        "[u for ug in UserGroup for u  in User "
        "    if ug.group == obj and ug.user == u]"
    )

    binding.metadata(schema.PrimaryKey('gid'))

class UserGroup(schema.Entity):
    user = schema.Reference(User)
    group = schema.Reference(Group)

    binding.metadata(schema.PrimaryKey('user','group'))


class UGStat(schema.Entity):
    ug = schema.Reference(UserGroup)
    term = schema.Attribute(date)
    value = schema.Attribute(int)

    binding.metadata(schema.PrimaryKey('ug','term'))


class Folder(schema.Entity):
    name = schema.Attribute(str)
    parent = schema.Reference(__name__+'.Folder')
    binding.metadata(schema.PrimaryKey('name'))



u = IQueryable(User).alias('u')
g = IQueryable(Group).alias('g')
ug = IQueryable(UserGroup).alias('ug')
ugs = IQueryable(UGStat).alias('ugs')
fldr = IQueryable(Folder).alias('fldr')

u_uid = subvar(u,'uid')
u_login = subvar(u,'login')
u_name = subvar(u,'name')
g_gid = subvar(g,'gid')

C = Const

class V(Abstract):

    def __init__(self, var):
        self.var = str(var)

    def __repr__(self):
        return 'V(%s)' % self.var

    relvars = binding.Make(lambda s: {s.var:s})

    hashcmp = binding.Make(lambda s: (s.__class__, s.var))

a = V('a')
b = V('b')
c = V('c')
