# -*- coding: ISO-8859-2 -*-

from unittest import *
import sys
from wm4.api import *
from wm4.query.api import *
from wm4.query.builder import parse

from app import *

gv = subvar

def f(a,b,*c, **d):
    return (a,b,c,d)

class TestBase(TestCase):

    def setUp(self):
        self.state = config.snapshot()
        config.loadConfig()
        settings.storage_url('mockdb:')
        self.con = settings.storage_connection()

    def tearDown(self):
        config.swap(self.state)

    def p(self, expr, up=1):
        f = sys._getframe(up)
        return parse(
            expr, self.con, f.f_locals, f.f_globals, f.f_builtins
        )

    def q(self, expr):
        return IQuery(self.p(expr, up=2))



class TestExpressions(TestBase):

    def test10_Constant(self):
        self.assertEqual(self.p('1'), C(1))
        self.assertEqual(self.p("'1'"), C('1'))
        self.assertEqual(self.p("'foo'"), C('foo'))

    def test11_Variable(self):
        a = 1
        self.assertEqual(self.p('a'), C(1)) # local
        self.assertEqual(self.p('sys'), C(sys)) # global
        self.assertRaises(NameError, self.p, 'neex')

    def test20_UnaryOps(self):
        self.assertEqual(self.p('-a'), NEG(a))
        self.assertEqual(self.p('-1'), C(-1))
        self.assertEqual(self.p('+a'), POS(a))
        self.assertEqual(self.p('~a'), INV(a))
        self.assertEqual(self.p('not a'), NOT(a))

    def test21_ArithmeticOps(self):
        self.assertEqual(self.p('a+b'), ADD(a,b))
        self.assertEqual(self.p('a+b+c'), ADD(ADD(a,b),c))
        self.assertEqual(self.p('a-b'), SUB(a,b))
        self.assertEqual(self.p('a*b'), MUL(a,b))
        self.assertEqual(self.p('a/b'), DIV(a,b))
        self.assertEqual(self.p('a%b'), MOD(a,b))
        self.assertEqual(self.p('a//b'), FLOOR(a,b))
        self.assertEqual(self.p('a**b'), POW(a,b))
        self.assertEqual(self.p('a<<b'), LSHIFT(a,b))
        self.assertEqual(self.p('a>>b'), RSHIFT(a,b))
        self.assertEqual(self.p('a+b*c**a'), ADD(a,MUL(b,POW(c,a))))
        self.assertEqual(self.p('a+1/0.4'), ADD(a,C(2.5)))

    def test23_BitwiseOps(self):
        self.assertEqual(self.p('a&b'), BAND(a,b))
        self.assertEqual(self.p('a&b&c'), BAND(BAND(a,b),c))
        self.assertEqual(self.p('a&b|c'), BOR(BAND(a,b),c))
        self.assertEqual(self.p('a^b|c'), BOR(BXOR(a,b),c))
        self.assertEqual(self.p('(1^2|3) & 0xfe'), C(2))

    def test25_ComparisonOps(self):
        self.assertEqual(self.p('a<b'),  LT(a,b))
        self.assertEqual(self.p('a<=b'), LE(a,b))
        self.assertEqual(self.p('a==b'), EQ(a,b))
        self.assertEqual(self.p('a<>b'), NE(a,b))
        self.assertEqual(self.p('a!=b'), NE(a,b))
        self.assertEqual(self.p('a>=b'), GE(a,b))
        self.assertEqual(self.p('a>b'), GT(a,b))
        self.assertEqual(self.p('a>b>c'), AND(GT(a,b),GT(b,c)))
        self.assertEqual(self.p('a>b<c'), AND(GT(a,b),LT(b,c)))
        self.assertEqual(self.p('a<1'), LT(a,C(1)))
        self.assertEqual(self.p('0<1'), C(True))
        self.assertEqual(self.p('0>1'), C(False))
        self.assertEqual(self.p('0==1'), C(False))
        self.assertEqual(self.p('0!=1'), C(True))

    def test27_LogicalOps(self):
        self.assertEqual(self.p('a and b'), AND(a,b))
        self.assertEqual(self.p('a and b and c'), AND(a,b,c))
        self.assertEqual(self.p('a and (b and c)'), AND(a,b,c))
        self.assertEqual(self.p('a or b'), OR(a,b))
        self.assertEqual(self.p('a or b or c'), OR(a,b,c))
        self.assertEqual(self.p('a and b or c'), OR(AND(a,b),c))
        self.assertEqual(self.p('a and not b'), AND(a,NOT(b)))
        self.assertEqual(self.p('a and 1'), a)
        self.assertEqual(self.p('a and 0'), C(0))
        self.assertEqual(self.p('a and 1 and 2 and b'), AND(a,b))
        self.assertEqual(self.p('a or 0 or b'), OR(a,b))

    def test30_Getattr(self):
        self.assertEqual(self.p('a.x'), Getattr(a,'x'))
        self.assertEqual(self.p('a.x.y'), Getattr(Getattr(a,'x'),'y'))

    def test32_Tuple(self):
        self.assertEqual(self.p('a,b'), Tuple(a,b))
        self.assertEqual(self.p('a,1'), Tuple(a,C(1)))
        self.assertEqual(self.p('(1,)'), C((1,)))
        self.assertEqual(self.p('1,2,3'), C((1,2,3)))
        self.assertEqual(self.p('(1,2,3)'), C((1,2,3)))

    def test33_List(self):
        self.assertEqual(self.p('[a,b]'), List(a,b))
        self.assertEqual(self.p('[a,1]'), List(a,C(1)))
        self.assertEqual(self.p('[1]'), C([1]))
        self.assertEqual(self.p('[1,2,3]'), C([1,2,3]))

    def test34_Dict(self):
        self.assertEqual(self.p('{}'), C({}))
        self.assertEqual(self.p('{a:1}'), Dict(Tuple(a,C(1))))
        self.assertEqual(self.p('{2:1}'), C({2:1}))

    def test40_CallWithConstantArgs(self):
        self.assertEqual(self.p('len("ab")'), C(2))
        self.assertEqual(self.p('f(1,2)'), C((1,2,(),{})))
        self.assertEqual(self.p('f(a=1, b=2)'), C((1,2,(),{})))
        self.assertEqual(self.p('f(1, 2, 3, 4)'), C((1,2,(3,4),{})))
        va = (3,4)
        self.assertEqual(self.p('f(1, 2, *va)'), C((1,2,(3,4),{})))
        ka = dict(x=6, y=7)
        self.assertEqual(self.p('f(1, 2, **ka)'), C((1,2,(),{'x':6,'y':7})))
        self.assertEqual(
            self.p('f(1, 2, *va, **ka)'), C((1,2,(3,4),{'x':6,'y':7}))
        )
        f = self.p('Folder(name="root")')
        self.assertEqual(f.__class__, Const)
        self.assertEqual(f.value.__class__, Folder)
        self.assertEqual(f.value.name, 'root')
        self.assertEqual(f.value.parent, None)

    def test41_CallWithVariableArgs(self):
        self.assertRaises(NotImplementedError, self.p, 'f(*a)', up=2)
        self.assertRaises(NotImplementedError, self.p, 'f(**a)', up=2)
        self.assertRaises(NotImplementedError, self.p, 'a()', up=2)

        self.assertEqual(self.p('f(a, b, x=12)'), Call(f, (a,b), dict(x=C(12))))
        self.assertEqual(self.p('f(a, x=b)'), Call(f, (a,), dict(x=b)))
        va = (3,4)
        ka = dict(x=6, y=7)
        self.assertEqual(self.p('f(a, *va)'), Call(f, (a,C(3),C(4))))
        self.assertEqual(self.p('f(a,**ka)'), Call(f,(a,),dict(x=C(6),y=C(7))))

    def test42_CallTransformation(self):
        self.assertEqual(self.p('len(u_login)'), Length(u_login))

    def test45_ExplicitCast(self):
        self.assertEqual(self.p('Cast(a,str)'), Cast(a, C(str)))

    def test46_TransformedCast(self):
        self.assertEqual(self.p('str(a)'), Cast(a, C(str)))
        self.assertEqual(self.p('int(a)'), Cast(a, C(int)))

    def test50_Getitem2Substr(self):
        a = Const('ABCDEF')
        self.assertEqual(
            self.p('"ABCDEF"[0]'), Substr(a, C(1), C(1)))
        self.assertEqual(
            self.p('"ABCDEF"[1]'), Substr(a, C(2), C(1)))
        self.assertEqual(
            self.p('"ABCDEF"[-2]'), Substr(a, SUB(Length(a),C(1)), C(1)))
        # ??? self.assertEqual(self.p('"ABCDEF"[...]'),Getitem(a, Const(Ellipsis)))

    def test51_Getslice2Substr(self):
        a = Const('ABCDEF')
        self.assertEqual(self.p('"ABCDEF"[:]'),
            a)
        self.assertEqual(self.p('"ABCDEF"[1:2]'),
            Substr(a, C(2), C(1)))
        self.assertEqual(self.p('"ABCDEF"[1:4]'),
            Substr(a, C(2), C(3)))
        self.assertEqual(self.p('"ABCDEF"[6:10]'),
            Substr(a, C(7), C(4)))
        self.assertEqual(self.p('"ABCDEF"[0:3]'),
            Substr(a, C(1), C(3)))
        self.assertEqual(self.p('"ABCDEF"[0:]'),
            Substr(a, C(1), C(None)))
        self.assertEqual(self.p('"ABCDEF"[1:]'),
            Substr(a, C(2), C(None)))
        self.assertEqual(self.p('"ABCDEF"[-2:]'),
            Substr(a, SUB(Length(a),C(1)), C(None)))
        self.assertEqual(self.p('"ABCDEF"[-1:]'),
            Substr(a, Length(a), C(None)))
        self.assertEqual(self.p('"ABCDEF"[:3]'),
            Substr(a, C(1), C(3)))
        self.assertEqual(self.p('"ABCDEF"[:-2]'),
            Substr(a, C(1), SUB(Length(a),C(2))))

    def test52_Sliceobj2Substr(self):
        a = Const('ABCDEF')
        self.assertEqual(self.p('"ABCDEF"[::]'), a)
        self.assertEqual(self.p('"ABCDEF"[::1]'), a)
        self.assertEqual(self.p('"ABCDEF"[::None]'), a)
        self.assertEqual(self.p('"ABCDEF"[1:2:1]'),
            Substr(a, C(2), C(1)))
        self.assertEqual(self.p('"ABCDEF"[1:2:None]'),
            Substr(a, C(2), C(1)))
        self.assertEqual(self.p('"ABCDEF"[1::1]'),
            Substr(a, C(2), C(None)))
        self.assertEqual(self.p('"ABCDEF"[:-2:]'),
            Substr(a, C(1), SUB(Length(a),C(2))))

    def test60_Alias(self):
        self.assertEqual(self.p('As(a+1,"x")'), As(ADD(a,C(1)),C('x')))

    def test70_Case(self):
        c = self.p('Case((u.uid == 1, 42), (u.uid > 1, 43), default=41)')
        self.assertEqual(c, Case(Tuple(EQ(u_uid,C(1)), C(42)), Tuple(GT(u_uid,C(1)), C(43)), default=C(41)))






class TestVariables(TestBase):

    def test1_Typ(self):
        self.assertEqual(gv(ug,'user').typ, User)
        self.assertEqual(gv(ug,'$user').typ, User)
        self.assertEqual(gv(ug,'user.uid').typ, int)
        self.assertEqual(gv(ug,'$user.uid').typ, int)
        self.assertEqual(gv(ug,'group').typ, Group)

    def test2_Comparison(self):
        u1 = gv(ug,'user')
        u2 = gv(ug,'user')
        self.assertEqual(u1, u2)
        u3 = gv(ug,'$user')
        self.assertNotEqual(u1, u3)

    def test3_StorageNode(self):
        self.assertEqual(gv(ug,'group').storage_node, gv(ug,'group'))
        self.assertEqual(gv(ug,'^group.gid').storage_node, ug)
        self.assertEqual(gv(ug,'$group').storage_node, ug)
        self.assertEqual(gv(ug,'$group.gid').storage_node, ug)
        self.assertEqual(gv(ugs,'ug.group').storage_node, gv(ugs,'ug.group'))
        self.assertEqual(gv(ugs,'^ug.^group.gid').storage_node, ugs)
        self.assertEqual(gv(ugs,'^ug.$group').storage_node, ugs)
        self.assertEqual(gv(ugs,'^ug.$group.gid').storage_node, ugs)
        self.assertEqual(gv(ugs,'$ug.$group').storage_node, ugs)
        self.assertEqual(gv(ugs,'$ug.$group.gid').storage_node, ugs)







class TestQueries(TestBase):

    def test01_Comparison(self):
        self.assertEqual(u, IQueryable(User).alias('u'))
        self.assertEqual(gv(u,'uid'), u_uid)
        self.assertNotEqual(gv(u,'login'), u_uid)

    def test10_Simplest(self):
        self.assertEqual(
            self.q('[u for u in User]'),
            Select(u, [('u',u,None,None)])
        )

    def test11_SimpleWithCondition(self):
        self.assertEqual(
            self.q('[u for u in User if u.uid == 1]'),
            Select(u, [('u',u,None,None)], where=EQ(u_uid,C(1)))
        )

    def test12_SimpleScalar(self):
        self.assertEqual(
            self.q('[u.uid for u in User]'),
            Select(u_uid, [('u',u,None,None)])
        )

    def test13_SimpleTuple(self):
        self.assertEqual(
            self.q('[(u.uid, u.login) for u in User]'),
            Select(Tuple(u_uid, u_login), [('u',u,None,None)])
        )

    def test14_UidInConstant(self):
        self.assertEqual(
            self.q('[u for u in User if u.uid in (1,2)]'),
            Select(u, [('u',u,None,None)], IN(u_uid,C((1,2))))
        )
        self.assertEqual(
            self.q('[u for u in User if u.uid in [1,2]]'),
            Select(u, [('u',u,None,None)], IN(u_uid,C([1,2])))
        )

    def test15_ArithExpression(self):
        self.assertEqual(
            self.q('[u.uid+100 for u in User]'),
            Select(ADD(u_uid,C(100)), [('u',u,None,None)])
        )

    def test16_Length(self):
        self.assertEqual(
            self.q('[len(u.login)+100 for u in User]'),
            Select(ADD(Length(u_login),C(100)), [('u',u,None,None)])
        )

    def test17_DateTimePart(self):
        self.assertEqual(
            self.q('[ugs.term.year for ugs in UGStat]'),
            Select(DatePart(gv(ugs,'term'),C('year')), [('ugs',ugs,None,None)]
        ))
        self.assertEqual(
            self.q('[ugs.term.year-1 for ugs in UGStat]'),
            Select(
                SUB(DatePart(gv(ugs,'term'),C('year')),C(1)),
                [('ugs',ugs,None,None)]
        ))

    def test18_Slice(self):
        self.assertEqual(
            self.q('[u.login[0:1] for u in User]'),
            Select(Substr(u_login, C(1), C(1)), [('u',u,None,None)])
        )
        self.assertEqual(
            self.q('[u.login[1:-1] for u in User]'),
            Select(
                Substr(u_login, C(2), SUB(Length(u_login),C(2))),
                [('u',u,None,None)])
        )
        self.assertEqual(
            self.q('[u.login[1:] for u in User]'),
            Select(Substr(u_login, C(2), C(None)), [('u',u,None,None)])
        )
        self.assertEqual(
            self.q('[u.login[2] for u in User]'),
            Select(Substr(u_login, C(3), C(1)), [('u',u,None,None)])
        )

    def test20_SelectWithoutRelations(self):
        self.assertEqual(
            self.q('NextVal("user_uid_seq")'),
            Select(NextVal(C("user_uid_seq"))),
        )

    def test30_CrossJoin(self):
        self.assertEqual(
            self.q(
                '[(u,ug) for u  in User      if u.uid>1 '
                        'for ug in UserGroup if ug.user==u]'),
            Select(
                Tuple(u,ug),
                [('u',u,None,None), ('ug',ug,None,None)],
                AND(GT(u_uid,C(1)), EQ(gv(ug,'^user.uid'),gv(u,'uid'))),
        ))

    def test31_NaturalJoin(self):
        self.assertEqual(
            self.q(
                '[(u,ug) for u  in User      if u.uid>1 '
                        'for ug in UserGroup if Join(ug.user==u)]'),
            Select(
                Tuple(u,ug),
                [('u',u,None,None),
                 ('ug',ug, Join(EQ(gv(ug,'^user.uid'),u_uid)),None)],
                GT(u_uid,C(1)),
        ))
        self.assertEqual(
            self.q(
                '[u for u in User for ug in UserGroup if Join(ug.user==u)]'),
            Select(
                u,
                [('u',u,None,None),
                 ('ug',ug, Join(EQ(gv(ug,'^user.uid'),gv(u,'uid'))),None)],
        ))
        self.assertEqual(
            self.q('[u for u in User for ug in UserGroup '
                   'if Join(ug.user.uid==u.uid)]'),
            Select(
                u,
                [('u',u,None,None),
                 ('ug',ug, Join(EQ(gv(ug,'^user.uid'),gv(u,'uid'))),None)],
        ))

    def test32_LeftJoin(self):
        self.assertEqual(
            self.q(
                '[(u,g) for u in User '
                       'for ug in UserGroup if LeftJoin(u==ug.user) '
                       'for g in Group if LeftJoin(ug.group==g) '
                       'and g.gid > 100]'),
            Select(
                Tuple(u,g),
                [('u', u,  None,None),
                 ('ug',ug, LeftJoin(EQ(gv(ug,'^user.uid'),u_uid)),None),
                 ('g', g,  LeftJoin(EQ(gv(ug,'^group.gid'),g_gid)),None)],
                GT(g_gid, C(100)),
        ))

    def test33_LeftJoinReordering(self):
        self.assertEqual(
            self.q(
                '[(u,g) for u in User '
                       'for ug in UserGroup '
                       'for g in Group '
                        'if LeftJoin(u==ug.user) and LeftJoin(ug.group==g) '
                        'and g.gid > 100]'),
            Select(
                Tuple(u,g),
                [('u', u,  None, None),
                 ('ug',ug, LeftJoin(EQ(gv(ug,'^user.uid'),u_uid)),None),
                 ('g', g,  LeftJoin(EQ(gv(ug,'^group.gid'),g_gid)),None)],
                GT(g_gid, C(100)),
        ))

    def test35_SelectWithGeneratedJoins(self):
        self.assertEqual(
            self.q('[(ug,ug.user) for ug in UserGroup]'),
            Select(
                Tuple(ug, gv(ug,'user')),
                [('ug',ug,None,None),
                 ('ug.user',gv(ug,'user'),
                   LeftJoin(EQ(gv(ug,'^user.uid'),gv(ug,'$user.uid'))),'1:1'),
                ],
        ))
        self.assertEqual(
            self.q('[(ugs, ugs.ug.user) for ugs in UGStat]'),
            Select(
                Tuple(ugs, gv(ugs,'^ug.user')),
                [('ugs',ugs, None, None),
                 ('ugs.^ug.user',gv(ugs,'^ug.user'),
                   LeftJoin(
                        EQ(gv(ugs,'^ug.^user.uid'), gv(ugs,'^ug.$user.uid'))),
                   '1:1'),
                ],
        ))
        self.assertEqual(
            self.q('[(ugs, ugs.ug) for ugs in UGStat]'),
            Select(
                Tuple(ugs, gv(ugs,'ug')),
                [('ugs',ugs, None,None),
                 ('ugs.ug',gv(ugs,'ug'),
                   LeftJoin(AND(
                    EQ(gv(ugs,'^ug.^user.uid'), gv(ugs,'$ug.^user.uid')),
                    EQ(gv(ugs,'^ug.^group.gid'), gv(ugs,'$ug.^group.gid')))
                 ),'1:1'),
                ],
        ))

    def test36_SelectWithOmitedGeneratedJoins(self):
        self.assertEqual(
            self.q('[(ugs, ugs.ug.user.uid) for ugs in UGStat]'),
            Select(
                Tuple(ugs, gv(ugs,'^ug.^user.uid')),
                [('ugs',ugs,None,None)],
        ))
        self.assertEqual(
            self.q('[(ugs, ugs.ug.user.uid+1) for ugs in UGStat]'),
            Select(
                Tuple(ugs, ADD(gv(ugs,'^ug.^user.uid'),C(1))),
                [('ugs',ugs,None,None)],
        ))

    def test37_RecursiveJoin(self):
        root = Folder(name='root')
        self.assertEqual(
            self.q('[fldr for fldr in Folder if fldr.parent==root]'),
            Select(
                fldr,
                [('fldr',fldr,None,None)],
                where = EQ(gv(fldr,'^parent.name'), C(root.name))
        ))
        self.assertEqual(
            self.q('[fldr for fldr in Folder if fldr.parent.name==root.name]'),
            Select(
                fldr,
                [('fldr',fldr,None,None)],
                where = EQ(gv(fldr,'^parent.name'), C(root.name))
        ))
        self.assertEqual(
            self.q('[fldr for fldr in Folder if fldr.parent.parent==root]'),
            Select(
                fldr,
                [('fldr',fldr,None,None),
                 ('fldr.parent',gv(fldr,'parent'), LeftJoin(
                    EQ(gv(fldr,'^parent.name'), gv(fldr,'$parent.name'))),
                    '1:1'),
                ],
                where = EQ(gv(fldr,'parent.^parent.name'), C(root.name))
        ))

    def test38_InSubselect(self):
        self.assertEqual(
            self.q('[u for u in User if u in [ug.user for ug in UserGroup]]'),
            Select(
                u,
                [('u',u,None,None)],
                where = IN(gv(u,'uid'), Select(
                    gv(ug,'^user.uid'),
                    [('ug',ug,None,None)],
                ))
            )
        )

    def test3A_Lock(self):
        self.assertEqual(
            self.q('[ExclusiveLock(u) for u in User]'),
            Select(
                u,
                [('u',u,None,None)],
                lock = ExclusiveLock,
            )
        )

    def test3F_Distinct(self):
        r = self.q('[Distinct(u.name) for u in User]')
        e = Select(
                Tuple(u_name),
                [('u',u,None,None)],
                distinct = True,
            )
        self.assertEqual(r, e)

    def test3C_Subselect(self):
        s = SubselectVar('o', self.q('[u for u in User]'))
        self.assertEqual(
            self.q('[o for o in [u for u in User]]'),
            Select(s, [('o', s, None, None)])
        )

    def test3D_Subselect(self):
        s = self.q('[u for u in User]')
        q = parse(
            '[Count() for o in s]', self.con, {'s':s}, globals()
        )
        self.assertEqual(
            self.q('[Count() for o in s]'),
            Select(Count(), [('o', SubselectVar('o',s), None, None)])
        )


    def test40_OrderBy(self):
        self.assertEqual(
            self.q('[u for u in User if OrderBy(u.uid, u.login)]'),
            Select(u, [('u',u,None,None)], orderBy=OrderBy(u_uid, u_login)),
        )

    def test41_AggregatingOrderBy(self):
        self.assertEqual(
            self.q('[u for u in User if OrderBy(u.uid) and OrderBy(u.login)]'),
            Select(u, [('u',u,None,None)], orderBy=OrderBy(u_uid, u_login)),
        )

    def test42_OrderingOperatro(self):
        self.assertEqual(
            self.q('[u for u in User if OrderBy(Desc(u.uid), Asc(u.login))]'),
            Select(u, [('u',u,None,None)],
                   orderBy=OrderBy(Desc(u_uid),Asc(u_login))),
        )

    def test50_AggregateWithoutGroupBy(self):
        self.assertEqual(
            self.q('[Count(u) for u in User]'),
            Select(Count(u), [('u',u,None,None)])
        )

    def test51_ImplicitGroupBy(self):
        self.assertEqual(
            self.q('[(u.name,Count(u)) for u in User]'),
            Select(
                Tuple(u_name, Count(u)),
                [('u',u,None,None)],
                groupBy = GroupBy(u_name),
        ))

    def test52_ExplicitGroupBy(self):
        self.assertEqual(
            self.q('[Count(u) for u in User if GroupBy(u.name)]'),
            Select(
                Count(u),
                [('u',u,None,None)],
                groupBy = GroupBy(u_name),
        ))

    def test53_ImplicitHaving(self):
        self.assertEqual(
            self.q('[(u.name,Count(u)) for u in User if Count(u) > 1]'),
            Select(
                Tuple(u_name, Count(u)),
                [('u',u,None,None)],
                groupBy = GroupBy(u_name),
                having = GT(Count(u), C(1)),
        ))

    def test54_ExplicitHaving(self):
        self.assertEqual(
            self.q('[(u.name,Count(u)) for u in User if Having(Count(u) > 1)]'),
            Select(
                Tuple(u_name, Count(u)),
                [('u',u,None,None)],
                groupBy = GroupBy(u_name),
                having = GT(Count(u), C(1)),
        ))

    def test60_Update(self):
        self.assertEqual(
            self.q('[Update(u, uid=u.uid+2) for u in User if u.uid < 100]'),
            UpdateQuery(u, dict(uid=ADD(u_uid,C(2))), LT(u_uid, C(100))),
        )

    def test61_UpdateAll(self):
        self.assertEqual(
            self.q('[Update(u, uid=u.uid+2) for u in User]'),
            UpdateQuery(u, dict(uid=ADD(u_uid,C(2)))),
        )

    def test70_Delete(self):
        self.assertEqual(
            self.q('[Delete(u) for u in User if u.uid < 100]'),
            DeleteQuery(u, LT(u_uid, C(100))),
        )

    def test71_DeleteAll(self):
        self.assertEqual(
            self.q('[Delete(u) for u in User]'),
            DeleteQuery(u),
        )

    def test80_Insert(self):
        self.assertEqual(
            self.q('Insert(User, uid=0, login="root")'),
            Insert(User, uid=0, login='root'),
        )


class TestQueryNamespace(TestBase):

    def test01_SimpleQuery(self):
        q = self.q('[u for u in User]')
        self.assertEqual(q.namespace, {'u':u})
        self.assertEqual(q.getExpr('u'), u)
        self.assertEqual(q.getExpr('u.uid'), u_uid)
        self.assertEqual(q.getExpr('u.name'), u_name)

    def test02_JoinedQuery(self):
        q = self.q('[u for u in User for g in Group]')
        self.assertEqual(q.namespace, {'u':u, 'g':g})
        self.assertEqual(q.getExpr('u.uid'), u_uid)
        self.assertEqual(q.getExpr('g.gid'), g_gid)

    def test10_QueryWithAlias(self):
        q = self.q('[As(u.uid+1,"x") for u in User]')
        self.assertEqual(q.namespace, {'u':u, 'x':ADD(u_uid,C(1))})
        self.assertEqual(q.getExpr('u.uid'), u_uid)
        self.assertEqual(q.getExpr('x'), ADD(u_uid,C(1)))

    def test11_QueryWithAlias(self):
        q = self.q('[(As(u.uid+1,"x"),As(u.uid-1,"y"))  for u in User]')
        self.assertEqual(
            q.namespace, {'u':u, 'x':ADD(u_uid,C(1)), 'y':SUB(u_uid,C(1))})


class TestSubexpressionParser(TestBase):

    def test01_SimpleQuery(self):
        a = 1
        q = self.q('[u for u in User]')
        e = q.parseSubexpr('u')
        self.assertEqual(e, u)
        e = q.parseSubexpr('u.uid')
        self.assertEqual(e, u_uid)
        e = q.parseSubexpr('u.uid + a')
        self.assertEqual(e, ADD(u_uid,C(1)))
        e = q.parseSubexpr('u.uid == 42')
        self.assertEqual(e, EQ(u_uid,C(42)))
        e = q.parseSubexpr('u.uid == a')
        self.assertEqual(e, EQ(u_uid,C(1)))

    def test10_QueryWithAlias(self):
        q = self.q('[(u,As(u.uid,"x")) for u in User]')
        a = 1
        e = q.parseSubexpr('x + a')
        self.assertEqual(e, ADD(u_uid,C(1)))
        e = q.parseSubexpr('x == 42')
        self.assertEqual(e, EQ(u_uid,C(42)))


class TestQueryModification(TestBase):

    def test10_WithExpr(self):
        q1 = self.q('[u for u in User]')
        q2 = q1.withExpr(self.con, q1.parseSubexpr('Count(u)'))
        self.assertEqual(q2, Select(Count(u), [('u',u,None,None)]))

    def test11_WithExpr(self):
        q1 = self.q('[u for u in User if u.uid > 100]')
        q2 = q1.withExpr(self.con, q1.parseSubexpr('Count(u)'))
        self.assertEqual(q2, Select(
            Count(u), [('u',u,None,None)], where=GT(u_uid,C(100))
        ))

    def test12_WithExpr(self):
        q1 = self.q('[u for u in User]')
        q2 = q1.withExpr(self.con, q1.parseSubexpr('(u,Count(u))'))
        self.assertEqual(q2,
            Select(Tuple(u,Count(u)), [('u',u,None,None)], groupBy=GroupBy(u))
        )

    def test20_WithCond(self):
        q1 = self.q('[u for u in User]')
        q2 = q1.withCondition(self.con, q1.parseSubexpr('u.name == "John"'))
        self.assertEqual(q2, Select(
            u, [('u',u,None,None)], where=EQ(u_name, C("John"))
        ))

    def test22_WithCond(self):
        q1 = self.q('[u for u in User if u.uid > 100]')
        q2 = q1.withCondition(self.con, q1.parseSubexpr('u.name == "John"'))
        self.assertEqual(q2, Select(
            u, [('u',u,None,None)],
            where=AND(GT(u_uid,C(100)), EQ(u_name, C("John")))
        ))

    def test23_DistinctWithCond(self):
        q1 = self.q('[Distinct(u) for u in User if u.uid > 100]')
        q2 = q1.withCondition(self.con, q1.parseSubexpr('u.name == "John"'))
        qr = Select(
            Tuple(u), [('u',u,None,None)],
            where=AND(GT(u_uid,C(100)), EQ(u_name, C("John"))),
            distinct=True
        )
        self.assertEqual(q2, qr)

    def test30_AsCount(self):
        q1 = self.q('[u for u in User if u.name=="John"]')
        q2 = q1.asCount(self.con)
        self.assertEqual(q2, Select(
            Count(), [('u',u,None,None)], where=EQ(u_name, C("John"))
        ))

    def test31_AsCountWithOrderBy(self):
        q1 = self.q('[u for u in User if u.name=="John" and OrderBy(u.name)]')
        q2 = q1.asCount(self.con)
        self.assertEqual(q2, Select(
            Count(), [('u',u,None,None)], where=EQ(u_name, C("John"))
        ))

    def test31_AsCountWithDistinct(self):
        q1 = self.q('[Distinct(u.name) for u in User]')
        q2 = q1.asCount(self.con)
        self.assertEqual(q2,
            Select(Count(), [('a',SubselectVar('a',q1),None,None)])
        )


class TestBoundQueries(TestBase):

    def test01(self):
        a = 1
        s = query('[u for u in User if u.uid == a]')
        q = parse(s, self.con)
        self.assertEqual(
            q, Select(u, [('u',u,None,None)], where=EQ(u_uid,C(1)))
        )


def suite():
    return TestSuite((
        makeSuite(TestExpressions),
        makeSuite(TestVariables),
        makeSuite(TestQueries),
        makeSuite(TestQueryNamespace),
        makeSuite(TestSubexpressionParser),
        makeSuite(TestQueryModification),
        makeSuite(TestBoundQueries),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
