# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.query.api import *

from app import *

class TestConsts(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, C)
        self.assertRaises(TypeError, C, 1, 2)

    def test20_Comparison(self):
        self.assertEqual(C(42), C(42))
        self.assertNotEqual(C(42), C(43))

    def test30_Type(self):
        self.assertEqual(C(42).typ, int)
        self.assertEqual(C('foo').typ, str)

    def test40_Relvars(self):
        self.assertEqual(C(42).relvars, {})

    def test45_Aggregates(self):
        self.assertEqual(C(42).aggregates, set())

    def test40_Adaptation(self):
        e = IExpression(42)
        assert isinstance(e, C)
        assert e.value == 42
        e = IExpression(None)
        assert isinstance(e, C)
        assert e.value == None


class TestUnary(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, POS)
        self.assertRaises(TypeError, POS, 1, 2)

    def test20_Comparison(self):
        self.assertEqual(NEG(a), NEG(a))
        self.assertNotEqual(NEG(a), NEG(b))

    def test40_Relvars(self):
        self.assertEqual(NEG(a).relvars, {'a':a})

    def test45_Aggregates(self):
        self.assertEqual(NEG(a).aggregates, set())


class TestArithmetic(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, ADD)
        self.assertRaises(TypeError, ADD, 1)
        self.assertRaises(TypeError, ADD, 1, 2, 3)

    def test20_Comparison(self):
        self.assertEqual(ADD(a,b), ADD(a,b))
        self.assertEqual(ADD(a,b), ADD(b,a))
        self.assertNotEqual(ADD(a,b), ADD(a,c))
        self.assertEqual(ADD(a,C(1)), ADD(C(1),a))

    def test40_Relvars(self):
        self.assertEqual(ADD(a,b).relvars, {'a':a, 'b':b})

    def test45_Aggregates(self):
        self.assertEqual(ADD(a,b).aggregates, set())


class TestComparison(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, LE)
        self.assertRaises(TypeError, LE, 1)
        self.assertRaises(TypeError, LE, 1, 2, 3)

    def test20_Comparison(self):
        self.assertEqual(LE(a,b), LE(a,b))
        self.assertNotEqual(LE(a,b), LE(b,a))
        #self.assertEqual(LE(a,b), GT(b,a)) XXX future
        self.assertEqual(LE(a,C(1)), LE(a,C(1)))

    def test40_Relvars(self):
        self.assertEqual(EQ(a,b).relvars, {'a':a, 'b':b})

    def test45_Aggregates(self):
        self.assertEqual(EQ(a,b).aggregates, set())


class TestBoolean(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, AND)

    def test20_Comparison(self):
        self.assertEqual(AND(a,b), AND(a,b))
        self.assertNotEqual(AND(a,b), AND(b,a))
        self.assertNotEqual(AND(a,b,c), AND(c,b,a))

    def test31_Conjuncts(self):
        self.assertEqual(AND(b,c).conjuncts(), (b,c))
        self.assertEqual(AND(a, AND(b,c)), AND(a,b,c))
        self.assertEqual(AND(a, OR(b,c)), AND(a, OR(b,c)))

    def test32_Disjuncts(self):
        self.assertEqual(OR(b,c).disjuncts(), (b,c))
        self.assertEqual(OR(a, OR(b,c)), OR(a,b,c))
        self.assertEqual(OR(a, AND(b,c)), OR(a, AND(b,c)))

    def test40_Relvars(self):
        self.assertEqual(AND(a,b).relvars, {'a':a, 'b':b})
        self.assertEqual(OR(a,b).relvars, {'a':a, 'b':b})

    def test45_Aggregates(self):
        self.assertEqual(AND(a,b).aggregates, set())
        self.assertEqual(OR(a,b).aggregates, set())



class TestGetattr(TestCase):

    def test10_Constructor(self):
        self.assertRaises(TypeError, Getattr)
        self.assertRaises(TypeError, Getattr, a)
        self.assertRaises(TypeError, Getattr, a, 'b', 'c')

    def test20_Comparison(self):
        self.assertEqual(Getattr(a,'x'), Getattr(a,'x'))
        self.assertNotEqual(Getattr(a,'x'), Getattr(a,'y'))
        self.assertNotEqual(Getattr(a,'x'), Getattr(b,'x'))

    def test40_Relvars(self):
        self.assertEqual(Getattr(a,'x').relvars, {'a':a})

    def test45_Aggregates(self):
        self.assertEqual(Getattr(a,'x').aggregates, set())





class TestContainers(TestCase):

    def test20_TupleComparison(self):
        self.assertEqual(Tuple(), Tuple())
        self.assertEqual(Tuple(a,b), Tuple(a,b))
        self.assertNotEqual(Tuple(a), Tuple(a,b))
        self.assertNotEqual(Tuple(b,a), Tuple(a,b))

    def test21_ListComparison(self):
        self.assertEqual(List(), List())
        self.assertEqual(List(a,b), List(a,b))
        self.assertNotEqual(List(a), List(a,b))
        self.assertNotEqual(List(b,a), List(a,b))

    def test22_DictComparison(self):
        self.assertEqual(Dict(), Dict())
        self.assertNotEqual(Dict(Tuple(a,b)), Dict())
        self.assertEqual(Dict(Tuple(a,b)), Dict(Tuple(a,b)))
        self.assertNotEqual(Dict(Tuple(b,a)), Dict(Tuple(a,b)))
        self.assertEqual(
            Dict(Tuple(b,a),Tuple(c,a)), Dict(Tuple(c,a),Tuple(b,a)),
        )

    def test40_Relvars(self):
        self.assertEqual(Tuple(a).relvars, {'a':a})
        self.assertEqual(List(a,C(1)).relvars, {'a':a})
        self.assertEqual(Dict(Tuple(a,C(1))).relvars, {'a':a})

    def test45_Aggregates(self):
        self.assertEqual(Tuple(a).aggregates, set())
        self.assertEqual(List(a).aggregates, set())
        self.assertEqual(Dict(Tuple(a,C(1))).aggregates, set())




class TestFunctions(TestCase):

    def test10_Comparison(self):
        self.assertEqual(Call(len, (a,)), Call(len, (a,)))
        self.assertNotEqual(Call(len, (a,)), Call(len, (b,)))



def suite():
    return TestSuite((
        makeSuite(TestConsts),
        makeSuite(TestUnary),
        makeSuite(TestArithmetic),
        makeSuite(TestComparison),
        makeSuite(TestBoolean),
        makeSuite(TestGetattr),
        makeSuite(TestContainers),
        makeSuite(TestFunctions),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
