# -*- coding: utf-8 -*-

# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/500261

from wm4.api import *
from itertools import count


next_index = count().next

class Column(object):

    def __init__(self, name, typ, index=None):
        self.name = name
        self.typ = typ
        if index is None:
            index = next_index()
        self.index = index

    def __get__(self, obj, typ=None):
        if obj is None:
            return self
        return obj[self.index]

    def __set__(self, obj, val):
        raise AttributeError('readonly attribute')

    def __repr__(self):
        return 'Column(%s of %s at %s)' % (self.name, self.typ, self.index)


class RelationMeta(type):

    def __new__(meta, name, bases, cdict):
        if not '__columns__' in cdict:
            cdict['__columns__'] = []
        return type.__new__(meta, name, bases, cdict)


class Relation(tuple):
    __metaclass__ = RelationMeta
    __columns__ = ()



if __name__ == '__main__':

    class X(Relation):
        a = Column('a', int)
        b = Column('b', int)
        c = Column('c', int)

    print X.a
    print X.b
    print X.c
    print
    x = X([0,1,2])
    print x.a
    print x.b
    print x.c
