from wm4.api import *

from expressions import Abstract, EQ
from queries import LeftJoin, Select
from interfaces import *

__all__ = [
    'Variable', 'AttrVar', 'OidVar',
    'CompoundVar', 'PKeyVar', 'RelVar', 'ClonedVar',
    'SubselectVar', 'SelectAsQueryable', 'EntityAsQueryable',
]

# These variables are tailored for wm4.schema

class Variable(Abstract):

    protocols.advise(instancesProvide=[IVariable])

    parent  = binding.Require(suggestParent=False)
    var     = binding.Require(suggestParent=False)
    pmap    = binding.Require(suggestParent=False)

    typ     = binding.Obtain('pmap/typeObj', suggestParent=False)

    parents = binding.Make(lambda s:(
                s.parent and s.parent.parents + (s,) or (s,)))
    root    = binding.Make(lambda s: s.parents[0])

    repr    = ''
    path    = binding.Make(lambda s:(
                s.parent and s.parent.path+'.'+s.repr+s.var or s.repr+s.var))

    hashcmp = binding.Make(lambda s:(
                s.__class__,s.var, s.parent and s.parent.hashcmp or None))

    def storage_node(self):
        while self:
            if isinstance(self, RelVar):
                return self
            self = self.parent
        return self
    storage_node = binding.Make(storage_node)

    joins = binding.Delegate('storage_node')
    relvars = binding.Delegate('root')

    def __init__(self, var, pmap, parent=None):
        self.var = var
        self.pmap = pmap
        self.typ = pmap.typeObj
        self.parent = parent

class AttrVar(Variable):

    def __repr__(self):
        return 'AV(%s:%s)' % (self.typ.__name__, self.path)


class OidVar(Abstract):

    def __init__(self, expr):
        self.expr = expr

    hashcmp = binding.Make(lambda s:(s.__class__,s.expr.hashcmp))

    def __repr__(self):
        return '%s._oid' % repr(self.expr)

class CompoundVar(Variable):
    pass


class PKeyVar(CompoundVar):

    repr    = '^'

    def __repr__(self):
        return 'PK(%s:%s)' % (self.typ.__name__, self.path)

class RelVar(CompoundVar):

    storage_node = binding.Make(lambda s: s)

    def __repr__(self):
        return 'RV(%s:%s)' % (self.typ.__name__, self.path)

    relvars = binding.Make(lambda s: {s.var:(s,None)})

    def joins(self, builder):
        parent = self.parent
        if not parent:
            return
        for j in parent.joins(builder):
            yield j
        clone = ClonedVar(self.var, self.pmap, parent)
        # XXX There is no need for LeftJoin if the reference is "Required"
        yield (
            self.path, self, LeftJoin(builder.transform(EQ(self, clone))), '1:1'
        )

class ClonedVar(CompoundVar):

    repr = '$'

    def __repr__(self):
        return 'CV(%s:%s)' % (self.typ.__name__, self.path)


class SubselectVar(CompoundVar):

    hashcmp = binding.Make(lambda s:(s.__class__, s.var, s.sel.hashcmp))

    def __init__(self, var, sel):
        self.var = var
        self.sel = sel
        self.parent = None

    def joins(self, builder):
        yield (self.var, self.sel, None, None)

class SelectAsQueryable(object):

    __slots__ = ('sel',)

    protocols.advise(
        instancesProvide=[IQueryable],
        asAdapterForTypes=[Select]
    )

    def __init__(self, sel):
        self.sel = sel

    def alias(self, var):
        return SubselectVar(var, self.sel)




class EntityAsQueryable(object):

    __slots__ = ('entCls',)

    protocols.advise(
        instancesProvide=[IQueryable],
        asAdapterForTypes=[schema.EntityMeta, schema.TypeRef]
    )

    def __init__(self, entCls):
        self.entCls = schema.TypeRef(entCls)

    def alias(self, var):
        return RelVar(
            var=var, pmap=schema.TypeConfig(self.entCls())(), parent=None
        )


def StringAsQueryable(s):
    return IQueryable(settings.workspace()[s])

protocols.declareAdapter(StringAsQueryable, [IQueryable], forTypes=[basestring])
