from wm4.api import *
from lib4.imports import importString
import os, sys, options

__all__ = [
    'ICommandLineAppFactory', 'ICommandLineApp', 'InvocationError',
    'AbstractCommand', 'CommandLineApp', 'ConfiguredCommandLineApp',
    'WSGIServer',
]

def runMain(argv=sys.argv):
    config.loadConfig('pkgfile:wm4/core.ini')
    argv.pop(0)
    factory = importString(argv[0])
    command = factory(argv=argv)
    command.run()



class ICommandLineAppFactory(protocols.Interface):

    def __call__(self,
            argv  = sys.argv,
            stdin = sys.stdin,
            stdout = sys.stdout,
            stderr = sys.stderr,
            environ = os.environ, **otherAttrs):
        '''Command constructor'''

class ICommandLineApp(protocols.Interface):

    def run():
        '''Perform command functionality'''



class InvocationError(Exception):
    """Problem with command arguments or environment"""




class AbstractCommand(binding.Component):

    protocols.advise(
        instancesProvide = [ICommandLineApp],
        classProvides    = [ICommandLineAppFactory]
    )

    argv    =  binding.Make(lambda: settings.argv())
    stdin   =  binding.Make(lambda: settings.stdin())
    stdout  =  binding.Make(lambda: settings.stdout())
    stderr  =  binding.Make(lambda: settings.stderr())
    environ =  binding.Make(lambda: settings.environ())

    usage = 'AbstractCommand'

    option_parser = binding.Make(
        lambda self: options.make_parser(self)
    )

    parsed_args = binding.Make(
        lambda self: self.option_parser.parse_args(self.argv[1:],self)[1]
    )

    def showHelp(self):
        """Display usage message on stderr"""
        print >>self.stderr, self.usage
        print >>self.stderr, self.option_parser.format_help()
        return 0

    @options.option_handler('--help', value=None, help="Show help",sortKey=1000)
    def show_help(self, parser, optname, optval, remaining_args):
        sys.exit(self.showHelp())


    def isInteractive(self):
        """True if 'stdin' is a terminal"""
        try:
            isatty = self.stdin.isatty
        except AttributeError:
            return False
        else:
            return isatty()
    isInteractive = binding.Make(isInteractive)

    def _invocationError(self, msg):
        """Write msg and usage to stderr if interactive, otherwise re-raise"""
        if self.isInteractive:
            self.showHelp()
            print >>self.stderr, '\n%s: %s\n' % (self.argv[0], msg)
            # XXX output last traceback frame?
            return 1    # exit errorlevel
        else:
            raise   # XXX is this really appropriate?

    def run(self):
        self.parsed_args
        try:
            return self.main() or 0
        except SystemExit, v:
            return v.args and v.args[0] or 0
        except InvocationError, msg:
            return self._invocationError(msg)


class CommandLineApp(AbstractCommand):

    def main(self):
        self.parsed_args

class ConfiguredCommandLineApp(CommandLineApp):

    ini_files = binding.Make(list,
            options.Append('--ini', type=str, help='Configuration files'))

    def main(self):
        self.parsed_args
        config.loadConfig(*self.ini_files)




class SCGIServer(CommandLineApp):

    port = binding.Constant(4000,
            options.Set('--port', type=int, help='Listen on port'))

    config = binding.Constant(None,
            options.Set('--config', type=str, help='Bind address'))

    def get_app(self):
        config.loadConfig(self.config)
        return settings.web_app()

    app = binding.Make(get_app)




class WSGIServer(CommandLineApp):

    port = binding.Constant(8000,
            options.Set('--port', type=int, help='Listen on port'))

    bind = binding.Constant('',
            options.Set('--bind', type=str, help='Bind address'))

    config = binding.Constant(None,
            options.Set('--config', type=str, help='Bind address'))

    https = binding.Constant(False,
            options.Set('--https', value=True, help='Use HTTPS?'))

    ssl_cert = binding.Constant(None,
            options.Set('--ssl-cert', type=str, help='SSL certificate'))

    ssl_key = binding.Constant(None,
            options.Set('--ssl-key', type=str, help='SSL key'))

    process_model = binding.Constant(None,
            options.Set('--process-model', type=str, help='forking/threading'))


    def get_app(self):
        config.loadConfig(self.config)
        return settings.web_app()

    app = binding.Make(get_app)

    def main(self):
        import wsgi_server, ssl_server

        self.parsed_args

        if self.https:
            cls = wsgi_server.SSLWSGIServer
        else:
            cls = wsgi_server.WSGIServer

        if self.process_model == 'forking':

            from SocketServer import ForkingMixIn
            class Server(cls, ForkingMixIn):
                pass

        elif self.process_model == 'threading':

            from SocketServer import ThreadingMixIn
            class Server(cls, ThreadingMixIn):
                pass

        else:
            Server = cls

        print Server, Server.__bases__

        if self.https:
            httpd = Server(
                (self.bind, self.port), wsgi_server.SSLWSGIRequestHandler,
                ssl_server.SSLContext(self.ssl_cert, self.ssl_key),
            )
            print "Serving HTTPS on", self.bind or '*', "port", self.port, "..."

        else:
            httpd = Server(
                (self.bind, self.port), wsgi_server.WSGIRequestHandler
            )
            print "Serving HTTP on", self.bind or '*', "port", self.port, "..."

        httpd.set_app(self.app)
        return httpd.serve_forever()
