from SocketServer import TCPServer, BaseServer
from OpenSSL import SSL
import socket

__all__ = ['SSLWrapper', 'SSLContext', 'SSLTCPServer', 'SSLRequestHandlerMixIn']


class SSLWrapper:
    """
    This whole class exists just to filter out a parameter
    passed in to the shutdown() method in SimpleXMLRPC.doPOST()
    """
    def __init__(self, conn):
        """
        Connection is not yet a new-style class,
        so I'm making a proxy instead of subclassing.
        """
        self.__dict__["conn"] = conn
    def __getattr__(self,name):
        return getattr(self.__dict__["conn"], name)
    def __setattr__(self,name, value):
        setattr(self.__dict__["conn"], name, value)
    def shutdown(self, how=1):
        """
        SimpleXMLRpcServer.doPOST calls shutdown(1),
        and Connection.shutdown() doesn't take
        an argument. So we just discard the argument.
        """
        self.__dict__["conn"].shutdown()
    def accept(self):
        """
        This is the other part of the shutdown() workaround.
        Since servers create new sockets, we have to infect
        them with our magic. :)
        """
        c, a = self.__dict__["conn"].accept()
        return (SSLWrapper(c), a)


def SSLContext(
    SSLCertFile,
    SSLKeyFile = None,
    SSLCAFile = None,
    #SSLVersion = SSL.TLSv1_METHOD,
    SSLVersion = SSL.SSLv23_METHOD,
    SSLOptions = SSL.OP_NO_SSLv2,
):
    ## Same as normal, but make it secure:
    ctx = SSL.Context(SSLVersion)
    ctx.set_options(SSLOptions)
    ctx.use_privatekey_file(SSLKeyFile or SSLCertFile)
    ctx.use_certificate_file(SSLCertFile)
    return ctx



class SSLTCPServer(TCPServer):

    def __init__(self, server_address, request_handler, ssl_context):
        BaseServer.__init__(self, server_address, request_handler)
        self.no_ssl_socket = socket.socket(self.address_family,self.socket_type)
        self.socket = SSLWrapper(
            SSL.Connection(ssl_context, self.no_ssl_socket)
        )
        self.server_bind()
        self.server_activate()

    def server_close(self):
        self.socket.close()
        #self.no_ssl_socket.shutdown(socket.SHUT_RDWR)
        self.no_ssl_socket.close()





class SSLRequestHandlerMixIn:

    def setup(self):
        """
        We need to use socket._fileobject Because SSL.Connection
        doesn't have a 'dup'. Not exactly sure WHY this is, but
        this is backed up by comments in socket.py and SSL/connection.c
        """
        self.connection = self.request # for doPOST
        self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
        self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)
