__all__ = [
'PersistentMixin', 'PersistentList', 'PersistentDict', 'PersistentDictOfLists',
]


class PersistentMixin(object):

    _p_register = None

    def _p_changed(self):
        change = self.__dict__.get('_p_changed', self)
        if change is self:
            change = self.__dict__['_p_changed'] = self._p_register(self)
        return change
    _p_changed = property(_p_changed)

    def _p_clean(self):
        self.__dict__.pop('_p_changed', None)

    def _p_deactivate(self):
        pass


class PersistentList(list, PersistentMixin):

    def __init__(self, initlist=None):
        if initlist is not None:
            w = self._p_wrap
            super(PersistentList, self).__init__(w(i) for i in initlist)

    def _p_wrap(self, v):
        return v
    
    def _p_unwrap(self, v):
        return v

    def _p_clean(self):
        super(PersistentList, self)._p_clean()
        for v in self:
            if isinstance(v, PersistentMixin):
                v._p_clean()

    def _p_preload(self, v):
        l = len(self)
        list.__delslice__(self, 0, l)
        for i in v:
            list.append(self, i)

    def __setitem__(self, i, v):
        self._p_changed
        list.__setitem__(self, i, self._p_wrap(v))

    def __setslice__(self, i, j, v):
        self._p_changed
        w = self._p_wrap
        list.__setslice__(self, i, j, [w(x) for x in v])

    def __delitem__(self, i):
        self._p_changed
        list.__delitem__(self, i)

    def __delslice__(self, i):
        self._p_changed
        list.__delslice__(self, i)

    def __iadd__(self, v):
        self._p_changed
        w = self._p_wrap
        return list.__iadd__(self, [w(x) for x in v])

    def __imul__(self, v):
        self._p_changed
        return list.__imul__(self, v)

    def append(self, v):
        self._p_changed
        list.append(self, self._p_wrap(v))

    def extend(self, v):
        self._p_changed
        w = self._p_wrap
        list.extend(self, [w(x) for x in v])

    def insert(self, i, v):
        self._p_changed
        list.insert(self, i, self._p_wrap(v))

    def pop(self, i=-1):
        self._p_changed
        return list.pop(self, i)

    def remove(self, v):
        self._p_changed
        list.remove(self, v)

    def reverse(self):
        self._p_changed
        list.reverse(self)

    def sort(self):
        self._p_changed
        list.sort(self)


class PersistentDict(dict, PersistentMixin):

    def __init__(self, initdict=None, **kwargs):
        d = {}
        wrap = self._p_wrap
        if initdict is not None:
            if isinstance(initdict, dict):
                initdict = initdict.iteritems()
            d = dict((wrap(k,v)) for k,v in initdict)
        if len(kwargs):
            d.update(dict((wrap(k,v)) for k,v in kwargs.iteritems()))
        super(PersistentDict, self).__init__(d)

    def _p_wrap(self, k, v):
        return k, v

    def _p_clean(self):
        super(PersistentDict, self)._p_clean()
        for v in self.itervalues():
            if isinstance(v, PersistentMixin):
                v._p_clean()

    def __setitem__(self, key, val):
        self._p_changed
        key, val = self._p_wrap(key, val)
        dict.__setitem__(self, key, val)

    def __delitem__(self, key):
        self._p_changed
        dict.__delitem__(self, key)

    def clear(self):
        self._p_changed
        dict.clear(self)

    def update(self, b):
        self._p_changed
        w = self._p_wrap
        dict.update(self, dict((w(k,v)) for k,v in b.iteritems()))

    def setdefault(self, key, val=None):
        # XXX not thread safe
        if not self.has_key(key):
            self._p_changed
        key, val = self._p_wrap(key, val)
        return dict.setdefault(self, key, val)

    def pop(self, key, *args):
        self._p_changed
        return dict.pop(self, key, *args)

    def popitem(self):
        self._p_changed
        return dict.popitem(self)




class PersistentDictOfLists(PersistentDict):

    def _p_wrap(self, k, v):
        pl = PersistentList(v)
        pl._p_register = self._p_item_register
        return k, pl

    def _p_item_register(self, pl):
        self._p_changed
