from wm4.api import NOT_GIVEN, i18n

__all__ = [
    'SchemaError', 'FeatureError', 'NormalizationError', 'FeatureSyntaxError',
    'RequiredError', 'MinLengthError', 'MaxLengthError',
    'NestedEntityError', 'EntityError', 'catchFeatureErrors',
]

_ = i18n.MessageIDFactory('wm4')


class SchemaError(ValueError):
    pass

class FeatureError(SchemaError):

    msgId = None
    attr = None

    def __init__(self, attr, msgId=NOT_GIVEN, **mapping):
        self.attr = attr
        if msgId is NOT_GIVEN:
            msgId = self.msgId
        self.msgId  = msgId % mapping

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.msgId)
    __str__ = __repr__


class NormalizationError(FeatureError):
    msgId = _(u"Cannot convert input value $value to type $type_name")

class FeatureSyntaxError(FeatureError):
    msgId = _(u"Invalid syntax of input value $value")

class RequiredError(FeatureError):
    msgId = _(u"Attribute must have value")


class MinLengthError(FeatureError):
    msgId = _(u"Minimal length is $len")

class MaxLengthError(FeatureError):
    msgId = _(u"Maximal length is $len")

class NestedEntityError(FeatureError):
    def __init__(self, attr, entErr):
        self.attr = attr
        self.entErr = entErr



class EntityError(SchemaError):

    def __init__(self, **featErrors):
        self.errors = {}
        for a,e in featErrors.iteritems():
            self[a] = e

    def add(self, featErr):
        self[featErr.attr] = featErr

    def __setitem__(self, attr, featErr):
        assert isinstance(featErr, FeatureError)
        if isinstance(featErr, NestedEntityError):
            for a,el in featErr.entErr.iteritems():
                for e in el:
                    na = attr and attr+'.'+a or a
                    self.errors.setdefault(na, []).append(e)
        else:
            self.errors.setdefault(attr, []).append(featErr)

    def __getitem__(self, attr): return self.errors[attr]
    def __delitem__(self, attr): del self.errors[attr]
    def has_key(self, attr): return self.errors.has_key(attr)
    def get(self, attr, default=None): return self.errors.get(attr,default)
    def pop(self, attr, default): return self.errors.pop(attr, default)
    def keys(self): return self.errors.keys()
    def values(self): return self.errors.values()
    def items(self): return self.errors.items()
    def iterkeys(self): return self.errors.iterkeys()
    def itervalues(self): return self.errors.itervalues()
    def iteritems(self): return self.errors.iteritems()
    def __iter__(self): return iter(self.errors)
    def __nonzero__(self): return self.errors and 1 or 0
    def __len__(self): return len(self.errors)

    def __repr__(self):
        return "EntityError(%s)" % ', '.join([
            '%s=%s' % (a, ','.join([str(ei) for ei in e]))
            for a,e in self.errors.iteritems()
        ])

    __str__ = __repr__



def catchFeatureErrors(meth):

    def catchFeatureErrors(*a):
        ee = EntityError()
        try:
            return meth(*a)
        except FeatureError, fe:
            ee.add(fe)
            raise ee

    return catchFeatureErrors
