from wm4.core import *
from wm4.binding.metadata import *
from types import ClassType
from adt4.serial import Serial
from wm4.utils.property_rules import Value

__all__ = [
    'Unique', 'PrimaryKey', 'SequenceName',
    'Default', 'ComputeDefault', 'ComputeDefaultForType',
    'defaultDatetime', 'defaultDate',
    'Normalize', 'normalize', 'wrapNormalizeFunc',
    'ReferenceEndOrder',
]


class Unique(tuple):
    def __new__(klass, *features):
        return tuple.__new__(klass, features)
    #def __init__(self, *features):
    #    tuple.__init__(self, features)
    def __repr__(self):
        return 'Unique(%s)' % ','.join(repr(f) for f in self)

[binding.declareClassMetadata.when(Unique)]
def declareUniqueFeatures(classobj, metadata):
    classobj.unique_features.append(metadata)

class PrimaryKey(ClassMetadata):
    '''Feature names that make primary key'''
    def __init__(self, *features):
        super(PrimaryKey, self).__init__(features)



def Default(value):
    '''Constant default value'''
    return ComputeDefault(Value(lambda o,a,v: value))

class ComputeDefault(AttributeMetadata):
    '''Function for computing default values'''

class ComputeDefaultForType(ClassMetadata):
    pass
ComputeDefaultForType(Value(lambda o,a,v: None)).register('*')

def newSerial(obj, attr, values):
    try:
        ws = settings.workspace() #XXX
    except:
        return None
    from wm4.query.api import NextVal
    seq = SequenceName.of(obj.__class__, attr)
    if not seq:
        return None
    else:
        return ~ws('NextVal(seq)')

ComputeDefaultForType(Value(newSerial)).register(Serial)



def defaultDatetime(o,a,v):
    return settings.time_service().now()
defaultDatetime = ComputeDefault(Value(defaultDatetime))

def defaultDate(o,a,v):
    return settings.time_service().today()
defaultDate = ComputeDefault(Value(defaultDate))


class SequenceName(AttributeMetadata):
    '''Sequence name for given feature of Serial type'''

def defaultSequenceName(search, found):
    ent,attr = search
    if attr == '*':
        return None
    # XXX only for postgres
    # XXX importing storage is bad
    from wm4.storage.api import SqlTableName
    return '%s_%s_seq' % (SqlTableName(ent)(), attr)

SequenceName(defaultSequenceName).register(object, '*')





class Normalize(ClassMetadata):
    '''Returns normalization function for given type (class metadata)'''

def normalize(val, typ):
    '''Think of this method as of kind of adaptation'''
    return Normalize.of(typ)(val)

def wrapNormalizeFunc(func):
    def normalize(val):
        if val is None : return None
        else : return func(val)
    return normalize

def defaultNormalization(klass, fCls):
    def normalize(val):
        if not isinstance(val, klass):
            try :
                val = klass(val)
            except :
                raise ValueError('Cannot normalize', val, klass)
        return val
    return wrapNormalizeFunc(normalize)


Normalize(defaultNormalization).register('*')

from lib4.timeutils import Date, Datetime, Time
from datetime import date, datetime, time

Normalize(Value(wrapNormalizeFunc(Date))).register(date)
Normalize(Value(wrapNormalizeFunc(Datetime))).register(datetime)
Normalize(Value(wrapNormalizeFunc(Time))).register(time)

def normalizeFloat(val):
    if not isinstance(val, float):
        if isinstance(val, basestring):
            val = val.replace(',', '.')
        try:
            val = float(val)
        except:
            raise ValueError('Cannot normalize', val, float)
    return val
Normalize(Value(wrapNormalizeFunc(normalizeFloat))).register(float)


from adt4.enums import Enum

def normalizeEnumRule(klass, fCls):
    def normalize(val):
        if val is None : return None
        elif isinstance(val, klass): return val
        else : return klass[val]
    return normalize
Normalize(normalizeEnumRule).register(Enum)


from adt4.blob import Blob

def normalizeBlob(val):
    if isinstance(val, Blob):
        return val

    bdm = settings.blob_manager()
    if hasattr(val,'read'):
        pass
    elif hasattr(val,'file') and hasattr(val,'type'):
        val = val.file
    else:
        raise ValueError('Unsupported type', val, type(val))
    b = bdm.new()
    b.open('w')
    size = 131072
    while True:
        data = val.read(size)
        b.write(data)
        if len(data) < size:
            break
    b.close()
    return b

Normalize(Value(wrapNormalizeFunc(normalizeBlob))).register(Blob)




class ReferenceEndOrder(AttributeMetadata):
    '''Ordering of ReferenceEnd results'''

    def __init__(self, *features):
        super(ReferenceEndOrder, self).__init__(features)

    def register(self, clsObj, attr=None):
        return super(ReferenceEndOrder,self).register(clsObj, attr)
