# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.schema.api import *
from wm4.schema.tests.utils import *

class TestConstraints(TestCase):

    def test10_NewRequired(self):
        class E(Entity):
            a = Attribute(int, Required)
        assertEntityError(self, [('a',RequiredError)], E)

    def test11_SetattrRequired(self):
        class E(Entity):
            a = Attribute(int, Required)
        e = E(a=1)
        assertEntityError(self, [('a',RequiredError)], setattr, e, 'a', None)

    def test12_UpdateRequired(self):
        class E(Entity):
            a = Attribute(int, Required)
        e = E(a=1)
        assertEntityError(self, [('a',RequiredError)], e._setattrs, a=None)

    def test13_RequiredPrimaryKey(self):
        class E(Entity):
            a = b = Attribute(int)
            binding.metadata(PrimaryKey('a'))
        assertEntityError(self, [('a',RequiredError)], E)
        e = E(a=1)
        assertEntityError(self, [('a',RequiredError)], e._setattrs, a=None)

    def test20_NewReadOnly(self):
        class E(Entity):
            a = Attribute(int, ReadOnly)
        e = E()
        self.assertEqual(e.a, None)
        self.assertEqual(ReadOnly.of(E, 'a'), True)

    def test21_SetattrReadOnly(self):
        class E(Entity):
            a = Attribute(int, ReadOnly)
        e = E(a=1)
        self.assertRaises(AttributeError, setattr, e, 'a', 1)

    def test22_UpdateReadOnly(self):
        class E(Entity):
            a = Attribute(int, ReadOnly)
        e = E(a=1)
        self.assertRaises(AttributeError, e._setattrs, a=1)

    syntaxConstraint = Syntax('A{2,3}')

    def test30_NewSyntax(self):
        class E(Entity):
            a = Attribute(str, self.syntaxConstraint)
        assertEntityError(self, [('a',FeatureSyntaxError)], E, a='B')

    def test31_SetattrSyntax(self):
        class E(Entity):
            a = Attribute(str, self.syntaxConstraint)
        e = E(a='AA')
        assertEntityError(self,[('a',FeatureSyntaxError)],setattr,e,'a','B')

    def test33_UpdateSyntax(self):
        class E(Entity):
            a = Attribute(str, self.syntaxConstraint)
        e = E(a='AA')
        assertEntityError(self,[('a',FeatureSyntaxError)],e._setattrs,a='B')



def suite():

    return TestSuite((
        makeSuite(TestConstraints),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
