# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.schema.api import *


EE = EntityError
RE = RequiredError
NE = NormalizationError
NEE = NestedEntityError

class TestErrors(TestCase):

    def test01_ErrorAccumulation(self):
        ee = EE()
        ee['a'] = RE('a')
        ee['a'] = NE('a')
        self.assertEqual(len(ee['a']), 2)

    def test03_AddError(self):
        ee = EE()
        ee.add(RE('a'))
        ee.add(NE('a'))
        self.assertEqual(len(ee['a']), 2)

    def test05_SetWithDifferentName(self):
        ee = EE()
        ee['a'] = RE('b')
        ee['a'] = NE('c')
        self.assertEqual(len(ee['a']), 2)

    def test10_NonzeroAndLength(self):
        ee = EE()
        self.failIf(ee)
        self.assertEqual(len(ee), 0)
        ee['a'] = RE('a')
        ee['a'] = NE('a')
        self.failUnless(ee)
        self.assertEqual(len(ee), 1)

    def test20_NestedErrors(self):
        ne = NEE('x.y', EE(a=RE('a'), b=RE('b')))
        ee = EE()
        ee['x.y'] = ne
        self.assertEqual(len(ee), 2)
        self.assertEqual(len(ee['x.y.a']), 1)
        self.assertEqual(len(ee['x.y.b']), 1)

    def test22_RootNestedErrors(self):
        ne = NEE('', EE(a=RE('a'), b=RE('b')))
        ee = EE()
        ee[''] = ne
        self.assertEqual(len(ee), 2)
        self.assertEqual(len(ee['a']), 1)
        self.assertEqual(len(ee['b']), 1)



def suite():
    return TestSuite((
        makeSuite(TestErrors),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
