# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.schema.api import *

from wm4.query.tests.app import *

class E10(Entity):
    a = Attribute(int)
    b = Attribute(unicode)
    binding.metadata(PrimaryKey('a'))


class E20(Entity):
    a = b = Attribute(int)
    def __cmp__(self, other): return cmp(self.a, other.a)
    binding.metadata(PrimaryKey('a'))
class F20(Entity):
    r = Reference(E20)
    s = Attribute(int)
    binding.metadata(PrimaryKey('r'))


class E30(Entity):
    a = b = Attribute(int)
    def __cmp__(self, other): return cmp((self.a,self.b), (other.a,other.b))
    binding.metadata(PrimaryKey('a','b'))
class F30(Entity):
    r = Reference(E30)
    s = Attribute(int)
    binding.metadata(PrimaryKey('r'))


class E40(Entity):
    a = b = Attribute(int)
    def __cmp__(self, other): return cmp((self.a,self.b), (other.a,other.b))
    binding.metadata(PrimaryKey('a','b'))
class F40(Entity):
    f = Reference(E40)
    def __cmp__(self, other): return cmp(self.f, other.f)
    binding.metadata(PrimaryKey('f'))
class G40(Entity):
    r = Reference(F40)
    s = Attribute(int)
    binding.metadata(PrimaryKey('r'))


class E50(Entity):
    a = b = c = Attribute(int)
    def __cmp__(self, other): return cmp(self.a, other.a)
    binding.metadata(PrimaryKey('a'))
class F50(Entity):
    f = Reference(E50)
    g = Reference(E50)
    h = Attribute(int)
    def __cmp__(self, other): return cmp(self.f, other.f)
    binding.metadata(PrimaryKey('f'))
class G50(Entity):
    r = Reference(F50)
    s = Reference(E50)
    t = Attribute(int)
    binding.metadata(PrimaryKey('r','s'))


def TC(ent):
    return TypeConfig(ent)()

def frc(ent, *apath):
    m = p(ent)
    for a in apath:
        m = m[a]
    return m.from_row_convertor()


class TestMapping(TestCase):

    def test10_PrimitiveAttrs(self):
        self.assertEqual(E10.a.storageAttr, 'a')
        self.assertEqual(E10.b.storageAttr, 'b')

    def test11_ValueFromObj(self):
        e = E10(a=1, b='u')
        m = TC(E10)
        self.assertEqual(m.valueFromObj(e), e)
        self.assertEqual(m['a'].valueFromObj(e), 1)
        self.assertEqual(m['a'].valueFromObj(1,m['a']), 1)
        self.assertEqual(m['b'].valueFromObj(e), 'u')

    def test12_OidPart(self):
        self.assertEqual(E10.a.oidPart, True)
        self.assertEqual(E10.a.rootOidPart, True)
        self.assertEqual(E10.b.oidPart, False)
        self.assertEqual(E10.b.rootOidPart, False)

    def test13_OidManagement(self):
        e = E10(a=1, b='u')
        m = TC(E10)

        o = m.oidFromObj(e)
        self.assertEqual(o, 1)

        self.assertEqual(m['a'].valueFromOID(o), 1)
        self.assertRaises(TypeError, getattr, m['b'], 'valueFromOID')

        self.assertEqual(m.oidToAttrs(o), {'a':1})
        self.assertEqual(m.oidToPrimitives(o), {'a':1})

        self.assertEqual(m.oidFromInput(1), 1)
        self.assertRaises(ValueError, m.oidFromInput, 'a')

        self.assertEqual(e._oid, 1)

    def test20_PrimitiveAttrs(self):
        self.assertEqual(F20.r['a'].storageAttr, 'r')
        self.assertEqual(F20.s.storageAttr, 's')

    def test21_ValueFromObj(self):
        e = F20(r=E20(a=1, b=2))
        m = TC(F20)
        self.assertEqual(m.valueFromObj(e), e)
        self.assertEqual(m['r'].valueFromObj(e), e.r)
        self.assertEqual(m['r']['a'].valueFromObj(e), 1)
        self.assertEqual(m['r']['b'].valueFromObj(e), 2)
        self.assertEqual(m['r']['a'].valueFromObj(e.r,m['r']), 1)

    def test22_OidPart(self):
        self.assertEqual(F20.r.oidPart, True)
        self.assertEqual(F20.r.rootOidPart, True)
        self.assertEqual(F20.r['a'].oidPart, True)
        self.assertEqual(F20.r['a'].rootOidPart, True)
        self.assertEqual(F20.r['b'].oidPart, False)
        self.assertEqual(F20.r['b'].rootOidPart, False)
        self.assertEqual(F20.s.oidPart, False)
        self.assertEqual(F20.s.rootOidPart, False)

    def test23_OidManagement(self):
        e = F20(r=E20(a=1, b=2))
        m = TC(F20)

        o = m.oidFromObj(e)
        self.assertEqual(o, 1)

        self.assertEqual(m['r'].valueFromOID(o), e.r)
        self.assertEqual(m['r']['a'].valueFromOID(o), 1)
        self.assertRaises(TypeError, getattr, m['r']['b'], 'valueFromOID')
        self.assertRaises(TypeError, getattr, m['s'], 'valueFromOID')

        self.assertEqual(m.oidToAttrs(o), {'r':e.r})
        self.assertEqual(m.oidToPrimitives(o), {'r':1})

        self.assertEqual(m.oidFromInput(E20(a=1,b=2)), 1)
        self.assertRaises(ValueError, m.oidFromInput, 1)

        self.assertEqual(e._oid, 1)

    def test30_PrimitiveAttrs(self):
        self.assertEqual(F30.r['a'].storageAttr, 'r_a')
        self.assertEqual(F30.r['b'].storageAttr, 'r_b')
        self.assertEqual(F30.s.storageAttr, 's')

    def test31_ValueFromObj(self):
        e = F30(r=E30(a=1, b=2), s=3)
        m = TC(F30)
        self.assertEqual(m.valueFromObj(e), e)
        self.assertEqual(m['r'].valueFromObj(e), e.r)
        self.assertEqual(m['r']['a'].valueFromObj(e), 1)
        self.assertEqual(m['r']['b'].valueFromObj(e), 2)
        self.assertEqual(m['s'].valueFromObj(e), 3)

    def test32_OidPart(self):
        self.assertEqual(F30.r.oidPart, True)
        self.assertEqual(F30.r.rootOidPart, True)
        self.assertEqual(F30.r['a'].oidPart, True)
        self.assertEqual(F30.r['a'].rootOidPart, True)
        self.assertEqual(F30.r['b'].oidPart, True)
        self.assertEqual(F30.r['b'].rootOidPart, True)
        self.assertEqual(F30.s.oidPart, False)
        self.assertEqual(F30.s.rootOidPart, False)

    def test33_OidManagement(self):
        e = F30(r=E30(a=1, b=2), s=3)
        m = TC(F30)

        o = m.oidFromObj(e)
        self.assertEqual(o, (1,2))

        self.assertEqual(m['r'].valueFromOID(o), e.r)
        self.assertEqual(m['r']['a'].valueFromOID(o), 1)
        self.assertEqual(m['r']['b'].valueFromOID(o), 2)

        self.assertEqual(m.oidToAttrs(o), {'r':e.r})
        self.assertEqual(m.oidToPrimitives(o), {'r_a':1, 'r_b':2})

        self.assertEqual(m.oidFromInput(E30(a=1,b=2)), (1,2))
        self.assertRaises(ValueError, m.oidFromInput, 1)

        self.assertEqual(e._oid, (1,2))

    def test40_PrimitiveAttrs(self):
        self.assertEqual(G40.r['f']['a'].storageAttr, 'r_f_a')
        self.assertEqual(G40.r['f']['b'].storageAttr, 'r_f_b')
        self.assertEqual(G40.s.storageAttr, 's')

    def test41_ValueFromObj(self):
        e = G40(r=F40(f=E40(a=1, b=2)), s=3)
        m = TC(G40)
        self.assertEqual(m.valueFromObj(e), e)
        self.assertEqual(m['r'].valueFromObj(e), e.r)
        self.assertEqual(m['r']['f'].valueFromObj(e), e.r.f)
        self.assertEqual(m['r']['f']['a'].valueFromObj(e), 1)
        self.assertEqual(m['r']['f']['b'].valueFromObj(e), 2)
        self.assertEqual(m['s'].valueFromObj(e), 3)

    def test42_OidPart(self):
        self.assertEqual(G40.r.oidPart, True)
        self.assertEqual(G40.r.rootOidPart, True)
        self.assertEqual(G40.r['f'].oidPart, True)
        self.assertEqual(G40.r['f'].rootOidPart, True)
        self.assertEqual(G40.r['f']['a'].oidPart, True)
        self.assertEqual(G40.r['f']['a'].rootOidPart, True)
        self.assertEqual(G40.r['f']['b'].oidPart, True)
        self.assertEqual(G40.r['f']['b'].rootOidPart, True)
        self.assertEqual(G40.s.oidPart, False)
        self.assertEqual(G40.s.rootOidPart, False)

    def test43_OidManagement(self):
        e = G40(r=F40(f=E40(a=1, b=2)), s=3)
        m = TC(G40)

        o = m.oidFromObj(e)
        self.assertEqual(o, (1,2))

        self.assertEqual(m['r'].valueFromOID(o), e.r)
        self.assertEqual(m['r']['f'].valueFromOID(o), e.r.f)
        self.assertEqual(m['r']['f']['a'].valueFromOID(o), 1)
        self.assertEqual(m['r']['f']['b'].valueFromOID(o), 2)

        self.assertEqual(m.oidToAttrs(o), {'r':e.r})
        self.assertEqual(m.oidToPrimitives(o), {'r_f_a':1, 'r_f_b':2})

        self.assertEqual(m.oidFromInput(F40(f=E40(a=1,b=2))), (1,2))
        self.assertRaises(ValueError, m.oidFromInput, 1)

        self.assertEqual(e._oid, (1,2))

    def test50_PrimitiveAttrs(self):
        self.assertEqual(G50.r['f']['a'].storageAttr, 'r')
        self.assertEqual(G50.s['a'].storageAttr, 's')

    def test51_ValueFromObj(self):
        e = G50(r=F50(f=E50(a=1, b=2)), s=E50(a=3,b=4))
        m = TC(G50)
        self.assertEqual(m.valueFromObj(e), e)
        self.assertEqual(m['r'].valueFromObj(e), e.r)
        self.assertEqual(m['r']['f'].valueFromObj(e), e.r.f)
        self.assertEqual(m['r']['f']['a'].valueFromObj(e), 1)
        self.assertEqual(m['r']['f']['b'].valueFromObj(e), 2)
        self.assertEqual(m['s'].valueFromObj(e), e.s)
        self.assertEqual(m['s']['a'].valueFromObj(e), 3)
        self.assertEqual(m['s']['b'].valueFromObj(e), 4)
        self.assertEqual(m['r']['f']['b'].valueFromObj(e.r,m['r']), 2)
        self.assertEqual(m['r']['f']['b'].valueFromObj(e.r.f,m['r']['f']), 2)

    def test52_OidPart(self):
        self.assertEqual(G50.r.oidPart, True)
        self.assertEqual(G50.r.rootOidPart, True)
        self.assertEqual(G50.r['f'].oidPart, True)
        self.assertEqual(G50.r['f'].rootOidPart, True)
        self.assertEqual(G50.r['f']['a'].oidPart, True)
        self.assertEqual(G50.r['f']['a'].rootOidPart, True)
        self.assertEqual(G50.r['f']['b'].oidPart, False)
        self.assertEqual(G50.r['f']['b'].rootOidPart, False)
        self.assertEqual(G50.r['g'].oidPart, False)
        self.assertEqual(G50.r['g'].rootOidPart, False)
        self.assertEqual(G50.r['g']['a'].oidPart, True)
        self.assertEqual(G50.r['g']['a'].rootOidPart, False)
        self.assertEqual(G50.r['g']['b'].oidPart, False)
        self.assertEqual(G50.r['g']['b'].rootOidPart, False)
        self.assertEqual(G50.r['h'].oidPart, False)
        self.assertEqual(G50.r['h'].rootOidPart, False)
        self.assertEqual(G50.s.oidPart, True)
        self.assertEqual(G50.s.rootOidPart, True)
        self.assertEqual(G50.s['a'].oidPart, True)
        self.assertEqual(G50.s['a'].rootOidPart, True)
        self.assertEqual(G50.s['b'].oidPart, False)
        self.assertEqual(G50.s['b'].rootOidPart, False)
        self.assertEqual(G50.t.oidPart, False)
        self.assertEqual(G50.t.rootOidPart, False)

    def test53_OidManagement(self):
        e = G50(r=F50(f=E50(a=1, b=2)), s=E50(a=3,b=4))
        m = TC(G50)

        o = m.oidFromObj(e)
        self.assertEqual(o, (1,3))

        self.assertEqual(m['r'].valueFromOID(o), e.r)
        self.assertEqual(m['r']['f'].valueFromOID(o), e.r.f)
        self.assertEqual(m['r']['f']['a'].valueFromOID(o), 1)
        self.assertEqual(m['s'].valueFromOID(o), e.s)
        self.assertEqual(m['s']['a'].valueFromOID(o), 3)

        self.assertEqual(m.oidToAttrs(o), {'r':e.r,'s':e.s})
        self.assertEqual(m.oidToPrimitives(o), {'r':1,'s':3})

        self.assertEqual(m.oidFromInput((e.r,e.s)), (1,3))
        self.assertRaises(ValueError, m.oidFromInput, 1)
        self.assertRaises(ValueError, m.oidFromInput, (e.s, e.r))

        self.assertEqual(e._oid, (1,3))

def suite():
    return TestSuite((
        makeSuite(TestMapping),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
