# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.schema.api import *
from adt4.enums import *
from datetime import date, datetime

class TestNormalize (TestCase) :

    def test10_Integer (self) :
        norm_int = lambda v: normalize(v, int)
        self.assertEqual(norm_int(1), 1)
        self.assertEqual(norm_int(1.1), 1)
        self.assertEqual(norm_int('1'), 1)
        self.assertEqual(norm_int(None), None)
        self.assertRaises(ValueError, norm_int, 'a')


    def test11_Date (self) :
        norm_date = lambda v: normalize(v, date)
        self.assertEqual(norm_date(date(2004,1,2)), date(2004,1,2))
        self.assertEqual(norm_date('2004-1-2'), date(2004,1,2))
        self.assertEqual(norm_date('2.1.2004'), date(2004,1,2))
        self.assertEqual(norm_date('today'), date.today())
        self.assertEqual(norm_date((2004,1,2)), date(2004,1,2))
        self.assertEqual(norm_date(None), None)
        self.assertRaises(ValueError, norm_date, 'ax')

    def test12_Datetime (self) :
        from lib4.timeutils import Datetime as DT
        norm_dtime = lambda v: normalize(v, datetime)
        dtstr = '2004-1-2 11:10'
        self.assertEqual(norm_dtime(DT(dtstr)), DT(dtstr))
        self.assertEqual(norm_dtime(dtstr), DT(dtstr))
        self.assertEqual(norm_dtime(None), None)
        self.assertEqual(norm_dtime('2.1.2004 11:10'),DT(dtstr))
        assert isinstance(norm_dtime('now'), datetime)
        self.assertRaises(ValueError, norm_dtime, 'ax')

    def test13_Class (self) :
        class C (object) :
            def __init__ (self, a) :
                self.a = normalize(a, int)
            def __cmp__ (self, other) :
                return cmp(self.a, other.a)
        self.assertEqual(normalize(C(1), C), C(1))
        self.assertEqual(normalize(C('1'), C), C(1))

    def test14_Enums (self) :
        class Kind (Enum) :
            system = enum()
            user   = enum()
        assert normalize(None, Kind) is None
        assert normalize(Kind.user, Kind) is Kind.user
        assert normalize('user', Kind) is Kind.user
        self.assertRaises(KeyError, normalize, 'neex', Kind)
        self.assertRaises(KeyError, normalize, Kind, Kind)

    def test15_Float (self) :
        norm_float = lambda v: normalize(v, float)
        self.assertEqual(norm_float(1), 1.)
        self.assertEqual(norm_float(1.1), 1.1)
        self.assertEqual(norm_float('1.1'), 1.1)
        self.assertEqual(norm_float('1,1'), 1.1)
        self.assertEqual(norm_float(None), None)
        self.assertRaises(ValueError, norm_float, 'a')



def suite() :
    return TestSuite((
        makeSuite(TestNormalize),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
