from unittest import TestCase
from wm4.api import *
from urllib import quote

__all__ = [
'with_ini', 'assertEntityError', 'assertEqualEntityError', 'MockWorkspaceCase']


def with_ini(ini, **add_globals):
    u = 'data:,' + quote(ini)
    def ini_wrapper(func):
        def f(*a, **k):
            config.parseConfig(u, **add_globals)
            return func(*a, **k)
        return f
    return ini_wrapper



def assertEntityError(test, featErrors, fun, *args, **kwds):
    try:
        fun(*args, **kwds)
    except schema.EntityError, ee :
        assertEqualEntityError(test, ee, featErrors)
    else:
        test.fail('EntityError was not raised')


def assertEqualEntityError(test, ee, featErrors):
    expected  = {}
    for attr,errCls in featErrors:
        key = (attr, errCls)
        expected[key] = expected.get(key, 0) + 1

    for attr,errList in ee.errors.iteritems():
        for errInst in errList:
            errCls = errInst.__class__
            key = (attr, errCls)

            if key not in expected :
                test.fail('Unexpected %s found' % repr(key))

            count = expected[key]
            if count < 1 :
                test.fail('Unexpected count of %s' % repr(key))

            expected[key] = count - 1

    for key, count in expected.iteritems() :
        if count > 0 :
            test.fail('Expected %s not found' % repr(key))


class MockWorkspaceCase(TestCase):

    cfgURLs = ('pkgfile:wm4/core.ini',)

    def setUp(self):
        self.old = config.snapshot()
        config.loadConfig(*self.cfgURLs)
        settings.storage_url('mockdb:')

        self.con = settings.storage_connection()
        self.ws = settings.workspace()
        self.ts = settings.transaction_service()

    def tearDown(self):
        if self.ts.isActive():
            self.ts.abort()
        config.swap(self.old)
        self.con.verify()

    def expects(self, **r):
        self.con.expects(**r)
