from wm4.api import *
from _pg import INV_READ, INV_WRITE
from sql4.utils import bytea_escape, bytea_unescape
from adt4.blob import *


class PGBlobDM (binding.Component, AbstractBlobDM) :

    con = binding.Make(lambda: settings.storage_connection())

    bufSize = 131072

    def newOID (self) :
        return int((~self.con(
            "SELECT lo_creat(%d)" % (INV_READ|INV_WRITE)
        ))[0])

    def find (self, oid) :
        oid = int(oid)
        count = int((~self.con(
            "SELECT count(*) FROM pg_largeobject WHERE loid=%d" % oid
        ))[0])
        if count < 1 :
            raise KeyError('Blob not found', oid)
        else :
            return oid

    def delete (self, oid) :
        oid = int(oid)
        self.con("SELECT lo_unlink(%d)" % oid)

    def open (self, b, mode='r') :
        m = 0
        if 'r' in mode : m |= INV_READ
        if 'w' in mode : m |= INV_WRITE
        return int((~self.con(
            "SELECT lo_open(%d,%d)" % (b.oid, m)
        ))[0])

    def close (self, h) :
        self.con("SELECT lo_close(%d)" % h)

    def read (self, h, size=None) :
        if size is None :
            bs = self.bufSize
            data = []
            while True :
                d = (~self.con(
                    "SELECT loread(%d,%d)" % (h, bs)
                ))[0]
                if d.startswith('\\x'):
                    d = d[2:].decode('hex')
                else:
                    d = bytea_unescape(d)
                data.append(d)
                if len(d) < bs :
                    break
            data = ''.join(data)
        else :
            data = (~self.con(
                "SELECT loread(%d,%d)" % (h, size)
            ))[0]
            if data.startswith('\\x'):
                data = data[2:].decode('hex')
            else:
                data = bytea_unescape(data)
        return data

    def write (self, h, data) :
        return (~self.con(
            "SELECT lowrite(%d,E'%s'::bytea)" % (h, bytea_escape(data))
        ))[0]

    def seek (self, h, off, whence=0) :
        return int((~self.con(
            "SELECT lo_lseek(%d,%d,%d)" % (h, off, whence)
        ))[0])

    def tell (self, h) :
        return int((~self.con(
            "SELECT lo_tell(%d)" % h
        ))[0])












import os
import os.path
import errno
import random

from transactions import TransactionComponent, TxnAttr


class FSBlobDM (TransactionComponent, AbstractBlobDM) :
    NEW_OID_MAX_TRY = 10000
    DIR_MODS = (256, 16)
    base_dir = binding.Make(lambda: os.path.join(
        '/var/app4', settings.application_name(), 'blobs'))

    #TODO zmeny provadet v docasnych souborech, az pri commitu je prejmenovat
    createdOids = TxnAttr(lambda: [])
    removedOids = TxnAttr(lambda: [])

    def _dirname(self, oid):
        dirs = []
        num = oid
        for m in self.DIR_MODS:
            (_d, _m) = divmod(num, m)
            dirs.insert(0, str(_m))
            num = _d
        return os.path.join(self.base_dir, *dirs)

    def _filename(self, oid):
        return os.path.join(self._dirname(oid), str(oid))

    def _createfile(self, oid):
        flags = os.O_RDWR | os.O_CREAT | os.O_EXCL
        dirname = self._dirname(oid)
        if not os.path.isdir(dirname):
            os.makedirs(dirname)
        fd = os.open(self._filename(oid), flags, 0664)
        os.close(fd)

    def newOID(self):
        self.joinedTxn
        for seq in xrange(self.NEW_OID_MAX_TRY):
            oid = random.randint(0, 2147483647)
            try:
                self._createfile(oid)
                self.createdOids.append(oid)
                return oid
            except OSError, e:
                if e.errno == errno.EEXIST:
                    continue # try again
                raise
        raise ValueError('Unable to create blob file')

    def new_withoid (self, oid) :
        '''create new blob with given oid.'''
        self._createfile(oid)
        b = object.__new__(Blob)
        b.oid = oid
        b.bdm = self
        b.handler = None
        return b


    def find (self, oid) :
        if not os.path.isfile(self._filename(oid)):
            raise KeyError('Blob not found', oid)
        return oid

    def delete (self, oid) :
        self.joinedTxn
        self.removedOids.append(oid)

    def open (self, b, mode='r') :
        self.joinedTxn
        return file(self._filename(b.oid), mode)

    def close (self, h) :
        self.joinedTxn
        h.close()

    def read (self, h, size=None) :
        self.joinedTxn
        return h.read(size or -1)

    def write (self, h, data) :
        self.joinedTxn
        return h.write(data)

    def seek (self, h, off, whence=0) :
        self.joinedTxn
        return h.seek(off, whence)

    def tell (self, h) :
        self.joinedTxn
        return h.tell()




    def abortTransaction(self, txnService):
        for oid in self.createdOids:
            os.unlink(self._filename(oid))

    def commitTransaction(self, txnService):
        for oid in self.removedOids:
            os.unlink(self._filename(oid))
