# -*- coding: utf-8 -*-

import re

from wm4.api import *

__all__ = ['ldap_url', 'LDAPURL', 'LDAPConnection']


ldap_url = config.Setting('ldap_url')



class LDAPURL(naming.AbstractURL):

    scheme = None

    SCHEMES = set(['ldap', 'ldaps', 'ldapi'])

    slots = 'user', 'password', 'host', 'port'

    MATCH = re.compile(
        ('^//'
         '((?P<user>[^/:@]+)(:(?P<password>[^/:@]+))?@)?'
         '((?P<host>%s)(:(?P<port>%s))?)?') % (
         naming.DNSNAME_RE, naming.PORT_RE), re.I).match

    def __str__(self):
        u = '%s://' % self.scheme
        if self.user:
            u += self.user
            if self.password:
                u+= ':' + self.password
            u+= '@'
        if self.host:
            u+= self.host
            if self.port:
                u+= ':' + self.port
        return u

    def getObject(self):
        return LDAPConnection(address=self)


class LDAPConnection(binding.Component):

    api = binding.Make(lambda: __import__('ldap'))

    address = binding.Require()

    def connection(self):
        a = self.address
        c = self.api.open(a.host, a.port or 389)
        c.bind(a.user, a.password)
        return c
    connection = binding.Make(connection)

    def scope_map(self, scope):
        if isinstance(scope, basestring):
            return getattr(self.api, 'SCOPE_%s' % scope.upper())
        return scope

    def __call__(self, base, scope='onelevel', filter='(objectClass=*)'):
        return self.connection.search_s(base, self.scope_map(scope), filter)



if __name__ == '__main__':
    import sys, pprint, ldap

    url = 'ldap://cn=admin,dc=free2b,dc=cz:%s@localhost' % sys.argv[1]
    con = naming.lookup(url)
    print con
    print con.address
    print con.connection
    r = con.connection.search_s("ou=users,dc=free2b,dc=cz", ldap.SCOPE_SUBTREE, "objectclass=posixAccount")
    print pprint.PrettyPrinter(indent=2).pformat(r)
