from wm4.api import *
from transactions import TransactionComponent
from connection import DBAPIConnection
import re, difflib

__all__ = ['MockDB', 'MockSqlURL', 'MockSqlConnection']



class MockCursor:

    def __init__(self, expectations):
        self.expectations = expectations

    def execute(self, sql):
        assert self.expectations, 'expected %s but empty found' % repr(sql)
        exp = self.expectations.pop(0)
        assert 'sql' in exp, 'query %s not in %s' % (sql, exp)
        qry = exp['sql']
        if isinstance(qry, basestring):
            assert sql.split() == qry.split(), '%s\n%s' % (sql, qry)
            res = exp.get('res')
        else:
            res = qry(sql)
        if res is not None:
            self.res = list(res)
            self.rowcount = len(self.res)
        else:
            self.res = None
            self.rowcount = None
        self.description = exp.get('description')

    def fetchone(self):
        if self.res:
            return self.res.pop(0)
        return None

    def close(self):
        pass



class MockDB(TransactionComponent):

    expectations = binding.Require(suggestParent=False)

    def onJoinTxn(self, txnService):
        self.begin()

    def begin(self):
        assert self.expectations, 'expected begin but empty found'
        r = self.expectations.pop(0)
        assert r.get('transaction') == 'begin', '%s x "begin"' % repr(r)

    def commit(self):
        assert self.expectations, 'expected commit but empty found'
        r = self.expectations.pop(0)
        assert r.get('transaction') == 'commit', '%s x "commit"' % repr(r)

    def rollback(self):
        assert self.expectations, 'expected abort but empty found'
        r = self.expectations.pop(0)
        assert r.get('transaction') == 'abort', '%s x "abort"' % repr(r)

    def cursor(self):
        self.joinedTxn
        return MockCursor(self.expectations)




class MockSqlURL(naming.AbstractURL):

    scheme = None
    slots = ()

    MATCH = re.compile('^$').match

    def __str__(self):
        return '%s:' % self.scheme

    SCHEMES = set(['mockdb'])

    def getObject(self):
        return MockSqlConnection(address=self)


class MockSqlConnection(DBAPIConnection):

    def open(self, addr):
        import mockdb
        return mockdb.MockDB(expectations=self.expectations)

    expectations = binding.Make(list)

    def expects(self, **r):
        self.expectations.append(r)

    def verify(self):
        assert not self.expectations, repr(self.expectations)
