from wm4.api import *
from sql4.utils import *
from datetime import date, datetime

__all__ = ['SqlTypeDescriptor']





class SqlTypeDescriptor(config.TypeProperty):
    '''Returns object that handles type conversions'''







class SQLDefault:
    def __init__(self, typ):
        self.typ = typ
        self.norm = schema.Normalize.of(typ)
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            v = sql_escape(str(val))
            if '\\' in v:
                return "E'%s'" % v
            else:
                return "'%s'" % v
    def fromSql(self, val):
        if val is None:
            return None
        else:
            return self.norm(val)
SqlTypeDescriptor('*', lambda r,f: SQLDefault(r))


class SQLNonQuoting(SQLDefault):
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            return "%s" % str(val)
SqlTypeDescriptor(int, lambda r,f: SQLNonQuoting(r))
SqlTypeDescriptor(long, lambda r,f: SQLNonQuoting(r))
SqlTypeDescriptor(float, lambda r,f: SQLNonQuoting(r))


class SQLUnicode:
    def __init__(self, typ):
        self.typ = typ
        self.norm = schema.Normalize.of(typ)
        self.con = settings.storage_connection()
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            v = sql_escape(unicode(val).encode(self.con.encoding))
            if '\\' in v:
                return "E'%s'" % v
            else:
                return "'%s'" % v
    def fromSql(self, val):
        if val is None:
            return None
        else:
            if isinstance(val, unicode):
                return self.typ(val)
            else:
                return self.typ(unicode(val, self.con.encoding))
SqlTypeDescriptor(unicode, lambda r,f: SQLUnicode(r))


class SQLBool(SQLDefault):
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            return val and "'t'" or "'f'"
    def fromSql(self, val):
        if val is None:
            return None
        elif isinstance(val,bool):
            return val
        else:
            return val in ('t', 'True', 'true')
SqlTypeDescriptor(bool, lambda r,f: SQLBool(r))


from adt4.blob import Blob
class SQLBlob:
    def __init__(self, typ):
        self.con = settings.storage_connection()
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            return "%d" % val.oid
    def fromSql(self, val):
        if val is None:
            return None
        else:
            bdm = settings.blob_manager()
            return bdm._preload(int(val))
SqlTypeDescriptor(Blob, lambda r,f: SQLBlob(r))


from adt4.binary import Binary
class SQLBinary:
    def __init__(self, typ):
        pass
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            return "E'%s'::bytea" % bytea_escape(val)
    def fromSql(self, val):
        if val is None:
            return None
        else:
            return Binary(bytea_unescape(val))
SqlTypeDescriptor(Binary, lambda r,f: SQLBinary(r))


from adt4.enums import Enum
class SQLEnum:
    def __init__(self, typ):
        self.typ = typ
    def toSql(self, val):
        if val is None:
            return 'NULL'
        else:
            return "'%s'" % sql_escape(val.name)
    def fromSql(self, val):
        if val is None:
            return None
        else:
            return self.typ[val]
SqlTypeDescriptor(Enum, lambda r,f: SQLEnum(r))
