from unittest import *
from wm4.api import *
from urllib import quote
from lib4.imports import importObject
from lib4.timeutils import *
from types import ClassType
from lib4.cgi import *
import types, sys, new, unittest

__all__ = [
    'BrokenTest', 'broken_test_XXX', 'find_broken_tests',
    'main', 'TestCase', 'TestSuite',  'FunctionTestCase',
    'Suite', 'makeSuite', 'Main',
    'Record', 'TEST_EPOCH', 'MyTimeService', 'with_ini',
    'ConfiguredTestCase', 'MockWorkspaceCase', 'SqliteWorkspaceCase',
    'test_environ', 'test_req', 'with_req', 'WebCase',
]


def test_case_method(func):
    amg = globals().copy()
    amg.update(dict(__unittest=1))
    setattr(TestCase, func.func_name, new.function(
        func.func_code, amg, func.func_name, func.func_defaults,
    ))

@test_case_method
def assertEqualList(self, first, second, msg=None):
    first = tuple(first)
    second = tuple(second)
    fl = len(first)
    sl = len(second)
    diff = []
    for i in range(max(fl, sl)):
        if i >= fl:
            diff.append('[%d]:<MISSING>\n[%d]:%s' % (i, i, second[i]))
        elif i >= sl:
            diff.append('[%d]:%s\n[%d]:<MISSING>' % (i, first[i], i))
        elif first[i] != second[i]:
            diff.append('[%d]:%s\n[%d]:%s' % (i, first[i], i, second[i]))
    if diff:
        raise self.failureException('\n'+'\n'.join(diff))

@test_case_method
def assertEqualDict(self, first, second, msg=None):
    diff = []
    for k in sorted(first.keys() + second.keys()):
        if k not in first:
            diff.append('[%s]:<MISSING>\n[%s]:%s' % (k, k, second[i]))
        elif k not in second:
            diff.append('[%s]:%s\n[%s]:<MISSING>' % (k, k, first[i]))
        elif first[k] != second[k]:
            diff.append('[%s]:%s\n[%s]:%s' % (k, first[k], k, second[k]))
    if diff:
        raise self.failureException('\n'+'\n'.join(diff))





# Broken test support from recipe
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/466288

class BrokenTest(TestCase.failureException):
    def __repr__(self):
        name, reason = self.args
        return '%s: %s: %s works now' % (
            (self.__class__.__name__, name, reason))

def broken_test_XXX(reason, *exceptions):
    '''Indicates a failing (or erroneous) test case fails that should succeed.
    If the test fails with an exception, list the exception type in args'''
    def wrapper(test_method):
        def replacement(*args, **kwargs):
            try:
                test_method(*args, **kwargs)
            except exceptions or TestCase.failureException:
                pass
            else:
                raise BrokenTest(test_method.__name__, reason)
        replacement.__doc__ = test_method.__doc__
        replacement.__name__ = 'XXX_' + test_method.__name__
        replacement.todo = reason
        return replacement
    return wrapper

def find_broken_tests(module):
    '''Generate class, methodname for test cases marked "broken".'''
    for class_name in dir(module):
        class_ = getattr(module, class_name)
        if (isinstance(class_, (type, types.ClassType)) and
                      issubclass(class_, TestCase)):
            for test_name in dir(class_):
                if test_name.startswith('test'):
                    test = getattr(class_, test_name)
                    if (hasattr(test, '__name__') and
                                test.__name__.startswith('XXX_')):
                        yield class_, test_name








_this_dict = sys.modules[__name__].__dict__

def is_case(obj, mod):
    if not isinstance(obj, (ClassType, type)):
        return False
    if not issubclass(obj, TestCase):
        return False
    if obj in _this_dict:
        return False
    if obj.__module__ != mod.__name__:
        return False
    return True


class Suite:

    def __init__(self, *cases):
        self.cases   = cases
        self.modname = sys._getframe(1).f_globals['__name__']
        self.globals = sys.modules[self.modname].__dict__

    def __call__(self):
        cases = []
        for c in self.cases:
            c = importObject(c, self.globals)
            if isinstance(c, types.ModuleType):
                cases.append(c.suite())
            else:
                cases.append(TestSuite(makeSuite(c)))
        return TestSuite(cases)



def Main(defaultTest='suite') :
    main(defaultTest='suite')





class Record(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def __eq__(self, other):
        if self.__class__ != other.__class__:
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other

    def copy(self):
        return self.__class__(**self.__dict__)

    def __repr__(self):
        return 'Record(%s)' % ','.join(
            '%s=%r' % (a,v) for a,v in self.__dict__.iteritems()
        )




TEST_EPOCH = Datetime(2006,4,28,10,15,0, tzinfo=TZUTC)

class MyTimeService(TimeService):

    def __init__(self, start=TEST_EPOCH, time_incr=SECOND):
        self.stamp = start
        self.time_incr = time_incr

    def today(self):
        return Date(self.stamp)

    def now(self):
        s = self.stamp
        self.stamp = s + self.time_incr
        return s




def with_ini(ini, **add_globals):
    u = 'data:,' + quote(ini)
    def ini_wrapper(func):
        def f(*a, **k):
            config.loadConfig(u, **add_globals)
            return func(*a, **k)
        f.__name__ = func.__name__
        return f
    return ini_wrapper



class ConfiguredTestCase(TestCase, object):

    cfgURLs = ('pkgfile:wm4/core.ini',)

    def setUp(self):
        self.old = config.snapshot()
        config.loadConfig(*self.cfgURLs)

    def tearDown(self):
        config.swap(self.old)

    def parse_ini(self, lines):
        from wm4.config.ini_files import ConfigReader
        ConfigReader().readString(lines)







class MockWorkspaceCase(ConfiguredTestCase):

    def setUp(self):
        super(MockWorkspaceCase, self).setUp()
        settings.storage_url('mockdb:')
        self.con = settings.storage_connection()
        self.ws = settings.workspace()
        self.ts = settings.transaction_service()

    def tearDown(self):
        if self.ts.isActive():
            self.ts.abort()
        self.con.verify()
        super(MockWorkspaceCase, self).tearDown()

    def expects(self, **r):
        self.con.expects(**r)


class SqliteWorkspaceCase(ConfiguredTestCase):

    def setUp(self):
        super(SqliteWorkspaceCase, self).setUp()
        settings.storage_url('sqlite:')
        self.con = settings.storage_connection()
        self.ws = settings.workspace()
        self.ts = settings.transaction_service()

    def tearDown(self):
        if self.ts.isActive():
            self.ts.abort()
        super(SqliteWorkspaceCase, self).tearDown()


    def _schema(self, *ddls):
        cur = self.con.dbcon.cursor()
        for ddl in ddls:
            for cmd in open(ddl).read().split(';'):
                cmd = cmd.strip()
                if not cmd:
                    continue
                if cmd.upper().split()[0] in ('BEGIN','START','END','COMMIT'):
                    continue
                if cmd.upper().startswith('CREATE TRIGGER'):
                    continue
                cur.execute(cmd)
        cur.close()
        cur = self.con.dbcon.commit()


def test_req(url='/',env={},get={},post={},cookies={},auth=None):
    env,inp = test_environ(url,env,get,post,cookies,auth)
    sane_environ(env)
    return web.Request(environ=env)

def with_req(url='/',env={},get={},post={},cookies={},auth=None):
    def req_wrapper(func):
        def f(*a, **k):
            req = test_req(url,env,get,post,cookies,auth)
            exit = config.enter_(web.request(req))
            try:
                return func(*a, **k)
            finally:
                exit()
        f.__name__ = func.__name__
        return f
    return req_wrapper


class WebCase(ConfiguredTestCase):

    cfgURLs = ('pkgfile:wm4/core.ini', 'pkgfile:wm4.web/web.ini')

    def env(self,url='/',env={},get={},post={},cookies={},auth=None):
        env,inp = test_environ(url,env,get,post,cookies,auth)
        sane_environ(env)
        return env

    def req(self,url='/',env={},get={},post={},cookies={},auth=None):
        return test_req(url,env,get,post,cookies,auth)

    def mkapp(self, *components):
        return web.makeApp(components=components)

    def handle(self, app, env=None):
        if env is None:
            env = self.env()
        sane_environ(env)
        resp = [None,None]
        def start_response(status, response_headers, exc_info=None):
            resp[0] = status
            resp[1] = response_headers
        body = app(env, start_response)
        s = resp[0]
        h = resp[1]
        b = ''.join(body)
        return s,h,b

    def parse(self, src, tmpl_namespace={}):
        exit_ = config.enter_(config.Config())
        try:
            web.TEMPLATE_GLOBALS(tmpl_namespace)
            t = parseSrc(src)
        finally:
            exit_()
        return Template(template=t)


    def render(self, tmpl, req=None, tmpl_namespace={}, **params):
        if req is None:
            req = self.req()
        if params:
            req.params.update(params)
        if isinstance(tmpl, basestring):
            tmpl = self.parse(tmpl, tmpl_namespace=tmpl_namespace)
        else:
            assert not tmpl_namespace
        return tmpl.render(req)

    def assertEqualXML(self, first, second):
        if not first == second:
            for i, (f,s) in enumerate(map(None,
                first.splitlines(), second.splitlines()
            )):
                if f != s:
                    raise self.failureException(
                        'Markup differs on line %d %r x %r' % (i+1, f, s)
                    )
