# http://pypi.python.org/pypi/monkey
# http://mail.python.org/pipermail/python-dev/2008-January/076194.html

from peak.util.decorators import decorate_class
from types import ClassType
import new

__all__ = ['patch', 'patchClass']




def patch(attrs):
    '''Patches class hierarchy

        class A:
            a1 = 'A1'
            class B:
                def b1(): return 'B1'
                b2 = 'B2'
                class C:
                    c1 = 'C1'
                    c2 = 'C2'

        class PA(A):
            patch({
                'a1'     : lambda: 'PA1',
                'B.b1'   : 'PB1',
                'B.C.c1' : lambda: 'PC1',
            })
    '''

    def callback(klass):
        patchClass(klass, attrs)
        return klass

    decorate_class(callback)




def patchClass(klass, attrs):
    if not attrs:
        return klass

    gp = {(): PatchInfo(klass)}

    for spec, val in attrs.items():
        spec = spec.split('.')
        path = tuple(spec[:-1])
        attr = spec[-1]

        for pp in iterParents(path):
            info = gp.get(pp)
            if info is None:
                info = PatchInfo(findClass(klass, pp))
                gp[pp] = info

        info.cdict[attr] = val

    for path, info in sorted(gp.items(), key=lambda i: -len(i[0])):
        info.cdict.setdefault('__module__', info.old.__module__)
        if path:
            cls = new.classobj(info.old.__name__, (info.old,), info.cdict)
            parent, pattr = path[:-1], path[-1]
            gp[parent].cdict[pattr] = cls

    for attr, val in gp[()].cdict.iteritems():
        setattr(klass, attr, val)
    return klass











class PatchInfo:

    old = new = patch = None

    def __init__(self, old):
        self.old = old
        self.cdict = {}



def findClass(root, path):
    cls = root
    if not isinstance(cls, (type, ClassType)):
        raise TypeError('%r is not class object', cls)
    for a in path:
        cls = getattr(cls, a)
        if not isinstance(cls, (type, ClassType)):
            raise TypeError('%r is not class object', cls)
    return cls

def iterParents(path):
    yield ()
    for i in xrange(1, len(path)+1):
        yield tuple(path[:i])
