from wm4.core import *
from dispatch.functions import GenericFunction
from dispatch.strategy import default, Argument, Inequality, Signature
from peak.util.decorators import decorate_class, decorate_assignment, frameinfo

from dispatch.predicates import CriteriaBuilder, Const, Call
from dispatch.ast_builder import parse_expr
from dispatch.interfaces import ICriterion
import inspect, sys, operator


__all__ = ['VarArgsFunction']


class VarArgsFunction(GenericFunction):

    def __init__(self):
        super(VarArgsFunction,self).__init__(lambda ac,args:None)

    def when(self,cond=default): return self._decorate(cond)
    def around(self,cond=default): return self._decorate(cond,'around')
    def before(self,cond=default): return self._decorate(cond,'before')
    def after(self,cond=default): return self._decorate(cond,'after')

    def parseWithFunction(self, cond, func, *namespaces):
        if isinstance(func,tuple):
            _,func = func

        args, star, dstar, defaults = inspect.getargspec(func)
        if dstar:
            raise TypeError('Keywords arguments are not supported')
        if not defaults:
            defaults = ()
        if args and args[0] == 'next_method':
            args = args[1:]

        amax = star and sys.maxint or len(args)
        amin = len(args) - len(defaults)

        precond = Signature([(
            Argument(pos=0),
            Inequality('..', [(amin,amin),(amin,amax),(amax,amax)])
        )])

        if isinstance(cond, basestring):
            argNS = dict(
                (a, Call(operator.getitem,Argument(pos=1),Const(i)))
                for i,a in enumerate(args)
            )
            builder=CriteriaBuilder(argNS, *namespaces)
            cond = parse_expr(cond, builder)
        return precond & cond


    def _decorate(self, cond, qualifier=None, frame=None, depth=2):

        frame = frame or sys._getframe(depth)

        def registerMethod(frm,name,value,old_locals):
            if qualifier is None:
                func = value
            else:
                func = qualifier,value

            sig = self.parseWithFunction(
                cond, func, frame.f_locals, frame.f_globals, frame.f_builtins
            )

            kind,module,locals_,globals_ = frameinfo(frm)
            if kind=='class':
                # 'when()' in class body; defer adding the method
                def registerClassSpecificMethod(cls):
                    req = Signature(
                        [(Argument(0),ICriterion(cls))]
                    )
                    self.addMethod(req & sig, func)
                    return cls
                decorate_class(registerClassSpecificMethod, frame=frm)
            else:
                self.addMethod(sig, func)

            if old_locals.get(name) in (self,self.delegate):
                return self.delegate

            return value

        return decorate_assignment(registerMethod,frame=frame)

    def __call__(self, *args):
        meth = self[(len(args), args)]
        return meth(*args)
