from protocols import Interface,adapt,advise,declareAdapter,NO_ADAPTER_NEEDED
from types import FunctionType
from lib4.imports import importObject
from lib4.runtime import compile_with_lineinfo


__all__ = ['IRule', 'Value', 'Rule', 'ExpressionRule', 'ImportRule']


class IRule(Interface):

    def __call__(search, found):
        '''Compute value for property `search' with rule found under
           property `found'.
        '''

declareAdapter(NO_ADAPTER_NEEDED, provides=[IRule], forTypes=[FunctionType])



class Value(object):

    __slots__ = 'value',

    advise(instancesProvide=[IRule], asAdapterForTypes=[object])

    def __init__(self, value):
        self.value = value

    def __call__(self, lookProp, foundProp):
        return self.value

    def __repr__(self):
        return 'Value(%s)' % repr(self.value)



class Rule(object):

    __slots__ = 'callable',

    advise(instancesProvide=[IRule])

    def __init__(self, callable):
        self.callable = callable

    def __call__(self, property, foundProperty):
        return self.callable(property, foundProperty)

    def __repr__(self):
        return 'Rule(%s)' % repr(self.callable)


class ExpressionRule(object):

    __slots__ = 'expr', 'code', 'gDict'

    advise(instancesProvide=[IRule])

    def __init__(self, expr, gDict, name='expr',filename='<str>', lineno=1):
        self.expr = expr
        self.code = compile_with_lineinfo(expr, filename, lineno, name)
        self.gDict = gDict

    def __call__(self, property, foundProperty):
        return eval(self.code, self.gDict, locals())

    def __repr__(self):
        return 'ExpressionRule(%s)' % repr(self.expr)

class ImportRule(object):

    __slots__ = 'expr',

    advise(instancesProvide=[IRule])

    def __init__(self, expr):
        self.expr = expr

    def __call__(self, property, foundProperty):
        return importObject(self.expr)

    def __repr__(self):
        return 'ImportRule(%s)' % repr(self.expr)
