from unittest import *
from wm4.utils.clspatch import *



class TestClassPatch(TestCase):

    def test01_EmptyPatch(self):
        class A:
            patch({})

    def test02_PatchNonclass(self):

        class A:
            class Z:
                a = 43
            z = Z()

        self.assertRaises(TypeError, patchClass, A, {'Z.a':42, 'z.a':42})

    def test10_OldStyleClass(self):

        class A:
            a1 = 'A1'
            class B:
                def b1():
                    return 'B1'
                b2 = 'B2'
                class C:
                    c1 = 'C1'
                    c2 = 'C2'
                class X:
                    class Y:
                        class Z:
                            pass

        class PA(A):
            a1 = 'PA1'
            a2 = 'PA2'

            patch({
                'a3'      : 'PA3',
                'B.b1'    : 'PB1',
                'B.C.c1'  : 'PC1',
                'B.C.c2'  : 'PC2',
                'B.C.c3'  : 'PC3',
                'B.X.Y.Z' : None,
            })

        self.assertEqual(PA.a1, 'PA1')
        self.assertEqual(PA.a2, 'PA2')
        self.assertEqual(PA.a3, 'PA3')

        self.assertEqual(PA.B.b1, 'PB1')
        self.assertEqual(PA.B.b2, 'B2')

        self.assertEqual(PA.B.C.c1, 'PC1')
        self.assertEqual(PA.B.C.c2, 'PC2')
        self.assertEqual(PA.B.C.c3, 'PC3')

        self.assertEqual(PA.B.X.Y.Z, None)


    def test20_NewStyleClass(self):

        class A(object):
            a1 = 'A1'
            class B(object):
                def b1():
                    return 'B1'
                b2 = 'B2'
                class C(object):
                    c1 = 'C1'
                    c2 = 'C2'
                class X(object):
                    class Y(object):
                        class Z(object):
                            pass

        class PA(A):
            a1 = 'PA1'
            a2 = 'PA2'

            patch({
                'a3'      : 'PA3',
                'B.b1'    : 'PB1',
                'B.C.c1'  : 'PC1',
                'B.C.c2'  : 'PC2',
                'B.C.c3'  : 'PC3',
                'B.X.Y.Z' : None,
            })

        self.assertEqual(PA.a1, 'PA1')
        self.assertEqual(PA.a2, 'PA2')
        self.assertEqual(PA.a3, 'PA3')

        self.assertEqual(PA.B.b1, 'PB1')
        self.assertEqual(PA.B.b2, 'B2')

        self.assertEqual(PA.B.C.c1, 'PC1')
        self.assertEqual(PA.B.C.c2, 'PC2')
        self.assertEqual(PA.B.C.c3, 'PC3')

        self.assertEqual(PA.B.X.Y.Z, None)


def suite():
    return TestSuite((
        makeSuite(TestClassPatch),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
