from wm4.core import *
from unittest import *
from wm4.utils.dispatching import *

class X: pass
class Y(X): pass
class Z(X): pass

class O: pass

x = X()
y = Y()
z = Z()

VAF = VarArgsFunction

AM = dispatch.AmbiguousMethod
NAM = dispatch.NoApplicableMethods

class TestMatcher(TestCase):

    def test01_DispatchByNewStyleClasses(self):
        vaf = VAF()
        @vaf.when('a in object')
        def f(a): return 'obj'
        @vaf.when('a in int')
        def f(a): return 'int'
        @vaf.when('a in str')
        def f(a): return 'str'

        self.assertEqual(vaf(42), 'int')
        self.assertEqual(vaf(3.14), 'obj')
        self.assertEqual(vaf('foo'), 'str')
        self.assertEqual(vaf(x), 'obj')

    def test02_DispatchByOldStyleClasses(self):
        vaf = VAF()
        @vaf.when('a in X')
        def f(a): return 'X'
        @vaf.when('a in Y')
        def f(a): return 'Y'

        self.assertEqual(vaf(x), 'X')
        self.assertEqual(vaf(y), 'Y')
        self.assertEqual(vaf(z), 'X')
        self.assertRaises(NAM, vaf, O())

    def test03_DefaultType(self):
        vaf = VAF()
        @vaf.when()
        def f(a): return 'D'
        @vaf.when('a in Y')
        def f(a): return 'Y'
        @vaf.when('a in int')
        def f(a): return 'I'

        self.assertEqual(vaf(y), 'Y')
        self.assertEqual(vaf(42), 'I')
        self.assertEqual(vaf('foo'), 'D')

    def test10_DefaultValues(self):
        vaf = VAF()
        @vaf.when('a in X')
        def f(a,b=42,c='foo'): return (a,b,c)

        self.assertEqual(vaf(x), (x,42,'foo'))
        self.assertEqual(vaf(x,1), (x,1,'foo'))
        self.assertEqual(vaf(x,1,'bar'), (x,1,'bar'))

    def test15_DispatchByArgumentCount(self):
        vaf = VAF()
        @vaf.when('a in int')
        def f(a): return 1
        @vaf.when('a in int and b in int')
        def f(a,b): return 2
        @vaf.when('a in int and b in int and c in int')
        def f(a,b,c): return 3

        self.assertRaises(NAM, vaf)
        self.assertEqual(vaf(0), 1)
        self.assertEqual(vaf(0,0), 2)
        self.assertEqual(vaf(0,0,0), 3)
        self.assertRaises(NAM, vaf, 0, 0, 0, 0)

    def test16_EmptyRule(self):
        vaf = VAF()
        @vaf.when()
        def f(): return 'e'

        self.assertEqual(vaf(), 'e')
        self.assertRaises(NAM, vaf, 0)

    def test17_StarArgs(self):
        vaf = VAF()
        @vaf.when('a in int')
        def f(a,*s): return (a,) + s

        self.assertEqual(vaf(0), (0,))
        self.assertEqual(vaf(0,0), (0,0))
        self.assertEqual(vaf(0,0,0), (0,0,0))

    def test30_AdditionalConditions(self):
        vaf = VAF()
        @vaf.when('a in int and a == 1')
        def f(a,*s): return 1
        @vaf.when('a in int and a == 2')
        def f(a,*s): return 2
        @vaf.when('a in int')
        def f(a,*s): return 0

        self.assertEqual(vaf(0), 0)
        self.assertEqual(vaf(1), 1)
        self.assertEqual(vaf(2), 2)
        self.assertEqual(vaf(3), 0)


def suite():
    return TestSuite((
        makeSuite(TestMatcher),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
