import cgi
import os
import sys
from urllib import quote
from cStringIO import StringIO
from new import instancemethod
from wsgiref.util import shift_path_info

from lib4.urls import URL
from lib4.imports import importObject
from lib4.data.languages import LANGUAGES

from wm4.api import *
from wm4.utils.files import IFileInfo

import interfaces, errors, utils


__all__ = [
    'Fields', 'URL', 'serverURL', 'appURL', 'requestURL', 'requestURI',
    'Request', 'Response', 'EnvironmentMiddleware',
]


class Fields(cgi.FieldStorage, object):

    __metaclass__ = binding.Activator

    @classmethod
    def get(klass, env):
        self = env.get('windmill.fields')
        if self is None:
            fact = env.get('windmill.fields.factory', klass)
            self = env['windmill.fields'] = fact(
                environ=env, fp=env['wsgi.input'], keep_blank_values=1
            )
        return self

    encoding = 'utf-8' # XXX

    uploadManager = binding.Make(
                        lambda: importObject(interfaces.upload_manager())())

    def make_file(self, binary=''):
        if sys.platform == 'win32':
            class W:
                def __init__(self, fd):
                    self.fd = fd
                def write(self, data):
                    return self.fd.write(data)
                def seek(self, *a):
                    return self.fd.seek(*a)
                def read(self, *a):
                    x = self.fd.read(*a)
                    return x
        else:
            W = lambda fd:fd
        fn = self.uploadManager.mktemp()
        if binary:
            return W(open(fn, 'w+b'))
        else:
            return W(open(fn, 'w+'))

    def getValue(self, key, default=None):
        val = self.getvalue(key, NOT_FOUND)
        if val is NOT_FOUND:
            return default
        else:
            if isinstance(val, list):
                raise errors.BadRequest(
                    'Single value for %s required but many found' % key
                )
            try:
                return unicode(val, self.encoding)
            except UnicodeDecodeError:
                raise errors.BadRequest('not a utf-8 string')

    def getList(self, key):
        enc = self.encoding
        try:
            return [unicode(v,enc) for v in self.getlist(key)]
        except UnicodeDecodeError:
            raise errors.BadRequest('not a utf-8 string')

    def getUpload(self, key, default=None):
        if not self.has_key(key):
            return default
        val = self[key]
        return FileUpload.fromFieldStorage(val)

    def __repr__(self):
        if self.filename:
            return "Field(%r,filename=%r)" % (self.name, self.filename)
        else:
            return "Field(%r,value=%r)" % (self.name, self.value)


class FileUpload(object):

    protocols.advise(instancesProvide=[IFileInfo])

    def __init__(self, path, filename, content_type, size):
        self.path = path
        self.filename = filename
        self.content_type = content_type
        self.size = size

    @classmethod
    def fromFieldStorage(klass, fs):
        from lib4.content_types import guessContentType, guessExtension
        if fs.file is None or not os.path.isfile(getattr(fs.file,'name','')):
            value = fs.value
            if not fs.value and not fs.filename:
                return None
            umgr = importObject(interfaces.upload_manager())()
            path = umgr.mktemp()
            open(path, 'wb').write(value)
        else:
            path = fs.file.name
        if fs.filename:
            filename = unicode(fs.filename,'utf-8', 'ignore')
        else:
            filename = None

        if fs.type:
            content_type = fs.type
            if content_type in (
                'application/octet-stream',
                'application/force-download',
                'application/x-download',
                'application/unknown',
            ):
                content_type = guessContentType(filename, path=path)
            if guessExtension(content_type) == '.bin':
                content_type = guessContentType(filename, path=path)
        elif filename:
            content_type = guessContentType(filename, path=path)
        else:
            content_type = 'application/octet-stream'

        if not filename:
            ext = guessExtension(content_type)
            filename = fs.name + ext
        filename = (filename.split('/')[-1]).split('\\')[-1]
        if not isinstance(filename, unicode):
            filename = unicode(filename,'utf-8', 'ignore')
        size = os.stat(path).st_size
        return klass(path, filename, content_type, size)

    def getPath(self):
        return self.path


def serverURL(environ):
    u = environ['wsgi.url_scheme']+'://'
    if environ.get('HTTP_HOST'):
        u += environ['HTTP_HOST']
    else:
        u += environ['SERVER_NAME']
        if environ['wsgi.url_scheme'] == 'https':
            if environ['SERVER_PORT'] != '443':
                u += ':'+environ['SERVER_PORT']
        else:
            if environ['SERVER_PORT'] != '80':
                u += ':'+environ['SERVER_PORT']
    return URL(u)

def appURL(environ):
    return URL(serverURL(environ) + quote(environ.get('SCRIPT_NAME') or '/'))

def requestURL(environ):
    u = requestURI(environ)
    if u.params:
        return URL(u, params={})
    return u

def requestURI(environ):
    if 'REQUEST_URI' in environ:
        u = serverURL(environ) + (environ.get('REQUEST_URI') or '/')
    else:
        u = appURL(environ)
        path_info = quote(environ.get('PATH_INFO',''))
        if not environ.get('SCRIPT_NAME'):
            u += path_info[1:]
        else:
            u += path_info
        if environ.get('QUERY_STRING'):
            u += '?' + environ['QUERY_STRING']
    return URL(u)











class Request(object):

    __metaclass__ = binding.Activator


    def __init__(self, environ):
        self.environ = environ
        self._id_counter = 0

    logger = binding.Make(lambda: naming.lookup('logger:web'))

    req_log = binding.Make(list)

    def _envValue(meth):
        def env_value(s, d, a):
            if a in s.environ:
                return s.environ[a]
            else:
                v = s.environ[a] = importObject(meth)(s.environ)
            return v
        return binding.Make(env_value)

    params = binding.Make(dict)

    pathInfo   = binding.Make(lambda s: s.environ.get('PATH_INFO','/'))
    serverURL  = _envValue(serverURL)
    appURL     = _envValue(appURL)
    requestURL = _envValue(requestURL)
    requestURI = _envValue(requestURI)

    fields    = _envValue(Fields.get)
    getValue  = \
    getList   = \
    getUpload = \
    has_key   = \
    keys      = binding.Delegate('fields')

    def __contains__(self, key):
        return self.has_key(key)

    cookies   = _envValue('wm4.web.cookies._getCookies')
    getCookie = \
    setCookie = \
    delCookie = binding.Delegate('cookies')


    def session(self):
        from wm4.web.sessions import getSession
        return getSession(self.environ)
    session = binding.Make(session)

    def getResource(self, path):
        return web.getResource(path)

    response = binding.Make(lambda s: Response(s))

    output_encoding = 'utf-8'
    input_encoding = 'utf-8'


    def apply(self, callable):
        from lib4.runtime import signature
        args,_,_,defaults = signature(callable)
        vect = []
        req_attrs = set(('response','environ','user'))
        for a in args:
            if a == 'req':
                vect.append(self)
            elif a in req_attrs:
                vect.append(getattr(self,a))
            else:
                v = self.params.get(a,NOT_FOUND)
                if v is NOT_FOUND:
                    v = self.getValue(a,NOT_FOUND)
                if v is NOT_FOUND:
                    v = defaults.get(a)
                vect.append(v)
        return callable(*vect)

    # authentication, authorizaction

    def getUser(self, auth_domain=None):
        from authentication import getUser, DEFAULT_AUTH_DOMAIN
        if auth_domain is None:
            auth_domain = DEFAULT_AUTH_DOMAIN
        return getUser(self.environ, auth_domain)
    user = binding.Make(lambda s: s.getUser())

    def allows(self, perm):
        return interfaces.permission_checker().allows(perm, self.environ)

    # sitemap

    sitemap = binding.Make(lambda s: s.environ['windmill.sitemap'])
    route = binding.Make(lambda s: s.environ['windmill.route'])

    def route_for(self, *a):
        return self.sitemap.route_for(*a)

    def get_url(self, *a):
        r = self.route_for(*a)
        if r is None:
            raise ValueError('route not found', a)
        u = r.pathInfo
        a = self.appURL
        if a.endswith('/'):
            return URL(a + u[1:])
        else:
            return URL(a + u)

    def absURL(self, *a):
        return self.get_url(*a)

    def relURL(self, *a):
        return self.get_url(*a)


    # request kind

    method     = binding.Make(
                 lambda s: str(s.environ.get('REQUEST_METHOD','GET').upper()))

    accept = binding.Make(lambda s:(
        utils.parse_accepts(s.environ.get('HTTP_ACCEPT',''))))

    def accept_resolve(self, variants, default):
        for p, a in self.accept:
            a = a.split(';')[0]
            if a in variants:
                return a
        return default

    def content_length(self):
        try:
            return int(self.environ.get('CONTENT_LENGTH', 0))
        except:
            return 0
    content_length = binding.Make(content_length)

    content_type = binding.Make(lambda s: s.environ.get('CONTENT_TYPE'))

    # i18n

    accept_charset = binding.Make(lambda s:(
        utils.parse_accepts(s.environ.get('HTTP_ACCEPT_CHARSET',''))))

    input_encoding = 'utf-8' # XXX
    output_encoding = 'utf-8' # XXX

    accept_language = binding.Make(lambda s:(
        utils.parse_accepts(s.environ.get('HTTP_ACCEPT_LANGUAGE', ''))))

    force_language = binding.Make(lambda s: s.environ.get('FORCE_LANGUAGE'))

    url_language = binding.Make(lambda s: s.environ.get('URL_LANGUAGE'))

    def cookie_language(self):
        return self.getCookie('wm4lang', default=None)
    cookie_language = binding.Make(cookie_language)

    default_language = binding.Make(lambda: settings.language())

    def language(self):
        if self.force_language:
            lang = self.force_language
        else:
            if self.url_language:
                lang = self.url_language
            elif self.cookie_language:
                lang = self.cookie_language
            else:
                if self.accept_language:
                    lang = self.accept_language[0][1]
                else:
                    lang = self.default_language
        if not lang:
            return lang
        lang = str(lang)
        if len(lang) > 2:
            lang = lang[:2]
        return lang
    language = binding.Make(language)

    translation_service = binding.Make(lambda: settings.translation_service())

    def translate(self, msg):
        return self.translation_service.translate(msg, lang=self.language)

    # id generator

    def get_new_id(self):
        self._id_counter += 1
        return self._id_counter

    def subreq(self, url, environ=None):
        if environ is None:
            environ = self.environ
        apu = self.appURL
        url = self.requestURI / url

        # URL must be to the same application
        assert url.scheme == apu.scheme
        assert url.netloc == apu.netloc
        assert url.path.startswith(apu.path)

        environ = environ.copy()
        environ['HTTP_METHOD'] = 'GET'
        environ['CONTENT_TYPE'] = 'application/x-www-form-urlencoded'
        environ['CONTENT_LENGTH'] = '0'
        environ['PATH_INFO'] = url.path
        environ['QUERY_STRING'] = url.query or ''
        environ['wsgi.input'] = StringIO()
        exit = config.enter_(config.ResourceArea())
        try:
            app = web.makeApp()
            _resp = []
            def start_response(status, headers, exc_info=None):
                _resp.append(status)
                _resp.append(headers)
            body = app(environ, start_response)
            return _resp[0], _resp[1], list(body)
        finally:
            exit()


class Response(binding.Component):
    status  = '200 OK'
    headers = binding.Make(list)
    body    = []

    def addheader(self, name, value):
        self.headers.append((name,value))

    def replaceheader(self, name, value):
        raise NotImplementedError

    def removeheader(self, name):
        raise NotImplementedError

    def clear(self):
        self.status = self.__class__.status
        self.headers = []
        self.body = []
        return self

    def wsgiResponse(self):
        return self.status, self.headers, self.body



class EnvironmentMiddleware:

    requestFactory = Request
    fieldsFactory = Fields

    def __init__(
        self, application, set_env={}, default_env={}, url_languages=None,
    ):
        self.application = application
        self.set_env = set_env
        self.default_env = default_env
        self.url_languages = url_languages

    def set_url_language(self, environ):
        p1 = environ.get('PATH_INFO', '').lstrip('/').split('/')[0]
        lang = self.url_languages.get(p1)
        if lang is not None:
            assert shift_path_info(environ) == p1
            environ['URL_LANGUAGE'] = lang
        else:
            # default, aby se prerazily pripadne nastaveni cookies
            environ['URL_LANGUAGE'] = settings.language()

    def __call__(self, environ, start_response):
        if self.url_languages:
            self.set_url_language(environ)
        for k,v in self.set_env.iteritems():
            environ[k] = v
        for k,v in self.default_env.iteritems():
            environ.setdefault(k,v)

        exit_env = config.enter_(settings.environ(environ))

        req = self.requestFactory(environ=environ)
        exit_req = config.enter_(interfaces.request(req))

        try:
            return self.application(environ, start_response)
        finally:
            exit_req()
            exit_env()
