# -*- coding: utf-8 -*-
'''
    class UserLst(ListingCtrl):

        baseQuery = query.query('[u for u in User if u.uid > 1]')

        defaultOrder = '+login'

        class uid(Column):
            queryExpr = 'u.uid'
            ctrlLabel = u'UID'

        class login(InputFilter):
            queryExpr = 'u.login'
            ctrlLabel = u'Login'

        class name(InputFilter):
            queryExpr = 'u.name'
            ctrlLabel = u'Name'


    class UserHandler(web.Handler):

        def do_list(self):
            lst = UserLst(self.req); lst()
            return self.render(listing=lst)


    <link wm:resource="'wm4.web.static/css/listings.css'" />
    <form form:form="listing">

        <div class="wm4-filter">
            name: <input form:input="name"/>
                  <input form:submit="" />
                  <input form:button="all" />
        </div>

        <p form:listing.pager="" />

        <table class="wm4-listing">
          <tr>
              <th><a form:listing.order="uid" /></th>
              <th><a form:listing.order="login" /></th>
              <th><a form:listing.order="name" /></th>
          </tr>
          <tr py:for="i in listing.decorated" class="$i.itemClass">
            <td>$i.item.uid</td>
            <td>$i.item.login</td>
            <td>$i.item.name</td>
          </tr>
        </table>

        <p form:listing.pager="" />

    </form>
'''

from wm4.api import *
from wm4.web.interfaces import FORM_WIDGET
from controllers import (
    event_handler, OnClickEvent, ClientState, FormMethod, FormCtrl,
    RepeatCtrl, AbstractCtrl, InputCtrl, SelectCtrl, ButtonCtrl, SubmitCtrl)
from lib4.iterutils import iterDecorator
import sys

__all__ = [
    'IColumn', 'IFilter', 'Column', 'FilterMixin', 'LikeFilterMixin',
    'InputFilter', 'LikeFilter', 'MulticolLikeFilter', 'SelectFilter',
    'PageLink', 'PagerState', 'ListingCtrl', 'pageRange',
]


_ = i18n.MessageIDFactory('wm4')


class IColumn(protocols.Interface):
    columnExpr = protocols.Attribute('query expression')

class IFilter(protocols.Interface):
    def ulrState():
        pass
    def makeCond():
        pass
    def all():
        pass




class Column(AbstractCtrl):

    protocols.advise(instancesProvide=[IColumn])

    queryExpr = None

    defaultDirection = '+'

    def listingCtrl(self):
        for p in self._ctrlParents:
            if isinstance(p, ListingCtrl):
                return p
        return None
    listingCtrl = binding.Make(listingCtrl)

    baseUrl = parsedOrder = binding.Delegate('listingCtrl')

    def urls(self):
        direction = toggleUrl = ascUrl = descUrl = None
        if self.queryExpr is None:
            return locals()
        c = self.ctrlName
        if len(self.parsedOrder.keys()) > 1:
            direction = None
            u_dir = self.defaultDirection
        else:
            direction, _ = self.parsedOrder.get(c, (None,None))
            if direction == '+':
                u_dir = '-'
            elif direction == '-':
                u_dir = '+'
            else:
                u_dir = self.defaultDirection
        if self.listingCtrl:
            v = self.listingCtrl['order'].ctrlId
        else:
            v = 'order'
        toggleUrl = self.baseUrl % {v: u_dir+c}
        ascUrl = self.baseUrl % {v: '+'+c}
        descUrl = self.baseUrl % {v: '-'+c}
        return locals()
    direction = toggleUrl = ascUrl = descUrl = binding.MultiMake(urls)




class FilterMixin(Column):

    protocols.advise(instancesProvide=[IFilter])

    def urlState(self):
        val = self.inputText
        if val not in (NOT_GIVEN, None, ''):
            yield self.ctrlId, val
        for c in self.ctrlChildren():
            for id, val in c.urlState():
                yield id, val

    def parseSubexpr(self, expr):
        frame = sys._getframe(1)
        return self.listingCtrl.parsedQuery.parseSubexpr(expr, frame)

    def makeCond(self):
        val = self.newValue
        if val in (NOT_GIVEN, None, ''):
            return
        yield self.parseSubexpr('%s == val' % self.queryExpr)

    def all(self):
        self.inputText = ''
        self.__dict__['newValue'] = None


class LikeFilterMixin(FilterMixin):

    def makeCond(self):
        val = self.newValue
        if val in (NOT_GIVEN, None, ''):
            return
        lval = val.lower().replace('%','\\%').replace('_', '\\_')
        lval = '%%%s%%' % lval
        yield self.parseSubexpr('query.ILike(query.Lower(%s),lval)' % (
                    self.queryExpr))




class InputFilter(InputCtrl, FilterMixin):
    pass


class LikeFilter(InputCtrl, LikeFilterMixin):
    pass


class MulticolLikeFilter(InputCtrl, FilterMixin):

    filteredExpressions = ()

    def makeCond(self):
        val = self.newValue
        if val in (NOT_GIVEN, None, ''):
            return
        lval = val.lower().replace('%','\\%').replace('_', '\\_')
        lval = '%%%s%%' % lval
        yield self.parseSubexpr(' or '.join(
                'query.ILike(%s,lval)' % e for e in self.filteredExpressions
        ))


class SelectFilter(SelectCtrl, FilterMixin):
    pass












class PageLink(object):

    def __init__(self, page, url, selected):
        self.page = page
        self.url = url
        self.selected = selected


class PagerState(binding.Component):

    page = pageParam = limitParam = itemCount = baseUrl = itemLimit = binding.Require()

    pageLimit = 10

    # Number of pages in listing
    def pageCount(self):
        if not self.itemLimit:
            return 0
        else:
            return (self.itemCount+self.itemLimit-1) / self.itemLimit
    pageCount = binding.Make(pageCount)

    # Sanitized page number
    def currentPage(self):
        if not self.itemLimit or not self.pageCount:
            return 0
        page = self.page
        if page in (None, NOT_GIVEN, ''):
            page = 1
        page = int(page)
        if page < 1:
            page = 1
        if page > self.pageCount:
            page = self.pageCount
        return page - 1
    currentPage = binding.Make(currentPage)

    # Yields PageLink instances in the pageLimit range
    def __iter__(self):
        for p in pageRange(self.pageCount, self.currentPage, self.pageLimit):
            yield self[p]

    # PageLink for any page num
    def __getitem__(self, idx):
        url = self.baseUrl % {self.pageParam: idx+1}
        return PageLink(idx, url, idx == self.currentPage)

    # Shortcuts
    first = binding.Make(lambda s:(
            s.pageCount and s[0] or None))
    prev = binding.Make(lambda s:(
            s.pageCount and s[max(s.currentPage-1,0)] or None))
    current = binding.Make(lambda s:(
            s.pageCount and s[s.currentPage] or None))
    next = binding.Make(lambda s:(
            s.pageCount and s[min(s.currentPage+1,s.pageCount-1)] or None))
    last = binding.Make(lambda s:(
            s.pageCount and s[s.pageCount-1] or None))

    def __nonzero__(self):
        return bool(self.pageCount)








class ListingCtrl(FormCtrl):

    ctrlAllowedMethods = set([FormMethod.get])

    baseQuery = binding.Require()

    # All IColumn's
    def columnMap(self):
        return dict(
            (c.ctrlName,c) for c in self.ctrlChildren()
                            if IColumn(c, None) is not None
        )
    columnMap = binding.Make(columnMap)

    workspace = binding.Make(lambda: settings.workspace())

    def urlState(self):
        s = {}
        for c in self.ctrlChildren():
            m = getattr(c, 'urlState', None)
            if m is not None:
                for k,v in m():
                    s[k] = v
        return s

    #baseUrl = binding.Make(lambda s: s.ctrlReq.requestURL % s.urlState())
    baseUrl = binding.Make(lambda s: s.formURL % s.urlState())

    ### Max number of items per page

    class limit(InputCtrl):
        ctrlType = int
        def urlState(self):
            val = self.newValue
            if val is NOT_GIVEN:
                return
            if val != self.ctrlParent.itemLimitDefault:
                yield self.ctrlId, str(self.newValue)

    itemLimitDefault = 20
    itemLimitMax = 200

    # Sanitized number of items per page
    def itemLimit(self):
        val = self['limit'].newValue
        if val is NOT_GIVEN:
            val = self.ctrlReq.params.get('limit', self.itemLimitDefault)
        try:
            val = int(val)
        except ValueError:
            val = self.itemLimitDefault
        if val < 0:
            val = self.itemLimitDefault
        elif val > self.itemLimitMax:
            val = self.itemLimitMax
        return val
    itemLimit = binding.Make(itemLimit)


    ### Paging

    # Input that holds current page. First page has number 1
    class page(InputCtrl):
        ctrlType = int
        def defaultInput(self):
            return '1'
        def defaultOutput(self):
            return 1

    # Max number pages in pager
    pageLimit = binding.Make(lambda s: s.ctrlReq.params.get('pageLimit',10))

    def pager(self):
        return PagerState(self,
            page      = self['page'].newValue,
            pageLimit = self.pageLimit,
            itemCount = self.itemCount,
            itemLimit = self.itemLimit,
            pageParam = self['page'].ctrlId,
            limitParam = self['limit'].ctrlId,
            baseUrl   = self.baseUrl,
        )
    pager = binding.Make(pager)


    ### Column ordering

    # Variable that holds ordering information on client
    class order(InputCtrl):
        ctrlType = str
        def defaultOutput(self):
            return self.ctrlParent.defaultOrder
        def urlState(self):
            val = self.newValue
            if val is NOT_GIVEN:
                return
            if val != self.ctrlParent.defaultOrder:
                yield self.ctrlId, str(self.newValue)

    # Default order as key to columnMap with direction '+/-' prefix
    defaultOrder = None

    # Order information as (<+/->, <columnMap key>) tuple'''
    def parsedOrder(self):
        '''Returns (direction, column) tuple'''
        orderlist = self['order'].newValue
        if orderlist in (NOT_GIVEN, None):
            if not self.defaultOrder:
                return {}
            orderlist = self.defaultOrder
        po = {}
        for v, o in enumerate(orderlist.split(',')):
            if o[:1] in '+-':
                d, c = o[:1], o[1:]
            else:
                d = '+'
                c = o
            if c not in self.columnMap:
                raise web.BadRequest('invalid order column %s' % o)
            po[c] = (d, v)
        return po
    parsedOrder = binding.Make(parsedOrder)

    # Order by as wm4.query expression
    def orderBy(self):
        order = []
        pq = self.parsedQuery
        for v,c,d in sorted((v,c,d) for c,(d,v) in self.parsedOrder.items()):
            e = self.columnMap[c].queryExpr
            if e is None:
                continue
            if isinstance(e, basestring):
                e = pq.parseSubexpr(e)
            if d == '-':
                order.append(query.Desc(e))
            else:
                order.append(e)
        if not order:
            return None
        return query.OrderBy(*order)
    orderBy = binding.Make(orderBy)


    ### Query building

    precond = None

    def where(self):
        conds = []
        if self.precond is not None:
            p = self.precond
            if isinstance(p, basestring):
                p = self.parsedQuery.parseSubexpr(p)
            conds.append(p)
        for filter in self.ctrlChildren():
            if IFilter(filter, None) is not None:
                for c in filter.makeCond():
                    if isinstance(c, basestring):
                        c = self.parsedQuery.parseSubexpr(c)
                    conds.append(c)
        if not conds:
            return None
        return query.AND(*conds)
    where = binding.Make(where)

    def parsedQuery(self):
        return self.workspace.parse(self.baseQuery)
    parsedQuery = binding.Make(parsedQuery)

    def countQuery(self):
        q = self.parsedQuery
        if self.where is not None:
            q = q.withCondition(self.workspace.connection, self.where)
        q = q.asCount(self.workspace.connection)
        return q
    countQuery = binding.Make(countQuery)

    itemCount = binding.Make(lambda s: ~s.workspace(s.countQuery))

    def itemQuery(self):
        q = self.parsedQuery.copy()
        if self.where is not None:
            q = q.withCondition(self.workspace.connection, self.where)
        q.orderBy = self.orderBy
        if self.itemLimit:
            q.limit = query.Const(self.itemLimit)
            q.offset = query.Const(self.pager.currentPage * self.itemLimit)
        return q
    itemQuery = binding.Make(itemQuery)

    items = binding.Make(lambda s: list(s.workspace(s.itemQuery)))

    decorated = binding.Make(lambda s: list(iterDecorator(s.items)))


    ### Actions

    def formAction(self):
        action = None
        for child in self.ctrlIter():
            act = child.ctrlGetAction()
            if act is not None:
                if action is not None:
                    raise web.BadRequest(
                        'More than one action', action.ctrlId, act.ctrlId)
                action = act
        # Autocommit self
        if action is None:
            action = self['doSubmit']
        return action
    formAction = binding.Make(formAction)

    class doSubmit(SubmitCtrl):
        ctrlLabel = _(u'Search')

    class all(ButtonCtrl):
        ctrlLabel = _(u'All')

    @event_handler(OnClickEvent, 'all')
    def clear_filters(self, evt):
        for filter in self.ctrlChildren():
            if isinstance(filter, FilterMixin):
                filter.all()





def pageRange(pageCount, currentPage=0, pageLimit=10):
    '''Returns list of page numbers around current page'''
    if not pageCount:
        return []
    #if page >= pageCount:
    #    raise ValueError('page >= pageCount', page, pageCount)
    if pageLimit < 2:
        raise ValueError('pageLimit < 2', pageLimit)
    if pageCount <= pageLimit:
        return range(0, pageCount)
    else:
        before = (pageLimit+1) / 2
        start = max(currentPage-before, 0)
        end = start + pageLimit
        if end > pageCount:
            diff = end - pageCount
            end = pageCount
            start = max(start-diff, 0)
        return range(start, end)
