
import os.path

from wm4.api import *
from xml4 import xhtml as xh
from xml4.elements import Literal, Comment, Element
from xml4.templates import Domlet, Deferred
from xml4.utils import attr_list, escape
from xml4.const import *
from lib4.imports import importObject
from lib4.iterutils import iterDecorator
from itertools import chain
import datetime

from wm4.web.publishing import render
from wm4.web.interfaces import NSDomletProperty, FORM_NAMESPACE
from wm4.web.utils import render_js_value
from controllers import FormMethod

__all__ = [
    'CURRENT', 'Form',
    'FormWidget', 'IndirectWidget', 'InputWidget', 'PasswordWidget',
    'HypertextualWidget',
    'FileWidget', 'AreaWidget', 'PreAreaWidget',
    'RestWidget', 'HtmlFckWidget', 'HtmlFckUploadWidget', 'HtmlTinyWidget',
    'DateWidget', 'DatetimeWidget',
    'SelectWidget', 'MultiSelectWidget',
    'MultiCheckboxWidget', 'HiearchicalMultiCheckboxWidget', 'CheckboxWidget',
    'RadioWidget', 'PopupWidget',
    'RepeatWidget', 'WithWidget',
    'ButtonWidget', 'SubmitWidget', 'DependentSpanWidget', 'ErrorWidget',
    'FieldsetWidget',

    'ListingPagerWidget', 'ListingOrderWidget',
]

_ = i18n.MessageIDFactory('wm4')

CURRENT = 'form'


ALL_JS = (
    'onclick', 'ondblclick', 'onmousedown', 'onmouseup', 'onmouseover',
    'onmousemove', 'onmouseout', 'onkeypress', 'onkeydown', 'onkeyup',
    'onchange',
)


class Form(Domlet):

    def expand(self, state, form=None):
        req = state['req']
        if form is None:
            form = state['form']
        #web.getResource('wm4.web.static/css/forms.css').render(req)
        state = state.copy({CURRENT:form})

        excluded = set()
        title = None
        submit = None
        for child in self._children:
            if not isinstance(child, (Element,Domlet)):
                continue
            if child.tag == 'title':
                attrs = dict(child._attrs)
                if 'class' in attrs:
                    attrs['class'] += ' wm4-form-caption'
                else:
                    attrs['class'] = 'wm4-form-caption'
                title = xh.div(
                    attrs,
                    xh.div(
                        {'class': 'wm4-form-caption-right-corner'},
                        xh.h2(*child)
                ))
                excluded.add(child)

            elif child.tag == 'submit':
                attrs = dict(child._attrs)
                if 'class' in attrs:
                    attrs['class'] += ' wm4-form-submit'
                else:
                    attrs['class'] = 'wm4-form-submit'
                submit = xh.div(
                    attrs,
                    xh.div(
                        {'class': 'wm4-form-submit-right-corner'},
                        *child
                ))
                excluded.add(child)



        if form.ctrlReadOnly:
            attrs = self.acquireAttrs(state, ('id', 'class'))
            attrs.setdefault('class', 'wm4-form wm4-form-readonly')
            tag = u'div'
        else:
            attrs = self.acquireAttrs(state,
                ('method','enctype','id','class','style','onsubmit','onreset'),
                method='post', enctype='application/x-www-form-urlencoded'
            )
            attrs.setdefault('class', 'wm4-form wm4-form-editable')
            tag = u'form'
            fm = FormMethod.find(
                (attrs['method'].lower(), attrs['enctype'].lower())
            )
            if not fm in form.ctrlAllowedMethods:
                fm = sorted(form.ctrlAllowedMethods)[0]
            attrs['method'] = fm.value[0]
            attrs['enctype'] = fm.value[1]
            attrs['accept-charset'] = form.ctrlReq.output_encoding
            attrs['action'] = form.formURL

        attrs = attrs.items()
        yield START_TAG, (tag, attrs)
        yield TEXT, '\n'

        if title is not None:
            for e in title.expand(state):
                yield e
            yield TEXT, '\n'

        for e in xh.div(
            {'class':'wm4-form-top'},
            xh.div({'class':'wm4-form-top-right-corner'}, ' ')
        ).expand(state):
            yield e
        yield TEXT, '\n'

        form_in = ('div', attr_list([('class','wm4-form-in')]))
        yield START_TAG, form_in

        if not form.ctrlReadOnly:
            for a,v in sorted(form.ctrlState()):
                yield TEXT, u'\n'
                for e in xh.input(type='hidden', name=a, value=v).expand():
                    yield e
        for child in self._children:
            if child in excluded:
                continue
            for e in child.expand(state):
                yield e

        yield TEXT, '\n'
        yield END_TAG, form_in
        yield TEXT, '\n'

        for e in xh.div(
            {'class':'wm4-form-bottom'},
            xh.div({'class':'wm4-form-bottom-right-corner'}, ' ')
        ).expand(state):
            yield e
        yield TEXT, '\n'

        if submit is not None and not form.ctrlReadOnly:
            for e in submit.expand(state):
                yield e
            yield TEXT, '\n'

        yield END_TAG, (tag, attrs)



class IndirectWidget(Domlet):

    def __call__(self, state):
        felem = state[CURRENT][self.value]
        try:
            factory = importObject(felem.ctrlWidget())
        except:
            raise ValueError('cannot find widget',felem.ctrlId,felem.ctrlWidget)
        for e in factory(
            self.qname, self.value, self.gdict,
            tag=self._tag, attrs=self._attrs, children=self._children,
            child_domlet=self.child_domlet, line_info=self.line_info,
        )(state):
            yield e



class FormWidget(Domlet):

    __metaclass__ = binding.Activator

    def __call__(self, state):
        req = state['req']
        felem = state[CURRENT][self.value]
        state = state.copy({CURRENT: felem})

        if felem.ctrlVisibility == 'none':
            method = self.invisible
        elif felem.ctrlVisibility == 'hidden':
            #if felem.ctrlReadOnly or felem.oldValue is NOT_GIVEN:
            #    method = self.invisible
            #else:
            method = self.hidden
        else:
            if felem.ctrlReadOnly:
                method = self.view
            else:
                method = self.edit
        markup = method(state, felem, req)
        if markup is not None:
            for e in markup.expand(state):
                yield e

    def view(self, state, felem, req):
        return xh.div(u'[FormWidget(id=%r)]' % (felem.ctrlId))

    def invisible(self, state, felem, req):
        return None





class InputWidget(FormWidget):
    type  = 'text'

    def view(self, state, felem, req):
        attrs = self.acquireAttrs(state, ('class','id','style'))
        return xh.span(attrs, felem.inputText)

    def hidden(self, state, felem, req):
        attrs = {
            'type' : 'hidden',
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        attrs.update(self.acquireAttrs(state, ('class','id')+ALL_JS))
        return xh.input(attrs)

    def edit(self, state, felem, req):
        attrs = {
            'type' : self.type,
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        max = getattr(felem, 'maxlength', None)
        if max is not None:
            attrs['maxlength'] = max
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey',
                'maxlength', 'placeholder')+ALL_JS,
        ))
        return xh.input(attrs)


class PasswordWidget(InputWidget):
    type  = 'password'


class HypertextualWidget(InputWidget):

    def view(self, state, felem, req):
        from lib4.text import hypertextize
        attrs = self.acquireAttrs(state, ('class','id','style'))
        span = xh.span(attrs)
        for t,u in hypertextize(felem.inputText or ''):
            if u:
                span.append(xh.a(t, href=u))
            else:
                span.append(t)
        return span


class FileWidget(InputWidget):

    type = 'file'

    def view(self, state, felem, req):
        old = felem.oldValue
        if old in (None, NOT_GIVEN):
            return xh.span()
        title='%s (%s bytes)' % (old.undir, felem.size)
        try:
            url = req.get_url(old)
            return xh.a(title, href=url, title=title)
        except:
            return xh.span(title)

    def hidden(self, state, felem, req):
        return None

    def edit(self, state, felem, req):
        attrs = {
            'type' : self.type,
            'name' : felem.ctrlId,
            'value': felem.inputText,
        }
        attrs.update(self.acquireAttrs(state,
            ('accept','size','class','style','id','tabindex','accesskey')+ALL_JS
        ))
        if felem.filename is NOT_GIVEN:
            return xh.span(xh.input(attrs))
        else:
            if felem.ctrlRequired:
                dc = ''
            else:
                de = felem['deleteCurrent']
                sc = state.copy({CURRENT:de})
                bw = ButtonWidget(
                    (self.qname[0],'input'), 'deleteCurrent', self.gdict,
                    tag=self._tag,
                )
                dc = bw.edit(sc, de, req)
            return xh.div(
                xh.div(self.view(state, felem, req), ' ', dc),
                xh.input(attrs),
            )

class ImageWidget(FileWidget):
    # XXX TODO
    def view(self, state, felem, req):
        old = felem.oldValue
        if old in (None, NOT_GIVEN):
            return xh.span()
        try:
            url = req.get_url(old)
            return xh.img(
                src=url, width=old.width, height=old.height,
                title='%s (%s bytes)' % (old.undir, felem.size)
            )
        except:
            return xh.span('%s (%s bytes)' % (felem.filename, felem.size))

class AreaWidget(InputWidget):

    def edit(self, state, felem, req):
        attrs = self.acquireAttrs(state,
            ('rows','cols','class','style','id','tabindex','accesskey')+ALL_JS,
            rows=felem.rows, cols=felem.cols
        )
        attrs['name'] = felem.ctrlId
        return xh.textarea(felem.inputText, **attrs)


class PreAreaWidget(AreaWidget):

    def view(self, state, felem, req):
        attrs = self.acquireAttrs(state, ('class','id','style')+ALL_JS)
        return xh.pre(attrs, felem.inputText)


class RestWidget(AreaWidget):

    def view(self, state, felem, req):
        raise Exception('ahoj')
        attrs = self.acquireAttrs(state, ('class','id','style')+ALL_JS)
        val = felem.inputText
        if val:
            from lib4.rest import reST2html
            val = Literal(reST2html(val))
        return xh.div(attrs, val)

    def edit(self, state, felem, req):
        attrs = self.acquireAttrs(state,
            ('rows','cols','class','style','id','tabindex','accesskey')+ALL_JS,
            rows=felem.rows, cols=felem.cols
        )
        attrs['name'] = felem.ctrlId
        return xh.div(
            xh.div(xh.textarea(felem.inputText, **attrs)),
            xh.input(type='file', name=felem.ctrlId + '$upload'),
        )


FCK_SCRIPT = '''
$(document).ready(function() {
var e = new FCKeditor(%(id)s);
e.BasePath = '/++res++/wm4.web.static/fck/';
e.Height = 300;
var c = e.Config;
c["SkinPath"] = e.BasePath+'editor/skins/silver/';
c["CustomConfigurationsPath"] = '/++res++/wm4.web.static/js/fck_toolbarsets.js';
c["EditorAreaCSS"] = '';
c["AutoDetectLanguage"] = false;
c["DefaultLanguage"] = "cs";
c["IncludeLatinEntities"] = false;
c["FontFormats"] = "p;h2;h3;h4";
e.ToolbarSet = '%(toolbarSet)s';
e.ReplaceTextarea();
});
'''

class HtmlFckWidget(AreaWidget):
    _FCK_SCRIPT = FCK_SCRIPT

    def view(self, state, felem, req):
        return xh.Literal(felem.inputText)

    def get_fck_script_attrs(self, state, felem, req, attrs):
        return dict(
                id=render_js_value(req,attrs['id']),
                toolbarSet=getattr(felem, 'toolbarSet'),
                )

    def edit(self, state, felem, req):
        web.getResource('wm4.web.static/fck/fckeditor.js').render(req)
        web.getResource('wm4.web.static/jquery/jquery.js').render(req)
        attrs = self.acquireAttrs(state,
            ('rows','cols','class','style','id','tabindex','accesskey')+ALL_JS,
            rows=felem.rows, cols=felem.cols, id=felem.ctrlId
        )
        attrs['name'] = felem.ctrlId

        return xh.frag(
            xh.script(Comment(self._FCK_SCRIPT % self.get_fck_script_attrs(
                state, felem, req, attrs))),
            xh.textarea(felem.inputText, **attrs),
        )


FCK_UPLOAD_SCRIPT = '''
$(document).ready(function() {
var e = new FCKeditor(%(id)s);
e.BasePath = '%(BasePath)s';
e.Height = 300;
var c = e.Config;
c["SkinPath"] = e.BasePath+'editor/skins/silver/';
c["CustomConfigurationsPath"] = '/++res++/wm4.web.static/js/fck_toolbarsets.js';
c["EditorAreaCSS"] = '';
c["AutoDetectLanguage"] = false;
c["DefaultLanguage"] = "cs";
c["IncludeLatinEntities"] = false;
c["FontFormats"] = "p;h2;h3;h4";
c["LinkBrowserURL"] = '%(LinkBrowserURL)s';
c["ImageBrowserURL"] = '%(ImageBrowserURL)s';
c["FlashBrowserURL"] = '%(FlashBrowserURL)s';
c["LinkUploadURL"] = '%(LinkUploadURL)s';
c["ImageUploadURL"] = '%(ImageUploadURL)s';
c["FlashUploadURL"] = '%(FlashUploadURL)s';
e.ToolbarSet = '%(toolbarSet)s';
e.ReplaceTextarea();
});
'''

class HtmlFckUploadWidget(HtmlFckWidget):
    _FCK_SCRIPT = FCK_UPLOAD_SCRIPT

    def get_fck_script_attrs(self, state, felem, req, attrs):
        bp = config.Property('fckconnector.BasePath')()
        fck_upload_id = getattr(felem, 'fckUploadId', '__default')
        return dict(
                id=render_js_value(req,attrs['id']),
                toolbarSet=getattr(felem, 'toolbarSet'),
                BasePath=bp,
                LinkBrowserURL=os.path.join(bp, config.Property(
                    'fckconnector.LinkBrowserURL')() % fck_upload_id),
                ImageBrowserURL=os.path.join(bp, config.Property(
                    'fckconnector.ImageBrowserURL')() % fck_upload_id),
                FlashBrowserURL=os.path.join(bp, config.Property(
                    'fckconnector.FlashBrowserURL')() % fck_upload_id),
                LinkUploadURL=os.path.join(bp, config.Property(
                    'fckconnector.LinkUploadURL')() % fck_upload_id),
                ImageUploadURL=os.path.join(bp, config.Property(
                    'fckconnector.ImageUploadURL')() % fck_upload_id),
                FlashUploadURL=os.path.join(bp, config.Property(
                    'fckconnector.FlashUploadURL')() % fck_upload_id),
                )



class DateWidget(InputWidget):

    defaultSize = 10
    _datepickerconfig = '{}'

    def edit(self, state, felem, req):
        for res in (
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/jquery/ui.datepicker.js',
            'wm4.web.static/jquery/ui.datepicker-cs.js',
            'wm4.web.static/jquery/ui.datepicker.css',
        ):
            web.getResource(res).render(req)
        id = 'WM4-DATE-'+felem.ctrlId.replace('$','-').upper()
        attrs = {'name': felem.ctrlId, 'value':felem.inputText, 'type':'text'}
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey')+ALL_JS,
            size=self.defaultSize, id=id
        ))
        id = attrs['id']
        return xh.div(
            xh.input(attrs),
            xh.script(
              "$(document).ready(function(){$('#%s').datepicker(%s);});" % (
                  id, self._datepickerconfig),
              type="text/javascript",
            ),
            klass='wm4-form-widget-date',
        )

class DatetimeWidget(DateWidget):
    defaultSize = 18
    _datepickerconfig = '''{showTime: true, dateFormat: 'dd.mm.yy HH:II'}'''



class SelectWidget(InputWidget):

    def view(self, state, felem, req):
        attrs = self.acquireAttrs(state, ('class','id','style'))
        for o in felem.options:
            if o.selected:
                return xh.span(attrs, render(req, o.label))
        return xh.span(attrs, '')

    def edit(self, state, felem, req):
        attrs = {'name' : felem.ctrlId}
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey')+ALL_JS
        ))
        s = xh.select(attrs)
        for o in felem.options:
            l = render(req, o.label)
            if o.selected:
                s.append(xh.option(l, value=o.key, selected="selected"))
            else:
                s.append(xh.option(l, value=o.key))
        return s

class MultiSelectWidget(InputWidget):

    def edit(self, state, felem, req):
        attrs = {'name' : felem.ctrlId}
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey')+ALL_JS
        ))
        attrs['multiple'] = 'multiple'
        s = xh.select(attrs)
        for o in felem.options:
            l = render(req, o.label)
            if o.selected:
                s.append(xh.option(l, value=o.key, selected="selected"))
            else:
                s.append(xh.option(l, value=o.key))
        return s

class MultiCheckboxWidget(InputWidget):

    def edit(self, state, felem, req):
        attrs = self.acquireAttrs(state, ('class',)+ALL_JS)
        if 'class' in attrs:
            attrs['class'] += ' wm4-form-multicheckbox'
        else:
            attrs['class'] = 'wm4-form-multicheckbox'
        mcb = xh.div(attrs)
        for o in felem.options:
            item = xh.label(klass="checkbox")
            if o.selected:
                item.append(xh.input(
                    type="checkbox", name=felem.ctrlId, value=o.key,
                    checked="checked",
                ))
            else:
                item.append(xh.input(
                    type="checkbox", name=felem.ctrlId, value=o.key
                ))
            item.append(o.label)
            mcb.append(item)
        return mcb

class HiearchicalMultiCheckboxWidget(InputWidget):

    def edit(self, state, felem, req):
        if not felem.valuesRoot:
            return None
        attrs = self.acquireAttrs(state, ('class',)+ALL_JS)
        if 'class' in attrs:
            attrs['class'] += ' wm4-form-hiearchicalmulticheckbox'
        else:
            attrs['class'] = 'wm4-form-hiearchicalmulticheckbox'
        mcb = xh.div(attrs)
        inp = felem.inputText
        fgk = felem.valuesGetKey
        for node in felem.valuesRoot.iterDepthFirst(
                                    include_self=felem.includeRoot):
            item = xh.label(klass="level-%d" % node.treeLevel)
            if fgk(node) in inp:
                item.append(xh.input(
                    type="checkbox",
                    name=felem.ctrlId,
                    value=fgk(node),
                    checked="checked",
                ))
            else:
                item.append(xh.input(
                    type="checkbox",
                    name=felem.ctrlId,
                    value=fgk(node),
                ))
            item.append(felem.valuesGetLabel(node))
            mcb.append(item)
        return mcb

class CheckboxWidget(InputWidget):
    def edit(self, state, felem, req):
        s = xh.span()
        inp = felem.inputText
        chb = xh.input(type="checkbox", name=felem.ctrlId, value="1")
        if inp not in (None, NOT_GIVEN, ''):
            chb.setdefault('checked', 'checked')
        s.append(xh.label(chb, felem.ctrlLabel))
        return s

class RadioWidget(SelectWidget):

    def edit(self, state, felem, req):
        attrs = {'name' : felem.ctrlId}
        attrs.update(self.acquireAttrs(state,
            ('size','class','style','id','tabindex','accesskey')+ALL_JS
        ))
        s = xh.div(attrs)
        for o in felem.options:
            radio = xh.input(type="radio", value=o.key, name=felem.ctrlId)
            l = render(req, o.label)
            if o.selected:
                radio.setdefault('checked', 'checked')
            s.append(xh.label(radio, l, klass="radio"))
        return s


class PopupWidget(FormWidget):
    width  = binding.Make(
                lambda s: s.element._attrs.get('popup:width', 800))
    height = binding.Make(
                lambda s: s.element._attrs.get('popup:height', 600))

    def view(self, state, felem, req):
        return None

    def edit(self, state, felem, req):
        for res in (
            'wm4.web.static/js/x_core.js',
            'wm4.web.static/js/x_event.js',
            'wm4.web.static/js/x_dom.js',
            'wm4.web.static/js/wm_popup.js',
        ):
            web.getResource(res).render(req)

        pi = req.params.setdefault('popup_instances', {})
        if felem.popVar not in pi:
            attrs = self.acquireAttrs(state, ('popup:width', 'popup:height'))
            attrs.setdefault('popup:width', '800')
            attrs.setdefault('popup:height', '600')

            width = attrs['popup:width']
            height = attrs['popup:height']

            script = "%s = new wmPopupFormIframe('%s', %s, %s" % (
                felem.popVar, felem.popupUrl, width, height
            )
            for a,v in felem.popupMap.iteritems():
                script += ", '%s'" % a
                script += ", '%s'" % felem[v].ctrlId
            script += ");"
            script = xh.script(script, type='text/javascript')
            pi[felem.popVar] = True
        else:
            script = ''

        click = "%s.toggle(event); return false;" % felem.popVar
        if self._tag in (None, 'input', 'button'):
            return xh.span(
                script,
                xh.button(
                    felem.ctrlLabel,
                    id      = id,
                    type    = "button",
                    onclick = click,
                    name    = felem.ctrlId,
                ),
                klass = 'wm4-form-popup',
            )
        else:
            attrs = tuple(self._attrs.expand(state))
            attrs += (('onclick', click),)
            return xh.span(
                script,
                Element(self._tag, attrs, self._children),
            )


class LabelWidget(InputWidget):

    def view(self, state, felem, req):
        attrs = self.acquireAttrs(state, ('class','id','style'))
        attrs.setdefault('class', 'wm4-form-label')
        return xh.span(attrs, render(req, felem.ctrlLabel))

    def edit(self, state, felem, req):
        attrs = {'for' : felem.ctrlId}
        attrs.update(self.acquireAttrs(state,
            ('class','style','id','accesskey')+ALL_JS
        ))
        attrs.setdefault('class', 'wm4-form-label')
        return xh.label(attrs, render(req, felem.ctrlLabel))



class RepeatWidget(FormWidget):

    def __call__(self, state):
        felem = state[CURRENT][self.value]
        state = state.copy({CURRENT: felem})
        for item in felem.ctrlItems:
            cp = state.copy({CURRENT:item})
            for e in self.child_domlet(cp):
                yield e



class WithWidget(FormWidget):

    def __call__(self, state):
        felem = state[CURRENT][self.value]
        state = state.copy({CURRENT: felem})
        for e in self.child_domlet(state):
            yield e




class ButtonWidget(InputWidget):

    type = 'submit'

    def view(self, state, felem, req):
        return None

    def edit(self, state, felem, req):
        attrs = dict(self._attrs.expand(state))
        if felem.jsOnClick:
            attrs['onclick'] = felem.jsOnClick
        if self._tag == 'input':
            attrs['name'] = felem.ctrlId
            attrs.setdefault('value', felem.ctrlLabel)
            attrs['value'] = web.render(req, attrs['value'])
            attrs['type'] = 'submit'
            return Element(self._tag, attrs.items(), self._children)
        else:
            for res in ('wm4.web.static/js/wm_forms.js',):
                web.getResource(res).render(req)

            onclick = "wm4.forms.submitWithAction(wm4.forms.findFormForElem(this), '%s'); return false;" % felem.ctrlId
            attrs['onclick'] = onclick
            return xh.span(
                Element(self._tag, attrs.items(), self._children),
            )



class SubmitWidget(ButtonWidget):

    def __init__(self, *a, **k):
        super(SubmitWidget, self).__init__(*a, **k)
        if not self.value:
            self.value = 'doSubmit'




class DependentSpanWidget(FormWidget):

    def view(self, state, felem, req):
        return xh.span(felem[felem.dependsOn].inputText)


    def edit(self, state, felem, req):
        for res in (
            'wm4.web.static/js/wm_forms.js',
            'wm4.web.static/jquery/jquery.js',
        ):
            web.getResource(res).render(req)

        e = felem[felem.dependsOn]
        i = felem.spanId
        return xh.span(
            xh.script("""
            $(function () {
                var i = 'WM4%d';
                var s = document.getElementById(i);
                var d = wm4.forms.getField(wm4.forms.findFormForElem(s),'%s');
                d.onchange = function (evt) {
                    s.innerHTML = d.value;
                };
            });
            """ % (i, e.ctrlId)),
            xh.span(e.inputText or felem.defaultText, id='WM4%d' % i),
        )


class ErrorWidget(FormWidget):

    def view(self, state, felem, req):
        return None

    hidden = view

    def edit(self, state, felem, req):
        errs = felem.ctrlRoot.formErrors
        if not errs:
            return
        if felem.ctrlId:
            fp = felem.ctrlId.replace('$','.')
        else:
            fp = None

        attrs = self.acquireAttrs(state, ('class','id'))
        attrs.setdefault('class', 'wm4-form-error')
        ul = xh.ul(attrs)

        attrs = self.acquireAttrs(state, ('class',))
        if not fp:
            def deferred_error_widget():
                for fp, errLst in sorted(errs.items()):
                    for err in errLst:
                        ul.append(xh.li(attrs, fp, ': ', render(req, err)))
                if ul:
                    return unicode(ul)
                else:
                    return ''
            return Deferred(deferred_error_widget)
        else:
            for err in errs.pop(fp, default=()):
                ul.append(xh.li(attrs, render(req, err)))
            if ul:
                return ul


class FieldsetWidget(FormWidget):

    def __call__(self, state):
        excluded = set()
        title = None
        actions = None
        for child in self._children:
            if not isinstance(child, (Element,Domlet)):
                continue
            if child.tag == 'title':
                attrs = dict(child._attrs)
                if 'class' in attrs:
                    attrs['class'] += ' wm4-form-fieldset-title'
                else:
                    attrs['class'] = 'wm4-form-fieldset-title'
                title = xh.div(attrs, *child)
                excluded.add(child)
            elif child.tag == 'actions':
                attrs = dict(child._attrs)
                if 'class' in attrs:
                    attrs['class'] += ' wm4-form-fieldset-actions'
                else:
                    attrs['class'] = 'wm4-form-fieldset-actions'
                actions = xh.div(attrs, *child)
                excluded.add(child)

        content = [xh.div(
            {'class':'wm4-form-fieldset-top'},
            xh.div({'class':'wm4-form-fieldset-top-right-corner'},' ')
        )]
        if actions is not None:
            content.append('\n')
            content.append(actions)
        if title is not None:
            content.append('\n')
            content.append(title)
        content.append('\n')
        content.append(xh.div(
            {'class':'wm4-form-fieldset-body'},
            *[c for c in self._children if c not in excluded]
        ))
        content[-1].append('\n')
        content.append('\n')
        content.append(xh.div(
            {'class':'wm4-form-fieldset-bottom'},
            xh.div({'class':'wm4-form-fieldset-bottom-right-corner'},' ')
        ))
        content.append('\n')

        attrs = dict(self._attrs)
        if 'class' in attrs:
            attrs['class'] += ' wm4-form-fieldset'
        else:
            attrs['class'] = 'wm4-form-fieldset'
        yield TEXT, '\n'

        for e in xh.div(
            attrs,
            xh.div(
                {'class':'wm4-form-fieldset-right-border'},
                xh.div(
                    {'class':'wm4-form-fieldset-left-border'},
                    '\n',
                    *content
                )
            )
        ).expand(state):
            yield e



class ListingPagerWidget(FormWidget):
    pageLimitOfferedValues = [5,10,15,20,50,100]

    def countInfo(self, req, pager):
        return xh.span(
            xh.span(render(req, _(u'Total')),':', klass='lbl'),
            ' ',
            xh.span(pager.itemCount, klass='info'),
            klass='count-info'
        )

    def pageInfo(self, req, pager):
        c = pager.currentPage + 1
        a = pager.pageCount
        return xh.span(
            #xh.span(render(req, _(u'Page')),':', klass='lbl'),
            ' ',
            xh.span('%d/%d' % (c, a), klass='info'),
            klass='page-info'
        )

    def relativeUrl(self, req, pager, rel, txt, title, klass):
        txt = render(req, txt)
        title = render(req, title)
        if rel.page != pager.currentPage:
            return xh.a(
                txt, href=rel.url, klass='relative %s' % klass, title=title
            )
        else:
            return xh.span(txt, klass='relative %s' % klass)

    def pages(self, req, pager):
        pages = xh.span('\n', klass='pages')
        for d in iterDecorator(pager):
            p = d.item
            klass = []
            if p.page == pager.currentPage:
                klass.append('current')
            if d.isFirst:
                klass.append('first')
            if d.isLast:
                klass.append('last')
            if klass:
                pages.append(xh.a(p.page+1, href=p.url, klass=' '.join(klass)))
            else:
                pages.append(xh.a(p.page+1, href=p.url))
            pages.append('\n')
        return pages

    def pageLimit(self, req, pager):
        # XXX option value must be integer and URL should be generated in JS
        sel = xh.select(
            '\n',
            name = '_limit',
            onchange = "if(this.value != '') window.location.href = this.value;"
        )
        offered = list(self.pageLimitOfferedValues)
        if pager.itemLimit and pager.itemLimit not in offered:
            import bisect
            bisect.insort(offered, pager.itemLimit)
        for lim in offered:
            url = pager.first.url % dict([(pager.limitParam, lim)])
            if lim == pager.itemLimit:
                sel.append(xh.option(lim, value=url, selected='selected'))
            else:
                sel.append(xh.option(lim, value=url))
            sel.append('\n')
        return xh.span(
            xh.span(render(req, _(u'Items per page')), ':', klass='lbl'),
            ' ',
            sel,
            klass='page-limit'
        )

    def __call__(self, state):
        req = state['req']
        felem = state[CURRENT][self.value]
        pg = felem.pager
        if not pg:
            return
        top = getattr(xh, self._tag, 'span')
        attrs = self.acquireAttrs(state, ('class',),)
        attrs.setdefault('class', 'wm4-pager')

        markup = top(
            '\n',
            self.countInfo(req, pg), '\n',
            self.relativeUrl(
                req, pg, pg.first, '|<', _(u'First page'), 'first-page'
            ), '\n',
            self.relativeUrl(
                req, pg, pg.prev, '<<', _(u'Previous page'), 'prev-page'
            ), '\n',

            self.pages(req, pg), '\n',

            self.relativeUrl(
                req, pg, pg.next, '>>', _(u'Next page'), 'next-page'
            ), '\n',
            self.relativeUrl(
                req, pg, pg.last, '>|', _(u'Last page'), 'last-page'
            ), '\n',
            self.pageInfo(req, pg), '\n',
            self.pageLimit(req, pg), '\n',
            **attrs
        )
        for e in markup.expand(state):
            yield e



class ListingOrderWidget(FormWidget):

    def __call__(self, state):
        req = state['req']
        felem = state[CURRENT][self.value]
        label = web.render(req, felem.ctrlLabel)
        if not felem.queryExpr:
            for e in xh.span(label, klass='wm4-listing-order').expand(state):
                yield e
        if felem.direction:
            klass = 'wm4-listing-order ordered'
        else:
            klass = 'wm4-listing-order'
        if felem.direction == '+':
            d = xh.span(u'\u2193', klass='direction asc')
        elif felem.direction == '-':
            d = xh.span(u'\u2191', klass='direction desc')
        else:
            d = ''
        markup = xh.a(label, d, href=felem.toggleUrl, klass=klass)
        for e in markup.expand(state):
            yield e


TINY_SCRIPT = '''
    tinyMCE.init({
            language: "cs",
            theme: "advanced",
            mode: "exact",
            elements: '%(id)s',
            plugins: "nonbreaking",
            theme_advanced_buttons3_add: "nonbreaking",
            relative_urls: false,
            remove_script_host: true,
            plugins: "pagebreak,style,layer,table,save,advhr,"
                     +"advimage,advlink,emotions,iespell,inlinepopups,"
                     +"insertdatetime,preview,media,searchreplace,print,"
                     +"contextmenu,paste,directionality,fullscreen,"
                     +"noneditable,visualchars,nonbreaking,xhtmlxtras,"
                     +"template,advlist",
            theme_advanced_buttons1: "bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,formatselect,|,cut,copy,paste,pastetext,pasteword,|,cleanup,help,code,|,fullscreen",
            theme_advanced_buttons2: "bullist,numlist,|,outdent,indent,|,undo,redo,|,link,unlink,anchor,|,hr,removeformat,visualaid,|,sub,sup,|,charmap",
            theme_advanced_buttons3: "tablecontrols",
            theme_advanced_buttons4: "",
            theme_advanced_toolbar_location: "top",
            theme_advanced_toolbar_align: "left",
            theme_advanced_statusbar_location: "bottom",
            theme_advanced_resizing: true,
            nonbreaking_force_tab: true
        });
    '''
class HtmlTinyWidget(AreaWidget):
    _FCK_SCRIPT = TINY_SCRIPT

    def view(self, state, felem, req):
        return xh.Literal(felem.inputText)

    def get_fck_script_attrs(self, state, felem, req, attrs):
        return dict(
                #id=render_js_value(req,attrs['id']),
                id=attrs['id'],
                toolbarSet=getattr(felem, 'toolbarSet'),
                )

    def edit(self, state, felem, req):
        web.getResource('wm4.web.static/jquery/jquery.js').render(req)
        web.getResource('wm4.web.static/tinymce/jscripts/tiny_mce/tiny_mce.js').render(req)
        attrs = self.acquireAttrs(state,
            ('rows','cols','class','style','id','tabindex','accesskey')+ALL_JS,
            rows=felem.rows, cols=felem.cols, id=felem.ctrlId
        )
        attrs['name'] = felem.ctrlId

        return xh.frag(
            xh.textarea(felem.inputText, **attrs),
            xh.script(Comment(self._FCK_SCRIPT % self.get_fck_script_attrs(
                state, felem, req, attrs))),
        )
