from wm4.core import config, NOT_GIVEN
from protocols import Interface, Attribute
from lib4.symbols import *


__all__ = [
    'makeApp',
    'TEMPLATE_GLOBALS',
    'PY_NAMESPACE', 'FORM_NAMESPACE', 'KNOWN_NAMESPACES', 'DOMLET_NAMESPACES',
    'WM_WIDGET', 'FORM_WIDGET',
    'MiddlewareRegistry',
    'request', 'permission_checker', 'translations',
    'resource_manager', 'RES_FACTORY', 'filenameAsProperty',
    'IUser', 'IGroup',
    'upload_manager', 'app_secret',
    'NSDomletProperty',
    'IFormElement', 'IFormInput', 'IFormAction',
    'ILinkFactory', 'ILink', 'IMetadata',
]


def makeApp(application=None, components=None):
    from lib4.imports import importObject

    if components is None:
        components = config.Property('wsgi.components')()

    for fact in reversed(components):
        if isinstance(fact, basestring):
            reg = MiddlewareRegistry(fact)().copy()
            fact = importObject(reg.pop('factory'))
            if fact is not None:
                application = fact(application, **reg)
        else:
            if fact is not None:
                application = fact(application)
    return application


TEMPLATE_GLOBALS = config.Property('wm4.web.template.globals', {})

PY_NAMESPACE   = 'http://windmill.cz/xml4/py'
WM_NAMESPACE   = 'http://windmill.cz/wm4/web'
FORM_NAMESPACE = 'http://windmill.cz/wm4/web/form'

KNOWN_NAMESPACES  = config.Property('wm4.web.templates.known_namespaces')
DOMLET_NAMESPACES = config.Property('wm4.web.templates.domlet_namespaces')

def FORM_WIDGET(widget, default=NOT_GIVEN):
    return NSDomletProperty(FORM_NAMESPACE, widget, default)

def WM_WIDGET(widget, default=NOT_GIVEN):
    return NSDomletProperty(WM_NAMESPACE, widget, default)


class NSDomletProperty(config.AbstractProperty):

    def __new__(klass, ns, domlet, default=NOT_GIVEN):
        domlet = config.property_name(domlet)
        self = config.AbstractProperty.__new__(klass)
        self.value = (ns, domlet)
        if default is not NOT_GIVEN:
            self.setdefault(default)
        return self

    def regKey(self):
        klass = self.__class__
        ns, val = self.value
        return (klass, ns, val)

    def lookupKeys(self):
        klass = self.__class__
        ns, val = self.value
        keys = [(klass, ns, val)]
        if val.endswith('*'):
            return keys
        name = val
        while '.' in name:
            name = name[:name.rindex('.')]
            keys.append((klass, ns, name+'.*'))
        keys.append((klass, ns, '*'))
        return keys




class MiddlewareRegistry(config.Registry):
    pass



request = config.Global('request')

permission_checker = config.Resource(
    'permission_checker', factory='wm4.web.authorization.PermissionChecker')

translations = config.Property('wm4.web.translations', [])

resource_manager = config.Resource(
    'resource_manager', factory='wm4.web.resources.ResourceManager')

def RES_FACTORY(p, default=NOT_GIVEN):
    return config.Property(
        'wm4.web.resources.factory.%s' % filenameAsProperty(p), default)

def filenameAsProperty(name):
    """Convert a filename (base, no path) into a usable property string"""
    parts = filter(None,name.lower().split('.'))
    parts.reverse()
    return '.'.join(parts)




class IUser(Interface):
    '''Object representing informations about logged user'''
    uid = Attribute('Unique user ID')
    name = Attribute('User name')
    login = Attribute('Login')
    password = Attribute('Password')
    groups = Attribute('IGroup list')

class IGroup(Interface):
    '''Group of users '''
    gid = Attribute('Unique group ID')
    name = Attribute('Group name')





upload_manager = config.Setting(
    'upload_manager', 'wm4.web.sessions.UploadManager')
app_secret = config.Setting(
    'app_secret', 'xaaxf1xd2xcdxd6Zx9eApxbeZxc1Hx130aP')






class IFormElement(Interface):
    pass


class IFormInput(IFormElement):
    pass


class IFormAction(IFormElement):
    pass




class ILinkFactory(Interface):

    def __call__(req):
        '''Returns sequence of ILink instances'''


class ILink(Interface):
    label = Attribute('Visible text')
    help = Attribute('Description of target URL')

    url = Attribute('URL')
    onclick = Attribute('Javascript validator')

    visible = Attribute('Render link?')
    enabled = Attribute('Can click?')
    selected = Attribute('Is this link currently showed')

    css = Attribute('CSS class or None')



class IMetadata(Interface):
    title = Attribute('title')
    author = Attribute('author')
    keywords = Attribute('keywords or subject')
    description = Attribute('description')
