from wm4.api import *
from lib4.imports import importObject

__all__ = [
    'PagerItem', 'PagerState', 'ColumnState', 'ListState', 'QueryListState',
]



class PagerItem(object):

    def __init__(self, page, first, last, url, selected=False):
        self.page = page
        self.first = first
        self.last = last
        self.url = url
        self.selected = selected


class PagerState(binding.Component):
    # http://pylonshq.com/WebHelpers/module-webhelpers.pagination.html

    req = itemLimit = itemCount = binding.Require(suggestParent=False)

    pageParam = 'page'

    def pageCount(self):
        if not self.itemLimit:
            return 0
        else:
            return (self.itemCount+self.itemLimit-1) / self.itemLimit
    pageCount = binding.Make(pageCount)

    def __len__(self):
        return self.pageCount

    def __nonzero__(self):
        return self.pageCount > 0


    def _current(self):
        if not self.itemLimit or not self.pageCount:
            return None
        try:
            page = int(self.req.getValue('page', 0))
        except:
            page = 0
        if page < 0:
            page = 0
        if page >= self.pageCount:
            page = self.pageCount - 1
        return page
    _current = binding.Make(_current)


    baseURL = binding.Obtain('req/requestURI')

    def pageURL(self, page):
        return self.baseURL % {self.pageParam:page}

    def __getitem__(self, page):

        if page < 0 or page >= self.pageCount:
            raise IndexError('page num out of range', page)

        limit = self.itemLimit

        return PagerItem(
            page     = page,
            first    = page * limit,
            last     = min((page+1) * limit - 1, self.itemCount-1),
            url      = self.pageURL(page),
            selected = page == self._current,
        )

    def iter_around_current(self, num=10):
        # XXX
        for p in range(self.count):
            yield self[p]

    first=binding.Make(lambda s: s and s[0] or None)

    prev=binding.Make(lambda s: s and s[max(s._current-1,0)] or None)

    current=binding.Make(lambda s: s and s[s._current] or None)

    next=binding.Make(lambda s:s and s[min(s._current+1,s.pageCount-1)] or None)

    last=binding.Make(lambda s: s and s[s.pageCount-1] or None)




class ColumnState(binding.Component):

    req = binding.Obtain('../__req__')

    path = colExpr = binding.Require()



class ListState(binding.Component):

    req = count = items = binding.Require(suggestParent=False)

    limit = 20

    offset = 0

    def orderedBy(self):
        e = self.req.getValue('order_asc', None)
        if e is not None:
            return 'asc', self.col(e)
        e = self.req.getValue('order_desc', None)
        if e is not None:
            return 'desc', self.col(e)
        return None, None
    orderedBy = binding.Make(orderedBy)

    def col(self, colspec):
        raise NotImplementedError


class QueryListState(ListState):

    query = binding.Require(suggestParent=False)

    def col(self, colspec):
        colExpr = self.query.getExpr(colspec)
        return ColumnState(self, colExpr=colExpr, path=colspec)
