# http://cheeseshop.python.org/pypi/z3c.breadcrumb

from wm4.api import *

from interfaces import ILinkFactory, ILink, request

__all__ = ['DELIMITER', 'Link', 'actionLinks', 'pathLinks']


DELIMITER = object()

_ = i18n.MessageIDFactory('wm4')

class Link(object):
    '''Hypertext link that is aware of permission
       and can render itself.
    '''
    __metaclass__ = binding.Activator

    protocols.advise(instancesProvide=[ILinkFactory, ILink])

    label     = binding.Require()
    pathInfo  = binding.Require()
    help      = None
    confirm   = None
    enabled   = True
    visible   = binding.Obtain('enabled')
    css       = None
    req       = binding.Make(lambda: request())

    def __init__(self, **kw):
        binding.initAttrs(self, kw.items())

    @classmethod
    def fromRoute(klass, route, **kwds):
        kwds['req'] = req = request()
        if 'label' not in kwds:
            kwds['label'] = _(route.__class__.__name__.capitalize())
        if 'enabled' not in kwds:
            kwds['enabled'] = bool(req.allows(getattr(route,'permission',None)))
        if 'pathInfo' not in kwds:
            kwds['pathInfo'] = route.pathInfo
        return klass(**kwds)

    def selected(self):
        return self.pathInfo == self.req.pathInfo
    selected = binding.Make(selected)

    def onclick(self):
        if not self.confirm:
            return None
        from publishing import render
        confirm = render(self.req, self.confirm)
        # XXX js escape
        return "return confirm('%s');" % confirm
    onclick = binding.Make(onclick)

    def url(self):
        app = self.req.appURL
        if app.endswith('/'):
            return app + self.pathInfo[1:]
        else:
            return app + self.pathInfo
    url = binding.Make(url)

    def __repr__(self):
        return 'Link(%r -> %r)' % (self.label, self.pathInfo)



def actionLinks(baseRoute, *links):
    instances = []
    for lnk in links:
        if isinstance(lnk, basestring):
            route = baseRoute / lnk
            if route is None:
                raise ValueError('Route not found', baseRoute.pathInfo, lnk)
            lnk = route.asAction
        instances.append(lnk)
    return instances


def pathLinks(baseRoute, include=True):
    obj = baseRoute
    if not include:
        obj = obj.parentRoute
    path = []
    while obj:
        link = getattr(obj, 'asPath', None)
        if link is not None:
            path.append(link)
        obj = obj.parentRoute
    path.reverse()
    return path
