from wm4.api import *
from dispatch.strategy import default
from lib4.imports import importObject, importString
from types import FunctionType, MethodType
import sys, os

import interfaces
from errors import NotFound
from environ import Request

__all__ = ['DEFAULT_METHODS', 'PublishingMiddleware', 'render']


DEFAULT_METHODS = set(('GET', 'HEAD', 'POST'))

class PublishingMiddleware:
    application = None

    def __init__(self, application, **params):
        self.application = application
        self.__dict__.update(params)

    def __call__(self, environ, start_response):
        req = interfaces.request()
        route = environ['windmill.route']
        req.params.update(route.get_params())
        handler = route.handler
        resp = self.handleHTTP(req, handler)
        if isinstance(resp, tuple):
            status, headers, body = resp
            start_response(status, headers)
            return body
        else:
            start_response(resp.status, resp.headers)
            return resp.body

    # HTTP handler

    @dispatch.generic()
    def handleHTTP(self, req, handler):
        '''HTTP handler'''

    @handleHTTP.when('handler == None')
    def handle_not_found(self, req, handler):
        raise NotFound(req.requestURL)

    @handleHTTP.when('handler in basestring')
    def handle_with_string_handler(self, req, handler):
        return self.handleHTTP(req, importString(handler))


    @handleHTTP.when('handler in FunctionType or handler in MethodType')
    def handle_with_callable_handler(self, req, handler):
        allowed = getattr(handler, 'allowed_methods', DEFAULT_METHODS)
        if req.method not in allowed:
            raise web.MethodNotAllowed(req.method)
        return req.apply(handler)





@dispatch.generic()
def render(req, ob):
    '''Render various instances within given context'''

@render.when(default)
def render_default(req, ob):
    return unicode(ob)


@render.when('ob is None')
def render_none(req, ob):
    return u''


import datetime

@render.when('ob in unicode')
def render_unicode(req, ob):
    return ob


#@render.when('ob in basestring')
#def render_basestring(req, ob):
#    return unicode(ob, req.output_encoding)


@render.when('ob in datetime.date')
def render_date(req, ob):
    # XXX i18n
    return u'%02d.%02d.%d' % (ob.day, ob.month, ob.year)

@render.when('ob in datetime.datetime')
def render_date(req, ob):
    # XXX i18n
    return u'%02d.%02d.%d %02d:%02d' % (ob.day, ob.month, ob.year,
                                        ob.hour, ob.minute)



from wm4.i18n.api import MessageID, EncapsulatedMessageID

@render.when('ob in MessageID')
def render_message_id(req, ob):
    return req.translate(ob)

@render.when('ob in EncapsulatedMessageID')
def render_encapsulated_message_id(req, ob):
    return ob.translate(req.language)


@render.when('ob in schema.FeatureError')
def render_feature_error(req, ob):
    try:
        return req.translate(ob.msgId)
    except:
        return ob.msgId

from xml4.elements import Node
from xml4.serialization import xhtml_serializer

@render.when('ob in Node')
def render_xml_node(req, ob):
    # XXX the same as Template.render
    return u''.join(xhtml_serializer(ob.expand(None)))
