# Public SOAP services
# http://www.xmethods.net/
#
# WSDL styles
# http://www-128.ibm.com/developerworks/webservices/library/ws-whichwsdl/
#
# Web Services Transactions specifications
# http://www-128.ibm.com/developerworks/library/specification/ws-tx/
#
# soaplib
# http://trac.optio.webfactional.com/

from wm4.api import *
from lib4.imports import lazyModule
import sys

from publishing import PublishingMiddleware
from errors import WebException, NotAllowed, MethodNotAllowed, BadRequest

__all__ = ['RpcError', 'rpc', 'RpcHandler']


class RpcError(Exception):
    pass


def rpc(permission=None):
    def rpc(func):
        # XXX generate default signature and returns for SOAP
        func.permission = permission
        func.isRPC = True
        return func
    return rpc


class RpcHandlerMeta(type(binding.Component)):

    def __get__(klass, obj, typ=None):
        if obj is None:
            return klass
        attr = klass.__name__
        odct = obj.__dict__
        val = odct.get(attr, NOT_FOUND)
        if val is NOT_FOUND:
            val = odct[attr] = klass(req=obj.req)
        return val


class RpcHandler(binding.Component):

    __metaclass__ = RpcHandlerMeta

    req = binding.Require()
    isRPC = True
    permission = None
    logger = binding.Make(lambda: naming.lookup('logger:web.rpc'))

    def _rpc_dispatch(self, method):
        meth = self
        req = self.req
        path = [p for p in method.split('.') if p]
        while meth is not None:
            if not getattr(meth, 'isRPC', None):
                raise RpcError('private method', method)
            perm = meth.permission
            if perm is not None and not req.allows(perm):
                raise web.NotAllowed('method not allowed', method, perm)
            if not path:
                return meth
            a = path.pop(0)
            meth = getattr(meth, a, None)
        raise RpcError('method not found', method)

    def _rpc_request(self):
        req = self.req
        env = req.environ
        if req.method != 'POST':
            raise MethodNotAllowed("Invalid REQUEST_METHOD %r" % req.method)
        if env['CONTENT_TYPE'] != 'text/xml':
            raise MethodNotAllowed(
                "Invalid CONTENT_TYPE %r" % env['CONTENT_TYPE'])
        cl = int(env['CONTENT_LENGTH'])
        data = env['wsgi.input'].read(cl)
        try:
            return self.loads(data)
        except:
            raise BadRequest("cannot parse request", str(sys.exc_value))

    def _rpc_response(self, response):
        res = self.req.response
        res.addheader('Content-Type', 'text/xml')
        res.addheader('Content-Length', str(len(response)))
        res.body = [response]
        return res

    xmlrpclib = lazyModule('xmlrpclib')

    dumps = loads = Fault = binding.Delegate('xmlrpclib')

    def __call__(self):
        # generate response
        try:
            params, method = self._rpc_request()
            meth = self._rpc_dispatch(method)
            response = meth(*params)
            # wrap response in a singleton tuple
            response = (response,)
            response = self.dumps(response, methodresponse=1)
        except WebException, e:
            raise
        except self.Fault, fault:
            self.logger.exception('rpc call failed')
            response = self.dumps(fault)
        except:
            # report exception back to server
            self.logger.exception('rpc call raised exception')
            response = self.dumps(
                self.Fault(1, "%s:%s" % (sys.exc_type, sys.exc_value))
            )
        return self._rpc_response(response)


@PublishingMiddleware.handleHTTP.when('handler in RpcHandlerMeta')
def handle_handler_class(publisher, req, handler):
    return handler(req=req)()
