from wm4.api import *
from xml4.elements import Element, Node, Literal
from xml4.templates import TemplateParser as _TemplateParser, State
from xml4.const import DEFER
from xml4.utils import escape
from xml4.serialization import xhtml_serializer
from lib4.imports import importObject

from resources import Resource
from interfaces import *
from publishing import render
from utils import render_js_value


__all__ = [
    'Template', 'makeParser', 'parseFile', 'parseSrc', 'renderTemplate',
]



# Default globals for every template

def renderTemplate(__template, req, **add_locals):
    ldict = dict(req=req)
    ldict.update(req.params)
    ldict.update(add_locals)
    ldict.setdefault('workspace', settings.workspace())
    state = State(ldict=ldict, renderer=lambda s,o:render(req,o))
    return u''.join(xhtml_serializer(__template.expand(state)))


class Template(Resource):

    template = binding.Make(lambda s: parseFile(s.filename))

    tmpl_namespace = expand = binding.Delegate('template')

    def render(_self, req, **add_locals):
        ldict = dict(req=req)
        ldict.update(req.params)
        ldict.update(add_locals)
        ldict.setdefault('workspace', settings.workspace())
        state = State(ldict=ldict, renderer=lambda s,o:render(req,o))
        return u''.join(xhtml_serializer(_self.template.expand(state)))

    def handleHTTP(_self, req, **add_locals):
        resp = req.response
        enc = req.output_encoding
        output = _self.render(req, **add_locals).encode(enc)
        resp.addheader('Content-type', 'text/html;charset=%s' % enc)
        resp.addheader('Content-Length', str(len(output)))
        resp.body = [output]
        return resp



def findDomlet(state, ns, name):
    fact = state.get((ns,name))
    if fact is None:
        fact = importObject(NSDomletProperty(ns, name)())
    return fact


class TemplateParser(_TemplateParser):

    def findTemplate(self, tmpl):
        if isinstance(tmpl, basestring):
            tmpl = resource_manager().getResource(tmpl)
        return tmpl

def makeParser():
    tns = {
        'config':config, 'i18n':i18n, 'settings':settings, 'web':web,
        'NOT_GIVEN':NOT_GIVEN, 'NOT_FOUND':NOT_FOUND,
        'render_js_value': render_js_value, 'Literal': Literal,
    }
    tns.update(TEMPLATE_GLOBALS())
    return TemplateParser(
        tmpl_namespace = tns,
        domlet_namespaces = dict((n,findDomlet) for n in DOMLET_NAMESPACES()),
        known_namespaces = KNOWN_NAMESPACES(),
    )

def parseFile(filename):
    return makeParser().parseFile(filename)

def parseSrc(src):
    return makeParser().parse(src)
