# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.authorization import *
from wm4.web.routes import Route, Const
from comp4.users.entities import *
import sys

from utils import *

a = Permission()
b = Permission()
c = Permission()
d = Permission()


g10 = Group(gid=10, name='g10')
g20 = Group(gid=20, name='g20')
u1 = User(uid=1, name='User1', login='u1', email='u1@x')
u1.groups = [g10]
u2 = User(uid=2, name='User2', login='u2', email='u1@x')
u2.groups = [g10, g20]

class TestPermissionChecker(WebCase):

    def setUser(self, env, usr):
        class Auth:
            def getUser(self, env):
                return usr
        env.setdefault('windmill.authenticators', {})['remote_user'] = Auth()

    def test01_And(self):
        p = a & b
        self.failUnless(isinstance(p, _And))
        self.failUnless(a is p.perms[0])
        self.failUnless(b is p.perms[1])

    def test02_Or(self):
        p = a | b
        self.failUnless(isinstance(p, _Or))
        self.failUnless(a is p.perms[0])
        self.failUnless(b is p.perms[1])

    def test03_Not(self):
        p = ~a
        self.failUnless(isinstance(p, _Not))
        self.failUnless(a is p.perm)

    def test04_ComplexExpression(self):
        p = a | ~b & c
        self.failUnless(isinstance(p,_Or))
        self.failUnless(p.perms[0] is a)
        self.failUnless(isinstance(p.perms[1],_And))
        self.failUnless(isinstance(p.perms[1].perms[0],_Not))
        self.failUnless(p.perms[1].perms[0].perm is b)
        self.failUnless(p.perms[1].perms[1] is c)

    def test05_AndOrWithOneArgument(self):
        self.assertEqual(_And(a), a)
        self.assertEqual(_Or(a), a)

    def test06_NotReduction(self):
        self.assertEqual(_Not(_Not(a)), a)
        self.assertEqual(~~(a), a)
        self.assertEqual(_Not(Anybody()), Nobody())
        self.assertEqual(~Anybody(), Nobody())
        self.assertEqual(_Not(Nobody()), Anybody())
        self.assertEqual(~Nobody(), Anybody())

    def test10_SimpleComparison(self):
        p1 = Authenticated()
        p2 = Authenticated()
        assert p1 is not p2
        self.assertEqual(p1, p2)
        p1 = MemberOf('foo')
        p2 = MemberOf('foo')
        assert p1 is not p2
        self.assertEqual(p1, p2)
        p1 = MemberOf('foo')
        p2 = MemberOf('bar')
        self.assertNotEqual(p1, p2)

    def test12_ExpressionComparison(self):
        p1 = _Not(Authenticated())
        p2 = _Not(Authenticated())
        assert p1 is not p2
        self.assertEqual(p1, p2)
        p1 = _Or(Authenticated(), _Not(MemberOf('foo')))
        p2 = _Or(Authenticated(), _Not(MemberOf('foo')))
        assert p1 is not p2
        self.assertEqual(p1, p2)
        p1 = _Or(Authenticated(), _Not(MemberOf('foo')))
        p2 = _Or(Authenticated(), _Not(MemberOf('bar')))
        self.assertNotEqual(p1, p2)


    def test20_Authenticated(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        self.assertEqual(pc.allows(Authenticated(),env), True)
        assert isinstance(pc.allows(~Authenticated(),env), web.Denial)
        self.setUser(env, None)
        assert isinstance(pc.allows(Authenticated(),env), web.Denial)
        self.assertEqual(pc.allows(~Authenticated(),env), True)

    def test21_Anybody(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        self.assertEqual(pc.allows(Anybody(),env), True)
        
    def test22_Nobody(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        assert isinstance(pc.allows(Nobody(),env), web.Denial)
        
    def test30_UserName(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        self.assertEqual(pc.allows(UserName('u1'),env), True)
        self.assertEqual(pc.allows(~UserName('u2'),env), True)
        self.assertEqual(pc.allows(UserName('u1')|UserName('u2'),env), True)
        assert isinstance(
            pc.allows(UserName('u2'), env), web.Denial)
        
    def test35_MemberOf(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        self.assertEqual(pc.allows(MemberOf('g10'),env), True)
        self.assertEqual(pc.allows(~MemberOf('g30'),env), True)
        self.assertEqual(pc.allows(MemberOf('g10')|MemberOf('g30'),env), True)
        assert isinstance(
            pc.allows(MemberOf('g30'), env), web.Denial)
        assert isinstance(
            pc.allows(MemberOf('g10') & MemberOf('g30'),env), web.Denial
        )

    def test40_RemoteAddr(self):
        pc, env = web.PermissionChecker(), self.env()
        perm = RemoteAddr('127.0.0.0/24')
        self.assertEqual(pc.allows(perm, env), True)

    def test41_RemoteAddr(self):
        pc, env = web.PermissionChecker(), self.env()
        perm = RemoteAddr('10.1.1.0/24')
        self.failIf(pc.allows(perm, env))

    def test50_DefaultList(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        assert isinstance(pc.allows(ListPerm(), env), web.Denial)
        self.assertEqual(pc.allows(ListPerm(default=Anybody()),env), True)
        self.assertEqual(pc.allows(ListPerm(default=Authenticated()),env), True)

    def test51_ListWithExplicitUser(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        p = ListPerm(UserName('u1'), ~GroupName('g10'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(~UserName('u1'), GroupName('g10'))
        assert isinstance(pc.allows(p, env), web.Denial)

    def test52_ListWithExplicitGroup(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u1)
        p = ListPerm(GroupName('g10'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(UserName('u2'), GroupName('g10'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(~GroupName('g10'))
        assert isinstance(pc.allows(p, env), web.Denial)
        p = ListPerm(~GroupName('g10'), default=Anybody())
        assert isinstance(pc.allows(p, env), web.Denial)
        p = ListPerm(~UserName('u2'), ~GroupName('g10'))
        assert isinstance(pc.allows(p, env), web.Denial)

    def test53_ListWithMoreThanOneMatchingGroup(self):
        pc, env = web.PermissionChecker(), self.env()
        self.setUser(env, u2)
        p = ListPerm(GroupName('g10'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(GroupName('g20'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(GroupName('g10'), GroupName('g20'))
        self.assertEqual(pc.allows(p, env), True)
        p = ListPerm(GroupName('g10'), ~GroupName('g20'), default=Anybody())
        assert isinstance(pc.allows(p, env), web.Denial)
        p = ListPerm(~GroupName('g10'), GroupName('g20'), default=Anybody())
        assert isinstance(pc.allows(p, env), web.Denial)


class TestPermissionParser(TestCase):
    
    def parseACL(self, expr):
        return web.PermissionChecker().parse(expr)

    def test10_EmptyExpression(self):
        r = self.parseACL('')
        self.assertEqual(r, None)
        r = self.parseACL(None)
        self.assertEqual(r, None)
   
    def test20_SinglePermissionWithoutParameter(self):
        r = self.parseACL('auth()')
        self.assertEqual(r, Authenticated())
        
    def test22_SinglePermissionWithParameter(self):
        r = self.parseACL('user("foo")')
        self.assertEqual(r, UserName('foo'))

    def test24_SingleNonexistentPermission(self):
        self.assertRaises(NameError, self.parseACL, 'foo("bar")')
    
    def test30_Expression(self):
        r = self.parseACL('user("foo") & group("admin")')
        self.assertEqual(r, _And(UserName('foo'), MemberOf('admin')))
        
    def test31_Expression(self):
        r = self.parseACL('user("foo") | ~group("bar")')
        self.assertEqual(r, _Or(UserName('foo'), _Not(MemberOf('bar'))))
    
    def test32_ListPerm(self):
        r = self.parseACL('list(user("foo"), auth(), ~group("bar"))')
        self.assertEqual(r,
            ListPerm(UserName('foo'), Authenticated(), _Not(MemberOf('bar'))))
        
    def test39_NonexistentPermissionClass(self):
        self.assertRaises(NameError, self.parseACL, 'user("foo") | ~baz("bar")')
    
    def test40_InvalidExpression(self):
        self.assertRaises(NameError,
            self.parseACL, 'os.system("rm -rf /")')
        











def handler(response, environ):
    response.body = ['OK']
    return response

class S(Route):
    handler = Const(handler)
    class anybody(Route):
        match = 'anybody'
    class nobody(Route):
        match = 'nobody'
        permission = Const(Nobody())
    class auth(Route):
        match = 'auth'
        permission = Const(Authenticated())


SM = "[WSGI Middleware sitemap]\nsitemap = importString('%s.S')\n" % __name__

class User(Record):
    protocols.advise(instancesProvide=[web.IUser])

_users = dict(
    ian = User(username='ian', password='foo'),
    bob = User(username='bob', password='bar'),
)

def find_user(middleware, environ, username):
    return _users.get(username)

AU = '[WSGI Middleware authentication]\nfind_user="%s.find_user"\n' % __name__



class TestAuthorizationMiddleware(WebCase):

    @with_ini(SM)
    def test10_Anybody(self):
        s,h,b = self.handle(web.makeApp(), self.env('/anybody'))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'OK')

    @with_ini(SM)
    def test20_Nobody(self):
        s,h,b = self.handle(web.makeApp(), self.env('/nobody'))
        self.assertEqual(s, '403 Forbidden')

    @with_ini(SM+AU)
    def test30_Unauthenticated(self):
        s,h,b = self.handle(web.makeApp(), self.env('/auth'))
        self.assertEqual(s, '403 Forbidden')

    @with_ini(SM+AU)
    def test31_BadUser(self):
        app = web.makeApp()
        s,h,b = self.handle(
            app, self.env('https://localhost/auth', auth=('ian','fo'))
        )
        self.assertEqual(s, '401 Unauthorized')

    @with_ini(SM+AU)
    def test32_Authenticated(self):
        app = web.makeApp()
        s,h,b = self.handle(
            app, self.env('https://localhost/auth', auth=('ian','foo'))
        )
        self.assertEqual(s, '200 OK')


def suite():
    return TestSuite((
        makeSuite(TestPermissionChecker),
        makeSuite(TestPermissionParser),
        makeSuite(TestAuthorizationMiddleware),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
