# -*- coding: utf-8 -*-

from unittest import *
from wm4.api import *

from utils import *


class FooMiddleware:

    def __init__(self, application, **kw):
        self.application = application
        self.__dict__.update(kw)

class BarMiddleware(FooMiddleware):
    pass

class BazMiddleware(FooMiddleware):
    pass

class TestConfig(WebCase):

    @with_ini((
        '[foo]\n'
        'bar = 2 * 21\n'
    ))
    def test01_WithIni(self):
        self.assertEqual(config.Property('foo.bar')(), 42)

    def test10_Middleware(self):
        self.parse_ini((
            '[Import On Demand]\n'
            'this_mod = "%(this_mod)s"\n'

            '[wsgi]\n'
            'components = "foo", "bar"\n'

            '[WSGI Middleware foo]\n'
            'factory = this_mod.FooMiddleware\n'
            'f1 = 42\n'
            'f2 = "F"\n'

            '[WSGI Middleware bar]\n'
            'factory = "%(this_mod)s.BarMiddleware"\n'
            'b1 = 43\n'
            'b2 = "B"\n'
        ) % {'this_mod':__name__})
        app = web.makeApp()
        self.failUnless(isinstance(app, FooMiddleware))
        self.assertEqual(app.f1, 42)
        self.assertEqual(app.f2, 'F')

        self.failUnless(isinstance(app.application, BarMiddleware))
        self.assertEqual(app.application.b1, 43)
        self.assertEqual(app.application.b2, 'B')

        self.assertEqual(app.application.application, None)

    def test12_MiddlewareInheritance(self):
        self.parse_ini((
            '[Import On Demand]\n'
            'this_mod = "%(this_mod)s"\n'

            '[wsgi]\n'
            'components = "foo", "bar"\n'

            '[WSGI Middleware foo]\n'
            'factory = this_mod.FooMiddleware\n'
            'f1 = 42\n'
            'f2 = "F"\n'

            '[WSGI Middleware bar]\n'
            'factory = "%(this_mod)s.BarMiddleware"\n'
            'b1 = 43\n'
            'b2 = "B"\n'
        ) % {'this_mod':__name__})

        exit = config.enter_(config.Config())

        self.parse_ini((
            '[Import On Demand]\n'
            'this_mod = "%(this_mod)s"\n'

            '[WSGI Middleware bar]\n'
            'factory = "%(this_mod)s.BazMiddleware"\n'
            'b1 = 44\n'
        ) % {'this_mod':__name__})

        app = web.makeApp()
        self.failUnless(isinstance(app, FooMiddleware))
        self.assertEqual(app.f1, 42)
        self.assertEqual(app.f2, 'F')

        self.failUnless(isinstance(app.application, BazMiddleware))
        self.assertEqual(app.application.b1, 44)
        self.assertEqual(app.application.b2, 'B')

        self.assertEqual(app.application.application, None)

        exit()

        app = web.makeApp()
        self.failUnless(isinstance(app, FooMiddleware))
        self.assertEqual(app.f1, 42)
        self.assertEqual(app.f2, 'F')

        self.failUnless(isinstance(app.application, BarMiddleware))
        self.assertEqual(app.application.b1, 43)
        self.assertEqual(app.application.b2, 'B')

        self.assertEqual(app.application.application, None)


    def test20_Domlets(self):
        from wm4.web.widgets import DebugWidget
        p = web.NSDomletProperty('http://windmill.cz/wm4/web', 'debug')
        self.assertEqual(p(), DebugWidget)

        from wm4.web.forms.widgets import ErrorWidget
        p = web.NSDomletProperty('http://windmill.cz/wm4/web/form', 'errors')
        self.assertEqual(p(), ErrorWidget)



def suite():
    return TestSuite((
        makeSuite(TestConfig),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
