# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.cookies import *

from utils import *



class TestCookies(WebCase):

    def test10_UndefinedCookie(self):
        def test(env, start_response):
            self.assertEqual(getCookie(env, 'c'), None)
            self.assertEqual(getCookie(env, 'c', 42), 42)
            start_response('200', [])
            return ['ok']
        app = self.mkapp(
            'environment', 'errors', 'transaction',
            'cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env())
        self.assertEqual(h, [])

    def test12_DefinedCookie(self):
        def test(env, start_response):
            self.assertEqual(getCookie(env, 'c'), '42')
            self.assertEqual(getCookie(env, 'd'), u'čeština')
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env(cookies={'c':42, 'd':u'čeština'}))
        self.assertEqual(h, [])

    def test20_NewCookie(self):
        def test(env, start_response):
            setCookie(env, 'c', 'C')
            setCookie(env, 'd', 42, max_age=100)
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env(cookies={'c':42, 'd':u'čeština'}))
        self.assertEqual(strip_semicol(h), [
            ('Set-Cookie','c=C'),
            ('Set-Cookie','d=42; Max-Age=100'),
        ])

    def test22_OldCookieDoesntProduceHeader(self):
        def test(env, start_response):
            self.assertEqual(getCookie(env, 'c'), 'C')
            setCookie(env, 'd', 42)
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env(cookies={'c':'C'}))
        self.assertEqual(strip_semicol(h), [('Set-Cookie','d=42')])

    def test24_UpdateCookieValue(self):
        def test(env, start_response):
            setCookie(env, 'c', 42)
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env(cookies={'c':42}))
        self.assertEqual(strip_semicol(h), [('Set-Cookie','c=42')])

    def test30_DeleteCookie(self):
        def test(env, start_response):
            delCookie(env, 'c')
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        s,h,b = self.handle(app, self.env(cookies={'c':42}))
        self.assertEqual(strip_semicol(h), [
            ('Set-Cookie','c=; expires=Thu, 01-01-1970 00:00:01 GMT')
        ])

    def test40_LargeCookie(self):
        def test(env, start_response):
            setCookie(env, 'c', 'x' * 50000)
            start_response('200', [])
            return ['ok']
        app = self.mkapp('cookies', funcMiddleware(test))
        self.assertRaises(ValueError, self.handle, app)

def suite():
    return TestSuite((
        makeSuite(TestCookies),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
