# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.forms.controllers import *
from wm4.schema.tests.utils import assertEqualEntityError
from os.path import join, dirname, abspath
from wm4.utils.files import IFileInfo
from lib4.extremes import Any

from utils import *

NE = schema.NormalizationError
SE = schema.FeatureSyntaxError
FE = schema.FeatureError

class ToplevelFormController(InstanceCtrl, FormCtrl):
    pass

class FormCase(WebCase):

    def form(self, klass, oldValue=NOT_GIVEN, get={}, post={}):
        f = klass(ctrlReq=self.req(get=get, post=post), oldValue=oldValue)
        f.res = f()
        return f

    def check(self, elem, **equals):
        for attr, rv in equals.items():
            cv = getattr(elem, attr)
            if attr == 'ctrlError' and rv is not None:
                self.failUnless(isinstance(cv, rv),
                    '%s.%s: %r is not instance of %r' % (
                    elem.ctrlId or '$', attr, cv, rv))
            else:
                self.assertEqual(cv, rv, '%s.%s: %r != %r' % (
                    elem.ctrlId or '$', attr, cv, rv))

class LoggingInput(InputCtrl):

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.ctrlRoot.log.add('change_%s' % self.ctrlId)

    @event_handler(OnErrorEvent)
    def on_error(self, evt):
        self.ctrlRoot.log.add('error_%s' % self.ctrlId)


class LoggingButtonCtrl(ButtonCtrl):

    @event_handler(OnClickEvent)
    def on_submit(self, evt):
        self.ctrlRoot.log.add('submit_%s' % self.ctrlId)










class FIN(ToplevelFormController):

    ctrlType = Record

    log = binding.Make(set)

    class a(LoggingInput):
        ctrlType = int

    class b(LoggingInput):

        def defaultInput(self):
            return u'bDefault'

    class s(InstanceCtrl):

        ctrlType = Record

        class sa(LoggingInput):
            ctrlType = int

        class sb(LoggingInput):
            ctrlType = str

    @event_handler(OnChangeEvent, 's')
    def on_change(self, evt):
        self.log.add('change_s')

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.log.add('submit')



class TestControllers(WebCase):

    def test10_HierarchyTraversal(self):
        f = FIN(ctrlReq=self.req())
        assert isinstance(f['a'], FIN.a)
        assert isinstance(f['s'], FIN.s)
        assert isinstance(f['s/sa'], FIN.s.sa)
        assert f[''] is f['.'] is f['/'] is f
        assert f['a'] is f['/a'] is f['a']['']
        assert f['s/sa'] is f['/s/sa'] is f['a/../s/sb/../sa']

    def test11_ChildControllersAreCached(self):
        f = FIN(ctrlReq=self.req())
        a1 = f['a']
        a2 = f['/a']
        assert a1 is a2

    def test20_ParentComponent(self):
        f = FIN(ctrlReq=self.req())
        GPC = binding.getParentComponent
        self.assertEqual(f.ctrlParent, None)
        self.assertEqual(GPC(f), None)
        self.assertEqual(f['a'].ctrlParent, f)
        self.assertEqual(GPC(f['a']), f)
        self.assertEqual(f['s'].ctrlParent, f)
        self.assertEqual(GPC(f['s']), f)
        self.assertEqual(f['s/sa'].ctrlParent, f['s'])
        self.assertEqual(GPC(f['s/sa']), f['s'])

    def test21_ComponentName(self):
        f = FIN(ctrlReq=self.req())
        GCN = binding.getComponentName
        self.assertEqual(f.ctrlName, None)
        self.assertEqual(GCN(f), None)
        self.assertEqual(f['a'].ctrlName, 'a')
        self.assertEqual(GCN(f['a']), 'a')
        self.assertEqual(f['s'].ctrlName, 's')
        self.assertEqual(GCN(f['s']), 's')
        self.assertEqual(f['s/sa'].ctrlName, 'sa')
        self.assertEqual(GCN(f['s/sa']), 'sa')

    def test22_ComponentPath(self):
        f = FIN(ctrlReq=self.req())
        GCP = lambda s,r=None: str(binding.getComponentPath(s,r))
        self.assertEqual(GCP(f), '')
        self.assertEqual(GCP(f['a']), '/a')
        self.assertEqual(GCP(f['s']), '/s')
        self.assertEqual(GCP(f['s/sa']), '/s/sa')

    def test23_ComponentRoot(self):
        f = FIN(ctrlReq=self.req())
        GRC = binding.getRootComponent
        assert GRC(f) is f is f.ctrlRoot
        assert GRC(f['a']) is f is f['a'].ctrlRoot
        assert GRC(f['s']) is f is f['s'].ctrlRoot
        assert GRC(f['s/sa']) is f is f['/s/sa'].ctrlRoot

    def test30_CtrlId(self):
        f = FIN(ctrlReq=self.req())
        self.assertEqual(f['a'].ctrlId, 'a')
        self.assertEqual(f['s'].ctrlId, 's')
        self.assertEqual(f['s/sa'].ctrlId, 's$sa')

    def test30_CtrlIdWithPrefix(self):
        f = FIN(ctrlReq=self.req(), ctrlName='p')
        self.assertEqual(f['a'].ctrlId, 'p$a')
        self.assertEqual(f['s'].ctrlId, 'p$s')
        self.assertEqual(f['s/sa'].ctrlId, 'p$s$sa')

    def test31_CtrlLabel(self):
        f = FIN(ctrlReq=self.req())
        self.assertEqual(f['a'].ctrlLabel, 'A')
        self.assertEqual(f['s'].ctrlLabel, 'S')
        self.assertEqual(f['s/sa'].ctrlLabel, 'Sa')

    def test32_CtrlVisibility(self):
        f = FIN(ctrlReq=self.req())
        self.assertEqual(f.ctrlVisibility, 'visible')
        self.assertEqual(f['a'].ctrlVisibility, 'visible')
        self.assertEqual(f['s'].ctrlVisibility, 'visible')
        self.assertEqual(f['s/sa'].ctrlVisibility, 'visible')
        f = FIN(ctrlReq=self.req(), ctrlVisibility='hidden')
        self.assertEqual(f.ctrlVisibility, 'hidden')
        self.assertEqual(f['a'].ctrlVisibility, 'hidden')
        self.assertEqual(f['s'].ctrlVisibility, 'hidden')
        self.assertEqual(f['s/sa'].ctrlVisibility, 'hidden')

    def test33_CtrlReadOnly(self):
        f = FIN(ctrlReq=self.req())
        self.assertEqual(f.ctrlReadOnly, False)
        self.assertEqual(f['a'].ctrlReadOnly, False)
        self.assertEqual(f['s'].ctrlReadOnly, False)
        self.assertEqual(f['s/sa'].ctrlReadOnly, False)
        f = FIN(ctrlReq=self.req(), ctrlReadOnly=True)
        self.assertEqual(f.ctrlReadOnly, True)
        self.assertEqual(f['a'].ctrlReadOnly, True)
        self.assertEqual(f['s'].ctrlReadOnly, True)
        self.assertEqual(f['s/sa'].ctrlReadOnly, True)

    def test40_EventHandlers(self):
        log = set([])
        class F2(FIN):
            class b(FIN.b):
                @event_handler(OnChangeEvent)
                def on_change_b(self, evt):
                    log.add('change_b')
            @event_handler(OnChangeEvent)
            def on_change(self, evt):
                log.add('change')
            @event_handler(OnChangeEvent, 'b')
            def on_change_b(self, evt):
                log.add('change_b2')
        f2 = F2(ctrlReq=self.req()); f2()
        OnChangeEvent(f2, old=NOT_GIVEN, new=NOT_GIVEN)
        self.assertEqual(log, set(['change']))
        log.clear()
        OnChangeEvent(f2['b'], old=NOT_GIVEN, new=NOT_GIVEN)
        self.assertEqual(log, set(['change_b', 'change_b2']))














class FB(ToplevelFormController):

    log = binding.Make(set)

    class a(LoggingButtonCtrl):
        pass

    class b(LoggingButtonCtrl):
        pass

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.log.add('submit')


class TestButtons(FormCase):

    def test10_NoAction(self):
        f = self.form(FB)
        self.check(f, formAction=None, log=set([]), res=False)

    def test20_Submit(self):
        f = self.form(FB, post={'doSubmit':'Save'})
        self.check(f, formAction=f['doSubmit'], log=set(['submit']), res=True)

    def test21_SubmitA(self):
        f = self.form(FB, post={'a':'Click'})
        self.check(f, formAction=f['a'], log=set(['submit_a']), res=False)

    def test30_GetSubmit(self):
        f = self.form(FB, get={'doSubmit':'Save'})
        self.check(f, formAction=f['doSubmit'], log=set(['submit']), res=True)

    def test40_MultiSubmit(self):
        self.assertRaises(web.WebException,self.form,FB,post={'a':'A','b':'B'})









class TestInputs(FormCase):

    def test10_DefaultText(self):
        f = self.form(FIN)
        self.check(f['a'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['b'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='bDefault',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f, oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                log=set([]), res=False)

    def test12_InputTextOnAdd(self):
        f = self.form(FIN, post={'doSubmit':'Save', 'b':' boo ', 's$sb':'iSB'})
        self.check(f['a'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['b'], oldValue=NOT_GIVEN, newValue='boo',
                inputText='boo',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue='iSB',
                inputText='iSB',  ctrlError=None)
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=Record(sb='iSB'))
        self.check(f, oldValue=NOT_GIVEN, res=True,
                newValue=Record(b='boo', s=Record(sb='iSB')),
                log=set(['change_b','change_s','change_s$sb','submit']))

    def test15_InputTextOnError(self):
        f = self.form(FIN, post={'doSubmit':'','a':'foo', 's$sa':'bar'})
        self.check(f['a'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='foo',  ctrlError=NE)
        self.check(f['b'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='bar',  ctrlError=NE)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f, oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                log=set(['error_a','error_s$sa','submit']), res=False)
        assertEqualEntityError(self, f.formErrors, [('a',NE),('s.sa',NE)])

    def test16_ValueWithoutSubmit(self):
        f = self.form(FIN, post={'b':'foo'})
        self.check(f['a'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['b'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='foo',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f, oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                log=set([]), res=False)

    def test20_OldValues(self):
        r = Record(a=23, b='vB', s=Record(sb='vSB'))
        f = self.form(FIN, oldValue=r)
        self.check(f['a'], oldValue=23, newValue=23,
                inputText='23',  ctrlError=None)
        self.check(f['b'], oldValue='vB', newValue='vB',
                inputText='vB',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue='vSB', newValue='vSB',
                inputText='vSB',  ctrlError=None)
        self.check(f['s'], oldValue=r.s, newValue=r.s)
        self.check(f, oldValue=r, newValue=r, log=set([]), res=False)

    def test22_InputTextOnEdit(self):
        r = Record(a=23, b='vB', s=Record(sb='vSB'))
        f = self.form(FIN, oldValue=r.copy(),
            post={'doSubmit':'Save', 'a':42, 'b':' boo ', 's$sb':'iSB'})
        self.check(f['a'], oldValue=23, newValue=42,
                inputText='42',  ctrlError=None)
        self.check(f['b'], oldValue='vB', newValue='boo',
                inputText='boo',  ctrlError=None)
        self.check(f['s/sa'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue='vSB', newValue='iSB',
                inputText='iSB',  ctrlError=None)
        r = Record(a=42, b='boo', s=Record(sb='iSB'))
        self.check(f['s'], oldValue=r.s, newValue=r.s)
        self.check(f, oldValue=r, newValue=r, res=True,
            log=set(['change_a','change_b','change_s','change_s$sb','submit']))

    def test23_InputTextOnEditWithoutChanges(self):
        r = Record(a=23, b='vB', s=Record(sa=42))
        f = self.form(FIN, oldValue=r,
            post={'doSubmit':'Save', 'a':23, 'b':'vB ', 's$sa':'42'})
        self.check(f['a'], oldValue=23, newValue=23,
                inputText='23',  ctrlError=None)
        self.check(f['b'], oldValue='vB', newValue='vB',
                inputText='vB',  ctrlError=None)
        self.check(f['s/sa'], oldValue=42, newValue=42,
                inputText='42',  ctrlError=None)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s'], oldValue=r.s, newValue=r.s)
        self.check(f, oldValue=r, newValue=r, log=set(['submit']), res=True)

    def test24_InputTextOnEditWithoutInput(self):
        r = Record(a=23, b='vB', s=Record(sa=42))
        f = self.form(FIN, oldValue=r, post={'doSubmit':'Save'})
        self.check(f['a'], oldValue=23, newValue=23,
                inputText='',  ctrlError=None)
        self.check(f['b'], oldValue='vB', newValue='vB',
                inputText='',  ctrlError=None)
        self.check(f['s/sa'], oldValue=42, newValue=42,
                inputText='',  ctrlError=None)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s'], oldValue=r.s, newValue=r.s)
        self.check(f, oldValue=r, newValue=r, log=set(['submit']), res=True)

    def test25_InputTextOnError(self):
        r = Record(a=23, b='vB', s=Record(sa=42))
        f = self.form(FIN, oldValue=r,
            post={'doSubmit':'','a':'foo', 'b':'vB', 's$sa':'bar'},
        )
        self.check(f['a'], oldValue=23, newValue=NOT_GIVEN,
                inputText='foo',  ctrlError=NE)
        self.check(f['b'], oldValue='vB', newValue='vB',
                inputText='vB',  ctrlError=None)
        self.check(f['s/sa'], oldValue=42, newValue=NOT_GIVEN,
                inputText='bar',  ctrlError=NE)
        self.check(f['s/sb'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText='',  ctrlError=None)
        self.check(f['s'], oldValue=r.s, newValue=r.s)
        self.check(f, oldValue=r, newValue=r, res=False,
                log=set(['error_a', 'error_s$sa', 'submit'])),
        assertEqualEntityError(self, f.formErrors, [('a',NE),('s.sa',NE)])

    def test26_ValueWithoutSubmit(self):
        f = self.form(FIN, oldValue=Record(b='vB'), post={'b':'foo'})
        self.check(f['b'], oldValue='vB', newValue='vB',
                inputText='foo',  ctrlError=None)
        self.check(f, log=set([]), res=False)

    def test30_Unicode(self):
        f = self.form(FIN, oldValue=Record(b=u'etina'))
        self.check(f['b'], oldValue=u'etina', newValue=u'etina',
                inputText=u'etina',  ctrlError=None)
        f = self.form(FIN, get={'b':u'etina'})
        self.check(f['b'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                inputText=u'etina',  ctrlError=None)








class FST(ToplevelFormController):
    log = binding.Make(set)
    ctrlType = Record

    class a(LoggingInput):
        ctrlType = int

    class b(LoggingInput):
        ctrlType = str

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.log.add('change')

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.log.add('submit')


class TestInstances(FormCase):

    def test10_DefaultValue(self):
        f = self.form(FST)
        self.check(f,
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN, res=False,
            oldFeatures={}, newFeatures={},
            log=set([]),
        )

    def test12_OnAdd(self):
        f = self.form(FST, post={'a':'12','b':'foo','doSubmit':''})
        self.check(f,
            oldValue=NOT_GIVEN, newValue=Record(a=12,b='foo'), res=True,
            oldFeatures={}, newFeatures={'a':12, 'b':'foo'},
            log=set(['change','change_a','change_b','submit']),
        )


    def test14_OnAddWithError(self):
        f = self.form(FST, post={'a':'x12','b':'foo','doSubmit':''})
        self.check(f,
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN, res=False,
            oldFeatures={}, newFeatures={'b':'foo'},
            log=set(['error_a','change_b','submit']),
        )
        assertEqualEntityError(self, f.formErrors, [('a',NE)])

    def test16_ValueWithoutSubmit(self):
        f = self.form(FST, post={'a':'12'})
        self.check(f,
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN, res=False,
            oldFeatures={}, newFeatures={},
            log=set([]),
        )

    def test20_OldValues(self):
        r = Record(a=23, b='vB')
        f = self.form(FST, oldValue=r.copy())
        self.check(f,
            oldValue=r, newValue=r, res=False,
            oldFeatures={}, newFeatures={},
            log=set([]),
        )

    def test21_OnEdit(self):
        r = Record(a=23, b='vB')
        f = self.form(FST, oldValue=r.copy(), post={'a':'12','doSubmit':''})
        self.check(f,
            oldValue=Record(a=12,b='vB'),newValue=Record(a=12,b='vB'), res=True,
            oldFeatures={'a':23}, newFeatures={'a':12},
            log=set(['change','change_a','submit']),
        )

    def test23_OnEditWithoutChange(self):
        r = Record(a=23, b='vB')
        f = self.form(FST, oldValue=r.copy(),
            post={'a':'23','b':'vB','doSubmit':''})
        self.check(f,
            oldValue=r, newValue=r, res=True,
            oldFeatures={}, newFeatures={},
            log=set(['submit']),
        )

    def test24_OnEditWithError(self):
        r = Record(a=23, b='vB')
        f = self.form(FST, oldValue=r.copy(),
            post={'a':'foo','b':'iB','doSubmit':''})
        self.check(f,
            oldValue=r, newValue=r, res=False,
            oldFeatures={'b':'vB'}, newFeatures={'b':'iB'},
            log=set(['error_a', 'change_b', 'submit']),
        )
        assertEqualEntityError(self, f.formErrors, [('a',NE)])







class FSL(ToplevelFormController):
    log = binding.Make(set)
    ctrlType = Record

    class s(SelectCtrl):
        selectOptions = [('a','A',1), ('b','B',2)]
        def defaultInput(self):
            return 1
        @event_handler(OnChangeEvent)
        def on_change(self, evt):
            self.ctrlRoot.log.add('change_s')

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.log.add('change')

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.log.add('submit')


O = Option

class TestSelects(FormCase):

    def test10_DefaultValue(self):
        f = self.form(FSL)
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='a',  ctrlError=None, options=[
            O('','',None), O('a','A',1,True), O('b','B',2)])
        self.check(f, log=set([]), res=False)

    def test12_OnAdd(self):
        f = self.form(FSL, post={'s':'b','doSubmit':''})
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=2,
            inputText='b',  ctrlError=None, options=[
            O('','',None), O('a','A',1), O('b','B',2,True)])
        self.check(f, log=set(['change','change_s','submit']), res=True)

    def test13_OnAddWithError(self):
        f = self.form(FSL, post={'s':'x','doSubmit':''})
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='x',  ctrlError=SE, options=[
            O('','',None), O('a','A',1), O('b','B',2)])
        self.check(f, log=set(['submit']), res=False)

    def test13_OnAddNone(self):
        f = self.form(FSL, post={'s':'','doSubmit':''})
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=None,
            inputText='',  ctrlError=None, options=[
            O('','',None, True), O('a','A',1), O('b','B',2)])
        self.check(f, log=set(['change','change_s','submit']), res=True)

    def test16_ValueWithoutSubmit(self):
        f = self.form(FSL, post={'s':'b'})
        self.check(f['s'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='b',  ctrlError=None, options=[
            O('','',None), O('a','A',1), O('b','B',2,True)])
        self.check(f, log=set([]), res=False)

    def test20_OldValues(self):
        f = self.form(FSL, oldValue=Record(s=1))
        self.check(f['s'], oldValue=1, newValue=1,
            inputText='a',  ctrlError=None, options=[
            O('','',None), O('a','A',1,True), O('b','B',2)])
        self.check(f, log=set([]), res=False)

    def test21_OnEdit(self):
        f = self.form(FSL, oldValue=Record(s=1), post={'s':'b','doSubmit':''})
        self.check(f['s'], oldValue=1, newValue=2,
            inputText='b',  ctrlError=None, options=[
            O('','',None), O('a','A',1), O('b','B',2,True)])
        self.check(f, log=set(['change','change_s','submit']), res=True)

    def test23_OnEditWithoutChange(self):
        f = self.form(FSL, oldValue=Record(s=1), post={'s':'a','doSubmit':''})
        self.check(f['s'], oldValue=1, newValue=1,
            inputText='a',  ctrlError=None, options=[
            O('','',None), O('a','A',1,True), O('b','B',2)])
        self.check(f, log=set(['submit']), res=True)

    def test24_OnEditWithError(self):
        f = self.form(FSL, oldValue=Record(s=1), post={'s':'x','doSubmit':''})
        self.check(f['s'], oldValue=1, newValue=NOT_GIVEN,
            inputText='x',  ctrlError=SE, options=[
            O('','',None), O('a','A',1), O('b','B',2)])
        self.check(f, log=set(['submit']), res=False)

    def test25_OnEditNone(self):
        f = self.form(FSL, oldValue=Record(s=1), post={'s':'','doSubmit':''})
        self.check(f['s'], oldValue=1, newValue=None,
            inputText='',  ctrlError=None, options=[
            O('','',None,True), O('a','A',1), O('b','B',2)])
        self.check(f, log=set(['change','change_s','submit']), res=True)







class Ent(schema.Entity):
    x = schema.Attribute(int)
    y = schema.Attribute(unicode)
    binding.metadata(schema.PrimaryKey('x'))

E42 = Ent(x=42, y='foo')
E23 = Ent(x=23, y='bar')
Ent.cache = {42:E42, 23:E23}

class FRF(ToplevelFormController):
    log = binding.Make(set)
    ctrlType = Record

    class r(EntityReferenceCtrl):
        ctrlType = Ent

        def _workspace(self):
            class WS:
                def get(self, klass, oid):
                    assert klass is Ent
                    return Ent.cache[int(oid)]
            return WS()
        _workspace = binding.Make(_workspace)

        @event_handler(OnChangeEvent)
        def on_change(self, evt):
            self.ctrlRoot.log.add('change_r')

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.log.add('change')

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.log.add('submit')


class TestReferences(FormCase):

    def test10_DefaultValue(self):
        f = self.form(FRF)
        self.check(f['r'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='',  ctrlError=None)
        self.check(f, log=set([]), res=False)

    def test12_OnAdd(self):
        f = self.form(FRF, post={'r':'42','doSubmit':''})
        self.check(f['r'], oldValue=NOT_GIVEN, newValue=E42,
            inputText='42',  ctrlError=None)
        self.check(f, log=set(['change','change_r','submit']), res=True)

    def test13_OnAddWithError(self):
        f = self.form(FRF, post={'r':'x','doSubmit':''})
        self.check(f['r'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='x',  ctrlError=FE)
        self.check(f, log=set(['submit']), res=False)
        assertEqualEntityError(self, f.formErrors, [('r',FE)])

    def test16_ValueWithoutSubmit(self):
        f = self.form(FRF, post={'r':'23'})
        self.check(f['r'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='23',  ctrlError=None)
        self.check(f, log=set([]), res=False)

    def test20_OldValues(self):
        f = self.form(FRF, oldValue=Record(r=E42))
        self.check(f['r'], oldValue=E42, newValue=E42,
            inputText='42',  ctrlError=None)
        self.check(f, log=set([]), res=False)

    def test21_OnEdit(self):
        f = self.form(FRF,oldValue=Record(r=E42),post={'r':'23','doSubmit':''})
        self.check(f['r'], oldValue=E42, newValue=E23,
            inputText='23',  ctrlError=None)
        self.check(f, log=set(['change','change_r','submit']), res=True)

    def test23_OnEditWithoutChange(self):
        f = self.form(FRF,oldValue=Record(r=E42),post={'r':'42','doSubmit':''})
        self.check(f['r'], oldValue=E42, newValue=E42,
            inputText='42',  ctrlError=None)
        self.check(f, log=set(['submit']), res=True)

    def test24_OnEditWithError(self):
        f = self.form(FRF,oldValue=Record(r=E42),post={'r':'x','doSubmit':''})
        self.check(f['r'], oldValue=E42, newValue=NOT_GIVEN,
            inputText='x',  ctrlError=FE)
        self.check(f, log=set(['submit']), res=False)
        assertEqualEntityError(self, f.formErrors, [('r',FE)])








class FCS(ToplevelFormController):

    u = ClientState(default=lambda s,o,a:'foo')
    i = ClientState(int)
    s = ClientState(int, secure=True)
    z = ClientState(secure=True)

    class x(InputCtrl):
        o = ClientState()

    @event_handler(OnClickEvent, 'doSubmit')
    def on_submit(self, evt):
        self.u = 'baz'
        self.i = '3'
        self['x'].o = 4

    class secure(ButtonCtrl): pass

    @event_handler(OnClickEvent, 'secure')
    def on_secure(self, evt):
        self.s = 400
        self.z = u'etina'

class TestClientState(FormCase):

    def test10_DefaultValues(self):
        f = self.form(FCS)
        self.assertEqual(f.u, 'foo')
        self.assertEqual(f.i, NOT_GIVEN)
        self.assertEqual(f.s, NOT_GIVEN)
        self.assertEqual(f['x'].o, NOT_GIVEN)
        self.assertEqual(dict(f.ctrlState()), {'u':u'foo'})

    def test12_ExplicitValues(self):
        f = self.form(FCS, post={'u':'bar', 'i':'4', 'x$o':'old'})
        self.assertEqual(f.u, u'bar')
        self.assertEqual(f.i, 4)
        self.assertEqual(f.s, NOT_GIVEN)
        self.assertEqual(f['x'].o, 'old')
        self.assertEqual(dict(f.ctrlState()),{'u':u'bar','i':'4','x$o':'old'})

    def test14_EditingValues(self):
        f = self.form(FCS, post={'u':'bar', 'i':'4','x$o':'old', 'doSubmit':''})
        self.assertEqual(f.u, u'baz')
        self.assertEqual(f.i, 3)
        self.assertEqual(f.s, NOT_GIVEN)
        self.assertEqual(f['x'].o, '4')
        self.assertEqual(dict(f.ctrlState()),{'u':u'baz','i':'3','x$o':'4'})

    def test16_DeletingValues(self):
        f = self.form(FCS, post={'u':'bar', 'i':'4','x$o':'old'})
        self.assertEqual(f.i, 4)
        del f.i; del f.i
        self.assertEqual(f.i, NOT_GIVEN)
        self.assertEqual(dict(f.ctrlState()), {'u':u'bar','x$o':'old'})

    def test20_EditingSecureValue(self):
        f = self.form(FCS, post={'secure':''})
        self.assertEqual(f.s, 400)
        self.assertEqual(f.z, u'etina')
        ts = CSValue(val='400', secret=web.app_secret())
        tz = CSValue(val=u'etina'.encode('utf-8'), secret=web.app_secret())
        self.assertEqual(dict(f.ctrlState()),{
            's': str(ts), 'z': str(tz), 'u':'foo',
        })

    def test22_SecureValues(self):
        ts = CSValue(val='300', secret=web.app_secret())
        tz = CSValue(val=u'e'.encode('utf-8'), secret=web.app_secret())
        f = self.form(FCS, post={'s':str(ts), 'z':str(tz)})
        self.assertEqual(f.s, 300)
        self.assertEqual(f.z, u'e')
        self.assertEqual(dict(f.ctrlState()),{
            's': str(ts), 'z': str(tz), 'u':'foo',
        })

    def test30_StateValueInConstructor(self):
        req = self.req()
        f = FCS(ctrlReq=req, u='init')
        self.assertEqual(f.u, 'init')





class FIT(InstanceCtrl, SequenceItemCtrl):
    ctrlType = Record

    class k(LoggingInput):
        ctrlType = str

    class v(LoggingInput):
        ctrlType = int

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.ctrlRoot.log.add('change_%s' % self.ctrlId)

class FFR(ToplevelFormController):
    log = binding.Make(set)
    ctrlType = Record

    class i(SequenceCtrl):
        itemClass = binding.Constant(FIT)

        def makeDefaultItems(self):
            i1 = FIT(self.ctrlReq, '0', self)
            i1['k'].inputText = 'a'
            i1['v'].inputText = '1'
            i2 = FIT(self.ctrlReq, '1', self)
            i2['k'].inputText = 'b'
            i2['v'].inputText = '3'
            return [i1, i2]

        @event_handler(OnChangeEvent)
        def on_change(self, evt):
            self.ctrlRoot.log.add('change_i')

        @event_handler(OnAddEvent)
        def on_add(self, evt):
            self.ctrlRoot.log.add('add_%s' % evt.item.ctrlId)

        @event_handler(OnRemoveEvent)
        def on_add(self, evt):
            self.ctrlRoot.log.add('remove_%s' % evt.item.ctrlId)

class FFRI(FFR):
    class i(FFR.i):
        class itemClass(FIT):
            pass


class TestSequence(FormCase):

    def test10_DefaultValue(self):
        f = self.form(FFR)
        self.assertEqual(len(f['i'].ctrlItems), 2)
        self.check(f['i/0/k'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='a',  ctrlError=None)
        self.check(f['i/0/v'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='1',  ctrlError=None)
        self.check(f['i/1/k'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='b',  ctrlError=None)
        self.check(f['i/1/v'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='3',  ctrlError=None)
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'-1', 'i$1$itemState':'-1',
        })

    def test11_ItemClassInlineDefinition(self):
        f = self.form(FFRI)
        self.assertEqual(len(f['i'].ctrlItems), 2)
        self.check(f['i/0/k'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='a',  ctrlError=None)
        self.check(f['i/0/v'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='1',  ctrlError=None)
        self.check(f['i/1/k'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='b',  ctrlError=None)
        self.check(f['i/1/v'], oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
                               inputText='3',  ctrlError=None)
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'-1', 'i$1$itemState':'-1',
        })

    def test12_AddItem(self):
        f = self.form(FFR, post={
            'i$0$k':'a', 'i$0$v':1,
            'i$1$k':'b', 'i$1$v':3,
            'i$itemCount':'2', 'i$0$itemState':'-1', 'i$1$itemState':'-1',
            'i$addItem':'',
            'i$k': 'foo',
        })
        self.assertEqual(len(f['i'].ctrlItems), 3)
        self.assertEqual(f['i/0/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/0/k'].inputText, 'a')
        self.assertEqual(f['i/0/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/0/v'].inputText, '1')
        self.assertEqual(f['i/1/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/1/k'].inputText, 'b')
        self.assertEqual(f['i/1/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/1/v'].inputText, '3')
        self.assertEqual(f['i/2/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/2/k'].inputText, '')
        self.assertEqual(f['i/2/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/2/v'].inputText, '')
        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'3',
            'i$0$itemState':'-1',
            'i$1$itemState':'-1',
            'i$2$itemState':'-1',
        })
        self.assertEqual(f.log, set([]))

    def test14_EditNewItemWithError(self):
        f = self.form(FFR, post={
            'i$0$k':'a', 'i$0$v':1,
            'i$1$k':'b', 'i$1$v':'nonum',
            'i$itemCount':'2', 'i$0$itemState':'-1', 'i$1$itemState':'-1',
            'i$k': 'foo', 'doSubmit':'S',
        })
        self.assertEqual(len(f['i'].ctrlItems), 2)
        self.assertEqual(f['i/0/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/0/k'].inputText, 'a')
        self.assertEqual(f['i/0/k'].newValue, 'a')
        self.assertEqual(f['i/0/k'].ctrlError, None)
        self.assertEqual(f['i/0/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/0/v'].inputText, '1')
        self.assertEqual(f['i/0/v'].newValue, 1)
        self.assertEqual(f['i/0/v'].ctrlError, None)
        self.assertEqual(f['i/1/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/1/k'].inputText, 'b')
        self.assertEqual(f['i/1/k'].newValue, 'b')
        self.assertEqual(f['i/1/k'].ctrlError, None)
        self.assertEqual(f['i/1/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/1/v'].inputText, 'nonum')
        self.failUnless(isinstance(f['i/1/v'].ctrlError, NE))
        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'-1', 'i$1$itemState':'-1',
        })
        self.assertEqual(f.log, set([
            'change_i$0', 'change_i$0$k', 'change_i$0$v',
            'change_i$1$k', 'error_i$1$v',
        ]))
        assertEqualEntityError(self, f.formErrors, [('i.1.v',NE)])

    def test20_OldValues(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'a')
        self.assertEqual(f['i/0/k'].inputText, 'a')
        self.assertEqual(f['i/0/k'].newValue, 'a')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 1)
        self.assertEqual(f['i/0/v'].inputText, '1')
        self.assertEqual(f['i/0/v'].newValue, 1)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(f['i/1/k'].oldValue, 'b')
        self.assertEqual(f['i/1/k'].inputText, 'b')
        self.assertEqual(f['i/1/k'].newValue, 'b')
        self.assertEqual(f['i/1/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 2)
        self.assertEqual(f['i/1/v'].inputText, '2')
        self.assertEqual(f['i/1/v'].newValue, 2)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
        })
        self.assertEqual(f.log, set([]))

    def test22_EditOldItem(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':1,
                'i$1$k':'b', 'i$1$v':3,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$k': 'foo', 'doSubmit':'S',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'a')
        self.assertEqual(f['i/0/k'].inputText, 'A')
        self.assertEqual(f['i/0/k'].newValue, 'A')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 2)
        self.assertEqual(f['i/1/v'].inputText, '3')
        self.assertEqual(f['i/1/v'].newValue, 3)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
        })
        self.assertEqual(f.log, set([
            'change_i$0', 'change_i$0$k',
            'change_i$1', 'change_i$1$v',
        ]))

    def test24_EditOldItemWithError(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':1,
                'i$1$k':'b', 'i$1$v':'nonum',
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'doSubmit':'S',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'a')
        self.assertEqual(f['i/0/k'].inputText, 'A')
        self.assertEqual(f['i/0/k'].newValue, 'A')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 2)
        self.assertEqual(f['i/1/v'].inputText, 'nonum')
        self.assertEqual(f['i/1/v'].newValue, NOT_GIVEN)
        self.failUnless(isinstance(f['i/1/v'].ctrlError, NE))

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
        })
        self.assertEqual(f.log, set([
            'change_i$0', 'change_i$0$k',
            'error_i$1$v'
        ]))
        assertEqualEntityError(self, f.formErrors, [('i.1.v',NE)])

    def test26_AddItemToExistingSequence(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':1,
                'i$1$k':'b', 'i$1$v':3,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$addItem':'',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 3)

        self.assertEqual(f['i/0/k'].oldValue, 'a')
        self.assertEqual(f['i/0/k'].inputText, 'A')
        self.assertEqual(f['i/0/k'].newValue, 'a')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 2)
        self.assertEqual(f['i/1/v'].inputText, '3')
        self.assertEqual(f['i/1/v'].newValue, 2)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(f['i/2/k'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/2/k'].inputText, '')
        self.assertEqual(f['i/2/k'].newValue, NOT_GIVEN)
        self.assertEqual(f['i/2/k'].ctrlError, None)

        self.assertEqual(f['i/2/v'].oldValue, NOT_GIVEN)
        self.assertEqual(f['i/2/v'].inputText, '')
        self.assertEqual(f['i/2/v'].newValue, NOT_GIVEN)
        self.assertEqual(f['i/2/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'3', 'i$0$itemState':'0', 'i$1$itemState':'1',
            'i$2$itemState':'-1',
        })
        self.assertEqual(f.log, set([]))

    def test28_RemoveItemFromExistingSequence(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':1,
                'i$1$k':'b', 'i$1$v':3,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$0$removeItem':'',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 1)

        self.assertEqual(f['i/0/k'].oldValue, 'b')
        self.assertEqual(f['i/0/k'].inputText, 'b')
        self.assertEqual(f['i/0/k'].newValue, 'b')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 2)
        self.assertEqual(f['i/0/v'].inputText, '3')
        self.assertEqual(f['i/0/v'].newValue, 2)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'1', 'i$0$itemState':'1',
        })
        self.assertEqual(f.log, set(['remove_i$0']))

    def test30_SwapExistingSequence(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':42,
                'i$1$k':'B', 'i$1$v':43,
                'i$itemCount':'2', 'i$0$itemState':'1', 'i$1$itemState':'0',
                'doSubmit':'S',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'b')
        self.assertEqual(f['i/0/k'].inputText, 'A')
        self.assertEqual(f['i/0/k'].newValue, 'A')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 2)
        self.assertEqual(f['i/0/v'].inputText, '42')
        self.assertEqual(f['i/0/v'].newValue, 42)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(f['i/1/k'].oldValue, 'a')
        self.assertEqual(f['i/1/k'].inputText, 'B')
        self.assertEqual(f['i/1/k'].newValue, 'B')
        self.assertEqual(f['i/1/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 1)
        self.assertEqual(f['i/1/v'].inputText, '43')
        self.assertEqual(f['i/1/v'].newValue, 43)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'1', 'i$1$itemState':'0',
        })
        self.assertEqual(f.log, set([
            'change_i$0', 'change_i$0$k', 'change_i$0$v',
            'change_i$1', 'change_i$1$k', 'change_i$1$v',
            'change_i',
        ]))

    def test32_MoveItemUp(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':42,
                'i$1$k':'B', 'i$1$v':43,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$1$moveUp':'U',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'b')
        self.assertEqual(f['i/0/k'].inputText, 'B')
        self.assertEqual(f['i/0/k'].newValue, 'b')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 2)
        self.assertEqual(f['i/0/v'].inputText, '43')
        self.assertEqual(f['i/0/v'].newValue, 2)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(f['i/1/k'].oldValue, 'a')
        self.assertEqual(f['i/1/k'].inputText, 'A')
        self.assertEqual(f['i/1/k'].newValue, 'a')
        self.assertEqual(f['i/1/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 1)
        self.assertEqual(f['i/1/v'].inputText, '42')
        self.assertEqual(f['i/1/v'].newValue, 1)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'1', 'i$1$itemState':'0',
        })
        self.assertEqual(f.log, set([]))

    def test33_MoveItemUpWithouDataChange(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'a', 'i$0$v':1,
                'i$1$k':'b', 'i$1$v':2,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$1$moveUp':'U',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'b')
        self.assertEqual(f['i/0/k'].inputText, 'b')
        self.assertEqual(f['i/0/k'].newValue, 'b')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 2)
        self.assertEqual(f['i/0/v'].inputText, '2')
        self.assertEqual(f['i/0/v'].newValue, 2)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(f['i/1/k'].oldValue, 'a')
        self.assertEqual(f['i/1/k'].inputText, 'a')
        self.assertEqual(f['i/1/k'].newValue, 'a')
        self.assertEqual(f['i/1/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 1)
        self.assertEqual(f['i/1/v'].inputText, '1')
        self.assertEqual(f['i/1/v'].newValue, 1)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'1', 'i$1$itemState':'0',
        })
        self.assertEqual(f.log, set([]))

    def test34_MoveItemDown(self):
        f = self.form(FFR,
            oldValue=Record(i=[Record(k='a',v=1), Record(k='b',v=2)]),
            post={
                'i$0$k':'A', 'i$0$v':42,
                'i$1$k':'B', 'i$1$v':43,
                'i$itemCount':'2', 'i$0$itemState':'0', 'i$1$itemState':'1',
                'i$0$moveDown':'D',
            },
        )
        self.assertEqual(len(f['i'].ctrlItems), 2)

        self.assertEqual(f['i/0/k'].oldValue, 'b')
        self.assertEqual(f['i/0/k'].inputText, 'B')
        self.assertEqual(f['i/0/k'].newValue, 'b')
        self.assertEqual(f['i/0/k'].ctrlError, None)

        self.assertEqual(f['i/0/v'].oldValue, 2)
        self.assertEqual(f['i/0/v'].inputText, '43')
        self.assertEqual(f['i/0/v'].newValue, 2)
        self.assertEqual(f['i/0/v'].ctrlError, None)

        self.assertEqual(f['i/1/k'].oldValue, 'a')
        self.assertEqual(f['i/1/k'].inputText, 'A')
        self.assertEqual(f['i/1/k'].newValue, 'a')
        self.assertEqual(f['i/1/k'].ctrlError, None)

        self.assertEqual(f['i/1/v'].oldValue, 1)
        self.assertEqual(f['i/1/v'].inputText, '42')
        self.assertEqual(f['i/1/v'].newValue, 1)
        self.assertEqual(f['i/1/v'].ctrlError, None)

        self.assertEqual(dict(f.ctrlState()), {
            'i$itemCount':'2', 'i$0$itemState':'1', 'i$1$itemState':'0',
        })
        self.assertEqual(f.log, set([]))




class FileRec(Record):
    protocols.advise(instancesProvide=[IFileInfo])

class FUP(ToplevelFormController):
    ctrlType = Record
    log = binding.Make(set)

    class u(FileUploadCtrl):
        ctrlType = FileRec
        emptyIsNone = True

        @event_handler(OnChangeEvent)
        def on_change(self, evt):
            self.ctrlRoot.log.add('change_u')

    @event_handler(OnChangeEvent)
    def on_change(self, evt):
        self.ctrlRoot.log.add('change')



US = TestUpload(join(RES_DIR, 'small.txt'))
UZ = TestUpload(join(RES_DIR, 'small.txt.gz'))
UJ = TestUpload(join(RES_DIR, 'i.jpg'))

class TestUploads(FormCase):

    def test10_DefaultValue(self):
        f = self.form(FUP)
        self.check(f['u'],
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'0'})

    def test12_Add(self):
        f = self.form(FUP, post={'u':US, 'doSubmit':''})
        self.check(f['u'],
            oldValue=NOT_GIVEN, newValue=FileRec(
                filename='small.txt', path=Any,
                content_type='text/plain', size=10),
            inputText='', ctrlError=None,
            filename='small.txt', path=Any, content_type='text/plain', size=10,
        )
        self.check(f, log=set(['change', 'change_u']), res=True)
        self.assertEqual(dict(f.ctrlState()), {
            'u$deleted':'0', 'u$filename':'small.txt', 'u$path':Any,
            'u$content_type':'text/plain', 'u$size':'10',
        })

    def test14_AddInvisible(self):
        f = self.form(FUP, post={'doSubmit':''})
        self.check(f['u'],
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f, log=set([]), res=True)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'0'}) # XXX empty

    def test16_AddNoValue(self):
        f = self.form(FUP, post={'u':'', 'doSubmit':''})
        self.check(f['u'],
            oldValue=NOT_GIVEN, newValue=None,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f, log=set(['change', 'change_u']), res=True)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'0'})

    def test18_DeleteAdded(self):
        p = CSValue(
            val = join(RES_DIR, 'small.txt'),
            secret = web.app_secret())
        f = self.form(FUP, post={
            'u':'', 'u$deleteCurrent':'', 'u$path':str(p),
            'u$deleted':'0', 'u$filename':'small.txt',
            'u$content_type':'text/plain', 'u$size':'10',
        })
        self.check(f['u'],
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'1'})

    def test20_EditForm(self):
        o = FileRec(
            filename='small.txt', content_type='text/plain', size=10,
            path=join(RES_DIR, 'small.txt')
        )
        f = self.form(FUP, oldValue=Record(u=o))
        self.check(f['u'],
            oldValue=o, newValue=o,
            inputText='', ctrlError=None,
            filename='small.txt', path=NOT_GIVEN,
            content_type='text/plain', size=10,
        )
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {
            'u$deleted':'0', 'u$filename':'small.txt',
            'u$content_type':'text/plain', 'u$size':'10',
        })

    def test22_Edit(self):
        o = FileRec(
            filename='small.txt', content_type='text/plain', size=10,
            path=join(RES_DIR, 'small.txt')
        )
        f = self.form(FUP, oldValue=Record(u=o), post={'u':UJ, 'doSubmit':''})
        self.check(f['u'],
            oldValue=o, newValue=FileRec(
                filename='i.jpg',path=Any,content_type='image/jpeg',size=14590),
            inputText='', ctrlError=None,
            filename='i.jpg', path=Any,
            content_type='image/jpeg', size=14590,
        )
        self.check(f, log=set(['change', 'change_u']), res=True)
        self.assertEqual(dict(f.ctrlState()), {
            'u$deleted':'0', 'u$filename':'i.jpg', 'u$path':Any,
            'u$content_type':'image/jpeg', 'u$size':'14590',
        })

    def test24_EditWithNoValue(self):
        o = FileRec(
            filename='small.txt', content_type='text/plain', size=10,
            path=join(RES_DIR, 'small.txt')
        )
        f = self.form(FUP, oldValue=Record(u=o), post={
            'u':'', 'doSubmit':'',
            'u$deleted':'0', 'u$filename':'small.txt',
            'u$content_type':'text/plain', 'u$size':'10',
        })
        self.check(f['u'],
            oldValue=o, newValue=o,
            inputText='', ctrlError=None,
            filename='small.txt', path=NOT_GIVEN,
            content_type='text/plain', size=10,
        )
        self.check(f, log=set([]), res=True)
        self.assertEqual(dict(f.ctrlState()), {
            'u$deleted':'0', 'u$filename':'small.txt',
            'u$content_type':'text/plain', 'u$size':'10',
        })

    def test26_EditInvisible(self):
        o = FileRec(
            filename='small.txt', content_type='text/plain', size=10,
            path=join(RES_DIR, 'small.txt')
        )
        f = self.form(FUP, oldValue=Record(u=o), post={'doSubmit':''})
        self.check(f['u'],
            oldValue=o, newValue=o,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f, log=set([]), res=True)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'0'}) # XXX empty

    def test28_DeleteExisting(self):
        o = FileRec(
            filename='small.txt', content_type='text/plain', size=10,
            path=join(RES_DIR, 'small.txt')
        )
        f = self.form(FUP, oldValue=Record(u=o), post={
            'u':'', 'u$deleteCurrent':'',
            'u$deleted':'0', 'u$filename':'small.txt',
            'u$content_type':'text/plain', 'u$size':'10',
        })
        self.check(f['u'],
            oldValue=o, newValue=o,
            inputText='', ctrlError=None,
            filename=NOT_GIVEN, path=NOT_GIVEN,
            content_type=NOT_GIVEN, size=NOT_GIVEN,
        )
        self.check(f['u']['deleteCurrent'],
            oldValue=NOT_GIVEN, newValue=NOT_GIVEN)
        self.check(f, log=set([]), res=False)
        self.assertEqual(dict(f.ctrlState()), {'u$deleted':'1'})






def suite():
    return TestSuite((
        makeSuite(TestControllers),
        makeSuite(TestButtons),
        makeSuite(TestInputs),
        makeSuite(TestInstances),
        makeSuite(TestSelects),
        makeSuite(TestReferences),
        makeSuite(TestClientState),
        makeSuite(TestSequence),
        makeSuite(TestUploads),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
