# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *

from utils import *

class TestHtmlInput(WebCase):

    def test_NoAutofill(self):
        html = self.render('''<input h:input="" size="20" />''')
        self.assertEqualXML(html, '<input size="20" />')
        html = self.render('''<input h:input="" size="20" value="baz" />''')
        self.assertEqualXML(html, '<input size="20" value="baz" />')

    def test_AutofillWithoutValue(self):
        html = self.render('''<input h:input="'foo'" size="20" />''')
        self.assertEqualXML(html, '<input size="20" name="foo" />')
        html = self.render('''<input h:input="'foo'" name="foo" size="20" />''')
        self.assertEqualXML(html, '<input name="foo" size="20" />')
        html = self.render('''<input h:input="'foo'" value="bar" />''')
        self.assertEqualXML(html, '<input value="bar" name="foo" />')

    def test_AutofillWithEmptyValue(self):
        req = self.req(get={'foo':''})
        html = self.render('''<input h:input="'foo'" value="bar" />''', req=req)
        self.assertEqualXML(html, '<input value="" name="foo" />')

    def test_AutofillWithValue(self):
        req = self.req(get={'foo':'bar'})
        html = self.render('''<input h:input="'foo'" />''', req=req)
        self.assertEqualXML(html, '<input name="foo" value="bar" />')
        html = self.render('''<input value="baz" h:input="'foo'" />''', req=req)
        self.assertEqualXML(html, '<input value="bar" name="foo" />')

    def test_AutofillWithDifferentName(self):
        req = self.req(get={'foo':'bar'})
        html = self.render('''<input h:input="'foo'" name="hello"/>''', req=req)
        self.assertEqualXML(html, '<input name="hello" value="bar" />')

    def test_ValueEscaping(self):
        req = self.req(get={'foo':'<bar>'})
        html = self.render('''<input h:input="'foo'" />''', req=req)
        self.assertEqualXML(html, '<input name="foo" value="&lt;bar&gt;" />')

    def test_WithExplicitValue(self):
        req = self.req(get={'foo':'bar'})
        html = self.render(
            '''<input h:input="'foo', value=v" />''',
            req=req, v='baz'
        )
        self.assertEqualXML(html, '<input name="foo" value="baz" />')

    def test_WithExplicitFormData(self):
        req = self.req(get={'foo':'bar'})
        html = self.render(
            '''<form h:form="data=obj"><input h:input="'foo'" /></form>''',
            req=req, obj={'foo':'baz'}
        )
        self.assertEqualXML(
                html, '<form><input name="foo" value="baz" /></form>')



class TestHtmlSelect(WebCase):

    def test_WithoutValue(self):
        html = self.render(
            '''<select h:select="'foo'">'''
                '''<option>A</option>'''
                '''<option>B</option>'''
                '''<option>C</option>'''
            '''</select>'''
        )
        self.assertEqualXML(html,
            '''<select name="foo">'''
                '''<option>A</option>'''
                '''<option>B</option>'''
                '''<option>C</option>'''
            '''</select>'''
        )

    def test_WithValue(self):
        html = self.render(
            '''<select h:select="'foo'">'''
                '''<option>A</option>'''
                '''<option>B</option>'''
                '''<option selected="selected">C</option>'''
            '''</select>''',
            req=self.req(get={'foo':'B'})
        )
        self.assertEqualXML(html,
            '''<select name="foo">'''
                '''<option>A</option>'''
                '''<option selected="selected">B</option>'''
                '''<option>C</option>'''
            '''</select>'''
        )

    def test_WithValueAsAttr(self):
        html = self.render(
            '''<select h:select="'foo'">'''
                '''<option value="a">A</option>'''
                '''<option value="b">B</option>'''
                '''<option value="c" selected="selected">C</option>'''
            '''</select>''',
            req=self.req(get={'foo':'b'})
        )
        self.assertEqualXML(html,
           u'''<select name="foo">'''
                '''<option value="a">A</option>'''
                '''<option value="b" selected="selected">B</option>'''
                '''<option value="c">C</option>'''
            '''</select>'''
        )

    def test_WithEmptyValueAsAttr(self):
        html = self.render(
            '''<select h:select="'foo'">'''
                '''<option value="">default</option>'''
                '''<option value="a">A</option>'''
                '''<option value="b">B</option>'''
                '''<option value="c" selected="selected">C</option>'''
            '''</select>''',
            req=self.req(get={'foo':''})
        )
        self.assertEqualXML(html,
           u'''<select name="foo">'''
                '''<option value="" selected="selected">default</option>'''
                '''<option value="a">A</option>'''
                '''<option value="b">B</option>'''
                '''<option value="c">C</option>'''
            '''</select>'''
        )
    def test_WithDynamicValues(self):
        html = self.render(
            '''<select h:select="'foo'">'''
                '''<option value="a">$txta</option>'''
                '''<option value="$varb">B</option>'''
                '''<option value="c">C</option>'''
                '''<option value="d" selected="selected"></option>'''
            '''</select>''',
            req=self.req(get={'foo':'b'}),
            varb='b', txta='A',
        )
        self.assertEqualXML(html,
           u'''<select name="foo">'''
                '''<option value="a">A</option>'''
                '''<option value="b" selected="selected">B</option>'''
                '''<option value="c">C</option>'''
                '''<option value="d" />'''
            '''</select>'''
        )

    def test_WithGeneratedOptions(self):
        html = self.render(
            '''<select h:select="'foo'">'''
            '''<option py:for="v in values" value="$v">val(${v.upper()})'''
            '''</option>'''
            '''</select>''',
            req=self.req(get={'foo':'b'}),
            values='abc',
        )
        self.assertEqualXML(html,
           u'''<select name="foo">'''
                '''<option value="a">val(A)</option>'''
                '''<option value="b" selected="selected">val(B)</option>'''
                '''<option value="c">val(C)</option>'''
            '''</select>'''
        )

    def test_WithOptionsAsArgument(self):
        html = self.render(
            '''<select h:select="'foo', options=options">'''
                '''<p>contents of select should be ignored</p>'''
            '''</select>''',
            req=self.req(get={'foo':'b'}),
            options=[('a', 'A'), ('b', 'B'), 'c'],
        )
        self.assertEqualXML(html,
           u'''<select name="foo">'''
                '''<option value="a">A</option>'''
                '''<option selected="selected" value="b">B</option>'''
                '''<option value="c">c</option>'''
            '''</select>'''
        )

    def test_WithExplicitValue(self):
        html = self.render(
            '''<form h:form="data=obj">'''
                '''<select h:select="'foo', value=v, options=options">'''
                    '''<p>contents of select should be ignored</p>'''
                '''</select>'''
            '''</form>''',
            req = self.req(get={'foo':'b'}),
            obj = {'foo': 'a'},
            v = 'c',
            options = [('a', 'A'), ('b', 'B'), 'c'],
        )
        self.assertEqualXML(html,
            '''<form><select name="foo">'''
                '''<option value="a">A</option>'''
                '''<option value="b">B</option>'''
                '''<option selected="selected" value="c">c</option>'''
            '''</select></form>'''
        )

    def test_WithExplicitFormData(self):
        html = self.render(
            '''<form h:form="data=obj">'''
                '''<select h:select="'foo', options=options">'''
                    '''<p>contents of select should be ignored</p>'''
                '''</select>'''
            '''</form>''',
            req = self.req(get={'foo':'b'}),
            obj = {'foo': 'a'},
            options = [('a', 'A'), ('b', 'B'), 'c'],
        )
        self.assertEqualXML(html,
            '''<form><select name="foo">'''
                '''<option selected="selected" value="a">A</option>'''
                '''<option value="b">B</option>'''
                '''<option value="c">c</option>'''
            '''</select></form>'''
        )

def suite():
    return TestSuite((
        makeSuite(TestHtmlInput),
        makeSuite(TestHtmlSelect),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
