# -*- coding: ISO-8859-2 -*-

from wm4.testing import *
from wm4.api import *
from wm4.web.forms.api import *
from wm4.web.forms.listings import *
from wm4.query.tests.app import *

from utils import *

class TestPageRange(WebCase):

    def test01_NoPages(self):
        self.assertEqual(pageRange(0), [])

    def test02_OnePage(self):
        self.assertEqual(pageRange(1), [0])

    def test10_UnderLimit(self):
        self.assertEqual(pageRange(4), [0,1,2,3])
        self.assertEqual(pageRange(4,currentPage=1), [0,1,2,3])
        self.assertEqual(pageRange(4,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(4,currentPage=1,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(4,currentPage=2,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(4,currentPage=3,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(10), [0,1,2,3,4,5,6,7,8,9])
        self.assertEqual(pageRange(10,currentPage=9), [0,1,2,3,4,5,6,7,8,9])

    def test20_OverLimit_Start(self):
        self.assertEqual(pageRange(42,currentPage=0,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(42,currentPage=2,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(5,currentPage=0,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(5,currentPage=1,pageLimit=4), [0,1,2,3])
        self.assertEqual(pageRange(5,currentPage=2,pageLimit=4), [0,1,2,3])

    def test21_OverLimit_Middle(self):
        self.assertEqual(pageRange(42,currentPage=3,pageLimit=4), [1,2,3,4])
        self.assertEqual(pageRange(42,currentPage=4,pageLimit=4), [2,3,4,5])
        self.assertEqual(pageRange(42,currentPage=8,pageLimit=4), [6,7,8,9])
        self.assertEqual(pageRange(42,currentPage=37,pageLimit=4),[35,36,37,38])
        self.assertEqual(pageRange(42,currentPage=38,pageLimit=4),[36,37,38,39])
        self.assertEqual(pageRange(42,currentPage=39,pageLimit=4),[37,38,39,40])

    def test22_OverLimit_End(self):
        self.assertEqual(pageRange(42,currentPage=40,pageLimit=4),[38,39,40,41])
        self.assertEqual(pageRange(42,currentPage=41,pageLimit=4),[38,39,40,41])
        self.assertEqual(pageRange(5,currentPage=3,pageLimit=4), [1,2,3,4])
        self.assertEqual(pageRange(5,currentPage=4,pageLimit=4), [1,2,3,4])
        self.assertEqual(pageRange(3,currentPage=2,pageLimit=4), [0,1,2])
        self.assertEqual(pageRange(3,currentPage=6,pageLimit=4), [0,1,2])



class TestPager(WebCase):

    def preq(self, page):
        return self.req(get=dict(page=page))

    def lst(self, ctrlReq=None, **kw):
        if ctrlReq is None:
            ctrlReq = self.req()
        lst = ListingCtrl(ctrlReq, **kw)
        lst()
        return lst

    def test10_PageCount(self):
        lst = self.lst(itemCount=0)
        pc = lst.pager
        self.assertEqual(lst.pageCount, 0)
        pc = self.lst(itemCount=1, itemLimit=10).pager
        self.assertEqual(pc.pageCount, 1)
        pc = self.lst(itemCount=9, itemLimit=10).pager
        self.assertEqual(pc.pageCount, 1)
        pc = self.lst(itemCount=10, itemLimit=10).pager
        self.assertEqual(pc.pageCount, 1)
        pc = self.lst(itemCount=11, itemLimit=10).pager
        self.assertEqual(pc.pageCount, 2)
        pc = self.lst(itemCount=90, itemLimit=10).pager
        self.assertEqual(pc.pageCount, 9)
        lst = self.lst(itemCount=99, itemLimit=10)
        pc = lst.pager
        self.assertEqual(pc.pageCount, 10)

    def test11_Unlimited(self):
        pc = self.lst(itemCount=10, itemLimit=0).pager
        self.assertEqual(pc.pageCount, 0)

    def test16_LimitInRequest(self):
        pc = self.lst(
            ctrlReq=self.req(get=dict(limit=21)), itemCount=30
        ).pager
        self.assertEqual(pc.itemLimit, 21)
        self.assertEqual(pc.itemCount, 30)
        self.assertEqual(pc.pageCount, 2)

    def test17_LimitInParams(self):
        rq = self.req()
        rq.params['limit'] = '21'
        pc = self.lst(ctrlReq=rq, itemCount=30).pager
        self.assertEqual(pc.itemLimit, 21)
        self.assertEqual(pc.itemCount, 30)
        self.assertEqual(pc.pageCount, 2)

    def test20_CurrentPage(self):
        pc = self.lst(itemCount=20, itemLimit=10,ctrlReq=self.preq(-1)).pager
        self.assertEqual(pc._current, 0)
        pc = self.lst(itemCount=20, itemLimit=10,ctrlReq=self.preq(0)).pager
        self.assertEqual(pc._current, 0)
        pc = self.lst(itemCount=25, itemLimit=10,ctrlReq=self.preq(1)).pager
        self.assertEqual(pc._current, 1)
        pc = self.lst(itemCount=25, itemLimit=10,ctrlReq=self.preq(2)).pager
        self.assertEqual(pc._current, 2)
        pc = self.lst(itemCount=25, itemLimit=10,ctrlReq=self.preq(3)).pager
        self.assertEqual(pc._current, 2)
        pc = self.lst(itemCount=25, itemLimit=10,ctrlReq=self.preq(4)).pager
        self.assertEqual(pc._current, 2)

    def test21_CurrentPageWhileNoLimit(self):
        pc = self.lst(itemCount=1000, itemLimit=0).pager
        self.assertEqual(pc._current, None)

    def test21_CurrentPageWhileNoItems(self):
        pc = self.lst(itemCount=0, itemLimit=20).pager
        self.assertEqual(pc._current, None)

    def test30_PageURL(self):
        pc = self.lst(ctrlReq=self.req('http://foo.cz/aaa')).pager
        self.assertEqual(pc.pageURL(0), 'http://foo.cz/aaa?page=0')
        self.assertEqual(pc.pageURL(2), 'http://foo.cz/aaa?page=2')
        pc = self.lst(ctrlReq=self.req('http://foo.cz/aaa?page=1')).pager
        self.assertEqual(pc.pageURL(0), 'http://foo.cz/aaa?page=0')
        self.assertEqual(pc.pageURL(2), 'http://foo.cz/aaa?page=2')

    def test40_PageItem(self):
        pc = self.lst(
            ctrlReq=self.req('http://foo.cz/aaa'), itemCount=22, itemLimit=10
        ).pager
        p0 = pc.page(0)
        self.assertEqual(p0.page, 0)
        self.assertEqual(p0.first, 0)
        self.assertEqual(p0.last, 9)
        self.assertEqual(p0.url, 'http://foo.cz/aaa?page=0')
        self.assertEqual(p0.selected, True)
        p1 = pc.page(1)
        self.assertEqual(p1.page, 1)
        self.assertEqual(p1.first, 10)
        self.assertEqual(p1.last, 19)
        self.assertEqual(p1.url, 'http://foo.cz/aaa?page=1')
        self.assertEqual(p1.selected, False)
        p2 = pc.page(2)
        self.assertEqual(p2.page, 2)
        self.assertEqual(p2.first, 20)
        self.assertEqual(p2.last, 21)
        self.assertEqual(p2.url, 'http://foo.cz/aaa?page=2')
        self.assertEqual(p2.selected, False)

    def test41_PredefinedPagesWhenNoPages(self):
        pc = self.lst(
            ctrlReq=self.req('http://foo.cz/aaa'), itemCount=22, itemLimit=0
        ).pager
        self.assertEqual(pc.first, None)
        self.assertEqual(pc.prev, None)
        self.assertEqual(pc.current, None)
        self.assertEqual(pc.next, None)
        self.assertEqual(pc.last, None)

    def test42_PredefinedPagesWhenOnePage(self):
        ls = self.lst(
            ctrlReq=self.req('http://foo.cz/aaa'), itemCount=8, itemLimit=10
        )
        pc = ls.pager
        self.assertEqual(pc.first.page, 0)
        self.assertEqual(pc.prev.page, 0)
        self.assertEqual(pc.current.page, 0)
        self.assertEqual(pc.next.page, 0)
        self.assertEqual(pc.last.page, 0)

    def test43_PredefinedPagesWhenManyPages(self):
        ls = self.lst(
            ctrlReq=self.req('/?page=3'), itemCount=100, itemLimit=10
        )
        pc = ls.pager
        self.assertEqual(pc.first.page, 0)
        self.assertEqual(pc.prev.page, 2)
        self.assertEqual(pc.current.page, 3)
        self.assertEqual(pc.next.page, 4)
        self.assertEqual(pc.last.page, 9)






class LST(ListingCtrl):

    baseQuery = query.query('[u for u in User if u.uid > 1]')

    class uid(Column):
        queryExpr = 'u.uid'
    class name(Column):
        queryExpr = 'u.name'
    class login(InputFilter):
        queryExpr = 'u.login'


class TestListing(WebCase):

    def setUp(self):
        WebCase.setUp(self)
        settings.storage_url('mockdb:')
        self.con = settings.storage_connection()

    def expects(self, **r):
        self.con.expects(**r)

    def lst(self, ctrlReq=None, **kw):
        if ctrlReq is None:
            ctrlReq = self.req()
        lst = LST(ctrlReq, **kw)
        lst()
        return lst

    def test01_ParamItemLimit(self):
        # default
        lst = self.lst(self.req())
        self.assertEqual(lst.itemLimit, 20)
        # in params
        req = self.req()
        req.params['limit'] = 30
        lst = self.lst(req)
        self.assertEqual(lst.itemLimit, 30)
        # in request
        lst = self.lst(self.req(get={'limit':'30'}))
        self.assertEqual(lst.itemLimit, 30)

    def test02_ParamPageCount(self):
        # default
        lst = self.lst(self.req(), itemCount=300)
        self.assertEqual(lst.pager.pageCount, 15)
        # explicit limit
        lst = self.lst(self.req(get={'limit':'100'}), itemCount=300)
        self.assertEqual(lst.pager.pageCount, 3)
        # unlimited listing
        lst = self.lst(self.req(get={'limit':'0'}), itemCount=300)
        self.assertEqual(lst.pager.pageCount, 0)

    def test03_ParamPage(self):
        # default
        lst = self.lst(self.req())
        self.assertEqual(lst['page'].newValue, 1)
        # in request
        lst = self.lst(self.req(get={'page':'30'}))
        self.assertEqual(lst['page'].newValue, 30)

    def test04_ParamPageRange(self):
        lst = self.lst(
            self.req(get={'page':'3','limit':'10'}),
            itemCount=42,pageLimit=4,
        )
        self.assertEqual(lst['page'].newValue, 3)


    def test10_BaseQueryParsing(self):
        lst = self.lst(self.req(), itemCount=63)
        pq = lst.parsedQuery
        self.assertEqual(
            pq, query.Select(u, [('u',u,None,None)], where=query.GT(u_uid,C(1)))
        )
        self.assertEqual(lst.itemQuery, query.Select(
            u, [('u',u,None,None)],
            where=query.GT(u_uid,C(1)),
            limit=query.Const(20),
            offset=query.Const(0)
        ))
        self.assertEqual(lst.countQuery, query.Select(
            query.Count(), [('u',u,None,None)],
            where=query.GT(u_uid,C(1)),
        ))

    def test20_DefaultOrderingDefault(self):
        lst = self.lst(self.req(), itemCount=63)
        self.assertEqual(lst.orderBy, None)
        self.assertEqual(lst.itemQuery, query.Select(
            u, [('u',u,None,None)],
            where=query.GT(u_uid,C(1)),
            limit=query.Const(20),
            offset=query.Const(0)
        ))

    def test21_AscendingOrdering(self):
        lst = self.lst(self.req(get={'order':'+uid'}), itemCount=63)
        self.assertEqual(lst.orderBy, query.OrderBy(u_uid))
        self.assertEqual(lst.itemQuery, query.Select(
            u, [('u',u,None,None)],
            where=query.GT(u_uid,C(1)),
            orderBy=query.OrderBy(u_uid),
            limit=query.Const(20),
            offset=query.Const(0)
        ))

    def test22_DescendingOrdering(self):
        lst = self.lst(self.req(get={'order':'-uid','page':'3'}),itemCount=63)
        self.assertEqual(lst.orderBy, query.OrderBy(query.Desc(u_uid)))
        self.assertEqual(lst.itemQuery, query.Select(
            u, [('u',u,None,None)],
            where=query.GT(u_uid,C(1)),
            orderBy=query.OrderBy(query.Desc(u_uid)),
            limit=query.Const(20),
            offset=query.Const(40)
        ))

    def test23_InvalidDirectionOrdering(self):
        lst = self.lst(self.req(get={'order':'=uid'}))
        self.assertRaises(web.BadRequest, getattr, lst, 'orderBy')

    def test30_Filtering(self):
        pass


    def test40_Pager(self):
        ls = self.lst(self.req(get={}),itemCount=63)
        pg = ls.pager
        self.assertEqual(pg.pageCount, 4)
        self.assertEqual(pg.first.page, 0)
        self.assertEqual(pg.first.selected, True)
        self.assertEqual(pg.first.url, 'http://localhost/?page=1')
        self.assertEqual(pg.prev.page, 0)
        self.assertEqual(pg.prev.selected, True)
        self.assertEqual(pg.prev.url, 'http://localhost/?page=1')
        self.assertEqual(pg.current.page, 0)
        self.assertEqual(pg.current.selected, True)
        self.assertEqual(pg.current.url, 'http://localhost/?page=1')
        self.assertEqual(pg.next.page, 1)
        self.assertEqual(pg.next.selected, False)
        self.assertEqual(pg.next.url, 'http://localhost/?page=2')
        self.assertEqual(pg.last.page, 3)
        self.assertEqual(pg.last.selected, False)
        self.assertEqual(pg.last.url, 'http://localhost/?page=4')

    def test41_Pager(self):
        ls = self.lst(self.req(get={'page':2}),itemCount=63)
        pg = ls.pager
        self.assertEqual(pg.pageCount, 4)
        self.assertEqual(pg.first.page, 0)
        self.assertEqual(pg.first.selected, False)
        self.assertEqual(pg.first.url, 'http://localhost/?page=1')
        self.assertEqual(pg.prev.page, 0)
        self.assertEqual(pg.prev.selected, False)
        self.assertEqual(pg.prev.url, 'http://localhost/?page=1')
        self.assertEqual(pg.current.page, 1)
        self.assertEqual(pg.current.selected, True)
        self.assertEqual(pg.current.url, 'http://localhost/?page=2')
        self.assertEqual(pg.next.page, 2)
        self.assertEqual(pg.next.selected, False)
        self.assertEqual(pg.next.url, 'http://localhost/?page=3')
        self.assertEqual(pg.last.page, 3)
        self.assertEqual(pg.last.selected, False)
        self.assertEqual(pg.last.url, 'http://localhost/?page=4')
        self.assertEqual([p.page for p in pg], [0,1,2,3])

    def test70_ListingInitialScreen(self):
        lst = self.lst(self.req())
        self.expects(transaction='begin')
        self.expects(
            sql='SELECT COUNT(*) FROM "user" WHERE uid > 1', res = [[63]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 'WHERE uid > 1 LIMIT 20 OFFSET 0'),
            res = [[i, 'N%d' % i, 'l%d' % i, TEST_EPOCH] for i in range(20)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 63)
        self.assertEqual(lst.pager.currentPage, 0)
        self.assertEqual(lst.pager.pageCount, 4)
        self.assertEqual(lst.pager.next.page, 1)
        self.assertEqual(len(lst.items), 20)
        self.assertEqual(
            str(lst.pager.next.url), 'http://localhost/?page=2')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?order=%2Buid')
        storage.commit()

    def test71_ListingWithExplicitItemLimit(self):
        lst = self.lst(self.req(get={'limit':35}))
        self.expects(transaction='begin')
        self.expects(
            sql= 'SELECT COUNT(*) FROM "user" WHERE uid > 1', res = [[63]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 'WHERE uid > 1 LIMIT 35 OFFSET 0'),
            res = [[i, 'N%d' % i, 'l%d' % i, TEST_EPOCH] for i in range(35)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 63)
        self.assertEqual(lst.pager.pageCount, 2)
        self.assertEqual(len(lst.items), 35)
        self.assertEqual(
            str(lst.pager.next.url), 'http://localhost/?limit=35&page=2')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?limit=35&order=%2Buid')
        storage.commit()

    def test72_ListingWithExplicitItemLimitAndPage(self):
        lst = self.lst(self.req(get={'limit':30, 'page':2}))
        self.expects(transaction='begin')
        self.expects(
            sql= 'SELECT COUNT(*) FROM "user" WHERE uid > 1', res = [[58]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 'WHERE uid > 1 LIMIT 30 OFFSET 30'),
            res = [[i, 'N%d' % i, 'l%d' % i, TEST_EPOCH] for i in range(28)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 58)
        self.assertEqual(lst.pager.currentPage, 1)
        self.assertEqual(lst.pager.pageCount, 2)
        self.assertEqual(len(lst.items), 28)
        self.assertEqual(
            str(lst.pager.next.url),
            'http://localhost/?limit=30&page=2')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?limit=30&order=%2Buid')
        storage.commit()

    def test73_ListingWithDefaultOrder(self):
        lst = self.lst(self.req(get={'limit':35,'page':2}),defaultOrder='+name')
        self.expects(transaction='begin')
        self.expects(
            sql= 'SELECT COUNT(*) FROM "user" WHERE uid > 1', res = [[63]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 'WHERE uid > 1 ORDER BY name LIMIT 35 OFFSET 35'),
            res = [[i, 'N%d' % i, 'l%d' % i, TEST_EPOCH] for i in range(28)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 63)
        self.assertEqual(lst.pager.pageCount, 2)
        self.assertEqual(len(lst.items), 28)
        self.assertEqual(
            str(lst.pager.next.url),
            'http://localhost/?limit=35&page=2')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?limit=35&order=%2Buid')
        self.assertEqual(
            str(lst['name'].toggleUrl),
            'http://localhost/?limit=35&order=-name')
        storage.commit()

    def test74_ListingWithExplicitOrder(self):
        lst = self.lst(self.req(get={'limit':30, 'page':2, 'order':'-uid'}))
        self.expects(transaction='begin')
        self.expects(
            sql= 'SELECT COUNT(*) FROM "user" WHERE uid > 1', res = [[150]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 'WHERE uid > 1 ORDER BY uid DESC LIMIT 30 OFFSET 30'),
            res = [[i, 'N%d' % i, 'l%d' % i, TEST_EPOCH] for i in range(30)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 150)
        self.assertEqual(lst.pager.pageCount, 5)
        self.assertEqual(len(lst.items), 30)
        self.assertEqual(
            str(lst.pager.next.url),
            'http://localhost/?limit=30&order=-uid&page=3')
        self.assertEqual(
            str(lst.pager.prev.url),
            'http://localhost/?limit=30&order=-uid&page=1')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?limit=30&order=%2Buid')
        self.assertEqual(
            str(lst['name'].toggleUrl),
            'http://localhost/?limit=30&order=%2Bname')
        storage.commit()

    def test75_ListingWithCondition(self):
        lst = self.lst(self.req(get={'login':'ian', 'order':'-uid'}))
        self.expects(transaction='begin')
        self.expects(
            sql= ('SELECT COUNT(*) FROM "user" '
                  "WHERE (uid > 1) AND (login = 'ian')"), res = [[1]])
        self.expects(
            sql=('SELECT uid, name, login, mtime FROM "user" '
                 "WHERE (uid > 1) AND (login = 'ian') "
                 'ORDER BY uid DESC LIMIT 20 OFFSET 0'),
            res = [[i, 'N%d' % i, 'ian', TEST_EPOCH] for i in range(1)]
        )
        self.expects(transaction='commit')
        storage.begin()
        self.assertEqual(lst.itemCount, 1)
        self.assertEqual(lst.pager.pageCount, 1)
        self.assertEqual(len(lst.items), 1)
        self.assertEqual(
            str(lst.pager.next.url),
            'http://localhost/?login=ian&order=-uid&page=1')
        self.assertEqual(
            str(lst.pager.prev.url),
            'http://localhost/?login=ian&order=-uid&page=1')

        self.assertEqual(
            str(lst['uid'].ascUrl),
            'http://localhost/?login=ian&order=%2Buid')
        self.assertEqual(
            str(lst['uid'].toggleUrl),
            'http://localhost/?login=ian&order=%2Buid')
        self.assertEqual(
            str(lst['uid'].descUrl),
            'http://localhost/?login=ian&order=-uid')

        self.assertEqual(
            str(lst['name'].ascUrl),
            'http://localhost/?login=ian&order=%2Bname')
        self.assertEqual(
            str(lst['name'].toggleUrl),
            'http://localhost/?login=ian&order=%2Bname')
        self.assertEqual(
            str(lst['name'].descUrl),
            'http://localhost/?login=ian&order=-name')

        self.assertEqual(
            str(lst['login'].ascUrl),
            'http://localhost/?login=ian&order=%2Blogin')
        self.assertEqual(
            str(lst['login'].toggleUrl),
            'http://localhost/?login=ian&order=%2Blogin')
        self.assertEqual(
            str(lst['login'].descUrl),
            'http://localhost/?login=ian&order=-login')

        storage.commit()



suite = Suite(TestPageRange, TestListing)

if __name__ == '__main__':
    Main()
