# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.lists import *
import sys

from utils import *

PS = PagerState

class TestPager(WebCase):

    def preq(self, page):
        return self.req(get=dict(page=page))

    def test10_PageCount(self):
        ps = PS(itemCount=0, itemLimit=10)
        self.assertEqual(ps.pageCount, 0)
        ps = PS(itemCount=1, itemLimit=10)
        self.assertEqual(ps.pageCount, 1)
        ps = PS(itemCount=9, itemLimit=10)
        self.assertEqual(ps.pageCount, 1)
        ps = PS(itemCount=10, itemLimit=10)
        self.assertEqual(ps.pageCount, 1)
        ps = PS(itemCount=11, itemLimit=10)
        self.assertEqual(ps.pageCount, 2)
        ps = PS(itemCount=90, itemLimit=10)
        self.assertEqual(ps.pageCount, 9)
        ps = PS(itemCount=99, itemLimit=10)
        self.assertEqual(ps.pageCount, 10)
        # unlimited page
        ps = PS(itemCount=10, itemLimit=0)
        self.assertEqual(ps.pageCount, 0)

    def test12_PagerLength(self):
        ps = PS(itemCount=42, itemLimit=20)
        self.assertEqual(len(ps), 3)
        ps = PS(itemCount=0, itemLimit=20)
        self.assertEqual(len(ps), 0)
        ps = PS(itemCount=100, itemLimit=0)
        self.assertEqual(len(ps), 0)

    def test14_PagerAsBoolean(self):
        ps = PS(itemCount=42, itemLimit=20)
        self.failUnless(ps)
        ps = PS(itemCount=100, itemLimit=0)
        self.failIf(ps)

    def test20_CurrentPage(self):
        ps = PS(itemCount=20, itemLimit=10, req=self.preq(-1))
        self.assertEqual(ps._current, 0)
        ps = PS(itemCount=20, itemLimit=10, req=self.preq(0))
        self.assertEqual(ps._current, 0)
        ps = PS(itemCount=25, itemLimit=10, req=self.preq(1))
        self.assertEqual(ps._current, 1)
        ps = PS(itemCount=25, itemLimit=10, req=self.preq(2))
        self.assertEqual(ps._current, 2)
        ps = PS(itemCount=25, itemLimit=10, req=self.preq(3))
        self.assertEqual(ps._current, 2)

    def test21_CurrentPageWhileNoLimit(self):
        ps = PS(itemCount=1000, itemLimit=0)
        self.assertEqual(ps._current, None)

    def test21_CurrentPageWhileNoItems(self):
        ps = PS(itemCount=0, itemLimit=20)
        self.assertEqual(ps._current, None)

    def test30_PageURL(self):
        ps = PS(req=self.req('http://foo.cz/aaa'))
        self.assertEqual(ps.pageURL(0), 'http://foo.cz/aaa?page=0')
        self.assertEqual(ps.pageURL(2), 'http://foo.cz/aaa?page=2')
        ps = PS(req=self.req('http://foo.cz/aaa?page=1'))
        self.assertEqual(ps.pageURL(0), 'http://foo.cz/aaa?page=0')
        self.assertEqual(ps.pageURL(2), 'http://foo.cz/aaa?page=2')

    def test40_PageItem(self):
        ps = PS(req=self.req('http://foo.cz/aaa'), itemCount=22, itemLimit=10)
        p0 = ps[0]
        self.assertEqual(p0.page, 0)
        self.assertEqual(p0.first, 0)
        self.assertEqual(p0.last, 9)
        self.assertEqual(p0.url, 'http://foo.cz/aaa?page=0')
        self.assertEqual(p0.selected, True)
        p1 = ps[1]
        self.assertEqual(p1.page, 1)
        self.assertEqual(p1.first, 10)
        self.assertEqual(p1.last, 19)
        self.assertEqual(p1.url, 'http://foo.cz/aaa?page=1')
        self.assertEqual(p1.selected, False)
        p2 = ps[2]
        self.assertEqual(p2.page, 2)
        self.assertEqual(p2.first, 20)
        self.assertEqual(p2.last, 21)
        self.assertEqual(p2.url, 'http://foo.cz/aaa?page=2')
        self.assertEqual(p2.selected, False)

    def test41_PredefinedPagesWhenNoPages(self):
        ps = PS(req=self.req('http://foo.cz/aaa'), itemCount=22, itemLimit=0)
        self.assertEqual(ps.first, None)
        self.assertEqual(ps.prev, None)
        self.assertEqual(ps.current, None)
        self.assertEqual(ps.next, None)
        self.assertEqual(ps.last, None)

    def test42_PredefinedPagesWhenOnePage(self):
        ps = PS(req=self.req('http://foo.cz/aaa'), itemCount=8, itemLimit=10)
        self.assertEqual(ps.first.page, 0)
        self.assertEqual(ps.prev.page, 0)
        self.assertEqual(ps.current.page, 0)
        self.assertEqual(ps.next.page, 0)
        self.assertEqual(ps.last.page, 0)

    def test43_PredefinedPagesWhenManyPages(self):
        ps = PS(req=self.req('/?page=3'), itemCount=100, itemLimit=10)
        self.assertEqual(ps.first.page, 0)
        self.assertEqual(ps.prev.page, 2)
        self.assertEqual(ps.current.page, 3)
        self.assertEqual(ps.next.page, 4)
        self.assertEqual(ps.last.page, 9)


from wm4.query.tests.app import *
from wm4.query.builder import parse
from wm4.web.lists import *

class TestListState(WebCase):

    def setUp(self):
        WebCase.setUp(self)
        settings.storage_url('mockdb:')
        self.con = settings.storage_connection()

    def q(self, expr, up=1):
        f = sys._getframe(up)
        return query.IQuery(parse(
            expr, self.con, f.f_locals, f.f_globals, f.f_builtins
        ))

    def test20_ColumnStateForSimpleQuery(self):
        q = self.q('[u for u in User]')
        ls = QueryListState(query=q,req=self.req(get=dict(order_asc='u.login')))
        #print ls.orderedBy
        c_uid = ls.col('u.uid')
        #print c_uid.colExpr


def suite():
    return TestSuite((
        makeSuite(TestPager),
        makeSuite(TestListState),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
