# -*- coding: utf-8 -*-

from wm4.testing import *
from wm4.api import *

from utils import *


mem_logger = logs.Memory(level=logs.DEBUG)

ini = '''
[Import on Demand]
mem_logger = "%s.mem_logger"
[wm4.logs]
* = logs.Request(emitHandler=mem_logger)
''' % __name__


class TestRequestLogger(WebCase):

    def setUp(self):
        super(TestRequestLogger, self).setUp()
        mem_logger._clearLog()

    @with_ini(ini)
    def test10_NoEmit(self):
        log = []
        def test(env, start_response):
            logs.getLogger('foo').debug('FOO')
            logs.getLogger('bar').warning('BAR')
            start_response('200', [])
            log.extend(web.request().req_log)
            return ['ok']
        app = self.mkapp(
            'environment','errors','transaction','http_exceptions',
            funcMiddleware(test)
        )
        s,h,b = self.handle(app, self.env())
        self.assertEqual(s, '200')
        self.assertEqual(h, [])
        self.assertEqual(b, 'ok')
        self.assertEqual(len(log), 2)
        r0, r1 = log
        self.assertEqual(r0.levelno, logs.DEBUG)
        self.assertEqual(r0.name, 'foo')
        self.assertEqual(r0.msg, 'FOO')
        self.assertEqual(r1.levelno, logs.WARN)
        self.assertEqual(r1.name, 'bar')
        self.assertEqual(r1.msg, 'BAR')
        self.assertEqual(mem_logger._getLog(), [])

    @with_ini(ini)
    def test20_Emit(self):
        log = []
        def test(env, start_response):
            logs.getLogger('foo').debug('FOO')
            logs.getLogger('bar').warning('BAR')
            logs.getLogger('baz').error('BAZ')
            start_response('200', [])
            log.extend(web.request().req_log)
            return ['ok']
        app = self.mkapp(
            'environment','errors','transaction','http_exceptions',
            funcMiddleware(test)
        )
        s,h,b = self.handle(app, self.env())
        self.assertEqual(len(log), 0)
        self.assertEqual(mem_logger._getLog(), ['FOO\nBAR\nBAZ'])

    @with_ini(ini)
    def test22_Exception(self):
        log = []
        def test(env, start_response):
            logs.getLogger('foo').debug('FOO')
            logs.getLogger('bar').warning('BAR')
            1/0
            start_response('200', [])
            log.extend(web.request().req_log)
            return ['ok']
        app = self.mkapp(
            'environment','errors','transaction','http_exceptions',
            funcMiddleware(test)
        )
        s,h,b = self.handle(app, self.env())
        self.assertEqual(len(log), 0)
        ml = mem_logger._getLog()
        self.assertEqual(len(ml), 1)
        m0 = ml[0]
        assert m0.startswith('FOO\nBAR\n')
        assert m0.endswith(
            'ZeroDivisionError: integer division or modulo by zero')


suite = Suite(TestRequestLogger)

if __name__ == '__main__':
    Main()
