# -*- coding: ISO-8859-2 -*-

from unittest import *
from lib4.cgi import *
from wm4.api import *
from wm4.web.routes import *
from wm4.web.navigation import *

from utils import *


class S(Route):
    match = 'test/'
    actionLinks = Lazy(lambda s: actionLinks(s, '', './priv'))

    class public(Route):
        match = ''
        permission = Const(None)
        asAction = Lazy(lambda s: Link.fromRoute(s, label='Pub'))

    class private(Route):
        match = 'priv'
        permission = Const(web.Nobody())
        asAction = Lazy(Link.fromRoute)

class TestLinks(WebCase):

    @with_req()
    def test10_PublicNotSelectedLink(self):
        r = S.traverse('test/')
        a = r.asAction
        self.assertEqual(a.pathInfo, '/test/')
        self.assertEqual(a.url, 'http://localhost/test/')
        self.assertEqual(a.selected, False)
        self.assertEqual(a.enabled, True)
        self.assertEqual(a.visible, True)
        self.assertEqual(a.enabled, True)
        self.assertEqual(a.label, 'Pub')

    @with_req('/test/')
    def test11_PublicSelectedLink(self):
        r = S.traverse('/test/')
        a = r.asAction
        self.assertEqual(a.pathInfo, '/test/')
        self.assertEqual(a.url, 'http://localhost/test/')
        self.assertEqual(a.selected, True)
        self.assertEqual(a.visible, True)
        self.assertEqual(a.enabled, True)
        self.assertEqual(a.label, 'Pub')

    @with_req()
    def test12_PrivateNotSelectedLink(self):
        r = S.traverse('/test/priv')
        a = r.asAction
        self.assertEqual(a.pathInfo, '/test/priv')
        self.assertEqual(a.url, 'http://localhost/test/priv')
        self.assertEqual(a.selected, False)
        self.assertEqual(a.visible, False)
        self.assertEqual(a.enabled, False)
        self.assertEqual(a.label, 'Private')

    @with_req('/test/priv')
    def test13_PrivateSelectedLink(self):
        r = S.traverse('/test/priv')
        a = r.asAction
        self.assertEqual(a.pathInfo, '/test/priv')
        self.assertEqual(a.url, 'http://localhost/test/priv')
        self.assertEqual(a.selected, True)
        self.assertEqual(a.visible, False)
        self.assertEqual(a.enabled, False)
        self.assertEqual(a.label, 'Private')


class TestActionLinks(WebCase):

    @with_req('/test/')
    def test1(self):
        r = S.traverse('/test/')
        a = r.actionLinks
        self.assertEqual(len(a), 2)
        a0 = a[0]
        self.assertEqual(a0.pathInfo, '/test/')
        self.assertEqual(a0.selected, True)
        self.assertEqual(a0.visible, True)
        a1 = a[1]
        self.assertEqual(a1.pathInfo, '/test/priv')
        self.assertEqual(a1.selected, False)
        self.assertEqual(a1.visible, False)



class P(Route):
    match = ''
    pathLinks = Lazy(lambda s: pathLinks(s), virtual=True)
    asPath    = Lazy(Link.fromRoute, virtual=True)
    class a(Route):
        match = 'a/'
        class aa(Route):
            match = 'aa'
        class ab(Route):
            match = 'ab'
            asPath = Lazy(lambda s: Link.fromRoute(s, label='A&B'))
    class b(Route):
        match = 'b/'
        class ba(Route):
            match = 'ba'
            pathLinks = Lazy(lambda s: pathLinks(s, False))


class TestPathLinks(WebCase):

    @with_req('/a/ab')
    def test1(self):
        r = P.traverse('/a/ab')
        p = r.pathLinks
        self.assertEqual(len(p), 3)
        p0 = p[0]
        self.assertEqual(p0.pathInfo, '/')
        self.assertEqual(p0.selected, False)
        self.assertEqual(p0.label, 'P')
        p1 = p[1]
        self.assertEqual(p1.pathInfo, '/a/')
        self.assertEqual(p1.selected, False)
        self.assertEqual(p1.label, 'A')
        p2 = p[2]
        self.assertEqual(p2.pathInfo, '/a/ab')
        self.assertEqual(p2.selected, True)
        self.assertEqual(p2.label, 'A&B')

    @with_req('/b/ba')
    def test2(self):
        r = P.traverse('/b/ba')
        p = r.pathLinks
        self.assertEqual(len(p), 2)
        p0 = p[0]
        self.assertEqual(p0.pathInfo, '/')
        self.assertEqual(p0.selected, False)
        self.assertEqual(p0.label, 'P')
        p1 = p[1]
        self.assertEqual(p1.pathInfo, '/b/')
        self.assertEqual(p1.selected, False)
        self.assertEqual(p1.label, 'B')





def suite():
    return TestSuite((
        makeSuite(TestLinks),
        makeSuite(TestActionLinks),
        makeSuite(TestPathLinks),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
