# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.routes import Route, Const

from utils import *

def handler(response, template=None):
    response.body = [str(template)]
    return response

def mhandler(response):
    response.body = ['MHandler']
    return response
mhandler.allowed_methods = set(('PUT', 'DELETE'))

class chandler(web.Handler):
    def do_test(self):
        resp = self.req.response
        resp.body = ['TEST']
        return resp
    def do_test_OPTIONS(self):
        resp = self.req.response
        resp.body = ['OPTIONS']
        return resp


class S(Route):
    template = Const('home.wt')
    class of(Route):
        match = 'of'
        handler = Const(handler)
    class sf(Route):
        match = 'sf'
        handler = Const('%s.handler' % __name__)
    class oc(Route):
        match = 'oc'
        handler = Const(chandler)
        action = Const('test')
    class sc(Route):
        match = 'sc'
        handler = Const('%s.chandler' % __name__)
        action = Const('test')
    class om(Route):
        match = 'om'
        handler = Const(mhandler)


INI = """\
[wm4.logs]
* = logs.Null()

[WSGI Middleware sitemap]
sitemap = importString('%s.S')
""" % __name__


class TestPublisher(WebCase):

    @with_ini(INI)
    def test10_FunctionHandler(self):
        app = web.makeApp()
        s,h,b = self.handle(app, self.env('/of'))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'home.wt')

    @with_ini(INI)
    def test11_StringHandler(self):
        app = web.makeApp()
        s,h,b = self.handle(app, self.env('/of'))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'home.wt')

    @with_ini(INI)
    def test12_ClassHandler(self):
        app = web.makeApp()
        s,h,b = self.handle(app, self.env('/oc'))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'TEST')

    @with_ini(INI)
    def test13_ClassHandler(self):
        app = web.makeApp()
        s,h,b = self.handle(app, self.env('/sc'))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'TEST')

    @with_ini(INI)
    def test30_InvalidRequestMethod(self):
        app = web.makeApp()
        s,h,b = self.handle(app, self.env('/of',env={'REQUEST_METHOD':'PUT'}))
        self.assertEqual(s, '405 Method Not Allowed')
        s,h,b = self.handle(app, self.env('/sf',env={'REQUEST_METHOD':'PUT'}))
        self.assertEqual(s, '405 Method Not Allowed')
        s,h,b = self.handle(app, self.env('/oc',env={'REQUEST_METHOD':'PUT'}))
        self.assertEqual(s, '405 Method Not Allowed')
        s,h,b = self.handle(app, self.env('/sc',env={'REQUEST_METHOD':'PUT'}))
        self.assertEqual(s, '405 Method Not Allowed')

    @with_ini(INI)
    def test32_ExplicitRequestMethod(self):
        app = web.makeApp()
        s,h,b =self.handle(app,self.env('/oc',env={'REQUEST_METHOD':'OPTIONS'}))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'OPTIONS')

    @with_ini(INI)
    def test33_ExplicitRequestMethod(self):
        app = web.makeApp()
        s,h,b =self.handle(app,self.env('/om',env={'REQUEST_METHOD':'PUT'}))
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'MHandler')
        # default GET is not allowed
        s,h,b = self.handle(app, self.env('/om'))
        self.assertEqual(s, '405 Method Not Allowed')



def suite():
    return TestSuite((
        makeSuite(TestPublisher),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
