# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from lib4.cgi import TestUpload
from urllib import quote
import os

from utils import *

class TestRequest(WebCase):

    def getReq(self, *args, **kwds):
        req = []
        def reqapp(env, start):
            req.append(web.Request(environ=env))
            start('200 OK', [])
            return 'OK'
        app = self.mkapp(
            'environment', 'errors', 'transaction', 'cookies',
            'http_exceptions', 'session', 'authentication',
            funcMiddleware(reqapp)
        )
        env = self.env(*args, **kwds)
        s,h,b = self.handle(app, env=env)
        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [])
        self.assertEqual(b, 'OK')
        return req[0]

    def test02_RequestCharacteristic(self):
        r = self.req()
        self.assertEqual(r.method, 'GET')

    def test10_URLs(self):
        r = self.req('http://www.dat.cz/')
        self.assertEqual(r.serverURL, 'http://www.dat.cz')
        self.assertEqual(r.appURL, 'http://www.dat.cz/')
        self.assertEqual(r.requestURL, 'http://www.dat.cz/')
        self.assertEqual(r.requestURI, 'http://www.dat.cz/')
        self.assertEqual(r.pathInfo, '/')

    def test11_URLs(self):
        r = self.req('http://www.dat.cz/aaa/bbb')
        self.assertEqual(r.serverURL, 'http://www.dat.cz')
        self.assertEqual(r.appURL, 'http://www.dat.cz/')
        self.assertEqual(r.requestURL, 'http://www.dat.cz/aaa/bbb')
        self.assertEqual(r.requestURI, 'http://www.dat.cz/aaa/bbb')
        self.assertEqual(r.pathInfo, '/aaa/bbb')

    def test12_URLs(self):
        r = self.req(
            'http://www.dat.cz/aaa/bbb',env={'SCRIPT_ALIAS_MATCH':'/aaa'}
        )
        self.assertEqual(r.serverURL, 'http://www.dat.cz')
        self.assertEqual(r.appURL, 'http://www.dat.cz/aaa')
        self.assertEqual(r.requestURL, 'http://www.dat.cz/aaa/bbb')
        self.assertEqual(r.requestURI, 'http://www.dat.cz/aaa/bbb')
        self.assertEqual(r.pathInfo, '/bbb')

    def test13_URLs(self):
        r = self.req('http://www.dat.cz:8000/')
        self.assertEqual(r.serverURL, 'http://www.dat.cz:8000')
        r = self.req('https://www.dat.cz/')
        self.assertEqual(r.serverURL, 'https://www.dat.cz')
        r = self.req('https://www.dat.cz:8443/')
        self.assertEqual(r.serverURL, 'https://www.dat.cz:8443')

    def test20_GetValue(self):
        r = self.req(get=dict(a=1, l=[1,2,3]))
        self.assertEqual(r.getValue('a'), '1')
        self.assertEqual(r.getValue('c'), None)
        self.assertRaises(web.BadRequest, r.getValue, 'l')
        r = self.req(post=dict(a=1, l=[1,2,3]))
        self.assertEqual(r.getValue('a'), '1')
        self.assertEqual(r.getValue('c'), None)
        self.assertRaises(web.BadRequest, r.getValue, 'l')

    def test20_GetValueWithInvaliedEncoding(self):
        r = self.req(get=dict(a='\xee\xee'))
        self.assertRaises(web.BadRequest, r.getValue, 'a')

    def test21_GetList(self):
        r = self.req(get=dict(a=[1,2], b=3))
        self.assertEqual(r.getList('a'), ['1','2'])
        self.assertEqual(r.getList('b'), ['3'])
        self.assertEqual(r.getList('c'), [])
        r = self.req(post=dict(a=[1,2], b=3))
        self.assertEqual(r.getList('a'), ['1','2'])
        self.assertEqual(r.getList('b'), ['3'])
        self.assertEqual(r.getList('c'), [])

    def test21_GetListWithInvaliedEncoding(self):
        r = self.req(get=dict(a=['a', 'b', '\xee\xee']))
        self.assertRaises(web.BadRequest, r.getList, 'a')

    def test22_GetUpload(self):
        f = os.path.join(RES_DIR, 'small.txt')
        r = self.req(post=dict(f=TestUpload(f)))
        u = r.getUpload('f')
        self.assertEqual(u.filename, 'small.txt')
        self.assertEqual(u.content_type, 'text/plain')
        self.failUnless(os.path.isfile(u.getPath()))
        self.assertEqual(os.stat(u.getPath()).st_size, 10)
        self.assertEqual(u.size, 10)

    def test22_GetUploadWithUselessContentType(self):
        f = os.path.join(RES_DIR, 'small.txt')
        for ct in (
            'application/octet-stream',
            'application/force-download',
            'application/x-download',
            'application/unknown',
        ):
            r = self.req(post=dict(f=TestUpload(f, content_type=ct)))
            u = r.getUpload('f')
            self.assertEqual(u.filename, 'small.txt')
            self.assertEqual(u.content_type, 'text/plain')
            self.failUnless(os.path.isfile(u.getPath()))
            self.assertEqual(os.stat(u.getPath()).st_size, 10)
            self.assertEqual(u.size, 10)

    def test23_GetUploadWithUnsupportedContentType(self):
        f = os.path.join(RES_DIR, 'small.txt')
        for ct in (
            'application/foo',
            'foo/bar',
            'text/english',
            'inode/blockdevice',
            'image/cgm',
            'application/x-rx',
        ):
            r = self.req(post=dict(f=TestUpload(f, content_type=ct)))
            u = r.getUpload('f')
            self.assertEqual(u.filename, 'small.txt')
            self.assertEqual(u.content_type, 'text/plain')
            self.failUnless(os.path.isfile(u.getPath()))
            self.assertEqual(os.stat(u.getPath()).st_size, 10)
            self.assertEqual(u.size, 10)

    def test23_GetEncodedUpload(self):
        f = os.path.join(RES_DIR, 'small.txt.gz')
        r = self.req(post=dict(f=TestUpload(f, )))
        u = r.getUpload('f')
        self.assertEqual(u.filename, 'small.txt.gz')
        self.assertEqual(u.size, 40)
        self.assertEqual(u.content_type, 'application/gzip')
        self.failUnless(os.path.isfile(u.getPath()))
        self.assertEqual(os.stat(u.getPath()).st_size, 40)

    def test24_GetNonUpload(self):
        r = self.req(post=dict(f=''))
        u = r.getUpload('f')
        self.assertEqual(u, None)

    def test25_GetEmptyFile(self):
        f = os.path.join(RES_DIR, 'empty.txt')
        r = self.req(post=dict(f=TestUpload(f, )))
        u = r.getUpload('f')
        self.assertEqual(u.filename, 'empty.txt')
        self.assertEqual(u.size, 0)
        self.assertEqual(u.content_type, 'text/plain')
        self.failUnless(os.path.isfile(u.getPath()))
        self.assertEqual(os.stat(u.getPath()).st_size, 0)

    def test26_StrippingDirectoryFromUpload(self):
        f = os.path.join(RES_DIR, 'small.txt')
        for fn in (
            'C:\\Desktop\\foo.txt',
            '/home/agata/foo.txt',
            'foo.txt',
        ):
            r = self.req(post=dict(f=TestUpload(f, filename=fn)))
            u = r.getUpload('f')
            # compare in latin2 because of various unicode normalization
            # forms
            self.assertEqual(u.filename, 'foo.txt')
            self.assertEqual(u.content_type, 'text/plain')
            self.failUnless(os.path.isfile(u.getPath()))
            self.assertEqual(os.stat(u.getPath()).st_size, 10)
            self.assertEqual(u.size, 10)

    def test30_Cookies(self):
        r = self.getReq(cookies=dict(s='42'))
        self.assertEqual(r.getCookie('s'), '42')
        self.assertEqual(r.getCookie('t'), None)

    def test40_I18nDefaultLanguage(self):
        r = self.getReq()
        self.assertEqual(r.force_language, None)
        self.assertEqual(r.accept_language, [])
        self.assertEqual(r.language, 'en')

    def test41_I18nLanguageFromHeader(self):
        r = self.getReq(env={'HTTP_ACCEPT_LANGUAGE':'en;q=0.4,cs,sk;q=0.9'})
        self.assertEqual(r.force_language, None)
        self.assertEqual(r.accept_language, [(1.0,'cs'),(0.9,'sk'),(0.4,'en')])
        self.assertEqual(r.language, 'cs')

    def test42_I18nLanguageFromEnviron(self):
        r = self.getReq(env={'FORCE_LANGUAGE':'de'})
        self.assertEqual(r.force_language, 'de')
        self.assertEqual(r.accept_language, [])
        self.assertEqual(r.language, 'de')

    def test42_I18nLanguageFromChromek(self):
        # GET / HTTP/1.1
        # User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/5
        # Accept: application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;
        # Accept-Encoding: gzip,deflate,bzip2,sdch
        # Accept-Language: cs-CZ,cs;q=0.8
        # Accept-Charset: windows-1250,utf-8;q=0.7,*;q=0.3
        r = self.getReq(env={'HTTP_ACCEPT_LANGUAGE':'cs-CZ,cs;q=0.8'})
        self.assertEqual(r.force_language, None)
        self.assertEqual(r.accept_language, [(1.0,'cs-CZ'),(0.8,'cs')])
        self.assertEqual(r.language, 'cs')

    def test90_Apply(self):
        r = self.req(get=dict(a='A'))
        r.params['p'] = 'P'
        self.assertEqual(r.apply(lambda req:req), r)
        self.assertEqual(r.apply(lambda req,p:(req,p)), (r,'P'))

    def test91_ApplyToOldStyleMethod(self):
        r = self.req(get=dict(a='A'))
        class Old:
            def __call__(self, req,a):
                return (req,a)
        self.assertEqual(r.apply(Old()), (r,'A'))

    def test91_ApplyToNewStyleMethod(self):
        r = self.req(get=dict(a='A'))
        class New(object):
            def __call__(self, req,a):
                return (req,a)
        self.assertEqual(r.apply(New()), (r,'A'))



SUBREQ_INI = '''\
[wsgi]
components = (
        'environment',
        'errors',
        'transaction',
        'http_exceptions',
        'subapp',
    )

[WSGI Middleware errors]
show_details = True
throw_errors = False

[WSGI Middleware subapp]
factory = '%(mod)s.submid'

[wm4.logs]
#* = logs.File(filename='/tmp/dev.log', level=logs.DEBUG)
* = logs.Memory()
''' % dict(mod=__name__)







def subapp(env, start):
    pi = env.get('PATH_INFO')
    req = web.request()
    if pi == '/error':
        1/0
    elif pi == '/hello':
        start('200 OK', [('Content-Type', 'text/plain')])
        return [str(pi)]
    else:
        start('200 OK', [('Content-Type', 'text/plain')])
        if req.getValue('error',None):
            x = req.subreq('/error')
        else:
            x = req.subreq('/hello')
        b = ['SUBREQUEST', '', x[0]]
        for h,v in x[1]:
            b.append('%s: %s' % (h,v))
        b.append('')
        b.extend(x[2])
        return ['\n'.join(b)]
submid = funcMiddleware(subapp)


class TestSubrequest(WebCase):

    cfgURLs = WebCase.cfgURLs + ('data:,%s' % quote(SUBREQ_INI),)

    def test10_GET(self):
        app = web.makeApp()
        env = self.env('/')
        s, h, b = self.handle(app, env)
        self.assertEqual(s, '200 OK')
        self.assertEqual(b, (
            'SUBREQUEST\n\n'
            '200 OK\n'
            'Content-Type: text/plain\n\n'
            '/hello'
        ))

    def test11_GET_with_error(self):
        logs.Memory.clearLog()
        app = web.makeApp()
        env = self.env('/?error=1')
        s, h, b = self.handle(app, env)
        self.assertEqual(s, '200 OK')
        b = [l for l in b.splitlines() if l]
        self.assertEqual(b[0], 'SUBREQUEST')
        self.assertEqual(b[1], '500 Internal Server Error')
        self.assertEqual(
                b[-1], 'ZeroDivisionError: integer division or modulo by zero')
        l = logs.Memory.getLog()
        self.assertEqual(len(l), 1)
        l = l[0]
        self.failUnless('ZeroDivisionError' in l)


def suite():
    return TestSuite((
        makeSuite(TestRequest),
        makeSuite(TestSubrequest),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
