# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.api import *
from lib4.cgi import etag_create, http_time
from os.path import join, dirname, abspath, realpath
import wm4.web.tests.res as test_res
from wm4.web.resources import *
import os

from utils import *



test_res_dir = abspath(dirname(test_res.__file__))

class TestResources(WebCase):

    def resapp(self, app):
        app = self.mkapp(
            'environment', 'errors', 'transaction', 'cookies',
            'http_exceptions', 'session', 'authentication', 'resources',
            funcMiddleware(app)
        )
        return app

    def get(self, name):
        return getResource(name)

    def test20_FileResource(self):
        rs = self.get('wm4.web.tests.res/small.txt')
        f = realpath(abspath(join(dirname(__file__), 'res/small.txt')))
        m = os.stat(f).st_mtime
        assert isinstance(rs, Resource)
        self.assertEqual(rs.content_type, 'text/plain')
        self.assertEqual(rs.module, 'wm4.web.tests.res')
        self.assertEqual(rs.modpath, 'small.txt')
        self.assertEqual(rs.filename, join(test_res_dir, 'small.txt'))
        self.assertEqual(rs.size, 10)

    def test21_ResourceCaching(self):
        s1 = getResource('wm4.web.tests.res/small.txt')
        s2 = getResource('wm4.web.tests.res/small.txt')
        assert s1 is s2
        i1 = getResource('wm4.web.tests.res/i.jpg')
        i2 = getResource('wm4.web.tests.res/i.jpg')
        assert i1 is i2

    def test30_ImageResource(self):
        rs = self.get('wm4.web.tests.res/i.jpg')
        f = realpath(abspath(join(dirname(__file__), 'res/i.jpg')))
        m = os.stat(f).st_mtime
        assert isinstance(rs, ImageResource)
        self.assertEqual(rs.content_type, 'image/jpeg')
        self.assertEqual(rs.module, 'wm4.web.tests.res')
        self.assertEqual(rs.modpath, 'i.jpg')
        self.assertEqual(rs.filename, join(test_res_dir, 'i.jpg'))
        self.assertEqual(rs.size, 14590)
        self.assertEqual(rs.width, 177)
        self.assertEqual(rs.height, 262)

    def test31_JavascriptResource(self):
        rs = self.get('wm4.web.static/js/x_core.js')
        assert isinstance(rs, JavascriptResource)
        self.assertEqual(rs.content_type, 'application/x-javascript')
        self.assertEqual(rs.module, 'wm4.web.static')
        self.assertEqual(rs.modpath, 'js/x_core.js')

    def test32_CSSResource(self):
        rs = self.get('wm4.web.static/css/forms.css')
        assert isinstance(rs, CSSResource)
        self.assertEqual(rs.content_type, 'text/css')

    def test40_InvalidModule(self):
        rs = self.get('neex/foo.bar')
        self.assertEqual(rs, None)

    def test41_InvalidPath(self):
        rs = self.get('wm4.web.static/x-css/forms.css')
        self.assertEqual(rs, None)

    def test50_ResourcePublishing(self):
        def handle_res(env, start_response):
            obj = getResource('wm4.web.tests.res/small.txt')
            resp = obj.handleHTTP(web.Request(env))
            start_response(resp.status, resp.headers)
            return resp.body
        s,h,b = self.handle(self.resapp(handle_res))
        self.assertEqual(s, '200 OK')
        headers = dict(h)
        filename = join(test_res_dir, 'small.txt')
        self.assertEqual(headers['Content-Length'], '10')
        self.assertEqual(headers['ETag'], etag_create(filename))
        self.assertEqual(headers['Content-Type'], 'text/plain')
        self.assertEqual(
            headers['Last-Modified'], http_time(os.stat(filename).st_mtime)
        )
        self.assertEqual(''.join(b), open(filename).read())

    def test51_ResourcePublishingWithSendfile(self):
        def handle_res(env, start_response):
            obj = getResource('wm4.web.tests.res/small.txt')
            resp = obj.handleHTTP(web.Request(env))
            start_response(resp.status, resp.headers)
            return resp.body
        s,h,b = self.handle(
            self.resapp(handle_res),
            self.env(env={'HTTP_X_SENDFILE':'X-Sendfile'}),
        )
        self.assertEqual(s, '200 OK')
        headers = dict(h)
        filename = join(test_res_dir, 'small.txt')
        self.assertEqual(headers['Content-Type'], 'text/plain')
        self.assertEqual(headers['X-Sendfile'], filename)
        self.assertEqual(''.join(b), '')

def suite():
    return TestSuite((
        makeSuite(TestResources),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
