# -*- coding: ISO-8859-2 -*-

from unittest import *
from lib4.cgi import *
from wm4.api import *
from wm4.web.routes import *
from wm4.web.sitemaps import *
import sys

from utils import *




THIS_MODULE = sys.modules[__name__]

RP = lambda *p: RoutePattern(p)

class TestRoutePattern(TestCase):

    def test10_Empty(self):
        rp = RP('')
        self.assertEqual(rp.fmt, '')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(rp.match('').groupdict(), {'r0':''})

    def test11_StaticName(self):
        rp = RP('foo/')
        self.assertEqual(rp.fmt, 'foo/')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(rp.match('foo/').groupdict(), {'r0':'foo/'})

    def test12_StaticNameNoEndSlash(self):
        rp = RP('foo')
        self.assertEqual(rp.fmt, 'foo')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(rp.match('foo').groupdict(), {'r0':'foo'})

    def test13_StaticPath(self):
        rp = RP('foo/bar/')
        self.assertEqual(rp.fmt, 'foo/bar/')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(rp.match('foo/bar/').groupdict(), {'r0':'foo/bar/'})

    def test14_StaticPathMultipart(self):
        rp = RP('foo/', 'bar/')
        self.assertEqual(rp.fmt, 'foo/bar/')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(
            rp.match('foo/bar/').groupdict(),
            {'r0':'foo/', 'r1':'bar/'}
        )

    def test15_StaticPathNoEndSlash(self):
        rp = RP('foo/bar')
        self.assertEqual(rp.fmt, 'foo/bar')
        self.assertEqual(rp.variables, set([]))
        self.assertEqual(rp.match('foo/bar').groupdict(), {'r0':'foo/bar'})

    def test20_Parametrized(self):
        rp = RP(':foo/')
        self.assertEqual(rp.fmt, '%(p0_foo)s/')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('30/').groupdict(),
            {'r0':'30/', 'p0_foo':'30'}
        )

    def test21_ParametrizedMultipart(self):
        rp = RP(':foo', '/')
        self.assertEqual(rp.fmt, '%(p0_foo)s/')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('30/').groupdict(),
            {'r0':'30', 'r1':'/', 'p0_foo':'30'}
        )
    def test22_ParametrizedNoEndSlash(self):
        rp = RP(':foo')
        self.assertEqual(rp.fmt, '%(p0_foo)s')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('30').groupdict(),
            {'r0':'30', 'p0_foo':'30'}
        )

    def test23_ParametrizedWithAction(self):
        rp = RP(':foo/action')
        self.assertEqual(rp.fmt, '%(p0_foo)s/action')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('30/action').groupdict(),
            {'r0':'30/action', 'p0_foo':'30'}
        )

    def test23_ParametrizedWithActionMultipart(self):
        rp = RP(':foo/', 'action')
        self.assertEqual(rp.fmt, '%(p0_foo)s/action')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('30/action').groupdict(),
            {'r0':'30/', 'r1':'action', 'p0_foo':'30'}
        )

    def test26_ParametrizedSub(self):
        rp = RP('bar:(foo)')
        self.assertEqual(rp.fmt, 'bar%(p0_foo)s')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('bar30').groupdict(),
            {'r0':'bar30', 'p0_foo':'30'}
        )

    def test27_ParametrizedSubMultipart(self):
        rp = RP('bar', ':(foo)')
        self.assertEqual(rp.fmt, 'bar%(p1_foo)s')
        self.assertEqual(rp.variables, set(['p1_foo']))
        self.assertEqual(
            rp.match('bar30').groupdict(),
            {'r0':'bar', 'r1':'30', 'p1_foo':'30'}
        )

    def test30_Subpath(self):
        rp = RP('*foo/')
        self.assertEqual(rp.fmt, '%(p0_foo)s/')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('a/b/c/').groupdict(),
            {'r0':'a/b/c/', 'p0_foo':'a/b/c'}
        )

    def test31_SubpathMultipart(self):
        rp = RP('bar/', '*foo', '/')
        self.assertEqual(rp.fmt, 'bar/%(p1_foo)s/')
        self.assertEqual(rp.variables, set(['p1_foo']))
        self.assertEqual(
            rp.match('bar/a/b/c/').groupdict(),
            {'r0':'bar/', 'r1':'a/b/c', 'r2':'/', 'p1_foo':'a/b/c'}
        )

    def test32_SubpathNoEndSlash(self):
        rp = RP('*foo')
        self.assertEqual(rp.fmt, '%(p0_foo)s')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('a/b/c').groupdict(),
            {'r0':'a/b/c', 'p0_foo':'a/b/c'}
        )

    def test35_SubpathWithAction(self):
        rp = RP('*foo/action')
        self.assertEqual(rp.fmt, '%(p0_foo)s/action')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('a/b/c/action').groupdict(),
            {'r0':'a/b/c/action', 'p0_foo':'a/b/c'}
        )

    def test36_SubpathWithActionMultipart(self):
        rp = RP('*foo/', 'action')
        self.assertEqual(rp.fmt, '%(p0_foo)s/action')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('a/b/c/action').groupdict(),
            {'r0':'a/b/c/', 'r1':'action', 'p0_foo':'a/b/c'}
        )

    def test37_SubpathWithActionMultipart(self):
        rp = RP('*foo', '/action')
        self.assertEqual(rp.fmt, '%(p0_foo)s/action')
        self.assertEqual(rp.variables, set(['p0_foo']))
        self.assertEqual(
            rp.match('a/b/c/action').groupdict(),
            {'r0':'a/b/c', 'r1':'/action', 'p0_foo':'a/b/c'}
        )

    def test40_Complex(self):
        rp = RP('ent/', ':oid/', 'docs/', '*foo/', ':action')
        self.assertEqual(rp.fmt, 'ent/%(p1_oid)s/docs/%(p3_foo)s/%(p4_action)s')
        self.assertEqual(rp.variables, set(['p1_oid', 'p3_foo', 'p4_action']))
        self.assertEqual(
            rp.match('ent/10/docs/a/b/c/view').groupdict(),
            {'r0':'ent/', 'r1':'10/', 'r2':'docs/', 'r3':'a/b/c/', 'r4':'view',
             'p1_oid':'10', 'p3_foo':'a/b/c', 'p4_action':'view'}
        )

    def test50_InitialSlash(self):
        rp = RP('/foo', '/bar')
        self.assertEqual(rp.fmt, '/foo/bar')
        self.assertEqual(rp.variables, set())
        self.assertEqual(
            rp.match('/foo/bar').groupdict(),
            {'r0':'/foo', 'r1':'/bar'},
        )

    def test51_InitialSlash(self):
        rp = RP('/:oid', '/bar')
        self.assertEqual(rp.fmt, '/%(p0_oid)s/bar')
        self.assertEqual(rp.variables, set(['p0_oid']))
        self.assertEqual(
            rp.match('/23/bar').groupdict(),
            {'r0':'/23', 'r1':'/bar', 'p0_oid':'23'},
        )

    def test90_Sorting(self):
        assert RP('') < RP('foo')
        assert RP('foo') < RP(':foo')
        assert RP('foo') < RP('foo/bar')
        assert RP('foo/bar') < RP('foo/*path')
        assert RP('foo/:(path)xx') < RP('foo/:path')
        assert RP('foo/:path') < RP('foo/*path')





class TestRoutes(TestCase):

    def test10_RouteClassAsComponent(self):
        class R(Route):
            a = Const(42)
            class S(Route):
                pass
        self.assertEqual(binding.getParentComponent(R), THIS_MODULE)
        self.assertEqual(binding.getParentComponent(R.S), R)
        self.assertEqual(binding.getComponentName(R), 'R')
        self.assertEqual(binding.getComponentName(R.S), 'S')
        self.assertEqual(str(binding.getComponentPath(R.S, THIS_MODULE)),'R/S')
        self.assertEqual(str(binding.getComponentPath(R, THIS_MODULE)),'R')

    def test11_CopyWithParent(self):
        class r(Route):
            a = Const(42)
            class s(Route):
                match = ':oid/'
                class t(Route):
                    pass
        self.assertEqual(r.__bases__, (Route,))
        self.assertEqual(r.__parent__, (THIS_MODULE,))
        self.assertEqual(r.__all_parents__, (r,))
        self.assertEqual(r.__pattern__, RP('/'))

        s = r._child_route('s')
        self.assertEqual(s.__bases__, (r.s,))
        self.assertEqual(s.__parent__, (r,))
        self.assertEqual(s.__all_parents__, (r, s))
        self.assertEqual(s.__pattern__, RP('/', ':oid/'))

        t = r._child_route('s/t')
        self.assertEqual(t.__bases__, (r.s.t,))
        self.assertEqual(t.__parent__, (s,))
        self.assertEqual(t.__all_parents__, (r, s, t))
        self.assertEqual(t.__pattern__, RP('/', ':oid/', ''))

    def test12_ClassRegistries(self):
        class R(Route):
            a = Const(42)
            class S(Route):
                pass
        self.assertEqual(R.__params__, {'a':R.a})
        self.assertEqual(R.__children__, {'S':R._child_route('S')})
        self.assertEqual(R.S.__params__, {})
        self.assertEqual(R.S.__children__, {})
        self.assertEqual(R.__parent__, (THIS_MODULE,))
        self.assertEqual(R.__all_parents__, (R,))
        self.assertEqual(R.S.__parent__, (R,))
        self.assertEqual(R.S.__all_parents__, (R, R.S))

    def test15_ClassAttrs(self):
        class view(ActionRoute):
            foo = Const('foo')
            class detail(ActionRoute):
                foo = binding.classAttr(binding.Make(
                        lambda cls: cls.__parent__[0].foo
                ))
        self.assertEqual(view.match, 'view')
        assert isinstance(view.action, Const), repr(view.action)
        self.assertEqual(view.action.value, 'view')
        self.assertEqual(view.action.typ, str)

        self.assertEqual(view.detail.match, 'detail')
        assert isinstance(view.detail.action, Const), repr(view.detail.action)
        self.assertEqual(view.detail.action.value, 'detail')
        self.assertEqual(view.detail.action.typ, str)

        assert view.foo is view.detail.foo

    def test16_InheritedDoesntOverrideClassAttr(self):
        class x(Route):
            foo = bar = Const('FOO_BAR')
            class y(Route):
                foo = binding.classAttr(binding.Make(lambda:Const('FOO')))
        self.assertEqual(x.foo.value, 'FOO_BAR')
        self.assertEqual(x.bar.value, 'FOO_BAR')
        self.assertEqual(x._child_route('y').foo.value, 'FOO')
        ybar = x._child_route('y').bar
        assert isinstance(ybar, Acquire)
        self.assertEqual(ybar.attr, 'bar')
        self.assertEqual(ybar.depth, 1)

    def test18_ConstructorCheckAttrExistences(self):
        class R(Route):
            a = Const(42)
        r = R({})
        self.assertRaises(AttributeError, R, {'c':1})

    def test20_ComputedParams(self):
        class R(Route):
            a = Const(42)
            b = Param(lambda s: s.a+1, typ=int)
        r = R({})
        self.assertEqual(r.a, 42)
        self.assertEqual(r.b, 43)

    def test22_NonvirtualParamIsComputedOnOriginalRoute(self):
        class R(Route):
            c = Const(21)
            p = Param(lambda s: s.c * 2)
            class S(Route):
                c = Const(0)
        r = R()
        self.assertEqual(r.p, 42)
        r = R._child_route('S')()
        self.assertEqual(r.p, 42)

    def test23_VirtualParamIsComputedOnOriginalRoute(self):
        class R(Route):
            c = Const(21)
            p = Param(lambda s: s.c * 2, virtual=True)
            class S(Route):
                c = Const(0)
        r = R()
        self.assertEqual(r.p, 42)
        r = R._child_route('S')()
        self.assertEqual(r.p, 0)

    def test24_ConstantConflict(self):
        class C(Route):
            a = Const(42)
        c = C()
        c.a = 42
        self.assertRaises(SitemapError, setattr, c, 'a', 43)

    def test25_IndirectConstantConflict(self):
        class C(Route):
            a = Const(42)
            b = Param(lambda s: 42, typ=int, equalTo='a')
        c = C()
        self.assertRaises(SitemapError, setattr, c, 'b', 43)

    def test30_StaticPathInfo(self):
        class R(Route):
            match = 'static'
        r = R({})
        self.assertEqual(r.pathInfo, '/static')
        class R(Route):
            match = '/static'
        r = R({})
        self.assertEqual(r.pathInfo, '/static')

    def test31_GeneratedPathInfo(self):
        class R(Route):
            match = ':oid/view'
            oid = Param(lambda s: str(s.obj / 2))
            obj = Param(lambda s: int(s.oid) * 2, typ=int)
        r = R({'oid':42})
        self.assertEqual(r.pathInfo, '/42/view')
        r = R({'obj':42})
        self.assertEqual(r.pathInfo, '/21/view')

    def test32_GeneratedPathInfo(self):
        class R(Route):
            match = 'mod/:oid/:action'
            action = Param()
            oid = Param(lambda s: str(s.obj / 2))
            obj = Param(lambda s: int(s.oid) * 2, typ=int)
        r = R({'oid':42, 'action':'view'})
        self.assertEqual(r.pathInfo, '/mod/42/view')
        r = R({'obj':42, 'action':'view'})
        self.assertEqual(r.pathInfo, '/mod/21/view')

    def test42_RouteForClasses(self):
        class SM(Route):
            class r(Route):
                match = 'r'
                clsObj = Const(Record)
                signature = Signature('clsObj')
            class u(Route):
                match = 'u'
                clsObj = Const(User)
                signature = Signature('clsObj')

        r = SM.route_for(Record)
        self.assertEqual(r.pathInfo, '/r')
        self.assertEqual(r.clsObj, Record)
        p = r.parentRoute
        assert isinstance(p, SM)
        self.assertEqual(p.pathInfo, '/')

        r = SM.route_for(User)
        self.assertEqual(r.pathInfo, '/u')
        self.assertEqual(r.clsObj, User)
        p = r.parentRoute
        assert isinstance(p, SM)
        self.assertEqual(p.pathInfo, '/')

    def test50_ParameterAcquisition(self):
        pass

    def test51_ParameterAcquisition(self):
        obj = Record(x='1', y='2', z='3')
        class X(Route):
            match = ':x/'
            x = Param(lambda s: s.o.x)
            o = Param(lambda s: obj, typ=Record)
            class Y(Route):
                match = ':x/'
                x = Param(lambda s: s.o.y)
                _x = Acquire('../x')
                class Z(Route):
                    match = ':x/'
                    x = Param(lambda s: s.o.z)
                    _x = Acquire('../x')
                    class V(Route):
                        match = 'view'

        for v in [
            X._child_route('Y/Z/V')({'o': obj}),
            X.traverse('/1/2/3/view'),
        ]:
            self.assertEqual(v.o, obj)
            self.assertEqual(v.x, '3')
            self.assertEqual(v._x, '2')
            self.assertEqual(v.pathInfo, '/1/2/3/view')

            z = v.parentRoute
            self.assertEqual(z.o, obj)
            self.assertEqual(z.x, '3')
            self.assertEqual(z._x, '2')
            self.assertEqual(z.pathInfo, '/1/2/3/')

            y = z.parentRoute
            self.assertEqual(y.o, obj)
            self.assertEqual(y.x, '2')
            self.assertEqual(y._x, '1')
            self.assertEqual(y.pathInfo, '/1/2/')

            x = y.parentRoute
            self.assertEqual(x.o, obj)
            self.assertEqual(x.x, '1')
            self.assertEqual(x.pathInfo, '/1/')

    def test90_ActionRoute(self):
        class foo(ActionRoute):
            pass
        self.assertEqual(foo.match, 'foo')
        self.assertEqual(foo.action.value, 'foo')

    def test91_InstanceRoute(self):
        class foo(InstanceRoute):
            obj = Param(typ=Record)
        self.assertEqual(foo.match, 'foo')
        self.assertEqual(foo.action.value, 'foo')

        r = foo.route_for(Record(), 'foo')
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/foo')

    def test92_ViewRoute(self):
        class foo(ViewRoute):
            obj = Param(typ=Record)
        self.assertEqual(foo.match, '')
        self.assertEqual(foo.action.value, 'foo')

        obj = Record()

        r = foo.route_for(obj)
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/')
        self.assertEqual(r.obj, obj)

        r = foo.route_for(obj, 'foo')
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/')
        self.assertEqual(r.obj, obj)

    def test95_ClassRoute(self):
        class foo(ClassRoute):
            clsObj = Const(Record)
        self.assertEqual(foo.match, 'foo')
        self.assertEqual(foo.action.value, 'foo')

        r = foo.route_for(Record, 'foo')
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/foo')
        self.assertEqual(r.clsObj, Record)

    def test96_ListRoute(self):
        class foo(ListRoute):
            clsObj = Const(Record)
        self.assertEqual(foo.match, '')
        self.assertEqual(foo.action.value, 'foo')

        r = foo.route_for(Record, 'foo')
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/')

        r = foo.route_for(Record)
        self.failUnless(r.__class__ is foo)
        self.assertEqual(r.pathInfo, '/')
        self.assertEqual(r.clsObj, Record)





class User(Record):
    pass

U1, U2, U3 = User(uid=1), User(uid=2), User(uid=3)
UserDB = {1:U1, 2:U2, 3:U3}

class UserSitemap(Route):
    match = 'user/'
    handler = Const('UserHandler')
    clsObj = Const(User)
    class list(ListRoute):
        pass
    class add(ClassRoute):
        pass
    class user(Route):
        match = ':oid/'
        oid = Param(lambda s: str(s.obj.uid))
        obj = Param(lambda s: UserDB[int(s.oid)], typ=User)
        class view(ViewRoute):
            pass
        class edit(InstanceRoute):
            pass
        class delete(InstanceRoute):
            pass

class TestUserSitemap(TestCase):

    def test30_TraverseNonexistentRoute(self):
        r = UserSitemap.traverse('xxx')
        self.assertEqual(r, None)

    def test31_TraverseEmptyRoute(self):
        r = UserSitemap.traverse('user/')
        assert r.__class__ is UserSitemap._child_route('list')
        self.assertEqual(r.pathInfo, '/user/')
        self.assertEqual(r.handler, 'UserHandler')
        self.assertEqual(r.clsObj, User)
        self.assertEqual(r.action, 'list')

    def test32_TraverseStaticRoute(self):
        r = UserSitemap.traverse('user/add')
        assert r.__class__ is UserSitemap._child_route('add')
        self.assertEqual(r.pathInfo, '/user/add')
        self.assertEqual(r.handler, 'UserHandler')
        self.assertEqual(r.clsObj, User)

    def test33_TraverseParametrizedRoute(self):
        r = UserSitemap.traverse('user/1/')
        assert r.__class__ is UserSitemap._child_route('user/view')
        self.assertEqual(r.pathInfo, '/user/1/')
        self.assertEqual(r.obj, U1)
        self.assertEqual(r.oid, '1')
        self.assertEqual(r.handler, 'UserHandler')
        self.assertEqual(r.clsObj, User)

        r = UserSitemap.traverse('user/1/edit')
        assert r.__class__ is UserSitemap._child_route('user/edit')
        self.assertEqual(r.pathInfo, '/user/1/edit')
        self.assertEqual(r.obj, U1)
        self.assertEqual(r.handler, 'UserHandler')
        self.assertEqual(r.clsObj, User)

    def test40_RouteForNonexistentSignature(self):
        self.assertEqual(UserSitemap.route_for(object), None)

    def test41_RouteForStatic(self):
        self.assertEqual(UserSitemap.route_for(User).pathInfo, '/user/')
        self.assertEqual(UserSitemap.route_for(User,'list').pathInfo, '/user/')
        self.assertEqual(UserSitemap.route_for(User,'add').pathInfo,'/user/add')
        self.assertEqual(UserSitemap.route_for(User,'foo'), None)

    def test42_RouteForParametrized(self):
        self.assertEqual(UserSitemap.route_for(U1).pathInfo, '/user/1/')
        self.assertEqual(UserSitemap.route_for(U1,'view').pathInfo, '/user/1/')
        self.assertEqual(
            UserSitemap.route_for(U1,'edit').pathInfo,'/user/1/edit')
        self.assertEqual(UserSitemap.route_for(U1, 'foo'), None)

    def test50_ParentRoutes(self):
        e1 = UserSitemap.traverse('user/1/edit')
        e2 = UserSitemap.route_for(U1,'edit')

        p1 = e1.parentRoute
        p2 = e2.parentRoute
        assert isinstance(p1, UserSitemap._child_route('user'))
        assert isinstance(p2, UserSitemap._child_route('user'))
        self.assertEqual(p1.pathInfo, '/user/1/')
        self.assertEqual(p2.pathInfo, '/user/1/')

        q1 = p1.parentRoute
        q2 = p2.parentRoute
        assert isinstance(q1, UserSitemap)
        assert isinstance(q2, UserSitemap)
        self.assertEqual(q1.pathInfo, '/user/')
        self.assertEqual(q2.pathInfo, '/user/')

        self.assertEqual(q1.parentRoute, None)
        self.assertEqual(q2.parentRoute, None)

    def test60_RelativeRoutes(self):
        r = UserSitemap.traverse('/user/1/') / './edit'
        self.assertEqual(r.pathInfo, '/user/1/edit')
        r = UserSitemap.traverse('/user/1/edit') / './delete'
        self.assertEqual(r.pathInfo, '/user/1/delete')
        r = UserSitemap.traverse('/user/1/edit') / '../1/delete'
        self.assertEqual(r.pathInfo, '/user/1/delete')
        r = UserSitemap.traverse('/user/1/edit') / '../1/'
        self.assertEqual(r.pathInfo, '/user/1/')
        r = UserSitemap.traverse('/user/1/') / './'
        self.assertEqual(r.pathInfo, '/user/1/')
        r = UserSitemap.traverse('/user/1/') / '..'
        self.assertEqual(r.pathInfo, '/user/')











class TestConstraints(TestCase):

    def test10_Equal(self):
        class R(Route):
            oid = Param(typ=int)
            iseven = Param(lambda s: s.oid % 2 == 0, typ=bool, equalTo='even')
            even = Const(True)
        r = R(dict(oid=12))
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.even, True)
        self.assertRaises(SitemapError, R, {'oid':13})

    def test12_EqualInHierarchy(self):
        class R(Route):
            even = Const(True)
            class N(Route):
                oid = Param(typ=int)
                even = Param(lambda s: s.oid%2 == 0,typ=bool,equalTo='../even')
        N = R._child_route('N')
        r = N(dict(oid=12))
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.even, True)
        self.assertEqual(r.parentRoute.even, True)
        self.assertRaises(SitemapError, N, {'oid':13})

    def test20_DispatchByConstraint(self):
        class nums(Route):
            match = 'nums/'
            grp = None # required
            class obj(Route):
                match = ':oid/'
                oid = Param(typ=int)
                grp = Param(lambda s: s.oid % 2, typ=int, equalTo='../grp')
                class view(Route):
                    match = 'view'
                    signature = Signature('oid')
        class S(Route):
            class even(nums):
                match = 'even/'
                grp = Const(0)
            class odd(nums):
                match = 'odd/'
                grp = Const(1)
        e1 = S.traverse('/even/24/view')
        self.assertEqual(e1.pathInfo, '/even/24/view')
        assert isinstance(e1, S._child_route('even/obj/view'))
        o1 = S.traverse('/odd/23/view')
        self.assertEqual(o1.pathInfo, '/odd/23/view')
        assert isinstance(o1, S._child_route('odd/obj/view'))
        e1 = S.route_for(24)
        self.assertEqual(e1.pathInfo, '/even/24/view')
        assert isinstance(e1, S._child_route('even/obj/view'))
        o1 = S.route_for(23)
        self.assertEqual(o1.pathInfo, '/odd/23/view')
        assert isinstance(o1, S._child_route('odd/obj/view'))
        self.assertRaises(SitemapError, S.traverse, '/even/23/view')
        self.assertRaises(SitemapError, S.traverse, '/odd/24/view')

    def XXXtest40_SimpleCmsSitemap(self):
        class Document(Record): pass
        cmsdb = {
            41 : Document(oid=41, partition='mistopis'),
            42 : Document(oid=42, partition='obec.12'),
            43 : Document(oid=43, partition='oblast.13'),
            44 : Document(oid=44, partition='aktivity'),
        }
        class Obec(Record): pass
        obecdb = {
            12 : Obec(oid=12),
        }
        class Oblast(Record): pass
        oblastdb = {
            13 : Oblast(oid=13),
        }
        class Partition(Record): pass

        class CMS(Route):
            match = 'cms/'
            partition = None
            mgr = Param(
                lambda s: Partition(partition=s.partition), typ=Partition)
            part_check = Param(lambda s: s.mgr.partition, equalTo='partition')

            class ADD(Route):
                match = 'add'
                action = Const('add')
                signature = Signature('mgr', 'action')

            class DOC(Route):
                match     = ':inode/'
                inode     = Param(lambda s: s.obj.oid, typ=int)
                obj       = Param(lambda s: cmsdb[s.inode], typ=Document)
                partition = Param(lambda s: s.obj.partition,
                            equalTo='../partition')
                class VIEW(Route):
                    match = ''
                    signature = Signature('obj')

        class X(Route):

            class MISTOPIS(Route):
                match = 'mistopis/'
                class C(CMS):
                    partition = Const('mistopis')

            class OBEC(Route):
                match = 'obec/:oid/'
                oid   = Param(lambda s: s.obj.oid, typ=int)
                obj   = Param(lambda s: obecdb[s.oid], typ=Obec)
                prefix = Const('obec')
                signature = Signature('obj')
                class C(CMS):
                    rootObj = Acquire('../obj')
                    #partition = Param(lambda s: 'obec.%d' % s.oid, typ=str)
                    partition = Param(
                        lambda s: '%s.%d'%(s.parentRoute.prefix,s.oid),typ=str)
                    prefix = Param(
                        lambda s: s.partition.split('.')[0],
                        equalTo='../prefix',
                    )
                    oid = Param(
                        lambda s: int(s.partition.split('.')[1]),
                        equalTo='../oid',
                    )

            class OBLAST(Route):
                match  = 'oblast/:oid/'
                oid    = Param(lambda s: s.obj.oid, typ=int)
                obj    = Param(lambda s: oblastdb[s.oid], typ=Oblast)
                prefix = Const('oblast')
                signature = Signature('obj')
                class C(CMS):
                    rootObj = Acquire('../obj')
                    #partition = Param(lambda s: 'oblast.%d' % s.oid, typ=str)
                    partition = Param(
                        lambda s: '%s.%d'%(s.parentRoute.prefix,s.oid),typ=str)
                    prefix = Param(
                        lambda s: s.partition.split('.')[0],
                        equalTo='../prefix')
                    _oid = Param(
                        lambda s: int(s.partition.split('.')[1]),
                        equalTo='../oid',
                    )

            class AKTIVITY(Route):
                match = 'aktivity/'
                class C(CMS):
                    partition = Const('aktivity')

        r = X.route_for(123)
        self.assertEqual(r, None)
        u = X.get_url(123)
        self.assertEqual(u, None)

        r = X.route_for(obecdb[12])
        self.assertEqual(r.pathInfo, '/obec/12/')
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.obj, obecdb[12])
        r = X.traverse('/obec/12/')
        self.assertEqual(r.pathInfo, '/obec/12/')
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.obj, obecdb[12])

        r = X.route_for(oblastdb[13])
        self.assertEqual(r.pathInfo, '/oblast/13/')
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.obj, oblastdb[13])
        r = X.traverse('/oblast/13/')
        self.assertEqual(r.pathInfo, '/oblast/13/')
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.obj, oblastdb[13])

        r = X.traverse('/mistopis/cms/41/')
        self.assertEqual(r.pathInfo, '/mistopis/cms/41/')
        self.assertEqual(r.obj, cmsdb[41])
        self.assertEqual(r.inode, 41)
        self.assertEqual(r.partition, 'mistopis')
        self.assertEqual(r.mgr, Partition(partition='mistopis'))

        r = X.route_for(cmsdb[41])
        self.assertEqual(r.pathInfo, '/mistopis/cms/41/')
        self.assertEqual(r.obj, cmsdb[41])
        self.assertEqual(r.inode, 41)
        self.assertEqual(r.partition, 'mistopis')
        self.assertEqual(r.mgr, Partition(partition='mistopis'))

        r = X.traverse('/mistopis/cms/add')
        self.assertEqual(r.pathInfo, '/mistopis/cms/add')
        self.assertEqual(r.partition, 'mistopis')
        self.assertEqual(r.mgr, Partition(partition='mistopis'))

        r = X.route_for(Partition(partition='mistopis'), 'add')
        self.assertEqual(r.pathInfo, '/mistopis/cms/add')
        self.assertEqual(r.partition, 'mistopis')
        self.assertEqual(r.mgr, Partition(partition='mistopis'))

        r = X.traverse('/obec/12/cms/42/')
        self.assertEqual(r.pathInfo, '/obec/12/cms/42/')
        self.assertEqual(r.obj, cmsdb[42])
        self.assertEqual(r.inode, 42)
        self.assertEqual(r.rootObj, obecdb[12])
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.partition, 'obec.12')
        self.assertEqual(r.mgr, Partition(partition='obec.12'))

        r = X.route_for(cmsdb[42])
        self.assertEqual(r.pathInfo, '/obec/12/cms/42/')
        self.assertEqual(r.obj, cmsdb[42])
        self.assertEqual(r.inode, 42)
        self.assertEqual(r.rootObj, obecdb[12])
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.partition, 'obec.12')
        self.assertEqual(r.mgr, Partition(partition='obec.12'))

        r = X.traverse('/obec/12/cms/add')
        self.assertEqual(r.pathInfo, '/obec/12/cms/add')
        self.assertEqual(r.partition, 'obec.12')
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.obj, obecdb[12])
        self.assertEqual(r.mgr, Partition(partition='obec.12'))

        r = X.route_for(Partition(partition='obec.12'), 'add')
        self.assertEqual(r.pathInfo, '/obec/12/cms/add')
        self.assertEqual(r.partition, 'obec.12')
        self.assertEqual(r.oid, 12)
        self.assertEqual(r.obj, obecdb[12])
        self.assertEqual(r.mgr, Partition(partition='obec.12'))

        r = X.traverse('/oblast/13/cms/43/')
        self.assertEqual(r.pathInfo, '/oblast/13/cms/43/')
        self.assertEqual(r.obj, cmsdb[43])
        self.assertEqual(r.inode, 43)
        self.assertEqual(r.rootObj, oblastdb[13])
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.partition, 'oblast.13')
        self.assertEqual(r.mgr, Partition(partition='oblast.13'))

        r = X.route_for(cmsdb[43])
        self.assertEqual(r.pathInfo, '/oblast/13/cms/43/')
        self.assertEqual(r.obj, cmsdb[43])
        self.assertEqual(r.inode, 43)
        self.assertEqual(r.rootObj, oblastdb[13])
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.partition, 'oblast.13')
        self.assertEqual(r.mgr, Partition(partition='oblast.13'))

        r = X.traverse('/oblast/13/cms/add')
        self.assertEqual(r.pathInfo, '/oblast/13/cms/add')
        self.assertEqual(r.partition, 'oblast.13')
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.obj, oblastdb[13])
        self.assertEqual(r.mgr, Partition(partition='oblast.13'))

        r = X.route_for(Partition(partition='oblast.13'), 'add')
        self.assertEqual(r.pathInfo, '/oblast/13/cms/add')
        self.assertEqual(r.partition, 'oblast.13')
        self.assertEqual(r.oid, 13)
        self.assertEqual(r.obj, oblastdb[13])
        self.assertEqual(r.mgr, Partition(partition='oblast.13'))

        r = X.traverse('/aktivity/cms/44/')
        self.assertEqual(r.pathInfo, '/aktivity/cms/44/')
        self.assertEqual(r.obj, cmsdb[44])
        self.assertEqual(r.inode, 44)
        self.assertEqual(r.partition, 'aktivity')
        self.assertEqual(r.mgr, Partition(partition='aktivity'))

        r = X.route_for(cmsdb[44])
        self.assertEqual(r.pathInfo, '/aktivity/cms/44/')
        self.assertEqual(r.obj, cmsdb[44])
        self.assertEqual(r.inode, 44)
        self.assertEqual(r.partition, 'aktivity')
        self.assertEqual(r.mgr, Partition(partition='aktivity'))

        # XXX should raise SitemapError
        self.assertRaises(SitemapError, X.traverse, '/mistopis/cms/44/')
        self.assertRaises(SitemapError, X.traverse, '/obec/13/cms/42/')
        self.assertRaises(SitemapError, X.traverse, '/obec/12/cms/43/')
        self.assertRaises(SitemapError, X.traverse, '/aktivity/cms/42/')

    def test80_AmbiguousRouteFor(self):
        class A(Route):
            class R1(Route):
                p = Param(typ=int)
                signature = Signature('p')
            class R2(Route):
                q = Param(typ=int)
                signature = Signature('q')
        self.assertRaises(dispatch.AmbiguousMethod, A.route_for, 42)

    def test90_ExplicitWhen(self):
        # XXX Deprecated
        class SM(Route):
            match = ':oid/'
            oid = Param(lambda s: str(s.obj.uid))
            obj = Param(lambda s: UserDB[int(s.oid)], typ=User)
            clsObj = Const(User)
            class r1(Route):
                match = 'r1'
                signature = Signature('obj',when='obj.uid == 1')
            class r2(Route):
                match = 'r2'
                signature = Signature('obj',when='obj.uid == 2')


        r = SM.route_for(U1)
        self.assertEqual(r.pathInfo, '/1/r1')
        self.assertEqual(r.oid, '1')
        self.assertEqual(r.obj, U1)
        self.assertEqual(r.clsObj, User)
        p = r.parentRoute
        assert isinstance(p, SM)
        self.assertEqual(p.pathInfo, '/1/')
        self.assertEqual(p.oid, '1')
        self.assertEqual(p.obj, U1)
        self.assertEqual(p.clsObj, User)

        r = SM.route_for(U2)
        self.assertEqual(r.pathInfo, '/2/r2')
        self.assertEqual(r.oid, '2')
        self.assertEqual(r.obj, U2)
        self.assertEqual(r.clsObj, User)
        p = r.parentRoute
        assert isinstance(p, SM)
        self.assertEqual(p.pathInfo, '/2/')
        self.assertEqual(p.oid, '2')
        self.assertEqual(p.obj, U2)
        self.assertEqual(p.clsObj, User)

        r = SM.route_for(U3)
        self.assertEqual(r, None)





























GV = 'foo'

class TestBackwardCompatibility(WebCase):

    def test10_ExplicitUrlForParser(self):
        LV = 'bar'
        class S(Route):
            @url_for('a == GV')
            def ugv(a):
                return 'gv'

            @url_for('a == LV')
            def ulv(a):
                return 'lv'

        r = S.route_for('foo')
        self.assertEqual(r.pathInfo, '/gv')
        r = S.route_for('bar')
        self.assertEqual(r.pathInfo, '/lv')
        r = S.route_for(3.4)
        self.assertEqual(r, None)

    def test12_ExplicitUrlForWithDefaultValues(self):
        class S(Route):
            @url_for('a in int')
            def uint(a, b='B', c='C', d='D'):
                return '%s/%s/%s/%s' % (a,b,c,d)
        r = S.route_for(1)
        self.assertEqual(r.pathInfo, '/1/B/C/D')
        r = S.route_for(1, 'b')
        self.assertEqual(r.pathInfo, '/1/b/C/D')
        r = S.route_for(1, 'b', 'c')
        self.assertEqual(r.pathInfo, '/1/b/c/D')
        r = S.route_for(1, 'b', 'c', 'd')
        self.assertEqual(r.pathInfo, '/1/b/c/d')
        r = S.route_for(3.4)
        self.assertEqual(r, None)

    def test14_UrlForUniqueness(self):
        class I(Route):
            @url_for('a in int')
            def uint(a):
                return 'int/%s' % a
        class S(Route):
            class i1(I): pass
            class i2(I): pass
        r = S.route_for(1)
        self.assertEqual(r.pathInfo, '/int/1')

    def test20_UrlFor(self):
        loc = Location(a=42)
        loc.url_for(
            'o in int',  lambda o: 'int/%d' % o)
        loc.url_for(
            'o in float',lambda o: 'float/%.2f' % o)
        loc.url_for(
            'o in str',  lambda o,a='v': '/str/%s/%s' % (o,a!='v'and a or ''))

        s = loc.makeRoute()
        r = s.route_for(1)
        self.assertEqual(r.pathInfo, '/int/1')
        r = s.route_for(3.14)
        self.assertEqual(r.pathInfo, '/float/3.14')
        r = s.route_for('foo')
        self.assertEqual(r.pathInfo, '/str/foo/')
        r = s.route_for('foo', 'v')
        self.assertEqual(r.pathInfo, '/str/foo/')
        r = s.route_for('foo', 'e')
        self.assertEqual(r.pathInfo, '/str/foo/e')

    def test30_Route(self):
        loc = Location(defaultA=42, defaultB='Bee')
        loc.route('i/:obj/', 'IntH', None, action='view')
        loc.route('i/:obj/edit', 'IntH', None, action='edit')
        loc.route('f/:obj/', 'FloatH', 12, action='view')
        s = loc.makeRoute()

        r = s.traverse('/i/42/edit')
        self.assertEqual(r.pathInfo, '/i/42/edit')
        self.assertEqual(r.get_params(), dict(
            defaultA=42, defaultB='Bee',
            handler='IntH', action='edit', obj='42', permission=None
        ))

        r = s.traverse('/f/3.14/')
        self.assertEqual(r.pathInfo, '/f/3.14/')
        self.assertEqual(r.get_params(), dict(
            defaultA=42, defaultB='Bee',
            handler='FloatH', action='view', obj='3.14', permission=12
        ))









class TestSitemapMiddleware(WebCase):

    def siteapp(self, app):
        app = self.mkapp(
            'environment', 'errors', 'transaction', 'cookies',
            'http_exceptions', 'session', 'authentication', 'resources',
            'sitemap', funcMiddleware(app),
        )
        return app

    @with_ini((
        "[WSGI Middleware sitemap]\n"
        "sitemap = importString('%s.UserSitemap')\n") % (__name__))
    def test10_Traversal(self):
        r = []
        def get_route(env, start):
            start('200', [])
            r.append(env['windmill.route'])
            return  ['OK']
        app = self.siteapp(get_route)
        env = self.env('http://localhost/user/1/edit')
        s,h,b = self.handle(app, env)
        self.assertEqual(s, '200')
        self.assertEqual(h, [])
        self.assertEqual(b, 'OK')
        self.assertEqual(len(r), 1)
        route = r[0]
        assert isinstance(route, UserSitemap._child_route('user/edit'))
        self.assertEqual(route.pathInfo, '/user/1/edit')

    @with_ini((
        "[WSGI Middleware sitemap]\n"
        "sitemap = importString('%s.UserSitemap')\n") % (__name__))
    def test20_NotFound(self):
        r = []
        def get_route(env, start):
            start('200', [])
            r.append(env['windmill.route'])
            return  ['OK']
        app = self.siteapp(get_route)
        env = self.env('http://localhost/foo')
        s,h,b = self.handle(app, env)
        self.assertEqual(s, '404 Not Found')
        #self.assertEqual(h, [('Content-type', 'text/plain')])
        #self.assertEqual(b, '404 Not Found\r\n')


def suite():
    return TestSuite((
        makeSuite(TestRoutePattern),
        makeSuite(TestRoutes),
        makeSuite(TestUserSitemap),
        makeSuite(TestConstraints),
        makeSuite(TestBackwardCompatibility),
        makeSuite(TestSitemapMiddleware),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
