# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from wm4.web.templates import *
from wm4.web.interfaces import NSDomletProperty as DP
from wm4.config.ini_files import ConfigReader
from lib4.testing import *
from xml4.elements import Literal
from xml4.templates import Deferred
from os import path

from utils import *

this_dir = path.realpath(path.abspath(path.join(path.split(__file__)[0])))


TNS = 'http://windmill.cz/'

class TestConfig(WebCase):

    def parse(self, lines):
        cr = ConfigReader()
        cr.readString(lines)

    def test10_Property(self):
        p = DP(TNS, 'input', 12)
        self.assertEqual(p(), 12)

    def test20_IniFile(self):
        self.parse((
            '[Domlets For Namespace %s]\n'
            'input = 10\n'
            'input.* = 11\n'
            'form.* = 12\n'
            '[Domlets For Namespace foo@example.com]\n'
            'bar = "baz"\n'
        ) % TNS)
        self.assertEqual(DP(TNS,'input')(), 10)
        self.assertEqual(DP(TNS,'input.aa')(), 11)
        self.assertEqual(DP(TNS,'form.aa')(), 12)
        self.assertRaises(exceptions.PropertyNotFound, DP(TNS,'neex'))
        self.assertRaises(exceptions.PropertyNotFound, DP('NEEX','neex'))
        self.assertEqual(DP('foo@example.com','bar')(), 'baz')





class TestTemplates(WebCase):

    def test10_TemplateAsResource(self):
        t = web.resource_manager().getResource('wm4.web.tests.res/tmpl1.wt')
        self.assertEqual(t.__class__, Template)
        self.assertEqual(
            path.normcase(path.realpath(path.abspath(t.filename))),
            path.normcase(path.join(this_dir,'res', 'tmpl1.wt'))
        )

    def test11_HTTPHandle(self):
        def handle(environ, start):
            req = web.Request(environ=environ)
            t = req.getResource('wm4.web.tests.res/tmpl1.wt')
            r = t.handleHTTP(req)
            start(r.status, r.headers)
            return r.body

        app = self.mkapp(
            'environment', 'errors', 'transaction', 'cookies',
            'http_exceptions', 'session', 'authentication', 'resources',
            funcMiddleware(handle)
        )
        s,h,b = self.handle(app)

        o = U2UTF(
          u'\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" '
                   u'"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n'
          u'<html xmlns="http://www.w3.org/1999/xhtml">\n'
          u'<head>\n'
          u'  <title>Template Test</title>\n'
          u'</head>\n'
          u'<body>\n'
          u'  <h1>Template Test</h1>\n  \n'
          u'  <p>czech:&nbsp;ZKOUKA ETINY</p>\n'
          u'  <ul>\n'
          u'    <li>item 1</li><li>item 2</li><li>item 4</li><li>item 5</li>\n'
          u'  </ul>\n'
          u'</body>\n'
          u'</html>\n')

        self.assertEqual(s, '200 OK')
        self.assertEqual(h, [
            ('Content-type', 'text/html;charset=utf-8'),
            ('Content-Length', str(len(o)))
        ])
        self.assertEqual(b, o)

    def test20_Fragment(self):
        f = web.parseSrc('<p py:def="foo()">FOO</p>')
        self.assertEqual(
            self.render('<d py:content="foo()"/>', foo=f.tmpl_namespace['foo']),
            '<d><p>FOO</p></d>',
        )



class TestLayouts(WebCase):

    class layout_test(ctest):
        layout = None
        page = None
        result = None
        def namespace(self):
            return {}
        def do(self, case):
            ns = self.namespace()
            pl = case.parse(self.layout, tmpl_namespace=ns)
            ns['layout'] = pl
            pp = case.parse(self.page, tmpl_namespace=ns)

            def handle(environ, start):
                req = web.Request(environ=environ)
                r = pp.handleHTTP(req)
                start(r.status, r.headers)
                return r.body

            app = case.mkapp(
                'environment', 'errors', 'transaction', 'cookies',
                'http_exceptions', 'session', 'authentication', 'resources',
                funcMiddleware(handle)
            )
            s,h,b = case.handle(app)
            case.assertEqual(s, '200 OK')
            case.assertEqual(''.join(b), self.result)


    class test20_Layout(layout_test):
        layout = (
            '<html>'
              '<head><title py:content="title()">TITLE</title></head>'
              '<body><div py:replace="body()">BODY</div></body>'
            '</html>')
        page = (
            '<doc py:layout="layout">'
              '<x py:def="title()" py:strip="">Test</x>'
              '<y py:def="body()" py:strip="">Body <r>R</r></y>'
            '</doc>')
        result = (
        '<html>'
          '<head><title>Test</title></head>'
          '<body>Body <r>R</r></body>'
        '</html>')

    class test20_LayoutWithExtendedPage(layout_test):
        def namespace(self):
            macros = self.case.parse(
                '<m>'
                  '<t py:def="title()" py:strip="">Test</t>'
                  '<b py:def="body()" py:strip="">Nobody</b>'
                  '<r py:def="f()">R</r>'
                '</m>')
            return {'macros':macros}
        layout = (
            '<html>'
              '<head><title py:content="title()">TITLE</title></head>'
              '<body><div py:replace="body()">BODY</div></body>'
            '</html>')
        page = (
            '<doc py:extends="macros" py:layout="layout">'
             '<y py:def="body()" py:strip="">Body <f py:replace="f()"/></y>'
            '</doc>')
        result = (
            '<html>'
              '<head><title>Test</title></head>'
              '<body>Body <r>R</r></body>'
            '</html>')

    class test23_LayoutIgnoresPageDocElements(layout_test):
        layout = (
            '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" '
            '          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
            '<html>'
              '<head><title py:content="title()">TITLE</title></head>'
              '<body><div py:replace="body()">BODY</div></body>'
            '</html>')
        page = (
            '<?xml version="1.0" encoding="iso-8859-2"?>'
            '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" '
            '          "http://www.w3.org/TR/html4/frameset.dtd">'
            '<?py title = lambda: u"Test" ?>\n'
            '<?py x=12?>\n'
            '<doc py:layout="layout">'
              '<y py:def="body()" py:strip="">Body <r>R</r></y>'
            '</doc>\n')
        result = (
            '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"'
            ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
            '<html>'
              '<head><title>Test</title></head>'
              '<body>Body <r>R</r></body>'
            '</html>')


class TestPageResources(WebCase):

    def test10_JavaScriptResource(self):
        html = self.render(
            '<html>\n'
            '<head>\n'
            '<span wm:resource.container="" />\n'
            '</head>\n'
            '<body>\n'
            '<p wm:resource="\'wm4.web.static/js/x_core.js\'" />'
            '<p wm:resource="\'wm4.web.static/js/x_event.js\'" />'
            '<p wm:resource="\'wm4.web.static/js/x_core.js\'" />'
            '<p wm:resource="\'wm4.web.static/css/tools.css\'" />\n'
            '<p wm:resource="\'wm4.web.static/img/9x9/plus.gif\'" />\n'
            '</body>\n'
            '</html>\n'
        )
        self.assertEqualXML(html,
            '<html>\n'
            '<head>\n'
            '<script type="text/javascript" '
                     'src="/++res++/wm4.web.static/js/x_core.js"></script>\n'
            '<script type="text/javascript" '
                     'src="/++res++/wm4.web.static/js/x_event.js"></script>\n'
            '<link href="/++res++/wm4.web.static/css/tools.css" '
                   'rel="stylesheet" media="all" type="text/css" />\n'
            '\n'
            '</head>\n'
            '<body>\n'
            '\n'
            '<img width="9" height="9" src="/++res++/wm4.web.static/img/9x9/plus.gif" />\n'
            '</body>\n'
            '</html>\n'
        )

class TestDeferredRendering(WebCase):

    def test1(self):
        s = {}
        h = self.render(
            '<p><c>${c()}</c><d>${d(3)}</d></p>',
            c=lambda: Deferred(lambda: repr(s)),
            d=lambda x: s.setdefault('d',x)
        )
        self.assertEqual(h, "<p><c>{'d': 3}</c><d>3</d></p>")




def suite():
    return TestSuite((
        makeSuite(TestConfig),
        makeSuite(TestTemplates),
        makeSuite(TestLayouts),
        makeSuite(TestPageResources),
        makeSuite(TestDeferredRendering),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
