# -*- coding: ISO-8859-2 -*-

from unittest import *
from wm4.api import *
from lib4.trees import *
from wm4.web.trees import *

class N(AbstractTreeNode):
    parent = None
    def __init__(self, id, *children):
        self.id = id
        self.children = children
        for c in children:
            c.parent = self
    def getNodeId(self):
        return str(self.id)
    def __getitem__(self, idx):
        return self.children[idx]
    def __repr__(self):
        return 'N(%s)' % repr(self.id)

t1  = N(1)
tA = N(1,
        N(11,
            N(111),
            N(112)),
        N(12,
            N(121,
                N(1211))),
        N(13))
tL = N(1, N(11, N(111, N(1111, N(11111)))))



class TestTreeState(TestCase):

    def test10_NodeStateInstance(self):
        ts = TreeState(node=tA)
        self.assertEqual(ts.getNodeId(), '1')
        self.assertEqual(ts.children[0].getNodeId(), '11')
        self.failUnless(isinstance(ts.children[0], TreeNodeState))
        self.assertEqual(ts.children[0].children[1].getNodeId(), '112')
        self.assertEqual(ts.children[1].getNodeId(), '12')
        self.assertEqual(len(ts.children[2].children), 0)

    def test20_MarkSelectedByIDs(self):
        ts = TreeState(node=tA)
        ts.markSelectedByIDs(['121'])
        T,F = True, False

        self.assertEqual(ts.selected, F)
        self.assertEqual(ts.selectedPath, T)

        self.assertEqual(ts.children[0].selected, F)
        self.assertEqual(ts.children[0].selectedPath, F)
        self.assertEqual(ts.children[0].children[0].selected, F)
        self.assertEqual(ts.children[0].children[0].selectedPath, F)

        self.assertEqual(ts.children[1].selected, F)
        self.assertEqual(ts.children[1].selectedPath, T)
        self.assertEqual(ts.children[1].children[0].selected, T)
        self.assertEqual(ts.children[1].children[0].selectedPath, T)
        self.assertEqual(ts.children[1].children[0].children[0].selected, F)
        self.assertEqual(ts.children[1].children[0].children[0].selectedPath, F)

    def test21_MarkSelectedByNodes(self):
        ts = TreeState(node=tA)
        ts.markSelectedByNodes([tA[1][0]])
        T,F = True, False

        self.assertEqual(ts.selected, F)
        self.assertEqual(ts.selectedPath, T)

        self.assertEqual(ts.children[0].selected, F)
        self.assertEqual(ts.children[0].selectedPath, F)
        self.assertEqual(ts.children[0].children[0].selected, F)
        self.assertEqual(ts.children[0].children[0].selectedPath, F)

        self.assertEqual(ts.children[1].selected, F)
        self.assertEqual(ts.children[1].selectedPath, T)
        self.assertEqual(ts.children[1].children[0].selected, T)
        self.assertEqual(ts.children[1].children[0].selectedPath, T)
        self.assertEqual(ts.children[1].children[0].children[0].selected, F)
        self.assertEqual(ts.children[1].children[0].children[0].selectedPath, F)


def suite():
    return TestSuite((
        makeSuite(TestTreeState),
    ))


if __name__ == '__main__':
    main(defaultTest='suite')
