from unittest import TestCase
from wm4.api import *
from urllib import quote
from lib4.testing import Record
from lib4.imports import importObject
from lib4.cgi import *
import difflib, os
from os.path import join, dirname, abspath, normpath
from wm4.web.templates import *


__all__ = [
    'IL2', 'IL2UTF', 'U2UTF', 'THIS_DIR', 'RES_DIR', 'strip_semicol',
    'test_environ', 'test_req', 'with_req', 'with_ini',
    'funcMiddleware', 'WebCase', 'Record',
    'TestUpload', 'LoggingTransactionComponent'
]

THIS_DIR = normpath(abspath(dirname(__file__)))
RES_DIR = join(THIS_DIR, 'res')


IL2    = lambda s: unicode(s,'iso-8859-2')
IL2UTF = lambda s: unicode(s,'iso-8859-2').encode('utf-8')
U2UTF  = lambda u: unicode(u).encode('utf-8')


def strip_semicol(headers):
    res = []
    for h,v in headers:
        if h.lower() == 'set-cookie':
            v = v.rstrip(';')
        res.append((h,v))
    return res

class LoggingTransactionComponent(storage.TransactionComponent):

    log = binding.Make(list)

    def onJoinTxn(self, txnService):
        self.log.append('begin')

    def finishTransaction(self, txnService, committed):
        if committed:
            self.log.append('commit')
        else:
            self.log.append('abort')

def test_req(url='/',env={},get={},post={},cookies={},auth=None):
    env,inp = test_environ(url,env,get,post,cookies,auth)
    sane_environ(env)
    return web.Request(environ=env)


def with_req(url='/',env={},get={},post={},cookies={},auth=None):
    def req_wrapper(func):
        def f(*a, **k):
            req = test_req(url,env,get,post,cookies,auth)
            exit = config.enter_(web.request(req))
            try:
                return func(*a, **k)
            finally:
                exit()
        f.__name__ = func.__name__
        return f
    return req_wrapper


def with_ini(ini, **add_globals):
    u = 'data:,' + quote(ini)
    def ini_wrapper(func):
        def f(*a, **k):
            config.loadConfig(u, **add_globals)
            return func(*a, **k)
        f.__name__ = func.__name__
        return f
    return ini_wrapper


def funcMiddleware(func):
    def middleware(application):
        return func
    return middleware


class WebCase(TestCase):


    cfgURLs = ('pkgfile:wm4/core.ini', 'pkgfile:wm4.web/web.ini')

    def setUp(self):
        self.state = config.snapshot()
        config.loadConfig(*self.cfgURLs)

    def tearDown(self):
        config.swap(self.state)

    def parse_ini(self, lines):
        from wm4.config.ini_files import ConfigReader
        ConfigReader().readString(lines)

    def env(self,url='/',env={},get={},post={},cookies={},auth=None):
        env,inp = test_environ(url,env,get,post,cookies,auth)
        sane_environ(env)
        return env

    def req(self,url='/',env={},get={},post={},cookies={},auth=None):
        return test_req(url,env,get,post,cookies,auth)

    def mkapp(self, *components):
        return web.makeApp(components=components)

    def handle(self, app, env=None):
        if env is None:
            env = self.env()
        sane_environ(env)
        resp = [None,None]
        def start_response(status, response_headers, exc_info=None):
            resp[0] = status
            resp[1] = response_headers
        body = app(env, start_response)
        s = resp[0]
        h = resp[1]
        b = ''.join(body)
        return s,h,b

    def parse(self, src, tmpl_namespace={}):
        exit_ = config.enter_(config.Config())
        try:
            web.TEMPLATE_GLOBALS(tmpl_namespace)
            t = parseSrc(src)
        finally:
            exit_()
        return Template(template=t)


    def render(self, tmpl, req=None, tmpl_namespace={}, **params):
        if req is None:
            req = self.req()
        if params:
            req.params.update(params)
        if isinstance(tmpl, basestring):
            tmpl = self.parse(tmpl, tmpl_namespace=tmpl_namespace)
        else:
            assert not tmpl_namespace
        return tmpl.render(req)

    def assertEqualXML(self, first, second):
        if not first == second:
            for i, (f,s) in enumerate(map(None,
                first.splitlines(), second.splitlines()
            )):
                if f != s:
                    raise self.failureException(
                        'Markup differs on line %d %r x %r' % (i+1, f, s)
                    )
