from wm4.api import *
from lib4.trees import TreeNodeWrapper

__all__ = ['TreeNodeState', 'TreeState']


class TreeNodeState(TreeNodeWrapper):

    selected = False

    selectedPath = False


class TreeState(binding.Component, TreeNodeState):

    # Original tree (instance of some AbstractTreeNode subclass)
    node = binding.Require()

    parent = None

    def makeChildren(self):
        fact = TreeNodeState
        filt = self.filter
        return [
            fact(n, self, filt) for n in self.node.getChildren() if not filt(n)
        ]

    # Default filter

    levelLimit = None

    def filter(self):
        if self.levelLimit is not None:
            return lambda node: node.treeLevel > self.levelLimit
        else:
            return lambda node: False
    filter = binding.Make(filter)


    def idMap(self):
        return dict(
            (n.getNodeId(), n) for n in self.iterDepthFirst()
        )
    idMap = binding.Make(idMap)

    def markSelectedByIDs(self, idList):
        for id in idList:
            node = self.idMap.get(id)
            if node is None:
                continue
            node.selected = True
            for p in node.iterParents():
                p.selectedPath = True

    def markSelectedByNodes(self, nodeList):
        self.markSelectedByIDs([n.getNodeId() for n in nodeList])
