import time, os

from wm4.api import *
from xml4 import xhtml as xh

from xml4.serialization import xhtml_serializer
from xml4.templates import Domlet, Deferred
from xml4.elements import Node, Fragment, Literal
from xml4.const import *
from xml4.utils import attr_list

from publishing import render
from resources import HeaderResourceContainer
from navigation import DELIMITER

__all__ = [
    'ResourceContainerWidget', 'ResourceWidget',
    'NavigActionsWidget', 'NavigActionsWidgetUserOnly', 'NavigPathWidget',
    'DebugWidget', 'HelpWidget', 'CalendarWidget', 'ReSTWidget',
    'MetadataWidget', 'CacheWidget',
]

_ = i18n.MessageIDFactory('wm4')


class ResourceContainerWidget(Domlet):

    def expand(self, state):
        req = state['req']
        cls = HeaderResourceContainer
        if cls in req.params:
            registry = req.params[cls]
        else:
            registry = req.params[cls] = cls()
        def expand_later():
            return u''.join(unicode(res.include(req)) for res in registry)
        yield DEFER, expand_later


class ResourceWidget(Domlet):

    def expand(self, state, *resources):
        req = state['req']
        out = xh.frag()
        for res in resources:
            if isinstance(res, basestring):
                _res = res
                res = req.getResource(_res)
                if res is None:
                    raise ValueError('Resource not found', _res)
            mark = res.render(req)
            if mark is not None:
                out.append(mark)
        for e in out.expand(state):
            yield e



class NavigActionsWidget(Domlet):

    def expand(self, state,
        links=NOT_GIVEN, join=xh.span(' ', klass='wm4-none'),
        delimiter=xh.span(' | ', klass='wm4-delimiter'),
        empty=False, one=False
    ):
        req = state['req']
        attrs = self.acquireAttrs(state, ('class', 'id', 'style'))
        if 'class' in attrs:
            attrs['class'] += ' wm4-navig-actions'
        else:
            attrs['class'] = 'wm4-navig-actions'
        naa = xh.div(attrs)
        nab = xh.div(klass = 'wm4-navig-actions-right-border')

        lc = xh.div(klass = 'left-corner')
        rc = xh.div(klass = 'right-corner')
        cl = xh.div(klass = 'wm4-cleaner')

        he = xh.div(klass = 'wm4-navig-actions-header')
        he.append(lc)
        he.append(rc)
        he.append(cl)

        nab.append(he)

        nb = xh.div(klass = 'wm4-navig-actions-buttons')

        if links is NOT_GIVEN:
            nb.extend(self._children)

        else:
            links = [
                l for l in links
                   if isinstance(l,Node) or l is DELIMITER or l.visible
            ]
            if not one and len(links) == 1 and links[0].selected:
                # one links that points to currently shown page
                pass
            else:
                for i, link in enumerate(links):
                    if link is DELIMITER:
                        l = delimiter
                    elif isinstance(link, Node):
                        l = link
                    else:
                        attrs = {}
                        if link.selected:
                            if link.css:
                                attrs['class'] = 'selected %s' % link.css
                            else:
                                attrs['class'] = 'selected'
                        elif link.css:
                            attrs['class'] = link.css
                        if not isinstance(link.label, Node):
                            label = render(req, link.label)
                        else:
                            label = link.label
                        if not link.enabled:
                            l = xh.span(attrs, label)
                        else:
                            l = xh.a(attrs, label, href=link.url)
                            if link.onclick:
                                l.set('onclick', link.onclick)
                        l = xh.frag(l, ' ')
                    if i > 0:
                        nb.append(join)
                    nb.append(l)

        bg = xh.div(klass = 'wm4-navig-actions-background')
        bg.append(nb)
        nab.append(bg)

        fo = xh.div(klass = 'wm4-navig-actions-footer')
        fo.append(lc)
        fo.append(rc)
        fo.append(cl)

        nab.append(fo)
        naa.append(nab)

        if not empty and not nb:
            return iter(())
        else:
            web.getResource('wm4.web.static/css/navig.css').render(req)
            return iter(naa.expand(state))


class NavigActionsWidgetUserOnly(NavigActionsWidget):

    def expand(self, state,
        links=NOT_GIVEN, join=xh.span(' | ', klass='wm4-none'),
        delimiter=xh.span(' | ', klass='wm4-delimiter'),
        empty=False, one=False
    ):
        req = state['req']
        if not req.user:
            return iter(())
        else:
            return iter(super(NavigActionsWidgetUserOnly, self).expand(
                state, links, join, delimiter, empty, one
            ))




class NavigPathWidget(Domlet):

    def expand(self, state, links, delim=xh.frag(' ',xh.gt,'\n'), empty=False):
        req = state['req']
        attrs = self.acquireAttrs(state, ('class', 'id', 'style'))
        attrs.setdefault('class', 'wm4-navig-path')
        na = xh.div(attrs)
        for i, link in enumerate(links):
            if not link.visible:
                continue
            if not link.enabled:
                l = xh.span(render(req, link.label))
            else:
                l = xh.a(render(req, link.label), href=link.url)
                if link.onclick:
                    l.set('onclick', link.onclick)
            if link.css:
                l.set('class', link.css)
            na.append(l)
            na.append(delim)
        if not empty and not na:
            return iter(())
        else:
            web.getResource('wm4.web.static/css/navig.css').render(req)
            return iter(na.expand(state))


class DebugWidget(Domlet):

    def expand(self, state):
        dbg = xh.div({'class':'wm4-debug'})

        dbg.append(xh.h4('Locals'))
        ul = xh.ul()
        for k,v in sorted(state.items()):
            if k in ('value_of', 'defined', 'ns'):
                continue
            if v == self.gdict.get(k, self):
                continue
            ul.append(xh.li(str(k), ' = ', repr(v)[:128]))
        dbg.append(ul)

        dbg.append(xh.h4('Globals'))
        ul = xh.ul()
        for k,v in sorted(self.gdict.items()):
            ul.append(xh.li(str(k), ' = ', repr(v)[:128]))
        dbg.append(ul)

        dbg.append(xh.h4('Request'))
        req = state['req']
        dbg.append(xh.ul(
            xh.li('pathInfo = ', req.pathInfo),
            xh.li('serverURL = ', req.serverURL),
            xh.li('appURL = ', req.appURL),
            xh.li('requestURL = ', req.requestURL),
            xh.li('requestURI = ', req.requestURI),
            xh.li('accept_language = ', req.accept_language),
            xh.li('force_language = ', req.force_language),
            xh.li('url_language = ', req.url_language),
            xh.li('default_language = ', req.default_language),
            xh.li('language = ', req.language),
            xh.li('user = ', req.user),
        ))

        dbg.append(xh.h4('Params'))
        ul = xh.ul()
        for k,v in sorted(state['req'].params.items()):
            ul.append(xh.li(str(k), ' = ', repr(v)[:128]))
        dbg.append(ul)

        dbg.append(xh.h4('Environment'))
        ul = xh.ul()
        for k,v in sorted(state['req'].environ.items()):
            if k in ('REMOTE_PASSWORD',): continue
            ul.append(xh.li(str(k), ' = ', repr(v)[:128]))
        dbg.append(ul)
        return iter(dbg.expand(state))



class CalendarWidget(Domlet):

    def expand(self, state, **options):
        '''
        Available options:
            param_year, param_month, param_act_date:
                    names of URL parameters
            year, month: month displayed in calendar
            act_date:    active date
            durl:        function returning destination url when clicking on day
                         function accepts one argument - date
            daycontent:  function generating table cell content for day
            dayclass:    function generating class of day cell
        '''
        import datetime, calendar, dateutil.parser
        req = state['req']
        td = datetime.date.today()

        param_year = options.get('param_year', 'year')
        param_month = options.get('param_month', 'month')
        param_act_date = options.get('param_act_date', 'act_date')
        year = options.get('year', req.getValue(param_year, td.year))
        month = options.get('month', req.getValue(param_month, td.month))
        try:
            year, month = int(year), int(month)
            weeks = calendar.monthcalendar(year, month)
        except ValueError:
            year, month = td.year, td.month
            weeks = calendar.monthcalendar(year, month)
        try:
            act_date = dateutil.parser.parse(
                        options.get('act_date',
                                    req.getValue(param_act_date, ''))
                        ).date()
        except ValueError:
            act_date = td
        durl = options.get('durl',
                           lambda d, month=month, year=year: req.requestURL % {
                                        param_act_date: d,
                                        param_month: month,
                                        param_year: year})
        daycontent = options.get('daycontent',
                                 lambda d: xh.a(d.day,
                                                href=durl(d, month, year)))
        dayclass = options.get('dayclass',
                               lambda d: '')
        header = options.get('header', lambda m, y: '%s / %s' % (m, y))
        (next_month, next_year) = (month==12) and (1, year+1) \
                                               or (month+1, year)
        (prev_month, prev_year) = (month==1) and (12, year-1) \
                                               or (month-1, year)


        cal = xh.table(klass="calendar")

        cal.append(xh.tr(
            xh.th(xh.a('<<', href=req.requestURL % {param_month: prev_month,
                                                    param_year: prev_year,
                                                    param_act_date: act_date})),
            xh.th(header(month, year),
                  colspan="5"),
            xh.th(xh.a('>>', href=req.requestURL % {param_month: next_month,
                                                    param_year: next_year,
                                                    param_act_date: act_date}))
        ))
        cal.append(xh.tr(*[xh.td(label)
                           for label in [req.translate(_('Mo')),
                                         req.translate(_('Tu')),
                                         req.translate(_('We')),
                                         req.translate(_('Th')),
                                         req.translate(_('Fr')),
                                         req.translate(_('Sa')),
                                         req.translate(_('Su'))]],
                         **{'klass': 'day-name'}))
        def day_td(day):
            if day == 0:
                return xh.td(xh.nbsp)
            day_date = datetime.date(year, month, day)
            cl = []
            if day_date == act_date:
                cl.append('day-active')
            if day_date == td:
                cl.append('day-today')
            if day_date.weekday() in (5,6):
                cl.append('day-weekend')
            cl.append(dayclass(day_date))
            return xh.td(daycontent(day_date),
                         klass=' '.join(cl))
        for week in weeks:
            cal.append(xh.tr(*[day_td(week[i])
                               for i in range(7)]))

        attrs = self.acquireAttrs(state, ('class', 'id', 'style'))
        if 'class' in attrs:
            attrs['class'] += ' wm4-calendar'
        else:
            attrs['class'] = 'wm4-calendar'
        return iter(xh.div(cal, **attrs).expand(state))


class ReSTWidget(Domlet):

    def expand(self, state, rest):
        from xml4.rest import reST2html
        from comp4.dbgettext.api import Message
        req = state['req']
        if isinstance(rest, Message):
            rest = web.render(req, rest)
        if rest and rest.strip():
            h = xh.Literal(reST2html(rest, initial_header_level=2))
            attrs = self.acquireAttrs(state, ('class', 'id', 'style'))
            if 'class' in attrs:
                attrs['class'] += ' wm4-rest'
            else:
                attrs['class'] = 'wm4-rest'
            return iter(xh.div(h, **attrs).expand(state))
        else:
            return iter(())


class HelpWidget(Domlet):

    def expand(self, state, help=None):
        req = state['req']
        counter = state.get('counter', 1)
        state['counter'] = counter + 1

        #for res in ('wm4.web.static/js/wm_popup.js',):
        #    web.getResource(res).render(req)

        hid = 'WM4HELP%03d' % counter
        frame = u'div', attr_list([
            ('class','wm4-help-frame'),
            ('id', hid)
        ])
        button = u'a', attr_list([
            ('class', 'wm4-help-button'),
            ('onclick', "wmShowHelp('%s')" % hid),
        ])

        yield START_TAG, frame
        yield TEXT, u'\n'
        yield END_TAG, frame
        yield TEXT, u'\n'

        yield START_TAG, button
        yield TEXT, u'?'
        yield END_TAG, button


class LangJSSwitchWidget(Domlet):

    def expand(self, state, join=' '):
        from interfaces import translations
        trans = translations()
        if not trans:
            return ()
        base = settings.language()
        req = state['req']
        for res in (
            'wm4.web.static/jquery/jquery.js',
            'wm4.web.static/js/wm_i18n.js',
        ):
            web.getResource(res).render(req)

        onclick = "wmShowTranslation('%s')"

        w = xh.span(
            xh.a(base, onclick=onclick % base, rel=base, klass='selected'),
            klass='wm4-language-switch-handler'
        )
        for l in trans:
            w.append(join)
            w.append(xh.a(l, onclick=onclick % l, rel=l))
        return iter(w.expand(state))




class MetadataWidget(Domlet):

    def expand(self, state, metadata):
        if metadata is None:
            return iter(())

        frag = Fragment([])
        #xh.link(rel="schema.DC",href="http://purl.org/dc/elements/1.1/"),
        for field, html, dc in [
            ('author', 'author', 'DC.author'),
            ('description', 'description', 'DC.description'),
            ('keywords', 'keywords', 'DC.subject'),
        ]:
            value = getattr(metadata, field, None)
            if not value:
                continue
            if html:
                frag.append(xh.meta([('name',html), ('content',value)]))
                frag.append('\n')
            if 0 and dc:
                frag.append(xh.meta([('name',dc), ('content',value)]))
                frag.append('\n')
        return iter(frag.expand(state))



class CacheWidget(Domlet):


    def __init__(self, *a, **k):
        print 'CACHE INIT', os.getpid()
        super(CacheWidget, self).__init__(*a, **k)
        self._cache = {}

    def expand_children(self, state):
        for child in self._children:
            for e in child.expand(state):
                yield e

    def expand(self, state, key=None, timeout=60, serializer=xhtml_serializer):
        req = state['req']
        now = time.time()
        value, timestamp = self._cache.get(key, (NOT_FOUND, NOT_FOUND))
        if value is NOT_FOUND or (timestamp + timeout) < now:
            req.logger.info('CACHE COMPUTING')
            value = u''.join(xhtml_serializer(
                self.expand_children(state)
            ))
            self._cache[key] = (value, now)
        else:
            req.logger.info('CACHE PRECOMPUTED')
        yield LITERAL, value
