# -*- coding: ISO-8859-2 -*-

# http://projects.almad.net/czechtile
# http://www.strangegizmo.com/products/restxsl/

from docutils import core, nodes
from docutils.writers.html4css1 import Writer, HTMLTranslator


__all__ = ['reST2html']


class BodyOnlyHTMLTranslator(HTMLTranslator):

    def astext(self):
        return ''.join(self.body_pre_docinfo + self.docinfo + self.body)

    def visit_document(self, node):
        pass

    def depart_document(self, node):
        pass

    def visit_title(self, node):
        """Only 6 section levels are supported by HTML."""
        check_id = 0
        close_tag = '</p>\n'
        if isinstance(node.parent, nodes.topic):
            self.body.append(
                  self.starttag(node, 'p', '', CLASS='topic-title first'))
            check_id = 1
        elif isinstance(node.parent, nodes.sidebar):
            self.body.append(
                  self.starttag(node, 'p', '', CLASS='sidebar-title'))
            check_id = 1
        elif isinstance(node.parent, nodes.Admonition):
            self.body.append(
                  self.starttag(node, 'p', '', CLASS='admonition-title'))
            check_id = 1
        elif isinstance(node.parent, nodes.table):
            self.body.append(
                  self.starttag(node, 'caption', ''))
            check_id = 1
            close_tag = '</caption>\n'
        elif self.section_level == 0:
            # document title
            self.head.append('<title>%s</title>\n'
                             % self.encode(node.astext()))
            self.body.append(self.starttag(node, 'h1', '', CLASS='title'))
            self.context.append('</h1>\n')
            self.in_document_title = len(self.body)
        else:
            h_level = self.section_level + self.initial_header_level - 1
            self.body.append(
                  self.starttag(node, 'h%s' % h_level, ''))
            atts = {}
            if node.parent.hasattr('id'):
                atts['name'] = node.parent['id']
            if node.hasattr('refid'):
                atts['class'] = 'toc-backref anchor' # XXX added anchor class
                atts['href'] = '#' + node['refid']
            else:
                atts['class'] = 'anchor' # XXX added anchor class
            self.body.append(self.starttag({}, 'a', '', **atts))
            self.context.append('</a></h%s>\n' % (h_level))
        if check_id:
            if node.parent.hasattr('id'):
                self.body.append(
                    self.starttag({}, 'a', '', name=node.parent['id']))
                self.context.append('</a>' + close_tag)
            else:
                self.context.append(close_tag)


class BodyOnlyHTMLWriter(Writer):

    def __init__(self):
        Writer.__init__(self)
        self.translator_class = BodyOnlyHTMLTranslator

    def apply_template(self):
        template = u"""
        %(body_prefix)s
        %(body_pre_docinfo)s
        %(docinfo)s
        %(body)s
        %(body_suffix)s
        """
        subs = self.interpolation_dict()
        return template % subs



def reST2html(rest, **settings) :
    # nastaveni jazyka a kodovani
    #settings.setdefault('initial_header_level', 2)
    #settings.setdefault('input_encoding', 'utf-8')
    settings.setdefault('output_encoding', 'unicode')
    #settings.setdefault('language_code', 'cs')
    # bude do textu vypisovat varovani a chyby od urovne 2 vys
    settings.setdefault('report_level', 2)
    # uroven chyby, na ktere parser ukonci svou praci vyjimkou.
    settings.setdefault('halt_level', 6)
    return core.publish_string(
            unicode(rest),
            writer = BodyOnlyHTMLWriter(),
            settings_overrides = settings,
    )
