# -*- coding: iso-8859-2 -*-

from new import instancemethod
import __builtin__, sys

from lib4.text import interpolation_parser, ufTemplate
from lib4.runtime import code_firstlineno, throw_in_file

from utils import raw_attr_list, adjust_python_block, dynamic_attrs
from elements import *
from const import *
from definitions import NS_XML4_TMPL
from parser import XMLParser
from serialization import XHTMLSerializer


KNOWN_NAMESPACES = {'py':NS_XML4_TMPL}


__all__ = [
    'State', 'Deferred', 'Dynattr', 'Substitute', 'Exec', 'Domlet',
    'DynamicElement', 'Template', 'TemplateParser',
]


class DomletNamespace(object):
    pass


def default_renderer(state, obj):
    if obj is None:
        return u''
    else:
        return unicode(obj)


class domlet_getter:

    def __init__(self, ns=None, name=None):
        self.ns = ns
        self.name = name

    def __getattr__(self, attr):
        if self.ns:
            if self.name:
                return domlet_getter(ns=self.ns, name=self.name+'.'+attr)
            else:
                return domlet_getter(ns=self.ns, name=attr)
        else:
            return domlet_getter(ns=attr)

    def __call__(self, *args, **kwds):
        from wm4.web.interfaces import NSDomletProperty
        from lib4.imports import importObject
        domlet = importObject(NSDomletProperty(self.ns, self.name)())
        return '%r(%r,%r)' % (domlet, args, kwds)


class State(dict):

    __slots__ = ('renderer',)

    def __init__(self, ldict=None, renderer=default_renderer):
        if ldict is not None:
            dict.__init__(self, ldict)
        self.setdefault('defined', self.defined)
        self.setdefault('value_of', self.value_of)
        self.setdefault('ns', domlet_getter())
        self.renderer = renderer

    def defined(self, name):
        return name in self

    def value_of(self, name, default):
        return self.get(name, default)

    def copy(self, ldict={}):
        cp = self.__class__(self, self.renderer)
        if ldict:
            cp.update(ldict)
        return cp

    def expand_expr(self, value):
        if isinstance(value, Node):
            for e in value.expand(self):
                yield e
        else:
            yield TEXT, self.renderer(self, value)

    def render_expr(self, value):
        return self.renderer(self, value)


class Deferred(Node):
    __slots__ = ('callback',)

    def __init__(self, callback):
        self.callback = callback

    def expand(self, state):
        yield DEFER, self.callback



class Dynattr(object):

    __slots__ = ('code', 'expr', 'gdict')

    def __init__(
        self, expr, gdict, filename='<template>', firstlineno=1, name=None
    ):
        self.expr = expr
        self.gdict = gdict
        if name is None:
            name = expr.encode('utf-8')
        try:
            code = compile(expr, filename, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], name, filename, firstlineno)
        if code.co_firstlineno != firstlineno:
            code = code_firstlineno(code, firstlineno, name=name, lnotab='')
        self.code = code

    def eval(self, state):
        return eval(self.code, self.gdict, state)

    def __call__(self, state):
        return state.render_expr(self.eval(state))

    def __repr__(self):
        return '<Dynattr %s>' % repr(self.expr)



class Substitute(Node):

    __slots__ = ('code', 'expr', 'gdict')

    def __init__(self, expr, gdict, filename='<template>', firstlineno=1):
        self.expr = expr
        self.gdict = gdict
        name = '${%s}' % expr.encode('utf-8')
        try:
            code = compile(expr, filename, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], name, filename, firstlineno)
        if code.co_firstlineno != firstlineno:
            code = code_firstlineno(code, firstlineno, lnotab='', name=name)
        self.code = code

    def expand(self, state):
        for e in state.expand_expr(eval(self.code, self.gdict, state)):
            yield e



class Exec(ProcessingInstruction):

    __slots__ = ('code', 'gdict')

    def __init__(self, target, data, gdict,filename='<template>',firstlineno=1):
        self.data = data
        self.gdict = gdict
        data = adjust_python_block(data)
        code = compile(data, filename, 'exec')
        if code.co_firstlineno != firstlineno:
            code = code_firstlineno(code, firstlineno)
        self.code = code

    def expand(self, state):
        exec self.code in self.gdict, state
        return ()




def null_domlet(state):
    return ()

class Domlet(object):

    def __init__(self,
        qname, value, gdict,
        tag=None, attrs=dynamic_attrs(), children=[], child_domlet=null_domlet,
        line_info=('<template>', 1),
    ):
        self.qname = qname
        self.value = value
        self.gdict = gdict
        self._tag = tag
        self._attrs = attrs
        self._children = children
        self.child_domlet = child_domlet
        self.line_info = line_info

    def __call__(self, state):
        # XXX
        scopy = state.copy({'__self__':self, '__state__':state})
        code = '__self__.expand(__state__,%s)' % self.value
        exp = eval(code, self.gdict, scopy)
        for e in exp:
            yield e

    def expand(self, state):
        tag = self._tag
        attrs = self._attrs.expand(state)
        yield START_TAG, (tag, attrs)
        for child in self._children:
            for e in child.expand(state):
                yield e
        yield END_TAG, (tag, attrs)

    def acquireAttrs(self, state, attrs, **defaults):
        result = {}
        get = dict(self._attrs.expand(state)).get
        for a in attrs:
            v = get(a, defaults.get(a))
            if v is not None:
                result[a] = v
        return result

    def child_expander(self, **state_modifications):
        if state_modifications:
            StateModifications(self._children, state_modifications)
        else:
            return Fragment(self._children)

class StateModifications(Fragment):

    __slots__ = ('state_modifications',)

    def __init__(self, content, state_modifications):
        Fragment.__init__(self, content)
        self.state_modifications = state_modifications

    def expand(self, state):
        state = state.copy()
        state.update(self.state_modifications)
        for elem in self:
            for e in elem.expand(state):
                yield e




class DynamicElement(Element):

    __slots__ = ('expand',)

    def __init__(self, tag, attrs, content):
        self.tag = unicode(tag)
        self._attrs = attrs
        self._children = []
        self.extend(content)




class Template(Document):

    __slots__ = ('tmpl_namespace','expand')




class TemplateParser(XMLParser):

    url = '<template>'
    templateClass = Template

    def __init__(self,
        parser_params={}, tmpl_namespace={},
        known_namespaces=KNOWN_NAMESPACES, domlet_namespaces={},
        filters = [],
    ):
        super(TemplateParser, self).__init__(parser_params, known_namespaces)
        self.domlet_namespaces = domlet_namespaces
        self._tmpl_namespace = tmpl_namespace


    def setupData(self):
        data = super(TemplateParser,self).setupData()
        data['copy_state'] = []
        self.tmpl_namespace = tns = self._tmpl_namespace.copy()
        tns.setdefault('__builtins__', __builtin__) # required for globals
        tns.setdefault('__file__', self.url)
        return data

    def ProcessingInstruction(self, target, data):
        if target == 'python' or target == 'py':
            stack = self.stack
            elem = Exec(
                target, data, self.tmpl_namespace, self.url,
                self.parser.CurrentLineNumber
            )
            if len(stack) > 1:
                top = stack[-1]
                top['children'].append(elem)
                top['copy_state'].append(elem)
            else:
                exec elem.code in self.tmpl_namespace, self.tmpl_namespace
        else:
            super(TemplateParser,self).ProcessingInstruction(target,data)

    def CharacterData(self, text):
        cl = self.parser.CurrentLineNumber - text.count('\n')
        def mkexpr(expr, ln):
            return Substitute(expr, self.tmpl_namespace, self.url, ln)
        def mktxt(t, ln):
            return Text(t)
        self.stack[-1]['children'].extend(
            interpolation_parser(text, mktxt, mkexpr, cl)
        )

    def Comment(self, text):
        if not text.lstrip().startswith('#'):
            self.stack[-1]['children'].append(Comment(text))

    valid_constructs = dict.fromkeys((
      'layout', 'extends', 'children',
      'content', 'strip', 'replace', 'with', 'if', 'for', 'def'
    ))

    def StartElement(self, tag, raw_attrs):
        data = self.pushData(tag, raw_attrs)
        data['domlets'] = domlets = {}
        data['attrs'] = attrs = dynamic_attrs()
        data['copy_state'] = []
        data['current_line'] = cl = self.parser.CurrentLineNumber
        dns = self.domlet_namespaces
        for qname, ns, name, value in data['ns_attrs']:
            if ns == NS_XML4_TMPL:
                if name in domlets:
                    self.err('More one domlet %r' % qname)
                if name == 'attrs':
                    attrs.dict_expr = Dynattr(
                        value, self.tmpl_namespace, self.url, cl, 'py:attrs'
                    ).eval
                else:
                    if not name in self.valid_constructs:
                        self.err('Invalid construct %r' % qname)
                    if name == 'replace':
                        domlets['content'] = value
                        domlets['strip'] = 'True'
                    elif name == 'strip':
                        domlets[name] = value or 'True'
                    else:
                        domlets[name] = value
            elif ns in dns:
                if 'apply' in domlets:
                    self.err('More than one domlet py:apply')
                domlets['apply'] = (ns, name, value)
            elif '$' in value:
                _n =  ('%s="%s"' % (qname, value)).encode('utf-8')
                def mkexpr(s, n):
                    return Dynattr(
                        s, self.tmpl_namespace, self.url, n, _n)
                def mktxt(s, n):
                    return Text(s)
                ai = list(interpolation_parser(value, mktxt, mkexpr, cl))
                if len(ai) > 1:
                    attrs.expressions.append((qname, ufTemplate(*ai)))
                elif len(ai) == 1:
                    if isinstance(ai[0] , Dynattr):
                        attrs.expressions.append((qname, ai[0].eval))
                    else:
                        attrs.set(qname, ai[0])
            else:
                attrs.set(qname,value)


    def EndElement(self, tag):
        data = self.stack[-1]
        domlets = data['domlets']
        attrs = data['attrs']
        if data['copy_state']:
            domlets['copy_state'] = None
        if domlets or attrs.is_dynamic():
            domlets['children'] = None
            domlets.setdefault('strip', 'False')
            element = DynamicElement(data['tag'], attrs, data['children'])
            child_domlet = self.finish_children(element,None,None)
            for domlet in (
              'content','strip','with','if','for','apply','copy_state','def',
              'extends','layout',
            ):
                if not domlet in domlets:
                    continue
                factory = getattr(self, 'finish_'+domlet)
                value = domlets[domlet]
                child_domlet = factory(element, value, child_domlet)
            element.expand = child_domlet
        else:
            element = Element(data['tag'], attrs, data['children'])
        self.popData()
        self.stack[-1]['children'].append(element)


    def finish_children(self, element, value, child_domlet):
        def do_children(state):
            for child in element:
                for e in child.expand(state):
                    yield e
        return do_children


    def finish_content(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        try:
            code = compile(value, self.url, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], 'py:content', self.url, ln)
        code = code_firstlineno(code, ln, name='py:content', lnotab='')
        tns = self.tmpl_namespace
        def do_content(state):
            for e in state.expand_expr(eval(code, tns, state)):
                yield e
        return do_content


    def finish_strip(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        try:
            code = compile(value, self.url, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], 'py:strip', self.url, ln)
        code = code_firstlineno(code, ln, name='py:strip', lnotab='')
        tag = element.tag
        _attrs = element._attrs
        tns = self.tmpl_namespace
        def do_strip(state):
            if eval(code, tns, state):
                for e in child_domlet(state):
                    yield e
            else:
                attrs = raw_attr_list([
                    (a, state.render_expr(v)) for a,v in _attrs.expand(state)
                ])
                yield START_TAG, (tag, attrs)
                for e in child_domlet(state):
                    yield e
                yield END_TAG, (tag, attrs)
        return do_strip


    def finish_with(self, element, value, child_domlet):
        cname =  'py:with=' + value.encode('utf-8')
        ln = self.stack[-1]['current_line']
        try:
            code = compile(value, self.url, 'exec')
        except:
            throw_in_file(sys.exc_info()[1], cname, self.url, ln)
        code = code_firstlineno(code, ln, name=cname, lnotab='')
        def do_with(state):
            scp = state.copy()
            exec code in scp
            for e in child_domlet(scp):
                yield e
        return do_with


    def finish_if(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        try:
            code = compile(value, self.url, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], 'py:if', self.url, ln)
        code = code_firstlineno(code, ln, name='py:if', lnotab='')
        tns = self.tmpl_namespace
        def do_if(state):
            value = eval(code, tns, state)
            if value:
                for e in child_domlet(state):
                    yield e
        return do_if


    for_template = (
        'def __loop_generator__(__iterable__):\n'
        '  for %s in __iterable__:\n'
        '    yield locals()\n')
    def finish_for(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        if not ' in ' in value:
            throw_in_file(SyntaxError, 'py:for', self.url, ln)
        target, iterable = value.split(' in ', 1)
        try:
            i_code = code_firstlineno(
                    compile(iterable, self.url, 'eval'), ln, name='py:for')
        except:
            throw_in_file(sys.exc_info()[1], 'py:for', self.url, ln)
        try:
            g_code = compile(self.for_template % target, self.url, 'exec')
        except:
            throw_in_file(sys.exc_info()[1], 'py:for', self.url, ln)
        l={}
        exec g_code in l
        loop_gen = l['__loop_generator__']
        loop_gen.func_code = code_firstlineno(loop_gen.func_code,
            ln,  name='py:for', lnotab='')
        tns = self.tmpl_namespace
        def do_for(state):
            for loc in loop_gen(eval(i_code, tns, state)):
                istate = state.copy(loc)
                for e in child_domlet(istate):
                    yield e
        return do_for

    def finish_copy_state(self, element, value, child_domlet):
        def do_copy_state(state):
            state = state.copy()
            for e in child_domlet(state):
                yield e
        return do_copy_state

    def finish_apply(self, element, (ns,name,value), child_domlet):
        url = self.url
        ln = self.stack[-1]['current_line']

        def do_apply(state):
            factory = self.domlet_namespaces[ns](state, ns, name)
            domlet = factory(
                (ns,name), value, self.tmpl_namespace,
                element.tag, element._attrs, element._children, child_domlet,
                (url, ln),
            )
            for e in domlet(state):
                yield e
        return do_apply

    def finish_def(self, element, value, child_domlet):

        value = value.encode('utf-8')
        stack = self.stack
        ln = stack[-1]['current_line']

        if not '(' in value or ')' not in value:
            throw_in_file(SyntaxError, 'py:def=' + value, self.url, ln)

        name, sig = value.strip()[:-1].split('(',1)
        if '{' in name:
            assert name.startswith('{')
            ns, name = name[1:].split('}', 1)
        elif ':' in name:
            ns, name = name.split(':',1)
            ns = self.known_namespaces[ns]
        else:
            ns = None
        if ns is not None and len(stack) > 3:
            throw_in_file(
                SyntaxError('domlet must be on toplevel'),
                'py:def=' + value, self.url, ln
            )

        if ns is None:
            return self.finish_def_func(
                element, value, name, sig, child_domlet)
        else:
            return self.finish_def_domlet(
                element, value, ns, name, sig, child_domlet)

    def finish_def_func(self, element, value, name, sig, child_domlet):

        func_name = 'py:def=' + value

        def make_expand(child_domlet):
            def expand(self, state):
                state['dom_element'] = element
                for e in child_domlet(state.copy(self.ldict)):
                    yield e
            expand.func_code = code_firstlineno(
              expand.func_code, ln, lnotab='', filename=self.url, name=func_name
            )
            expand.__name__ = func_name
            return expand

        stack = self.stack
        ln = stack[-1]['current_line']
        try:
            dcode = compile(
                'def __init__(__self__,%s): __self__.ldict = locals()' % sig,
                self.url, 'exec'
            )
        except:
            throw_in_file(sys.exc_info()[1], 'py:def', self.url, ln)

        l = {}
        exec dcode in l
        def_init = l['__init__']
        def_init.func_code = code_firstlineno(
            def_init.func_code, ln, lnotab='', name=func_name
        )
        def_init.__name__ = func_name
        domlet = type(Node)(name, (Node,), {
            '__init__'    : def_init,
            'expand'      : make_expand(child_domlet),
        })
        if len(stack) in (2,3):
            self.tmpl_namespace[name] = domlet
        stack[-2]['copy_state'].append(domlet)
        def do_def(state):
            state[name] = domlet
            return ()
        return do_def

    def finish_def_domlet(self, element, value, ns, name, sig, child_domlet):

        if sig:
            raise NotImplementedError

        _self = self
        _cd = child_domlet

        class InlineDomlet(Domlet):

            def __init__(self,
                qname, value, gdict,
                tag=None, attrs=dynamic_attrs(), children=[],
                child_domlet=null_domlet,
                line_info=('<template>', 1),
            ):
                Domlet.__init__(self,
                    qname, value, _self.tmpl_namespace,
                    element.tag, element._attrs, element._children,
                    _cd, (_self.url, ln),
                )

            def __call__(self, state):
                up = {'domlet': self}
                for e in self.child_domlet(state.copy(up)):
                    yield e

        stack = self.stack
        ln = stack[-1]['current_line']
        spec = '{%s}%s' % (ns, name)
        self.tmpl_namespace[(ns,name)] = InlineDomlet
        stack[-2]['copy_state'].append(InlineDomlet)
        def do_def(state):
            state[(ns,name)] = InlineDomlet
            return ()
        return do_def

    def findTemplate(self, spec):
        return spec

    def finish_extends(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        if len(self.stack) != 2:
            self.err('py:extends on non-root element')
        try:
            code = compile(value+',', self.url, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], 'py:extends', self.url, ln)
        code = code_firstlineno(code, ln, name='py:extends', lnotab='')

        tns = self.tmpl_namespace

        #def do_extends(state):
        if 1:
            baseSpec = reversed(eval(code, tns, tns))
            for tmplSpec in baseSpec:
                base = self.findTemplate(tmplSpec)
                for k,v in base.tmpl_namespace.items():
                    tns.setdefault(k,v)

        return child_domlet
        return do_extends


    def finish_layout(self, element, value, child_domlet):
        ln = self.stack[-1]['current_line']
        if len(self.stack) != 2:
            self.err('py:layout on non-root element')
        try:
            code = compile(value, self.url, 'eval')
        except:
            throw_in_file(sys.exc_info()[1], 'py:layout', self.url, ln)
        code = code_firstlineno(code, ln, name='py:layout', lnotab='')

        tns = self.tmpl_namespace

        def do_layout(state):
            layoutSpec = eval(code, tns, state)
            layout = self.findTemplate(layoutSpec)
            s = state.copy(tns.copy())
            s.renderer = state.renderer
            for e,a in layout.expand(s):
                yield e,a

        self.stack[0]['main'] = do_layout
        return do_layout

    def Finish(self):
        data = self.stack.pop()
        assert len(self.stack) == 0

        children = data['children']
        doc = self.templateClass(children)
        doc.tmpl_namespace = tns = self.tmpl_namespace
        main = data.get('main')
        if main:
            def expand(state=None):
                #tns = doc.tmpl_namespace.copy()
                #state = state.copy(tns)
                for e in main(state):
                    yield e
        else:
            def expand(state=None):
                #tns = doc.tmpl_namespace.copy()
                #state = state.copy(tns)
                for elem in children:
                    for e in elem.expand(state):
                        yield e
        doc.expand = expand
        return doc
