import os, sys
import xml4.tests.comparison
from lib4.iterutils import iterDecorator
from time import clock

import kid
from genshi.template import TemplateLoader
from mako.template import Template

from xml4.templates import TemplateParser, State
from xml4.serialization import XHTMLSerializer

COMP_DIR = os.path.dirname(xml4.tests.comparison.__file__)


class X:
    def __init__(self,**kw):
        self.__dict__.update(kw)

ctx = dict(
    title = 'Title',
    data = [X(a='a%d' % i, b='b%d' % i, c='c%d' % i) for i in range(20)],
    iterDecorator = iterDecorator,
)



def genshiParse():
    loader = TemplateLoader([COMP_DIR], auto_reload=True)
    return loader.load('template.genshi')
def genshiRender(tmpl):
    return tmpl.generate(**ctx).render(method='html')
    
def kidParse():
    return kid.load_template('template.kid', cache=0)
def kidRender(tmpl):
    template = tmpl.Template(**ctx)
    return str(template)
    
def wmParse():
    filename = os.path.join(COMP_DIR, 'template.wt')
    parser = TemplateParser()
    return parser.parseFile('template.wt')
def wmRender(tmpl):
    state = State(ctx)
    serializer = XHTMLSerializer()
    return serializer(tmpl.expand(state))
   
def makoParse():
    return Template(filename=os.path.join(COMP_DIR,'template.mako'))
def makoRender(tmpl):
    return tmpl.render(**ctx)

def bench_parsing(label, parse, render, iters=1):
    start = clock()
    for x in xrange(iters):
        tmpl = parse()
        res = render(tmpl)
    print '%-10s %8.2f' % (label, clock()-start)
    return res


def bench_rendering(label, parse, render, iters=1):
    start = clock()
    tmpl = parse()
    for x in xrange(iters):
        res = render(tmpl)
    print '%-10s %8.2f' % (label, clock()-start)
    open(label +'.html', 'w').write(res)

if len(sys.argv) > 1:
    COUNT = int(sys.argv[1])
else:
    COUNT = 100

PARSING = COUNT

print 'parsing'
print '--------'
bench_parsing('genshi', genshiParse, genshiRender, PARSING)
bench_parsing('kid', kidParse, kidRender, PARSING)
bench_parsing('windmill', wmParse, wmRender, PARSING)
bench_parsing('mako', makoParse, makoRender, PARSING)
print

RENDERING = COUNT

print 'rendering'
print '--------'
bench_rendering('genshi', genshiParse, genshiRender, PARSING)
bench_rendering('kid', kidParse, kidRender, PARSING)
bench_rendering('windmill', wmParse, wmRender, PARSING)
bench_rendering('mako', makoParse, makoRender, PARSING)
