from xml4 import templates, const
from xml4.serialization import XHTMLSerializer
from xml4 import xhtml as xh
from time import clock


TMPL = '''\
<?py

  class X:
    def __init__(self,**kw):
      self.__dict__.update(kw)

  data = [X(a='a%d' % i, b='b%d' % i, c='c%d' % i) for i in range(20)]

  title = 'Title'

  from lib4.iterutils import iterDecorator
?>
<doc>
  <head>
    <title>$title</title>
  </head>
  <body>
    <table>
      <tr>
        <th>No</th>
        <th>Title A</th>
        <th>Title B</th>
        <th>Title C</th>
      </tr>
      <tr py:for="item in iterDecorator(data)">
        <td>${item.index+1}</td>
        <td>$item.item.a</td>
        <td>$item.item.b</td>
        <td py:content="item.item.c">C</td>
      </tr>
    </table>
  </body>
</doc>
'''

cycles = 1000

# template parsing
start = clock()
for i in xrange(cycles):
    parser = templates.TemplateParser()
    tmpl = parser.parse(TMPL)
end = clock()
print 'template parsing: %d cycles, total %.2fsec, %.2f cycles/sec' % (
    cycles, end-start, cycles/(end-start)
)


# parsed template rendering
tmpl = templates.TemplateParser().parse(TMPL)
serializer = XHTMLSerializer()
start = clock()
for i in xrange(cycles):
    state = templates.State()
    res = u''.join(serializer(tmpl.expand(state)))
end = clock()
print 'template rendering: %d cycles, total %.2fsec, %.2f cycles/sec' % (
    cycles, end-start, cycles/(end-start)
)


# combined test
start = clock()
for i in xrange(cycles):
    tmpl = templates.TemplateParser().parse(TMPL)
    serializer = XHTMLSerializer()
    res = u''.join(serializer(tmpl.expand(templates.State())))
end = clock()
print 'template parsing/rendering: %d cycles, total %.2fsec, %.2f cycles/sec' % (
    cycles, end-start, cycles/(end-start)
)
