#-*- coding: iso-8859-2 -*-
from unittest import *
from xml4.elements import *
import pickle

class TestElements(TestCase):

    def test10_Wrapping(self):
        e = Element('e', {}, [])
        self.failUnless(isinstance(e.tag, unicode))
        e.append('a')
        self.failUnless(isinstance(e[0], Text))
        t = Text('t')
        e.extend(('b',t))
        self.failUnless(isinstance(e[1], Text))
        self.failUnless(e[2] is t)


class TestPickling(TestCase):

    def test10_Element(self):
        e1 = Element('e', [('lang','cs'),('id','12')], [])
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is Element)
        self.assertEqual(e2.tag, 'e')
        self.assertEqual(list(e2.items()), [('lang','cs'),('id','12')])

    def test12_Text(self):
        e1 = Text('T')
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is Text)
        self.assertEqual(unicode(e2), 'T')

    def test13_Literal(self):
        e1 = Literal('T')
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is Literal)
        self.assertEqual(unicode(e2), 'T')

    def test14_Comment(self):
        e1 = Comment('C')
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is Comment)
        self.assertEqual(unicode(e2), 'C')

    def test15_ProcessingInstruction(self):
        e1 = ProcessingInstruction('P', 'X')
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is ProcessingInstruction)
        self.assertEqual(e2.target, 'P')
        self.assertEqual(e2.data, 'X')

    def test16_XmlDecl(self):
        e1 = XmlDecl('1.0', 'utf-8', True)
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is XmlDecl)
        self.assertEqual(e2.version, '1.0')
        self.assertEqual(e2.encoding, 'utf-8')
        self.assertEqual(e2.standalone, True)

    def test17_DocType(self):
        from xml4.definitions import DT_XHTML1_STRICT as e1
        s = pickle.dumps(e1)
        e2 = pickle.loads(s)
        self.failUnless(e2.__class__ is DocType)
        for a in ('name', 'sysid', 'pubid', 'has_internal_subset'):
            self.assertEqual(getattr(e1,a), getattr(e2,a))


    def test20_Document(self):
        from xml4.definitions import DT_XHTML1_STRICT
        import xml4.xhtml as h
        d1 = Document([
          XmlDecl('1.0','utf-8',False),
          DT_XHTML1_STRICT,
          h.html(
            {'lang':'cs','xmlns':'foo'},
            h.head(h.title('test')),
            Comment('body start'),
            h.body(
              h.h1('<Pickle', h.nbsp, 'Test>'),
            ),
            Comment('body end'),
          )
        ])
        s = pickle.dumps(d1)
        d2 = pickle.loads(s)
        assert d2.__class__ is Document
        self.assertEqual(len(d1), len(d2))
        self.assertEqual(unicode(d2),
            u'<?xml version="1.0" encoding="utf-8"?>'
            u'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" '
                 u'"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
            u'<html lang="cs" xmlns="foo">'
              u'<head><title>test</title></head>'
              u'<!-- body start -->'
              u'<body><h1>&lt;Pickle&nbsp;Test&gt;</h1></body>'
              u'<!-- body end -->'
            u'</html>'
        )
        assert d1 is not d2


def suite():
    return TestSuite((
        makeSuite(TestElements),
        makeSuite(TestPickling),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
