#-*- coding: iso-8859-2 -*-
from unittest import *
from xml4 import utils


class TestAdjustPythonBlock(TestCase):

    def checkAdjust(self, src):
        data = src.lstrip()
        result = utils.adjust_python_block(data)
        compile(src, '<test>', 'exec')
        self.assertEqual(src, result)

    def test10_OneLine(self):
        self.checkAdjust('a = 1')

    def test11_OneLine(self):
        self.checkAdjust('a = 1; b = 32')

    def test20_MultilineNoMargin(self):
        self.checkAdjust(
            'a = 1\n'
            'b = 32'
        )

    def test22_MoreSuites(self):
        self.checkAdjust(
            'def f1(x): return x*1\n'
            'def f2(x): return x*2'
        )
        self.checkAdjust(
            'def f1(x):\n'
            '  return x*1\n'
            'def f2(x): return x*2'
        )
        self.checkAdjust(
            'def f1(x):\n'
            '  return x*1\n'
            'def f2(x):\n'
            '  return x*2'
        )


class TestAttrList(TestCase):

    def test1_ListMethods(self):
        al = utils.attr_list([('a','A'),('b','B'),('c','C')])
        self.assertEqual(len(al), 3)
        self.assertEqual(list(al), [('a','A'),('b','B'),('c','C')])

    def test2_Get(self):
        al = utils.attr_list([('a','A'),('b','B')])
        self.assertEqual(al.get('a'), 'A')
        self.assertEqual(al.get('b'), 'B')
        self.assertEqual(al.get('c'), None)
        self.assertEqual(al.get('c','C'), 'C')

    def test3_SetExisting(self):
        al = utils.attr_list([('a','A'),('b','B')])
        al.set('a','X')
        self.assertEqual(list(al), [('a','X'), ('b','B')])
        self.assertEqual(al.get('a'), 'X')

    def test4_SetNonExisting(self):
        al = utils.attr_list([('a','A'),('b','B')])
        al.set('c','C')
        self.assertEqual(list(al), [('a','A'), ('b','B'), ('c','C')])
        self.assertEqual(al.get('c'), 'C')

    def test5_SetDefaultExisting(self):
        al = utils.attr_list([('a','A'),('b','B')])
        al.setdefault('a','X')
        self.assertEqual(list(al), [('a','A'), ('b','B')])
        self.assertEqual(al.get('a'), 'A')

    def test6_SetDefaultNonExisting(self):
        al = utils.attr_list([('a','A'),('b','B')])
        al.setdefault('c','C')
        self.assertEqual(list(al), [('a','A'), ('b','B'), ('c','C')])
        self.assertEqual(al.get('c'), 'C')



def suite():
    return TestSuite((
        makeSuite(TestAdjustPythonBlock),
        makeSuite(TestAttrList),
    ))

if __name__ == '__main__':
    main(defaultTest='suite')
