from utils import escape
from serialization import XMLSerializer

__all__ = [
    'HTML_FRAGMENT_ENVELOP', 'TextSerializer', 'html_to_text', 'text_to_html',
]


HTML_FRAGMENT_ENVELOP = u'''\
<?xml version="1.0"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title></title>
</head>
<body>
%s
</body>
</html>
'''

SPACING_TAGS = set([
    'p', 'div', 'blockquote',
    'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
    'td', 'li', 'dl', 'dd',
    'br', 'caption', 'iframe',
])

OMIT_TAGS = set(['head', 'script', 'frame'])


class TextSerializer(XMLSerializer):

    detectEmptyTags = False
    emptyEnabled = set()

    def __init__(self):
        self.omitting = 0
    render = True

    def start(self, (tag, attrs)):
        if tag in OMIT_TAGS:
            self.omitting += 1
        return ()

    def end(self, (tag, attrs)):
        if self.omitting:
            self.omitting -= 1
            return ()
        else:
            if tag in SPACING_TAGS:
                return (' ')
            else:
                return ()

    def empty(self, (tag, attrs)):
        if not self.omitting and tag in SPACING_TAGS:
            return (' ')
        else:
            return ()

    def text(self, txt):
        if self.omitting:
            return ()
        else:
            return txt,

    def literal(self, txt):
        if self.omitting:
            return ()
        else:
            return txt,

    def defer(self, cb):
        raise NotImplementedError

    def omit(self, arg):
        return ()
    doc_type = pi = comment = xml_decl = omit


def html_to_text(html, fragment=False):
    from parser import SimpleXMLParser
    if not html:
        return html
    sp = SimpleXMLParser()
    sr = TextSerializer()
    if fragment:
        src = HTML_FRAGMENT_ENVELOP % html
    else:
        src = html
    if isinstance(src, unicode):
        src = src.encode('utf-8')
    doc = sp.parse(src, encoding='utf-8')
    return u' '.join(u''.join(sr(doc.expand(None))).split())


def text_to_html(txt):
    '''Converts plain text to html

       Blank lines delimit paragraph boundaries.
    '''
    WHITE = w = 'white'
    PARA = 'para'
    h = []
    for l in txt.strip().splitlines():
        l = l.strip()
        if l:
            if w is WHITE:
                w = PARA
                h.append('<p>')
            else:
                h.append('\n')
            h.append(escape(l))
        else:
            if w is WHITE:
                pass
            else:
                h.append('</p>\n')
                w = WHITE
    if h:
        h.append('</p>\n')
    return ''.join(h)
