__all__ = ['quoteattr','escape','raw_attr_list','attr_list','dynamic_attrs']

def escape (data) :
    # must do ampersand first
    data = data.replace("&", "&amp;")
    data = data.replace(">", "&gt;")
    data = data.replace("<", "&lt;")
    return data


def quoteattr(data, escaped=False):
    if not escaped:
        data = escape(data)
    return '"%s"' % data.replace('"', "&quot;")



# This function is taken from kid parser

def _adjust_python_block(lines, tab='\t'):
    """Adjust the indentation of a Python block."""
    lines = [lines[0].strip()] + [line.rstrip() for line in lines[1:]]
    ind = None # find least index
    for line in lines[1:]:
        if line != '':
            s = line.lstrip()
            if s[0] != '#':
                i = len(line) - len(s)
                if ind is None or i < ind:
                    ind = i
                    if i == 0:
                        break
    if ind is not None or ind != 0: # remove indentation
        lines[1:] = [line[:ind].lstrip() + line[ind:]
            for line in lines[1:]]
    if lines[0] and not lines[0][0] == '#':
        # the first line contains code
        try: # try to compile it
            compile(lines[0], '<string>', 'exec')
            # if it works, line does not start new block
        except SyntaxError: # unexpected EOF while parsing?
            try: # try to compile the whole block
                block = '\n'.join(lines) + '\n'
                compile(block, '<string>', 'exec')
                # if it works, line does not start new block
            except IndentationError: # expected an indented block?
                # so try to add some indentation:
                lines2 = lines[:1] + [tab + line for line in lines[1:]]
                block = '\n'.join(lines2) + '\n'
                # try again to compile the whole block:
                compile(block, '<string>', 'exec')
                lines = lines2 # if it works, keep the indentation
            except:
                pass # leave it as it is
        except:
            pass # leave it as it is
    return lines



def adjust_python_block(txt):
    lines = txt.splitlines()
    if not lines:
        return ''
    elif len(lines) == 1:
        return lines[0].strip()
    else:
        return '\n'.join(_adjust_python_block(txt.splitlines()))



class raw_attr_list(object):
    '''Enhanced list for holding template element attributes'''

    __slots__ = ('_list', '_positions')

    def __init__(self, items):
        '''Requires iterator with two-element tuples'''
        self._list = []
        self._positions = {}
        self.extend(items)

    def extend(self, other):
        lst = self._list
        pos = self._positions
        for attr, value in other:
            idx = pos.get(attr)
            if idx is None:
                pos[attr] = len(lst)
                lst.append((attr,value))
            else:
                lst[idx] = value

    def set(self, attr, value):
        '''Sets value by attribute name'''
        lst = self._list
        pos = self._positions
        if attr in pos:
            lst[pos[attr]] = (attr,value)
        else:
            pos[attr] = len(lst)
            lst.append((attr, value))

    def setdefault(self, attr, value):
        '''Sets value by attribute name if it's not already set'''
        if attr not in self._positions:
            self.set(attr, value)

    def get(self, attr, default=None):
        '''Returns value by attribute name with eventual default'''
        idx = self._positions.get(attr)
        if idx is None:
            return default
        else:
            return self._list[idx][1]

    def pop(self, attr, default=None):
        lst = self._list
        pos = self._positions
        idx = pos.get(attr)
        if idx is None:
            return default
        else:
            value = lst[idx]
            del pos[attr]
            del lst[idx]
            return value

    def hasattr(self, attr):
        return attr in self._positions

    def __iter__(self):
        return iter(self._list)

    def __len__(self):
        return len(self._list)

    def __cmp__(self, other):
        if isinstance(other, self.__class__):
            return cmp(self._list, other._list)
        else:
            raise ValueError('Uncomparable', other)

    def expand(self, state):
        return self


class attr_list(raw_attr_list):
    '''Converts values to unicode'''

    __slots__ = ()

    def extend(self, other):
        lst = self._list
        pos = self._positions
        for attr, value in other:
            value = unicode(value)
            idx = pos.get(attr)
            if idx is None:
                pos[attr] = len(lst)
                lst.append((attr,value))
            else:
                lst[idx] = value

    def set(self, attr, value):
        '''Sets value by attribute name'''
        lst = self._list
        pos = self._positions
        value = unicode(value)
        if attr in pos:
            lst[pos[attr]] = (attr,value)
        else:
            pos[attr] = len(lst)
            lst.append((attr, value))


class dynamic_attrs(raw_attr_list):
    '''Can have expressions as values'''

    __slots__ = ('expressions', 'dict_expr')

    def __init__(self, items=()):
        super(dynamic_attrs, self).__init__(items)
        self.expressions = []
        self.dict_expr = None

    def is_dynamic(self):
        return self.expressions or self.dict_expr is not None

    def expand(self, state):
        attrs = raw_attr_list(self)

        for a,e in self.expressions:
            attrs.set(a, e(state))

        de = self.dict_expr
        if de is not None:
            for a,v in de(state).iteritems():
                attrs.set(a,v)

        for a,v in attrs:
            if v is None:
                attrs.pop(a)
        return attrs
