from elements import Element, Literal, Text, ElementFactory, Document, Fragment
from utils import escape, quoteattr

html_tags = [
    'a', 'abbr', 'acronym', 'address', 'applet', 'area', 'base',
    'basefont', 'bdo', 'big', 'blockquote', 'body', 'br', 'button',
    'caption', 'center', 'cite', 'code', 'col', 'colgroup', 'dd', 'dfn',
    'div', 'dl', 'dt', 'em', 'fieldset', 'font', 'form', 'frame',
    'frameset', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'hr', 'html',
    'iframe', 'img', 'input', 'ins', 'isindex', 'kbd', 'label', 'legend',
    'li', 'link', 'menu', 'meta', 'noframes', 'noscript', 'ol', 'optgroup',
    'option', 'p', 'param', 'pre', 'q', 's', 'samp', 'script', 'select',
    'small', 'span', 'strike', 'strong', 'style', 'sub', 'sup', 'table',
    'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'title', 'tr', 'tt',
    'u', 'ul', 'var'
]

html_literals = ['nbsp', 'amp', 'quot', 'gt', 'lt']

__all__ = html_tags + html_literals + [
    'XHTMLElement', 'XHTMLDocument', 'XHTMLFragment', 'Text', 'Literal',
    'template', 'escape', 'quoteattr', 'frag',
]

_glob = globals()

class XHTMLFragment(Fragment):

    def __unicode__(self):
        from serialization import xhtml_serializer
        return u''.join(xhtml_serializer(self.expand(None)))

def frag(*content):
    return XHTMLFragment(content)

class XHTMLElement(Element):

    def __unicode__(self):
        from serialization import xhtml_serializer
        return u''.join(xhtml_serializer(self.expand(None)))

class XHTMLDocument(Document):

    def __unicode__(self):
        from serialization import xhtml_serializer
        return u''.join(xhtml_serializer(self.expand(None)))

ATTR_MAP = {
    '_class'        : 'class',
    'class_'        : 'class',
    'klass'         : 'class',
    'http_equiv'    : 'http-equiv',
    'httpequiv'     : 'http-equiv',
    'accept_charset': 'accept-charset',
    'acceptcharset' : 'accept-charset',
    'typ'           : 'type',
}

for tag in html_tags:
    _glob[tag] = ElementFactory(
        tag=tag, elem_factory=XHTMLElement, attr_map=ATTR_MAP
    )

for lit in html_literals:
    _glob[lit] = Literal(u'&%s;' % lit)


def template():
    import definitions
    head = html.head()
    body = html.body()
    doc = XHTMLDocument([
        definitions.DT_XHTML1_STRICT, '\n',
        html(
            {'xmlns':definitions.NS_XHTML},
            '\n',
            head, '\n',
            body, '\n',
        ), '\n',
    ])
    doc.head = head
    doc.body = body
    return doc
